#!/usr/bin/perl -w

use vars qw($versionstring $copyright);

my $pwd = `pwd`; chomp($pwd);
my $header = "$pwd/../../shared/pgpBuild.h";
my @subs;
my $copyright_partial = 0;
my $plists =
[
	{
		"PGP.framework" =>
		{
			"plist" => "$pwd/../../console/osx/build/PGP.app/Contents/Frameworks/PGP.framework/Resources/Info.plist",
			"substitutions" => \@subs,
		},
	},
	{
		"PGPui.framework" =>
		{
			"plist" => "$pwd/../../console/osx/build/PGP.app/Contents/Frameworks/PGPui.framework/Resources/Info.plist",
			"substitutions" => \@subs,
		},
	},
	{
		"PGPservice" =>
		{
			"plist" => "$pwd/../../console/osx/build/PGP.app/Contents/Frameworks/PGP.framework/Resources/PGPservice.app/Contents/Info.plist",
			"substitutions" => \@subs,
		},
	},
	{
		"PGPclient.framework" =>
		{
			"plist" => "$pwd/../../console/osx/build/PGP.app/Contents/Frameworks/PGPclient.framework/Resources/Info.plist",
			"substitutions" => \@subs,
		},
	},
	{
		"PGPmail.bundle" =>
		{
			"plist" => "$pwd/../../console/osx/build/PGP.app/Contents/PlugIns/PGPmail.bundle/Contents/Info.plist",
			"substitutions" => \@subs,
		},
	},
	{
		"PGPmail.mailbundle" =>
		{
			"plist" => "$pwd/../../console/osx/build/PGP.app/Contents/PlugIns/PGPmail.bundle/Contents/Resources/PGPmail.mailbundle/Contents/Info.plist",
			"substitutions" => \@subs,
		},
	},
	{
		"PGPcontext.plugin"	=>
		{
			"plist" => "$pwd/../../console/osx/build/PGP.app/Contents/PlugIns/PGPmail.bundle/Contents/Resources/PGPcontext.plugin/Contents/Info.plist",
			"substitutions" => \@subs,
		},
	},
	{
		"PGPkeys.bundle" =>
		{
			"plist" => "$pwd/../../console/osx/build/PGP.app/Contents/PlugIns/PGPkeys.bundle/Contents/Info.plist",
			"substitutions" => \@subs,
		},
	},
	{
		"PGPdisk.bundle" =>
		{
			"plist"	=> "$pwd/../../console/osx/build/PGP.app/Contents/PlugIns/PGPdisk.bundle/Contents/Info.plist",
			"substitutions"	=> \@subs,
		},
	},
	{
		"PGPdiskDriver.kext" =>
		{
			"plist" => "$pwd/../../console/osx/build/PGP.app/Contents/PlugIns/PGPdisk.bundle/Contents/Resources/PGPdiskDriver.kext/Contents/Info.plist",
			"substitutions" => \@subs,
		},
	},
	{
		"PGPnet.bundle" =>
		{
			"plist" => "$pwd/../../console/osx/build/PGP.app/Contents/PlugIns/PGPnet.bundle/Contents/Info.plist",
			"substitutions" => \@subs,
		}
	},
	{
		"PGPnet.kext" =>
		{
			"plist" => "$pwd/../../console/osx/build/PGP.app/Contents/PlugIns/PGPnet.bundle/Contents/Resources/PGPnet.kext/Contents/Info.plist",
			"substitutions" => \@subs,
		},
	},
	{
		"PGPnetServiceUI.app" =>
		{
			"plist" => "$pwd/../../console/osx/build/PGP.app/Contents/PlugIns/PGPnet.bundle/Contents/Resources/PGPnetServiceUI.app/Contents/Info.plist",
			"substitutions" => \@subs,
		},
	},
	{
		"PGP.app" =>
		{
			"plist" => "$pwd/../../console/osx/build/PGP.app/Contents/Info.plist",
			"substitutions" => \@subs,
		},
	},
];

# Setup our text substitutions (ie. -VERSION- gets substituted with $versionstring)
push(@subs, ('s/-GETINFO-/$versionstring/g;', 's/-VERSION-/$versionstring/g;', 's/-COPYRIGHT-/$copyright/g;'));

# Get the version number
print "Parsing $header\n";
open(HEADER, "$header") or die "Couldn't open $header for reading\n";
my @lines = <HEADER>;
close(HEADER);

foreach (@lines)
{
	if ($copyright_partial)
	{
		/\"(.*)\"/;
		$copyright .=  $1;
		$copyright_partial = 0;
	}

	if (/PGPVERSIONSTRING\s+\"(.*)\"/)	{ $versionstring = $1; }
	elsif (/PGPCOPYRIGHT/)
	{
		($copyright) = /\"(.*)\" \\$/;
		$copyright_partial = 1;
	}
}

foreach my $hash (@$plists)
{
	my $name = (keys(%$hash))[0];
	my $plist = $hash->{"$name"}->{"plist"};
	my $subs = $hash->{"$name"}->{"substitutions"};
	
	if (!(-e "$plist"))	{ next; }

	print "Modifying Info.plist for $name\n";	
	open(PLIST, "$plist") or die "Couldn't open $plist for reading\n";
	@lines = <PLIST>;
	close(PLIST);
	
	foreach (@lines)
	{
		foreach my $substitution (@$subs)	{ eval($substitution); }
	}
	
	open(PLIST, ">$plist") or die "Couldn't open $plist for writing\n";
	print PLIST @lines;
	close(PLIST);
}
