/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation
    All rights reserved.

    $Id: CCAThread.m,v 1.3 2002/08/06 20:09:16 dallen Exp $
____________________________________________________________________________*/
#import <PGP/pgpOptionList.h>
#import <PGP/pgpKeyServer.h>
#import <PGPclient/pflPrefs.h>
#import <PGPclient/pgpClientPrefs.h>
#import <PGPclient/pflPrefTypes.h>
#import "CRemoteAuth.h"
#import "CCAThread.h"


	static PGPError
serverCallback(
	PGPContextRef 	context,
	PGPEvent 		*event,
	PGPUserValue 	userValue)
{
	CCAThread		*caThread = (CCAThread *)userValue;
	
	return [caThread serverCallback:event];
}

@implementation CCAThread

- (id)initCAThread:(PGPContextRef)context
		prefRef:(PGPPrefRef)prefRef
		tlsContext:(PGPtlsContextRef)tlsContext
		op:(PGPCAOp)caOp
		reqType:(PGPUInt32)reqType
		keyDB:(PGPKeyDBRef)keyDB
		targetKey:(PGPKeyDBObjRef)targetKey
		passphrase:(char *)passphrase
		avList:(PGPAttributeValue *)avList
		numAVs:(PGPUInt32)numAVs
		status:(NSTextField *)status
		abort:(BOOL *)abortSwitch
		target:(id)target
		didEndSelector:(SEL)didEndSelector
{
	[self init];
	mContext				= context;
	mPrefRef				= prefRef;
	mTLSContext				= tlsContext;
	mKeyDB					= keyDB;
	mCAOp					= caOp;
	mCARequestType			= reqType;
	mCATargetKey			= targetKey;
	mCAPassphrase			= passphrase;
	if( IsntNull( passphrase ) )
		mCAPassShared 		= YES;
	mCAAVList				= avList;
	mCANumAVs				= numAVs;
	mProgressAbort			= abortSwitch;
	mProgressStatusField	= status;
	mTarget					= target;
	mEndSelector			= didEndSelector;
	
	return self;
}

- (PGPError)run
{
	PGPError			err = kPGPError_NoErr;
	PGPKeyServerClass	serverClass;
	
	PGPGetPrefNumber( mPrefRef, kPGPPrefCAType, (PGPUInt32 *)&serverClass );
	if( !mCAPassShared && PGPKeyDBObjRefIsValid( mCATargetKey ) &&
		( ( ( mCAOp == kPGPCA_OP_RequestCert ) || ( mCAOp == kPGPCA_OP_RetrieveCert ) ) ||
		( ( serverClass == kPGPKeyServerClass_Verisign ) || ( serverClass == kPGPKeyServerClass_Entrust ) ) ) )
	{
		err = PGPclGetKeyPassphrase( mCATargetKey, mTLSContext, mPrefRef, &mCASplit, &mCAPassphrase, &mCAPassKey, &mCAPassKeySize );
	}
	if( IsntPGPError( err ) )
	{
		[NSThread detachNewThreadSelector:@selector(caServerThread:) toTarget:self withObject:self];
	}
	return err;
}

- (void)caServerThread:(id)sender
{
	NSAutoreleasePool				*pool = [[NSAutoreleasePool alloc] init];
	PGPError						err = kPGPError_NoErr;
	PGPKeyServerClass				serverClass;
	PGPtlsSessionRef				tls = kInvalidPGPtlsSessionRef;
	PGPFilterRef					filter = kInvalidPGPFilterRef;
	char							serverURL[256];
	PGPSize							expSize,
									reqSize,
									keyIDSize,
									iasnSize;
	PGPInputFormat					iFormat;
	PGPOutputFormat					oFormat;
	PGPExportFormat					eFormat;
	PGPByte *						keyID = NULL,
			*						iasn = NULL,
			*						expBuffer = NULL,
			*						reqBuffer = NULL;
	PGPKeyDBObjRef					caCert;
	PGPKeyDBObjRef					caKey;
	PGPKeyDBRef						tempKeyDB = kInvalidPGPKeyDBRef;
	PGPKeyDBRef						decodeDB = kInvalidPGPKeyDBRef;
	NSMethodSignature				*methodSignature;
	NSInvocation					*methodInvocation;
	
	err = PGPGetPrefData( mPrefRef, kPGPPrefCARootKeyID, &keyIDSize, (void *)&keyID );					CKERR;
	err = PGPGetPrefData( mPrefRef, kPGPPrefCARootX509IASN, &iasnSize, (void *)&iasn );					CKERR;
	err = PGPclX509CertFromExport( mContext, (PGPKeyID *)keyID, iasn, iasnSize, mKeyDB, &caCert );		CKERR;
	if( !PGPKeyDBObjRefIsValid( caCert ) )
	{
		err = kPGPError_MissingX509Certificate;
		goto done;
	}
	caKey = PGPPeekKeyDBObjKey( caCert );
	if( mCAOp == kPGPCA_OP_UpdateCRL )
		err = PGPGetPrefStringBuffer( mPrefRef, kPGPPrefCARevocationServerURL, sizeof(serverURL), serverURL );
	else
		err = PGPGetPrefStringBuffer( mPrefRef, kPGPPrefCAServerURL, sizeof(serverURL), serverURL );	CKERR;
	err = PGPGetPrefNumber( mPrefRef, kPGPPrefCAType, (PGPUInt32 *)&serverClass );						CKERR;
	err = PGPNewKeyServer( mContext, serverClass, &mServer,
				PGPONetURL( mContext, serverURL ),
				PGPOLastOption( mContext ) );															CKERR;
	mLastState = kPGPKeyServerState_Invalid;
	PGPSetKeyServerEventHandler( mServer, serverCallback, self );
	err = PGPNewTLSSession( mTLSContext, &tls );														CKERR;
	switch( serverClass )
	{
		case kPGPKeyServerClass_NetToolsCA:
			iFormat = kPGPInputFormat_NetToolsCAV1_DataInPKCS7;
			eFormat = kPGPExportFormat_NetToolsCAV1_CertReq;
			oFormat = kPGPOutputFormat_NetToolsCAV1_CertReqInPKCS7;
			break;
		case kPGPKeyServerClass_Netscape:
			iFormat = kPGPInputFormat_NetscapeV1_DataInPKCS7;
			eFormat = kPGPExportFormat_NetscapeV1_CertReq;
			oFormat = kPGPOutputFormat_NetscapeV1_CertReqInPKCS7;
			break;
		case kPGPKeyServerClass_Microsoft:
			iFormat = kPGPInputFormat_MicrosoftV1_DataInPKCS7;
			eFormat = kPGPExportFormat_MicrosoftV1_CertReq;
			oFormat = kPGPOutputFormat_MicrosoftV1_CertReqInPKCS7;
			break;
		case kPGPKeyServerClass_Verisign:
			iFormat = kPGPInputFormat_VerisignV1_DataInPKCS7;
			eFormat = kPGPExportFormat_VerisignV1_CertReq;
			oFormat = kPGPOutputFormat_VerisignV1_CertReqInPKCS7;
			break;
		case kPGPKeyServerClass_Entrust:
			iFormat = kPGPInputFormat_EntrustV1_DataInPKCS7;
			eFormat = kPGPExportFormat_EntrustV1_CertReq;
			oFormat = kPGPOutputFormat_EntrustV1_CertReqInPKCS7;
			break;
		default:
			err = kPGPError_ItemNotFound;
			goto done;
	}
	if( ( mCAOp != kPGPCA_OP_RequestCert ) || ( mCARequestType != kPGPPrefCertRequestPKCS10 ) )
	{
		err = PGPKeyServerOpen( mServer, tls );															CKERR;
	}
	switch( mCAOp )
	{
		case kPGPCA_OP_RequestCert:
			err = PGPExport( mContext,
					PGPOExportKeyDBObj( mContext, mCATargetKey ),
					PGPOAllocatedOutputBuffer( mContext, (void **)&expBuffer, MAX_PGPSize, &expSize ),
					PGPOExportFormat( mContext, eFormat ),
					mCASplit ?
						PGPOPasskeyBuffer( mContext, mCAPassKey, mCAPassKeySize ) :
						PGPOPassphrase( mContext, mCAPassphrase),
					PGPOAttributeValue( mContext, mCAAVList, mCANumAVs ),
					PGPOArmorOutput( mContext, ( mCARequestType == kPGPPrefCertRequestPKCS10 ) ? TRUE : FALSE ),
					PGPOLastOption( mContext ) );														CKERR;
			if( mCARequestType == kPGPPrefCertRequestPKCS10 )
			{
				[[NSPasteboard generalPasteboard] declareTypes:
					[NSArray arrayWithObjects:NSStringPboardType, nil] owner:self];
				[[NSPasteboard generalPasteboard] setString:[NSString stringWithCString:expBuffer length:expSize]
					forType:NSStringPboardType];
			}
			else
			{
				err = PGPEncode( mContext,
						PGPOInputBuffer( mContext, expBuffer, expSize),
						PGPOOutputFormat( mContext, oFormat ),
						PGPOAllocatedOutputBuffer( mContext, (void **)&reqBuffer, MAX_PGPSize, &reqSize ),
						PGPOSignWithKey( mContext, mCATargetKey,
							mCASplit ?
								PGPOPasskeyBuffer( mContext, mCAPassKey, mCAPassKeySize ) :
								PGPOPassphrase( mContext, mCAPassphrase),
							PGPOLastOption( mContext ) ),
						( serverClass >= kPGPKeyServerClass_Netscape ) ?
							PGPOEncryptToKeyDBObj( mContext, caKey ) :
							PGPONullOption( mContext ),
						PGPOLastOption( mContext ) );													CKERR;
				err = PGPSendCertificateRequest( mServer,
						PGPOKeyServerCAKey( mContext, caKey ),
						PGPOKeyServerRequestKey( mContext, mCATargetKey ),
						PGPOInputBuffer( mContext, reqBuffer, reqSize ),
						PGPOLastOption( mContext ) );													CKERR;
			}
			break;
		case kPGPCA_OP_RetrieveCert:
		{
			PGPByte			hashBuffer[128];
			PGPSize			hashSize;
			PGPUInt32		keysReturned = 0;
			
			switch( serverClass )
			{
				case kPGPKeyServerClass_NetToolsCA:
					err = PGPGetKeyDBObjDataProperty( mCATargetKey,
							kPGPKeyProperty_X509MD5Hash, hashBuffer,
							sizeof(hashBuffer), &hashSize );											CKERR;
					err = PGPNewKeyDBObjDataFilter( mContext,
							kPGPKeyProperty_X509MD5Hash, hashBuffer, hashSize,
							kPGPMatchCriterion_Equal, &filter);											CKERR;
					err = PGPRetrieveCertificate( mServer, 
							PGPOKeyServerSearchFilter( mContext, filter ),
							PGPOAllocatedOutputBuffer( mContext, (void **)&expBuffer, MAX_PGPSize, &expSize ),
							PGPOLastOption( mContext ) );												CKERR;
					break;
				case kPGPKeyServerClass_Microsoft:
				case kPGPKeyServerClass_Netscape:
				case kPGPKeyServerClass_Verisign:
				case kPGPKeyServerClass_Entrust:
					err = PGPRetrieveCertificate( mServer,
							PGPOKeyServerCAKey( mContext, caKey ),
							PGPOSignWithKey( mContext, mCATargetKey,
								mCASplit ?
									PGPOPasskeyBuffer( mContext, mCAPassKey, mCAPassKeySize ) :
									PGPOPassphrase( mContext, mCAPassphrase),
								PGPOLastOption( mContext ) ),
							PGPOKeyServerSearchKey( mContext, mCATargetKey ),
							PGPOAllocatedOutputBuffer( mContext,	(void **)&expBuffer, MAX_PGPSize, &expSize ),
							PGPOLastOption( mContext ) );												CKERR;
					break;
				default:
					break;
			}
			err = PGPNewKeyDB( mContext, &tempKeyDB );													CKERR;
			err = PGPNewKeyDB( mContext, &decodeDB );													CKERR;
			err = PGPCopyKeyDBObj( mCATargetKey, decodeDB, NULL );										CKERR;
			err = PGPDecode( mContext,
					PGPOInputBuffer( mContext, expBuffer, expSize ),
					PGPODiscardOutput( mContext, TRUE ),
					PGPOKeyDBRef( mContext, decodeDB ),
					PGPOImportKeysTo( mContext, tempKeyDB ),
					PGPOInputFormat( mContext, iFormat ),
					mCASplit ?
						PGPOPasskeyBuffer( mContext, mCAPassKey, mCAPassKeySize ) :
						PGPOPassphrase( mContext, mCAPassphrase),
					PGPOLastOption( mContext ) );														CKERR;
			err = PGPCountKeysInKeyDB( tempKeyDB, &keysReturned );										CKERR;
			if( keysReturned == 0 )
				err = kPGPError_MissingX509Certificate;
			else
			{
				err = PGPCopyKeys( PGPPeekKeyDBRootKeySet( tempKeyDB ), mKeyDB, NULL );					CKERR;
			}
			break;
		}
		case kPGPCA_OP_UpdateCRL:
			switch( serverClass )
			{
				case kPGPKeyServerClass_NetToolsCA:
				case kPGPKeyServerClass_Microsoft:
				case kPGPKeyServerClass_Netscape:
					err = PGPRetrieveCertificateRevocationList( mServer,
							( serverClass == kPGPKeyServerClass_NetToolsCA ) ?
								PGPOKeyServerCAKey( mContext, caKey ) :
								PGPONullOption( mContext ),
							PGPOAllocatedOutputBuffer( mContext,  (void **)&expBuffer, MAX_PGPSize, &expSize ),
							PGPOLastOption( mContext ) );												CKERR;
					break;
				case kPGPKeyServerClass_Verisign:
				case kPGPKeyServerClass_Entrust:
					err = PGPRetrieveCertificateRevocationList( mServer,
							PGPOKeyServerCAKey( mContext, caKey ),
							PGPOSignWithKey( mContext, mCATargetKey,
								mCASplit ?
									PGPOPasskeyBuffer( mContext, mCAPassKey, mCAPassKeySize ) :
									PGPOPassphrase( mContext, mCAPassphrase),
								PGPOLastOption( mContext ) ),
							PGPOKeyDBRef( mContext, mKeyDB ),
							PGPOAllocatedOutputBuffer( mContext, (void **)&expBuffer, MAX_PGPSize, &expSize ),
							PGPOLastOption( mContext ) );												CKERR;
					break;
				default:
					break;
			}
			err = PGPDecode( mContext,
					PGPOInputBuffer( mContext, expBuffer, expSize),
					PGPODiscardOutput( mContext, TRUE),
					PGPOImportKeysTo( mContext, mKeyDB ),
					PGPOInputFormat( mContext, iFormat ),
					PGPOLastOption( mContext ) );														CKERR;
			break;
		default:
			break;
	}
	PGPKeyServerClose( mServer );
done:
	if( IsntNull( iasn ) )
		PGPFreeData( iasn );
	if( IsntNull( keyID ) )
		PGPFreeData( keyID );
	if( IsntNull( reqBuffer ) )
		PGPFreeData( reqBuffer );
	if( IsntNull( expBuffer ) )
		PGPFreeData( expBuffer );
	if( IsntNull( mCAPassKey ) )
		PGPFreeData( mCAPassKey );
	if( !mCAPassShared && IsntNull( mCAPassphrase ) )
		PGPFreeData( mCAPassphrase );
	if( PGPFilterRefIsValid( filter ) )
		PGPFreeFilter( filter );
	if( PGPKeyDBRefIsValid( decodeDB ) )
		PGPFreeKeyDB( decodeDB );
	if( PGPKeyDBRefIsValid( tempKeyDB ) )
		PGPFreeKeyDB( tempKeyDB );
	if( PGPKeyServerRefIsValid( mServer ) )
		PGPFreeKeyServer( mServer );
	mServer = kInvalidPGPKeyServerRef;
	if( PGPtlsSessionRefIsValid( tls ) )
		PGPFreeTLSSession( tls );
	if( IsntNull( mCAAVList ) )
		PGPclFreeCACertReqAVList( mCAAVList, mCANumAVs );
	if( mEndSelector && [mTarget respondsToSelector:mEndSelector] )
	{
		methodSignature = [mTarget methodSignatureForSelector:mEndSelector];
		methodInvocation = [NSInvocation invocationWithMethodSignature:methodSignature];
		[methodInvocation setSelector:mEndSelector];
		[methodInvocation setTarget:mTarget];
		[methodInvocation setArgument:&self atIndex:2];
		[methodInvocation setArgument:&err atIndex:3];
		[methodInvocation invoke];
	}
	[pool release];
}

- (PGPError)serverCallback:(PGPEvent *)event
{
	PGPError		err = kPGPError_NoErr;
	
	switch( event->type )
	{
		case kPGPEvent_KeyServerEvent:
			if( mLastState != event->data.keyServerData.state )
			{
				NSString	*statusString = nil;
				
				switch( event->data.keyServerData.state )
				{
					case kPGPKeyServerState_Opening:
						statusString = LSCL( @"Opening", @"Server status:Opening" );
						break;
					case kPGPKeyServerState_Querying:
						statusString = LSCL( @"Querying", @"Server status:Querying" );
						break;
					case kPGPKeyServerState_ReceivingResults:
						statusString = LSCL( @"Receiving", @"Server status:Receiving" );
						break;
					case kPGPKeyServerState_ProcessingResults:
						statusString = LSCL( @"Processing", @"Server status:Processing" );
						break;
					case kPGPKeyServerState_Uploading:
						statusString = LSCL( @"Sending", @"Server status:Sending" );
						break;
					case kPGPKeyServerState_Deleting:
						statusString = LSCL( @"Deleting", @"Server status:Deleting" );
						break;
					case kPGPKeyServerState_Disabling:
						statusString = LSCL( @"Disabling", @"Server status:Disabling" );
						break;
					case kPGPKeyServerState_Closing:
						statusString = LSCL( @"Closing", @"Server status:Closing" );
						break;
					default:
						break;
				}
				if( statusString && mProgressStatusField )
					[mProgressStatusField setStringValue:statusString];
				mLastState = event->data.keyServerData.state;
			}
			if( *mProgressAbort )
				err = kPGPError_UserAbort;
			break;
		case kPGPEvent_KeyServerIdleEvent:
			if( *mProgressAbort && PGPKeyServerRefIsValid( mServer ) )
				PGPCancelKeyServerCall( mServer );
			break;
		default:
			break;
	}
	return err;
}

- (void)dealloc
{
	[super dealloc];
	PGPLog(( @"CCAThread dealloc" ));
}

@end

