/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation
    All rights reserved.

    $Id: CImageButton.m,v 1.8 2002/08/06 20:09:16 dallen Exp $
____________________________________________________________________________*/

#import "pgpClientLibCocoa.h"
#import "CImageButton.h"
#import "CImageCell.h"

@implementation CImageButton

+ (Class)cellClass
{
	return [CImageCell class];
}

- (void)setMinButtonSize:(NSSize)minSize
{
    mMinButtonSize = minSize;
}

- (void)setMaxButtonSize:(NSSize)maxSize
{
    mMaxButtonSize = maxSize;
}

- (void)setMinImageSize:(NSSize)minSize
{
    mMinImageSize = minSize;
}

- (void)setMaxImageSize:(NSSize)maxSize
{
    mMaxImageSize = maxSize;
}

- (void)setFrameSize:(NSSize)size
{
	NSSize					imageSize;
	NSSize					minSize;
	NSSize					maxSize;
	NSImage*				image;
	float					delta;
	
	maxSize = [self maxButtonSize];
	minSize = [self minButtonSize];
	
	if (size.width > maxSize.width)
		size.width = maxSize.width;
	
	if (size.height > maxSize.height)
		size.height = maxSize.height;
	
	if (size.width < minSize.width)
		size.width = minSize.width;
	
	if (size.height < minSize.height)
		size.height = minSize.height;
	
	delta = size.height / minSize.height;
	size.width = minSize.width * delta;

	if (image = [self image])
	{
		[image setScalesWhenResized:YES];
		[image setDataRetained:YES];

		imageSize = NSZeroSize;
		imageSize.width = [self minImageSize].width * delta;
		imageSize.height = [self minImageSize].height * delta;
		
		[image setSize:imageSize];
		[image recache];
	}

	[super setFrameSize:size];
}

- (void)setImagePosition:(NSCellImagePosition)position
{
    PGPLog((@"setImagePosition is not supported. Images are only displayed as centered."));
}

- (NSSize)minButtonSize
{
    return mMinButtonSize;
}

- (NSSize)maxButtonSize
{
    return mMaxButtonSize;
}

- (NSSize)minImageSize
{
    return mMinImageSize;
}

- (NSSize)maxImageSize
{
    return mMaxImageSize;
}

- (void)resizeWithOldSuperviewSize:(NSSize)oldSize
{
	NSRect		frame;
	NSString*	tooltip;
	
	frame = [self frame];
	frame.size = oldSize;
	
	tooltip = [self toolTip];
	[self setToolTip:nil];
	[self setFrame:frame];
	[self setToolTip:tooltip];
}

@end
