/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation
    All rights reserved.

    $Id: CKeyGen.m,v 1.28 2002/11/11 05:26:30 wprice Exp $
____________________________________________________________________________*/
#import "CKeyGen.h"
#import <PGPclient/pflPrefTypes.h>
#import <PGPclient/pgpClientPrefs.h>
#import <PGPclient/pgpClientLibCocoa.h>
#import <PGPclient/CRemoteAuth.h>
#import <unistd.h>

#define kCKeyGenNib						@"CKeyGen"
#define kCKeyGenMinSuggestedLength		8
#define kCKeyGenNameRequiredTitle		LSCL( @"Name Required", @"KeyGen Missing Name Title" )
#define kCKeyGenNameRequiredMessage		LSCL( @"Please enter a name for the key.", @"KeyGen Missing Name message" )
#define kCKeyGenInvalidExpirationTitle	LSCL( @"Invalid Expiration", @"KeyGen Invalid Expiration Title" )
#define kCKeyGenInvalidSizeTitle		LSCL( @"Invalid Size", @"KeyGen Invalid Size Title" )
#define kCKeyGenLegacySizesTitle		LSCL( @"(1024-2048)", @"(1024-2048)" )
#define kCKeyGenBigSizesTitle			LSCL( @"(1024-4096)", @"(1024-4096)" )
#define kCKeyGenOKButton				LSCL( @"OK", @"OK" )
#define kCKeyGenMaxKeyExpirationDays	10950
#define kCKeyGenEntropyInterval			0.1

enum
{
	kPGPGenViewState_Intro		= 0,
	kPGPGenViewState_Setup,
	kPGPGenViewState_Passphrase,
	kPGPGenViewState_Random,
	kPGPGenViewState_Generate,
	kPGPGenViewState_Conclusion
};

static PGPBoolean sKeyGenAbort = FALSE;

	PGPError
generateKeyCallback(
	PGPContextRef 	context,
	PGPEvent 		*event,
	PGPUserValue 	userValue)
{
	PGPError		err = kPGPError_NoErr;
	
	if( event->type == kPGPEvent_KeyGenEvent )
	{
		if( sKeyGenAbort )
			err = kPGPError_UserAbort;
	}
	
	return( err );
}

@implementation CKeyGen

- (id)init
{
	unsigned	i;
	
	if( !mInited )
	{
		[super init];
		
		srandom(time(NULL));
		for (i = 0; i < kMaxPassphraseLength; ++i)
			mSpaceCountList[i] = ((unsigned long) random() % kMaxSpacesPerCharacter) + 1;
		
		mInited = YES;
	}
	
	if( !mLoadedNib )
		[self initWithNib:kCKeyGenNib];
    return self;
}

- (id)initWithNib:(NSString*)nibFile
{
    if( nibFile && [nibFile length] )
        [NSBundle loadNibNamed:nibFile owner:self];    
	mLoadedNib = YES;
    return [self init];
}

- (id)initWithController:(id)controller keysView:(id)keysView forKeyDB:(PGPKeyDBRef)mainKeyDB
{
	NSImage			*origKeys,
					*alphaKeys;
	NSRect			frame;
	
	[self init];
	mOwnerController = controller;
	mKeysView = keysView;
	[mKeyGenWindow setDelegate:self];
	mKeyDB = mainKeyDB;
	mGeneratedKeyDB = kInvalidPGPKeyDBRef;
	mKeyGenError = kPGPError_NoErr;
	mPGPContext = PGPPeekKeyDBContext( mKeyDB );
	mPrefRef = [mOwnerController pgpPrefRef];
	mCurrentView = kPGPGenView_Invalid;
	mExpert = FALSE;
	mPassedKeyCheck = NO;
	mAddedKey = NO;
	mGenerating = NO;
	mEnforceQuality = mEnforceMinLength = FALSE;
	mMinPassLength = kCKeyGenMinSuggestedLength;
	mADKKey = kInvalidPGPKeyDBObjRef;
	mCorpKey = kInvalidPGPKeyDBObjRef;
	mRootCAKey = kInvalidPGPKeyDBObjRef;
	mRevokerKey = kInvalidPGPKeyDBObjRef;
	[mExpertExpiration setFormatter:[[[NSDateFormatter alloc]
					initWithDateFormat:[[NSUserDefaults standardUserDefaults] stringForKey:NSShortDateFormatString]
					allowNaturalLanguage:YES] autorelease]];
	[mExpertExpiration setObjectValue:[NSDate dateWithTimeIntervalSinceNow:kPGPcl_OneYearInSeconds]];
	PGPGetPrefBoolean( mPrefRef, kPGPPrefPresetKeyGeneration, &mPreset );
	if( mPreset )
		[mExpertButton setEnabled:NO];
	PGPGetPrefBoolean( mPrefRef, kPGPPrefAutoKeyReconstruction, &mRecon );
	PGPGetPrefBoolean( mPrefRef, kPGPPrefEnforceMinChars, &mEnforceMinLength );
	if( mEnforceMinLength )
		PGPGetPrefNumber( mPrefRef, kPGPPrefMinChars, &mMinPassLength );
	PGPGetPrefBoolean( mPrefRef, kPGPPrefEnforceMinQuality, &mEnforceQuality );
	if( mEnforceQuality )
	{
		PGPGetPrefNumber( mPrefRef, kPGPPrefMinChars, &mMinPassQuality );
		[mPassMinQualityIndicator setDoubleValue:mMinPassQuality];
		[mPEQualityIndicator setDoubleValue:mMinPassQuality];
	}
	else
	{
		[mPassMinQualityIndicator removeFromSuperview];
		[mPassMinQualityField removeFromSuperview];
		[mPEQualityBox removeFromSuperview];
	}
	[mPEMinLengthField setStringValue:[NSString stringWithFormat:LSCL( @"Please use a minimum of %d characters.",
		@"PhraseError MinLength" ), mMinPassLength]];
	[mPassMinLengthField setStringValue:[NSString stringWithFormat:LSCL( @"Your passphrase should be at least %d characters long.",
		@"Passphrase MinLength" ), mMinPassLength]];
	[mNameFullName setStringValue:NSFullUserName()];
	[mExpertFullName setStringValue:NSFullUserName()];

	// Configure the Passphrase view
	[mPEConfirmationBox retain];
	[mPELengthBox retain];
	[mPEQualityBox retain];
	[mPassCapsLockField retain];
	[mGenContinueField retain];
	[mGenCurStatusIndicator retain];
	[mGenCancelButton retain];
	[mPassCapsLockField removeFromSuperview];
	[mGenContinueField removeFromSuperview];

	[mPassphraseScrollView setBorderType:NSBezelBorder];
	[mPassphraseScrollView setHasVerticalScroller:YES];
	
	frame = [mPassphraseScrollView frame];
	
	frame.size = [NSScrollView contentSizeForFrameSize:frame.size
							   hasHorizontalScroller:NO
							   hasVerticalScroller:YES
							   borderType:NSBezelBorder];
	
	if (!mPassphraseField)
	{
		mPassphraseField = [[CPGPPassphraseTextView alloc] initWithFrame:NSMakeRect(0, 0, NSWidth(frame), NSHeight(frame))];
		[mPassphraseField setAutoresizingMask:(NSViewHeightSizable | NSViewWidthSizable)];
		[mPassphraseField setAutoresizesSubviews:NO];
		[mPassphraseField setDelegate:self];
		[[mPassphraseField layoutManager] setExtraLineFragmentRect:NSMakeRect(0, 0, NSWidth(frame), 14)
								   usedRect:NSMakeRect(0, 0, 10, 14)
								   textContainer:[mPassphraseField textContainer]];
	}
	
	if (!mSpaceField)
	{
		mSpaceField = [[CPGPPassphraseTextView alloc] initWithFrame:NSMakeRect(0, 0, NSWidth(frame), NSHeight(frame))];
		[mSpaceField setAutoresizingMask:(NSViewHeightSizable | NSViewWidthSizable)];
		[mSpaceField setAutoresizesSubviews:NO];
		[mSpaceField setDelegate:self];
		[[mSpaceField layoutManager] setExtraLineFragmentRect:NSMakeRect(0, 0, NSWidth(frame), 14)
								   usedRect:NSMakeRect(0, 0, 10, 14)
								   textContainer:[mSpaceField textContainer]];
	}

	[mPassConfirmationScrollView setBorderType:NSBezelBorder];
	[mPassConfirmationScrollView setHasVerticalScroller:YES];	

	frame = [mPassphraseScrollView frame];
	
	frame.size = [NSScrollView contentSizeForFrameSize:frame.size
							   hasHorizontalScroller:NO
							   hasVerticalScroller:YES
							   borderType:NSBezelBorder];
	
	if (!mConfirmationPassphraseField)
	{
		mConfirmationPassphraseField = [[CPGPPassphraseTextView alloc] initWithFrame:NSMakeRect(0, 0, NSWidth(frame), NSHeight(frame))];
		[mConfirmationPassphraseField setAutoresizingMask:(NSViewHeightSizable | NSViewWidthSizable)];
		[mConfirmationPassphraseField setAutoresizesSubviews:NO];
		[mConfirmationPassphraseField setDelegate:self];
		[[mConfirmationPassphraseField layoutManager] setExtraLineFragmentRect:NSMakeRect(0, 0, NSWidth(frame), 14)
								   usedRect:NSMakeRect(0, 0, 10, 14)
								   textContainer:[mConfirmationPassphraseField textContainer]];
	}
	
	if (!mConfirmationSpaceField)
	{
		mConfirmationSpaceField = [[CPGPPassphraseTextView alloc] initWithFrame:NSMakeRect(0, 0, NSWidth(frame), NSHeight(frame))];
		[mConfirmationSpaceField setAutoresizingMask:(NSViewHeightSizable | NSViewWidthSizable)];
		[mConfirmationSpaceField setAutoresizesSubviews:NO];
		[mConfirmationSpaceField setDelegate:self];
		[[mConfirmationSpaceField layoutManager] setExtraLineFragmentRect:NSMakeRect(0, 0, NSWidth(frame), 14)
								   usedRect:NSMakeRect(0, 0, 10, 14)
								   textContainer:[mConfirmationSpaceField textContainer]];
	}
	
	[mPassHideTypingCheckbox setState:NSOnState];
	
	// Create translucent PGP keys image
	origKeys = [[[NSImage alloc] initByReferencingFile:
		[[NSBundle bundleForClass:[CKeyGen class]] pathForImageResource:@"pgpkeys"]] autorelease];
	alphaKeys = [[[NSImage alloc] initWithSize:[origKeys size]] autorelease];
	[alphaKeys lockFocus];
	[origKeys dissolveToPoint:NSZeroPoint fraction:0.35];
	[alphaKeys unlockFocus];
	[mKeyImageView setImage:alphaKeys];
	
	mPassedKeyCheck = [self corpKeysAreValid];
	
	[mPanelBox setContentView:mIntroView];
	[self switchToView:kPGPGenView_Intro increment:NO];

	[mKeyGenWindow center];
	[mKeyGenWindow makeKeyAndOrderFront:self];
	
	mLastMouseLocation.v = mLastMouseLocation.h = 0;
	mNeededEntropy = 0;
	mEntropyTimer = [NSTimer scheduledTimerWithTimeInterval:kCKeyGenEntropyInterval
						target:self
						selector:@selector(addEntropy:)
						userInfo:nil
						repeats:YES];
	
	return self;
}

- (void)addEntropy:(NSTimer*)timer
{
	Point		mouseLocation;
	
	if( mGenerating )
		return;
	GetMouse(&mouseLocation);
	
	if( ( mouseLocation.v != mLastMouseLocation.v ) ||
		( mouseLocation.h != mLastMouseLocation.h ) )
	{
		PGPGlobalRandomPoolAddMouse(mouseLocation.h, mouseLocation.v);
		mLastMouseLocation = mouseLocation;
	}
	if( mNeededEntropy )
	{
		char		randomKeys[128],
					*rp;
		double		soFar,
					total;
		
		[[mRandomKeyboardField stringValue] getCString:randomKeys maxLength:sizeof( randomKeys ) - 1];
		[mRandomKeyboardField setStringValue:[NSString string]];
		for( rp = &randomKeys[0]; *rp; rp++ )
			PGPGlobalRandomPoolAddKeystroke( *rp );
		
		soFar = PGPContextReserveRandomBytes( mPGPContext, ( mNeededEntropy + 7 ) / 8 ) * 8;
		total = mNeededEntropy;
		if( soFar >= total )
		{
			[mRandomProgress setDoubleValue:100];
			[mNextButton setEnabled:YES];
			mNeededEntropy = 0;
		}
		else
			[mRandomProgress setDoubleValue:(soFar / total) * 100.0];
	}
}

- (BOOL)passphraseView:(CPGPPassphraseTextView*)passphraseView shouldInsertText:(NSString*)string
{
	unsigned				numChars;
	unsigned				numSpaces;
	unsigned				spaceIndex;
	unsigned				i;
	char					spaceStr[kMaxSpacesPerCharacter + 1];	
	CPGPPassphraseTextView*	selectedPassphraseView;
	CPGPPassphraseTextView*	selectedSpaceView;
	
	if (!string)
		return NO;
	
	numChars = [string length];
	
	if ((passphraseView == mPassphraseField) || (passphraseView == mSpaceField))
	{
		selectedPassphraseView = mPassphraseField;
		selectedSpaceView = mSpaceField;
	}
	else if ((passphraseView == mConfirmationPassphraseField) || (passphraseView == mConfirmationSpaceField))
	{
		selectedPassphraseView = mConfirmationPassphraseField;
		selectedSpaceView = mConfirmationSpaceField;
	}
	else
		return YES;
	
	/* If we've hit the max passphrase length, disable any further input. */
	if (([[selectedPassphraseView string] length] + numChars) > kMaxPassphraseLength)
		return NO;
	
	spaceIndex = [[selectedPassphraseView string] length] ? [[selectedPassphraseView string] length]: 0;
	
	for (i = 0; i < numChars; ++i, ++spaceIndex)
	{
		numSpaces = mSpaceCountList[spaceIndex];
		memset(spaceStr, ' ', numSpaces);
		spaceStr[numSpaces] = '\0';
		
		[selectedSpaceView superInsertText:[NSString stringWithFormat:@"%s", spaceStr]];
	}
	
	if ((passphraseView == mSpaceField) || (passphraseView == mConfirmationSpaceField))
	{
		[selectedPassphraseView superInsertText:string];
		
		return NO;
	}
	
	return YES;
}

- (BOOL)passphraseView:(CPGPPassphraseTextView*)passphraseView shouldDeleteBackward:(id)sender
{
	unsigned				numSpaces;
	unsigned				spaceIndex;
	unsigned				passLength;
	unsigned				spaceLength;
	unsigned				i;
	CPGPPassphraseTextView*	selectedPassphraseView;
	CPGPPassphraseTextView*	selectedSpaceView;
	
	if ((passphraseView == mPassphraseField) || (passphraseView == mSpaceField))
	{
		selectedPassphraseView = mPassphraseField;
		selectedSpaceView = mSpaceField;
	}
	else if ((passphraseView == mConfirmationPassphraseField) || (passphraseView == mConfirmationSpaceField))
	{
		selectedPassphraseView = mConfirmationPassphraseField;
		selectedSpaceView = mConfirmationSpaceField;
	}
	else
		return YES;
	
	passLength = [[selectedPassphraseView string] length];
	spaceLength = [[selectedSpaceView string] length];
	
	if (passLength)
	{
		[selectedPassphraseView superDeleteBackward:self];
		
		spaceIndex = passLength - 1;
		numSpaces = mSpaceCountList[spaceIndex];
		
		for (i = 0; i < numSpaces; ++i)
			[selectedSpaceView superDeleteBackward:self];
		
		return NO;
	}
	
	return YES;
}

- (BOOL)shouldPassphraseViewDisplayMarkedText:(CPGPPassphraseTextView*)passphraseView
{
	if ((passphraseView == mSpaceField) || (passphraseView == mConfirmationSpaceField))
		return NO;
	
	return YES;
}

- (void)capsLockStatusDidChange:(BOOL)enabled
{
	if (enabled)
		[mPassphraseView addSubview:mPassCapsLockField];
	else
		[mPassCapsLockField removeFromSuperview];
}

- (PGPError)sendToServer:(PGPKeyDBObjRef)key
{
	PGPError			err = kPGPError_NoErr;
	PGPKeyServerEntry	rootServer;
	PGPKeySetRef		singleKeySet = kInvalidPGPKeySetRef;
	PGPKeySetRef		failedKeys = kInvalidPGPKeySetRef;
	PGPKeyServerSpec	serverSpec;
	PGPtlsSessionRef	tls = kInvalidPGPtlsSessionRef;
	
	[mGenStatusField setStringValue:LSCL( @"Connecting to Keyserver", @"KeyGen Status: STS1" )];
	if( sKeyGenAbort )
		err = kPGPError_UserAbort;														CKERR;
	err = PGPGetRootKeyServer( mPrefRef, &rootServer );									CKERR;
	serverSpec.serverName = rootServer.serverDNS;
	err = PGPNewOneKeySet( key, &singleKeySet );										CKERR;
	err = PGPNewKeyServer( mPGPContext, kPGPKeyServerClass_PGP, &serverSpec.server,
			PGPONetHostName( mPGPContext, rootServer.serverDNS, rootServer.serverPort ),
			PGPOKeyServerProtocol( mPGPContext, rootServer.protocol ),
			PGPOKeyServerAccessType( mPGPContext, kPGPKeyServerAccessType_Normal ),
			PGPOKeyServerKeySpace( mPGPContext, kPGPKeyServerKeySpace_Normal ),
			PGPOLastOption( mPGPContext ) );											CKERR;
	err = PGPSetKeyServerIdleEventHandler( generateKeyCallback, self );					CKERR;
	err = PGPSetKeyServerEventHandler( serverSpec.server, generateKeyCallback, self );	CKERR;
	if( rootServer.protocol == kPGPKeyServerProtocol_LDAPS )
		err = PGPNewTLSSession( [mOwnerController tlsContext], &tls );					CKERR;
	err = PGPKeyServerOpen( serverSpec.server, tls );									CKERR;
	[mGenStatusField setStringValue:LSCL( @"Sending to server", @"KeyGen Status: STS2" )];
	err = PGPUploadToKeyServer( serverSpec.server, singleKeySet, &failedKeys );			CKERR;
	[mGenStatusField setStringValue:LSCL( @"Disconnecting", @"KeyGen Status: STS3" )];
	PGPKeyServerClose( serverSpec.server );
done:
	if( PGPKeyServerRefIsValid( serverSpec.server ) )
		PGPFreeKeyServer( serverSpec.server );
	if( PGPtlsSessionRefIsValid( tls ) )
		PGPFreeTLSSession( tls );
	if( PGPKeySetRefIsValid( singleKeySet ) )
		PGPFreeKeySet( singleKeySet );
	if( PGPKeySetRefIsValid( failedKeys ) )
		PGPFreeKeySet( failedKeys );
	return err;
}

- (PGPError)sendReconData:(PGPKeyDBObjRef)key
{
	PGPError			err = kPGPError_NoErr;
	PGPUInt32			numPref;
	PGPReconContextRef	reconRef = kInvalidPGPReconContextRef;
	char				username[256],
						password[256];
	
	[mGenStatusField setStringValue:LSCL( @"Connecting to Reconstruction Server", @"KeyGen Status: SRD1" )];
	if( sKeyGenAbort )
		err = kPGPError_UserAbort;																					CKERR;
	PGPGetPrefNumber( mPrefRef, kPGPPrefLDAPReconServerType, &numPref );
	username[0] = password[0] = '\0';
	if( numPref != kPGPKeyServerClass_PGP )
	{
		// Get username and password
		//if( !GetLegacyAuth( username, password ) )
		//	err = kPGPError_UserAbort;																				CKERR;
	}
	err = PGPNewReconstruct( key, mPrefRef, username, password, generateKeyCallback, self, &reconRef );				CKERR;
	err = PGPMakeReconstruction( reconRef, mReconPrompts, mReconPasses, (char *)[[mPassphraseField string] UTF8String] );	CKERR;
	[mGenStatusField setStringValue:LSCL( @"Sending to Reconstruction Server", @"KeyGen Status: SRD2" )];
	err = PGPSendReconstruction( reconRef );																		CKERR;
done:
	if( PGPReconContextRefIsValid( reconRef ) )
		PGPFreeReconstruct( reconRef );
	pgpClearMemory( password, sizeof(password) );
	return err;
}

- (void)generateKey:(id)sender
{
	NSAutoreleasePool		*pool = [[NSAutoreleasePool alloc] init];
	PGPError				err = kPGPError_NoErr;
	NSMutableString			*nameString;
	NSString				*emailString;
	PGPPublicKeyAlgorithm	algorithm = kPGPPublicKeyAlgorithm_DSA;
	PGPUInt32				signingSize = 1024,
							encryptionSize = 2048,
							expires = 0,
							numAllowed = 0;
	PGPBoolean				fastGen,
							sendToServer,
							requestCert;
	PGPCipherAlgorithm		*allowedAlgs = NULL;
	PGPPrefArray *			allowedArray;
	PGPKeySetRef			adkKeySet = kInvalidPGPKeySetRef,
							revokerKeySet = kInvalidPGPKeySetRef;
	PGPBoolean 				enforce = FALSE;
	PGPKeyDBObjRef			key = kInvalidPGPKeyDBObjRef;
	NSImage					*checkImage,
							*xImage;
	
	[mGenStatusField performSelectorOnMainThread:@selector(setStringValue:)
		withObject:LSCL( @"Generating Key", @"KeyGen Status1" ) waitUntilDone:NO];
	checkImage = [[[NSImage alloc] initByReferencingFile:[[NSBundle bundleForClass:[CKeyGen class]]
					pathForImageResource:@"check"]] autorelease];
	xImage = [[[NSImage alloc] initByReferencingFile:[[NSBundle bundleForClass:[CKeyGen class]]
					pathForImageResource:@"xfail"]] autorelease];
	PGPGetPrefBoolean( mPrefRef, kPGPPrefAlwaysSendKeys, &sendToServer );
	PGPGetPrefBoolean( mPrefRef, kPGPPrefKeyGenX509CertRequest, &requestCert );
	[mGenStep1Field performSelectorOnMainThread:@selector(setStringValue:)
		withObject:LSCL( @"Generate Key", @"KeyGen State1" ) waitUntilDone:NO];
	if( sendToServer )
		[mGenStep2Field performSelectorOnMainThread:@selector(setStringValue:)
			withObject:LSCL( @"Send to Server", @"KeyGen State2" ) waitUntilDone:NO];
	if( requestCert )
		[mGenStep3Field performSelectorOnMainThread:@selector(setStringValue:)
			withObject:LSCL( @"Request Certificate", @"KeyGen State3" ) waitUntilDone:NO];
	if( mRecon )
		[mGenStep4Field performSelectorOnMainThread:@selector(setStringValue:)
			withObject:LSCL( @"Send Reconstruction Data", @"KeyGen State4" ) waitUntilDone:NO];
	if( mPreset )
	{
		PGPUInt32	alg;
		
		PGPGetPrefNumber( mPrefRef, kPGPPrefKeySize, &encryptionSize );
		PGPGetPrefNumber( mPrefRef, kPGPPrefKeyExpiration, &expires );
		PGPGetPrefNumber( mPrefRef, kPGPPrefKeyPublicKeyAlgorithm, &alg );
		algorithm = (PGPPublicKeyAlgorithm) alg;
	}
	else if( mExpert )
		[self getExpertParams:&algorithm size:&encryptionSize expires:&expires];
	if( mExpert )
	{
		nameString	= [NSMutableString stringWithString:[mExpertFullName stringValue]];
		emailString	= [mExpertEmail stringValue];
	}
	else
	{
		nameString	= [NSMutableString stringWithString:[mNameFullName stringValue]];
		emailString	= [mNameEmail stringValue];
	}
	if( [emailString length] )
		[nameString appendFormat:@" <%@>", emailString];
	
	if( algorithm != kPGPPublicKeyAlgorithm_DSA )
		signingSize = encryptionSize;
	else
		signingSize = 1024;
	PGPGetPrefBoolean( mPrefRef, kPGPPrefFastKeyGen, &fastGen);
	err = PGPGetPrefArray( mPrefRef, kPGPPrefAllowedAlgorithmsList, &allowedArray );
	if( IsntPGPError( err ) )
	{
		PGPUInt32	allowedIndex = 0,
					origAllowed,
					origIndex;
		
		origAllowed = allowedArray->numElements;
		allowedAlgs = (PGPCipherAlgorithm *)PGPNewData( PGPPeekContextMemoryMgr( mPGPContext ),
						32 * sizeof(PGPCipherAlgorithm), kPGPMemoryMgrFlags_Clear );
		if( IsntNull( allowedAlgs ) )
		{
			for( origIndex = 0; origIndex < origAllowed; origIndex++ )
			{
				PGPUInt32	alg;
				
				PGPGetPrefArrayNumber( mPrefRef, allowedArray, origIndex, &alg );
				allowedAlgs[allowedIndex++] = (PGPCipherAlgorithm)alg;
				if( (PGPCipherAlgorithm)alg == kPGPCipherAlgorithm_AES256 )
				{
					allowedAlgs[allowedIndex++] = kPGPCipherAlgorithm_AES192;
					allowedAlgs[allowedIndex++] = kPGPCipherAlgorithm_AES128;
				}
			}
			numAllowed = allowedIndex;
		}
		PGPFreePrefArray( allowedArray );
	}
	if( PGPKeyDBObjRefIsValid( mADKKey ) )
	{
		err = PGPNewOneKeySet( mADKKey, &adkKeySet );	CKERR;
		PGPGetPrefBoolean( mPrefRef, kPGPPrefEnforceIncomingADK, &enforce );
	}
	if( ( algorithm != kPGPPublicKeyAlgorithm_RSA ) && PGPKeyDBObjRefIsValid( mRevokerKey ) )
	{
		err = PGPNewOneKeySet( mRevokerKey, &revokerKeySet );	CKERR;
	}
	err = PGPNewKeyDB( mPGPContext, &mGeneratedKeyDB );	CKERR;
	err = PGPGenerateKey( mPGPContext, &key,
						PGPOKeyGenParams( mPGPContext, ( algorithm == kPGPPublicKeyAlgorithm_RSASignOnly ) ?
							kPGPPublicKeyAlgorithm_RSA : algorithm, signingSize ),
						PGPOKeyGenName( mPGPContext, [nameString UTF8String], strlen([nameString UTF8String])),
						PGPOKeyDBRef( mPGPContext, mGeneratedKeyDB ),
						PGPOPassphrase( mPGPContext, [[mPassphraseField string] UTF8String] ),
						PGPOKeyGenFast( mPGPContext, fastGen ),
						PGPOExpiration( mPGPContext, expires ),
						PGPOPreferredAlgorithms( mPGPContext, allowedAlgs, numAllowed ),
						PGPOEventHandler( mPGPContext, generateKeyCallback, self ),
						PGPKeySetRefIsValid( adkKeySet ) ?
							PGPOAdditionalRecipientRequestKeySet( mPGPContext, adkKeySet, enforce ? 0x80 : 0 ) :
							PGPONullOption( mPGPContext ),
						PGPKeySetRefIsValid( revokerKeySet ) ?
							PGPORevocationKeySet( mPGPContext, revokerKeySet ) :
							PGPONullOption( mPGPContext ),
						( algorithm != kPGPPublicKeyAlgorithm_RSA ) ?
							PGPOKeyFlags( mPGPContext, kPGPKeyPropertyFlags_UsageSign ) :
							PGPONullOption( mPGPContext ),
						PGPOLastOption( mPGPContext ) );
	if( IsPGPError( err ) )
	{
		[mGenStep1Image performSelectorOnMainThread:@selector(setImage:)
			withObject:xImage waitUntilDone:NO];
		goto done;
	}
	[mGenOverallIndicator setDoubleValue:33];
	if( algorithm != kPGPPublicKeyAlgorithm_RSA)
	{
		PGPKeyDBObjRef subKey;
		
		[mGenStatusField performSelectorOnMainThread:@selector(setStringValue:)
			withObject:LSCL( @"Generating SubKey", @"KeyGen Status2" ) waitUntilDone:NO];
		err = PGPGenerateSubKey( mPGPContext, &subKey,
				PGPOKeyGenMasterKey( mPGPContext, key ),
				PGPOKeyGenParams( mPGPContext, algorithm == kPGPPublicKeyAlgorithm_DSA ?
						kPGPPublicKeyAlgorithm_ElGamal : kPGPPublicKeyAlgorithm_RSA, encryptionSize ),
				PGPOPassphrase( mPGPContext, [[mPassphraseField string] UTF8String] ),
				PGPOExpiration( mPGPContext, expires ),
				PGPOKeyFlags( mPGPContext, kPGPKeyPropertyFlags_UsageEncrypt ),
				PGPOKeyGenFast( mPGPContext, fastGen ),
				PGPOEventHandler( mPGPContext, generateKeyCallback, self ),
				PGPOLastOption( mPGPContext ) );
	}
	if( IsntPGPError( err ) )
		[mGenStep1Image setImage:checkImage];
	else
	{
		PGPKeySetRef genKeySet;
		
		[mGenStep1Image setImage:xImage];		
		if( IsntPGPError( PGPNewOneKeySet( key, &genKeySet ) ) )
		{
			PGPDeleteKeys( genKeySet );
			PGPFreeKeySet( genKeySet );
		}
		goto done;
	}
	[mGenOverallIndicator setDoubleValue:66];
	
	if( PGPclEnterprise() )
	{
		if( IsntPGPError( err ) )
		{
			PGPKeyDBObjRef	userID;
			
			if( mCorpKey )
			{
				PGPBoolean		metaCorp = FALSE;
	
				[mGenStatusField performSelectorOnMainThread:@selector(setStringValue:)
					withObject:LSCL( @"Signing Corporate Key", @"KeyGen Status3" ) waitUntilDone:NO];
				PGPGetPrefBoolean( mPrefRef, kPGPPrefMetaIntroducerCorp, &metaCorp);
				PGPGetPrimaryUserID( mCorpKey, &userID );
				PGPclGuaranteeMinimumEntropy( mPGPContext );
				PGPCertifyUserID( userID, key,
						PGPOPassphrase( mPGPContext, [[mPassphraseField string] UTF8String] ),
						PGPOExpiration( mPGPContext, 0 ),
						PGPOExportable( mPGPContext, FALSE ),
						PGPOSigTrust( mPGPContext, metaCorp ? 2 : 0, kPGPKeyTrust_Complete ),
						PGPOLastOption( mPGPContext ) );
				PGPSetKeyTrust( mCorpKey, kPGPKeyTrust_Complete );
				// Valid errors can occur here which do not appear
				// to have reasonable result codes, so we ignore errors
				// on this call.  For instance, an error will be returned
				// if the corporate key is on the keyring and is implicitly
				// trusted.
				[mGenOverallIndicator setDoubleValue:70];
			}
			if( mRootCAKey )
			{
				[mGenStatusField performSelectorOnMainThread:@selector(setStringValue:)
					withObject:LSCL( @"Signing Root CA", @"KeyGen Status4" ) waitUntilDone:NO];
				PGPGetPrimaryUserID( mRootCAKey, &userID );
				PGPclGuaranteeMinimumEntropy( mPGPContext );
				PGPCertifyUserID( userID, key,
						PGPOPassphrase( mPGPContext, [[mPassphraseField string] UTF8String] ),
						PGPOExpiration( mPGPContext, 0 ),
						PGPOExportable( mPGPContext, FALSE ),
						PGPOSigTrust( mPGPContext, 0, kPGPKeyTrust_Complete ),
						PGPOLastOption( mPGPContext ) );
				[mGenOverallIndicator setDoubleValue:75];
			}
			if( sendToServer )
			{
				err = [self sendToServer:key];
				if( IsntPGPError( err ) )
					[mGenStep2Image setImage:checkImage];
				else
				{
					[mGenStep2Image setImage:xImage];
					PGPclErrorBox( err );
				}
				[mGenOverallIndicator setDoubleValue:85];
			}
			if( requestCert )
			{
				[mGenOverallIndicator setDoubleValue:92];
			}
			if( mRecon )
			{
				err = [self sendReconData:key];
				if( IsntPGPError( err ) )
					[mGenStep4Image setImage:checkImage];
				else
				{
					[mGenStep4Image setImage:xImage];
					PGPclErrorBox( err );
				}
				[mGenOverallIndicator setDoubleValue:97];
			}
		}
	}
done:
	if( err == kPGPError_OutOfMemory )
		err = kPGPError_UserAbort;		// compensate for the oldest living SDK bug
	mKeyGenError = err;
	if( IsPGPError( mKeyGenError ) )
		[mGenStatusField performSelectorOnMainThread:@selector(setStringValue:)
			withObject:[NSString stringWithFormat:LSCL( @"Generation Error: %@", @"KeyGen Error" ),
			PGPGetErrorNSString( mKeyGenError ) ] waitUntilDone:NO];
	else
	{
		[mGenStatusField performSelectorOnMainThread:@selector(setStringValue:)
			withObject:LSCL( @"Done.", @"KeyGen Done" ) waitUntilDone:NO];
		[mGenerationView addSubview:mGenContinueField];
		[mGenOverallIndicator setDoubleValue:100];
		[mNextButton setEnabled:YES];
	}
	[mGenCurStatusIndicator removeFromSuperview];
	[mGenCancelButton removeFromSuperview];
	if( PGPKeySetRefIsValid( adkKeySet ) )
		PGPFreeKeySet( adkKeySet );
	if( PGPKeySetRefIsValid( revokerKeySet ) )
		PGPFreeKeySet( revokerKeySet );
	if( IsntNull( allowedAlgs ) )
		PGPFreeData( allowedAlgs );
	mGenerating = FALSE;
	[pool release];
}

- (BOOL)corpKeysAreValid
{
	BOOL			valid = YES;
	PGPBoolean		useADK,
					autoCorp,
					addRevoker,
					autoRootCA;
	PGPKeyID		*keyID;
	PGPSize			len;
	
	if( PGPclEnterprise() )
	{
		// Set MRK on this key?
		PGPGetPrefBoolean( mPrefRef, kPGPPrefUseDHADK, &useADK );
		if( useADK )
		{
			if( IsntPGPError( PGPGetPrefData( mPrefRef, kPGPPrefDHADKID, &len, (void **)&keyID ) ) )
			{
				PGPFindKeyByKeyID( mKeyDB, keyID, &mADKKey);
				if( PGPKeyDBObjRefIsValid( mADKKey ) )
					[mCorpADKField setStringValue:PGPclGetPrimaryUserIDName( mADKKey )];
				PGPFreeData( keyID );
			}
		}
		else
			[mCorpADKBox removeFromSuperview];
		// Should we auto-sign/trust the Corporate Key?
		PGPGetPrefBoolean( mPrefRef, kPGPPrefAutoSignTrustCorp, &autoCorp );
		if( autoCorp )
		{
			if( IsntPGPError( PGPGetPrefData( mPrefRef, kPGPPrefCorpKeyID, &len, (void **)&keyID ) ) )
			{
				PGPFindKeyByKeyID( mKeyDB, keyID, &mCorpKey );
				if( PGPKeyDBObjRefIsValid( mCorpKey ) )
					[mCorpSigningField setStringValue:PGPclGetPrimaryUserIDName( mCorpKey )];
				PGPFreeData(keyID);
			}
		}
		else
			[mCorpSigningBox removeFromSuperview];
		// Should we set a designated revoker?
		PGPGetPrefBoolean( mPrefRef, kPGPPrefAutoAddRevoker, &addRevoker );
		if( addRevoker )
		{
			if( IsntPGPError( PGPGetPrefData( mPrefRef, kPGPPrefRevokerKeyID, &len, (void **)&keyID ) ) )
			{
				PGPFindKeyByKeyID( mKeyDB, keyID, &mRevokerKey );
				if( PGPKeyDBObjRefIsValid( mRevokerKey ) )
					[mCorpRevokerField setStringValue:PGPclGetPrimaryUserIDName( mRevokerKey )];
				PGPFreeData(keyID);
			}
		}
		else
			[mCorpRevokerBox removeFromSuperview];
		// Should we auto-sign/trust the RootCA Key?
		PGPGetPrefBoolean( mPrefRef, kPGPPrefAutoSignRootCAKey, &autoRootCA );
		if( autoRootCA )
		{
			if( IsntPGPError( PGPGetPrefData( mPrefRef, kPGPPrefCARootKeyID, &len, (void **)&keyID ) ) )
			{
				PGPFindKeyByKeyID( mKeyDB, keyID, &mRootCAKey );
				PGPFreeData(keyID);
			}
		}
		if( useADK && !PGPKeyDBObjRefIsValid( mADKKey ) )
			valid = false;
		else if( autoCorp && !PGPKeyDBObjRefIsValid( mCorpKey ) )
			valid = false;
		else if( autoRootCA && !PGPKeyDBObjRefIsValid( mRootCAKey ) )
			valid = false;
		else if( addRevoker && !PGPKeyDBObjRefIsValid( mRevokerKey ) )
			valid = false;
	}
	return valid;
}

- (BOOL)getExpertParams:(PGPPublicKeyAlgorithm *)algorithm
	size:(PGPUInt32 *)size
	expires:(PGPUInt32 *)expires
{
	BOOL		validEntries = YES;
	
	if( ![[mExpertFullName stringValue] length] )
	{
		validEntries = NO;
		NSBeginAlertSheet( kCKeyGenNameRequiredTitle, kCKeyGenOKButton, nil, nil, mKeyGenWindow, 
			self, nil, nil, nil, kCKeyGenNameRequiredMessage );
		goto done;
	}
	
	switch( [mExpertAlgPopup indexOfSelectedItem] )
	{
		case 0:
			*algorithm = kPGPPublicKeyAlgorithm_DSA;
			break;
		case 1:
			*algorithm = kPGPPublicKeyAlgorithm_RSASignOnly;
			break;
		case 2:
			*algorithm = kPGPPublicKeyAlgorithm_RSA;
			break;
		default:
			pgpAssert(0);
			validEntries = NO;
			goto done;
	}
	*size = [mExpertSize intValue];
	if( *algorithm == kPGPPublicKeyAlgorithm_RSA )
	{
		if( *size < 1024 || *size > 2048 )
		{
			validEntries = NO;
			NSRunAlertPanel( kCKeyGenInvalidSizeTitle,
				LSCL( @"RSA Legacy keys must be at least 1024 and at most 2048 bits.", @"KeyGen Invalid Size Message" ), 
				kCKeyGenOKButton, nil, nil );
			goto done;
		}
	}
	else
	{
		if( *size < 1024 || *size > 4096 )
		{
			validEntries = NO;
			NSRunAlertPanel( kCKeyGenInvalidSizeTitle,
				LSCL( @"Key sizes must be at least 1024 and at most 4096 bits.", @"KeyGen Invalid Size2 Message" ), 
				kCKeyGenOKButton, nil, nil );
			goto done;
		}
	}
	if( ![mExpertExpirationMatrix selectedColumn] )
		*expires = 0;
	else
	{
		NSDate		*expireDate = [mExpertExpiration objectValue],
					*timeNow = [NSDate dateWithTimeIntervalSinceNow:0];
		
		if( [expireDate compare:timeNow] != NSOrderedDescending )
		{
			validEntries = NO;
			NSRunAlertPanel( kCKeyGenInvalidExpirationTitle,
				LSCL( @"The expiration date must be in the future.", @"KeyGen Invalid Expiration Message" ), 
				kCKeyGenOKButton, nil, nil );
			goto done;
		}
		else
		{
			*expires = ( [expireDate timeIntervalSince1970] - [timeNow timeIntervalSince1970] ) / 86400 + 1;
			if( *expires > kCKeyGenMaxKeyExpirationDays )
			{
				validEntries = NO;
				NSRunAlertPanel( kCKeyGenInvalidExpirationTitle,
					LSCL( @"The expiration date is too far in the future. Please decrease the expiration time.", @"KeyGen Invalid Expiration2 Message" ), 
					kCKeyGenOKButton, nil, nil );
				goto done;
			}
		}
	}
done:
	return validEntries;
}

- (BOOL)getReconParams
{
	BOOL		validEntries = YES;
	PGPInt32	reconIndex;
	
	[[mReconQ1Field stringValue] getCString:mReconPrompts[0] maxLength:kPGPRecon_MaxPromptLen];
	[[mReconQ2Field stringValue] getCString:mReconPrompts[1] maxLength:kPGPRecon_MaxPromptLen];
	[[mReconQ3Field stringValue] getCString:mReconPrompts[2] maxLength:kPGPRecon_MaxPromptLen];
	[[mReconQ4Field stringValue] getCString:mReconPrompts[3] maxLength:kPGPRecon_MaxPromptLen];
	[[mReconQ5Field stringValue] getCString:mReconPrompts[4] maxLength:kPGPRecon_MaxPromptLen];
	
	[[mReconA1Field stringValue] getCString:mReconPasses[0] maxLength:kPGPRecon_MaxPassLen];
	[[mReconA2Field stringValue] getCString:mReconPasses[1] maxLength:kPGPRecon_MaxPassLen];
	[[mReconA3Field stringValue] getCString:mReconPasses[2] maxLength:kPGPRecon_MaxPassLen];
	[[mReconA4Field stringValue] getCString:mReconPasses[3] maxLength:kPGPRecon_MaxPassLen];
	[[mReconA5Field stringValue] getCString:mReconPasses[4] maxLength:kPGPRecon_MaxPassLen];
	
	for( reconIndex = 0; reconIndex < kPGPRecon_NumShares; reconIndex++ )
	{
		if( !mReconPrompts[reconIndex][0] || !mReconPasses[reconIndex][0] )
		{
			NSRunAlertPanel( LSCL( @"Missing Reconstruction Entries", @"KeyGen recon error title"),
				LSCL( @"You must fill in all fields to enable key reconstruction.", @"KeyGen recon error message" ), 
				kCKeyGenOKButton, nil, nil );
			validEntries = NO;
			break;
		}
	}
	return validEntries;
}

- (void)switchToView:(PGPGenViewID)newView increment:(BOOL)increment
{
	BOOL	directional = NO,
			abort = NO;
	
	if( newView == kPGPGenView_Invalid )
	{
		directional = YES;
		if( increment )
			newView = mCurrentView + 1;
		else
			newView = mCurrentView - 1;
	}
	if( directional )
	{
		if( newView == kPGPGenView_Expert && !mExpert )
		{
			if( increment )
				newView++;
			else
				newView--;
		}
		if( newView == kPGPGenView_Name && mExpert )
		{
			if( increment )
				newView++;
			else
				newView--;
		}
		if( newView == kPGPGenView_CorpWarn )
		{
			if( !mPassedKeyCheck )
			{
				NSRunAlertPanel( LSCL( @"Missing Required Keys", @"KeyGen Missing Keys Title" ),
					LSCL( @"One or more keys required to generate keys in your copy of PGP are missing. Contact your PGP Administrator.", @"KeyGen Missing Keys message" ),
					kCKeyGenOKButton, nil, nil );
				abort = YES;
			}
			else if( !PGPKeyDBObjRefIsValid( mADKKey ) &&
				!PGPKeyDBObjRefIsValid( mCorpKey ) &&
				!PGPKeyDBObjRefIsValid( mRevokerKey ) &&
				!PGPKeyDBObjRefIsValid( mRootCAKey ) )
			{
				if( increment )
					newView++;
				else
					newView--;
			}
		}
		if( newView == kPGPGenView_Name && mExpert )
		{
			if( increment )
				newView++;
			else
				newView--;
		}
		if( newView == kPGPGenView_Recon )
		{
			if( !mRecon )
			{
				if( increment )
					newView++;
				else
					newView--;
			}
		}
		if( newView == kPGPGenView_PassError )
		{
			if( !increment )
				newView--;
			else if( [self phraseIsGood] )
			{
				newView++;
			}
		}
		if( newView == kPGPGenView_Recon )
		{
			if( !mRecon )
			{
				if( increment )
					newView++;
				else
					newView--;
			}
		}
		if( newView == kPGPGenView_Random )
		{
			if( [self hasNeededEntropy] )
				if( increment )
					newView++;
		}
	}
	if( !abort && [self leaveView:newView] )
		[self enterView:newView];
	if( abort )
		[mKeyGenWindow performClose:self];
}

- (BOOL)hasNeededEntropy
{
	PGPPublicKeyAlgorithm	algorithm;
	PGPUInt32				signingSize,
							encryptionSize,
							expires;
	PGPBoolean				fastGen;
	double					soFar,
							total;

	if( mPreset )
	{
		PGPUInt32	alg;
		
		PGPGetPrefNumber( mPrefRef, kPGPPrefKeySize, &encryptionSize );
		PGPGetPrefNumber( mPrefRef, kPGPPrefKeyPublicKeyAlgorithm, &alg );
		algorithm = (PGPPublicKeyAlgorithm) alg;
	}
	else if( mExpert )
		[self getExpertParams:&algorithm size:&encryptionSize expires:&expires];
	else
	{
		algorithm = kPGPPublicKeyAlgorithm_DSA;
		encryptionSize = 2048;
	}
	if( algorithm != kPGPPublicKeyAlgorithm_DSA )
		signingSize = encryptionSize;
	else
		signingSize = 1024;
	PGPGetPrefBoolean( mPrefRef, kPGPPrefFastKeyGen, &fastGen);
	mNeededEntropy = PGPGetKeyEntropyNeeded( mPGPContext,
			PGPOKeyGenParams( mPGPContext, algorithm, signingSize ),
			PGPOKeyGenFast( mPGPContext, fastGen ),
			PGPOLastOption( mPGPContext ) );
	if( algorithm != kPGPPublicKeyAlgorithm_RSA )
	{
		mNeededEntropy += PGPGetKeyEntropyNeeded( mPGPContext,
			PGPOKeyGenParams( mPGPContext,
				algorithm == kPGPPublicKeyAlgorithm_DSA ?
				kPGPPublicKeyAlgorithm_ElGamal :
				kPGPPublicKeyAlgorithm_RSAEncryptOnly, encryptionSize ),
			PGPOKeyGenFast( mPGPContext, fastGen ),
			PGPOLastOption( mPGPContext ) );
	}
	soFar = PGPContextReserveRandomBytes( mPGPContext, ( mNeededEntropy + 7 ) / 8 ) * 8;
	total = mNeededEntropy;
	if( soFar >= total )
	{
		mNeededEntropy = 0;
		return YES;
	}
	else
		return NO;
}

- (void)enterView:(PGPGenViewID)viewID
{
	NSView		*view = nil;
	PGPUInt32	displayState = 0;
	NSString	*displayTitle = nil;
	NSResponder	*firstResponder = nil;
	
	[mNextButton setEnabled:YES];
	[mPrevButton setEnabled:YES];
	switch( viewID )
	{
		case kPGPGenView_Intro:
			view = mIntroView;
			mExpert = FALSE;
			[mPrevButton setEnabled:NO];
			displayState = kPGPGenViewState_Intro;
			break;
		case kPGPGenView_Expert:
			view = mExpertView;
			displayState = kPGPGenViewState_Setup;
			firstResponder = mExpertFullName;
			break;
		case kPGPGenView_Name:
			view = mNameView;
			displayState = kPGPGenViewState_Setup;
			firstResponder = mNameFullName;
			break;
		case kPGPGenView_CorpWarn:
			view = mCorpWarnView;
			displayState = kPGPGenViewState_Setup;
			break;
		case kPGPGenView_Passphrase:
			view = mPassphraseView;
			displayState = kPGPGenViewState_Passphrase;
			[mPassphraseField clearPassphrase];
			[mSpaceField clearPassphrase];
			[mConfirmationPassphraseField clearPassphrase];
			[mConfirmationSpaceField clearPassphrase];
			[self updatePassphraseQuality:0 sender:mPassphraseField];
			[self hideTypingPressed:self];
			
			if ([mPassHideTypingCheckbox state] == NSOnState)
				firstResponder = mSpaceField;
			else
				firstResponder = mPassphraseField;
			break;
		case kPGPGenView_PassError:
		{
			BOOL	noMatch,
					tooShort,
					lowQuality;
			
			view = mPhraseErrorView;
			[self phraseAttributes:&noMatch tooShort:&tooShort lowQuality:&lowQuality];
			if( noMatch || lowQuality || ( tooShort && mEnforceMinLength ) )
				[mNextButton setEnabled:NO];
			if( noMatch )
			{
				if( ![mPEConfirmationBox superview] )
					[mPhraseErrorView addSubview:mPEConfirmationBox];
			}
			else
				[mPEConfirmationBox removeFromSuperview];
			if( lowQuality )
			{
				if( ![mPEQualityBox superview] )
					[mPhraseErrorView addSubview:mPEQualityBox];
			}
			else
				[mPEQualityBox removeFromSuperview];
			if( tooShort )
			{
				if( ![mPELengthBox superview] )
					[mPhraseErrorView addSubview:mPELengthBox];
			}
			else
				[mPELengthBox removeFromSuperview];
			displayState = kPGPGenViewState_Passphrase;
			break;
		}
		case kPGPGenView_Recon:
			view = mReconView;
			displayState = kPGPGenViewState_Passphrase;
			firstResponder = mReconQ1Field;
			break;
		case kPGPGenView_Random:
			view = mRandomView;
			firstResponder = mRandomKeyboardField;
			[mPrevButton setEnabled:NO];
			[mNextButton setEnabled:NO];
			displayState = kPGPGenViewState_Random;
			break;
		case kPGPGenView_Generate:
			view = mGenerationView;
			[mPrevButton setEnabled:NO];
			[mNextButton setEnabled:NO];
			displayState = kPGPGenViewState_Generate;
			[mGenCurStatusIndicator startAnimation:self];
			sKeyGenAbort = FALSE;
			mGenerating = TRUE;
			[NSThread detachNewThreadSelector:@selector(generateKey:) toTarget:self withObject:self];
			break;
		case kPGPGenView_Conclusion:
			view = mConclusionView;
			[mPrevButton setEnabled:NO];
			displayState = kPGPGenViewState_Conclusion;
			break;
		default:
			view = nil;
			break;
	}
	if( view )
	{
		[mPanelBox setContentView:view];
		[mKeyGenWindow makeFirstResponder:firstResponder];
		mCurrentView = viewID;
	}
	switch( displayState )
	{
		case kPGPGenViewState_Intro:
			displayTitle = LSCL( @"Introduction", @"KeyGen Display State 0" );
			break;
		case kPGPGenViewState_Setup:
			displayTitle = LSCL( @"Setup", @"KeyGen Display State 1" );
			break;
		case kPGPGenViewState_Passphrase:
			displayTitle = LSCL( @"Passphrase", @"KeyGen Display State 2" );
			break;
		case kPGPGenViewState_Random:
			displayTitle = LSCL( @"Random Data", @"KeyGen Display State 3" );
			break;
		case kPGPGenViewState_Generate:
			displayTitle = LSCL( @"Generation", @"KeyGen Display State 4" );
			break;
		case kPGPGenViewState_Conclusion:
			displayTitle = LSCL( @"Conclusion", @"KeyGen Display State 5" );
			break;
	}
	if( displayTitle )
		[mPanelTitleField setStringValue:displayTitle];
	[mSetupStateField setTextColor:( displayState >= kPGPGenViewState_Setup ) ? [NSColor blackColor] : [NSColor grayColor]];
	[mPassphraseStateField setTextColor:( displayState >= kPGPGenViewState_Passphrase )?[NSColor blackColor]:[NSColor grayColor]];
	[mRandomStateField setTextColor:( displayState >= kPGPGenViewState_Random ) ? [NSColor blackColor] : [NSColor grayColor]];
	[mGenerationStateField setTextColor:( displayState >= kPGPGenViewState_Generate )?[NSColor blackColor]:[NSColor grayColor]];
	[mConclusionStateField setTextColor:( displayState >= kPGPGenViewState_Conclusion )?[NSColor blackColor]:[NSColor grayColor]];
}

- (BOOL)leaveView:(PGPGenViewID)newView
{
	BOOL	leave = YES;
	
	switch( mCurrentView )
	{
		case kPGPGenView_Name:
			if( newView > mCurrentView )
			{
				if( ![[mNameFullName stringValue] length] )
				{
					NSBeginAlertSheet( kCKeyGenNameRequiredTitle, kCKeyGenOKButton, nil, nil, mKeyGenWindow, 
						self, nil, nil, nil, kCKeyGenNameRequiredMessage );
					leave = NO;
				}
				else if( ![[mNameEmail stringValue] length] )
				{
					
					if( NSRunAlertPanel( LSCL( @"No Email Address", @"KeyGen Missing Email Title" ),
						LSCL( @"In order to take advantage of PGP's email integration features (and so others can find your key), it is strongly advised that you enter an email address.\n\nContinue without entering an email address?", @"KeyGen Missing Email message" ),
						LSCL( @"Yes", @"Yes" ), LSCL( @"No", @"No" ), nil )
						!= NSAlertDefaultReturn )
						leave = NO;
				}
			}
			break;
		case kPGPGenView_Expert:
		{
			PGPPublicKeyAlgorithm	algorithm;
			PGPUInt32				size,
									expires;
			
			if( newView > mCurrentView )
			{
				if( ![self getExpertParams:&algorithm size:&size expires:&expires] )
					leave = NO;
			}
			break;
		}
		case kPGPGenView_Recon:
			if( newView > mCurrentView )
			{
				if( ![self getReconParams] )
					leave = NO;
			}
			break;
		default:
			break;
	}
	return leave;
}

- (BOOL)phraseAttributes:(BOOL *)noMatch tooShort:(BOOL *)tooShort lowQuality:(BOOL *)lowQuality
{
	PGPUInt32		passphraseLength;
	PGPUInt32		confirmationLength;
	BOOL			good = YES;

	*lowQuality = *tooShort = *noMatch = NO;
	passphraseLength = [[mPassphraseField string] length];
	confirmationLength = [[mConfirmationPassphraseField string] length];
	if( memcmp( [[mPassphraseField string] UTF8String], [[mConfirmationPassphraseField string] UTF8String], passphraseLength ) )
	{
		*noMatch = YES;
		good = NO;
	}
	if( passphraseLength < mMinPassLength )
	{
		*tooShort = YES;
		good = NO;
	}
	if( mEnforceQuality )
	{
		if( PGPEstimatePassphraseQuality( [[mPassphraseField string] UTF8String] ) < mMinPassQuality )
		{
			*lowQuality = YES;
			good = NO;
		}
	}
	return good;
}

- (BOOL)phraseIsGood
{
	BOOL	noMatch,
			tooShort,
			lowQuality;
	
	return [self phraseAttributes:&noMatch tooShort:&tooShort lowQuality:&lowQuality];
}

- (void)nextView
{
	if( mCurrentView != kPGPGenView_Conclusion )
		[self switchToView:kPGPGenView_Invalid increment:YES];
	else
	{
		if( PGPKeyDBRefIsValid( mGeneratedKeyDB ) && IsntPGPError( mKeyGenError ) )
		{
			if( IsntPGPError( PGPCopyKeys( PGPPeekKeyDBRootKeySet( mGeneratedKeyDB ), mKeyDB, NULL ) ) )
			{
				PGPFlushKeyDB( mKeyDB );
				if( [mKeysView lookupDefaultKey] == kInvalidPGPKeyDBObjRef )
				{
					PGPError		err = kPGPError_NoErr;
					PGPKeyIterRef	keyIter = kInvalidPGPKeyIterRef;
					PGPKeyDBObjRef	key = kInvalidPGPKeyDBObjRef;
					PGPKeyID		keyID;
					
					err = PGPNewKeyIterFromKeyDB( mGeneratedKeyDB, &keyIter );					CKERR;
					err = PGPKeyIterNextKeyDBObj( keyIter, kPGPKeyDBObjType_Key, &key );		CKERR;
					PGPGetKeyID( key, &keyID );
					PGPSetPrefData( mPrefRef, kPGPPrefDefaultKeyID, sizeof(PGPKeyID), &keyID );
					PGPclWritePrefs( mPrefRef );
				done:
					if( keyIter != kInvalidPGPKeyIterRef )
						PGPFreeKeyIter( keyIter );
				}
				mAddedKey = YES;
			}
		}
		[mKeyGenWindow performClose:self];
	}
}

- (void)prevView
{
	if( mCurrentView != kPGPGenView_Intro )
		[self switchToView:kPGPGenView_Invalid increment:NO];
}

- (IBAction)expertPressed:(id)sender
{
	mExpert = YES;
	[self nextView];
}

- (IBAction)algorithmChanged:(id)sender
{
	switch( [sender indexOfSelectedItem] )
	{
		case 0:
		case 1:
			[mExpertSizeRange setStringValue:kCKeyGenBigSizesTitle];
			break;
		case 2:
			[mExpertSizeRange setStringValue:kCKeyGenLegacySizesTitle];
			break;
		default:
			break;
	}
}

- (IBAction)hideTypingPressed:(id)sender
{
	if ([mPassHideTypingCheckbox state] == NSOnState)
	{
		[mPassphraseScrollView setDocumentView:mSpaceField];
		[mPassConfirmationScrollView setDocumentView:mConfirmationSpaceField];
		
		[mSpaceField setNextKeyView:mConfirmationSpaceField];
		[mConfirmationSpaceField setNextKeyView:mSpaceField];
	}
	else
	{
		[mPassphraseScrollView setDocumentView:mPassphraseField];
		[mPassConfirmationScrollView setDocumentView:mConfirmationPassphraseField];
		
		[mPassphraseField setNextKeyView:mConfirmationPassphraseField];
		[mConfirmationPassphraseField setNextKeyView:mPassphraseField];
	}
}

- (IBAction)switchPanelAction:(id)sender
{
	if( ![sender tag] )
		[self nextView];
	else
		[self prevView];
}

- (IBAction)cancelKeyGen:(id)sender
{
	sKeyGenAbort = TRUE;
}

- (void)updatePassphraseQuality:(PGPUInt32)quality sender:(id)sender
{
	if( sender == mPassphraseField )
		[mPassQualityIndicator setDoubleValue:quality];
}

- (NSWindow *)window
{
	return mKeyGenWindow;
}

- (BOOL)windowShouldClose:(id)sender
{
	if( mGenerating )
		return NO;
	[mEntropyTimer invalidate];
	[mOwnerController endKeyGen:mKeysView success:mAddedKey];
	return YES;
}

- (void)dealloc
{
	[mPEConfirmationBox release];	// Previously retained so that we could safely remove them from the view
	[mPELengthBox release];
	[mPEQualityBox release];
	[mPassCapsLockField release];
	[mGenContinueField release];
	[mGenCurStatusIndicator release];
	[mGenCancelButton release];
	
	pgpClearMemory( mReconPrompts, sizeof( mReconPrompts ) );
	pgpClearMemory( mReconPasses, sizeof( mReconPasses ) );
	if( PGPKeyDBRefIsValid( mGeneratedKeyDB ) )
		PGPFreeKeyDB( mGeneratedKeyDB );
	
	[super dealloc];
	PGPLog(( @"CKeyGen dealloc" ));
}

@end

