/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation
    All rights reserved.

    $Id: CKeyInfo.m,v 1.33 2002/11/11 05:26:30 wprice Exp $
____________________________________________________________________________*/
#import "CKeyInfo.h"
#import "CRemoteAuth.h"
#import "pgpClientLibCocoa.h"
#import "pgpClientPrefs.h"
#import "pflPrefTypes.h"
#import "pgpHashWords.h"
#import "pgpClientShared.h"
#import "pgpEndianConversion.h"
#import <PGPui/CPGPIcons.h>
#import <PGPui/CBrowserCell.h>

#define kCKeyInfoNib					@"CKeyInfo"
#define kKeyInfoUnknownKeyString		LSCL( @"Unknown Key, Key ID is %s", "Unknown Key" )
#define kKeyInfoJoinButtonTitle			LSCL( @"Join", @"KeyInfo Join button title" )
#define kKeyInfoChangePassButtonTitle	LSCL( @"Change Passphrase", @"Change Pass button title" )

enum
{
	kPGPKeyInfoTab_General		= 1,
	kPGPKeyInfoTab_Subkeys,
	kPGPKeyInfoTab_Revokers,
	kPGPKeyInfoTab_ADK,
	
	kPGPKeyInfoButton_NewSubkey	= 400,
	kPGPKeyInfoButton_RevokeSubkey,
	kPGPKeyInfoButton_RemoveSubkey,
	kPGPKeyInfoButton_UpdateRevoker,
	kPGPKeyInfoButton_UpdateADK
};

enum
{
	kPGPPhotoUserIDStatus_Normal	= 1,
	kPGPPhotoUserIDStatus_NoValidity,
	kPGPPhotoUserIDStatus_Revoked,
	kPGPPhotoUserIDStatus_Expired
	
};

static NSMutableArray	*sKeyInfoArray = nil;
static PGPBoolean		sSubkeyGenAbort = FALSE;
static BOOL				sGeneratingSubkey = NO;

	PGPError
generateSubkeyCallback(
	PGPContextRef 	context,
	PGPEvent 		*event,
	PGPUserValue 	userValue)
{
	PGPError		err = kPGPError_NoErr;
	
	if( event->type == kPGPEvent_KeyGenEvent )
	{
		if( sSubkeyGenAbort )
			err = kPGPError_UserAbort;
	}
	
	return( err );
}

@implementation CKeyInfo

+ (id)showKeyInfo:(id)controller forKey:(PGPKeyDBObjRef)keyDBObj
		mainDB:(PGPKeyDBRef)mainKeyDB lock:(BOOL)lockdown
{
	CKeyInfo		*keyInfo = nil;
	
	if( !sKeyInfoArray )
		sKeyInfoArray = [[NSMutableArray alloc] init];
	{
		NSEnumerator	*keyInfoEnumerator = [sKeyInfoArray objectEnumerator];

		while( keyInfo = [keyInfoEnumerator nextObject] )
		{
			if( keyDBObj == [keyInfo keyDBItem] )
				break;
		}
		if( !keyInfo )
		{
			keyInfo = [[CKeyInfo alloc] initWithController:controller
				forKey:keyDBObj mainDB:mainKeyDB lock:lockdown];
			[sKeyInfoArray addObject:keyInfo];
			[keyInfo release];
		}
		[[keyInfo window] makeKeyAndOrderFront:controller];
	}
	return keyInfo;
}

+ (void)updateAllForDB:(PGPKeyDBRef)keyDB
{
	if( sKeyInfoArray )
	{
		NSEnumerator	*keyInfoEnumerator = [sKeyInfoArray objectEnumerator];
		CKeyInfo		*keyInfo;
	
		while( keyInfo = [keyInfoEnumerator nextObject] )
			if( keyDB == [keyInfo keyDB] )
				[keyInfo updateKeyDB];
	}
}

+ (void)closeAll:(id)sender forDB:(PGPKeyDBRef)keyDB
{
	NSAutoreleasePool *pool =[[NSAutoreleasePool alloc] init];
	if( sKeyInfoArray )
	{
		NSEnumerator	*keyInfoEnumerator = [sKeyInfoArray objectEnumerator],
						*trashEnumerator;
		NSMutableArray	*trashArray = [[NSMutableArray alloc] init];
		CKeyInfo		*keyInfo;
	
		while( keyInfo = [keyInfoEnumerator nextObject] )
		{
			if( keyDB == [keyInfo keyDB] )
				[trashArray addObject:keyInfo];
		}
		trashEnumerator = [trashArray objectEnumerator];
		while( keyInfo = [trashEnumerator nextObject] )
		{
			[[keyInfo window] performClose:sender];
		}
		[trashArray release];
		if( ![sKeyInfoArray count] )
		{
			[sKeyInfoArray release];
			sKeyInfoArray = nil;
		}
	}
	[pool release];
}

- (id)init
{
	if( !mInited )
	{
		[super init];
		mInited = YES;
	}
	if( !mLoadedNib )
		[self initWithNib:kCKeyInfoNib];
    return self;
}

- (id)initWithNib:(NSString*)nibFile
{
    if( nibFile && [nibFile length] )
        [NSBundle loadNibNamed:nibFile owner:self];
    
	mLoadedNib = YES;
	
    return [self init];
}

- (id)initWithController:(id)controller
		forKey:(PGPKeyDBObjRef)keyDBObj
		mainDB:(PGPKeyDBRef)mainKeyDB
		lock:(BOOL)lockdown
{
	[self init];
	mController = controller;
	[mNewSubkeyParamsBox retain];
	[mNewSubkeyGenProgress retain];
	[mNewSubkeyGenProgress removeFromSuperview];
	mKey = keyDBObj;
	mKeyDB = mainKeyDB;
	PGPNewKeyIterFromKeyDB( PGPPeekKeyDBObjKeyDB( mKey ), &mKeyIter );
	mLock = lockdown;
	mChanged = NO;
	[self setInfo];
	return self;
}

- (NSTabViewItem *)getKeyInfoTabItem:(PGPInt32)num
{
	NSEnumerator	*tviEnumerator = [[mKeyInfoTabView tabViewItems] objectEnumerator];
	NSTabViewItem	*tvi = nil;
	
	while( tvi = [tviEnumerator nextObject] )
		if( [[tvi identifier] intValue] == num )
			break;
	return tvi;
}

- (void)setInfo
{
	char						cstr[kPGPcl_MaxPGPUserIDLength];
	PGPSize						len;
	PGPBoolean					expired,
								revoked,
								disabled,
								secret,
								split,
								axiomatic,
								useHex;
	PGPInt32					algorithm,
								version,
								validity,
								validityValue,
								trust,
								bits,
								subBits,
								numADKs,
								numRevokers;
	PGPKeyID					keyID;
	PGPTime						pgpTime;
	NSString					*nStr;
	NSMutableString				*nmStr;
	NSFont						*smallFont		= [[NSFont systemFontOfSize:[NSFont smallSystemFontSize]] retain];
	NSFont						*miniFont		= [[NSFont userFontOfSize:10.0] retain];
	PGPPrefRef					prefRef			= [mController pgpPrefRef];
	PGPKeyDBObjRef				photoID;
	NSBrowserCell				*bCell;
	NSImageCell					*iCell;
	NSTextFieldCell				*tCell;
	NSUserDefaults				*defaults		= [NSUserDefaults standardUserDefaults];

	nStr = [[PGPclGetPrimaryUserIDName( mKey ) componentsSeparatedByString:@" <"] objectAtIndex:0];
	if( nStr == nil )
		nStr = @" ";
	[mKeyInfoWindow setTitle:nStr];
	PGPGetKeyDBObjBooleanProperty( mKey, kPGPKeyProperty_IsExpired, &expired );
	PGPGetKeyDBObjBooleanProperty( mKey, kPGPKeyProperty_IsRevoked, &revoked );
	PGPGetKeyDBObjBooleanProperty( mKey, kPGPKeyProperty_IsDisabled, &disabled );
	PGPGetKeyDBObjBooleanProperty( mKey, kPGPKeyProperty_IsAxiomatic, &axiomatic );
	PGPGetKeyDBObjBooleanProperty( mKey, kPGPKeyProperty_IsSecret, &secret );
	PGPGetKeyDBObjBooleanProperty( mKey, kPGPKeyProperty_IsSecretShared, &split );
	PGPGetKeyDBObjNumericProperty( mKey, kPGPKeyProperty_AlgorithmID, &algorithm );
	PGPGetKeyDBObjNumericProperty( mKey, kPGPKeyProperty_Version, &version );
	if( expired || revoked )
		mLock = YES;
	PGPCountAdditionalRecipientRequests( mKey, &numADKs );
	if( !numADKs )
	{
		mADKTabView = [[self getKeyInfoTabItem:kPGPKeyInfoTab_ADK] retain];
		[mKeyInfoTabView removeTabViewItem:mADKTabView];
	}
	PGPCountRevocationKeys( mKey, &numRevokers );
	if( !numRevokers )
	{
		mRevokerTabView = [[self getKeyInfoTabItem:kPGPKeyInfoTab_Revokers] retain];
		[mKeyInfoTabView removeTabViewItem:mRevokerTabView];
	}
	if( version != kPGPKeyVersion_V4 )
	{
		mSubkeyTabView = [[self getKeyInfoTabItem:kPGPKeyInfoTab_Subkeys] retain];
		[mKeyInfoTabView removeTabViewItem:mSubkeyTabView];
	}
	
	if( IsntPGPError( PGPGetKeyID( mKey, &keyID ) ) )
	{
		char	keyIDString[ kPGPMaxKeyIDStringSize ];
		
		PGPGetKeyIDString( &keyID, kPGPKeyIDString_Abbreviated, keyIDString);
		keyIDString[10] = '\0';
		[mKeyIDField setFont:smallFont];
		[mKeyIDField setStringValue:[NSString stringWithCString:keyIDString]];
	}
	PGPclGetFingerprintString( mKey, cstr );
	[mFingerprintField setFont:miniFont];
	[mFingerprintField setStringValue:[NSString stringWithCString:cstr]];
	
	[mBioInfoField1 setFont:miniFont];
	[mBioInfoField2 setFont:miniFont];
	[mBioInfoField3 setFont:miniFont];
	[mBioInfoField4 setFont:miniFont];
	{
		PGPByte		fingerprint[ 32 ];
		PGPByte		*wordStr;
		PGPUInt32	wordsPerColumn,
					columnNum,
					wordNum,
					fingIndex = 0;
		
		PGPGetKeyDBObjDataProperty( mKey, kPGPKeyProperty_Fingerprint, fingerprint, sizeof( fingerprint ), &len);
		wordsPerColumn = len / 4;
		for( columnNum = 1; columnNum <= 4; columnNum++ )
		{
			cstr[0] = '\0';
			for( wordNum = 0; wordNum < wordsPerColumn;wordNum++ )
			{
				fingIndex = columnNum + ( wordNum * 4 ) - 1;
				if( fingIndex % 2 )
					wordStr = hashWordListOdd[fingerprint[fingIndex]];
				else
					wordStr = hashWordListEven[fingerprint[fingIndex]];
				strcat( cstr, wordStr );
				strcat( cstr, "\r" );
			}
			switch( columnNum )
			{
				case 1:
					[mBioInfoField1 setStringValue:[NSString stringWithCString:cstr]];
					break;
				case 2:
					[mBioInfoField2 setStringValue:[NSString stringWithCString:cstr]];
					break;
				case 3:
					[mBioInfoField3 setStringValue:[NSString stringWithCString:cstr]];
					break;
				case 4:
					[mBioInfoField4 setStringValue:[NSString stringWithCString:cstr]];
					break;
			}
		}
	}
	PGPGetPrefBoolean( prefRef, kPGPPrefUseHexFingerprint, &useHex );
	[mHexadecimalCheckbox setState:useHex];
	mFingerFrame = [mFingerprintField frame];
	mBioFrame = [mBioFingerprintField frame];
	[self fingerprintDisplayChange:mHexadecimalCheckbox];
	
	[mEnabledCheckbox setState:!disabled];
	if( mLock || axiomatic )
		[mEnabledCheckbox setEnabled:NO];
	
	PGPGetKeyDBObjNumericProperty( mKey, kPGPKeyProperty_Trust, &trust );
	switch( trust & kPGPKeyTrust_Mask )
	{
		case kPGPKeyTrust_Undefined:
		case kPGPKeyTrust_Unknown:
		case kPGPKeyTrust_Never:
		default:
			trust = 0;
			break;
		case kPGPKeyTrust_Marginal:
			trust = 1;
			break;
		case kPGPKeyTrust_Complete:
		case kPGPKeyTrust_Ultimate:
			trust = 2;
			break;
	}
	[mTrustSlider setIntValue:trust];
	
	PGPGetKeyDBObjNumericProperty( mKey, kPGPKeyProperty_Validity, &validity );
	switch( validity )
	{
		case kPGPValidity_Unknown:
		case kPGPValidity_Invalid:
		default:
			validityValue = 0;
			break;
		case kPGPValidity_Marginal:
			validityValue = 1;
			break;
		case kPGPValidity_Complete:
			validityValue = 2;
			break;
	}
	[mValidityIndicator setDoubleValue:validityValue];
	[mImplicitTrustCheckbox setState:axiomatic];
	if( axiomatic || mLock )
		[mTrustSlider setEnabled:NO];
	
	PGPGetKeyDBObjTimeProperty( mKey, kPGPKeyProperty_Creation, &pgpTime );
	[mCreatedField setFont:smallFont];
	[mCreatedField setObjectValue:
		[[NSCalendarDate dateWithTimeIntervalSince1970:PGPGetStdTimeFromPGPTime( pgpTime )]
		descriptionWithCalendarFormat:[defaults stringForKey:NSShortDateFormatString]
		locale:[defaults dictionaryRepresentation]]];
	
	PGPGetKeyDBObjTimeProperty( mKey, kPGPKeyProperty_Expiration, &pgpTime );
	if( pgpTime )
	{
		[mExpiresField setObjectValue:
			[[NSCalendarDate dateWithTimeIntervalSince1970:PGPGetStdTimeFromPGPTime( pgpTime )]
			descriptionWithCalendarFormat:[NSString stringWithFormat:LSCL(@"EXPIRES %@", @"KeyInfo expirationDateFormat"),
			[defaults stringForKey:NSShortDateFormatString]]
			locale:[defaults dictionaryRepresentation]]];
	}
	
	nStr = nil;
	switch(algorithm)
	{
		case kPGPPublicKeyAlgorithm_RSA:
		case kPGPPublicKeyAlgorithm_RSAEncryptOnly:
		case kPGPPublicKeyAlgorithm_RSASignOnly:
			if( version == kPGPKeyVersion_V4 )
				nStr = LSCL(@"RSA", @"RSA Key Type");
			else
				nStr = LSCL(@"RSA Legacy", @"RSA Legacy Key Type");
			break;
		case kPGPPublicKeyAlgorithm_DSA:
		case kPGPPublicKeyAlgorithm_ElGamal:	// Technically not
			nStr = LSCL(@"DH/DSS", @"DS/DSS Key Type");
			break;
		case kPGPPublicKeyAlgorithm_ECEncrypt:
		case kPGPPublicKeyAlgorithm_ECSign:
			nStr = LSCL(@"EC", @"EC Key Type");
			break;
		default:
			nStr = LSCL(@"Unknown", @"Unknown Key Type");
			break;
	}
	[mKeyTypeField setFont:smallFont];
	if( nStr )
		[mKeyTypeField setStringValue:nStr];
	
	nStr = nil;
	PGPGetKeyDBObjNumericProperty( mKey, kPGPKeyProperty_Bits, &bits );
	{
		PGPKeyDBObjRef	subKey;
		
		PGPKeyIterSeek( mKeyIter, mKey );
		if( IsntPGPError( PGPKeyIterNextKeyDBObj( mKeyIter, kPGPKeyDBObjType_SubKey, &subKey ) ) &&
			IsntNull( subKey ) )
		{
			PGPGetKeyDBObjNumericProperty( subKey, kPGPSubKeyProperty_Bits, &subBits );
			nStr = [NSString stringWithFormat:@"%u/%u", subBits, bits];
		}
		else
			nStr = [[NSNumber numberWithInt:bits] stringValue];
	}
	[mKeySizeField setFont:smallFont];
	if( nStr )
		[mKeySizeField setStringValue:nStr];
	
	{
		PGPInt32	propBuff[32];
		PGPInt32	algCount,
					algIndex;
		
		nmStr = [NSMutableString string];
		PGPGetKeyDBObjDataProperty( mKey, kPGPKeyProperty_PreferredAlgorithms,
				propBuff, sizeof(propBuff), &len);
		algCount = len / sizeof(PGPCipherAlgorithm);
		if( algCount )
		{
			for( algIndex = 0; algIndex < algCount; algIndex++ )
			{
				if( algIndex )
					[nmStr appendString:@"\n"];
				switch( propBuff[algIndex] )
				{
					case kPGPCipherAlgorithm_IDEA:
						[nmStr appendString:@"IDEA"];
						break;
					case kPGPCipherAlgorithm_3DES:
						[nmStr appendString:@"TripleDES"];
						break;
					case kPGPCipherAlgorithm_CAST5:
						[nmStr appendString:@"CAST"];
						break;
					case kPGPCipherAlgorithm_AES128:
						[nmStr appendString:@"AES-128"];
						break;
					case kPGPCipherAlgorithm_AES192:
						[nmStr appendString:@"AES-192"];
						break;
					case kPGPCipherAlgorithm_AES256:
						[nmStr appendString:@"AES-256"];
						break;
					case kPGPCipherAlgorithm_Twofish256:
						[nmStr appendString:@"Twofish"];
						break;
					default:
						[nmStr appendString:@"Unknown"];
						break;
				}
			}
		}
		else
		{
			if( version != kPGPKeyVersion_V4 )
				[nmStr appendString:@"IDEA"];
			else
				[nmStr appendString:@"CAST"];
		}
		[mCiphersField setFont:smallFont];
		[mCiphersField setStringValue:nmStr];
	}
	
	if( !secret || mLock )
	{
		[mPassphraseButton setEnabled:NO];
		[mImplicitTrustCheckbox setEnabled:NO];
	}
	else if( split )
		[mPassphraseButton setTitle:kKeyInfoJoinButtonTitle];
	
	photoID = [self updatePhotoID];
	{
		PGPInt32					photoStatus = kPGPPhotoUserIDStatus_Normal;
		NSMutableAttributedString	*photoString;
		NSString					*photoRawString = nil;
		NSRange						range = { 0, 0 };
		float						overlayAngle = 0.0,
									overlaySize = 24.0;
			
		if( revoked )
			photoStatus = kPGPPhotoUserIDStatus_Revoked;
		else if( expired )
			photoStatus = kPGPPhotoUserIDStatus_Expired;
		else if( !axiomatic )
		{
			PGPBoolean	marginalIsInvalid;
						
			if( PGPKeyDBObjRefIsValid( photoID ) )
				PGPGetKeyDBObjNumericProperty( photoID, kPGPUserIDProperty_Validity, &validity );
			if( validity == kPGPValidity_Complete )
				photoStatus = kPGPPhotoUserIDStatus_Normal;
			else if( validity == kPGPValidity_Marginal )
			{
				PGPGetPrefBoolean( [mController pgpPrefRef], kPGPPrefMarginalIsInvalid, &marginalIsInvalid);
				if( marginalIsInvalid )
					photoStatus = kPGPPhotoUserIDStatus_NoValidity;
			}
			else
				photoStatus = kPGPPhotoUserIDStatus_NoValidity;
		}
		switch( photoStatus )
		{
			case kPGPPhotoUserIDStatus_Normal:
				break;
			case kPGPPhotoUserIDStatus_Revoked:
				photoRawString = LSCL( @"REVOKED", "Photo ID Overlay Revoked" );
				overlayAngle = 55;
				break;
			case kPGPPhotoUserIDStatus_Expired:
				photoRawString = LSCL( @"EXPIRED", "Photo ID Overlay Expired" );
				overlayAngle = 55;
				break;
			case kPGPPhotoUserIDStatus_NoValidity:
				photoRawString = LSCL( @"?", "Photo ID Overlay Invalid" );
				overlaySize = 128;
				break;
		}
		if( photoRawString )
		{
			photoString = [[NSMutableAttributedString alloc] initWithString:photoRawString];
			range.length = [photoString length];
			[photoString addAttribute:NSForegroundColorAttributeName value:[[NSColor redColor] colorWithAlphaComponent:0.6] range:range];
			[photoString addAttribute:NSFontAttributeName value:[NSFont systemFontOfSize:overlaySize] range:range];
			[mPhotoIDImageView setOverlayString:photoString rotate:overlayAngle];
			[photoString release];
		}
		[mPhotoIDImageView setEditable:NO];
	}
		
	[mSubkeyTableView setDataSource:self];
    [mSubkeyTableView setTarget:self];
    [mSubkeyTableView setDelegate:self];
	bCell = [[CBrowserCell alloc] init];
	[bCell setLeaf:YES];
	[bCell setFont:[NSFont systemFontOfSize:[NSFont smallSystemFontSize]]];
	[[[mSubkeyTableView tableColumns] objectAtIndex:0] setDataCell:bCell];
	[bCell release];
	
	tCell = [[NSTextFieldCell alloc] init];
	[tCell setFont:[NSFont systemFontOfSize:[NSFont smallSystemFontSize]]];
	[[[mSubkeyTableView tableColumns] objectAtIndex:1] setDataCell:tCell];
	[tCell release];

	tCell = [[NSTextFieldCell alloc] init];
	[tCell setFont:[NSFont systemFontOfSize:[NSFont smallSystemFontSize]]];
	[[[mSubkeyTableView tableColumns] objectAtIndex:2] setDataCell:tCell];
	[tCell release];
	
	[mRevokerTableView setDataSource:self];
    [mRevokerTableView setTarget:self];
    [mRevokerTableView setDelegate:self];
	bCell = [[CBrowserCell alloc] init];
	[bCell setLeaf:YES];
	[bCell setFont:[NSFont systemFontOfSize:[NSFont smallSystemFontSize]]];
	[[[mRevokerTableView tableColumns] objectAtIndex:0] setDataCell:bCell];
	[bCell release];
	
	[mADKTableView setDataSource:self];
    [mADKTableView setTarget:self];
    [mADKTableView setDelegate:self];
	bCell = [[CBrowserCell alloc] init];
	[bCell setFont:[NSFont systemFontOfSize:[NSFont smallSystemFontSize]]];
	[bCell setLeaf:YES];
	[[[mADKTableView tableColumns] objectAtIndex:0] setDataCell:bCell];
	[bCell release];
	
	iCell = [[NSImageCell alloc] init];
	[[[mADKTableView tableColumns] objectAtIndex:1] setDataCell:iCell];
	[iCell release];
	
	[miniFont release];
	[smallFont release];
	[self adjustButtons];
}

- (PGPKeyDBObjRef)updatePhotoID
{
	PGPContextRef	context = PGPPeekKeyDBContext( mKeyDB );
	PGPMemoryMgrRef	memMgr = PGPPeekContextMemoryMgr( context );
	PGPKeyDBObjRef	userID;
	PGPBoolean		isAttributeID;
	PGPInt32		attributeType;
	PGPSize			bufferSize;
	BOOL			foundPhoto = NO;
	
	[mPhotoIDImageView setImage:nil];
	PGPKeyIterSeek( mKeyIter, mKey );
	while( !foundPhoto && IsntPGPError( PGPKeyIterNextKeyDBObj( mKeyIter, kPGPKeyDBObjType_UserID, &userID ) ) )
	{
		if( IsntPGPError( PGPGetKeyDBObjBooleanProperty( userID, kPGPUserIDProperty_IsAttribute, &isAttributeID ) ) &&
			isAttributeID )
		{
			if( IsntPGPError( PGPGetKeyDBObjNumericProperty( userID,
				kPGPUserIDProperty_AttributeType, &attributeType ) ) &&
				( attributeType == kPGPAttribute_Image ) )
			{
				PGPGetKeyDBObjDataProperty( userID, kPGPUserIDProperty_AttributeData, NULL, 0, &bufferSize );
				if( bufferSize > 0 )
				{
					void	*photoBuffer = NULL;
					
					photoBuffer = PGPNewData( memMgr, bufferSize, 0 );
					if( IsntNull( photoBuffer ) && IsntPGPError( PGPGetKeyDBObjDataProperty( userID,
						kPGPUserIDProperty_AttributeData, (char *) photoBuffer, bufferSize, &bufferSize ) ) )
					{
						NSImage					*photo;
						PGPPhotoUserIDHeader	*header;
						PGPByte					*dataStart;
						PGPUInt16				headerLength;
						
						header 			= (PGPPhotoUserIDHeader *) photoBuffer;
						headerLength	= PGPStorageToUInt16( (PGPByte *) &header->headerLength );
						dataStart		= (PGPByte *) photoBuffer + headerLength;
						if( ( header->headerVersion == kPGPPhotoIDHeaderVersion ) &&
							( header->photoIDFormat == kPGPPhotoIDFormat_JPEG ) )
						{
							photo = [[NSImage alloc] initWithData:
								[NSData dataWithBytes:dataStart length:bufferSize - headerLength]];
							[mPhotoIDImageView setImage:photo];
							[photo release];
							foundPhoto = YES;
						}
					}
					if( IsntNull( photoBuffer ) )
						PGPFreeData( photoBuffer );
				}
			}
		}
	}
	if( !foundPhoto )
		return kInvalidPGPKeyDBObjRef;
	else
		return userID;
}

- (void)adjustButtons
{
	PGPPrefRef		prefRef = [mController pgpPrefRef];
	BOOL			enableNew		= NO;
	BOOL			enableRemove 	= NO;
	BOOL			enableRevoke 	= NO;
	BOOL			enableADKUpdate	= NO;
	BOOL			enableRevUpdate	= NO;
	PGPBoolean		secret,
					expired,
					revoked,
					skRevoked;
	
	if( !mLock )
	{
		PGPGetKeyDBObjBooleanProperty( mKey, kPGPKeyProperty_IsExpired, &expired );
		PGPGetKeyDBObjBooleanProperty( mKey, kPGPKeyProperty_IsRevoked, &revoked );
		PGPGetKeyDBObjBooleanProperty( mKey, kPGPKeyProperty_IsSecret, &secret );
		
		if( secret && !revoked && !expired )
			enableNew = YES;
		if( [mSubkeyTableView numberOfSelectedRows] > 0 )
		{
			PGPKeyDBObjRef	subKey;
			PGPInt32		subkeyIndex = 0;
			int				row = [mSubkeyTableView selectedRow];
			
			PGPKeyIterRewind( mKeyIter, kPGPKeyDBObjType_SubKey );
			while( IsntPGPError( PGPKeyIterNextKeyDBObj( mKeyIter, kPGPKeyDBObjType_SubKey, &subKey ) ) )
				if( subkeyIndex++ == row )
				{
					PGPGetKeyDBObjBooleanProperty( subKey, kPGPSubKeyProperty_IsRevoked, &skRevoked );
					if( secret && !revoked && !expired && !skRevoked )
						enableRevoke = YES;
					break;
				}
			if( secret )
				enableRemove = YES;
		}
		if( [mRevokerTableView numberOfSelectedRows] > 0 )
			enableRevUpdate = YES;
		if( [mADKTableView numberOfSelectedRows] > 0 )
			enableADKUpdate = YES;
	}
	if( PGPclEnterprise() )
	{
		PGPBoolean	allowKeyGen,
					preset;
		
		PGPGetPrefBoolean( prefRef, kPGPPrefAllowKeyGeneration, &allowKeyGen );
		PGPGetPrefBoolean( prefRef, kPGPPrefPresetKeyGeneration, &preset );
		if( !allowKeyGen || preset )
			enableNew = NO;
		if( preset )
		{
			enableRemove = NO;
			enableRevoke = NO;
		}
	}
	
	[mNewSubkeyButton setEnabled:enableNew];
	[mRevokeSubkeyButton setEnabled:enableRevoke];
	[mRemoveSubkeyButton setEnabled:enableRemove];
	[mUpdateRevokerButton setEnabled:enableRevUpdate];
	[mUpdateADKButton setEnabled:enableADKUpdate];
}

- (IBAction)attributeChanged:(id)sender
{
	mChanged = YES;
}

- (IBAction)implicitTrustChanged:(id)sender
{
	if( [sender state] )
	{
		[mTrustSlider setIntValue:2];
		[mTrustSlider setEnabled:NO];
		[mValidityIndicator setDoubleValue:2];
		[mEnabledCheckbox setState:1];
		[mEnabledCheckbox setEnabled:NO];
	}
	else
	{
		[mTrustSlider setEnabled:YES];
		[mTrustSlider setIntValue:0];
		[mValidityIndicator setDoubleValue:0];
		[mEnabledCheckbox setEnabled:YES];
	}
	mChanged = YES;
}

- (IBAction)changePassphrasePressed:(id)sender
{
	PGPError			err = kPGPError_NoErr;
	char				*passphrase  = NULL;
	PGPByte				*passKey = NULL;
	PGPSize				passKeySize;
	PGPBoolean			split;
	char				*newPhrase = NULL;
	PGPPrefRef			prefRef = [mController pgpPrefRef];
	PGPContextRef		context = PGPPeekKeyDBObjContext( mKey );
	PGPKeyDBObjRef		subKey;
	PGPBoolean			enforceLength = FALSE,
						enforceQuality = FALSE;
	PGPUInt32			minLength,
						minQuality;
	
	err = PGPclGetKeyPassphrase( mKey, [mController tlsContext], prefRef, &split, &passphrase, &passKey, &passKeySize );	CKERR;
	if( PGPclEnterprise() )
	{
		PGPGetPrefBoolean( prefRef, kPGPPrefEnforceMinChars, &enforceLength);
		PGPGetPrefBoolean( prefRef, kPGPPrefEnforceMinQuality, &enforceQuality);
		PGPGetPrefNumber( prefRef, kPGPPrefMinChars, &minLength);
		PGPGetPrefNumber( prefRef, kPGPPrefMinQuality, &minQuality);
	}
	err = PGPConfirmationPassphraseDialog( context,
				PGPOUIDialogPrompt( context,
					[LSCL( @"Enter new passphrase:", @"Change Passphrase confirmation" ) cString]),
				PGPOUIOutputPassphrase( context, &newPhrase ),
				enforceLength ?
					PGPOUIMinimumPassphraseLength( context, minLength ) :
					PGPONullOption( context ),
				enforceQuality ?
					PGPOUIMinimumPassphraseQuality( context, minQuality ) :
					PGPONullOption( context ),
				PGPOLastOption( context ) );	CKERR;
	err = PGPChangePassphrase( mKey, split ?
				PGPOPasskeyBuffer( context, passKey, passKeySize ) :
				PGPOPassphrase( context, passphrase ),
				PGPOPassphrase( context, newPhrase ),
				PGPOLastOption( context ) );	CKERR;
	
	PGPKeyIterRewind( mKeyIter, kPGPKeyDBObjType_SubKey );
	while( IsntPGPError ( err ) &&
		IsntPGPError( PGPKeyIterNextKeyDBObj( mKeyIter, kPGPKeyDBObjType_SubKey, &subKey ) ) )
	{
		err = PGPChangePassphrase( subKey, split ?
			PGPOPasskeyBuffer( context, passKey, passKeySize ) :
			PGPOPassphrase( context, passphrase ),
			PGPOPassphrase( context, newPhrase ),
			PGPOLastOption( context ) );
	}
	PGPFlushKeyDB( PGPPeekKeyDBObjKeyDB( mKey ) );
	mChanged = YES;
	[mImplicitTrustCheckbox setState:1];
	[mPassphraseButton setTitle:kKeyInfoChangePassButtonTitle];
	NSBeginInformationalAlertSheet(
		split ?
			LSCL( @"Key Rejoined", @"Title of Key Rejoined informational" ) :
			LSCL( @"Passphrase Changed", @"Title of Changed Pass informational" ),
		LSCL( @"OK", @"" ), nil, nil, mKeyInfoWindow, 
		self, NULL, NULL, NULL,
		split?
			LSCL( @"The key has been rejoined. The previous split shares will no longer work.", @"Key rejoined informational text" ) :
			LSCL( @"The passphrase has been changed.", @"Pass changed informational text" ) );
done:
	if( passphrase )
		PGPFreeData( passphrase );
	if( passKey )
		PGPFreeData( passKey );
	if( IsntNull( newPhrase ) )
		PGPFreeData( newPhrase );
}

- (IBAction)fingerprintDisplayChange:(id)sender
{
	PGPBoolean		useHex;
	PGPPrefRef		prefRef = [mController pgpPrefRef];
	
	useHex = [sender state];
	PGPSetPrefBoolean( prefRef, kPGPPrefUseHexFingerprint, useHex );
	if( useHex )
	{
		[mBioFingerprintField setFrame:NSZeroRect];
		[mFingerprintField setFrame:mFingerFrame];
		[[mFingerprintField superview] setNeedsDisplay:YES];
	}
	else
	{
		[mFingerprintField setFrame:NSZeroRect];
		[mBioFingerprintField setFrame:mBioFrame];
		[[mBioFingerprintField superview] setNeedsDisplay:YES];
	}
}

- (void)didEndRevokeConfirmSheet:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(PGPKeyDBObjRef)subKey
{
	PGPError		err = kPGPError_NoErr;
	PGPContextRef	context = PGPPeekKeyDBObjContext( subKey );
	PGPBoolean		split;
	char			*passphrase  = NULL;
	PGPByte			*passKey = NULL;
	PGPSize			passKeySize;
	
    if( returnCode == NSAlertDefaultReturn )
	{
		err = PGPclGetKeyPassphrase( PGPPeekKeyDBObjKey( subKey ), [mController tlsContext], [mController pgpPrefRef],
				&split, &passphrase, &passKey, &passKeySize );
		if( IsntPGPError( err ) )
		{
			PGPclGuaranteeMinimumEntropy( context );
			err = PGPRevoke( subKey, split ?
					PGPOPasskeyBuffer( context, passKey, passKeySize ) :
					PGPOPassphrase( context, passphrase ),
					PGPOLastOption( context ) );
			if( IsntPGPError( err ) )
			{
				[mSubkeyTableView reloadData];
				[self adjustButtons];
			}
		}
	}
	if( passphrase )
		PGPFreeData( passphrase );
	if( passKey )
		PGPFreeData( passKey );
}

- (void)didEndRemoveConfirmSheet:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(PGPKeyDBObjRef)subKey
{
    if( ( returnCode == NSAlertDefaultReturn ) && ( IsntPGPError( PGPDeleteKeyDBObj( subKey ) ) ) )
	{
		[mSubkeyTableView reloadData];
		[self adjustButtons];
	}
}

- (void)generateSubKey:(id)sender
{
	NSAutoreleasePool		*pool		= [[NSAutoreleasePool alloc] init];
	PGPContextRef			context		= PGPPeekKeyDBObjContext( mKey );
	PGPPrefRef				prefRef		= [mController pgpPrefRef];
	PGPError				err			= kPGPError_NoErr;
	PGPKeyDBObjRef			subKey;
	PGPInt32				masterAlgorithm;
	PGPBoolean				fastGen;
	
	PGPGetPrefBoolean( prefRef, kPGPPrefFastKeyGen, &fastGen );
	PGPGetKeyDBObjNumericProperty( mKey, kPGPKeyProperty_AlgorithmID, &masterAlgorithm );
	err = PGPGenerateSubKey( context, &subKey, PGPOKeyGenMasterKey( context, mKey ),
					PGPOKeyGenParams( context, ( masterAlgorithm == kPGPPublicKeyAlgorithm_DSA ) ?
						kPGPPublicKeyAlgorithm_ElGamal : kPGPPublicKeyAlgorithm_RSA, [mNewSubkeySize intValue] ),
					PGPOCreationDate( context, mSubKeyGenStartDate ),
					PGPOExpiration( context, mSubKeyGenExpireDays ),
					PGPOKeyFlags( context, kPGPKeyPropertyFlags_UsageEncrypt ),
					PGPOKeyGenFast( context, fastGen ),
					mSplit ?
						PGPOPasskeyBuffer( context, mPassKey, mPassKeySize ) :
						PGPOPassphrase( context, mPassphrase ),
					PGPOEventHandler(context, generateSubkeyCallback, (PGPUserValue) self ),
					PGPOLastOption( context ) );
	if( mPassphrase )
		PGPFreeData( mPassphrase );
	if( mPassKey )
		PGPFreeData( mPassKey );
	mPassphrase = NULL;
	mPassKey = NULL;
	[mNewSubkeyGenProgress stopAnimation:self];
	if( ( IsPGPError( err ) ) && ( err != kPGPError_OutOfMemory ) )
		NSRunInformationalAlertPanel(
			LSCL( @"Subkey Generation Error", @"Subkey Generation error" ),
			[NSString stringWithFormat:
			LSCL( @"There was a problem generating the subkey (%@).", @"Bad Expiration" ),
				PGPGetErrorNSString( err ) ],
			LSCL( @"OK", @"" ), nil, nil );
	sGeneratingSubkey = NO;
	[[NSApplication sharedApplication] endSheet:mNewSubkeyPanel returnCode:err];
	[mNewSubkeyGenProgress removeFromSuperview];
	[[mNewSubkeyPanel contentView] addSubview:mNewSubkeyParamsBox];
	[mNewSubkeyOKButton setEnabled:YES];
	if( IsntPGPError( err ) )
	{
		[mSubkeyTableView reloadData];
		[self adjustButtons];
		mChanged = YES;
	}
	[pool release];
}

- (IBAction)newSubkeyAction:(id)sender
{
	BOOL			doGen		= YES;
	PGPError		err			= kPGPError_NoErr;
	PGPPrefRef		prefRef		= [mController pgpPrefRef];
	PGPContextRef	context		= PGPPeekKeyDBObjContext( mKey );
	
	if( sGeneratingSubkey )
	{
		if( ![sender tag] )
			sSubkeyGenAbort = YES;
		return;
	}
	if( [sender tag] )
	{
		PGPInt32		subkeySize		= [mNewSubkeySize intValue];
		NSDate			*startNSDate	= [mNewSubkeyStartDate objectValue],
						*endNSDate		= [mNewSubkeyExpirationDate objectValue],
						*created;
		BOOL			expires =		( [mNewSubkeyExpireType selectedTag] == 1 );
		PGPInt32		masterAlgorithm;
		PGPTime			tm = 0;
		PGPBoolean		fastGen;
		PGPUInt32		neededBits;
		PGPUInt32		currentBits;
		
		mPassphrase = NULL;
		mPassKey = NULL;
		
		mSubKeyGenStartDate = PGPGetPGPTimeFromStdTime( [startNSDate timeIntervalSince1970] );
		mSubKeyGenExpireDays = PGPGetPGPTimeFromStdTime( [endNSDate timeIntervalSince1970] );
		if( ( subkeySize < 1024 ) || ( subkeySize > 4096 ) )
		{
			NSRunInformationalAlertPanel(
				LSCL( @"Invalid Size", @"Title of Bad SubkeySize informational" ),
				LSCL( @"Subkey sizes must be at least 1024 and no more than 4096.", @"Bad Subkey size" ),
				LSCL( @"OK", @"" ), nil, nil );
			doGen = NO;
		}
		if( doGen && expires )
		{
			if( ( [endNSDate compare:startNSDate] != NSOrderedDescending ) ||
				( [endNSDate compare:[NSDate dateWithTimeIntervalSinceNow:0]] != NSOrderedDescending ) )
			{
				NSRunInformationalAlertPanel(
					LSCL( @"Invalid Expiration", @"Title of Bad Expiration informational" ),
					LSCL( @"The subkey must expire after today and after the start date.", @"Bad Expiration" ),
					LSCL( @"OK", @"" ), nil, nil );
				doGen = NO;
			}
		}
		if( doGen )
		{
			err = PGPGetKeyDBObjTimeProperty(mKey, kPGPKeyProperty_Creation, &tm);
			
			if (IsPGPError(err))
				PGPLog((@"Failed to get key creation time (err=%d)", err));
			else
			{
				created = [NSDate dateWithTimeIntervalSince1970:PGPGetStdTimeFromPGPTime(tm)];
				
				if ([[created earlierDate:startNSDate] isEqualTo:startNSDate])
				{
					NSRunInformationalAlertPanel(
						LSCL(@"Invalid Start Date", @"Title of Bad Start Date informational"),
						LSCL(@"The subkey's start date must be later than the master key's creation date.", @"Bad Start Date"),
						LSCL(@"OK", @""), nil, nil);
					doGen = NO;
				}
			}
		}
		if( doGen )
		{
			err = PGPclGetKeyPassphrase( mKey, [mController tlsContext], prefRef,
					&mSplit, &mPassphrase, &mPassKey, &mPassKeySize );
			if( IsntPGPError( err ) )
			{
				PGPGetKeyDBObjNumericProperty( mKey, kPGPKeyProperty_AlgorithmID, &masterAlgorithm );
				PGPGetPrefBoolean( prefRef, kPGPPrefFastKeyGen, &fastGen );
				neededBits = PGPGetKeyEntropyNeeded( context,
									PGPOKeyGenParams( context, ( masterAlgorithm == kPGPPublicKeyAlgorithm_DSA ) ?
										kPGPPublicKeyAlgorithm_ElGamal : kPGPPublicKeyAlgorithm_RSAEncryptOnly, subkeySize ),
									PGPOKeyFlags( context, kPGPKeyPropertyFlags_UsageEncrypt ),
									PGPOKeyGenFast( context, fastGen ),
									PGPOLastOption( context ) );
				currentBits = PGPContextReserveRandomBytes( context, (neededBits + 7) / 8 ) * 8;
				while( neededBits > currentBits )
				{
					err = PGPCollectRandomDataDialog( context, neededBits, PGPOLastOption( context ) );
					currentBits = PGPContextReserveRandomBytes( context, (neededBits + 7) / 8 ) * 8;
				}
				if( expires )
					mSubKeyGenExpireDays = ( mSubKeyGenExpireDays - mSubKeyGenStartDate ) / 86400 + 1;
				else
					mSubKeyGenExpireDays = 0;
				[mNewSubkeyParamsBox removeFromSuperview];
				[[mNewSubkeyPanel contentView] addSubview:mNewSubkeyGenProgress];
				[mNewSubkeyOKButton setEnabled:NO];
				[mNewSubkeyGenProgress startAnimation:self];
				sSubkeyGenAbort = FALSE;
				sGeneratingSubkey = YES;
				[NSThread detachNewThreadSelector:@selector(generateSubKey:) toTarget:self withObject:self];
			}
		}
	}
	else
		[[NSApplication sharedApplication] endSheet:mNewSubkeyPanel returnCode:kPGPError_UserAbort];
}

- (void)newSubkeySheetDidEnd:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo
{
	[sheet orderOut:self];
}

- (void)updateKeyDB
{
	[mADKTableView reloadData];
	[mRevokerTableView reloadData];
}

- (IBAction)tableButtonPressed:(id)sender
{
	PGPError			err = kPGPError_NoErr;
	PGPContextRef		pgpContext = PGPPeekKeyDBObjContext( mKey );
	PGPtlsContextRef	tlsContext = [mController tlsContext];
	PGPPrefRef			prefRef = [mController pgpPrefRef];
	BOOL				revoke = NO;
	int					tag = [sender tag];
	int					row = 0;
	PGPKeyDBRef			assocKeyDB = kInvalidPGPKeyDBRef;
	PGPKeyDBObjRef		assocKeyRef = kInvalidPGPKeyDBObjRef;
	PGPKeyID			assocKeyID;
	PGPByte				adkClass;
	
	switch( tag )
	{
		case kPGPKeyInfoButton_NewSubkey:
		{
			NSDateFormatter*	dateFormatter;
			NSMutableString*	dateFormat = [[[[NSUserDefaults standardUserDefaults] stringForKey:NSShortDateFormatString] mutableCopy] autorelease];
			
			dateFormatter = [[[NSDateFormatter alloc] initWithDateFormat:dateFormat allowNaturalLanguage:YES] autorelease];
			
			[mNewSubkeySize setIntValue:2048];
			[mNewSubkeyStartDate setFormatter:dateFormatter];
			[mNewSubkeyStartDate setObjectValue:[NSDate dateWithTimeIntervalSinceNow:0]];
			[mNewSubkeyExpirationDate setFormatter:dateFormatter];
			[mNewSubkeyExpirationDate setObjectValue:[NSDate dateWithTimeIntervalSinceNow:kPGPcl_OneYearInSeconds]];
			[[NSApplication sharedApplication] beginSheet:mNewSubkeyPanel modalForWindow:mKeyInfoWindow
				modalDelegate:self didEndSelector:@selector(newSubkeySheetDidEnd:returnCode:contextInfo:)
				contextInfo:NULL];
			[mNewSubkeySize becomeFirstResponder];
			break;
		}
		case kPGPKeyInfoButton_RevokeSubkey:
			revoke = YES;
		case kPGPKeyInfoButton_RemoveSubkey:
		{
			PGPKeyDBObjRef	subKey;
			PGPInt32		subkeyIndex = 0;
			int				row = [mSubkeyTableView selectedRow];
			
			PGPKeyIterRewind( mKeyIter, kPGPKeyDBObjType_SubKey );
			while( IsntPGPError( PGPKeyIterNextKeyDBObj( mKeyIter, kPGPKeyDBObjType_SubKey, &subKey ) ) )
				if( subkeyIndex++ == row )
				{
					NSBeginAlertSheet(
						revoke ?
							LSCL( @"Revoke Subkey", @"Title of KeyInfo revoke confirmation" ) :
							LSCL( @"Remove Subkey", @"Title of KeyInfo remove confirmation" ),
						LSCL( @"OK", @"OK button" ),
						LSCL( @"Cancel", @"Cancel button" ), nil, mKeyInfoWindow, 
						self, NULL, revoke ? @selector(didEndRevokeConfirmSheet:returnCode:contextInfo:) :
						@selector(didEndRemoveConfirmSheet:returnCode:contextInfo:), subKey,
						revoke ?
							LSCL( @"Are you sure you want to revoke the selected subkey?", @"Confirm KeyInfo revoke" ) :
							( mNumSubkeys > 1 ?
								LSCL( @"Are you sure you want to remove the selected subkey?", @"Confirm KeyInfo remove" ) :
								LSCL( @"Are you sure you want to remove the selected subkey? This is the last subkey. Removing it will disable encryption to this key.", @"Confirm KeyInfo remove2" ) ) );
					break;
				}
			break;
		}
		case kPGPKeyInfoButton_UpdateRevoker:
			row = [mRevokerTableView selectedRow];
			err = PGPGetIndexedRevocationKey( mKey, row, &assocKeyRef, &assocKeyID );
			if( ( row < 0 ) || IsPGPError( err ) )
				break;
		case kPGPKeyInfoButton_UpdateADK:
			if( tag == kPGPKeyInfoButton_UpdateADK )
			{
				row = [mADKTableView selectedRow];
				err = PGPGetIndexedAdditionalRecipientRequestKey( mKey, row, &assocKeyRef, &assocKeyID, &adkClass );
			}
			if( ( row < 0 ) || IsPGPError( err ) )
				break;
			if( PGPKeyDBObjRefIsValid( assocKeyRef ) )
				err = PGPclGetKeyFromServer( pgpContext, tlsContext, prefRef, assocKeyRef, &assocKeyDB );
			else
				err = PGPclGetKeyIDFromServer( pgpContext, tlsContext, prefRef, &assocKeyID, &assocKeyDB );
			if( IsntPGPError( err ) )
			{
				PGPUInt32	keyCount;
				
				PGPCountKeysInKeyDB( assocKeyDB, &keyCount );
				if( keyCount > 0 )
					[mController importKeyDB:assocKeyDB confirm:NO];
				else
				{
					NSBeginInformationalAlertSheet(
						LSCL( @"Key Not Found", @"Title of Key Not Found informational" ),
						LSCL( @"OK", @"" ), nil, nil, mKeyInfoWindow, 
						self, NULL, NULL, NULL,
						LSCL( @"The selected key could not be found on the server(s).", @"Not found message" ) );
					PGPFreeKeyDB( assocKeyDB );
				}
			}
			else
				PGPclErrorBox( err );
			break;
	}
}

- (int)numberOfRowsInTableView:(NSTableView *)tableView
{
	int numRows = 0;
	
	if( tableView == mSubkeyTableView )
	{
		PGPKeyDBObjRef	subKey;
		
		PGPKeyIterRewind( mKeyIter, kPGPKeyDBObjType_SubKey );
		while( IsntPGPError( PGPKeyIterNextKeyDBObj( mKeyIter, kPGPKeyDBObjType_SubKey, &subKey ) ) )
			numRows++;
		mNumSubkeys = numRows;
	}
	else if( tableView == mRevokerTableView )
	{
		PGPCountRevocationKeys( mKey, &numRows );
	}
	else if( tableView == mADKTableView )
	{
		PGPCountAdditionalRecipientRequests( mKey, &numRows );
	}
	return numRows;
}

- (id)tableView:(NSTableView *)tableView objectValueForTableColumn:(NSTableColumn *)tableColumn row:(int)row
{
	id	value = nil;
	
	if( tableView == mSubkeyTableView )
	{
		PGPKeyDBObjRef	subKey;
		PGPInt32		subkeyIndex = 0,
						bits;
		PGPTime			pgpTime;
		NSUserDefaults	*defaults	= [NSUserDefaults standardUserDefaults];
		
		PGPKeyIterRewind( mKeyIter, kPGPKeyDBObjType_SubKey );
		while( IsntPGPError( PGPKeyIterNextKeyDBObj( mKeyIter, kPGPKeyDBObjType_SubKey, &subKey ) ) )
			if( subkeyIndex++ == row )
				break;
		if( PGPKeyDBObjRefIsValid( subKey ) )
			switch( [[tableColumn identifier] intValue] )
			{
				case 1:
					PGPGetKeyDBObjTimeProperty( subKey, kPGPSubKeyProperty_Creation, &pgpTime );
					value = [[NSCalendarDate dateWithTimeIntervalSince1970:PGPGetStdTimeFromPGPTime( pgpTime )]
								descriptionWithCalendarFormat:[defaults stringForKey:NSShortDateFormatString]
								locale:[defaults dictionaryRepresentation]];
					break;
				case 2:
					PGPGetKeyDBObjTimeProperty( subKey, kPGPSubKeyProperty_Expiration, &pgpTime );
					if( pgpTime )
					{
						value = [[NSCalendarDate dateWithTimeIntervalSince1970:PGPGetStdTimeFromPGPTime( pgpTime )]
									descriptionWithCalendarFormat:[defaults stringForKey:NSShortDateFormatString]
									locale:[defaults dictionaryRepresentation]];
					}
					break;
				case 3:
					PGPGetKeyDBObjNumericProperty( subKey, kPGPSubKeyProperty_Bits, &bits );
					value = [[NSNumber numberWithInt:bits] stringValue];
					break;
			}
	}
	return value;
}

- (void)tableView:(NSTableView *)tableView willDisplayCell:(id)cell forTableColumn:(NSTableColumn *)tableColumn row:(int)row
{
	NSImage		*image = nil;
	char		keyIDString[kPGPMaxKeyIDStringSize];
	
	if( tableView == mSubkeyTableView )
	{
		PGPKeyDBObjRef	subKey;
		PGPInt32		subkeyIndex = 0;
		PGPTime			pgpTime;
		
		PGPKeyIterRewind( mKeyIter, kPGPKeyDBObjType_SubKey );
		while( IsntPGPError( PGPKeyIterNextKeyDBObj( mKeyIter, kPGPKeyDBObjType_SubKey, &subKey ) ) )
			if( subkeyIndex++ == row )
				break;
		if( !PGPKeyDBObjRefIsValid( subKey ) )
			return;
		switch( [[tableColumn identifier] intValue] )
		{
			case 1:
				[cell setImage:[[CPGPIcons shared] getKeyDBObjIcon:subKey]];
				break;
			case 2:
				PGPGetKeyDBObjTimeProperty( subKey, kPGPSubKeyProperty_Expiration, &pgpTime );
				if( !pgpTime )
					[cell setStringValue:LSCL( @"Never", @"Subkey never expires" )];
				break;
		}
	}
	else if( tableView == mRevokerTableView )
	{
		PGPKeyDBObjRef	revokerRef;
		PGPKeyID		keyID;
		
		if( IsntPGPError( PGPGetIndexedRevocationKey( mKey, row, &revokerRef, &keyID ) ) )
		{
			if( PGPKeyDBObjRefIsValid( revokerRef ) )
			{
				image = [[CPGPIcons shared] getKeyDBObjIcon:revokerRef];
				[cell setStringValue:PGPclGetPrimaryUserIDName( revokerRef )];
			}
			else
			{
				image = [[[CPGPIcons shared] images] objectAtIndex:kPGPclImage_DSAKey];
				PGPGetKeyIDString( &keyID, kPGPKeyIDString_Abbreviated, keyIDString );
				[cell setStringValue:[NSString stringWithFormat:kKeyInfoUnknownKeyString, keyIDString]];
			}
			[cell setImage:image];
		}
	}
	else if( tableView == mADKTableView )
	{
		PGPKeyDBObjRef	adkRef;
		PGPKeyID		keyID;
		PGPByte			adkClass;
		
		if( IsntPGPError( PGPGetIndexedAdditionalRecipientRequestKey( mKey, row, &adkRef, &keyID, &adkClass ) ) )
		{
			switch( [[tableColumn identifier] intValue] )
			{
				case 1:
					if( PGPKeyDBObjRefIsValid( adkRef ) )
					{
						image = [[CPGPIcons shared] getKeyDBObjIcon:adkRef];
						[cell setStringValue:PGPclGetPrimaryUserIDName( adkRef )];
					}
					else
					{
						image = [[[CPGPIcons shared] images] objectAtIndex:kPGPclImage_DSAKey];
						PGPGetKeyIDString( &keyID, kPGPKeyIDString_Abbreviated, keyIDString );
						[cell setStringValue:[NSString stringWithFormat:kKeyInfoUnknownKeyString, keyIDString]];
					}
					break;
				case 2:
					if( ( adkClass & 0x80 ) != 0 )
						image = [[[CPGPIcons shared] images] objectAtIndex:kPGPclImage_DotRed];
					break;
			} 
			[cell setImage:image];
		}
	}
}

- (void)tableViewSelectionDidChange:(NSNotification *)notification
{
	[self adjustButtons];
}

- (NSWindow *)window
{
	return mKeyInfoWindow;
}

- (PGPKeyDBObjRef)keyDBItem
{
	return mKey;
}

- (PGPKeyDBRef)keyDB
{
	return PGPPeekKeyDBObjKeyDB( mKey );
}

- (BOOL)windowShouldClose:(id)sender
{
	if( mChanged && !mLock )
	{
		PGPBoolean	axiomatic,
					disabled;
		int			itBoxState = [mImplicitTrustCheckbox state];
		uchar		trust;
		
		PGPGetKeyDBObjBooleanProperty( mKey, kPGPKeyProperty_IsAxiomatic, &axiomatic );
		PGPGetKeyDBObjBooleanProperty( mKey, kPGPKeyProperty_IsDisabled, &disabled );
		if( !axiomatic && itBoxState )
		{
			if( disabled )
				PGPSetKeyEnabled(mKey, TRUE);
			PGPSetKeyAxiomatic( mKey, TRUE, PGPOLastOption( PGPPeekKeyDBObjContext( mKey ) ) );
		}
		else
		{
			if( !itBoxState && axiomatic )
				PGPSetKeyAxiomatic( mKey, FALSE, PGPOLastOption( PGPPeekKeyDBObjContext( mKey ) ) );
			else if( [mTrustSlider isEnabled] )
			{
				switch( [mTrustSlider intValue] )
				{
					default:
					case 0:
						trust = kPGPKeyTrust_Never;
						break;
					case 1:
						trust = kPGPKeyTrust_Marginal;
						break;
					case 2:
						trust = kPGPKeyTrust_Complete;
						break;
				}
				PGPSetKeyTrust( mKey, trust );
			}
			PGPSetKeyEnabled( mKey, ( [mEnabledCheckbox state] != 0 ) );
		}
	}
	[mController keyInfoClosed:self withChanges:mChanged];
	[sKeyInfoArray removeObject:self];
	return YES;
}

- (void)dealloc
{
	if( PGPKeyIterRefIsValid( mKeyIter ) )
	{
		PGPFreeKeyIter( mKeyIter );
		mKeyIter = kInvalidPGPKeyIterRef;
	}
	if( mSubkeyTabView )
		[mSubkeyTabView release];
	if( mRevokerTabView )
		[mRevokerTabView release];
	if( mADKTabView )
		[mADKTabView release];
	[mNewSubkeyParamsBox release];
	[mNewSubkeyGenProgress release];
	[super dealloc];
}

@end

