/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation
    All rights reserved.

    $Id: CKeyRecon.m,v 1.7 2002/08/06 20:09:17 dallen Exp $
____________________________________________________________________________*/
#import "CKeyRecon.h"
#import <PGPclient/pgpPFLPriv.h>
#import <PGPclient/CShareholderItem.h>
#import <PGPclient/CRemoteAuth.h>
#import <PGPclient/pgpFileSpec.h>
#import <PGPclient/pgpShareFile.h>
#import <PGPclient/pgpClientPrefs.h>
#import <PGPui/CPGPIcons.h>

#define kCKeyReconNib				@"CKeyRecon"
#define kCKeyReconIconColumn		0
#define kCKeyReconNameColumn		1
#define kCKeyReconSharesColumn		2

#define kCKeyReconStopString		LSCL( @"Stop", @"Key Recon network Stop button" )
#define kCKeyReconListenString		LSCL( @"Listen", @"Key Recon network Listen button" )
#define kCKeyReconOKString			LSCL( @"OK", @"OK Button" )

	PGPError
PGPclReconstitutionDialog(
	PGPKeyDBObjRef				keyToCollect,
	PGPKeyDBRef					keyDB,
	PGPtlsContextRef			tlsContext,
    PGPPrefRef					prefRef,
	PGPByte **					passKey,
	PGPSize *					passKeySize)
{
    PGPError					err = kPGPError_NoErr;
	CKeyRecon					*keyRecon;
	
    PGPValidatePtr(passKey);
	*passKey = NULL;
	PGPValidateParam(PGPKeyDBObjRefIsValid(keyToCollect));
	PGPValidateParam(PGPKeyDBRefIsValid(keyDB));
	PGPValidatePtr(passKeySize);
	do
	{
		keyRecon = [[CKeyRecon alloc] initKeyRecon:keyToCollect
						mainDB:keyDB
						tlsContext:tlsContext
						prefRef:prefRef
						passKey:passKey
						passKeySize:passKeySize];
						
		err = [keyRecon runKeyReconModalDialog];
	
		[keyRecon release];
		if( err == kPGPError_BadPassphrase )
		{
			PGPFreeData(*passKey);
			*passKey = NULL;
			NSRunAlertPanel( LSCL( @"The key could not be properly reconstituted using the shares provided.", @"Bad Reconstitution title" ),
				LSCL( @"Please verify that the shares being used are correct for this key, and that the key has not been re-split since the shares were created.", @"Bad Reconstitution msg" ),  kCKeyReconOKString, nil, nil );
		}
	} while( err == kPGPError_BadPassphrase );

    return( err );
}

	static PGPError
decryptCallback(
	PGPContextRef 	context,
	PGPEvent 		*event,
	PGPUserValue 	userValue)
{
	CKeyRecon		*keyRecon = (CKeyRecon *)userValue;
	
	return [keyRecon decryptCallback:event];
}

	PGPError
skepReconCallback(
	PGPskepRef			skepRef,
	PGPskepEvent		*event,
	PGPUserValue		userValue )
{
	CKeyRecon			*keyRecon = (CKeyRecon *)userValue;
	
	return [keyRecon skepReconCallback:event skep:skepRef];
}

@implementation CKeyRecon

- (id)init
{
	if( !mInited )
	{
		[super init];
		mInited = YES;
	}
	if( !mLoadedNib )
		[self initWithNib:kCKeyReconNib];
    return self;
}

- (id)initWithNib:
	(NSString*)			nibFile
{
	NSTextFieldCell		*tCell;
	NSImageCell			*iCell;
	NSFont				*smallFont = [NSFont systemFontOfSize:[NSFont smallSystemFontSize]];

	[NSBundle loadNibNamed:nibFile owner:self];
	mLoadedNib = YES;
	mUserImage			= [[NSImage alloc] initByReferencingFile:
							[[NSBundle bundleForClass:[CKeyRecon class]] pathForImageResource:@"user"]];
	
	iCell = [[NSImageCell alloc] init];
	[iCell setImageScaling:NSScaleProportionally];
	[iCell setImageAlignment:NSImageAlignCenter];
	[iCell setImageFrameStyle:NSImageFrameNone];
	[[[mShareTableView tableColumns] objectAtIndex:kCKeyReconIconColumn] setDataCell:iCell];
	[iCell release];

	tCell = [[NSTextFieldCell alloc] init];
	[tCell setFont:smallFont];
	[tCell setEditable:NO];
	[tCell setWraps:NO];
	[[[mShareTableView tableColumns] objectAtIndex:kCKeyReconNameColumn] setDataCell:tCell];
	[[[mShareTableView tableColumns] objectAtIndex:kCKeyReconSharesColumn] setDataCell:tCell];
	[tCell release];

    return [self init];
}

- (id)initKeyRecon:(PGPKeyDBObjRef)collectionKey
			mainDB:(PGPKeyDBRef)mainKeyDB
			tlsContext:(PGPtlsContextRef)tlsContext
			prefRef:(PGPPrefRef)prefRef
			passKey:(PGPByte **)passKey
			passKeySize:(PGPSize *)passKeySize;
{	
	[self init];
	mKey				= collectionKey;
	mTLSContext			= tlsContext;
	mPrefRef			= prefRef;
	mKeyDB				= mainKeyDB;
	mShares				= kInvalidPGPShareRef;
	mOutputPassKey		= passKey;
	mOutputPassKeySize	= passKeySize;
	mSKEPPassphrase		= nil;
	mAuthUserID			= nil;
	mShareArray			= [[NSMutableArray alloc] init];
	[mNetworkHelpField retain];
	[mStatusTitleField retain];
	[mStatusField retain];
	[mStatusTitleField removeFromSuperview];
	[mStatusField removeFromSuperview];
	
	[mKeyImage setImage:[[CPGPIcons shared] getKeyDBObjIcon:mKey]];
	[mKeyField setStringValue:PGPclGetPrimaryUserIDName( mKey )];
	
	[mOKButton setEnabled:NO];
	
	return self;
}

- (PGPError)runKeyReconModalDialog
{
	PGPError	err = kPGPError_NoErr;
	
	err = [[NSApplication sharedApplication] runModalForWindow:mKeyReconWindow];
	[mKeyReconWindow close];
    return err;
}

- (IBAction)dialogButtonPressed:(id)sender
{
	PGPError	err = kPGPError_NoErr;
	NSDate		*sleepUntil = [NSDate dateWithTimeIntervalSinceNow:1];
	
	while( mListening )
	{
		mAbortImmediate = YES;
		mAbortSKEP = YES;
		[NSThread sleepUntilDate:sleepUntil];
		[sleepUntil addTimeInterval:1];
	}
	if( [sender tag] )
		err = kPGPError_UserAbort;
	else
	{
		PGPContextRef		context		= PGPPeekKeyDBContext( mKeyDB );
		
		err = PGPGetPasskeyFromShares( mShares, mOutputPassKey, mOutputPassKeySize );
		if( IsntPGPError( err ) &&
			!PGPPassphraseIsValid( mKey,
				PGPOPasskeyBuffer( context, *mOutputPassKey, *mOutputPassKeySize ),
				PGPOLastOption( context ) ) )
			err = kPGPError_BadPassphrase;
	}
	[[NSApplication sharedApplication] stopModalWithCode:err];
}

- (void)addShares:(PGPShareRef *)shares from:(NSString *)userID
{
	PGPError		err = kPGPError_NoErr;
	PGPUInt32		numShares = PGPGetNumberOfShares( *shares );
	
	if( PGPShareRefIsValid( mShares ) )
	{
		PGPShareRef	combinedShares;
		
		err = PGPCombineShares(	mShares, *shares, &combinedShares );
		if( IsntPGPError( err ) )
		{
			PGPFreeShares( mShares );
			PGPFreeShares( *shares );
			*shares = kInvalidPGPShareRef;
			mShares = combinedShares;
		}
	}
	else
	{
		PGPKeyID	collectKeyID,
					keyID;

		err = PGPGetKeyIDFromShares( *shares, &keyID);
		if( IsntPGPError( err ) )
		{
			PGPGetKeyID( mKey, &collectKeyID );
			if( PGPCompareKeyIDs( &collectKeyID, &keyID ) != 0 )
			{
				NSRunAlertPanel( LSCL( @"The shares provided do not belong to this key.", @"Bad Reconstitution title 2" ),
					LSCL( @"The key cannot be reconstituted using the wrong shares.", @"Bad Reconstitution msg 2" ), 
					kCKeyReconOKString, nil, nil );
				err = kPGPError_BadParams;
			}
			else
			{
				[mNeededField setIntValue:PGPGetShareThreshold( *shares )];
				mShares = *shares;
				*shares = kInvalidPGPShareRef;
			}
		}
	}
	if( IsntPGPError( err ) )
	{
		CShareholderItem	*shItem = [[CShareholderItem alloc] initUserItem];
		
		[mCollectedField setIntValue:PGPGetNumberOfShares( mShares )];
		if( [mCollectedField intValue] >= [mNeededField intValue] )
			[mOKButton setEnabled:YES];
		[shItem setName:userID];
		[shItem setNumShares:numShares];
		[mShareArray addObject:shItem];
		[mShareTableView reloadData];
		[shItem release];
	}
}

- (IBAction)browsePressed:(id)sender
{
	NSOpenPanel			*openPanel = [NSOpenPanel openPanel];
	NSMutableArray*		types = [[[NSMutableArray alloc] init] autorelease];
	
	[types addObject:NSFileTypeForHFSTypeCode(kPGPShareFileOSType)];
	[types addObject:@"shf"];
	[openPanel setAllowsMultipleSelection:YES];
	if( [openPanel runModalForDirectory:nil file:nil types:types] == NSOKButton )
	{
		NSEnumerator		*fileEnumerator = [[openPanel filenames] objectEnumerator];
		NSString			*filename;
		
		while( filename = [fileEnumerator nextObject] )
		{
			PGPError			err			= kPGPError_NoErr;
			PGPContextRef		context		= PGPPeekKeyDBContext( mKeyDB );
			PFLFileSpecRef		fileSpecRef	= NULL;
			PGPShareFileRef		sfRef		= NULL;
			PGPShareRef			shares		= kInvalidPGPShareRef;
			PGPOptionListRef	decodeOpt	= kInvalidPGPOptionListRef;
			PGPSize				nameSize	= 0;
			char				name[kPGPcl_MaxPGPUserIDLength];
			
			err = PFLNewFileSpecFromFullPath( PGPPeekContextMemoryMgr( context ),
					[filename cString], &fileSpecRef);														CKERR;
			err = PGPOpenShareFile( fileSpecRef, &sfRef );													CKERR;
			err = PGPGetShareFileUserID( sfRef, sizeof( name ), name, &nameSize );							CKERR;
			err = PGPBuildOptionList( context, &decodeOpt,
					PGPOKeyDBRef( context, mKeyDB ),
					PGPOEventHandler( context, decryptCallback, self ),
					PGPOLastOption( context ) );															CKERR;
			err = PGPCopySharesFromFile( context, sfRef, decodeOpt, &shares );								CKERR;
			[self addShares:&shares from:[NSString stringWithCString:name]];
		done:
			if( PGPShareRefIsValid( shares ) )
				PGPFreeShares( shares );
			if( PGPKeySetRefIsValid( mRecipientSet ) )
				PGPFreeKeySet( mRecipientSet );
			if( IsntNull( mKeyIDList ) )
				PGPFreeData( mKeyIDList );
			if( PGPOptionListRefIsValid( decodeOpt ) )
				PGPFreeOptionList( decodeOpt );
			if( PGPShareFileRefIsValid( sfRef ) )
				PGPFreeShareFile( sfRef );
			if( IsntNull( fileSpecRef ) )
				PFLFreeFileSpec( fileSpecRef );
		}
	}
}

- (PGPError)decryptCallback:(PGPEvent *)event
{
	PGPError			err = kPGPError_NoErr;
	PGPContextRef		context = PGPPeekKeyDBContext( mKeyDB );
	
	switch( event->type )
	{
		case kPGPEvent_PassphraseEvent:
		{
			char				*passphrase;
			PGPKeyDBObjRef		key;

			if( event->data.passphraseData.fConventional )
			{
				err = PGPConventionalDecryptionPassphraseDialog( context,
						PGPOUIDialogPrompt( context,
						[LSCL(@"Please enter the share passphrase:", @"Key Recon Conv Passphrase") cString] ),
						PGPOUIOutputPassphrase( context, &passphrase ),
						PGPOLastOption( context ) );
			}
			else
			{
				err = PGPclDecryptionPassphraseDialog( context,
						mTLSContext, mPrefRef,
						[LSCL(@"Please enter the key passphrase:", @"Key Recon Passphrase PK1") cString],
						mRecipientSet, mKeyIDList, mKeyIDCount,
						kInvalidPGPOptionListRef, &passphrase, &key, NULL );
			}
			if( IsntPGPError( err ) )
			{
				err = PGPAddJobOptions( event->job,
							PGPOPassphrase( context, passphrase ),
							PGPOLastOption( context ) );
				PGPFreeData(passphrase);
			}
			break;
		}
		case kPGPEvent_RecipientsEvent:
			mRecipientSet = event->data.recipientsData.recipientSet;
			if( PGPKeySetRefIsValid( mRecipientSet ) )
				err = PGPIncKeySetRefCount( mRecipientSet );
			if( IsntPGPError( err ) && IsntNull( event->data.recipientsData.keyIDArray ) )
			{
				PGPSize	dataSize;
				
				mKeyIDCount 	= event->data.recipientsData.keyCount;
				dataSize		= mKeyIDCount * sizeof( PGPKeyID );
				mKeyIDList		= (PGPKeyID *)PGPNewData( PGPPeekContextMemoryMgr( context ), dataSize, 0 );
				if( IsntNull( mKeyIDList ) )
					pgpCopyMemory( event->data.recipientsData.keyIDArray, mKeyIDList, dataSize );
				else
					err = kPGPError_OutOfMemory;
			}
			break;
		default:
			break;
	}
	return err;
}

- (PGPError)skepReconCallback:(PGPskepEvent *)event skep:(PGPskepRef)skepRef
{
	PGPError			err = kPGPError_NoErr;
	
	if( mAbortSKEP )
	{
		PGPskepCancel( skepRef );
		err = kPGPError_UserAbort;
	}
	else switch( event->type )
	{
		case kPGPskepEvent_ConnectEvent:
			[mStatusField setStringValue:LSCL(@"Connected",@"Key Recon Status:Connected")];
			break;
		case kPGPskepEvent_ProgressEvent:
			[mStatusField setStringValue:LSCL(@"Receiving",@"Key Recon Status:Receiving")];
			break;
		case kPGPskepEvent_CloseEvent:
			[mStatusField setStringValue:LSCL(@"Listening",@"Key Recon Status:Listening")];
			break;
		case kPGPskepEvent_AuthenticateEvent:
		{
			PGPKeySetRef	keySet = kInvalidPGPKeySetRef;
			PGPInt32		validity = 0;
			
			if( mAuthUserID )
				[mAuthUserID release];
			mAuthUserID = nil;
			mAuthUserID = PGPclGetPrimaryUserIDName( event->data.ad.remoteKey );
			[mAuthUserID retain];
			keySet = PGPPeekKeyDBRootKeySet( PGPPeekKeyDBObjKeyDB( event->data.ad.remoteKey ) );
			PGPCheckKeyRingSigs( keySet, mKeyDB, FALSE, NULL, NULL );
			PGPCalculateTrust( keySet, mKeyDB );
			PGPGetKeyDBObjNumericProperty( event->data.ad.remoteKey, kPGPKeyProperty_Validity, &validity );
			if( validity != kPGPValidity_Complete )
				err = PGPclConfirmRemoteKeyDialog(
						[LSCL(@"Please confirm the remote key to verify that the correct party is sending shares to you.",
						@"Key Recon Confirm") cString],
						event->data.ad.remoteHostname,
						event->data.ad.remoteKey,
						event->data.ad.tlsCipher, TRUE );
			if( IsPGPError( err ) )
				break;
			[mStatusField setStringValue:LSCL(@"Authenticated",@"Key Recon Status:Authenticated")];
			break;
		}
		case kPGPskepEvent_ShareEvent:
			[self addShares:&event->data.sd.shares from:mAuthUserID];
			if( PGPShareRefIsValid( event->data.sd.shares ) )
				PGPFreeShares( event->data.sd.shares );
			break;
		default:
			break;
	}
	return err;
}

- (void)listenForShares:(id)sender
{
	NSAutoreleasePool	*pool	= [[NSAutoreleasePool alloc] init];
	PGPError			err		= kPGPError_NoErr;
	PGPContextRef		context = PGPPeekKeyDBContext( mKeyDB );
	PGPskepRef			skepRef	= kInvalidPGPskepRef;
			
	err = PGPNewSKEP( context, mTLSContext, &skepRef );
	if( IsntPGPError( err ) )
	{
		err = PGPskepSetEventHandler( skepRef, skepReconCallback, self );
		PGPclGuaranteeMinimumEntropy( context );
		if( IsntPGPError( err ) )
			err = PGPskepReceiveShares(	skepRef, mSKEPKey, mSKEPPassphrase );
		PGPFreeSKEP( skepRef );
	}
	if( !mAbortImmediate )
	{
		if( IsPGPError( err ) )
			PGPclErrorBox( err );
		[mStatusTitleField removeFromSuperview];
		[mStatusField removeFromSuperview];
		[mNetworkBoxView addSubview:mNetworkHelpField];
		[mListenButton setTitle:kCKeyReconListenString];
	}
	mListening = NO;
	[pool release];
}

- (IBAction)listenPressed:(id)sender
{
	if( mListening )
		mAbortSKEP = YES;
	else
	{
		PGPError			err = kPGPError_NoErr;
		PGPContextRef		context = PGPPeekKeyDBContext( mKeyDB );
		PGPKeyID			*keyID;
		PGPSize				keyIDLen;
		PGPKeyDBObjRef		defaultKey = kInvalidPGPKeyDBObjRef;
		
		mAbortSKEP = NO;
		err = PGPGetPrefData( mPrefRef, kPGPPrefDefaultKeyID, &keyIDLen, (void **)&keyID );
		if( IsntPGPError( err ) && ( keyIDLen == sizeof(PGPKeyID) ) )
		{
			PGPFindKeyByKeyID( mKeyDB, keyID, &defaultKey );
			PGPFreeData( keyID );
		}
		if( !PGPKeyDBObjRefIsValid( mSKEPKey ) )
		{
			err = PGPSigningPassphraseDialog( context, mKeyDB, &mSKEPKey,
					PGPOUIDialogPrompt(	context,
						[LSCL(@"Please enter the authentication passphrase:", @"Key Recon Auth Passphrase") cString] ),
					PGPOUIDefaultKey( context, defaultKey ),
					PGPOUIOutputPassphrase(	context, &mSKEPPassphrase ),
					PGPOLastOption( context ) );
		}
		if( IsntPGPError( err ) )
		{
			[mNetworkHelpField removeFromSuperview];
			[mNetworkBoxView addSubview:mStatusField];
			[mNetworkBoxView addSubview:mStatusTitleField];
			[mListenButton setTitle:kCKeyReconStopString];
			mListening = YES;
			[NSThread detachNewThreadSelector:@selector(listenForShares:) toTarget:self withObject:self];
		}
	}
}

- (int)numberOfRowsInTableView:(NSTableView *)tableView
{
	return [mShareArray count];
}

- (id)tableView:(NSTableView *)tableView objectValueForTableColumn:(NSTableColumn *)tableColumn row:(int)row
{
	id					value = nil;
	CShareholderItem	*shItem = [mShareArray objectAtIndex:row];
	PGPKeyID			keyID;
	PGPKeyDBObjRef		key;

	switch( [[tableColumn identifier] intValue] )
	{
		case kCKeyReconIconColumn:
			if( [shItem isPKItem] )
			{
				[shItem keyID:&keyID];
				if( IsntPGPError( PGPFindKeyByKeyID( mKeyDB, &keyID, &key ) ) )
					value = [[CPGPIcons shared] getKeyDBObjIcon:key];
			}
			else
				value = mUserImage;
			break;
		case kCKeyReconNameColumn:
			if( [shItem isPKItem] )
			{
				[shItem keyID:&keyID];
				if( IsntPGPError( PGPFindKeyByKeyID( mKeyDB, &keyID, &key ) ) )
					value = PGPclGetPrimaryUserIDName( key );
			}
			else
				return [shItem name];
			break;
		case kCKeyReconSharesColumn:
			value = [NSNumber numberWithInt:[shItem numShares]];
			break;
	}
	return value;
}

- (void)dealloc
{
	if( mShareArray )
	{
		[mShareArray release];
		mShareArray = nil;
	}
	if( IsntNull( mSKEPPassphrase ) )
		PGPFreeData( mSKEPPassphrase );
	if( PGPShareRefIsValid( mShares ) )
		PGPFreeShares( mShares );
	if( mAuthUserID )
		[mAuthUserID release];
	[mStatusTitleField release];
	[mStatusField release];
	[mNetworkHelpField release];
	[mUserImage release];
	[super dealloc];
	PGPLog(( @"CKeyRecon dealloc" ));
}

@end

