/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation
    All rights reserved.

    $Id: CKeyView.h,v 1.15 2002/10/30 07:29:01 wprice Exp $
____________________________________________________________________________*/
#import <Cocoa/Cocoa.h>
#import <PGP/pgpKeys.h>
#import "PGPController.h"

#define kPGPclKeyList_MaxColumns			9

enum
{
	kPGPclKeyList_UserIDColumn		= 0,
	kPGPclKeyList_ValidityColumn,
	kPGPclKeyList_TrustColumn,
	kPGPclKeyList_SizeColumn,
	kPGPclKeyList_CreationColumn,
	kPGPclKeyList_ExpirationColumn,
	kPGPclKeyList_KeyIDColumn,
	kPGPclKeyList_ADKColumn,
	kPGPclKeyList_DescriptionColumn
};

typedef struct PGPclKeyListColumnInfo
{
	PGPInt16	numActiveColumns;
	PGPInt16	columns[kPGPclKeyList_MaxColumns];
	PGPInt16	columnWidths[kPGPclKeyList_MaxColumns];
} PGPclKeyListColumnInfo;

extern PGPclKeyListColumnInfo kDefaultColumnInfo;

enum
{
	kPGPclKeyList_ReadOnly			= (1UL << 0),
	kPGPclKeyList_GreenDotMode		= (1UL << 1),
	kPGPclKeyList_MarginalIsInvalid	= (1UL << 2),
	kPGPclKeyList_SelectKeysOnly	= (1UL << 3),
	kPGPclKeyList_SingleSelect		= (1UL << 4),
	kPGPclKeyList_EnableDropIn		= (1UL << 5),
	kPGPclKeyList_EnableDragOut		= (1UL << 6),
	kPGPclKeyList_ValidityButton	= (1UL << 7)
};

// Selection info flags
enum
{
	kPGPclKeyList_None				= 0,
	kPGPclKeyList_Key				= (1UL << 0),
	kPGPclKeyList_UserID			= (1UL << 1),
	kPGPclKeyList_Signature			= (1UL << 2)
};

@class NSMutableArray;
@class CKeyDBItem;
@class CKeySelect;

@protocol PGPKeyView
- (PGPKeyDBRef)mainDB;
- (PGPPrefRef)pgpPrefRef;
- (PGPtlsContextRef)tlsContext;
- (void)selectionChanged;
- (void)validityButtonPressed:(id)sender;
@end


@interface CKeyView : NSResponder
{
	id <PGPKeyView>			mController;
	
	BOOL					mInited;
	BOOL					mLoadedNib;
	BOOL					mOwnsKeySet;
	BOOL					mInModal;
	
	IBOutlet NSView			*mCKeyView;
	IBOutlet NSOutlineView	*mKeyOutline;
	IBOutlet NSScrollView	*mKeyScrollView;

	PGPContextRef			mPGPContext;
	PGPKeyDBRef				mKeyDB;
	PGPKeyDBRef				mImportDB;
	PGPKeySetRef			mKeySet;
	PGPKeyListRef			mKeyList;
	PGPKeyIterRef			mKeyIter;
	PGPKeySetRef			mDragPBKeySet;
	PGPKeySetRef			mCopyPBKeySet;
	int						mCopyPBChangeCount;
	PGPPrefRef				mPrefRef;
	PGPUInt32				mFlags;
	CKeyDBItem				*mDefaultKey;
	CKeySelect				*mKeySelector;
	
	NSMutableArray			*mKeyArray;
	NSMutableString			*mSearchString;
	NSTimeInterval			mLastKeypress;
    NSTableColumn			*mColumns[kPGPclKeyList_MaxColumns];
}

- (id)init;
- (id)initWithController:(id)controller;
- (id)initWithNib:(NSString*)nibFile;

- (void)setActive:(BOOL)flag;
- (void)setColumns:(PGPclKeyListColumnInfo)columns;
- (PGPclKeyListColumnInfo)columns;
- (void)setFlags:(PGPUInt32)flags;
- (PGPError)loadKeys:		(PGPKeyDBRef)			keyDB
				showKeySet:	(PGPKeySetRef)			keySet;
- (void)emptyKeys;
- (PGPKeyDBObjRef)selectionInfo:(BOOL *)multiple withFlags:(PGPUInt32 *)flags;
- (void)selectAll:(id)sender;
- (void)deselectAll:(id)sender;
- (NSMutableArray *)selectedItemArray;
- (NSEnumerator *)selectedItemEnumerator;
- (void)selectItemsInArray:(NSArray *)array;
- (PGPKeySetRef)getEnumeratorKeySet:(NSEnumerator *)enumerator;
- (void)addKeySet:(PGPKeySetRef)keySet toPasteboard:(NSPasteboard *)pboard;
- (void)setDefault;
- (PGPKeyDBObjRef)lookupDefaultKey;
- (void)showInfo;
- (void)enableSelection:(BOOL)enabled;
- (BOOL)isModal;
- (PGPKeyDBRef)importFileList:(NSArray *)fileList;
- (void)importKeyDB:(PGPKeyDBRef)newKeyDB confirm:(BOOL)confirm;
- (void)exportSelection:(NSString *)filename privates:(BOOL)includePrivate extensions:(BOOL)newStyle hideSuffix:(BOOL)hideSuffix;

- (void)cut:(id)sender;
- (void)copy:(id)sender;
- (void)paste:(id)sender;
- (void)clear:(id)sender;
- (void)insertTab:(id)sender;
- (void)insertBacktab:(id)sender;
- (void)cleanPasteboards;
- (void)keyInfoClosed:(id)sender withChanges:(BOOL)changed;

- (PGPError)syncWithKeyDB:(BOOL)onlyUnmarked;
- (void)reload;
- (NSImage *)getValidityImage:(CKeyDBItem *)item;
- (void)selectSearchString;
- (id)view;
- (NSView *)keyView;
- (NSOutlineView*)outlineView;
- (PGPPrefRef)pgpPrefRef;
- (PGPKeyDBRef)mainDB;
- (PGPtlsContextRef)tlsContext;

- (void)dealloc;

@end

