/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation
    All rights reserved.

    $Id: CKeyView.m,v 1.38 2002/11/18 11:35:22 jeffc Exp $
____________________________________________________________________________*/
#import "pflPrefTypes.h"
#import "pgpClientPrefs.h"
#import "pgpVersionHeader.h"
#import "pgpClientLibCocoa.h"
#import "CKeyView.h"
#import "CKeyDBItem.h"
#import <PGPui/CPGPIcons.h>
#import "CImageButtonCell.h"
#import "CKeyInfo.h"
#import "CCertInfo.h"
#import "CRemoteAuth.h"
#import <PGPui/CBrowserCell.h>

#define kCKeyViewNib						@"CKeyView"
#define kCKeyViewOKButton					LSCL( @"OK", @"OK" )

const PGPUserValue		kUserValueMarked		=	(PGPUserValue)1;
const PGPUserValue		kUserValueUnmarked		=	(PGPUserValue)0;
const NSTimeInterval	kMaxKeypressInterval	=	1.0;

PGPclKeyListColumnInfo	kDefaultColumnInfo	=
	{
		5,
		{kPGPclKeyList_UserIDColumn,
		kPGPclKeyList_ValidityColumn,
		kPGPclKeyList_TrustColumn,
		kPGPclKeyList_SizeColumn,
		kPGPclKeyList_DescriptionColumn,
		0,0,0,0},
		{220, 47, 47, 70, 71, 71, 76, 35, 110}
	};


@implementation CKeyView

- (id)init
{
	if( !mInited )
	{
		[super init];
		mInited = YES;
		mKeyDB = mImportDB = kInvalidPGPKeyDBRef;
		mKeyArray = nil;
		mLastKeypress = 0.0;
		mSearchString = [[NSMutableString alloc] init];
		mPrefRef = [mController pgpPrefRef];
	}
	if( !mLoadedNib )
		[self initWithNib:kCKeyViewNib];
    return self;
}

- (id)initWithNib:(NSString*)nibFile
{
	NSEnumerator	*columnEnumerator;
	NSTableColumn	*column;
	int				columnIndex;
	
	if( nibFile && [nibFile length] )
		[NSBundle loadNibNamed:nibFile owner:self];
	mLoadedNib = YES;
	columnEnumerator = [[mKeyOutline tableColumns] objectEnumerator];
	for( columnIndex = 0; ( column = [columnEnumerator nextObject] ); columnIndex++ )
	{
		[column retain];
		mColumns[columnIndex] = column;
	}
	return [self init];
}

- (id)initWithController:(id)controller
{
	mController = controller;
	return [self init];
}

- (BOOL)acceptsFirstResponder
{
	return YES;
}

- (void)setActive:(BOOL)flag
{
	if( flag )
	{
		NSWindow*	window = [mKeyOutline window];
		
		[window makeFirstResponder:mKeyOutline];
		[mKeyOutline setNextResponder:self];
		[self setNextResponder:[mKeyOutline window]];
	}
}

- (id)view
{
	return mCKeyView;
}

- (NSView *)keyView
{
	return mKeyOutline;
}

- (NSOutlineView*)outlineView
{
	return mKeyOutline;
}

- (void)setFlags:(PGPUInt32)flags
{
	mFlags = flags;
}

- (PGPError)loadKeys:
				(PGPKeyDBRef)	keyDB
	showKeySet:	(PGPKeySetRef)	keySet
{
	PGPError					err = kPGPError_NoErr;
	
	if( !PGPKeyDBRefIsValid( keyDB ) )
	{
		err = kPGPError_BadParams;
		goto done;
	}
	mPGPContext = PGPPeekKeyDBContext( keyDB );
	[self emptyKeys];
	mKeyDB = keyDB;
	mKeySet = keySet;
	if( !PGPKeySetRefIsValid( mKeySet ) )
	{
		err = PGPNewKeySet( mKeyDB, &mKeySet );	CKERR;
		mOwnsKeySet = YES;
	}
	else
		mOwnsKeySet = NO;
	err = PGPOrderKeySet( mKeySet, kPGPKeyOrdering_UserID, FALSE, &mKeyList ); CKERR;
	err = PGPNewKeyIter( mKeyList, &mKeyIter );	CKERR;

	err = [self syncWithKeyDB:NO];	CKERR;
done:
	return err;
}

- (void)cleanPasteboards
{
	if( PGPKeySetRefIsValid( mCopyPBKeySet ) &&
		( [[NSPasteboard generalPasteboard] changeCount] == mCopyPBChangeCount ) )
	{
		[[NSPasteboard generalPasteboard] declareTypes:[NSArray arrayWithObjects:NSStringPboardType, nil] owner:nil];
		[self pasteboard:[NSPasteboard generalPasteboard] provideDataForType:NSStringPboardType];
	}
	if( PGPKeySetRefIsValid( mDragPBKeySet ) )
		[[NSPasteboard pasteboardWithName:NSDragPboard] declareTypes:nil owner:nil];
}

- (void)emptyKeys
{
	if( PGPKeyDBRefIsValid( mKeyDB ) )
	{
		[CKeyInfo closeAll:self forDB:mKeyDB];
		[CCertInfo closeAll:self forDB:mKeyDB];
	}
	[self cleanPasteboards];
	if( mKeyArray )
		[mKeyArray removeAllObjects];
	if( PGPKeyIterRefIsValid( mKeyIter ) )
		PGPFreeKeyIter( mKeyIter );
	if( PGPKeyListRefIsValid( mKeyList ) )
		PGPFreeKeyList( mKeyList );
	if( PGPKeySetRefIsValid( mCopyPBKeySet ) )
		PGPFreeKeySet( mCopyPBKeySet );
	if( PGPKeySetRefIsValid( mDragPBKeySet ) )
		PGPFreeKeySet( mDragPBKeySet );
	if( PGPKeySetRefIsValid( mKeySet ) && mOwnsKeySet )
		PGPFreeKeySet( mKeySet );
	mCopyPBKeySet = kInvalidPGPKeySetRef;
	mDragPBKeySet = kInvalidPGPKeySetRef;
	mKeyIter = kInvalidPGPKeyIterRef;
	mKeyList = kInvalidPGPKeyListRef;
	mKeySet = kInvalidPGPKeySetRef;
	mKeyDB = kInvalidPGPKeyDBRef;
	[mKeyOutline reloadData];
}

- (PGPKeyDBObjRef)lookupDefaultKey
{
	PGPError		err = kPGPError_NoErr;
	PGPKeyID		*keyID;
	PGPSize			keyIDLen;
	PGPKeyDBObjRef	key = kInvalidPGPKeyDBObjRef;
	
	err = PGPGetPrefData( mPrefRef, kPGPPrefDefaultKeyID, &keyIDLen, (void **)&keyID );
	if( IsntPGPError( err ) )
	{
		PGPFindKeyByKeyID( mKeyDB, keyID, &key );
		PGPFreeData( keyID );
	}
	return key;
}

- (void)unmarkKey:(PGPKeyDBObjRef)key
{
	PGPKeyDBObjRef	item;
	PGPError		err;

	PGPKeyIterRewind( mKeyIter, kPGPKeyDBObjType_Key );
	while( IsntPGPError( err = PGPKeyIterNextKeyDBObj( mKeyIter, kPGPKeyDBObjType_Key, &item ) ) )
	{
		if( item == key )
		{
			PGPSetKeyDBObjUserValue( item, kUserValueUnmarked );
			while( IsntPGPError( err = PGPKeyIterNextKeyDBObj( mKeyIter, kPGPKeyDBObjType_UserID, &item ) ) )
			{
				PGPSetKeyDBObjUserValue( item, kUserValueUnmarked );
				while( IsntPGPError( err = PGPKeyIterNextKeyDBObj( mKeyIter, kPGPKeyDBObjType_Signature, &item ) ) )
					PGPSetKeyDBObjUserValue( item, kUserValueUnmarked );
			}
			break;
		}
	}
}

- (PGPError)syncWithKeyDB:(BOOL)onlyUnmarked
{
	PGPError					err = kPGPError_NoErr;
	PGPKeyDBObjRef				key,
								user,
								sig,
								defaultKey;
	PGPUserValue				userVal;
	PGPUInt32					keyArrayIndex,
								userArrayIndex,
								sigArrayIndex,
								numKeys;
	CKeyDBItem					*kItem,
								*uItem,
								*sItem;
	
	if( !mKeyArray )
	{
		PGPCountKeys( mKeySet, &numKeys );
		mKeyArray = [[NSMutableArray alloc] initWithCapacity:numKeys * 4];
	}
	mDefaultKey = nil;
	defaultKey = [self lookupDefaultKey];
	PGPKeyIterRewind( mKeyIter, kPGPKeyDBObjType_Key );
	for( keyArrayIndex = 0; IsntPGPError( PGPKeyIterNextKeyDBObj( mKeyIter, kPGPKeyDBObjType_Key, &key ) ); keyArrayIndex++ )
	{
		PGPGetKeyDBObjUserValue(key, &userVal);
		if( ( userVal != kUserValueMarked ) || !onlyUnmarked)
		{
			kItem = [[CKeyDBItem alloc] initWithItem:key withParent:nil];
			[mKeyArray insertObject:kItem atIndex:keyArrayIndex];
			[kItem release];
			PGPSetKeyDBObjUserValue(key, kUserValueMarked);
		}
		else
			kItem = [mKeyArray objectAtIndex:keyArrayIndex];
		if( defaultKey == key )
		{
			[kItem setDefault:YES];
			mDefaultKey = kItem;
		}
		for( userArrayIndex = 0; IsntPGPError( PGPKeyIterNextKeyDBObj( mKeyIter, kPGPKeyDBObjType_UserID, &user ) ); userArrayIndex++ )
		{
			PGPGetKeyDBObjUserValue( user, &userVal );
			if( ( userVal != kUserValueMarked ) || !onlyUnmarked)
			{
				uItem = [[CKeyDBItem alloc] initWithItem:user withParent:kItem];
				[[kItem childArray] insertObject:uItem atIndex:userArrayIndex];
				[uItem release];
				PGPSetKeyDBObjUserValue(user, kUserValueMarked);
			}
			else
				uItem = [[kItem childArray] objectAtIndex:userArrayIndex];
			for( sigArrayIndex = 0; IsntPGPError( PGPKeyIterNextKeyDBObj( mKeyIter, kPGPKeyDBObjType_Signature, &sig ) ); sigArrayIndex++ )
			{
				PGPGetKeyDBObjUserValue( sig, &userVal );
				if( ( userVal != kUserValueMarked ) || !onlyUnmarked)
				{
					sItem = [[CKeyDBItem alloc] initWithItem:sig withParent:uItem];
					[[uItem childArray] insertObject:sItem atIndex:sigArrayIndex];
					[sItem release];
					PGPSetKeyDBObjUserValue(sig, kUserValueMarked);
				}
			}
		}
	}
	[mKeyOutline reloadData];
	return err;
}

- (void)reload
{
	[self syncWithKeyDB:YES];
}

- (NSImage *)getValidityImage:(CKeyDBItem *)item
{
	PGPInt32	validity;
	PGPBoolean	axiomatic;
	NSImage		*validityImage = nil;
	
	if( [item keyDBObjType] == kPGPKeyDBObjType_Key )
		PGPGetPrimaryUserIDValidity( [item keyDBItem], (PGPValidity *)&validity );
	else if( [item keyDBObjType] == kPGPKeyDBObjType_UserID )
		PGPGetKeyDBObjNumericProperty( [item keyDBItem], kPGPUserIDProperty_Validity, &validity );
	else
		return nil;
	PGPGetKeyDBObjBooleanProperty( PGPPeekKeyDBObjKey( [item keyDBItem] ), kPGPKeyProperty_IsAxiomatic, &axiomatic );
	if( axiomatic )
		validityImage = [[[CPGPIcons shared] images] objectAtIndex:kPGPclImage_DotAxiom];
	else switch( validity )
	{
		default:
		case kPGPValidity_Unknown:
		case kPGPValidity_Invalid:
		case kPGPValidity_Marginal:
			validityImage = [[[CPGPIcons shared] images] objectAtIndex:kPGPclImage_DotDimmed];
			break;
		case kPGPValidity_Complete:
			validityImage = [[[CPGPIcons shared] images] objectAtIndex:kPGPclImage_DotGreen];
			break;
	}
	return validityImage;
}

- (void)selectAll:(id)sender
{
	[mKeyOutline selectAll:self];
}

- (void)deselectAll:(id)sender
{
	[mKeyOutline deselectAll:self];
}

- (PGPKeyDBObjRef)selectionInfo:(BOOL *)multiple withFlags:(PGPUInt32 *)flags
{
	NSEnumerator	*rowEnumerator = [mKeyOutline selectedRowEnumerator];
	id				row;
	PGPKeyDBObjRef	firstItem = kInvalidPGPKeyDBObjRef,
					dbItem;
	
	*multiple = NO;
	*flags = kPGPclKeyList_None;
	while( ( row = [rowEnumerator nextObject] ) )
	{
		CKeyDBItem *item;
		
		item = [mKeyOutline itemAtRow:[row intValue]];
		dbItem = [item keyDBItem];
		if( !PGPKeyDBObjRefIsValid( firstItem ) )
			firstItem = dbItem;
		else
			*multiple = YES;
		switch( [item keyDBObjType] )
		{
			case kPGPKeyDBObjType_Key:
				*flags |= kPGPclKeyList_Key;
				break;
			case kPGPKeyDBObjType_UserID:
				*flags |= kPGPclKeyList_UserID;
				break;
			case kPGPKeyDBObjType_Signature:
				*flags |= kPGPclKeyList_Signature;
				break;
			default:
				break;
		}
	}
	return firstItem;
}

- (NSMutableArray *)selectedItemArray
{
	NSEnumerator	*rowEnumerator = [mKeyOutline selectedRowEnumerator];
	NSMutableArray	*selectedArray = [NSMutableArray array];
	NSNumber		*row;

	while( row = [rowEnumerator nextObject] )
		[selectedArray addObject:[mKeyOutline itemAtRow:[row intValue]]];
	return selectedArray;
}

- (NSEnumerator *)selectedItemEnumerator
{
	return [[self selectedItemArray] objectEnumerator];
}

- (void)selectItemsInArray:(NSArray *)array
{
	NSEnumerator	*enumerator;
	id				item;
	PGPInt32		index;
	
	if( !array )
		return;
	
	[self deselectAll:nil];
	
	enumerator = [array objectEnumerator];
	
	while( item = [enumerator nextObject] )
	{
		index = [mKeyArray indexOfObject:item];
		
		if (index != NSNotFound)
		{
			if ([mKeyOutline allowsMultipleSelection])
				[mKeyOutline selectRow:index byExtendingSelection:YES];
			else
				[mKeyOutline selectRow:index byExtendingSelection:NO];
		}
	}
}

- (PGPKeySetRef)getEnumeratorKeySet:(NSEnumerator *)enumerator
{
	PGPKeySetRef	selectedKeys = kInvalidPGPKeySetRef;
	CKeyDBItem		*item;
	
	while( item = [enumerator nextObject] )
	{
		if( [item keyDBObjType] == kPGPKeyDBObjType_Key )
		{
			if( !PGPKeySetRefIsValid( selectedKeys ) )
				PGPNewEmptyKeySet( mKeyDB, &selectedKeys );
			PGPAddKey( [item keyDBItem], selectedKeys );
		}
	}

	return selectedKeys;
}

- (void)exportSelection:(NSString *)filename privates:(BOOL)includePrivate extensions:(BOOL)newStyle hideSuffix:(BOOL)hideSuffix
{
	PGPError			err = kPGPError_NoErr;
	PGPFileSpecRef		fileSpec = kInvalidPGPFileSpecRef;
	PGPKeySetRef		keySet = kInvalidPGPKeySetRef;
	NSMutableString		*xfilename = [NSMutableString stringWithString:filename];
	CKeyDBItem			*item;
	PGPBoolean			x509,
						secret,
						split;
	char				*passphrase = NULL;
	PGPByte				*passKey = NULL;
	PGPSize				passKeySize;
	
	item = [mKeyOutline itemAtRow:[mKeyOutline selectedRow]];
	if( ( [mKeyOutline numberOfSelectedRows] == 1 ) &&
		( [item keyDBObjType] == kPGPKeyDBObjType_Signature ) &&
		IsntPGPError( PGPGetKeyDBObjBooleanProperty( [item keyDBItem], kPGPSigProperty_IsX509, &x509) ) && x509 )
	{
		PGPKeyDBObjRef	ownerKey = PGPPeekKeyDBObjKey( [item keyDBItem] );
		
		PGPGetKeyDBObjBooleanProperty( ownerKey, kPGPKeyProperty_IsSecret, &secret );
		if( !secret )
		{
			includePrivate = NO;
			if( ![xfilename hasSuffix:@".pem"] )
				[xfilename appendString:@".pem"];
		}
		else if( ![xfilename hasSuffix:@".p12"] )
			[xfilename appendString:@".p12"];
		err = PGPNewFileSpecFromFullPath( mPGPContext, [xfilename UTF8String], &fileSpec );	CKERR;
		if( includePrivate )
		{
			err = PGPclGetKeyPassphrase( ownerKey, [self tlsContext], mPrefRef, &split, &passphrase, &passKey, &passKeySize );	CKERR;
		}
		if( IsntPGPError( err ) )
			err = PGPExport( mPGPContext,
				PGPOExportKeyDBObj( mPGPContext, [item keyDBItem] ),
				PGPOExportFormat( mPGPContext, kPGPExportFormat_X509Cert ),
				PGPOExportPrivateKeys( mPGPContext, includePrivate ),
				PGPOOutputFile( mPGPContext, fileSpec ),
				PGPOArmorOutput( mPGPContext, !includePrivate ),
				includePrivate ?
					( split ?
						PGPOPasskeyBuffer( mPGPContext, passKey, passKeySize ) :
						PGPOPassphrase( mPGPContext, passphrase ) ) :
					PGPONullOption( mPGPContext ),
				PGPOLastOption( mPGPContext ) );
	}
	else
	{
		char		comment[256];
		PGPUInt32	keyCount;
	
		keySet = [self getEnumeratorKeySet:[self selectedItemEnumerator]];
		if( PGPKeySetRefIsValid( keySet ) && IsntPGPError( PGPCountKeys( keySet, &keyCount ) ) && ( keyCount > 0 ) )
		{
			comment[0] = 0;
			PGPGetPrefStringBuffer( mPrefRef, kPGPPrefComment, sizeof( comment ), comment );
			if( ![xfilename hasSuffix:@".asc"] )
				[xfilename appendString:@".asc"];
			err = PGPNewFileSpecFromFullPath( mPGPContext, [xfilename UTF8String], &fileSpec );	CKERR;
			err = PGPExport( mPGPContext,
					PGPOExportKeySet( mPGPContext, keySet ),
					PGPOOutputFile( mPGPContext, fileSpec ),
					PGPOExportPrivateKeys( mPGPContext, includePrivate ),
					PGPOVersionString( mPGPContext, pgpVersionHeaderString() ),
					PGPOCommentString( mPGPContext, comment ),
					newStyle ?
						PGPOExportFormat( mPGPContext, kPGPExportFormat_Complete ) :
						PGPOExportFormat( mPGPContext, kPGPExportFormat_Basic ),
					PGPOLastOption( mPGPContext ) );
		}
	}
	if( IsntPGPError( err ) )
	{
		[[NSFileManager defaultManager] changeFileAttributes:
			[NSDictionary dictionaryWithObject:[NSNumber numberWithBool:hideSuffix] forKey:NSFileExtensionHidden]
			atPath:xfilename];
	}
done:
	if( IsntNull( passphrase ) )
		PGPFreeData( passphrase );
	if( IsntNull( passKey ) )
		PGPFreeData( passKey );
	if( PGPKeySetRefIsValid( keySet ) )
		PGPFreeKeySet( keySet );
	if( PGPFileSpecRefIsValid( fileSpec ) )
		PGPFreeFileSpec( fileSpec );
}

- (void)pasteboard:(NSPasteboard *)sender provideDataForType:(NSString *)type
{
	PGPKeySetRef	sourceSet = kInvalidPGPKeySetRef;
	
	if( [[sender name] isEqual:NSGeneralPboard] )
		sourceSet = mCopyPBKeySet;
	else if( [[sender name] isEqual:NSDragPboard] )
		sourceSet = mDragPBKeySet;
	if( !PGPKeySetRefIsValid( sourceSet ) )
		return;
	if( [type isEqual:kPGPKeySetPBoardType] )
	{
		PGPKeySetPBoardStruct	ckvpb;
		
		ckvpb.context = mPGPContext;
		ckvpb.keySet  = sourceSet;
		[sender setData:[NSData dataWithBytes:&ckvpb length:sizeof(PGPKeySetPBoardStruct)]
			forType:kPGPKeySetPBoardType];
	}
	else if( [type isEqual:NSStringPboardType] || [type isEqual:NSFilenamesPboardType] )
	{
		PGPError		err = kPGPError_NoErr;
		void			*keyBuffer = NULL;
		UInt32			keyBufferLen;
		char			comment[256];
		PGPBoolean		compatible;
		NSMutableString	*filenameString;
		NSString		*finalString;
		PGPKeyIterRef	keyIter = kInvalidPGPKeyIterRef;
		PGPKeyDBObjRef	key;
		NSRange			emRange;
		
		comment[0] = 0;
		PGPGetPrefStringBuffer( mPrefRef, kPGPPrefComment, sizeof( comment ), comment );
		PGPGetPrefBoolean( mPrefRef, kPGPPrefExportKeysCompatible, &compatible );
		err = PGPExport( mPGPContext,
				PGPOExportKeySet( mPGPContext, sourceSet ),
				PGPOAllocatedOutputBuffer( mPGPContext, &keyBuffer, MAX_PGPUInt32, &keyBufferLen ),
				PGPOVersionString( mPGPContext, pgpVersionHeaderString() ),
				PGPOCommentString( mPGPContext, comment ),
				compatible ?
					PGPOExportFormat( mPGPContext, kPGPExportFormat_Basic ) :
					PGPOExportFormat( mPGPContext, kPGPExportFormat_Complete ),
				PGPOLastOption( mPGPContext ) ); 											CKERR;
		err = PGPNewKeyIterFromKeySet( sourceSet, &keyIter );								CKERR;
		err = PGPKeyIterNextKeyDBObj( keyIter, kPGPKeyDBObjType_Key, &key );				CKERR;
		filenameString = [NSMutableString stringWithString:PGPclGetPrimaryUserIDName( key )];
		emRange = [filenameString rangeOfString:@" <"];
		if( ( emRange.location != NSNotFound ) && ( emRange.location > 1 ) )
		{
			emRange.length = [filenameString length] - emRange.location;
			[filenameString deleteCharactersInRange:emRange];
		}
		[filenameString insertString:@"/tmp/" atIndex:0];
		[filenameString appendString:@".asc"];
		finalString = [NSString stringWithUTF8String:keyBuffer];
		if( [type isEqual:NSStringPboardType] )
			[sender setString:finalString forType:NSStringPboardType];
		else
		{
			if( [finalString writeToFile:filenameString atomically:NO] )
				[sender setPropertyList:[NSArray arrayWithObject:filenameString] forType:NSFilenamesPboardType];
		}
	done:
		if( PGPKeyIterRefIsValid( keyIter ) )
			PGPFreeKeyIter( keyIter );
		if( IsntNull( keyBuffer ) )
			PGPFreeData( keyBuffer );
	}
}

- (void)addKeySet:(PGPKeySetRef)keySet toPasteboard:(NSPasteboard *)pboard
{
	if( [[pboard name] isEqual:NSDragPboard] )
	{
		if( PGPKeySetRefIsValid( mDragPBKeySet ) )
			PGPFreeKeySet( mDragPBKeySet );
		mDragPBKeySet = keySet;
		[pboard declareTypes:[NSArray arrayWithObjects:
			kPGPKeySetPBoardType, NSStringPboardType, NSFilenamesPboardType, nil] owner:self];
	}
	else
	{
		if( PGPKeySetRefIsValid( mCopyPBKeySet ) )
			PGPFreeKeySet( mCopyPBKeySet );
		mCopyPBKeySet = keySet;
		[pboard declareTypes:[NSArray arrayWithObjects:
			NSFilenamesPboardType, NSStringPboardType, nil] owner:self];
		mCopyPBChangeCount = [pboard changeCount];
	}
}

- (PGPKeyDBRef)importFileList:(NSArray *)fileList
{
	PGPKeyDBRef		newKeyDB = kInvalidPGPKeyDBRef,
					tempKeyDB = kInvalidPGPKeyDBRef;
	NSEnumerator	*fileEnumerator = [fileList objectEnumerator];
	NSString		*filePath;
	PGPFileSpecRef	importFileSpec = kInvalidPGPFileSpecRef;
	PGPError		err = kPGPError_NoErr;
	
	while( filePath = [fileEnumerator nextObject] )
	{
		err = PGPNewFileSpecFromFullPath( mPGPContext, [filePath UTF8String], &importFileSpec );
		if( IsntPGPError( err ) )
		{
			if( [filePath hasSuffix:@"pfx"] || [filePath hasSuffix:@"p12"] || [filePath hasSuffix:@"pem"] )
			{
				char		*outputPassphrase = NULL;
				BOOL		isPEM = [filePath hasSuffix:@"pem"];
				
				if( !isPEM )
				{
					err = PGPPassphraseDialog( mPGPContext, 
							PGPOUIDialogPrompt( mPGPContext,
								[LSCL( @"Please enter the certificate passphrase:", @"P12 Import prompt" ) cString]),
							PGPOUIOutputPassphrase( mPGPContext, &outputPassphrase ),
							PGPOLastOption( mPGPContext ) );
				}
				if( IsntPGPError( err ) )
				{
					err = PGPImport( mPGPContext, &tempKeyDB,
							PGPOInputFormat( mPGPContext, isPEM ?
								kPGPInputFormat_PEMEncodedX509Cert : kPGPInputFormat_PKCS12 ),
							PGPOInputFile( mPGPContext, importFileSpec ),
							isPEM ?
								PGPONullOption( mPGPContext ) :
								PGPOPassphrase( mPGPContext, outputPassphrase ),
							PGPOLastOption( mPGPContext ) );
					if( IsntNull( outputPassphrase ) )
						PGPFreeData( outputPassphrase );
				}
			}
			else
			{
				err = PGPImport( mPGPContext, &tempKeyDB,
					PGPOInputFile( mPGPContext, importFileSpec ),
					PGPOLastOption( mPGPContext ) );
			}
		}
		if( IsntPGPError( err) && PGPKeyDBRefIsValid( tempKeyDB ) )
		{
			PGPUInt32	keyCount;
			
			PGPCountKeysInKeyDB( tempKeyDB, &keyCount );
			if( !keyCount )
				PGPFreeKeyDB( tempKeyDB );
			else if( !PGPKeyDBRefIsValid( newKeyDB ) )
				newKeyDB = tempKeyDB;
			else
			{
				PGPCopyKeys( PGPPeekKeyDBRootKeySet( tempKeyDB ), newKeyDB, NULL );
				PGPFreeKeyDB( tempKeyDB );
			}
			tempKeyDB = kInvalidPGPKeyDBRef;
		}
		else
			PGPclErrorBox( err );
		if( PGPFileSpecRefIsValid( importFileSpec ) )
		{
			PGPFreeFileSpec( importFileSpec );
			importFileSpec = kInvalidPGPFileSpecRef;
		}
	}
	return newKeyDB;
}

- (PGPKeyDBRef)importString:(NSString *)importString
{
	PGPKeyDBRef		newKeyDB = kInvalidPGPKeyDBRef;
	PGPBoolean		isPGP;
	PGPError		err = kPGPError_NoErr;
	
	isPGP = ( [importString rangeOfString:@"-----BEGIN PGP"].location != NSNotFound );
	err = PGPImport( mPGPContext, &newKeyDB,
		PGPOInputBuffer( mPGPContext, [importString UTF8String], [importString cStringLength] ),
		PGPOInputFormat( mPGPContext,
			isPGP ? kPGPInputFormat_PGP :
					kPGPInputFormat_PEMEncodedX509Cert ),
		PGPOLastOption( mPGPContext ) );
	return newKeyDB;
}

- (PGPKeyDBRef)importPasteboard:(NSPasteboard *)pboard done:(BOOL *)done
{
	NSEnumerator	*typeEnumerator = [[pboard types] objectEnumerator];
	NSString		*curType;
	PGPKeyDBRef		newKeyDB = kInvalidPGPKeyDBRef;
	
	*done = NO;
	while( curType = [typeEnumerator nextObject] )
	{
		if( [curType isEqual:kPGPKeySetPBoardType] )
		{
			PGPKeySetPBoardStruct	ckvpb;
			NSData					*keySetData;
			PGPError				err = kPGPError_NoErr;
			
			keySetData = [pboard dataForType:kPGPKeySetPBoardType];
			if( keySetData )
				[keySetData getBytes:&ckvpb length:sizeof(PGPKeySetPBoardStruct)];
			if( keySetData && ( mPGPContext == ckvpb.context ) && PGPKeySetRefIsValid( ckvpb.keySet ) )
			{
				err = PGPCopyKeys( ckvpb.keySet, mKeyDB, NULL );
				if( IsntPGPError( err ) )
				{
					[self syncWithKeyDB:YES];
					[CKeyInfo updateAllForDB:mKeyDB];
					*done = YES;
					break;
				}
			}
		}
		else if( [curType isEqual:NSStringPboardType] )
		{
			newKeyDB = [self importString:[pboard stringForType:NSStringPboardType]];
			break;
		}
		else if( [curType isEqual:NSFilenamesPboardType] )
		{
			newKeyDB = [self importFileList:[pboard propertyListForType:NSFilenamesPboardType]];
			break;
		}
	}
	return newKeyDB;
}

- (void)importKeyDB:(PGPKeyDBRef)newKeyDB confirm:(BOOL)confirm
{
	NSWindow		*window = [[NSApplication sharedApplication] keyWindow];
	PGPKeyDBRef		mainDB = [mController mainDB];
	
	if( [mCKeyView window] && [[mCKeyView window] isVisible] )
		window = [mCKeyView window];
	if( PGPKeyDBRefIsValid( newKeyDB ) )
	{
		mInModal = YES;
		[mController selectionChanged];
		mImportDB = newKeyDB;
		PGPclBeginKeySelectSheet( LSCL( @"Please select the keys you wish to import:", @"SelectKeysPrompt" ),
			window, mPrefRef, PGPPeekKeyDBRootKeySet(newKeyDB),
			PGPKeyDBRefIsValid( mainDB ) ? mainDB : mKeyDB, kPGPclKeySelect_ImportVariation,
			self, @selector(importSelectDidEnd:selectedKeys:) );
	}
	else if( confirm )
	{
		NSBeginInformationalAlertSheet(
			LSCL( @"No PGP Data", @"Title of Import Failed informational" ),
			kCKeyViewOKButton, nil, nil, window, self, NULL, NULL, NULL,
			LSCL( @"Nothing was found to import in the items provided.", @"Import Failed text" ) );
	}
}

- (void)keyInfoClosed:(id)sender withChanges:(BOOL)changed
{
	if( mInModal )
		[NSApp abortModal];
	if( changed )
		[mKeyOutline setNeedsDisplay:YES];
}

- (void)showInfo
{
	NSEnumerator	*rowEnumerator = [mKeyOutline selectedRowEnumerator];
	NSNumber		*row;
	CKeyDBItem		*item;
	PGPKeyDBObjRef	dbItem = kInvalidPGPKeyDBObjRef;
	PGPKeyDBRef		mainDB = [mController mainDB];

	if( [[NSApplication sharedApplication] modalWindow] )
		return;
	while( row = [rowEnumerator nextObject] )
	{
		dbItem = kInvalidPGPKeyDBObjRef;
		item = [mKeyOutline itemAtRow:[row intValue]];
		switch( [item keyDBObjType] )
		{
			case kPGPKeyDBObjType_UserID:
				dbItem = PGPPeekKeyDBObjKey( [item keyDBItem] );
			case kPGPKeyDBObjType_Key:
			{
				CKeyInfo *keyInfo;
				
				if( !PGPKeyDBObjRefIsValid( dbItem ) )
					dbItem = [item keyDBItem];
				keyInfo = [CKeyInfo showKeyInfo:self forKey:dbItem
							mainDB:mainDB
							lock:(mFlags & kPGPclKeyList_ReadOnly)];
				if( keyInfo && ( mFlags & kPGPclKeyList_ReadOnly ) && ( mainDB != mKeyDB ) )
				{
					NSModalSession	session;
					
					mInModal = YES;
					session = [NSApp beginModalSessionForWindow:[keyInfo window]];
					for (;;)
					{
						if ([NSApp runModalSession:session] != NSRunContinuesResponse)
							break;
					}
					[NSApp endModalSession:session];
					mInModal = NO;
				}
				break;
			}
			case kPGPKeyDBObjType_Signature:
			{
				CCertInfo *certInfo;
				
				dbItem = [item keyDBItem];
				certInfo = [CCertInfo showCertInfo:self forCert:dbItem];
				if( certInfo && ( mFlags & kPGPclKeyList_ReadOnly ) && ( mainDB != mKeyDB ) )
				{
					NSModalSession	session;
					
					mInModal = YES;
					session = [NSApp beginModalSessionForWindow:[certInfo window]];
					for (;;)
					{
						if ([NSApp runModalSession:session] != NSRunContinuesResponse)
							break;
					}
					[NSApp endModalSession:session];
					mInModal = NO;
				}
				break;
			}
			default:
				break;
		}
	}
}

- (PGPclKeyListColumnInfo)columns
{
	PGPclKeyListColumnInfo	columnInfo;
	NSEnumerator			*colEnum = [[mKeyOutline tableColumns] objectEnumerator];
	NSTableColumn			*column;
	
	columnInfo.numActiveColumns = 0;
	while( column = [colEnum nextObject] )
	{
		int	columnID = [[column identifier] intValue];

		columnInfo.columns[columnInfo.numActiveColumns] = columnID;
		columnInfo.columnWidths[columnID] = [column width];
		columnInfo.numActiveColumns++;
	}
	return columnInfo;
}

- (void)setColumns:(PGPclKeyListColumnInfo)columns
{
	NSEnumerator		*columnEnumerator = [[mKeyOutline tableColumns] objectEnumerator];
	NSMutableArray		*deadColumns = [NSMutableArray array];
	NSTableColumn		*column;
	PGPUInt16			columnIndex;
	CBrowserCell		*bCell;
	NSImageCell			*iCell;
	NSTextFieldCell		*tCell;
	NSSliderCell		*sliderCell;
	CImageButtonCell	*imbCell;
	NSFont				*font = [NSFont systemFontOfSize:[NSFont smallSystemFontSize]];
	
	[mKeyOutline setAutoresizesOutlineColumn:YES];
	[mKeyOutline setIntercellSpacing:NSMakeSize(7.0,2.0)];
	if( mFlags & kPGPclKeyList_SingleSelect )
		[mKeyOutline setAllowsMultipleSelection:NO];
	else
		[mKeyOutline setAllowsMultipleSelection:YES];
	[mKeyOutline unregisterDraggedTypes];
    [mKeyOutline setTarget:self];
    [mKeyOutline setDoubleAction:@selector(doubleClickCell:)];
	if( mFlags & kPGPclKeyList_EnableDropIn )
		[mKeyOutline registerForDraggedTypes:[NSArray arrayWithObjects:NSFilenamesPboardType, NSFileContentsPboardType, NSStringPboardType, nil]];
	for( columnIndex = 0; columnIndex < columns.numActiveColumns; columnIndex++ )
	{
		if( ![mColumns[columns.columns[columnIndex]] tableView] )
			[mKeyOutline addTableColumn:mColumns[columns.columns[columnIndex]]];
	}
	while( column = [columnEnumerator nextObject] )
	{
		BOOL	found = NO;
		
		for( columnIndex = 0; columnIndex < columns.numActiveColumns; columnIndex++ )
		{
			if( ( [[column identifier] intValue] == kPGPclKeyList_UserIDColumn ) ||
				( columns.columns[columnIndex] == [[column identifier] intValue] ) )
			{
				found = YES;
				break;
			}
		}
		if( !found )
			[deadColumns addObject:column];
	}
	columnEnumerator = [deadColumns objectEnumerator];
	while( column = [columnEnumerator nextObject] )
	{
		[mKeyOutline removeTableColumn:column];
		[column setTableView:nil];
	}
	columnEnumerator = [[mKeyOutline tableColumns] objectEnumerator];
	while( column = [columnEnumerator nextObject] )
	{
		columnIndex = [[column identifier] intValue];
		switch( columnIndex )
		{
			case kPGPclKeyList_UserIDColumn:
				bCell = [[CBrowserCell alloc] init];
				[bCell setLeaf:YES];
				[bCell setWraps:NO];
				[bCell setScrollable:NO];
				[column setDataCell:bCell];
				[bCell release];
				[mKeyOutline setHighlightedTableColumn:column];
				[column setMinWidth:80];
				break;
			case kPGPclKeyList_ValidityColumn:
				if( mFlags & kPGPclKeyList_ValidityButton )
				{
					imbCell = [[CImageButtonCell alloc] init];
					[imbCell setButtonType:NSPushOnPushOffButton];
					[imbCell setBezelStyle:NSShadowlessSquareBezelStyle];
					[imbCell setGradientType:NSGradientNone];
					[imbCell setBordered:NO];
					[imbCell setHighlightsBy:NSContentsCellMask];
					[imbCell setAlternateImage:[[[CPGPIcons shared] images] objectAtIndex:kPGPclImage_DotSelect]];
					[imbCell setAction:@selector(validityButtonPressed:)];
					[imbCell setTarget:self];
					[column setDataCell:imbCell];
					[imbCell release];
				}
				else
				{
					iCell = [[NSImageCell alloc] init];
					[column setDataCell:iCell];
					[iCell release];
				}
				break;
			case kPGPclKeyList_TrustColumn:
				sliderCell = [[NSSliderCell alloc] init];
				[sliderCell setControlSize:NSSmallControlSize];
				[sliderCell setMinValue:0];
				[sliderCell setMaxValue:2];
				[sliderCell setNumberOfTickMarks:3];
				[sliderCell setAllowsTickMarkValuesOnly:YES];
				[column setDataCell:sliderCell];
				[sliderCell release];
				break;
			case kPGPclKeyList_SizeColumn:
				tCell = [[NSTextFieldCell alloc] init];
				[tCell setFont:font];
				[tCell setWraps:NO];
				[column setDataCell:tCell];
				[tCell release];
				break;
			case kPGPclKeyList_CreationColumn:
				tCell = [[NSTextFieldCell alloc] init];
				[tCell setFont:font];
				[tCell setWraps:NO];
				[column setDataCell:tCell];
				[tCell release];
				break;
			case kPGPclKeyList_ExpirationColumn:
				tCell = [[NSTextFieldCell alloc] init];
				[tCell setFont:font];
				[tCell setWraps:NO];
				[column setDataCell:tCell];
				[tCell release];
				break;
			case kPGPclKeyList_KeyIDColumn:
				tCell = [[NSTextFieldCell alloc] init];
				[tCell setFont:font];
				[tCell setWraps:NO];
				[column setDataCell:tCell];
				[tCell release];
				break;
			case kPGPclKeyList_ADKColumn:
				iCell = [[NSImageCell alloc] init];
				[column setDataCell:iCell];
				[iCell release];
				break;
			case kPGPclKeyList_DescriptionColumn:
				tCell = [[NSTextFieldCell alloc] init];
				[tCell setFont:font];
				[tCell setWraps:NO];
				[column setDataCell:tCell];
				[tCell release];
				break;
		}
	}
}

- (void)deleteSelection
{
	NSAutoreleasePool	*pool = [[NSAutoreleasePool alloc] init];
	NSEnumerator		*rowEnumerator = [mKeyOutline selectedRowEnumerator];
	NSNumber			*row;
	CKeyDBItem			*item,
						*lastDeletedKey = nil,
						*lastDeletedUserID = nil;
	NSMutableArray		*trashArray = [NSMutableArray array];
	
	[CKeyInfo closeAll:self forDB:mKeyDB];
	[CCertInfo closeAll:self forDB:mKeyDB];
	while( row = [rowEnumerator nextObject] )
	{
		item = [mKeyOutline itemAtRow:[row intValue]];
		switch( [item keyDBObjType] )
		{
			case kPGPKeyDBObjType_Key:
				if( IsntPGPError( PGPDeleteKeyDBObj( [item keyDBItem] ) ) )
				{
					if( [mDefaultKey isEqual:item] )
						mDefaultKey = nil;
					[trashArray addObject:item];
					lastDeletedKey = item;
				}
				break;
			case kPGPKeyDBObjType_UserID:
				if( ![lastDeletedKey isEqual:[item parent]] )
				{
					PGPKeyIterRef	keyIter = kInvalidPGPKeyIterRef;
					PGPKeyDBObjRef	userID = kInvalidPGPKeyDBObjRef;
					PGPUInt32		numUserIDs = 0;
					
					// Make sure we don't delete the last user ID
					if( IsntPGPError( PGPNewKeyIterFromKeyDB( mKeyDB, &keyIter ) ) )
					{
						PGPKeyIterSeek( keyIter, [[item parent] keyDBItem] );
						while( IsntPGPError( PGPKeyIterNextKeyDBObj( keyIter, kPGPKeyDBObjType_UserID, &userID ) ) )
							numUserIDs++;
						PGPFreeKeyIter( keyIter );
					}
					if( ( numUserIDs > 1 ) && IsntPGPError( PGPDeleteKeyDBObj( [item keyDBItem] ) ) )
					{
						[self unmarkKey:[[item parent] keyDBItem]];
						[trashArray addObject:[item parent]];
						lastDeletedUserID = item;
					}
				}
				break;
			case kPGPKeyDBObjType_Signature:
				if( ![lastDeletedKey isEqual:[[item parent] parent]] &&
					![lastDeletedUserID isEqual:[item parent]] &&
					IsntPGPError( PGPDeleteKeyDBObj( [item keyDBItem] ) ) )
					[trashArray addObject:item];
				break;
			default:
				break;
		}
	}
	if( [trashArray count] )
	{
		NSEnumerator	*trashEnumerator = [trashArray objectEnumerator];
		
		while( item = [trashEnumerator nextObject] )
		{
			unsigned	trashIndex;
			
			if( [item keyDBObjType] == kPGPKeyDBObjType_Signature )
			{
				trashIndex = [[[item parent] childArray] indexOfObject:item];
				if( trashIndex != NSNotFound )
					[[[item parent] childArray] removeObjectAtIndex:trashIndex];
			}
			else
			{
				trashIndex = [mKeyArray indexOfObject:item];
				if( trashIndex != NSNotFound )
					[mKeyArray removeObjectAtIndex:trashIndex];
			}
		}
	}
	[mKeyOutline deselectAll:self];
	[self syncWithKeyDB:YES];
	[pool release];
}

- (void)didEndDeleteConfirmSheet:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo
{
    if( returnCode == NSAlertDefaultReturn )
		[self deleteSelection];
}

- (void)deleteConfirm
{
	NSBeginAlertSheet(
		LSCL( @"Delete Selection", @"Title of KeyView delete confirmation" ),
		kCKeyViewOKButton, LSCL( @"Cancel", @"Cancel button" ), nil, [mKeyOutline window], 
		self, NULL, @selector(didEndDeleteConfirmSheet:returnCode:contextInfo:), NULL,
		LSCL( @"Are you sure you want to delete the selected items?", @"Confirm KeyView deletion" ) );
}

- (void)enableSelection:(BOOL)enabled
{
	NSEnumerator	*rowEnumerator = [mKeyOutline selectedRowEnumerator];
	NSNumber		*row;
	CKeyDBItem		*item;
	
	while( row = [rowEnumerator nextObject] )
	{
		item = [mKeyOutline itemAtRow:[row intValue]];
		if( [item keyDBObjType] == kPGPKeyDBObjType_Key )
		{
			PGPSetKeyEnabled( [item keyDBItem], enabled );
			[mKeyOutline deselectRow:[row intValue]];
			[mKeyOutline selectRow:[row intValue] byExtendingSelection:YES];
		}
	}
}

- (void)setDefault
{
	int				row,
					selectedRow = [mKeyOutline selectedRow];
	CKeyDBItem		*selectedItem = [mKeyOutline itemAtRow:selectedRow];
	PGPKeyDBObjRef	keyDBItem = [selectedItem keyDBItem];
	PGPBoolean		secret;
	PGPError		err = kPGPError_NoErr;
	char			*passphrase  = NULL;
	PGPByte			*passKey = NULL;
	PGPSize			passKeySize;
	PGPBoolean		split;
	
	switch( [selectedItem keyDBObjType] )
	{
		case kPGPKeyDBObjType_Key:
			if( mDefaultKey )
			{
				[mDefaultKey setDefault:NO];
				if( ( ( row = [mKeyOutline rowForItem:mDefaultKey] ) >= 0 ) &&
					( row != selectedRow ) )
				{
					[mKeyOutline selectRow:row byExtendingSelection:NO];
					[mKeyOutline deselectRow:row];
				}
				mDefaultKey = nil;
			}
			PGPGetKeyDBObjBooleanProperty( keyDBItem, kPGPKeyProperty_IsSecret, &secret );
			if( ( selectedRow >= 0 ) && secret )
			{
				PGPKeyID		keyID;
				
				mDefaultKey = selectedItem;
				[mDefaultKey setDefault:YES];
				[mKeyOutline deselectRow:selectedRow];
				[mKeyOutline selectRow:selectedRow byExtendingSelection:NO];
				PGPGetKeyID( keyDBItem, &keyID );
				PGPSetPrefData( mPrefRef, kPGPPrefDefaultKeyID, sizeof(PGPKeyID), &keyID );
				PGPclWritePrefs( mPrefRef );
			}
			break;
		case kPGPKeyDBObjType_UserID:
		{
			PGPKeyDBObjRef	ownerKey = PGPPeekKeyDBObjKey( keyDBItem );
			
			PGPGetKeyDBObjBooleanProperty( ownerKey, kPGPKeyProperty_IsSecret, &secret );
			if( !secret )
				err = PGPSetPrimaryUserID( keyDBItem, PGPOLastOption( mPGPContext ) );
			else
			{
				err = PGPclGetKeyPassphrase( ownerKey, [self tlsContext], mPrefRef,
						&split, &passphrase, &passKey, &passKeySize );	CKERR;
				if( IsntPGPError( err ) )
				{
					err = PGPSetPrimaryUserID( keyDBItem, split ?
							PGPOPasskeyBuffer( mPGPContext, passKey, passKeySize ) :
							PGPOPassphrase( mPGPContext, passphrase ),
							PGPOLastOption( mPGPContext ) );
				}
			}
			if( IsntPGPError( err ) )
			{
				PGPKeyDBObjRef	primUserID;
				
				err = PGPGetPrimaryUserID( ownerKey, &primUserID );
				if( IsntPGPError( err ) )
				{
					if( primUserID != keyDBItem )
					{
						NSBeginInformationalAlertSheet(
							LSCL( @"Default User ID Locked", @"Can't set primary informational" ),
							kCKeyViewOKButton, nil, nil, [mKeyOutline window], self, NULL, NULL, NULL,
							LSCL( @"The primary user ID of this key has been locked by the owner.",
							@"Can't set primary msg" ) );
					}
					else
					{
						[mKeyOutline deselectAll:self];
						[self unmarkKey:ownerKey];
						[mKeyArray removeObjectAtIndex:[mKeyArray indexOfObject:[selectedItem parent]]];
						[self syncWithKeyDB:YES];
					}
				}
			}
			break;
		}
		default:
			break;
	}
done:
	if( passphrase )
		PGPFreeData( passphrase );
	if( passKey )
		PGPFreeData( passKey );
}

- (void)validityButtonPressed:(id)sender
{
	[mController validityButtonPressed:self];
}

- (void)doubleClickCell:(id)sender
{
	[self showInfo];
}

- (id)outlineView:(NSOutlineView *)outlineView child:(int)index ofItem:(id)item
{
    id		keyDBItem = nil;
	
	if( mKeyArray )
	{
		if( !item )
			keyDBItem = [mKeyArray objectAtIndex:index];
		else
			keyDBItem = [[item childArray] objectAtIndex:index];
	}
    return keyDBItem;
}

- (BOOL)outlineView:(NSOutlineView *)outlineView isItemExpandable:(id)item
{
	return ( [[item childArray] count] > 0 );
}

- (int)outlineView:(NSOutlineView *)outlineView numberOfChildrenOfItem:(id)item
{
    int			retValue = 0;
	
    if( mKeyArray )
	{
		if( !item )
			retValue = [mKeyArray count];
		else
			retValue = [[item childArray] count];
	}
    return retValue;
}

- (id)outlineView:(NSOutlineView *)outlineView objectValueForTableColumn:(NSTableColumn *)tableColumn byItem:(id)item
{
    id					retValue = nil;
	PGPKeyDBObjRef		keyItem;
	PGPKeyDBObjType		itemType;
	PGPTime				pgpTime;
	char				cstr[kPGPcl_MaxPGPUserIDLength];
    
	cstr[0] = '\0';
	if( item )
	{
		keyItem = [item keyDBItem];
		itemType = [item keyDBObjType];
		switch( [[tableColumn identifier] intValue] )
		{
            case kPGPclKeyList_UserIDColumn:
				break;
			case kPGPclKeyList_ValidityColumn:
				if( !(mFlags & kPGPclKeyList_ValidityButton) )
					retValue = [self getValidityImage:item];
				break;
			case kPGPclKeyList_TrustColumn:
			{
				PGPInt32	trust = 0;
				
				if( ( [item keyDBObjType] == kPGPKeyDBObjType_Key ) &&
					IsntPGPError( PGPGetKeyDBObjNumericProperty( [item keyDBItem], kPGPKeyProperty_Trust, &trust ) ) )
				{
					switch( trust )
					{
						default:
						case kPGPKeyTrust_Undefined:
						case kPGPKeyTrust_Unknown:
						case kPGPKeyTrust_Never:
							trust = 0;
							break;
						case kPGPKeyTrust_Marginal:
							trust = 1;
							break;
						case kPGPKeyTrust_Complete:
						case kPGPKeyTrust_Ultimate:
							trust = 2;
							break;
					}
				}
				retValue = [NSNumber numberWithInt:trust];
				break;
			}
			case kPGPclKeyList_SizeColumn:
				if( itemType == kPGPKeyDBObjType_Key )
				{
					PGPInt32			bits,
										subBits;
					PGPKeyDBObjRef		subKey;
					
					PGPGetKeyDBObjNumericProperty( keyItem, kPGPKeyProperty_Bits, &bits );
					PGPKeyIterSeek( mKeyIter, keyItem );
					if( IsntPGPError( PGPKeyIterNextKeyDBObj( mKeyIter, kPGPKeyDBObjType_SubKey, &subKey ) ) )
					{
						PGPGetKeyDBObjNumericProperty( subKey, kPGPSubKeyProperty_Bits, &subBits );
						retValue = [NSString stringWithFormat:@"%d/%d", subBits, bits];
					}
					else
						retValue = [[NSNumber numberWithInt:bits] stringValue];
				}
				break;
			case kPGPclKeyList_CreationColumn:
			{
				NSUserDefaults	*defaults		= [NSUserDefaults standardUserDefaults];
				
				switch( itemType )
				{
					case kPGPKeyDBObjType_Key:
						PGPGetKeyDBObjTimeProperty( keyItem, kPGPKeyProperty_Creation, &pgpTime );
						retValue = [[NSCalendarDate dateWithTimeIntervalSince1970:PGPGetStdTimeFromPGPTime( pgpTime )]
									descriptionWithCalendarFormat:[defaults stringForKey:NSShortDateFormatString]
									locale:[defaults dictionaryRepresentation]];
						break;
					case kPGPKeyDBObjType_UserID:
						break;
					case kPGPKeyDBObjType_Signature:
						PGPGetKeyDBObjTimeProperty( keyItem, kPGPSigProperty_Creation, &pgpTime );
						retValue = [[NSCalendarDate dateWithTimeIntervalSince1970:PGPGetStdTimeFromPGPTime( pgpTime )]
									descriptionWithCalendarFormat:[defaults stringForKey:NSShortDateFormatString]
									locale:[defaults dictionaryRepresentation]];
						break;
					default:
						break;
				}
				break;
			}
			case kPGPclKeyList_ExpirationColumn:
			{
				NSUserDefaults	*defaults		= [NSUserDefaults standardUserDefaults];
				
				switch( itemType )
				{
					case kPGPKeyDBObjType_Key:
						PGPGetKeyDBObjTimeProperty( keyItem, kPGPKeyProperty_Expiration, &pgpTime );
						if( pgpTime )
						{
							retValue = [[NSCalendarDate dateWithTimeIntervalSince1970:PGPGetStdTimeFromPGPTime( pgpTime )]
										descriptionWithCalendarFormat:[defaults stringForKey:NSShortDateFormatString]
										locale:[defaults dictionaryRepresentation]];
						}
						break;
					case kPGPKeyDBObjType_UserID:
						break;
					case kPGPKeyDBObjType_Signature:
						PGPGetKeyDBObjTimeProperty( keyItem, kPGPSigProperty_Expiration, &pgpTime );
						if( pgpTime )
						{
							retValue = [[NSCalendarDate dateWithTimeIntervalSince1970:PGPGetStdTimeFromPGPTime( pgpTime )]
										descriptionWithCalendarFormat:[defaults stringForKey:NSShortDateFormatString]
										locale:[defaults dictionaryRepresentation]];
						}
						break;
					default:
						break;
				}
				break;
			}
			case kPGPclKeyList_KeyIDColumn:
			{
				PGPKeyID	keyID;
				PGPSize		len;
				
				if( itemType == kPGPKeyDBObjType_Key )
				{
					PGPGetKeyID( keyItem, &keyID );
					PGPGetKeyIDString( &keyID, kPGPKeyIDString_Abbreviated, cstr );
					retValue = [NSString stringWithUTF8String:cstr];
				}
				else if( itemType == kPGPKeyDBObjType_Signature )
				{
					if( IsntPGPError( PGPGetKeyDBObjDataProperty( keyItem, kPGPSigProperty_KeyID, &keyID, sizeof(PGPKeyID), &len ) ) )
					{
						PGPGetKeyIDString( &keyID, kPGPKeyIDString_Abbreviated, cstr );
						retValue = [NSString stringWithUTF8String:cstr];
					}
				}
				break;
			}
			case kPGPclKeyList_ADKColumn:
				if( itemType == kPGPKeyDBObjType_Key )
				{
					PGPUInt32	numADKs = 0;
					
					PGPCountAdditionalRecipientRequests( keyItem, &numADKs );
					if( numADKs > 0 )
						retValue = [[[CPGPIcons shared] images] objectAtIndex:kPGPclImage_DotRed];
				}
				break;
			case kPGPclKeyList_DescriptionColumn:
				switch( itemType )
				{
					case kPGPKeyDBObjType_Key:
					{
						PGPBoolean	secret,
									shared = FALSE;
						PGPInt32	version;
						PGPInt32	algorithm;
			
						PGPGetKeyDBObjBooleanProperty( keyItem, kPGPKeyProperty_IsSecret, &secret );
						PGPGetKeyDBObjNumericProperty( keyItem, kPGPKeyProperty_AlgorithmID, &algorithm );
						PGPGetKeyDBObjBooleanProperty( keyItem, kPGPKeyProperty_IsSecretShared, &shared );
						PGPGetKeyDBObjNumericProperty( keyItem, kPGPKeyProperty_Version, &version );
						if( shared )
							retValue = LSCL(@"Split key pair", @"");
						else if( secret )
						{
							if( algorithm == kPGPPublicKeyAlgorithm_DSA )
								retValue = LSCL(@"DH/DSS key pair", @"");
							else
							{
								if( version == kPGPKeyVersion_V4 )
									retValue = LSCL(@"RSA key pair", @"");
								else
									retValue = LSCL(@"RSA legacy key pair", @"");
							}
						}
						else
						{
							if( algorithm == kPGPPublicKeyAlgorithm_DSA )
								retValue = LSCL(@"DH/DSS public key", @"");
							else
							{
								if( version == kPGPKeyVersion_V4 )
									retValue = LSCL(@"RSA public key", @"");
								else
									retValue = LSCL(@"RSA legacy public key", @"");
							}
						}
						break;
					}
					case kPGPKeyDBObjType_UserID:
						retValue = LSCL(@"User ID", @"");
						break;
					case kPGPKeyDBObjType_Signature:
					{
						PGPInt32	algorithm;
						PGPBoolean	x509;
						
						PGPGetKeyDBObjNumericProperty( keyItem, kPGPSigProperty_AlgorithmID, &algorithm );
						PGPGetKeyDBObjBooleanProperty( keyItem, kPGPSigProperty_IsX509, &x509 );
						if( x509 )
						{
							if( algorithm == kPGPPublicKeyAlgorithm_DSA )
								retValue = LSCL(@"DSS X.509 certificate", @"");
							else if( algorithm == kPGPPublicKeyAlgorithm_RSA )
								retValue = LSCL(@"RSA X.509 certificate", @"");
							else
								retValue = LSCL(@"X.509 certificate", @"");
						}
						else
						{
							if( algorithm == kPGPPublicKeyAlgorithm_DSA )
								retValue = LSCL(@"DSS signature", @"");
							else if( algorithm == kPGPPublicKeyAlgorithm_RSA )
								retValue = LSCL(@"RSA signature", @"");
							else
								retValue = LSCL(@"Signature", @"");
						}
						break;
					}
					default:
						break;
				}
				break;
		}
	}

	return retValue;
}

- (void)outlineView:(NSOutlineView *)outlineView setObjectValue:(id)object forTableColumn:(NSTableColumn *)tableColumn byItem:(id)item
{
	switch( [[tableColumn identifier] intValue] )
	{
		case kPGPclKeyList_TrustColumn:
			if( [item keyDBObjType] == kPGPKeyDBObjType_Key )
			{
				PGPInt32	trust;
				PGPBoolean	axiomatic;

				switch( [object intValue] )
				{
					default:
					case 0:
						trust = kPGPKeyTrust_Never;
						break;
					case 1:
						trust = kPGPKeyTrust_Marginal;
						break;
					case 2:
						trust = kPGPKeyTrust_Complete;
						break;
				}
				PGPGetKeyDBObjBooleanProperty( [item keyDBItem], kPGPKeyProperty_IsAxiomatic, &axiomatic );
				if( !axiomatic)
				{
					PGPSetKeyTrust( [item keyDBItem], trust );
					[mKeyOutline setNeedsDisplay:YES];
				}
			}
			break;
	}
}

- (void)outlineView:(NSOutlineView *)outlineView willDisplayCell:(id)cell forTableColumn:(NSTableColumn *)tableColumn item:(id)item
{
	char						cstr[kPGPcl_MaxPGPUserIDLength];
	PGPSize						len;
	NSMutableAttributedString	*attributedString = nil;
	NSFont						*font = nil;
	NSRange						range = { 0, 0 };
	NSString*					userName = nil;
	
	switch( [[tableColumn identifier] intValue] )
	{
		case kPGPclKeyList_UserIDColumn:
		{
			PGPBoolean	disabled;
			PGPInt32	attributeType;
			PGPInt32	indentLevel = 0;
			
			[cell setImage:[[CPGPIcons shared] getKeyDBObjIcon:[item keyDBItem]]];
			attributedString = [[NSMutableAttributedString alloc] autorelease];
			switch( [item keyDBObjType] )
			{
				case kPGPKeyDBObjType_Key:
					PGPGetKeyDBObjBooleanProperty( [item keyDBItem], kPGPKeyProperty_IsDisabled, &disabled );
					userName = PGPclGetPrimaryUserIDName([item keyDBItem]);
					if (userName)
						[attributedString initWithString:userName];
					range.length = [attributedString length];
					if( disabled )
						[attributedString addAttribute:NSForegroundColorAttributeName value:[NSColor grayColor] range:range];
					if( [item isDefault] )
						font = [[NSFont boldSystemFontOfSize:[NSFont smallSystemFontSize]] retain];
					break;
				case kPGPKeyDBObjType_UserID:
				{
					PGPBoolean	isAttributeID;
					
					indentLevel = 1;
					PGPGetKeyDBObjBooleanProperty( [item keyDBItem], kPGPUserIDProperty_IsAttribute, &isAttributeID );
					if( isAttributeID )
					{
						PGPGetKeyDBObjNumericProperty( [item keyDBItem], kPGPUserIDProperty_AttributeType, &attributeType );
						if( attributeType == kPGPAttribute_Image )
							[attributedString initWithString:LSCL(@"Photograph", @"")];
						else
							[attributedString initWithString:LSCL(@"Unknown", @"")];
					}
					else
					{
						userName = PGPclGetUserIDName([item keyDBItem]);
						if (userName)
							[attributedString initWithString:userName];
					}
					break;
				}
				case kPGPKeyDBObjType_Signature:
				{
					PGPKeyDBObjRef	certKey;
					PGPBoolean		x509;
					PGPKeyDBRef		mainDB = [mController mainDB];
					
					indentLevel = 2;
					PGPGetKeyDBObjBooleanProperty( [item keyDBItem], kPGPSigProperty_IsX509, &x509 );
					PGPGetSigCertifierKey( [item keyDBItem], mKeyDB, &certKey );
					if( !PGPKeyDBObjRefIsValid( certKey ) && ( mKeyDB != mainDB ) &&
						PGPKeyDBRefIsValid( mainDB ) )
						PGPGetSigCertifierKey( [item keyDBItem], mainDB, &certKey );
					if( PGPKeyDBObjRefIsValid( certKey ) )
					{
						userName = PGPclGetPrimaryUserIDName(certKey);
						if (userName)
							[attributedString initWithString:userName];
					}
					else if( x509 )
					{
						PGPGetKeyDBObjDataProperty( [item keyDBItem], kPGPSigProperty_X509IssuerLongName,
							cstr, kPGPcl_MaxPGPUserIDLength - 1, &len );
						[attributedString initWithString:[NSString stringWithUTF8String:cstr]];
						range.length = len;
						[attributedString addAttribute:NSForegroundColorAttributeName value:[NSColor grayColor] range:range];
					}
					else
					{
						PGPKeyID		keyID;
						char			keyIDString[ kPGPMaxKeyIDStringSize ];
						
						PGPGetKeyDBObjDataProperty( [item keyDBItem], kPGPSigProperty_KeyID, &keyID, sizeof(PGPKeyID), &len );
						PGPGetKeyIDString( &keyID, kPGPKeyIDString_Abbreviated, keyIDString);
						keyIDString[10] = '\0';
						[attributedString initWithString:[NSString stringWithFormat:
							LSCL(@"Unknown Signer, Key ID is %s", @""),keyIDString]];
						range.length = [attributedString length];
						[attributedString addAttribute:NSForegroundColorAttributeName value:[NSColor grayColor] range:range];
					}
					break;
				}
				default:
					break;
			}
			if( !font )
				font = [[NSFont systemFontOfSize:[NSFont smallSystemFontSize]] retain];
			range.length = [attributedString length];
			[attributedString addAttribute:NSFontAttributeName value:font range:range];
			if (range.length)
			{
				BOOL	truncated = NO;
				float	maxTextWidth;
				
				maxTextWidth = [tableColumn width] - 48 - ( indentLevel * 18 );
				if( maxTextWidth > 32 )
					while( [attributedString size].width > maxTextWidth )
					{
						[attributedString deleteCharactersInRange:NSMakeRange( [attributedString length] / 2, 1 )];
						truncated = YES;
					}
				if( truncated )
					[attributedString replaceCharactersInRange:NSMakeRange( [attributedString length] / 2, 1 ) withString:@""];
			}
			if (!range.length)
				[attributedString initWithString:@""];
			[cell setAttributedStringValue:attributedString];
			[font release];
			break;
		}
		case kPGPclKeyList_ValidityColumn:
			if( mFlags & kPGPclKeyList_ValidityButton )
			{
				[cell setImage:[self getValidityImage:item]];
				if( ![cell image] )
					[cell setTransparent:YES];
				else
					[cell setTransparent:NO];
			}
			break;
		case kPGPclKeyList_TrustColumn:
			if( [item keyDBObjType] != kPGPKeyDBObjType_Key )
				[cell setEnabled:NO];
			else
				[cell setEnabled:YES];
			break;
        default:
            break;
	}
}

- (BOOL)outlineView:(NSOutlineView *)outlineView shouldEditTableColumn:(NSTableColumn *)tableColumn item:(id)item
{
	BOOL edit = NO;
	
	switch( [[tableColumn identifier] intValue] )
	{
		case kPGPclKeyList_TrustColumn:
			if( [item keyDBObjType] == kPGPKeyDBObjType_Key )
			{
				PGPBoolean	axiomatic;
				
				PGPGetKeyDBObjBooleanProperty( [item keyDBItem], kPGPKeyProperty_IsAxiomatic, &axiomatic );
				if( !axiomatic)
					edit = YES;
			}
			break;
	}
	return edit;
}

- (BOOL)outlineView:(NSOutlineView *)outlineView shouldSelectTableColumn:(NSTableColumn *)tableColumn
{
	return NO;
}

- (BOOL)outlineView:(NSOutlineView *)outlineView shouldSelectItem:(id)item
{
	BOOL select = YES;
	
	if( mFlags & kPGPclKeyList_SelectKeysOnly )
	{
		if( [item keyDBObjType] != kPGPKeyDBObjType_Key )
			return NO;
	}
	return select;
}

- (void)outlineViewSelectionDidChange:(NSNotification *)notification
{
	[mController selectionChanged];
}

- (BOOL)outlineView:(NSOutlineView *)outlineView writeItems:(NSArray*)items toPasteboard:(NSPasteboard*)pboard
{
	PGPKeySetRef	keySet;
	NSEnumerator	*itemEnumerator = [items objectEnumerator];
	
	if( !(mFlags & kPGPclKeyList_EnableDragOut) )
		return NO;
	if( PGPKeySetRefIsValid( keySet = [self getEnumeratorKeySet:itemEnumerator] ) )
		[self addKeySet:keySet toPasteboard:pboard];
	return YES;
}

- (NSDragOperation)outlineView:(NSOutlineView *)outlineView draggingSourceOperationMaskForLocal:(BOOL)flag
{
	return NSDragOperationGeneric;
}

- (NSDragOperation)outlineView:(NSOutlineView*)olv validateDrop:(id <NSDraggingInfo>)info proposedItem:(id)item proposedChildIndex:(int)index
{
	[mKeyOutline setDropItem:nil dropChildIndex:NSOutlineViewDropOnItemIndex];
	if( [[info draggingSource] isEqual:olv] || mInModal )
		return NSDragOperationNone;
	return NSDragOperationGeneric;
}

- (BOOL)outlineView:(NSOutlineView*)olv acceptDrop:(id <NSDraggingInfo>)info item:(id)item childIndex:(int)index
{
	BOOL	imported = NO;
	
	if( [[info draggingSource] isEqual:mKeyOutline] )
		return NO;
	[self importKeyDB:[self importPasteboard:[info draggingPasteboard] done:&imported] confirm:!imported];
	return YES;
}

- (void)importSelectDidEnd:(PGPError)resultCode selectedKeys:(PGPKeySetRef)keySet
{
	mInModal = NO;

	if( IsntPGPError( resultCode ) && PGPKeySetRefIsValid( keySet ) )
	{
		PGPError	err = kPGPError_NoErr;
		
		err = PGPCopyKeys( keySet, mKeyDB, NULL );
		PGPFreeKeySet( keySet );
		if( IsntPGPError( err ) )
		{
			[self syncWithKeyDB:YES];
			[CKeyInfo updateAllForDB:mKeyDB];
		}
	}
	if( PGPKeyDBRefIsValid( mImportDB ) )
	{
		PGPFreeKeyDB( mImportDB );
		mImportDB = kInvalidPGPKeyDBRef;
	}
}

	PGPInt16
MatchUserIDs(const char *uid1, const char *uid2)
{
	PGPInt16	matchChars = 0;
	char		a, b;
	
	while(*uid1 && *uid2)
	{
		a = *uid1++;	b = *uid2++;
		if((a >= 'a') && (a <= 'z'))
			a -= ('a' - 'A');
		if((b >= 'a') && (b <= 'z'))
			b -= ('a' - 'A');
		if(a == b)
			matchChars++;
		else
			break;
	}
	return matchChars;
}

- (void)selectSearchString
{
	NSEnumerator	*keyEnumerator = mKeyArray ? [mKeyArray objectEnumerator] : nil;
	CKeyDBItem		*cKey,
					*bestKey = nil;
	char			idStr[kPGPcl_MaxPGPUserIDLength];
	PGPSize			len;
	PGPInt32		searchScore,
					bestSearchScore = 0;
	
	if( !keyEnumerator )
		return;
	while( cKey = [keyEnumerator nextObject] )
	{
		if( IsntPGPError( PGPGetPrimaryUserIDName( [cKey keyDBItem], idStr, kPGPcl_MaxPGPUserIDLength - 1, &len ) ) )
		{
			idStr[len] = '\0';
			searchScore = MatchUserIDs( idStr, [mSearchString UTF8String] );
			if( searchScore > bestSearchScore )
			{
				bestSearchScore = searchScore;
				bestKey = cKey;	
			}
		}
	}
	if( !bestKey )
		[mSearchString deleteCharactersInRange:NSMakeRange(0,[mSearchString length])];
	else
	{
		int	bestRow = [mKeyOutline rowForItem:bestKey];
		
		[mKeyOutline selectRow:bestRow byExtendingSelection:NO];
		[mKeyOutline scrollRowToVisible:bestRow];
	}
}

- (void)keyDown:(NSEvent *)theEvent
{
	NSString	*pressed = [theEvent characters];
	PGPUInt32	sIndex;
	BOOL		optionDown = ( ( [theEvent modifierFlags] & NSAlternateKeyMask ) > 0 );
	
	for( sIndex = 0; sIndex < [pressed length]; sIndex++ )
	{
		unichar	pressedChar = [pressed characterAtIndex:sIndex];
#define char_Delete		127
		
		switch( pressedChar )
		{
			case char_Delete:
				if( !( mFlags & kPGPclKeyList_ReadOnly ) )
				{
					if( optionDown )
						[self deleteSelection];
					else
						[self deleteConfirm];
				}
				break;
			default:
				if( ( ( pressedChar >= ' ') && ( pressedChar <= 'Z') ) ||
					( ( pressedChar >= 'a' ) && ( pressedChar <= 'z' ) ) )
				{
					char	aStr[2];
					
					aStr[1] = '\0';
					if( [theEvent timestamp] - kMaxKeypressInterval <= mLastKeypress )
					{
						aStr[0] = pressedChar;
						[mSearchString appendString:[NSString stringWithUTF8String:aStr]];
					}
					else
					{
						[mSearchString deleteCharactersInRange:NSMakeRange(0,[mSearchString length])];
						aStr[0] = pressedChar;
						[mSearchString appendString:[NSString stringWithUTF8String:aStr]];
					}
					mLastKeypress = [theEvent timestamp];
					[self selectSearchString];
				}
				else
					[self interpretKeyEvents:[NSArray arrayWithObject:theEvent]];
				break;
		}
	}
}

- (BOOL)validateMenuItem:(id <NSMenuItem>)menuItem
{
	BOOL	enabled = NO;
	SEL		menuSelector = [menuItem action];
	
	if( ( menuSelector == @selector(copy:) ) ||
		( !( mFlags & kPGPclKeyList_ReadOnly ) &&
		( ( menuSelector == @selector(cut:) ) ||
		( menuSelector == @selector(clear:) ) ) ) )
	{
		enabled = ( [mKeyOutline numberOfSelectedRows] > 0 );
	}
	else if( menuSelector == @selector(paste:) )
	{
		enabled = YES;
	}
	else
		enabled = [super validateMenuItem:menuItem];
	return enabled;
}

- (void)copy:(id)sender
{
	NSPasteboard	*pboard = [NSPasteboard generalPasteboard];
	PGPKeySetRef	keySet;
	
	if( PGPKeySetRefIsValid( keySet = [self getEnumeratorKeySet:[self selectedItemEnumerator]] ) )
		[self addKeySet:keySet toPasteboard:pboard];
}

- (void)clear:(id)sender
{
	[self deleteConfirm];
}

- (void)cut:(id)sender
{
	NSPasteboard	*pboard = [NSPasteboard generalPasteboard];

	[self copy:sender];
	if( PGPKeySetRefIsValid( mCopyPBKeySet ) )
	{
		[pboard declareTypes:[NSArray arrayWithObjects:NSStringPboardType, nil] owner:self];
		[self pasteboard:pboard provideDataForType:NSStringPboardType];
		[self clear:sender];
	}
}

- (void)paste:(id)sender
{
	BOOL	imported = NO;
	
	[self importKeyDB:[self importPasteboard:[NSPasteboard generalPasteboard] done:&imported] confirm:!imported];
}

- (void)insertTab:(id)sender
{
	[[mKeyOutline window] selectNextKeyView:sender];
}

- (void)insertBacktab:(id)sender
{
	[[mKeyOutline window] selectPreviousKeyView:sender];
}

- (PGPPrefRef)pgpPrefRef
{
	return mPrefRef;
}

- (PGPKeyDBRef)mainDB;
{
	return [mController mainDB];
}

- (PGPtlsContextRef)tlsContext
{
	return [mController tlsContext];
}

- (BOOL)isModal
{
	return mInModal;
}

- (void)dealloc
{
	int	columnIndex;
	
	if( PGPKeyDBRefIsValid( mKeyDB ) )
	{
		[CKeyInfo closeAll:self forDB:mKeyDB];
		[CCertInfo closeAll:self forDB:mKeyDB];
	}
	[mSearchString release];
	if( mKeyArray )
		[mKeyArray release];
	if( PGPKeyIterRefIsValid( mKeyIter ) )
		PGPFreeKeyIter( mKeyIter );
	if( PGPKeyListRefIsValid( mKeyList ) )
		PGPFreeKeyList( mKeyList );
	if( PGPKeySetRefIsValid( mCopyPBKeySet ) )
		PGPFreeKeySet( mCopyPBKeySet );
	if( PGPKeySetRefIsValid( mDragPBKeySet ) )
		PGPFreeKeySet( mDragPBKeySet );
	if( PGPKeySetRefIsValid( mKeySet ) && mOwnsKeySet )
		PGPFreeKeySet( mKeySet );
	if( PGPKeyDBRefIsValid( mImportDB ) )
	{
		PGPFreeKeyDB( mImportDB );
		mImportDB = kInvalidPGPKeyDBRef;
	}
	for( columnIndex = 0; columnIndex < kPGPclKeyList_MaxColumns; columnIndex++ )
		[mColumns[columnIndex] release];
	[super dealloc];
	PGPLog((@"Deallocated CKeyView"));
}

@end

