/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation
    All rights reserved.

    $Id: CLicense.m,v 1.7 2002/11/15 02:09:16 jeffc Exp $
____________________________________________________________________________*/

#import "CLicense.h"
#import "pgpClientLib.h"
#import "pgpClientPrefs.h"
#import "PGPOpenPrefs.h"
#import "pflPrefTypes.h"

static CLicense*		sLicense;

PGPError PGPclLicenseCheck(PGPContextRef context)
{
	if (sLicense)
	{
		PGPLog((@"PGPclLicenseCheck called when a CLicense object already exists..."));
		PGPclLicenseFree(context);
	}
	
	if (!sLicense)
		sLicense = [[CLicense alloc] initWithContext:context];

#if PGP_LICENSENUMBERS	
	return [sLicense isAuthorized] ? kPGPError_NoErr : kPGPError_UnknownError;
#else
	return kPGPError_NoErr;
#endif
}

PGPError PGPclLicenseFree(PGPContextRef context)
{
	(void) context;
	
	if (sLicense)
	{
		[sLicense release];
		sLicense = nil;
	}
	
	return kPGPError_NoErr;
}

PGPError PGPclLicenseName(PGPByte* buffer, PGPSize len)
{
	if (!buffer || !len)
		return kPGPError_BadParams;
	
	buffer[0] = '\0';
	
#if PGP_LICENSENUMBERS
	pgpCopyMemory([[sLicense prefName] UTF8String], buffer, len - 1);
	buffer[len - 1] = '\0';
	
	return kPGPError_NoErr;
#else
	return kPGPError_NoErr;
#endif
}

PGPError PGPclLicenseCorp(PGPByte* buffer, PGPSize len)
{
	if (!buffer || !len)
		return kPGPError_BadParams;
	
	buffer[0] = '\0';
	
#if PGP_LICENSENUMBERS
	pgpCopyMemory([[sLicense prefOrganization] UTF8String], buffer, len - 1);
	buffer[len - 1] = '\0';
	
	return kPGPError_NoErr;
#else
	return kPGPError_NoErr;
#endif
}

PGPUInt8 PGPclLicenseProductID(void)
{
#if PGP_LICENSENUMBERS
	return [sLicense productID];
#else
	return 0;
#endif
}

PGPUInt16 PGPclLicenseEvalPeriod(void)
{
#if PGP_LICENSENUMBERS
	return [sLicense evalPeriod];
#else
	return 0;
#endif
}

PGPUInt32 PGPclLicenseQuantity(void)
{
#if PGP_LICENSENUMBERS
	return [sLicense numLicenses];
#else
	return 1;
#endif
}

PGPUInt32 PGPclLicenseSerial(void)
{
#if PGP_LICENSENUMBERS
	return [sLicense serialNumber];
#else
	return 0;
#endif
}

PGPInt32 PGPclLicenseDaysLeft(void)
{
#if PGP_LICENSENUMBERS
	return [sLicense daysLeft];
#else
	return 0x7FFFFFFF;
#endif
}

PGPBoolean PGPclEnterprise(void)
{
#if PGP_LICENSENUMBERS
	return [sLicense enterpriseEnabled];
#else
	return 1;
#endif
}

PGPBoolean PGPclPGPdiskEnabled(void)
{
#if PGP_LICENSENUMBERS
	return [sLicense diskEnabled];
#else
	return 1;
#endif
}

PGPBoolean PGPclPGPmailEnabled(void)
{
#if PGP_LICENSENUMBERS
	return [sLicense mailEnabled];
#else
	return 1;
#endif
}

PGPBoolean PGPclEvaluation(void)
{
#if PGP_LICENSENUMBERS
	return [sLicense evaluation];
#else
	return 0;
#endif
}

PGPBoolean PGPclDesktop(void)
{
#if PGP_LICENSENUMBERS
	return [sLicense desktop];
#else
	return 1;
#endif
}

PGPBoolean PGPclIsAuthorized(void)
{
#if PGP_LICENSENUMBERS
	return [sLicense isAuthorized];
#else
	return 1;
#endif
}

@implementation CLicense

- (id)init
{
	if (!mInited)
	{
		[super init];
		
		mInited = YES;
	}
	
	return self;
}

- (id)initWithContext:(PGPContextRef)context
{
	[self init];
	[self setContext:context];
	
	return self;
}

- (PGPContextRef)context
{
	return mContext;
}

- (void)setContext:(PGPContextRef)context
{
	mContext = context;
}

- (PGPLicenseNumberRef)ln
{
	NSString*		ln;
	PGPError		err;
	
	if (!mLNRef)
	{
		ln = [self prefLN];
		
		if (IsPGPError(err = PGPLNImportBase32(mContext, [ln UTF8String], &mLNRef)))
		{
			PGPLog((@"PGPLNImportBase32 returned err: %d", err));
			mLNRef = kInvalidPGPLicenseNumberRef;
		}
	}
	
	return mLNRef;
}

- (void)setLN:(PGPLicenseNumberRef)ln
{
	if (mLNRef)
	{
		PGPLNFree(mLNRef);
		mLNRef = kInvalidPGPLicenseNumberRef;
	}
	
	mLNRef = ln;
}

- (PGPLicenseAuthorizationRef)la
{
	NSString*		la;
	PGPError		err;
	
	if (!mLARef)
	{
		la = [self prefLA];
		
		if (IsPGPError(err = PGPLAImportBase64(mContext, [la UTF8String], &mLARef)))
		{
			PGPLog((@"PGPLAImportBase64 returned err: %d", err));
			mLARef = kInvalidPGPLicenseAuthorizationRef;
		}
	}
	
	return mLARef;
}

- (void)setLA:(PGPLicenseNumberRef)la
{
	if (mLARef)
	{
		PGPLAFree(mLARef);
		mLARef = kInvalidPGPLicenseAuthorizationRef;
	}
	
	mLARef = la;
}

- (PGPBoolean)isAuthorized
{
	if (!mAuthChecked)
	{
		PGPLicenseNumberRef			ln = [self ln];
		PGPLicenseAuthorizationRef	la = [self la];
		NSString*					name = [self prefName];
		NSString*					organization = [self prefOrganization];
		PGPError					err;
		
		err = [self authorize:ln licenseAuth:la name:name organization:organization];
		
		mAuthorized = IsPGPError(err) ? NO : YES;
		mAuthChecked = YES;
	}
	
	return mAuthorized;
}

- (PGPBoolean)diskEnabled
{
	return [self flags] & kPGPLicenseNumberFlag_Disk;
}

- (PGPBoolean)mailEnabled
{
	return [self flags] & kPGPLicenseNumberFlag_Mail;
}

- (PGPBoolean)enterpriseEnabled
{
	return [self flags] & kPGPLicenseNumberFlag_Enterprise;
}

- (PGPBoolean)evaluation
{
	return [self flags] & kPGPLicenseNumberFlag_Evaluation;
}

- (PGPBoolean)desktop
{
	return [self flags] & kPGPLicenseNumberFlag_Desktop;
}

- (PGPUInt8)productID
{
	PGPError		err;
	PGPUInt8		prodID = 0;
	
	if (IsPGPError(err = PGPLNGetProductID(mContext, [self ln], &prodID)))
		PGPLog((@"PGPLNGetProductID failed with err: %d", err));
	
	return prodID;
}

- (PGPUInt8)flags
{
	PGPError				err;
	PGPUInt8				flags = 0;
	PGPLicenseNumberRef*	lnRef = [self ln];
	
	if (lnRef)
	{
		if (IsPGPError(err = PGPLNGetFlags(mContext, lnRef, &flags)))
			PGPLog((@"PGPLNGetFlags failed with err: %d", err));
	}
	
	return flags;
}

- (PGPUInt16)evalPeriod
{
	PGPError		err;
	PGPUInt16		evalPeriod = 0;
	
	if (IsPGPError(err = PGPLNGetEvalPeriod(mContext, [self ln], &evalPeriod)))
		PGPLog((@"PGPLNGetEvalPeriod failed with err: %d", err));
	
	return evalPeriod;
}

- (PGPUInt32)numLicenses
{
	PGPError		err;
	PGPUInt32		numLicenses = 0;
	
	if (IsPGPError(err = PGPLNGetNumLicenses(mContext, [self ln], &numLicenses)))
		PGPLog((@"PGPLNGetNumLicenses failed with err: %d", err));
	
	return numLicenses;
}

- (PGPUInt32)serialNumber
{
	PGPError		err;
	PGPUInt32		serialNumber = 0;
	
	if (IsPGPError(err = PGPLNGetSN(mContext, [self ln], &serialNumber)))
		PGPLog((@"PGPLNGetSN failed with err: %d", err));
	
	return serialNumber;
}

- (PGPInt32)daysLeft
{
	PGPError		err;
	PGPInt32		daysLeft = 0;
	
	if (IsPGPError(err = PGPLADaysLeft(mContext, [self ln], [self la], &daysLeft)))
		PGPLog((@"PGPLADaysLeft failed with err: %d", err));
	
	return daysLeft;
}

- (PGPError)authorize:(PGPLicenseNumberRef)ln licenseAuth:(PGPLicenseAuthorizationRef)la name:(NSString*)name organization:(NSString*)organization
{
	PGPError		err;
	
	if (!ln || !la || !name || !organization)
		return kPGPError_BadParams;
	
	if (IsPGPError(err = PGPLAVerify(mContext, ln, la, [name UTF8String], [organization UTF8String])))
		PGPLog((@"PGPLAVerify returned err: %d", err));
	
	return err;
}

- (NSString*)prefName
{
	return [self getPrefString:kPGPPrefLicenseName];
}

- (NSString*)prefOrganization
{
	return [self getPrefString:kPGPPrefLicenseCompany];
}

- (NSString*)prefLN
{
	return [self getPrefString:kPGPPrefLicenseNumber];
}

- (NSString*)prefLA
{
	return [self getPrefString:kPGPPrefLicenseAuthorization];
}

- (NSString*)getPrefString:(PGPPrefIndex)prefID
{
	PGPError		err;
	char*			szData = NULL;
	NSString*		string = nil;
	
	if (!mPrefRef)
	{
		err = PGPclOpenPrefs(PGPPeekContextMemoryMgr(mContext), kPGPclPrefsClientKey, nil, &mPrefRef, FALSE);
		
		if (IsPGPError(err))
			PGPLog((@"Failed to open prefs (err=%d)", err));
	}
	
	if (IsPGPError(err = PGPGetPrefStringAlloc(mPrefRef, prefID, &szData)))
		PGPLog((@"Failed to get pref string %d (err=%d)", prefID, err));
	else
	{
		string = [NSString stringWithUTF8String:szData];
		
		PGPFreeData(szData);
		szData = NULL;
	}
	
	return string;
}

- (void)dealloc
{
	[self setLN:NULL];
	[self setLA:NULL];
	
	if (PGPPrefRefIsValid(mPrefRef))
	{
		PGPFreePrefs(mPrefRef);
		mPrefRef = kInvalidPGPPrefRef;
	}
	
	[super dealloc];
}

@end
