/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation
    All rights reserved.

    $Id: CPassphraseField.h,v 1.2 2002/08/06 20:09:17 dallen Exp $
____________________________________________________________________________*/

#import <Cocoa/Cocoa.h>

@class CPassphraseFieldEditor;

@interface CPassphraseField : NSTextField
{
	NSMutableString*		mHiddenPassphraseString;	/* Does not contain the passphrase. This one is shadowed. */
	NSMutableString*		mActualPassphraseString;	/* Contains the passphrase. */
	unsigned				mLastStringLength;
	CPassphraseFieldEditor*	mFieldEditor;
	
	NSProgressIndicator*	mProgressIndicator;
	
	BOOL					mTypingHidden;
}

- (BOOL)becomeFirstResponder;

- (NSProgressIndicator*)progressIndicator;
- (void)setProgressIndicator:(NSProgressIndicator*)progressIndicator;

- (BOOL)isTypingHidden;
- (void)setTypingHidden:(BOOL)flag;

- (NSString*)passphrase;

- (void)addCharacter:(unichar)character;
- (void)deleteCharacter;
- (unsigned)lastStringLength;
- (void)updateStringLength;
- (void)displayHiddenText;
- (void)hideText;

- (void)clearCurrentPassphrase;

- (id)windowWillReturnFieldEditor:(NSWindow*)window toObject:(id)object;

- (void)dealloc;

@end

@interface CPassphraseFormatter : NSFormatter
{
	CPassphraseField*		mPassphraseField;	/* CPassphraseField for which we're formatting */
}

- (CPassphraseField*)passphraseField;
- (void)setPassphraseField:(CPassphraseField*)passphraseField;

/* Required for subclassers */
- (NSString*)stringForObjectValue:(id)object;
- (BOOL)getObjectValue:(id*)object forString:(NSString*)string errorDescription:(NSString**)error;
- (NSAttributedString*)attributedStringForObjectValue:(id)object withDefaultAttributes:(NSDictionary*)attributes;

- (BOOL)isPartialStringValid:(NSString*)string newEditingString:(NSString**)newString errorDescription:(NSString**)error;

- (void)dealloc;

@end

@interface CPassphraseFieldEditor : NSTextView
{
	CPassphraseField*		mField;
}

+ (id)sharedFieldEditor;

- (void)setRepresentedTextField:(CPassphraseField*)field;

- (void)keyDown:(NSEvent*)theEvent;
- (void)insertTab:(id)sender;

/* Disabled actions */
- (void)cut:(id)sender;
- (void)copy:(id)sender;
- (void)paste:(id)sender;
- (void)delete:(id)sender;
- (void)selectAll:(id)sender;

- (void)setSelectedRange:(NSRange)charRange affinity:(NSSelectionAffinity)affinity stillSelecting:(BOOL)flag;
- (NSRange)selectionRangeForProposedRange:(NSRange)proposed granularity:(NSSelectionGranularity)granularity;

@end