/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation
    All rights reserved.

    $Id: CTraceIntruder.m,v 1.4 2002/08/06 20:09:17 dallen Exp $
____________________________________________________________________________*/

#import "CTraceIntruder.h"
#import "CRemoteAuth.h"
#import "pgpSourceTrace.h"
#import "pflPrefTypes.h"
#import "pgpNetPrefs.h"
#import <arpa/inet.h>

#define kPGPclTraceServerName					@"CTraceIntruder.Server"
#define kPGPclTraceIPServer						@"CTraceIntruder:%s"
#define kTraceBufferSize						8192
#define kNoResponseString						LSCL(@"(no response)", @"Trace no response")

PGPError PGPclTraceIntruder(PGPContextRef context, PGPPrefRef netPrefRef, PGPUInt32 ipAddress, id rootObject)
{
	NSAutoreleasePool*	pool = [[NSAutoreleasePool alloc] init];
	NSConnection*		connection;
	NSPort*				sendPort = [NSPort port];
	NSPort*				receivePort = [NSPort port];
	NSArray*			ports;
	CTraceIntruder*		tracer;
	PGPError			err = kPGPError_NoErr;
	
	connection = [NSConnection connectionWithReceivePort:receivePort sendPort:sendPort];
	[connection setRootObject:rootObject];

	ports = [NSArray arrayWithObjects:sendPort, receivePort, nil];
	tracer = [[CTraceIntruder alloc] initWithIPAddress:ipAddress];
	[tracer setContext:context];
	[tracer setNetPrefRef:netPrefRef];
	[tracer setServerConnection:connection];
	
	[NSThread detachNewThreadSelector:@selector(beginTraceWithPorts:) toTarget:tracer withObject:ports];

	[pool release];
	
	return err;
}

	static void
sGetDomainFromDNSName(
		char *		pszDNS,
		char *		pszDomain )
{
	char*	p1	= NULL;
	char*	p2	= NULL;

	pszDomain[0] = '\0';

	p2 = strrchr (pszDNS, '.');
	if (p2 && (p2 != pszDNS))
	{
		*p2 = '\0';
		p1 = strrchr (pszDNS, '.');
		*p2 = '.';

		if (p1)
			p1++;
		else
			p1 = pszDNS;
	}

	if (p1)
		strcpy (pszDomain, p1);
}

@implementation CTraceIntruder

- (id)init
{
	if (!mInited)
	{
		[super init];
		
		mInited = YES;
	}

	[[NSNotificationCenter defaultCenter] addObserver:self
			selector:@selector(abortTrace:)
			name:kPGPclAbortTraceNotification
			object:nil];
	
	return self;
}

- (id)initWithIPAddress:(PGPUInt32)ipAddress
{
	mIPAddress = ipAddress;
	
	return [self init];
}

- (void)setContext:(PGPContextRef)context
{
	mContext = context;
}

- (void)setNetPrefRef:(PGPPrefRef)netPrefRef
{
	mNetPrefRef = netPrefRef;
}

- (void)setServerConnection:(NSConnection*)connection
{
	if (mServerConnection)
	{
		[mServerConnection release];
		mServerConnection = nil;
	}
	
	if (connection)
		mServerConnection = [connection retain];
}

- (void)beginTraceWithPorts:(NSArray*)ports
{
	NSAutoreleasePool*			pool = [[NSAutoreleasePool alloc] init];
	NSConnection*				connection = nil;
	id							proxy = nil;
	char						domain[128];
	char						cstr[256];
	char						cstr2[256];
	char						whois[256];
	char*						traceBuf = NULL;
	struct in_addr				addr;
	struct hostent*				hent = NULL;
	PGPSocketsThreadStorageRef	previousStorage;
	PGPMemoryMgrRef				memoryMgr = PGPPeekContextMemoryMgr(mContext);
	PGPError					err = kPGPError_NoErr;
	
	if (!ports)
		goto done;
	
	connection = [NSConnection connectionWithReceivePort:[ports objectAtIndex:0] sendPort:[ports objectAtIndex:1]];
	proxy = [connection rootProxy];
	[proxy setProtocolForProxy:@protocol(CTraceIntruderClient)];
	
	if (mTraceAborted)
		goto done;
	
	PGPGetPrefStringBuffer(mNetPrefRef, kPGPNetPrefWhoisServer, sizeof(whois), whois);
	
	domain[0] = '\0';
	addr.s_addr = mIPAddress;
	
	PGPSocketsInit();
	PGPSocketsCreateThreadStorage(&previousStorage);
	
	traceBuf = (char*) PGPNewData(memoryMgr, kTraceBufferSize, 0);
	if (!traceBuf || mTraceAborted)
		goto done;
	
	hent = gethostbyaddr((const char*) &addr, sizeof(addr), AF_INET);
	if (hent)
	{
		[proxy setDNSName:[NSString stringWithCString:hent->h_name]];
		sGetDomainFromDNSName(hent->h_name, domain);
	}
	else
		[proxy setDNSName:kNoResponseString];
	
	if (mTraceAborted)
		goto done;
	
	err = PGPTraceQueryNetBIOSInfo(mIPAddress, cstr, sizeof(cstr), traceBuf, kTraceBufferSize, cstr2);
	if (IsntPGPError(err))
	{
		[proxy setNetBTName:[NSString stringWithCString:cstr]];

		if (traceBuf[0])
		{
			[proxy addTraceLog:[LSCL(@"NetBIOS Users:\n", @"Trace Log NBT Title") cString] title:YES];
			[proxy addTraceLog:traceBuf title:NO];
			[proxy addTraceLog:"\n\n" title:NO];
		}
		if (cstr2[0])
		{
			[proxy addTraceLog:[LSCL(@"MAC Address:\n", @"Trace Log MAC Title") cString] title:YES];
			[proxy addTraceLog:cstr2 title:NO];
		}
	}
	else
		[proxy setNetBTName:kNoResponseString];
	
	[proxy addTraceLog:"\n\n" title:NO];
	
	if (mTraceAborted)
		goto done;
	
	[proxy addTraceLog:[LSCL(@"Telnet Banner:\n", @"Trace Log Telnet Title") cString] title:YES];
	
	err = PGPTraceQueryTelnetBanner(mIPAddress, traceBuf, kTraceBufferSize);
	if (IsntPGPError(err) && traceBuf[0])
		[proxy addTraceLog:traceBuf title:NO];
	else
		[proxy addTraceLog:[kNoResponseString cString] title:NO];
	
	[proxy addTraceLog:"\n\n" title:NO];
	
	if (mTraceAborted)
		goto done;
	
	[proxy addTraceLog:[LSCL(@"HTTP Banner:\n", @"Trace Log HTTP Title") cString] title:YES];
	
	err = PGPTraceQueryHTTPVersion(mIPAddress, traceBuf, kTraceBufferSize);
	if (IsntPGPError(err) && traceBuf[0])
		[proxy addTraceLog:traceBuf title:NO];
	else
		[proxy addTraceLog:[kNoResponseString cString] title:NO];
	
	[proxy addTraceLog:"\n\n" title:NO];
	
	if (mTraceAborted)
		goto done;
	
	[proxy addTraceLog:[LSCL(@"FTP Banner:\n", @"Trace Log FTP Title") cString] title:YES];
	
	err = PGPTraceQueryFTPBanner(mIPAddress, traceBuf, kTraceBufferSize);
	if (IsntPGPError(err) && traceBuf[0])
		[proxy addTraceLog:traceBuf title:NO];
	else
		[proxy addTraceLog:[kNoResponseString cString] title:NO];
	
	[proxy addTraceLog:"\n\n" title:NO];
	
	if (mTraceAborted)
		goto done;
	
	[proxy addTraceLog:[LSCL(@"SMTP Banner:\n", @"Trace Log SMTP Title") cString] title:YES];
	
	err = PGPTraceQuerySMTPBanner(mIPAddress, traceBuf, kTraceBufferSize);
	if (IsntPGPError(err) && traceBuf[0])
		[proxy addTraceLog:traceBuf title:NO];
	else
		[proxy addTraceLog:[kNoResponseString cString] title:NO];
	
	[proxy addTraceLog:"\n\n" title:NO];

	if (mTraceAborted)
		goto done;
	
	if (domain[0])
	{
		traceBuf[0] = '\0';
		
		[proxy addTraceLog:[[NSString stringWithFormat:
					LSCL(@"WHOIS Query for %s:\n", @"Trace Log SMTP Title"), domain] cString] title:YES];
		
		err = PGPTraceQueryWHOIS(mContext, whois, domain, traceBuf, kTraceBufferSize);
		if (IsntPGPError(err))
		{
			char*	altServerStr;
			
			altServerStr = strstr(traceBuf, "Whois Server:");
			
			if (IsntNull(altServerStr))
			{
				int	eIndex;
				
				altServerStr += 14;
				strncpy(cstr, altServerStr, sizeof(whois));
				whois[sizeof(whois) - 1] = '\0';
				eIndex = strcspn(whois, " \n\r\t");
				if (eIndex < sizeof(whois))
				{
					whois[eIndex] = '\0';
					PGPTraceQueryWHOIS(mContext, whois, domain, traceBuf, kTraceBufferSize);
				}
			}
			
			if (IsntPGPError(err) && traceBuf[0])
				[proxy addTraceLog:traceBuf title:NO];
		}
		
		if (!traceBuf[0])
			[proxy addTraceLog:[kNoResponseString cString] title:NO];
		
		[proxy addTraceLog:"\n\n" title:NO];
	}
	
	if (mTraceAborted)
		goto done;
	
	[proxy addTraceLog:[[NSString stringWithFormat:
				LSCL(@"WHOIS Query for %s:\n", @"Trace Log SMTP Title"), inet_ntoa(addr)] cString] title:YES];
	
	err = PGPTraceQueryWHOIS(mContext, "whois.arin.net", inet_ntoa(addr), traceBuf, kTraceBufferSize);
	if (IsntPGPError(err))
	{
		char*	ripeStr;
		char*	apnicStr;
		
		ripeStr = strstr(traceBuf, "RIPE");
		apnicStr = strstr(traceBuf, "APNIC");
		
		if (ripeStr && apnicStr)
		{
			if (apnicStr < ripeStr)
				ripeStr = NULL;
		}
		
		if (ripeStr)
			err = PGPTraceQueryWHOIS(mContext, "whois.ripe.net", cstr, traceBuf, kTraceBufferSize);
		else if (apnicStr)
			err = PGPTraceQueryWHOIS(mContext, "whois.apnic.net", cstr, traceBuf, kTraceBufferSize);
		if (traceBuf[0])
			[proxy addTraceLog:traceBuf title:NO];
	}
	
	if (!traceBuf[0])
		[proxy addTraceLog:[kNoResponseString cString] title:NO];

done:
	if (mTraceAborted)
		[proxy addTraceLog:[LSCL(@"\nTrace Aborted.\n", @"Trace Aborted Title") cString] title:YES];
	
	if (traceBuf)
		PGPFreeData(traceBuf);
	
	PGPSocketsDisposeThreadStorage(previousStorage);
	PGPSocketsCleanup();
	
	[proxy finish];
	
	if (connection)
		[connection invalidate];
	
	[self autorelease];
	
	[pool release];
}

- (void)abortTrace:(NSNotification*)notification
{
	mTraceAborted = YES;
}

- (void)dealloc
{
	PGPLog((@"Deallocated CTraceIntruder"));

	[[NSNotificationCenter defaultCenter] removeObserver:self];
	
	if (mServerConnection)
		[mServerConnection release];
	
	[super dealloc];
}

@end
