/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.
	
	$Id: PGPmisc.m,v 1.36 2002/11/18 11:35:22 jeffc Exp $
____________________________________________________________________________*/
#import "pgpClientLibCocoa.h"
#import "pgpClientPrefs.h"
#import "pgpNetPrefs.h"
#import "pflPrefTypes.h"
#import "pgpPFLPriv.h"
#import "CRemoteAuth.h"

#import <Cocoa/Cocoa.h>

static PGPError sTranslateDate(PGPUInt32* month, PGPUInt32* day, PGPUInt32* year, char* dateString);

	PGPError
PGPclOpenKeyrings(
	PGPContextRef				context,
	char*						pszPubPath,
	char*						pszPrivPath,
	PGPOpenKeyDBFileOptions		options,
	PGPKeyDBRef*				pkeydbOut)
{
	PGPError					err	= kPGPError_NoErr;
	PGPFileSpecRef				filerefPub = kInvalidPGPFileSpecRef;
	PGPFileSpecRef				filerefPriv	= kInvalidPGPFileSpecRef;
	//PGPBoolean					bReadOnly = FALSE;
	//PGPUInt32					uStatus;

	*pkeydbOut = kInvalidPGPKeyDBRef;

	/*uStatus = PGPclTestPathNameStatus (pszPubPath);
	if (uStatus & kPGPclMediaNotInserted)
	{
		err = kPGPError_CantOpenFile;
		goto done;
	}
	if (options & kPGPOpenKeyDBFileOptions_Mutable)
	{
		if (uStatus & (kPGPclIsReadOnly | kPGPclIsInReadOnlyFolder))
		{
			err = kPGPError_FilePermissions;
			goto done;
		}
	}
	uStatus = PGPclTestPathNameStatus (pszPrivPath);
	if (uStatus & kPGPclMediaNotInserted)
	{
		err = kPGPError_CantOpenFile;
		goto done;
	}

	if (options & kPGPOpenKeyDBFileOptions_Mutable)
	{
		if (uStatus & (kPGPclIsReadOnly | kPGPclIsInReadOnlyFolder))
		{
			err = kPGPError_FilePermissions;
			goto done;
		}
	}*/

	if( IsntNull( pszPubPath ) )
	{
		err = PGPNewFileSpecFromFullPath(context, pszPubPath, &filerefPub);				CKERR;
	}
	if( IsntNull( pszPrivPath ) )
	{
		err = PGPNewFileSpecFromFullPath(context, pszPrivPath, &filerefPriv);			CKERR;
	}
	err = PGPOpenKeyDBFile(context, options, filerefPub, filerefPriv, pkeydbOut);	CKERR;

done:
	if( PGPFileSpecRefIsValid(filerefPub) )
		PGPFreeFileSpec( filerefPub );
	if( PGPFileSpecRefIsValid( filerefPriv ) )
		PGPFreeFileSpec( filerefPriv );
	return err;
}

	PGPError
PGPclOpenDefaultKeyrings(
	PGPContextRef				context,
	PGPPrefRef					prefRef,
	PGPOpenKeyDBFileOptions		options,
	PGPKeyDBRef*				pkeydbOut )
{
	PGPError					err;
	char						szPubPath[kPGPcl_MaxPath];
	char						szPrivPath[kPGPcl_MaxPath];

	err = PGPclGetPath( prefRef, kInvalidPGPPrefRef, kPGPclPublicKeyringFile, szPubPath, sizeof(szPubPath)); CKERR;
	err = PGPclGetPath( prefRef, kInvalidPGPPrefRef, kPGPclPrivateKeyringFile, szPrivPath, sizeof(szPrivPath)); CKERR;
	err = PGPclOpenKeyrings( context, szPubPath, szPrivPath, options, pkeydbOut );

done:
	return err;
}

	PGPError
PGPclOpenNetKeyrings(
	PGPContextRef				context,
	PGPPrefRef					netPrefRef,
	PGPOpenKeyDBFileOptions		options,
	PGPKeyDBRef*				pkeydbOut )
{
	PGPError					err;
	char						szPubPath[kPGPcl_MaxPath];
	char						szPrivPath[kPGPcl_MaxPath];

	err = PGPclGetPath( kInvalidPGPPrefRef, netPrefRef, kPGPclNetPublicKeyringFile, szPubPath, sizeof(szPubPath)); CKERR;
	err = PGPclGetPath( kInvalidPGPPrefRef, netPrefRef, kPGPclNetPrivateKeyringFile, szPrivPath, sizeof(szPrivPath)); CKERR;
	err = PGPclOpenKeyrings( context, szPubPath, szPrivPath, options, pkeydbOut );

done:
	return err;
}

	PGPError
PGPclGetPath(
	PGPPrefRef		prefRef,
	PGPPrefRef		netPrefRef,
	PGPclPathIndex	index,
	char			*pszPath, 
	PGPInt32		iLen)
{
	PGPError			err	= kPGPError_NoErr;
	NSAutoreleasePool *	pool = [[NSAutoreleasePool alloc] init];

	if( IsNull(pszPath) )
		return kPGPError_BadParams;
	pszPath[0] = '\0';
	switch( index )
	{
		case kPGPclPublicKeyringFile:
			err = PGPGetPrefStringBuffer( prefRef, kPGPPrefPublicKeyringFile, iLen, pszPath );
			if( pszPath[0] == '\0' )
				[[NSString stringWithFormat:
					LSCL( @"%@/Documents/PGP/PGP Public Keyring.pkr", @"Default Public Keyring Path" ),
					NSHomeDirectory()] getCString:pszPath maxLength:iLen];
			break;
		case kPGPclPrivateKeyringFile:
			err = PGPGetPrefStringBuffer( prefRef, kPGPPrefPrivateKeyringFile, iLen, pszPath );
			if( pszPath[0] == '\0' )
				[[NSString stringWithFormat:
					LSCL( @"%@/Documents/PGP/PGP Private Keyring.skr", @"Default Private Keyring Path" ),
					NSHomeDirectory()] getCString:pszPath maxLength:iLen];
			break;
		case kPGPclRandseedFile:
			err = PGPGetPrefStringBuffer( prefRef, kPGPPrefRNGSeedFile, iLen, pszPath );
			if( pszPath[0] == '\0' )
				[[NSString stringWithFormat:
					LSCL( @"%@/Documents/PGP/PGP Random Seed.rnd", @"Default Randseed Path" ),
					NSHomeDirectory()] getCString:pszPath maxLength:iLen];
			break;
		case kPGPclNetPublicKeyringFile:
			err = PGPGetPrefStringBuffer( netPrefRef, kPGPNetPrefPublicKeyringFile, iLen, pszPath );
			if( pszPath[0] == '\0' )
				[[NSString stringWithFormat:
					LSCL( @"%@/Documents/PGP/PGP Public Keyring.pkr",@"Default Net Public Keyring Path" ),
					NSHomeDirectory()] getCString:pszPath maxLength:iLen];
			break;
		case kPGPclNetPrivateKeyringFile:
			err = PGPGetPrefStringBuffer( netPrefRef, kPGPNetPrefPrivateKeyringFile, iLen, pszPath );
			if( pszPath[0] == '\0' )
				[[NSString stringWithFormat:
					LSCL( @"%@/Documents/PGP/PGP Private Keyring.skr", @"Default Net Private Keyring Path" ),
					NSHomeDirectory()] getCString:pszPath maxLength:iLen];
			break;
		case kPGPclNetLogFile:
			[LSCL( @"/Library/Application Support/PGP/PGPnetLog", @"Default Net Log Path" )
				getCString:pszPath maxLength:iLen];
			break;
		case kPGPclNetLogEventDataFolder:
			[LSCL( @"/Library/Application Support/PGP/PGPnetCaptures/", @"Default Net Event Data Folder" )
				getCString:pszPath maxLength:iLen];
			break;
		case kPGPclNetRandseedFile:
			[LSCL( @"/Library/Application Support/PGP/PGP Random Seed.rnd", @"Default Net Randseed Path" )
				getCString:pszPath maxLength:iLen];
			break;
		default:
			err = kPGPError_BadParams;
	}
	[pool release];
	
	return err;
}

	void
PGPclGetFingerprintString(
	PGPKeyDBObjRef	key,
	char			*fingerString )
{
	static char hexDigit[] = "0123456789ABCDEF";
	PGPByte		fingerprint[64];
	char		cstr[256];
	PGPInt32	strIndex;
	PGPByte		*p;
	PGPSize		len;
	
	if( IsntPGPError( PGPGetKeyDBObjDataProperty( key, kPGPKeyProperty_Fingerprint,
			fingerprint, sizeof( fingerprint ), &len ) ) )
	{
		if(len == 20)
		{
			cstr[50] = '\0';
			p = cstr;
			for(strIndex = 0 ; strIndex < 20 ; strIndex++)
			{
				*p++ = hexDigit[fingerprint[strIndex]>>4];
				*p++ = hexDigit[fingerprint[strIndex]&0xF];
				if((strIndex == 1) || (strIndex == 3) || (strIndex == 5)
						|| (strIndex == 7) || (strIndex == 11) ||
						(strIndex == 13)
						|| (strIndex == 13) || (strIndex == 15) ||
						(strIndex == 17))
					*p++ = ' ';
				else if(strIndex == 9)
				{
					*p++ = ' ';
					*p++ = ' ';
				}
			}
		}
		else
		{
			cstr[40] = '\0';
			p = cstr;
			for(strIndex = 0 ; strIndex < 16 ; strIndex++)
			{
				*p++ = hexDigit[fingerprint[strIndex]>>4];
				*p++ = hexDigit[fingerprint[strIndex]&0xF];
				if((strIndex == 1) || (strIndex == 3) || (strIndex == 5)
					|| (strIndex == 9) || (strIndex == 11) || (strIndex == 13))
					*p++ = ' ';
				else if(strIndex == 7)
				{
					*p++ = ' ';
					*p++ = ' ';
				}
			}
		}
	}
	strcpy( fingerString, cstr );
}

	PGPError
PGPclGetKeyPassphrase(
	PGPKeyDBObjRef		key,
	PGPtlsContextRef	tlsContext,
	PGPPrefRef			prefRef,
	PGPBoolean			*isSplit,
	char				**passphrase,
	PGPByte				**passKey,
	PGPSize				*passKeySize )
{
	PGPError			err = kPGPError_NoErr;
	PGPContextRef		context = PGPPeekKeyDBObjContext( key );
	PGPKeyDBRef			keyDB = PGPPeekKeyDBObjKeyDB( key );
	PGPBoolean			split,
						needsPass;

	PGPValidatePtr( isSplit );
	PGPValidatePtr( passphrase );
	PGPValidatePtr( passKey );
	PGPValidatePtr( passKeySize );
	
	*isSplit = FALSE;
	PGPGetKeyDBObjBooleanProperty( key, kPGPKeyProperty_NeedsPassphrase, &needsPass );
	if( needsPass )
	{
		PGPGetKeyDBObjBooleanProperty( key, kPGPKeyProperty_IsSecretShared, &split );
		if( split )
		{
			if( PGPPrefRefIsValid( prefRef ) && PGPtlsContextRefIsValid( tlsContext ) )
			{
				err = PGPclReconstitutionDialog( key, keyDB, tlsContext, prefRef, passKey, passKeySize );	CKERR;
				*isSplit = TRUE;
			}
			else
				err = kPGPError_KeyUnusableForSignature;
		}
		else
		{
			[NSApp activateIgnoringOtherApps:YES];
			err = PGPKeyPassphraseDialog( context, key,
						PGPOUIOutputPassphrase( context, passphrase ),
						PGPOLastOption( context ) );
		}
	}
	else
		*passphrase = PGPNewData( PGPPeekContextMemoryMgr( context ), 1, kPGPMemoryMgrFlags_Clear );
done:
	return err;
}

	NSString *
PGPclGetPrimaryUserIDName(
	PGPKeyDBObjRef				key )
{
	char						cstr[kPGPcl_MaxPGPUserIDLength];
	NSString					*name = nil;
	PGPSize						len;
	PGPError					err;
	
	if( IsntPGPError( err = PGPGetPrimaryUserIDName( key, cstr, kPGPcl_MaxPGPUserIDLength - 1, &len ) ) )
	{
		name = [NSString stringWithUTF8String:cstr];
		
		if (!name)
		{
			name = [[[NSString alloc] initWithData:[NSData dataWithBytes:cstr length:len] encoding:NSMacOSRomanStringEncoding] autorelease];
		}
	}
	return name;
}

	NSString *
PGPclGetUserIDName(
	PGPKeyDBObjRef				userID )
{
	char						cstr[kPGPcl_MaxPGPUserIDLength];
	NSString					*name = nil;
	PGPSize						len;
	
	if( IsntPGPError( PGPGetKeyDBObjDataProperty( userID, kPGPUserIDProperty_Name,
		cstr, kPGPcl_MaxPGPUserIDLength - 1, &len ) ) )
	{
		name = [NSString stringWithUTF8String:cstr];
		
		if (!name)
		{
			name = [[[NSString alloc] initWithData:[NSData dataWithBytes:cstr length:len] encoding:NSMacOSRomanStringEncoding] autorelease];
		}
	}
	return name;
}

	NSString *
PGPGetErrorNSString( PGPError theError )
{
	NSString	*errString = @"";
	char		cstr[256];
	
	if( IsntPGPError( PGPGetErrorString( theError, sizeof(cstr) - 1, cstr ) ) )
		errString = [NSString stringWithCString:cstr];
	return errString;
}

	PGPError
PGPclErrorBox( PGPError theError )
{
	if( IsPGPError( theError ) && ( theError != kPGPError_UserAbort ) )
	{
		NSBeginInformationalAlertSheet(
			LSCL( @"Operation Failed", @"PGPclErrorBox Title" ), 
			LSCL( @"OK", @"OK" ), nil, nil,
			[[NSApplication sharedApplication] keyWindow], nil, NULL, NULL, NULL,
			[NSString stringWithFormat:LSCL( @"An error occurred while performing the operation: %@",
			@"Error text premable" ), PGPGetErrorNSString( theError )] );
	}
	return theError;
}

	PGPError
PGPclGetPrefDefault(
	PGPPrefIndex		prefID,
	PGPPrefDefinition	*prefDef)
{
	PGPUInt32	i;
	BOOL		found = NO;

	if (!prefDef)
		return kPGPError_BadParams;
	
	for (i = 0; (i < clientDefaultsSize) && !found; ++i)
	{
		if (clientDefaults[i].index == prefID)
		{
			found = YES;
			*prefDef = clientDefaults[i];
		}
	}
	
	return kPGPError_NoErr;
}

	PGPError
PGPclGetNetPrefDefault(
	PGPPrefIndex		prefID,
	PGPPrefDefinition	*prefDef)
{
	PGPUInt32	i;
	BOOL		found = NO;

	if (!prefDef)
		return kPGPError_BadParams;
	
	for (i = 0; (i < netDefaultsSize) && !found; ++i)
	{
		if (netDefaults[i].index == prefID)
		{
			found = YES;
			*prefDef = netDefaults[i];
		}
	}
	
	return kPGPError_NoErr;
}

PGPError PGPclHideFileExtensions(PGPBoolean* hideExtensions)
{
    NSUserDefaults*		defaults = [NSUserDefaults standardUserDefaults];
    NSDictionary*		dict = [defaults persistentDomainForName:NSGlobalDomain];
    NSNumber*			value;
    
    if (!hideExtensions)
        return kPGPError_BadParams;
    
    if (value = [dict objectForKey:@"AppleShowAllExtensions"])
        *hideExtensions = ![value boolValue];
    else
        *hideExtensions = TRUE;
    
    return kPGPError_NoErr;
}

PGPError PGPclKeyboardUIEnabled(PGPBoolean* keyboardNav)
{
    NSUserDefaults*		defaults = [NSUserDefaults standardUserDefaults];
    NSDictionary*		dict = [defaults persistentDomainForName:NSGlobalDomain];
    NSNumber*			value;
    
    if (!keyboardNav)
        return kPGPError_BadParams;
    
    if (value = [dict objectForKey:@"AppleKeyboardUIMode"])
	{
		if ([value intValue] & 1)
			*keyboardNav = [value intValue] & 2 ? TRUE : FALSE;
		else
			*keyboardNav = FALSE;
	}
    else
        *keyboardNav = FALSE;
    
    return kPGPError_NoErr;
}

/*
This function will only return an error if we are sure we won't
operate on the host OS. If the host OS can't be determined for
some reason, we don't return an error.
*/

PGPError PGPclIsOSSupported(void)
{
	NSAutoreleasePool*		pool = [[NSAutoreleasePool alloc] init];
	BOOL					osIsSupported = YES;
	PGPError				err = kPGPError_NoErr;

#if ! PGP_DEBUG	
    double					os10_2 = 663.0;

	if (NSAppKitVersionNumber < os10_2)
        osIsSupported = NO;
#endif
    
	if (!osIsSupported)
	{
		NSRunAlertPanel(LSCL(@"Unsupported OS version", @""),
			[NSString stringWithFormat:LSCL(@"PGP found that you are running a version of Mac OS X which is older than Mac OS X version 10.2. PGP requires Mac OS X version 10.2 or later in order to function properly. It is recommended that you upgrade to the latest version of Mac OS X.", @"")],
			LSCL(@"Quit", @""),
			nil,
			nil);
		
		err = kPGPError_FeatureNotAvailable;
	}

	[pool release];
	
	return err;
}

PGPError PGPclIsExpired(void)
{
	PGPError		err = kPGPError_NoErr;
	PGPUInt32		month, day, year;
	struct tm       tm;
	struct tm*		ptm;
	time_t			timeNow;
	time_t			timeOut;

#if PGP_BETA && PGP_BETA_DAYS

	/* Translate the compile date (MM/DD/YY), into a usable form. */
	sTranslateDate (&month, &day, &year, __DATE__);

	/* Seed the tm structure with the current time. */
	time(&timeNow);
	ptm = localtime(&timeNow);
	memcpy(&tm, ptm, sizeof(tm));

	/* Set the dates to the compile time. */
	tm.tm_mon = month - 1;
	tm.tm_mday = day; 
	tm.tm_year = year + 100;

	/* Turn the compile time into number of secs since 1/1/1970. */
	timeOut = mktime(&tm);

	timeOut += PGP_BETA_DAYS * 24 * 60 * 60;

	if (timeOut < timeNow)
		return kPGPError_FeatureNotAvailable;

#endif  /* PGP_BETA && PGP_BETA_DAYS */

	return err;
}

	PGPError
sTranslateDate(
	PGPUInt32* 	month,
	PGPUInt32* 	day,
	PGPUInt32* 	year,
	char* 		dateString)
{
	char*		dateChunk;
	char*		tempBuffer;
	PGPUInt32	bufferLen;

	PGPValidatePtr(dateString); 
	PGPValidatePtr(year);
	PGPValidatePtr(day);
	PGPValidatePtr(month);

	*day = 0;
	*month = 0;
	*year = 0;

	bufferLen = strlen(dateString) + 1;
	tempBuffer = malloc(bufferLen);
	
	if (IsNull(tempBuffer))
		return kPGPError_OutOfMemory;

	strncpy(tempBuffer, dateString, bufferLen);

	if (dateChunk = strtok(tempBuffer, "/"))
	{
		if (*dateChunk == '0')
			++dateChunk;
		
		*month = strtoq(dateChunk, NULL, 0);
	}
	
	if (dateChunk = strtok(NULL, "/"))
	{
		if (*dateChunk == '0')
			++dateChunk;
		
		*day = strtoq(dateChunk, NULL, 0);
	}
	
	if (dateChunk = strtok(NULL, "/"))
	{
		if (*dateChunk == '0')
			++dateChunk;
		
		*year = strtoq(dateChunk, NULL, 0);
	}
	
	free(tempBuffer);
	
	return kPGPError_NoErr;
}

#undef self

