/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation
    All rights reserved.

    $Id: pgpEncodeDecodeData.m,v 1.41 2002/11/15 10:17:32 wprice Exp $
____________________________________________________________________________*/
#import <PGP/PGP.h>
#import <PGPclient/pgpClientPrefs.h>
#import <PGPclient/CPGPPrefObj.h>
#import <pgpPFLPriv.h>
#import <pgpMemoryIO.h>
#import <pgpWordWrap.h>
#import <PGPclient/pgpClientLibOSX.h>
#import <PGPclient/PGPSharedEncryptDecrypt.h>
#import <PGPclient/SignatureStatusMessage.h>
#import <PGPclient/pgpVersionHeader.h>
#import <PGPclient/pgpClientLibCocoa.h>
#import <PGPclient/CRemoteAuth.h>

#import <Foundation/Foundation.h>

#define kNumFrames				18
#define kCursorDelay			5

#define kCautionStr						LSCL(@"Caution", @"Caution")
#define kStopStr						LSCL(@"Stop", @"Stop")
#define kOKStr							LSCL(@"OK", @"OK")

#define kUtilImportKeysStr				LSCL(@"Please select the keys you wish to import:", @"Import Keys")
#define kUtilBadSignatureStr			LSCL(@"The data in at least one section of this message does not match the data that was signed.", @"Bad Signature")
#define kUtilSignatureNotVerifiedStr	LSCL(@"Unable to verify signature on at least one section of this message.", @"Signature Not Verified")
#define kUtilIncorrectPassphraseStr		LSCL(@"That is not the correct passphrase. Please try again.", @"Incorrect Passphrase")

#define kSigLabelBeginStr				LSCL(@"*** PGP SIGNATURE VERIFICATION ***\n", @"")
#define kSigBeginVerifiedStr			LSCL(@"*** BEGIN PGP VERIFIED MESSAGE ***\n", @"Beging Verified")
#define kSigEndVerifiedStr				LSCL(@"*** END PGP VERIFIED MESSAGE ***\n", @"End Verified")
#define kSigBeginDecryptStr				LSCL(@"*** BEGIN PGP DECRYPTED/VERIFIED MESSAGE ***\n", @"Begin Decrypted")
#define kSigEndDecryptStr				LSCL(@"\n*** END PGP DECRYPTED/VERIFIED MESSAGE ***\n", @"End Decrypted")

	static PGPError
PGPClientEDEncodeEventHandler(
	PGPContextRef					inContext,
	PGPEvent *						inEvent,
	PGPClientEncodeDecodeDataRef	inRef);
    
    static PGPError
PGPClientEDDecodeEventHandler(
	PGPContextRef					inContext,
	PGPEvent *						inEvent,
	PGPClientEncodeDecodeDataRef	inRef);

	PGPError
PGPclSignerKeyLookup(
	PGPContextRef	context,
    PGPPrefRef		prefRef,
	PGPEvent *		event,
	PGPKeySetRef	defaultKeySet,
	PGPBoolean *	addedKeySetToJob,
	PGPKeyDBRef *	newKeys);

	PGPError
PGPclEncode(
	PGPClientEncodeDecodeDataRef	inRef,
	PGPBoolean						inSign,
	PGPGetPassphraseSettings		inSignSettings,
	PGPGetPassphraseOptions			inSignOptions,
	PGPBoolean						inUseSigningCache,
	PGPBoolean						inClearSign,
	PGPBoolean						inDataIsText,
	PGPBoolean						inEncrypt,
	PGPRecipientSettings			inEncryptSettings,
	PGPRecipientOptions				inEncryptOptions,
	PGPUInt32						inNumDefaultRecipients,
	const PGPRecipientSpec *		inDefaultRecipients,
	PGPBoolean						inArmorOutput,
	const PGPByte *					inData,
	PGPSize							inDataSize,
	PGPByte **						outData,
	PGPSize *						outDataSize)
{
    PGPError 		theError = kPGPError_NoErr;
    PGPKeyDBRef		recipientDB = kInvalidPGPKeyDBRef;
	PGPKeySetRef	recipientSet = kInvalidPGPKeySetRef;
	PGPByte			*convEncryptPassphrase;
	PGPKeyDBObjRef	signingKey = kInvalidPGPKeyDBObjRef;
	PGPByte			*signingBuffer = PGPNewSecureData( inRef->memoryMgr, kPGPcl_MaxPGPUserIDLength, 0);
	PGPSize			signingPasskeySize = 0;
	Boolean			isPassphrase = FALSE;
	PGPBoolean		askUser = TRUE;
	PGPByte			*wrappedData = NULL;
	PGPSize			wrappedDataSize = 0;
    
    PGPValidatePtr(outData);
	*outData = 0;
	PGPValidatePtr(outDataSize);
	*outDataSize = 0;
	PGPValidatePtr(inData);
	PGPValidateParam(PGPClientEncodeDecodeDataRefIsValid(inRef));
	PGPValidateParam(inDataSize > 0);
    
	PGPclUpdateEncodeDecodeData( inRef );
    
    if( IsntPGPError(theError) && inEncrypt )
    {
		PGPKeyDBRef	newKeys = kInvalidPGPKeyDBRef;
		
		theError = PGPclRecipientDialog(inRef->context, inRef->prefRef, inRef->tlsContext, inRef->allKeys,
					inNumDefaultRecipients, inDefaultRecipients, inEncryptOptions,
					inEncryptSettings, &inEncryptSettings, &recipientDB, &newKeys);
        
		if( IsntPGPError( theError ) && PGPKeyDBRefIsValid( recipientDB ) )
		{
			theError = PGPNewKeySet( recipientDB, &recipientSet );
		}
			
		if( IsntPGPError( theError ) && PGPKeyDBRefIsValid( newKeys ) )
		{
			PGPKeySetRef	selectedSet = kInvalidPGPKeySetRef;
			
            theError = kPGPError_UserAbort;

 			theError = PGPclSelectKeysDialog(inRef->context, inRef->prefRef, kPGPclKeySelect_ImportVariation,
						[kUtilImportKeysStr cString], newKeys, inRef->allKeys, &selectedSet);
                        
			if( theError != kPGPError_UserAbort)
            {
                theError = PGPCopyKeys( selectedSet, inRef->allKeys, NULL );
			}
		}
		
        if( IsntPGPError(theError) )
            if( (inEncryptSettings & kPGPRecipientSettingsConvEncrypt) != 0)
            {
                theError = PGPConventionalEncryptionPassphraseDialog(inRef->context,
                            PGPOUIOutputPassphrase(inRef->context, (char **)(&convEncryptPassphrase)),
                            PGPOLastOption(inRef->context));
            } else {
                PGPUInt32	numKeys = 0;
                
                theError = PGPCountKeys(recipientSet, &numKeys);
                if( numKeys == 0 )
                {
                    if( inSign)
                        inEncrypt = FALSE;
                    else
                    {
                        // *****
                        // ***** PGPThrowPGPError_(kPGPError_UserAbort);
                    }
                }
            }
	}

	if( IsntPGPError( theError) && inSign )
	{
		PGPSize			keyIDSize = 0;
		PGPKeyID *		keyID = NULL;
        
		theError = PGPGetPrefData(inRef->prefRef, kPGPPrefDefaultKeyID, &keyIDSize, (void *)(&keyID));
        
		if( IsntPGPError(theError) && (keyIDSize == sizeof(PGPKeyID)) )
		{
			theError = PGPFindKeyByKeyID(inRef->allKeys, keyID, &signingKey);
			if( IsntPGPError(theError) )
			{
				if( inUseSigningCache )
				{
					theError = PGPGetKeyDBObjBooleanProperty(signingKey,
								kPGPKeyProperty_NeedsPassphrase, &askUser);
					if( IsntPGPError(theError) && askUser )
						if( PGPPassphraseIsValid(signingKey, PGPOLastOption(inRef->context)) )
							askUser = FALSE;
				}
			}
		}
        // if the default key is not set then we still want to ask the signing passphrase 
        else if( kPGPError_PrefNotFound == theError )
            theError = kPGPError_NoErr;
                
		if( IsntPGPError( theError) && IsntNull( keyID ) )
    	{
			PGPFreeData(keyID);
			keyID = NULL;
		}
        
        if( IsntPGPError( theError ) && askUser )
		{
			theError = PGPclSigningPassphraseDialog(inRef->context, inRef->prefRef, inRef->allKeys,
						0, inSignOptions, inSignSettings, signingKey,
						(char *)(signingBuffer), &inSignSettings, &signingKey);
                        
			if( theError != kPGPError_KeyUnusableForSignature )
				isPassphrase = TRUE;
			else
			{
				PGPByte		*passkey = NULL;
				
				theError = PGPclReconstitutionDialog(signingKey, inRef->allKeys,
							inRef->tlsContext, inRef->prefRef, &passkey, &signingPasskeySize);
                            
				pgpCopyMemory(passkey, signingBuffer, signingPasskeySize);
				isPassphrase = FALSE;
            }
		}
        
		if( IsntPGPError( theError) && (! inEncrypt) && (inRef->wordWrapWidth > 0))
		{
			PGPIORef			inputIO;
			PGPIORef			outputIO;
			PGPFileOffset		bufSize;
			
			theError = PGPNewMemoryIOFixedBuffer(inRef->memoryMgr, (PGPByte *)(inData),
                                        inDataSize, (PGPMemoryIORef *)(&inputIO));
			theError = PGPNewMemoryIO(inRef->memoryMgr,(PGPMemoryIORef *)(&outputIO));
			theError = pgpWordWrapIO(inputIO, outputIO, inRef->wordWrapWidth, "\n");
			PGPFreeIO( inputIO );
			theError = PGPIOSetPos(outputIO, 0);
			theError = PGPIOGetEOF(outputIO, &bufSize);
			wrappedData = (PGPByte *)(PGPNewData(inRef->memoryMgr, bufSize, kPGPMemoryMgrFlags_None));
			theError = PGPIORead(outputIO, bufSize, wrappedData, &wrappedDataSize);
            PGPFreeIO( outputIO );
		}	
    }
            
    if( IsntPGPError( theError ) )
        theError= PGPclGuaranteeMinimumEntropy(inRef->context);
    
    if( IsntPGPError( theError ) )
        theError = PGPEncode( inRef->context,
                    (wrappedData != 0) ? PGPOInputBuffer(inRef->context, wrappedData,
                        wrappedDataSize) : PGPOInputBuffer(inRef->context, inData, inDataSize),
                    PGPOAllocatedOutputBuffer(inRef->context, (void *)outData, MAX_PGPSize, outDataSize),
                    PGPOEventHandler(inRef->context, (PGPEventHandlerProcPtr)(PGPClientEDEncodeEventHandler), inRef),
                    PGPOSendNullEvents(inRef->context, 100),
                    PGPOForYourEyesOnly(inRef->context, (inEncryptSettings & kPGPRecipientSettingsFYEO) != 0),
                    ([inRef->commentString cStringLength] != 0) ? PGPOCommentString(inRef->context,
                        [inRef->commentString cString]) : PGPONullOption(inRef->context),
                    PGPOVersionString(inRef->context, pgpVersionHeaderString()),
                    (inEncrypt && PGPKeySetRefIsValid(recipientSet)) ?
                        PGPOEncryptToKeySet(inRef->context, recipientSet) : PGPONullOption(inRef->context),
                    (inRef->allowedAlgorithms != 0) ? PGPOPreferredAlgorithms(inRef->context,
                        inRef->allowedAlgorithms, inRef->numAlgorithms) : PGPONullOption(inRef->context),
                    (inEncrypt && ((inEncryptSettings & kPGPRecipientSettingsConvEncrypt) != 0)) ?
                        PGPOConventionalEncrypt(inRef->context, PGPOPassphrase(inRef->context,
                        convEncryptPassphrase), PGPOLastOption(inRef->context)) :
                        PGPONullOption(inRef->context),
                    (inEncrypt && ((inEncryptSettings & kPGPRecipientSettingsConvEncrypt) != 0)) ?
                        PGPOCipherAlgorithm(inRef->context, inRef->preferredAlgorithm) :
                        PGPONullOption(inRef->context),
                    (inSign) ? PGPOSignWithKey(inRef->context, signingKey,
                        (askUser) ? ((isPassphrase) ? PGPOPassphrase(inRef->context,
                        (char *)(signingBuffer)) : PGPOPasskeyBuffer(inRef->context, signingBuffer,
                        signingPasskeySize)) : PGPONullOption(inRef->context),
                        (askUser) ? PGPOCachePassphrase(inRef->context,
                        (inRef->cacheType == kPGPPrefCacheTypeLogon) ? kPGPMaxTimeInterval :
                        inRef->cacheSeconds, inRef->cacheGlobal) : PGPONullOption(inRef->context), 
                        PGPOLastOption(inRef->context)) : PGPONullOption(inRef->context),
                    PGPOArmorOutput(inRef->context, inArmorOutput),
                    PGPODataIsASCII(inRef->context, inDataIsText),
                    PGPOClearSign(inRef->context, inClearSign),
                    PGPOLastOption(inRef->context));
                
    if( IsntNull( signingBuffer ) )
        PGPFreeData( signingBuffer );
    
    if( IsntNull( wrappedData ) )
        PGPFreeData( wrappedData );
        
    return( theError );
}

	PGPError
PGPclDecode(
	PGPClientEncodeDecodeDataRef	inRef,
	const PGPByte *					inData,
	PGPSize							inDataSize,
	PGPByte **						outData,
	PGPSize *						outDataSize)
{
    PGPError 			theError = kPGPError_NoErr;    
	PGPBoolean			cleanupClientDecodeEventHandler = FALSE;
	PGPByte 			*decodeData = NULL;
	PGPByte				*decryptBuffer = PGPNewSecureData( inRef->memoryMgr, kPGPcl_MaxPGPUserIDLength, 0);
	
	PGPValidatePtr(outData);
	*outData = 0;
	PGPValidatePtr(outDataSize);
	*outDataSize = 0;
	PGPValidatePtr(inData);
	PGPValidateParam(PGPClientEncodeDecodeDataRefIsValid(inRef));
	PGPValidateParam(inDataSize > 0);
	
    PGPclUpdateEncodeDecodeData( inRef );

	decodeData = (PGPByte *)( PGPNewData(inRef->memoryMgr, 1, kPGPMemoryMgrFlags_Clear) );
	inRef->decodeData = &decodeData;
    
	theError = PGPclInitDecodeEventHandlerData( inRef->context, inRef->tlsContext,
				inRef->allKeys, inRef->prefRef, &inRef->clientDecodeEventHandlerData );

	cleanupClientDecodeEventHandler = TRUE;
    if( IsntPGPError( theError ) )
        theError = PGPDecode( inRef->context,
                    PGPOInputBuffer(inRef->context, inData, inDataSize),
                    PGPOEventHandler(inRef->context, (PGPEventHandlerProcPtr)(PGPClientEDDecodeEventHandler), inRef),
                    PGPOSendNullEvents(inRef->context, 100),
                    PGPOKeyDBRef(inRef->context, inRef->allKeys),
                    PGPOSendEventIfKeyFound(inRef->context, TRUE),
                    PGPOPassThroughIfUnrecognized(inRef->context, TRUE),
                    PGPOLastOption(inRef->context));
    
	if( IsntPGPError( theError ) && PGPKeyDBRefIsValid( inRef->clientDecodeEventHandlerData.newKeys ) )
    {
		PGPUInt32		count = 0;
		PGPKeySetRef	selectedSet = kInvalidPGPKeySetRef;
		
		theError = PGPCountKeysInKeyDB( inRef->clientDecodeEventHandlerData.newKeys, &count );
        
		if( IsntPGPError( theError ) && count > 0 )
        {
 			theError = PGPclSelectKeysDialog( inRef->context, inRef->prefRef,
						kPGPclKeySelect_ImportVariation,
                        [kUtilImportKeysStr cString],
                        inRef->clientDecodeEventHandlerData.newKeys,
						inRef->allKeys, &selectedSet );
                        
			if( theError != kPGPError_UserAbort )
                theError = PGPCopyKeys( selectedSet, inRef->allKeys, NULL );
		}
	}
	
	if( (!inRef->sigChecked) || (!inRef->sigVerified) )
        NSRunAlertPanel(kCautionStr, (inRef->sigChecked) ? kUtilBadSignatureStr :
			kUtilSignatureNotVerifiedStr, kOKStr, nil, nil);
	
	if( inRef->fyeo )
		theError = PGPclForYourEyesOnlyDialog(inRef->context, inRef->prefRef, decodeData);
    else {
		*outData = decodeData;
		*outDataSize = strlen(decodeData);
		decodeData = 0;
	}
	
    if( cleanupClientDecodeEventHandler )
		PGPclCleanupDecodeEventHandlerData( &inRef->clientDecodeEventHandlerData );

    if( IsntNull( decryptBuffer ) )
        PGPFreeData( decryptBuffer );

    return( theError );
}

	static PGPError
PGPClientEDEncodeEventHandler(
	PGPContextRef					inContext,
	PGPEvent *						inEvent,
	PGPClientEncodeDecodeDataRef	inRef)
{
	(void) inContext;
	return( kPGPError_NoErr );
}

    static PGPError
PGPClientEDDecodeEventHandler(
	PGPContextRef					inContext,
	PGPEvent *						inEvent,
	PGPClientEncodeDecodeDataRef	inRef)
{
	PGPError	result = kPGPError_NoErr;

    switch (inEvent->type)
    {
        case kPGPEvent_InitialEvent:
        {
            inRef->frame = 0;
            inRef->fyeo = FALSE;
            inRef->sigChecked = TRUE;
            inRef->sigVerified = TRUE;
            inRef->abort = FALSE;
        }
        break;
    
        case kPGPEvent_OutputEvent:
        {
            if( inEvent->data.outputData.forYourEyesOnly && (! inRef->fyeo) )
            {
                PGPByte *	secureData = (PGPByte *)(PGPNewSecureData(
										inRef->memoryMgr,
										strlen((char *)(
											*inRef->decodeData)) + 1,
										kPGPMemoryMgrFlags_None));
			
                strcpy((char *)(secureData), (char *)(*inRef->decodeData));
                PGPFreeData(*inRef->decodeData);
                *inRef->decodeData = secureData;
                inRef->fyeo = TRUE;
            }
        }
        break;
	
        case kPGPEvent_EndLexEvent:
        {
            if( inRef->clientDecodeEventHandlerData.outputBuffer != 0 )
            {
                NSString		*sigString = nil;
                NSString		*prefix = 0;
                NSString 		*suffix = 0;
                PGPSize			newSize = inRef->clientDecodeEventHandlerData.outputBufferSize
                                            + strlen((char *)(*inRef->decodeData)) + 1;
			
                if( inRef->clientDecodeEventHandlerData.signatureDataValid )
                {
                    SignatureStatusInfo	status;
				
                    status.sigData = inRef->clientDecodeEventHandlerData.signatureData;
                    if( !status.sigData.checked)
                        inRef->sigChecked = FALSE;
                        
                    if( !status.sigData.verified)
                        inRef->sigVerified = FALSE;

                    result = PGPGetKeyIDString( &status.sigData.signingKeyID, kPGPKeyIDString_Abbreviated, status.keyIDString );

                    GetSignatureStatusMessage( inRef->context, inRef->prefRef, &status, &sigString );
                    
                    if( inRef->clientDecodeEventHandlerData.sectionType == kPGPAnalyze_Encrypted )
                    {
                        prefix = inRef->beginEncryptedAndSignedMessage;
                        suffix = inRef->endEncryptedAndSignedMessage;
                    } else {
                        prefix = inRef->beginSignedMessage;
                        suffix = inRef->endSignedMessage;
                    }
                    newSize +=	[kSigLabelBeginStr cStringLength] +[sigString cStringLength] +
                                [prefix cStringLength] + [suffix cStringLength];
                }
                result = PGPReallocData(inRef->memoryMgr, (void *)inRef->decodeData, newSize, kPGPMemoryMgrFlags_None);
                if( prefix != 0 )
                {
                    strcat((char *)(*inRef->decodeData), [kSigLabelBeginStr cString]);
                    strcat((char *)(*inRef->decodeData), [sigString cString]);
                    strcat((char *)(*inRef->decodeData), [prefix cString]);
                }
                strcat((char *)(*inRef->decodeData), (char *)(inRef->clientDecodeEventHandlerData.outputBuffer));
                if( suffix != 0 )
                {
                    strcat((char *)(*inRef->decodeData), [suffix cString]);
                }
            }
        }
        break;
        
        default:
        break;
    }
    
    result = PGPclDecodeEventHandler(inContext, inEvent, &inRef->clientDecodeEventHandlerData);
	
	if( IsPGPError(result) )
    {
		inRef->abort = TRUE;
	}
    
	return( result );
}

    PGPError
PGPclInitEncodeDecodeData(
    PGPContextRef					inContext,
    PGPtlsContextRef				inTLSContext,
    PGPKeyDBRef						inKeyDBRef,
    PGPPrefRef						inPrefRef,
    PGPClientEncodeDecodeData		*ioData )
{
    PGPError			theError = kPGPError_NoErr;
	PGPUInt32			numPublicKeys = 0;
	PGPUInt32			numPrivateKeys = 0;
    
    ioData->context = inContext;
    ioData->tlsContext = inTLSContext;
    ioData->allKeys = inKeyDBRef;
    ioData->prefRef = inPrefRef;
	ioData->wordWrapWidth = 0;
    ioData->allowedAlgorithms = 0;
    ioData->frame = 0;
	ioData->beginSignedMessage = kSigBeginVerifiedStr;
    ioData->endSignedMessage = kSigEndVerifiedStr;
	ioData->beginEncryptedAndSignedMessage = kSigBeginDecryptStr;
    ioData->endEncryptedAndSignedMessage = kSigEndDecryptStr;
    
	ioData->memoryMgr = PGPPeekContextMemoryMgr(ioData->context);
    
	theError = CountKeysInDefaultKeyring(ioData->context,
            ioData->prefRef, &numPublicKeys, &numPrivateKeys);
    
	if( numPublicKeys == 0 )
    {
        // *****
		// Warning Alert
    }
	if( numPrivateKeys == 0 )
    {
        // *****
        // Warning Alert
    }
    
    return( theError );
}
                
    void
PGPclUpdateEncodeDecodeData(
    PGPClientEncodeDecodeData 		*ioData )
{
	PGPError				err = kPGPError_NoErr;
	PGPPrefArray			*prefArray = NULL;
	PGPBoolean				wordWrapEnabled = FALSE;
	PGPBoolean				isUpdated = FALSE;
	char					tmpString[256];
	
	PGPGetPrefStringBuffer( ioData->prefRef, kPGPPrefComment, 255, tmpString );
	ioData->commentString = [NSString stringWithUTF8String:tmpString];
	
	(void) PGPGetPrefBoolean(ioData->prefRef, kPGPPrefWordWrapEnable, &wordWrapEnabled);
	if( wordWrapEnabled )
		(void) PGPGetPrefNumber( ioData->prefRef, kPGPPrefWordWrapWidth, &ioData->wordWrapWidth );
		
	ioData->numAlgorithms = 0;
	if( IsntNull( ioData->allowedAlgorithms ) )
	{
		PGPFreeData(ioData->allowedAlgorithms);
		ioData->allowedAlgorithms = NULL;
	}
	
	if( IsntPGPError( PGPGetPrefArray( ioData->prefRef, kPGPPrefAllowedAlgorithmsList, &prefArray ) ) )
	{
		ioData->allowedAlgorithms = (PGPCipherAlgorithm *) PGPNewData( ioData->memoryMgr,
					( prefArray->numElements + 2 ) * sizeof( PGPCipherAlgorithm ), 0);
		if( IsntNull( ioData->allowedAlgorithms ) )
		{
			PGPBoolean		aes256 = FALSE;
			PGPUInt32		algIndex = 0;

			ioData->numAlgorithms = prefArray->numElements;
			for( algIndex = 0; algIndex < ioData->numAlgorithms; algIndex++ )
			{
				ioData->allowedAlgorithms[algIndex] = (PGPCipherAlgorithm) prefArray->elements[algIndex].data;
				if( ioData->allowedAlgorithms[algIndex] == kPGPCipherAlgorithm_AES256 )
					aes256 = TRUE;
			}
			if( aes256 )
			{
				ioData->allowedAlgorithms[ioData->numAlgorithms++] = kPGPCipherAlgorithm_AES128;
				ioData->allowedAlgorithms[ioData->numAlgorithms++] = kPGPCipherAlgorithm_AES192;
			}
		}		
		(void) PGPFreePrefArray( prefArray );
	}
	
	err = PGPGetPrefNumber( ioData->prefRef, kPGPPrefPreferredAlgorithm, (PGPUInt32 *)(&ioData->preferredAlgorithm) );
	if( IsPGPError( err ) )
		ioData->preferredAlgorithm = kPGPCipherAlgorithm_AES256;

	(void) PGPGetPrefNumber( ioData->prefRef, kPGPPrefCacheType, &ioData->cacheType );
	(void) PGPGetPrefNumber( ioData->prefRef, kPGPPrefCacheSeconds, &ioData->cacheSeconds );
	(void) PGPGetPrefBoolean( ioData->prefRef, kPGPPrefShareCache, &ioData->cacheGlobal );
	(void) PGPKeyDBIsUpdated( ioData->allKeys, &isUpdated );
}
	                
    void
PGPclCleanupEncodeDecodeData(
    PGPClientEncodeDecodeData 		*ioData )
{
    ioData->beginSignedMessage = nil;
    ioData->endSignedMessage = nil;
	ioData->beginEncryptedAndSignedMessage = nil;
    ioData->endEncryptedAndSignedMessage = nil;

    if( ioData->allowedAlgorithms != 0)
        PGPFreeData(ioData->allowedAlgorithms);
}

	PGPError
PGPclInitEncodeDecode(
	PGPContextRef					inContext,
	PGPtlsContextRef				inTLSContext,
    PGPKeyDBRef						inKeyDBRef,
    PGPPrefRef						inPrefRef,    
	PGPClientEncodeDecodeData		**outRef)
{
	PGPError			theError = kPGPError_NoErr;

	PGPValidatePtr(outRef);
	*outRef = 0;
	PGPValidateParam(PGPtlsContextRefIsValid(inTLSContext));
	PGPValidateParam(PGPContextRefIsValid(inContext));
    
    *outRef = (PGPClientEncodeDecodeData *)PGPNewData( PGPPeekContextMemoryMgr(inContext),
            sizeof( PGPClientEncodeDecodeData ), kPGPMemoryMgrFlags_None );

    theError = PGPclInitEncodeDecodeData( inContext, inTLSContext, inKeyDBRef, inPrefRef, *outRef );
    
    return( theError );
}

	void
PGPclCleanupEncodeDecode(
	PGPClientEncodeDecodeData		*inRef)
{
	if( PGPClientEncodeDecodeDataRefIsValid(inRef) )
       	PGPFreeData( inRef );
}

	PGPError
PGPclDecodeEventHandler(
	PGPContextRef 					context,
	struct PGPEvent 				*event,
	PGPClientDecodeEventHandlerData *data )
{
	PGPError	err = kPGPError_NoErr;
	
	PGPValidatePtr( data );
	PGPValidateParam( data->context == context );
	
	switch( event->type )
	{
		case kPGPEvent_BeginLexEvent:
		{
			pgpAssert( IsNull( data->outputBuffer ) );
			pgpAssert( data->outputBufferSize == 0 );
			
			pgpClearMemory( &data->signatureData, 
						sizeof( data->signatureData ) );
						
			data->signatureDataValid = FALSE;
			data->conventional = FALSE;
			
			break;
		}
		
		case kPGPEvent_EndLexEvent:
		{
			if( IsntNull( data->outputBuffer ) )
				PGPFreeData( data->outputBuffer );
			if( PGPKeySetRefIsValid(data->recipientKeySet) )
				PGPFreeKeySet( data->recipientKeySet );
			if( IsntNull( data->recipientKeyIDList ) )
				PGPFreeData( data->recipientKeyIDList );
			
			data->recipientKeyIDList = NULL;
			data->recipientKeySet = kInvalidPGPKeySetRef;
			data->outputBuffer 		= NULL;
			data->outputBufferSize	= 0;
			
			break;
		}
		
		case kPGPEvent_AnalyzeEvent:
		{
			data->sectionType = event->data.analyzeData.sectionType;
			break;
		}
			
		case kPGPEvent_OutputEvent:
		{
			err = PGPAddJobOptions( event->job,
						PGPOAllocatedOutputBuffer( context, (void *)&data->outputBuffer,
                            MAX_PGPSize, &data->outputBufferSize ),
						PGPOLastOption( context ) );
			break;
		}
		
		case kPGPEvent_PassphraseEvent:
		{
			char	*passphrase = NULL;
			data->split 		= FALSE;
			data->decryptionKey	= kInvalidPGPKeyDBObjRef;
			
			if( event->data.passphraseData.fConventional )
			{
				if( data->conventional )
				{
                    NSRunAlertPanel(kCautionStr, kUtilIncorrectPassphraseStr, kOKStr, nil, nil);
				}
				err = PGPConventionalDecryptionPassphraseDialog( context,
							data->prompt[0] != 0 ?
								PGPOUIDialogPrompt( context, data->prompt ) :
								PGPONullOption( context ),
							PGPOUIOutputPassphrase( context, &passphrase ),
							PGPOLastOption( context ) );
			}
			else
			{
				PGPKeyDBRef	newKeys = kInvalidPGPKeyDBRef;
				
				err = PGPclDecryptionPassphraseDialog( context,
							data->tlsContext, data->prefRef,
							data->prompt[0] != 0 ? data->prompt : NULL,
							data->recipientKeySet,
							data->recipientKeyIDList, data->recipientKeyIDCount,
							kInvalidPGPOptionListRef, &passphrase,
							&data->decryptionKey, &newKeys );
                            
				if( IsntPGPError( err ) && PGPKeyDBRefIsValid( newKeys ) )
				{
					if( PGPKeyDBRefIsValid( data->newKeys ) )
					{
						PGPKeySetRef	newKeySet = kInvalidPGPKeySetRef;
						
						err = PGPNewKeySet( newKeys, &newKeySet );
						if( IsntPGPError( err ) )
						{
							err = PGPCopyKeys( newKeySet, data->newKeys, NULL );
							
							PGPFreeKeySet( newKeySet );
						}
					}
				}
			}
			
			data->conventional = event->data.passphraseData.fConventional;

			if( IsntPGPError( err ) )
			{
				err = PGPAddJobOptions( event->job,
							PGPOPassphrase( context, passphrase ),
							( PGPKeyDBObjRefIsValid( data->decryptionKey ) &&
								( data->cacheType != kPGPPrefCacheTypeNone ) ) ?
								PGPOCachePassphrase( context, ( data->cacheType == kPGPPrefCacheTypeLogon ) ?
									kPGPMaxTimeInterval : data->cacheSeconds, data->cacheGlobal ) :
								PGPONullOption( context ),
							PGPOLastOption( context ) );
				
				PGPFreeData( passphrase );
			}
			else if( err == kPGPError_KeyUnusableForDecryption )
			{
				PGPByte		*passKey = NULL;
				PGPSize		passKeySize = 0;
				
				data->split	= TRUE;
				
 				err = PGPclReconstitutionDialog( data->decryptionKey,
                            data->allKeys, kInvalidPGPtlsContextRef, data->prefRef,
							&passKey, &passKeySize);

				if( IsntPGPError( err ) )
				{
					pgpAssert( passKey != NULL );
					pgpAssert( passKeySize != 0 );
					
					err = PGPAddJobOptions( event->job,
							PGPOPasskeyBuffer( context, passKey, passKeySize ),
							( data->cacheType != kPGPPrefCacheTypeNone ) ?
								PGPOCachePassphrase( context, ( data->cacheType == kPGPPrefCacheTypeLogon ) ?
									kPGPMaxTimeInterval : data->cacheSeconds, data->cacheGlobal ) :
								PGPONullOption( context ),
							PGPOLastOption( context ) );
								
					PGPFreeData( passKey );
				}
			}
			
			break;
		}
	
		case kPGPEvent_RecipientsEvent:
		{
			pgpAssert( ! PGPKeySetRefIsValid( data->recipientKeySet ) );
			pgpAssert( IsNull( data->recipientKeyIDList ) );
			pgpAssert( data->recipientKeyIDCount == 0 );

			data->recipientKeySet = event->data.recipientsData.recipientSet;
			if( PGPKeySetRefIsValid( data->recipientKeySet ) )
			{
				err = PGPIncKeySetRefCount( data->recipientKeySet );
			}
			
			if( IsntPGPError( err ) &&
				IsntNull( event->data.recipientsData.keyIDArray ) )
			{
				PGPSize	dataSize = 0;
				
				data->recipientKeyIDCount = event->data.recipientsData.keyCount;
				dataSize = data->recipientKeyIDCount * sizeof( PGPKeyID );
				
				data->recipientKeyIDList = (PGPKeyID *) PGPNewData(
									PGPPeekContextMemoryMgr( data->context ), dataSize, 0 );
				if( IsntNull( data->recipientKeyIDList ) )
				{
					pgpCopyMemory( event->data.recipientsData.keyIDArray, data->recipientKeyIDList, dataSize );
				}
				else
				{
					err = kPGPError_OutOfMemory;
				}
			}
			
			break;
		}
		
		case kPGPEvent_SignatureEvent:
		{
			PGPBoolean		addedKeySetToJob = FALSE;
			PGPKeyDBRef		newKeys = kInvalidPGPKeyDBRef;

			err = PGPclSignerKeyLookup( context, data->prefRef, event,
                    kInvalidPGPKeySetRef, &addedKeySetToJob, &newKeys);
			if( IsntPGPError( err ) )
			{
				if( addedKeySetToJob )
				{
                    if( !PGPKeyDBRefIsValid( data->newKeys ) )
                        err = PGPNewKeyDB(context, &data->newKeys);
                        
					if( PGPKeyDBRefIsValid( data->newKeys ) )
					{
						PGPKeySetRef	newKeySet;
						
						err = PGPNewKeySet( newKeys, &newKeySet );
						if( IsntPGPError( err ) )
						{
							err = PGPCopyKeys( newKeySet, data->newKeys, NULL );
							PGPFreeKeySet( newKeySet );
						}
					}
				}
				else
				{
					data->signatureData 		= event->data.signatureData;
					data->signatureDataValid 	= TRUE;
				}
			}

			break;
		}

		case kPGPEvent_KeyFoundEvent:
		{
			PGPUInt32	numKeys = 0;
			
			err = PGPCountKeysInKeyDB( event->data.keyFoundData.keyDB, &numKeys );
			if( IsntPGPError( err ) && numKeys > 0 )
			{
				if( !PGPKeyDBRefIsValid( data->newKeys ) )
					err = PGPNewKeyDB(context, &data->newKeys);
				
				if( IsntPGPError( err ) )
				{
					PGPKeySetRef	keySet = kInvalidPGPKeySetRef;
				
					err = PGPNewKeySet( event->data.keyFoundData.keyDB, &keySet );
					if( IsntPGPError( err ) )
					{
						err = PGPCopyKeys( keySet, data->newKeys, NULL );
						PGPFreeKeySet( keySet );
					}
				}
			}
			break;
		}
		
		default:
			break;
	}
	
	return( err );
}
	PGPError
PGPclSignerKeyLookup(
	PGPContextRef	context,
    PGPPrefRef		prefRef,
	PGPEvent *		event,
	PGPKeySetRef	defaultKeySet,
	PGPBoolean *	addedKeySetToJob,
	PGPKeyDBRef *	newKeys)
{
	PGPError		result = kPGPError_NoErr;
	PGPKeyDBRef		signerKeyDB = kInvalidPGPKeyDBRef;
	PGPUInt32		count = 0;
	PGPBoolean		sync = FALSE;

	// Validation
	PGPValidatePtr(event);
	PGPValidateParam(event->type == kPGPEvent_SignatureEvent);
	PGPValidatePtr(addedKeySetToJob);
	*addedKeySetToJob = FALSE;
	if( newKeys != NULL )
    {
		PGPValidatePtr(newKeys);
		*newKeys = kInvalidPGPKeyDBRef;
	}
	
	// Lookup key if necessary
	if( !PGPKeyDBObjRefIsValid(event->data.signatureData.signingKey))
    {
		if( IsntPGPError(result) )
			result = PGPGetPrefBoolean(	prefRef, kPGPPrefKeyServerSyncOnVerify, &sync);
		
		if( IsntPGPError(result) && sync )
        {
			result = PGPclGetKeyIDFromServer( context, kInvalidPGPtlsContextRef, prefRef,
                                              &event->data.signatureData.signingKeyID, &signerKeyDB);
                                              
			if( IsntPGPError(result) && PGPKeyDBRefIsValid(signerKeyDB) )
                result = PGPCountKeysInKeyDB(signerKeyDB, &count);
                
			if( IsntPGPError(result) && (count > 0) )
            {
				result = PGPAddJobOptions(	event->job,
                    PGPOKeyDBRef( context, signerKeyDB ), PGPOLastOption(context) );
                    
				if( IsntPGPError(result) )
                {
					*addedKeySetToJob = TRUE;
					
					if( PGPKeySetRefIsValid( defaultKeySet ) )
                    {						
                        PGPKeySetRef		selectedKeySet = kInvalidPGPKeySetRef;
                        
                        result = PGPclSelectKeysDialog(
                                    context, prefRef,
                                    kPGPclKeySelect_ImportVariation,							
                                    [kUtilImportKeysStr cString],
                                    signerKeyDB,
                                    PGPPeekKeySetKeyDB( defaultKeySet ),
                                    &selectedKeySet);
                                    
                        if( IsntPGPError(result) )
                        {
                            result = PGPCopyKeys( selectedKeySet, PGPPeekKeySetKeyDB( defaultKeySet ), NULL );
                            
                            PGPFreeKeySet(selectedKeySet);
                        }
					}
					
					// Return the keyset
					if( IsntNull(newKeys) )
                    {
						*newKeys = signerKeyDB;
						signerKeyDB = kInvalidPGPKeyDBRef;
					}
				}
			}
			if( PGPKeyDBRefIsValid(signerKeyDB) )
				PGPFreeKeyDB(signerKeyDB);
		}
	}
	
	return( result );
}

    PGPError        
PGPclInitDecodeEventHandlerData(
    PGPContextRef						context,
    PGPtlsContextRef					tlsContext,
    PGPKeyDBRef							allKeys,
    PGPPrefRef							prefRef,
    PGPClientDecodeEventHandlerData 	*data)
{       
	PGPError        err = kPGPError_NoErr;

	PGPValidateParam( PGPContextRefIsValid( context ) );
	PGPValidatePtr( data );
               
	pgpClearMemory( data, sizeof( *data ) );

	data->context           = context;
	data->tlsContext        = tlsContext;
	data->allKeys           = allKeys;
	data->prefRef			= prefRef;
	err = PGPGetPrefNumber( prefRef, kPGPPrefCacheType, &(data->cacheType) );
	if( IsntPGPError( err ) )
        err = PGPGetPrefNumber( prefRef, kPGPPrefCacheSeconds, &(data->cacheSeconds) );
	if( IsntPGPError( err ) )
        err = PGPGetPrefBoolean( prefRef, kPGPPrefShareCache, &(data->cacheGlobal) );
	
    return( err );
}

    void
PGPclCleanupDecodeEventHandlerData(
	PGPClientDecodeEventHandlerData *data)
{
	if( IsntNull( data ) )
	{
        if( IsntNull( data->outputBuffer ) )
            PGPFreeData( data->outputBuffer );

        if( PGPKeySetRefIsValid( data->recipientKeySet ) )
            (void) PGPFreeKeySet( data->recipientKeySet );

        if( PGPKeyDBRefIsValid( data->newKeys ) )
            (void) PGPFreeKeyDB( data->newKeys );

        if( IsntNull( data->recipientKeyIDList ) )
            PGPFreeData( data->recipientKeyIDList );

        pgpClearMemory( data, sizeof( *data ) );
	}
}

	PGPError
PGPclRSASupported(
	PGPBoolean *	hasRSA)
{
	PGPError					result = kPGPError_NoErr;
	PGPUInt32					numAlgs = 0;
	PGPUInt32					index = 0;
	PGPPublicKeyAlgorithmInfo	info;
	
	PGPValidatePtr(hasRSA);
	*hasRSA = FALSE;
	
	result = PGPCountPublicKeyAlgorithms( &numAlgs );
	while( IsntPGPError(result) && (index < numAlgs) )
    {
		result = PGPGetIndexedPublicKeyAlgorithmInfo( index, &info );
		if( IsntPGPError(result) )
        {
			if((info.algID == kPGPPublicKeyAlgorithm_RSA)
			|| (info.algID == kPGPPublicKeyAlgorithm_RSAEncryptOnly)
			|| (info.algID == kPGPPublicKeyAlgorithm_RSASignOnly))
            {
				*hasRSA = TRUE;
				break;
			}
		}
		index++;
	}
    
	return( result );
}

/*__Editor_settings____

	Local Variables:
	tab-width: 4
	End:
	vi: ts=4 sw=4
	vim: si
_____________________*/
