/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation
    All rights reserved.

    $Id: CPGPLicenseAuth.h,v 1.7 2002/11/22 08:44:36 jeffc Exp $
____________________________________________________________________________*/

#import <Cocoa/Cocoa.h>
#import "PGPConsole.h"
#import "pgpLicenseNumber.h"

@class CPGPLicenseAuthFieldEditor;

@interface CPGPLicenseAuth : NSObject
{
	IBOutlet NSWindow*				mWindow;
	IBOutlet NSView*				mTopView;
	IBOutlet NSView*				mLAView;
	IBOutlet NSView*				mBottomView;
	IBOutlet NSTextField*			mName;
	IBOutlet NSTextField*			mOrganization;
	IBOutlet NSTextField*			mLN1;
	IBOutlet NSTextField*			mLN2;
	IBOutlet NSTextField*			mLN3;
	IBOutlet NSTextField*			mLN4;
	IBOutlet NSTextField*			mLN5;
	IBOutlet NSTextField*			mLN6;
	IBOutlet NSTextField*			mDisclosureTitle;
	IBOutlet NSButton*				mDisclosureButton;
	IBOutlet NSTextView*			mLA;
	IBOutlet NSButton*				mLater;
	IBOutlet NSButton*				mPurchaseNow;
	IBOutlet NSButton*				mAuthorize;
	
	IBOutlet NSWindow*				mNetWindow;
	IBOutlet NSTextField*			mNetName;
	IBOutlet NSTextField*			mNetOrganization;
	IBOutlet NSTextField*			mNetLN;
	IBOutlet NSProgressIndicator*	mNetProgress;
	IBOutlet NSTextField*			mNetStatus;
	
	NSImage*						mTriangleUp;
	NSImage*						mTriangleDown;
	CPGPLicenseAuthFieldEditor*		mLNFieldEditor;
	
	BOOL							mInited;
	BOOL							mLoadedNib;
}

+ (CPGPLicenseAuth*)sharedLicenseAuth;

- (id)init;
- (void)loadNib:(NSString*)nibFile;

+ (BOOL)isAuthorized;
+ (BOOL)evaluation;
+ (BOOL)diskEnabled;
+ (BOOL)mailEnabled;
+ (BOOL)enterpriseEnabled;
+ (BOOL)authorize:(NSString*)ln licenseAuth:(NSString*)la name:(NSString*)name organization:(NSString*)organization;

- (void)displayLicenseEntryDialog:(BOOL)quitOnDeferral;
- (void)resetLicenseEntryDialog;
- (void)saveLicenseNumber:(NSString*)ln licenseAuth:(NSString*)la name:(NSString*)name organization:(NSString*)organization;
- (void)rejectLicenseAuthorization:(NSString*)caption withMessage:(NSString*)message;
- (void)rejectLicenseNumber:(NSString*)caption withMessage:(NSString*)message;
- (void)finish;

- (NSWindow*)window;

- (NSString*)name;
- (NSString*)organization;
- (NSString*)ln;
- (NSString*)la;

+ (PGPError)prefLNRef:(PGPLicenseNumberRef*)lnRef;
+ (PGPError)prefLARef:(PGPLicenseAuthorizationRef*)laRef;
+ (PGPInt32)daysLeft;
+ (PGPUInt8)flags;

+ (NSString*)prefName;
+ (NSString*)prefOrganization;
+ (NSString*)prefLN;
+ (NSString*)prefLA;
+ (NSString*)getPrefString:(PGPPrefIndex)prefID;

- (IBAction)toggleLicenseAuthorizationEntry:(id)sender;
- (IBAction)showLicenseAuthorizationEntry:(id)sender;
- (IBAction)hideLicenseAuthorizationEntry:(id)sender;
- (IBAction)later:(id)sender;
- (IBAction)quit:(id)sender;
- (IBAction)purchaseNow:(id)sender;
- (IBAction)authorize:(id)sender;
- (IBAction)authorizeLA:(id)sender;
- (IBAction)authorizeLN:(id)sender;

/* Window delegate methods */
- (BOOL)windowShouldClose:(id)sender;
- (id)windowWillReturnFieldEditor:(NSWindow*)sender toObject:(id)object;

/* CPGPLicenseAuthFieldEditor delegate methods */
- (BOOL)shouldInsertText:(NSString*)text;
- (BOOL)shouldDeleteBackward:(id)sender;

- (void)dealloc;

@end

@interface CPGPLicenseAuthFieldEditor : NSTextView
{
	NSCharacterSet*			mCharSet;
	id						mOwner;
	
	BOOL					mInited;
}

- (id)init;

- (id)owner;
- (void)setOwner:(id)owner;

- (void)insertText:(NSString*)text;
- (void)paste:(id)sender;
- (void)deleteBackward:(id)sender;

- (void)dealloc;

@end
