/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation
    All rights reserved.

    $Id: CPGPToolbar.m,v 1.7 2002/08/06 20:09:21 dallen Exp $
____________________________________________________________________________*/

#import "CPGPToolbar.h"
#import "PGPConsole.h"
#import <PGPclient/CImageButton.h>

#define kPGPToolbar_HorizontalPadding		36
#define kPGPToolbar_VerticalPadding			12
#define kPGPToolbar_HorizontalSpacing		12

enum
{
	kPGPToolbarStyle_EvenlySpaced		= 1,
	kPGPToolbarStyle_LeftAligned,
	kPGPToolbarStyle_RightAligned,
	kPGPToolbarStyle_Centered
};

@implementation CPGPToolbar

- (id)init
{
    if (!mInited)
    {
        [super init];
        
        mInited = YES;
    }
    
    return self;
}

- (NSArray*)buttons
{
    return [self subviews];
}

- (void)addButton:(CImageButton*)button
{
    NSRect			frame = [self frame];
	
	frame.size.width -= 2 * kPGPToolbar_HorizontalPadding;
	frame.size.height -= 2 * kPGPToolbar_VerticalPadding;
	
    if (button)
    {
		[button setAutoresizingMask:NSViewNotSizable];
		[self addSubview:button];

		[self resizeSubviewsWithOldSize:NSMakeSize(NSWidth([self frame]) - (2 * kPGPToolbar_HorizontalPadding), NSHeight([self frame]) - (2 * kPGPToolbar_VerticalPadding))];
    }
	
	[self determineMinSize];
	[self determineMaxSize];
}

- (NSSize)maxSize
{
	return mMaxSize;
}

- (NSSize)minSize
{
	return mMinSize;
}

- (void)determineMinSize
{
	NSEnumerator*	enumerator;
	NSView*			subview;
	CImageButton*	button;
	
	mMinSize = NSZeroSize;
	
	enumerator = [[self subviews] objectEnumerator];
	
	while (subview = [enumerator nextObject])
	{
		if ([subview isKindOfClass:[CImageButton class]])
			button = (CImageButton*) subview;
		else
			button = nil;
		
		if (button)
		{
			if ([button minButtonSize].height > mMinSize.height)
				mMinSize.height = [button minButtonSize].height;
				
			mMinSize.width += [button minButtonSize].width;
		}
	}
	
	mMinSize.height += 2 * kPGPToolbar_VerticalPadding;
	mMinSize.width += 2 * kPGPToolbar_HorizontalPadding;
}

- (void)determineMaxSize
{
	NSEnumerator*	enumerator;
	NSView*			subview;
	CImageButton*	button;
	
	mMaxSize = NSZeroSize;
	
	enumerator = [[self subviews] objectEnumerator];
	
	while (subview = [enumerator nextObject])
	{
		if ([subview isKindOfClass:[CImageButton class]])
			button = (CImageButton*) subview;
		else
			button = nil;
		
		if (button)
		{
			if ([button maxButtonSize].height > mMaxSize.height)
				mMaxSize.height = [button maxButtonSize].height;
				
			mMaxSize.width += [button maxButtonSize].width;
		}
	}
	
	mMaxSize.height += 2 * kPGPToolbar_VerticalPadding;
	mMaxSize.width = 10000;
}

- (void)resizeSubviewsWithOldSize:(NSSize)oldFrameSize
{
    NSEnumerator*		enumerator;
    CImageButton*		button;
    NSRect				frame;
	NSSize				size;
	unsigned			numButtons;
	unsigned			buttonIndex;
	unsigned			multiplier;
	float				xOffset = 0;
	float				containerOffset;
	NSString*			tooltip;

    enumerator = [[self subviews] objectEnumerator];
	numButtons = [[self subviews] count];
	buttonIndex = 1;
	multiplier = 0;

    while (button = [enumerator nextObject])
    {
		size = [self frame].size;
		//size.width -= 2 * kPGPToolbar_HorizontalPadding;
		size.height -= 2 * kPGPToolbar_VerticalPadding;
		
		[button resizeWithOldSuperviewSize:size];

		frame = [button frame];
        frame.origin.y = (NSHeight([self frame]) / 2) - (NSHeight(frame) / 2);
		
		switch (mButtonAlignment)
		{
			case kPGPToolbarStyle_LeftAligned:
				if (xOffset == 0)
					xOffset = kPGPToolbar_HorizontalSpacing;
				
				frame.origin.x = xOffset;
				xOffset += kPGPToolbar_HorizontalSpacing + NSWidth(frame);
				break;
			case kPGPToolbarStyle_RightAligned:
				if (xOffset == 0)
					xOffset = NSWidth([self frame]) - kPGPToolbar_HorizontalSpacing - NSWidth(frame);
				
				frame.origin.x = xOffset;
				xOffset -= kPGPToolbar_HorizontalSpacing + NSWidth(frame);
				break;
			case kPGPToolbarStyle_Centered:
				if (xOffset == 0)
					xOffset = (NSWidth([self frame]) / 2) - ((numButtons * (NSWidth(frame) + kPGPToolbar_HorizontalSpacing)) / 2);
				
				frame.origin.x = xOffset;
				xOffset += kPGPToolbar_HorizontalSpacing + NSWidth(frame);
				break;
			case kPGPToolbarStyle_EvenlySpaced:	/* Fall through */
			default:
				containerOffset = size.width / numButtons;
				containerOffset *= buttonIndex;
				frame.origin.x = containerOffset - ((size.width / numButtons) / 2) - (NSWidth(frame) / 2);
				break;
		}
		
		tooltip = [button toolTip];
		[button setToolTip:nil];
        [button setFrame:frame];
		[button setToolTip:tooltip];
		
		++buttonIndex;
		++multiplier;
    }
}

- (void)setFrame:(NSRect)frame
{
	NSSize				minSize;
	NSSize				maxSize;

	minSize = [self minSize];
	maxSize = [self maxSize];
    
    if (NSHeight(frame) < minSize.height)
        frame.size.height = minSize.height;
    
    if (NSWidth(frame) < minSize.width)
        frame.size.width = minSize.width;
	
	if (NSHeight(frame) > maxSize.height)
		frame.size.height = maxSize.height;
	
	if (NSWidth(frame) > maxSize.width)
		frame.size.width = maxSize.width;

    [super setFrame:frame];
}

- (void)awakeFromNib
{
	NSData*					data;
	NSUserDefaults*			defaults = [NSUserDefaults standardUserDefaults];
	NSMutableDictionary*	dict = [[defaults persistentDomainForName:kPGPPrefsDomain] mutableCopy];

	if (data = [dict objectForKey:kPGPAppDefault_ToolbarStyle])
		[data getBytes:&mButtonAlignment length:sizeof(PGPUInt32)];
	else
	{
		mButtonAlignment = kPGPToolbarStyle_LeftAligned;
		[dict setObject:[NSData dataWithBytes:&mButtonAlignment length:sizeof(PGPUInt32)] forKey:kPGPAppDefault_ToolbarStyle];
		[defaults removePersistentDomainForName:kPGPPrefsDomain];
		[defaults setPersistentDomain:dict forName:kPGPPrefsDomain];
		[defaults synchronize];
	}
	
	[dict release];
}

- (void)dealloc
{
    [super dealloc];
}

@end
