/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation
    All rights reserved.

    $Id: PGPAppController.h,v 1.51 2002/11/11 10:17:14 jeffc Exp $
____________________________________________________________________________*/

#import "PGPConsole.h"

#include <Carbon/Carbon.h>

#define kPGPAppInitialDictionarySize		5
#define kPGPAppEntropyCollectionInterval	2
#define kPGPAppDocumentFlusherInterval		2
#define kPGPAppKeyDBCheckerInterval			1

#define kPGPAppBundlesKey				@"Bundles"
#define kPGPAppMenusKey					@"Menus"

@class NSString, NSMutableDictionary;
@class PGPInterfaceController, PGPBundleManager, PGPPrefsManager, CPGPDockMenu, CPGPKeyringSelection, CPGPLicenseAuth;

@interface PGPAppController : NSObject <PGPController>
{
    NSMutableDictionary*	pgpAttributes;
    PGPInterfaceController*	pgpInterface;
    PGPBundleManager*		pgpBundleManager;
    PGPPrefsManager*		pgpPrefsManager;
    
	PGPContextRef			pgpContext;
	PGPtlsContextRef		tlsContext;
	PGPKeyDBRef				mMainKeyDB;
    NSTimer*				entropyCollectionTimer;
    NSTimer*				mDocumentFlusher;
	NSTimer*				mKeyDBChecker;
	NSMutableArray*			mLaunchedDocuments;
    Point					lastMouseLocation;
	char					mPubKeyringPath[PATH_MAX];
	char					mSecKeyringPath[PATH_MAX];

	BOOL					mAppFinishedLaunching;
	BOOL					mHideMainWindow;
	BOOL					mLicenseAuthDisplayed;
	BOOL					mSDKInited;
	BOOL					mBundlesLoaded;
	BOOL					mFinishedInitialization;
	BOOL					mExitAfterDocumentsHandled;
    BOOL					inited;
}

+ (id)appController;

- (id)init;
- (id)initWithInterface:(PGPInterfaceController*)interface;
- (id)initWithBundleManager:(PGPBundleManager*)manager;
- (id)initWithPrefsManager:(PGPPrefsManager*)manager;

- (void)initSDK;
- (void)initPrefs;
- (void)loadBundles;

- (BOOL)finishedInitialization;
- (BOOL)isRunningOnReadOnlyFS;
- (BOOL)checkLicense;

- (void)keyringSelectionFinished:(CPGPKeyringSelection*)keyringSelection;
- (void)licenseAuthFinished:(CPGPLicenseAuth*)licenseAuth;
- (void)licenseAuthDeferred:(CPGPLicenseAuth*)licenseAuth;
- (void)finishedHandlingDocument:(NSString*)file;
- (void)uninstall:(BOOL)quitOnClose;
- (void)displayLicense;
- (void)changeLicense;
- (void)displayExpirationAlertAndExit;
- (void)displayReadOnlyAlertAndExit;
- (void)finishInitialization;
- (void)installEntropyCollectionTimer;
- (void)installDocumentFlusher;
- (void)installKeyDBCheckerTimer;
- (void)registerNotificationHandlers;
- (void)addEntropy:(NSTimer*)timer;
- (void)documentFlusher:(NSTimer*)timer;
- (void)keyDBUpdateChecker:(NSTimer*)timer;
- (void)purgePassphraseCache;

- (NSMutableDictionary*)attributes;
- (void)setAttributes:(NSMutableDictionary*)attributes;

- (PGPInterfaceController*)interface;
- (void)setInterface:(PGPInterfaceController*)interface;

- (PGPBundleManager*)bundleManager;
- (void)setBundleManager:(PGPBundleManager*)manager;

- (PGPPrefsManager*)prefsManager;
- (void)setPrefsManager:(PGPPrefsManager*)manager;

- (PGPContextRef)pgpContext;
- (PGPtlsContextRef)tlsContext;
- (PGPPrefRef)pgpPrefRef;
- (PGPKeyDBRef)mainKeyDB;

- (void)setView:(NSView*)view;
- (void)setCurrentBundle:(id <PGPBundle>)controller passive:(BOOL)passive;

- (BOOL)canDisplayMainWindow;
- (NSWindow*)mainWindow;
- (NSMenu*)mainMenu;
- (NSArray*)menuItems;
- (NSArray*)launchedDocuments;
- (void)addMenu:(NSMenu*)menu withIdentifier:(NSString*)identifier;
- (void)addMenu:(NSMenu*)menu withIdentifier:(NSString*)identifier atIndex:(unsigned)index;
- (void)addMenuItem:(NSMenuItem*)menuItem withIdentifier:(NSString*)identifier;
- (void)addMenuItem:(NSMenuItem*)menuItem withIdentifier:(NSString*)identifier atIndex:(unsigned)index;

- (void)signSelection:(NSPasteboard *)pboard userData:(NSString *)data error:(NSString **)error;
- (void)encryptSelection:(NSPasteboard *)pboard userData:(NSString *)data error:(NSString **)error;
- (void)signEncryptSelection:(NSPasteboard *)pboard userData:(NSString *)data error:(NSString **)error;
- (void)decryptVerifySelection:(NSPasteboard *)pboard userData:(NSString *)data error:(NSString **)error;
- (void)wipeFile:(NSPasteboard *)pboard userData:(NSString *)data error:(NSString **)error;

- (void)signSelection:(NSPasteboard*)pboard;
- (void)encryptSelection:(NSPasteboard*)pboard;
- (void)signEncryptSelection:(NSPasteboard*)pboard;
- (void)decryptVerifySelection:(NSPasteboard*)pboard;
- (void)wipeFile:(NSPasteboard*)pboard;

- (void)awakeFromNib;
- (void)applicationDidFinishLaunching:(NSNotification*)notification;
- (BOOL)application:(NSApplication*)app openFile:(NSString*)filename;
- (NSApplicationTerminateReply)applicationShouldTerminate:(NSApplication*)app;

/* Notification handlers */
- (void)passphraseCacheTypeDidChange:(NSNotification*)notification;
- (void)keyringsDidChange:(NSNotification*)notification;
- (BOOL)applicationShouldHandleReopen:(NSApplication*)app hasVisibleWindows:(BOOL)flag;

- (void)cleanupAndExit;
- (void)dealloc;

@end


@interface PGPScriptCommand : NSScriptCommand
@end
