/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation
    All rights reserved.

    $Id: PGPBundleManager.m,v 1.27 2002/11/21 03:40:37 jeffc Exp $
____________________________________________________________________________*/

#import "PGPBundleManager.h"

@implementation PGPBundleManager

+ (NSArray*)bundlesAtPath:(NSString*)path
{
	NSAutoreleasePool*		pool = [[NSAutoreleasePool alloc] init];
	NSMutableArray*			bundlesArray = [[NSMutableArray alloc] initWithCapacity:4];
	NSArray*				filesAtPath;
	NSString*				fileName;
	NSString*				fullPath;
	int						numFiles;
	int						index;
	
	if (!path)
		return nil;

	fileName = [path stringByStandardizingPath];
	fullPath = [NSString stringWithString:fileName];

	if (filesAtPath = [[NSFileManager defaultManager] directoryContentsAtPath:fullPath])
	{
		numFiles = [filesAtPath count];
		
		for (index = 0; index < numFiles; ++index)
		{
			if ([[[filesAtPath objectAtIndex:index] pathExtension] isEqualToString:@"bundle"])
			{
				fileName = [NSString stringWithString:fullPath];
				fileName = [fileName stringByAppendingPathComponent:[filesAtPath objectAtIndex:index]];
				[bundlesArray addObject:[NSString stringWithString:fileName]];
			}
		}
	}

	if (![bundlesArray count])
	{
		[bundlesArray release];
		[pool release];
		return nil;
	}
	
	[pool release];
	
	return bundlesArray;
}

+ (NSArray*)bundlesAtPaths:(NSArray*)paths
{
	NSMutableArray*		bundlesArray = [[NSMutableArray alloc] initWithCapacity:4];
	NSArray*			bundlesInPath = nil;
	int					numPaths;
	int					index;
	
	if (!paths)
		return nil;
	
	numPaths = [paths count];
	
	for (index = 0; index < numPaths; ++index)
	{
		bundlesInPath = [self bundlesAtPath:[paths objectAtIndex:index]];
		if (bundlesInPath)
		{
			[bundlesArray addObjectsFromArray:bundlesInPath];
			[bundlesInPath release];
		}
	}
	
	if (![bundlesArray count])
	{
		[bundlesArray release];
		return nil;
	}
	
	return bundlesArray;
}

- (id)init
{
    if (!inited)
    {
        [super init];
        
        inited = YES;
    }

	if (!myController)
		PGPLog((@"%s:%d: Warning... myController has not been set.", __FILE__, __LINE__));
	
    if (!bundles)
		bundles = [[NSMutableDictionary alloc] initWithCapacity:kPGPBundleInitialDictionarySize];
    
    if (!bundlePaths)
		bundlePaths = [[NSMutableDictionary alloc] init];
	
	return self;
}

- (id)initWithPaths:(NSArray*)paths
{
	int		numPaths;
	int		index;
    
	if (paths && (numPaths = [paths count]))
	{
		if (bundlePaths)
			bundlePaths = [[NSMutableArray alloc] init];
		
        /*
        For each entry in the paths array, add it to our bundlePaths array.
        Any duplicate entries will be skipped.
        */
		for (index = 0; index < numPaths; ++index)
			[self addBundleAtPath:[paths objectAtIndex:index]];
	}

	return [self init];
}

- (id)initWithController:(id)controller
{
    myController = controller;

    return [self init];
}

- (id)controller
{
    return myController;
}

- (void)setController:(id)controller
{
    if (controller)
        myController = controller;
}

/*
Given a path (i.e @"~/Library/PGP/PGPxxx.bundle"), this method
will determine if the path points to a directory. If so, we add
it to our list of bundle paths. We could do some more extensive
testing on the path, but there is a possibility that we would
want to load a bundle in a path not ending with ".bundle".
*/

- (void)addBundleAtPath:(NSString*)path
{
	int			pathIndex;
	BOOL		isDirectory = FALSE;
	NSString*	fullPath;
	NSString*	tempString;

	if (bundlePaths && path)
	{
        /* Skip the path if we already have it in our list */
		if ((pathIndex = [bundlePaths indexOfObject:path]) == NSNotFound)
        {
            /* Expand any tildes, and evaluate any symlinks */
			fullPath = [path stringByStandardizingPath];
			
			if ([[NSFileManager defaultManager] fileExistsAtPath:fullPath isDirectory:&isDirectory])
			{
                /* If the path points to a directory, add it to our list (making a copy of the string) */
				if (isDirectory)
				{
					tempString = [[NSString alloc] initWithString:fullPath];
					[bundlePaths addObject:tempString];
					[tempString release];
				}
			}
		}
	}
}

/*
This method allows the sender to set the bundlePaths array to
some preassigned array (unless the array is nil).
*/

- (void)setBundlePaths:(NSArray*)paths
{
    if (paths)
	{
		if (bundlePaths)
			[bundlePaths release];
		
        bundlePaths = [paths retain];
	}
}

- (BOOL)loadBundles
{
	if (!mLoadedBundles)
		return [self loadBundlesFromPaths:bundlePaths];
		
	return NO;
}

- (BOOL)loadBundlesFromPaths:(NSArray*)paths
{
	NSAutoreleasePool*		pool = [[NSAutoreleasePool alloc] init];
	NSBundle*				bundle;
	NSString*				bundleName = nil;
	NSString*				bundlePath = nil;
	NSMutableDictionary*	bundleDict = nil;
	NSDictionary*			bundleInfo = nil;
	id <PGPBundle>			bundleController;
	Class					principalClass;
	BOOL					success = TRUE;
	PGPInt32				days;
	int						numBundles;
	int						index;
	
	if (mLoadedBundles)
		return FALSE;
	
	if (!paths)
	{
		[pool release];
		return FALSE;
	}

	numBundles = [paths count];
	
	for (index = 0; index < numBundles; ++index)
	{
		if (bundlePath = [paths objectAtIndex:index])
		{
			bundleName = [bundlePath lastPathComponent];	/* Yields "PGPxxx.bundle" for "/Applications/PGP/PGPxxx.bundle" */
            
            /* Skip the bundle if we already have loaded one by the same name */
			if (![bundles objectForKey:bundleName])
			{
				bundleDict = [[NSMutableDictionary alloc] initWithCapacity:kPGPBundleInitialDictionarySize];
				
                [bundles setObject:bundleDict forKey:bundleName];
                
				if (bundle = [NSBundle bundleWithPath:bundlePath])
				{
					if ([[bundle principalClass] conformsToProtocol:@protocol(PGPBundle)])
					{
						[bundle load];
						
						principalClass = [bundle principalClass];
						
						/* PGPdisk.bundle disabled in freeware */
						if ([[principalClass identifier] isEqualToString:@"PGPdisk.bundle"])
						{
							if (!PGPclIsAuthorized())
							{
								days = PGPclLicenseDaysLeft();
								
								if ((!PGPclPGPdiskEnabled()) || (PGPclPGPdiskEnabled() && ((days + PGP_LICENSEGRACEPERIOD) <= 0)))
									continue;
							}
						}
						
						[[bundles objectForKey:bundleName] setObject:bundle
														   forKey:kPGPBundleInstanceKey];
                    
						bundleInfo = [self bundleInfoForBundle:bundleName];
						[[bundles objectForKey:bundleName] setObject:bundleInfo forKey:kPGPBundleInfoKey];
						[bundleInfo release];
						
						if (bundleController = [self createInstanceForBundle:bundleName])
						{
							[[bundles objectForKey:bundleName] setObject:bundleController forKey:kPGPBundleControllerKey];
							
							[(id)bundleController release];
						}
						else
							[bundles removeObjectForKey:bundleName];
					}
					else
					{
						PGPLog((@"%s:%d: %@ does not conform to PGPBundle", __FILE__, __LINE__,
							NSStringFromClass([bundle principalClass])));
					}
					
					[bundleDict release];
				}
				else
					success = FALSE;
			}
		}
	}
	
	[pool release];
	
	mLoadedBundles = YES;
	
	return success;
}

/*
This method will fill in the BundleInfo section of our main dictionary.
The values are retrieved from the bundle's infoDictionary.
*/
- (NSDictionary*)bundleInfoForBundle:(NSString*)bundle
{
    NSBundle*				bundleInstance;
    NSDictionary*			bundleDict;
    NSMutableDictionary*	bundleInfo;
    
    if (bundle && [bundles objectForKey:bundle])
    {
        if (!(bundleInstance = [[bundles objectForKey:bundle] objectForKey:kPGPBundleInstanceKey]))
            return nil;
            
        bundleDict = [bundleInstance infoDictionary];
        
        bundleInfo = [[NSMutableDictionary alloc] initWithCapacity:kPGPBundleInitialDictionarySize];
        
        [bundleInfo setObject:[bundleInstance bundlePath]
                    forKey:kPGPBundleInfoPathKey];
        [bundleInfo setObject:[NSString stringWithString:[bundleDict objectForKey:kCFBundleVersion]]
                    forKey:kPGPBundleInfoVersionKey];
        [bundleInfo setObject:[NSString stringWithString:[bundleDict objectForKey:kNSPrincipalClass]]
                    forKey:kPGPBundleInfoPrincipalClassName];
    }
    else
        return nil;

    return bundleInfo;
}

- (NSArray*)bundleControllers
{
	NSMutableArray*		controllerArray = [[[NSMutableArray alloc] init] autorelease];
	NSDictionary*		bundleDict;
	NSEnumerator*		bundleEnumerator;
	NSString*			bundleName;
	id					bundleController;
	
	if (bundles)
	{
		bundleEnumerator = [bundles keyEnumerator];
		
		while (bundleName = [bundleEnumerator nextObject])
		{
			bundleDict = [bundles objectForKey:bundleName];
			bundleController = [bundleDict objectForKey:kPGPBundleControllerKey];
																						
			if (bundleController)
				[controllerArray addObject:bundleController];
		}
		
		if ([controllerArray count])
			return controllerArray;
	}
	
	return nil;
}

/*
This method should return an array of NSStrings such as:
PGPxxx.bundle
PGPxxy.bundle
PGPxxz.bundle
*/
- (NSArray*)bundleNames
{
    if (bundles)
        return [bundles allKeys];
        
    return nil;
}

/*
This method will create an instance of a bundle's principal class.
*/
- (id)createInstanceForBundle:(NSString*)name
{
	NSAutoreleasePool*	pool = [[NSAutoreleasePool alloc] init];
	id				classInstance = nil;
    NSString*		className;
	Class			principalClass;
	
    if (name && [bundles objectForKey:name])
    {
        /*
        Once we get the NSString naming the principal class, we need to create a Class
        from it (using NSClassFromString).
        */
        className = [[[bundles objectForKey:name]
                               objectForKey:kPGPBundleInfoKey]
                               objectForKey:kPGPBundleInfoPrincipalClassName];
                                    
        if (className)
        {
            principalClass = NSClassFromString(className);
            
            if (principalClass)
			{
				NS_DURING
					classInstance = [[principalClass alloc] initWithController:myController];
				NS_HANDLER
					classInstance = nil;
					NSRunAlertPanel(kPGPString_ErrorLoadingBundle,
									kPGPString_UnableToCreateBundleController,
									kPGPString_OK,
									nil,
									nil,
									name);
				NS_ENDHANDLER
			}
        }
    }
	
	[pool release];
	
    return classInstance;
}

- (BOOL)releaseBundlesIfReadyToShutdown
{
	NSArray*		bundleControllers = [self bundleControllers];
	id <PGPBundle>	bundleController;
	int				i, numItems;
	BOOL			shutdownAllowed = YES;
	
	if (!bundleControllers)
		return YES;
	
	numItems = [bundleControllers count];
	for (i = 0; i < numItems; ++i)
	{
		bundleController = [bundleControllers objectAtIndex:i];;
		
		if ([(NSObject*) bundleController respondsToSelector:@selector(releaseIfReadyToShutdown)])
		{
			if (![bundleController releaseIfReadyToShutdown])	/* Send message to release the bundle if it is ready to shutdown */
				shutdownAllowed = NO;
		}
	}
	
	return shutdownAllowed;
}

/* Net refCount: retain=0, release=(-1 * numControllers) */
- (void)releaseBundleControllers
{
    NSAutoreleasePool*	pool = [[NSAutoreleasePool alloc] init];
	NSArray*			bundleControllers = [self bundleControllers];
	NSEnumerator*		bundleEnumerator;
	id					bundleController;
	
	if (bundleControllers)
	{
        /* Creating the bundleEnumerator automatically marks it for autorelease (apparently) */
		bundleEnumerator = [bundleControllers objectEnumerator];
		
        /*
        nextObject will cause the object to be retained. The object is released when the
        autorelease pool is released.
        */
		while (bundleController = [bundleEnumerator nextObject])
			[bundleController release];
	}
    
    [pool release];
}

- (NSDictionary*)bundles
{
	return bundles;
}

- (NSArray*)bundlePaths
{
	return bundlePaths;
}

- (id)bundleByName:(NSString *)name
{
	NSArray*		bundleControllers = [self bundleControllers];
	NSEnumerator*	enumerator;
	id <PGPBundle>	controller;
	
	if (!name || !bundleControllers)
		return nil;
	
	enumerator = [bundleControllers objectEnumerator];
	
	while (controller = [enumerator nextObject])
	{
		if ([[[(NSObject*) controller class] identifier] isEqualToString:name])
			return controller;
	}
    
    return nil;
}

- (void)dealloc
{
	if (bundlePaths)
		[bundlePaths release];
	
	[self releaseBundleControllers];

	[super dealloc];
}

@end
