/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation
    All rights reserved.

    $Id: PGPDocument.m,v 1.17 2002/09/11 08:13:10 jeffc Exp $
____________________________________________________________________________*/

#import "PGPDocument.h"
#import "PGPConsole.h"

@implementation PGPDocument

- (NSData*)dataRepresentationOfType:(NSString*)type
{
    return nil;
}

- (BOOL)readFromFile:(NSString*)filename ofType:(NSString*)type
{
	NSArray*			bundleControllers = [[[PGPAppController appController] bundleManager] bundleControllers];
	NSEnumerator*		enumerator;
	id <PGPBundle>		controller;
	id <PGPBundle>		diskController = nil;
	id <PGPBundle>		keysController = nil;
	id <PGPBundle>		mailController = nil;
	id <PGPBundle>		netController = nil;

	if (!bundleControllers)
		return NO;
    
	enumerator = [bundleControllers objectEnumerator];
	
	while (controller = [enumerator nextObject])
	{
		if (!diskController && [[[(NSObject*) controller class] identifier] isEqualToString:@"PGPdisk.bundle"])
			diskController = controller;
		else if (!keysController && [[[(NSObject*) controller class] identifier] isEqualToString:@"PGPkeys.bundle"])
			keysController = controller;
		else if (!mailController && [[[(NSObject*) controller class] identifier] isEqualToString:@"PGPmail.bundle"])
			mailController = controller;
		else if (!netController && [[[(NSObject*) controller class] identifier] isEqualToString:@"PGPnet.bundle"])
			netController = controller;
	}

	if(			[type isEqualToString:kPGPDetachedSigFileType] ||
				[type isEqualToString:kPGPExportedKeyFileType] ||
				[type isEqualToString:kPGPEncryptedFileType] )
	{
		PGPLog((@"Sending document to mail"));
		if (mailController)
        {
			[[[PGPAppController appController] interface] setCurrentBundle:mailController passive:YES];
			[mailController handleDocument:[self fileName] ofType:type];
        }
	}
	else if(	[type isEqualToString:kPGPDiskImageFileType] )
	{
		PGPLog((@"Sending disk image file to disk"));
		if (diskController)
        {
			/* If opening a PGPdisk, we don't want to bring up the PGPdisk window (passive=YES). */
			[[[PGPAppController appController] interface] setCurrentBundle:diskController passive:YES];
			[diskController handleDocument:filename ofType:type];
        }
	}
	else if(	[type isEqualToString:kPGPPubKeyringFileType] ||
				[type isEqualToString:kPGPSecKeyringFileType] ||
				[type isEqualToString:kPGPKeyShareFileType] ||
				[type isEqualToString:kPGPCertificateFileType] )
	{
		PGPLog((@"Sending document to keys"));
		if (keysController)
        {
			[[[PGPAppController appController] interface] setCurrentBundle:keysController passive:NO];
			[keysController handleDocument:filename ofType:type];
        }
	}
	else if(	[type isEqualToString:kPGPVPNHostsFileType] ||
				[type isEqualToString:kPGPFirewallRulesFileType] )
	{
		PGPLog((@"Sending document to net"));
		if (netController)
        {
			[[[PGPAppController appController] interface] setCurrentBundle:netController passive:NO];
			[netController handleDocument:filename ofType:type];
        }
	}
	else if(	[type isEqualToString:kPGPRandomSeedFileType] )
	{
		PGPLog((@"Ignoring Random Seed file"));
	}

	return YES;
}

@end
