/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation
    All rights reserved.

    $Id: PGPPrefsGeneral.h,v 1.18 2002/08/06 20:09:21 dallen Exp $
____________________________________________________________________________*/

#import <PGP/PGP.h>
#import <PGP/PGPPrivate.h>
#import <PGPclient/pgpClientLibCocoa.h>
#import <PGPclient/pgpClientPrefs.h>
#import <PGPclient/CPGPPrefObj.h>
#import <PGPclient/CIntegerFormatter.h>
#import <PGPclient/CDurationFieldEditor.h>
#import "PGPController.h"
#import "PGPPrefsController.h"

#define kPGPPrefsGeneralNib			@"PGPprefsGeneral"
#define kPGPPrefsGeneralIdentifier	NSLocalizedString(@"General", @"General Pref Panel")

@interface PGPPrefsGeneral : NSObject <PGPPrefsController>
{
	id <PGPController>		mController;

	NSArray*				mIdentifiers;
	id						mPreviousWindowDelegate;
	CDurationFieldEditor*	mHoursFieldEditor;
	CDurationFieldEditor*	mMinutesFieldEditor;
	CDurationFieldEditor*	mSecondsFieldEditor;
	CIntegerFormatter*		mIntFormatter;
	
	/* Custom view to be used when displaying our prefs */
	IBOutlet NSView*		mPrefsView;
		
	/* Options */
	IBOutlet NSButton*		mEncryptToDefaultKey;
	IBOutlet NSButton*		mDefaultToMacBinary;
	IBOutlet NSButton*		mFasterKeyGen;
	IBOutlet NSTextField*	mCommentBlock;

	/* Single Sign-On */
	IBOutlet NSMatrix*		mCacheMatrix;
	IBOutlet NSButtonCell*	mCacheWhileLoggedOn;
	IBOutlet NSButtonCell*	mCacheForDuration;
	IBOutlet NSTextField*	mCacheHours;
	IBOutlet NSTextField*	mCacheMinutes;
	IBOutlet NSTextField*	mCacheSeconds;
	IBOutlet NSStepper*		mDurationStepper;
	IBOutlet NSButtonCell*	mDoNotCache;
	IBOutlet NSButton*		mShareCache;
	
	/* File Wiping */
	IBOutlet NSButton*		mWarnOnWipe;
	IBOutlet NSTextField*	mNumPasses;
	IBOutlet NSStepper*		mPassesStepper;
	
	BOOL					mInited;
	BOOL					mLoadedNib;	
}

- (id)init;
- (id)initWithNib:(NSString*)nibFile;
- (id)initWithController:(id)controller;

- (NSArray*)identifiers;
- (id)prefsViewForIdentifier:(NSString*)identifier;

- (id)firstPrefsResponderForIdentifier:(NSString*)identifier;
- (id)lastPrefsResponderForIdentifier:(NSString*)identifier;
- (void)setupKeyViewLoopForIdentifier:(NSString*)identifier;

- (void)restorePanelPrefs:(NSString*)panelIdentifier;
- (void)restorePanelDefaults:(NSString*)panelIdentifier;
- (void)savePanelPrefs:(NSString*)panelIdentifier;

- (void)setEncryptToDefaultKey:(PGPBoolean)value enabled:(BOOL)enabled;
- (void)setDefaultToMacBinary:(PGPUInt32)value enabled:(BOOL)enabled;
- (void)setFasterKeyGen:(PGPBoolean)value enabled:(BOOL)enabled;
- (void)setCommentBlock:(char*)value enabled:(BOOL)enabled;
- (void)setCacheType:(PGPUInt32)value enabled:(BOOL)enabled;
- (void)setCacheSeconds:(PGPUInt32)value enabled:(BOOL)enabled;
- (void)setSharedCache:(PGPBoolean)value enabled:(BOOL)enabled;
- (void)setWarnOnWipe:(PGPBoolean)value enabled:(BOOL)enabled;
- (void)setFileWipePasses:(PGPUInt32)value enabled:(BOOL)enabled;

- (PGPBoolean)getEncryptToDefaultKey;
- (PGPUInt32)getDefaultToMacBinary;
- (PGPBoolean)getFasterKeyGen;
- (char*)getCommentBlock;
- (PGPUInt32)getCacheType;
- (PGPUInt32)getCacheSeconds;
- (PGPBoolean)getSharedCache;
- (PGPBoolean)getWarnOnWipe;
- (PGPUInt32)getFileWipePasses;

- (void)setCheckbox:(NSButton*)checkBox state:(BOOL)state enabled:(BOOL)enabled;
- (PGPBoolean)getCheckbox:(NSButton*)checkBox;

- (IBAction)cacheForDuration:(id)sender;
- (IBAction)otherCacheType:(id)sender;
- (IBAction)incrementDurationField:(id)sender;
- (IBAction)numPassesPressed:(id)sender;

- (void)incrementSeconds:(PGPInt32)increment;
- (void)incrementMinutes:(PGPInt32)increment;
- (void)incrementHours:(PGPInt32)increment;

/* Notifications */
- (void)prefsViewBecameActive:(NSNotification*)notification;
- (void)prefsViewBecameInactive:(NSNotification*)notification;

/* Window delegate methods */
- (id)windowWillReturnFieldEditor:(NSWindow*)sender toObject:(id)object;

- (void)dealloc;

@end
