/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation
    All rights reserved.

    $Id: PGPCMPluginImp.mm,v 1.11 2002/11/12 23:18:57 jeffc Exp $
____________________________________________________________________________*/
#include <CoreFoundation/CoreFoundation.h>
#include <CoreFoundation/CFPlugInCOM.h>
#include <Carbon/Carbon.h>
#include <Cocoa/Cocoa.h>
#include <syslog.h>
#include "PGPCMPlugin.h"

const SInt16		kEncrypt					=	1;
const SInt16		kSign						=	2;
const SInt16		kEncryptAndSign				=	3;
const SInt16		kDecryptVerify				=	4;
const SInt16		kWipe						=	5;
const OSType		kPGPSignature				=	'pgpD';
const AEEventClass	kPGPClass					=	'PGP ';
const AEEventID		kAESignFileID				=	'SiFi';
const AEEventID		kAEEncryptFileID			=	'EnFi';
const AEEventID		kAEEncryptAndSignFileID		=	'ESFi';
const AEEventID		kAEDecryptVerifyFileID		=	'DVFi';
const AEEventID		kAEWipeFileID				=	'WiFi';

#define				kPGPApplicationName				@"PGP"

	static void
addCommandToAEDescList(
	NSString			*		inCommandString,
	SInt32						inCommandID,
	NSAppleEventDescriptor *	ioCommandList )
{
	NSAppleEventDescriptor *	cmd = [NSAppleEventDescriptor recordDescriptor];

	[cmd setDescriptor:[NSAppleEventDescriptor descriptorWithString:inCommandString] forKeyword:keyAEName];
	[cmd setDescriptor:[NSAppleEventDescriptor descriptorWithInt32:inCommandID] forKeyword:keyContextualMenuCommandID];
	[ioCommandList insertDescriptor:cmd atIndex:0];
}

	OSStatus
PGPCMPluginExamineContextImp(
	void				*context,
	const AEDesc		*inContext,
	AEDescList			*outCommandPairs )
{
	CFStringRef			bundleIdentifier = CFBundleGetIdentifier(CFBundleGetMainBundle());
	
#if PGP_DEBUG
	openlog( "PGPcontext", LOG_PID | LOG_NDELAY, LOG_USER );
    syslog( LOG_ERR, "PGPCMPluginExamineContext:%s\n", CFStringGetCStringPtr( bundleIdentifier, kCFStringEncodingASCII ));
#endif
	if (inContext && bundleIdentifier && (CFStringCompare(bundleIdentifier, CFSTR("com.apple.finder"), 0) == kCFCompareEqualTo))
	{
		NSAutoreleasePool *			pool = [[NSAutoreleasePool alloc] init];
		NSAppleEventDescriptor *	theSubMenu = [NSAppleEventDescriptor recordDescriptor];
		NSAppleEventDescriptor *	theSubMenuList = [NSAppleEventDescriptor listDescriptor];

		// Get our submenu list
		addCommandToAEDescList( LSCO( @"Encrypt", @"PGP->Encrypt submenu item" ),
								kEncrypt, theSubMenuList);
		addCommandToAEDescList( LSCO( @"Sign", @"PGP->Sign submenu item" ),
								kSign, theSubMenuList);
		addCommandToAEDescList( LSCO( @"Encrypt & Sign", @"PGP->Encrypt+Sign submenu item" ),
								kEncryptAndSign, theSubMenuList);
		addCommandToAEDescList( LSCO( @"Decrypt/Verify", @"PGP->Decrypt/Verify submenu item" ),
								kDecryptVerify, theSubMenuList);
		addCommandToAEDescList( LSCO( @"Wipe", @"PGP->Wipe submenu item" ),
								kWipe, theSubMenuList);
	
		// Add PGP with submenu to ioCommands
		[theSubMenu setDescriptor:[NSAppleEventDescriptor descriptorWithString:LSCO(@"PGP", @"PGP top level menuitem")] forKeyword:keyAEName];
		[theSubMenu setDescriptor:theSubMenuList forKeyword:keyContextualMenuSubmenu];
		::AEPutDesc( outCommandPairs, 0, [theSubMenu aeDesc]);
		
		[pool release];
	}
	
	return noErr;
}

	OSStatus
PGPCMPluginHandleSelectionImp(
	void				*context,
	AEDesc				*inContext,
	SInt32				inCommandID )
{
	NSAutoreleasePool	*pool	= [[NSAutoreleasePool alloc] init];
	AEDesc				listDesc;
	OSStatus			err;

#if PGP_DEBUG
	openlog( "PGPcontext", LOG_PID, LOG_USER );
    syslog( LOG_ERR, "PGPCMPluginHandleSelectionImp:%s\n", [[[NSBundle mainBundle] bundleIdentifier] cString] );
#endif
	if((inContext != nil) && [[[NSBundle mainBundle] bundleIdentifier] isEqualToString:@"com.apple.finder"] &&
		(AECoerceDesc(inContext, typeAEList, &listDesc) == noErr)) {
		AEEventID	eventID;
		
#if PGP_DEBUG
    syslog( LOG_ERR, "PGPCMPluginHandleSelectionImp AECoerceDesc exited with error %ld", err );
#endif
		switch (inCommandID) {
			case kEncrypt:
			{
				eventID = kAEEncryptFileID;
			}
			break;
			
			
			case kSign:
			{
				eventID = kAESignFileID;
			}
			break;


			case kEncryptAndSign:
			{
				eventID = kAEEncryptAndSignFileID;
			}
			break;


			case kDecryptVerify:
			{
				eventID = kAEDecryptVerifyFileID;
			}
			break;


			case kWipe:
			{
				eventID = kAEWipeFileID;
			}
			break;
		}
		
		NSAppleEventDescriptor *	list = [[[NSAppleEventDescriptor alloc] initWithAEDescNoCopy:&listDesc] autorelease];
		NSAppleEventDescriptor *	event = [NSAppleEventDescriptor appleEventWithEventClass:kPGPClass eventID:eventID
												targetDescriptor:[NSAppleEventDescriptor
												descriptorWithDescriptorType:typeApplSignature bytes:&kPGPSignature
												length:sizeof(kPGPSignature)]
												returnID:kAutoGenerateReturnID transactionID:kAnyTransactionID];
		AppleEvent					reply;

		[event setParamDescriptor:list forKeyword:keyDirectObject];
		err = AECreateDesc(typeNull, NULL, 0, &reply);
#if PGP_DEBUG
    syslog( LOG_ERR, "PGPCMPluginHandleSelectionImp AECreateDesc exited with error %ld", err );
#endif
		if (err == noErr) {
			err = AESend([event aeDesc], &reply, kAEWaitReply, kAENormalPriority, kAEDefaultTimeout, NULL, NULL);
			if ((err == procNotFound) && [[NSWorkspace sharedWorkspace] launchApplication:kPGPApplicationName]) {
				int i;
			
				for (i = 0; (i < 10) && (err == procNotFound); i++) {
					[NSThread sleepUntilDate:[NSDate dateWithTimeIntervalSinceNow:1]];
					err = AESend([event aeDesc], &reply, kAEWaitReply, kAENormalPriority, kAEDefaultTimeout, NULL, NULL);
				}
			}
#if PGP_DEBUG
    syslog( LOG_ERR, "PGPCMPluginHandleSelectionImp AESend exited with error %ld", err );
#endif
		}
		[[[NSAppleEventDescriptor alloc] initWithAEDescNoCopy:&reply] autorelease];
	}

#if PGP_DEBUG
    syslog( LOG_ERR, "PGPCMPluginHandleSelectionImp exited with error %ld", err );
#endif
	
	[pool release];
	return noErr;
}

