/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation
    All rights reserved.

    $Id: CPGPdiskConversion.m,v 1.6 2002/11/09 03:05:39 jeffc Exp $
____________________________________________________________________________*/

#import "CPGPdiskConversion.h"
#import "CPGPdiskProgressPanel.h"
#import "PGPdiskController.h"
#import <PGPclient/pgpClientLibCocoa.h>

#define kPGPdiskString_ConvertingDiskCaption	LSPD(@"Converting PGPdisk", @"Conversion caption")

static BOOL						sConversionAborted = NO;

static PGPError sDiskConversionCallback(PGPUInt32 percent, void* userval)
{
    CPGPdiskProgressPanel*	panel = [CPGPdiskProgressPanel sharedPanel];
    NSProgressIndicator*	progressIndicator = [panel progressIndicator];
    
    if (sConversionAborted)
        return kPGPError_UserAbort;
 
	if ([[panel window] isVisible])
	{
		if (progressIndicator)
		{
			[progressIndicator setDoubleValue:percent];
			[progressIndicator displayIfNeeded];
		}
	}
		
	return kPGPError_NoErr;
}

@implementation CPGPdiskConversion

- (id)init
{
	if (!mInited)
	{
		[super init];
		
		mInited = YES;
	}
	
	return self;
}

- (void)setOriginalFilename:(NSString*)filename
{
	if (mOldFilename)
	{
		[mOldFilename release];
		mOldFilename = nil;
	}
	
	if (filename)
		mOldFilename = [filename retain];
}

- (void)setNewFilename:(NSString*)filename;
{
	if (mNewFilename)
	{
		[mNewFilename release];
		mNewFilename = nil;
	}
	
	if (filename)
		mNewFilename = [filename retain];
}

- (void)setPassphrase:(NSData*)passphrase
{
	if (mPassphrase)
	{
		[mPassphrase release];
		mPassphrase = nil;
	}
	
	if (passphrase)
		mPassphrase = [passphrase retain];
}

- (void)setServerConnection:(NSConnection*)connection
{
	if (mConnection)
	{
		[mConnection release];
		mConnection = nil;
	}
	
	if (connection)
		mConnection = [connection retain];
}

- (void)setPubRingPath:(char*)pubPath
{
	strncpy(mPubRingPath, pubPath, PATH_MAX);
}

- (void)setSecRingPath:(char*)secPath
{
	strncpy(mSecRingPath, secPath, PATH_MAX);
}

- (void)convertDiskWithPorts:(NSArray*)ports
{
	NSAutoreleasePool*		pool = [[NSAutoreleasePool alloc] init];
	NSConnection*			connection;
    CPGPdiskProgressPanel*	panel = [CPGPdiskProgressPanel sharedPanel];
	NSString*				caption;
	id						proxy;
	PGPError				err = kPGPError_NoErr;
	PGPContextRef			context = kInvalidPGPContextRef;
	PGPdiskContextRef		diskContext = kInvalidPGPdiskContextRef;
	const char*				passphrase;
	
	if (!ports || !mOldFilename || !mNewFilename || !mPassphrase)
	{
		PGPLog((@"Failed to convert OS9 disk due to invalid arguments"));
		goto done;
	}
	
	if (IsPGPError(err = PGPNewContext(kPGPsdkAPIVersion, &context)))
	{
		PGPLog((@"PGPNewContext returned err: %d", err));
		goto done;
	}
	
	if (IsPGPError(err = PGPdiskNewContext(context, kPGPdiskAPIVersion, &diskContext)))
	{
		PGPLog((@"PGPdiskNewContext returned err: %d", err));
		goto done;
	}
	
	if (IsPGPError(err = PGPdiskLoadKeyrings(diskContext, mPubRingPath, mSecRingPath)))
	{
		PGPLog((@"PGPdiskLoadKeyrings returned err: %d", err));
		goto done;
	}
	
	sConversionAborted = NO;
	
	connection = [NSConnection connectionWithReceivePort:[ports objectAtIndex:0] sendPort:[ports objectAtIndex:1]];
	proxy = [connection rootProxy];
	[proxy setProtocolForProxy:@protocol(CPGPdiskConversionClient)];
	passphrase = (const char*) [mPassphrase bytes];
	caption = [NSString stringWithFormat:@"%@ \"%@\"",
					kPGPdiskString_ConvertingDiskCaption,
					[[mOldFilename lastPathComponent] stringByDeletingPathExtension]];
					
	[panel setCaption:caption];

	err = PGPdiskConvertDisk(diskContext,
				kPGPdiskDiskFormat_OS9,
                [mOldFilename UTF8String],
				passphrase,
				[mNewFilename UTF8String],
				sDiskConversionCallback,
				NULL);
	
	if ([[panel window] isVisible])
		[panel endSheetWithReturnCode:err];
	
	if (IsPGPError(err))
	{
		PGPLog((@"Failed to convert OS9 disk: %@ (err=%d)", mOldFilename, err));
		[[NSFileManager defaultManager] removeFileAtPath:mNewFilename handler:nil];
		
		[proxy failedToConvertDisk:mNewFilename];
	}
	else
		[proxy completedConversionOfDisk:mNewFilename];
	
	[panel release];
    [self autorelease];
	
done:
	if (PGPdiskContextRefIsValid(diskContext))
		PGPdiskFreeContext(diskContext);
	
	if (PGPContextRefIsValid(context))
		PGPFreeContext(context);
	
	[pool release];
}

- (void)abortConversion
{
	sConversionAborted = YES;
}

- (void)dealloc
{
	PGPLog((@"Deallocated CPGPdiskConversion"));

	sConversionAborted = NO;
	
	if (mNewFilename)
		[mNewFilename release];
	
	if (mOldFilename)
		[mOldFilename release];
	
	if (mPassphrase)
		[mPassphrase release];
	
	if (mConnection)
		[mConnection release];
	
	[super dealloc];
}

@end
