/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation
    All rights reserved.

    $Id: CPGPdiskCreation.h,v 1.35 2002/11/14 01:32:49 jeffc Exp $
____________________________________________________________________________*/

#import "PGPdiskCommon.h"
#import "PGPdiskController.h"
#import <PGPclient/CIntegerFormatter.h>

@interface CPGPdiskCreation : NSObject
{
	IBOutlet NSWindow*				mWindow;
	IBOutlet NSTextField*			mSize;
	IBOutlet NSStepper*				mStepper;
	IBOutlet NSPopUpButton*			mSizeUnits;
	IBOutlet NSPopUpButton*			mCipher;
	IBOutlet NSPopUpButton*			mFormat;
	IBOutlet NSPopUpButton*			mUserType;
	IBOutlet NSTextField*			mName;
	IBOutlet NSTextField*			mUsername;
	IBOutlet NSBox*					mKeyBox;
    IBOutlet NSImageView*			mKeyIcon;
    IBOutlet NSTextField*			mKeyUserID;
	IBOutlet NSButton*				mAuthButton;
	IBOutlet NSButton*				mCancel;
	IBOutlet NSButton*				mCreate;
	
	IBOutlet NSWindow*				mProgressWindow;
	IBOutlet NSTextField*			mProgressCaption;
	IBOutlet NSTextField*			mFile;
	IBOutlet NSProgressIndicator*	mProgress;
	IBOutlet NSButton*				mStop;
	
	NSConnection*					mServerConnection;
	NSString*						mFilename;
	NSString*						mVolumeName;
	PGPUInt32						mLastSelectedAuthType;
	PGPKeyDBObjRef					mKeyDBObj;
	PGPKeyID						mKeyID;
	char*							mPassphrase;
	SInt32							mOSVersion;
	CPGPdiskObject*					mDisk;
	CIntegerFormatter*				mIntFormatter;
	BOOL							mHideExtension;
	BOOL							mInited;
	BOOL							mLoadedNib;
}

+ (id)sharedCreator;

- (id)init;
- (void)loadNib:(NSString*)nibFile;
- (void)resetWindow;
- (void)setupKeyViewLoop;
- (void)run;

- (BOOL)validateFields;
- (BOOL)isSizeValid;
- (BOOL)isUserValid;

- (NSString*)convertToMSDOSName:(NSString*)name;
- (NSString*)convertToHFSName:(NSString*)name;
- (NSString*)convertToHFSPlusName:(NSString*)name;
- (NSString*)convertToUFSName:(NSString*)name;

- (PGPUInt64)size;
- (PGPdiskEncryptionAlgorithm)cipher;
- (PGPdiskFilesystemType)format;
- (const char*)filename;
- (const char*)volumeName;
- (const char*)username;
- (const char*)passphrase;
- (PGPKeyDBObjRef)key;
- (PGPKeyID)keyID;
- (BOOL)hideExtension;

- (IBAction)userTypeChanged:(id)sender;
- (IBAction)stepperPressed:(id)sender;
- (IBAction)selectKey:(id)sender;
- (IBAction)setPassphrase:(id)sender;
- (IBAction)clearPassphrase:(id)sender;
- (IBAction)create:(id)sender;
- (IBAction)close:(id)sender;
- (IBAction)stopCreation:(id)sender;

- (void)diskCreationThread;
- (void)displayFailureAlert:(NSNumber*)errNum;
- (void)closeWindow;

- (NSString*)convertToVolumeName:(NSString*)name forFormat:(PGPUInt32)format;

- (void)mountNewDisk:(CPGPdiskObject*)disk;
- (void)keySelectDidEnd:(PGPError)resultCode selectedKeys:(void*)keySetPtr;
- (void)alertDidEnd:(NSWindow*)sheet returnCode:(int)returnCode contextInfo:(void*)contextInfo;
- (void)savePanelDidEnd:(NSWindow*)sheet returnCode:(int)returnCode contextInfo:(void*)contextInfo;
- (void)creationDidEnd:(NSWindow*)sheet returnCode:(int)returnCode contextInfo:(void*)contextInfo;
- (void)textDidChange:(NSNotification*)notification;
- (BOOL)panel:(id)sender isValidFilename:(NSString*)filename;
- (BOOL)windowShouldClose:(id)sender;
- (void)windowWillClose:(NSNotification*)notification;

- (void)dealloc;

@end