/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation
    All rights reserved.

    $Id: CPGPdiskCreation.m,v 1.54 2002/11/18 20:39:01 jeffc Exp $
____________________________________________________________________________*/

#import "CPGPdiskCreation.h"
#import "pgpPFLPriv.h"
#import <sys/param.h>
#import <sys/mount.h>

#define kPGPdiskCreationNib						@"PGPdiskCreation"
#define kPGPdiskCreationCipher_CAST				3
#define kPGPdiskCreationCipher_AES				5
#define kPGPdiskCreationMinPassphraseLen		8
#define kPGPdiskCreationAuthType_Passphrase		4000
#define kPGPdiskCreationAuthType_PubKey			4001
#define kPGPdiskCreationDefault_Size			100
#define kPGPdiskCreationDefault_SizeTag			2001	/* Megabytes */
#define kPGPdiskCreationDefault_Cipher			kPGPdiskCreationCipher_CAST
#define kPGPdiskCreationDefault_AuthType		kPGPdiskCreationAuthType_Passphrase	
#define kPGPdiskCreationMinSize_MSDOS			8423424	/* 8226 KB */
#define kPGPdiskCreationMinSize_HFS				524288	/* 512 KB */
#define kPGPdiskCreationMinSize_HFSPlus			4194304 /* 4 MB */
#define kPGPdiskCreationMinSize_UFS				131072	/* 128 KB */

#define kPGPdiskString_MSDOS					LSPD(@"MS-DOS", @"MS-DOS")
#define kPGPdiskString_MacOSStandard			LSPD(@"Mac OS Standard", @"HFS")
#define kPGPdiskString_MacOSExtended			LSPD(@"Mac OS Extended", @"HFS+")
#define kPGPdiskString_UFS						LSPD(@"UNIX File System", @"UFS")
#define kPGPdiskString_DefaultVolumeName		LSPD(@"Untitled", @"Default volume name")
#define kPGPdiskString_Username					LSPD(@"Username:", @"Username prompt")
#define kPGPdiskString_PublicKey				LSPD(@"Public key:", @"Public key prompt")
#define kPGPdiskString_CreatingPGPdisk			LSPD(@"Creating PGPdisk:", "Creating progress sheet caption")
#define kPGPdiskString_FormattingPGPdisk		LSPD(@"Formatting PGPdisk:", "Formatting progress sheet caption")
#define kPGPdiskString_MountingPGPdisk			LSPD(@"Mounting PGPdisk:", @"Mounting progress sheet caption")
#define kPGPdiskString_MissingKeyrings          LSPD(@"Unable to open keyrings.", @"Missing keyrings caption")
#define kPGPdiskString_MissingKeyringsMsg       LSPD(@"The default keyrings could not be opened. Please verify that you have specified the location of your keyrings in the preferences.", @"Missing keyrings message")
#define kPGPdiskString_InvalidDiskSize          LSPD(@"Invalid disk size specified.", @"Invalid size caption")
#define kPGPdiskString_InvalidDiskSizeMsg       LSPD(@"The minimum supported disk size for a volume of type \"%@\" is %@.", @"Invalid size message")
#define kPGPdiskString_MissingUsername			LSPD(@"Missing username", @"Missing username caption")
#define kPGPdiskString_MissingUsernameMsg		LSPD(@"You have not specified a username. Your PGPdisk must have at least one user. Please specify a username or select a public key user.", @"Missing username message")
#define kPGPdiskString_MissingPassphrase		LSPD(@"Missing passphrase", @"Missing passphrase caption")
#define kPGPdiskString_MissingPassphraseMsg		LSPD(@"You have not assigned a passphrase for the specified user. Please click the Set Passphrase button to assign a passphrase.", @"Missing passphrase message")
#define kPGPdiskString_MissingPubKey			LSPD(@"Missing public key user", @"Missing pubkey caption")
#define kPGPdiskString_MissingPubKeyMsg			LSPD(@"You have not selected a public key user for your PGPdisk. Please click the Select Key button to choose a public key user.", @"Missing pubkey message")
#define kPGPdiskString_DiskTooLarge             LSPD(@"Disk exceeds maximum file size", @"Disk too large")
#define kPGPdiskString_DiskTooLargeHFSMsg       LSPD(@"The specified disk size exceeds the HFS maximum file size. Please choose a disk size no greater than 2GB, or select a disk location on a non-HFS volume.", @"HFS Disk too large")
#define kPGPdiskString_DiskTooLargeUFSMsg       LSPD(@"The specified disk size exceeds the UFS maximum file size. Please choose a disk size no greater than 4GB, or select a disk location on a non-UFS volume.", @"UFS Disk too large")
#define kPGPdiskString_VolumeNotLargeEnough     LSPD(@"Disk is too large", @"Volume too small")
#define kPGPdiskString_VolumeNotLargeEnoughMsg  LSPD(@"The specified disk size exceeds the available number of bytes on the selected volume. Please select a disk size less than %.2f%@.", @"Volume too small")
#define kPGPdiskString_CantCreateMinSizeDisk    LSPD(@"Volume is too small", @"Volume is too small")
#define kPGPdiskString_CantCreateMinSizeDiskMsg LSPD(@"The specified disk location does not have enough available disk space to create a minimum size PGPdisk. Please choose another location.", @"Volume too small for min size PGPdisk")
#define kPGPdiskString_UnknownCreationFailure		LSPD(@"Failed to create new PGPdisk", @"Generic creation failure caption")
#define kPGPdiskString_UnknownCreationFailureMsg	LSPD(@"PGPdisk creation failed for an unknown reason. If problems persist, please contact your software vendor.", @"Generic creation failure message")

enum
{
	kPGPdiskCreationSizeUnits_KB	= 2000,
	kPGPdiskCreationSizeUnits_MB,
	kPGPdiskCreationSizeUnits_GB
};

static BOOL sContinue;
static CPGPdiskCreation*	sSharedInstance = nil;

static PGPError newdisk_callback(PGPBoolean isFormatting, PGPUInt32 percent, void* userValue)
{
    NSProgressIndicator*    progressIndicator = (NSProgressIndicator*) userValue;

    if (progressIndicator)
    {
        [progressIndicator setDoubleValue:percent];
        [progressIndicator displayIfNeeded];
    }

    return sContinue ? kPGPError_NoErr : kPGPError_UserAbort;
}

@protocol PGPdiskCreationProtocol

- (void)displayFailureAlert:(NSNumber*)errNum;
- (void)openDisk:(NSString*)path;
- (void)mountNewDisk:(CPGPdiskObject*)disk;
- (void)closeWindow;

@end

@implementation CPGPdiskCreation

+ (id)sharedCreator
{
	if (!sSharedInstance)
		sSharedInstance = [[[CPGPdiskCreation alloc] init] autorelease];
	
	return sSharedInstance;
}

- (id)init
{
	if (!mInited)
	{
		[super init];
		
		mInited = YES;
	}
	
	if (!mLoadedNib)
		[self loadNib:kPGPdiskCreationNib];
	
	return self;
}

- (void)loadNib:(NSString*)nibFile
{
	if (mLoadedNib || !nibFile)
		return;
	
	[NSBundle loadNibNamed:nibFile owner:self];

	[[NSNotificationCenter defaultCenter] addObserver:self
			selector:@selector(textDidChange:)
			name:NSControlTextDidChangeNotification
			object:nil];
	
	[mUsername retain];
	[mKeyBox retain];
	
	[mWindow setDelegate:self];
	
    if (!mIntFormatter)
    {
        mIntFormatter = [[CIntegerFormatter alloc] init];
        [mIntFormatter setFormat:@"##0"];
        [mIntFormatter setMinimum:[NSDecimalNumber one]];
    }
	
	[mSize setFormatter:mIntFormatter];
	
	Gestalt(gestaltSystemVersion, &mOSVersion);
	
	if (mOSVersion >= 0x1022)
	{
		[mFormat selectItemAtIndex:[mFormat indexOfItemWithTag:kPGPdiskFilesystemType_HFSPlusJournaled]];
		[mFormat sizeToFit];
	}
	else
	{
		[mFormat removeItemAtIndex:[mFormat indexOfItemWithTag:kPGPdiskFilesystemType_HFSPlusJournaled]];
		[mFormat selectItemAtIndex:[mFormat indexOfItemWithTag:kPGPdiskFilesystemType_HFSPlus]];
	}
	
	[mProgress setUsesThreadedAnimation:YES];
	
	mLoadedNib = YES;
}

/* Configure our window with the default values */
- (void)resetWindow
{
	/* Restore the size controls */
	[mSize setStringValue:[[NSNumber numberWithInt:kPGPdiskCreationDefault_Size] stringValue]];
	[mSizeUnits selectItemAtIndex:[mSizeUnits indexOfItemWithTag:kPGPdiskCreationDefault_SizeTag]];
	
	/* Restore the default cipher */
	[mCipher selectItemAtIndex:[mCipher indexOfItemWithTag:kPGPdiskCreationDefault_Cipher]];
	
	/* Restore the default format and volume name */
	[mFormat selectItemAtIndex:[mFormat indexOfItemWithTag:kPGPdiskFilesystemType_HFSPlus]];
	
	/* Restore the default auth type */
	[mUserType selectItemAtIndex:[mUserType indexOfItemWithTag:kPGPdiskCreationDefault_AuthType]];
	mLastSelectedAuthType = 0;
	[self userTypeChanged:nil];
	
	if (mDisk)
	{
		[mDisk release];
		mDisk = nil;
	}
	
	sContinue = YES;	/* Indicates whether the user cancels the create disk process */
}

- (void)setupKeyViewLoop
{
	[mSize setNextKeyView:mStepper];
	[mStepper setNextKeyView:mSizeUnits];
	[mSizeUnits setNextKeyView:mCipher];
	[mCipher setNextKeyView:mFormat];
	[mFormat setNextKeyView:mUserType];
	
	if ([[mUserType selectedItem] tag] == kPGPdiskCreationAuthType_Passphrase)
	{
		[mUserType setNextKeyView:mUsername];
		[mUsername setNextKeyView:mAuthButton];
	}
	else
		[mUserType setNextKeyView:mAuthButton];
	
	[mAuthButton setNextKeyView:mCancel];
	[mCancel setNextKeyView:mCreate];
	[mCreate setNextKeyView:mSize];
}

- (void)run
{
	NSPort*			sendPort = [NSPort port];
	NSPort*			receivePort = [NSPort port];
	
	if (![mWindow isVisible])
	{
		mServerConnection = [[NSConnection alloc] initWithReceivePort:receivePort sendPort:sendPort];
		[mServerConnection setRootObject:self];
		
		[self retain];
		[self resetWindow];
		[mWindow center];
	}
	
    [mWindow makeKeyAndOrderFront:self];
}

- (BOOL)validateFields
{
	return [self isSizeValid] && [self isUserValid];
}

- (BOOL)isSizeValid
{
	NSString*		fsType = nil;
	NSString*		minSize = nil;
	
	switch ([[mFormat selectedItem] tag])
	{
		case kPGPdiskFilesystemType_FAT16:
			if ([self size] < kPGPdiskCreationMinSize_MSDOS)
			{
				fsType = kPGPdiskString_MSDOS;
				minSize = LSPD(@"8226 KB", @"MS-DOS Minimum Size");
			}
			break;
			
		case kPGPdiskFilesystemType_HFS:
			if ([self size] < kPGPdiskCreationMinSize_HFS)
			{
				fsType = kPGPdiskString_MacOSStandard;
				minSize = LSPD(@"512 KB", @"HFS Minimum Size");
			}
			break;
		
		case kPGPdiskFilesystemType_HFSPlusJournaled:
			/* Fall through */
		case kPGPdiskFilesystemType_HFSPlus:
			if ([self size] < kPGPdiskCreationMinSize_HFSPlus)
			{
				fsType = kPGPdiskString_MacOSExtended;
				minSize = LSPD(@"4 MB", @"HFS+ Minimum Size");
			}
			break;
		
		case kPGPdiskFilesystemType_UFS:
			if ([self size] < kPGPdiskCreationMinSize_UFS)
			{
				fsType = kPGPdiskString_UFS;
				minSize = LSPD(@"128 KB", @"UFS Minimum Size");
			}
			break;
	}
	
	if (fsType && minSize)
	{
		NSBeginAlertSheet(kPGPdiskString_InvalidDiskSize,
			kPGPdiskString_OK,      /* defaultButton */
			nil,                    /* alternateButton */
			nil,                    /* otherButton */
			mWindow,				/* docWindow */
			self,                   /* modalDelegate */
			nil,                    /* willEndSelector */
			@selector(alertDidEnd:returnCode:contextInfo:), /* didEndSelector */
			mSize,					/* contextInfo */
			[NSString stringWithFormat:kPGPdiskString_InvalidDiskSizeMsg, fsType, minSize]);
		
		return NO;
	}
	
	return YES;
}

- (BOOL)isUserValid
{
	unsigned char*		utf8Name = "";
	
	if ([[mUserType selectedItem] tag] == kPGPdiskCreationAuthType_Passphrase)
	{
		if ([mUsername stringValue] && [[mUsername stringValue] length])
			utf8Name = (unsigned char*) [[mUsername stringValue] UTF8String];
		
		if (![[mUsername stringValue] length])
		{
			NSBeginAlertSheet(kPGPdiskString_MissingUsername,
				kPGPdiskString_OK,		/* defaultButton */
				nil,                    /* alternateButton */
				nil,                    /* otherButton */
				mWindow,				/* docWindow */
				self,                   /* modalDelegate */
				nil,                    /* willEndSelector */
				@selector(alertDidEnd:returnCode:contextInfo:), /* didEndSelector */
				mUsername,				/* contextInfo */
				kPGPdiskString_MissingUsernameMsg);
			
			return NO;
		}
		else if (strlen(utf8Name) > (kPGPdiskMaxUserNameSize - 1))
		{
			NSBeginAlertSheet(kPGPdiskString_UserNameTooLong,
				kPGPdiskString_OK,		/* defaultButton */
				nil,                    /* alternateButton */
				nil,                    /* otherButton */
				mWindow,				/* docWindow */
				self,                   /* modalDelegate */
				nil,                    /* willEndSelector */
				@selector(alertDidEnd:returnCode:contextInfo:), /* didEndSelector */
				mUsername,				/* contextInfo */
				kPGPdiskString_UserNameTooLongMsg);
			
			return NO;
		}
		else if (!mPassphrase)
		{
			NSBeginAlertSheet(kPGPdiskString_MissingPassphrase,
				kPGPdiskString_OK,		/* defaultButton */
				nil,                    /* alternateButton */
				nil,                    /* otherButton */
				mWindow,				/* docWindow */
				self,                   /* modalDelegate */
				nil,                    /* willEndSelector */
				@selector(alertDidEnd:returnCode:contextInfo:), /* didEndSelector */
				mAuthButton,			/* contextInfo */
				kPGPdiskString_MissingPassphraseMsg);
			
			return NO;
		}
	}
	else
	{
		if (!mKeyDBObj)
		{
			NSBeginAlertSheet(kPGPdiskString_MissingPubKey,
				kPGPdiskString_OK,		/* defaultButton */
				nil,                    /* alternateButton */
				nil,                    /* otherButton */
				mWindow,				/* docWindow */
				self,                   /* modalDelegate */
				nil,                    /* willEndSelector */
				@selector(alertDidEnd:returnCode:contextInfo:), /* didEndSelector */
				mAuthButton,			/* contextInfo */
				kPGPdiskString_MissingPubKeyMsg);
			
			return NO;
		}
	}
	
	return YES;
}

- (NSString*)convertToMSDOSName:(NSString*)name
{
	NSMutableCharacterSet*	badChars;
	NSScanner*				scanner;
	NSString*				volumeName = nil;
	
	badChars = [[[NSMutableCharacterSet alloc] init] autorelease];
	[badChars addCharactersInString:@"*+=\\|[]/?.><,;:\""];

	if (!name || ![name length])
		return nil;
	
	scanner = [NSScanner scannerWithString:name];
	[scanner setCaseSensitive:NO];
	
	if ([scanner scanUpToCharactersFromSet:badChars intoString:&volumeName])
	{
		if (![scanner isAtEnd] && [scanner scanCharactersFromSet:badChars intoString:nil])
			return nil;
		
		if ([volumeName length] > 11)
		{
			NSMutableString*	mutableString = [NSMutableString stringWithString:volumeName];
			
			[mutableString deleteCharactersInRange:NSMakeRange(11, [volumeName length] - 11)];
			volumeName = [NSString stringWithString:mutableString];
		}
	}
	
	return volumeName;
}

- (NSString*)convertToHFSName:(NSString*)name
{
	NSString*	volumeName = nil;
	
	if (!name || ![name length])
		return nil;
	
	if ([name length] > 27)
	{
		NSMutableString*	mutableString = [NSMutableString stringWithString:name];
		
		[mutableString deleteCharactersInRange:NSMakeRange(27, [name length] - 27)];
		volumeName = [NSString stringWithString:mutableString];
	}
	else
		volumeName = name;
	
	return volumeName;
}

- (NSString*)convertToHFSPlusName:(NSString*)name
{
	NSString*	volumeName = nil;
	
	if (!name || ![name length])
		return nil;
	
	if ([name length] > 62)
	{
		NSMutableString*	mutableString = [NSMutableString stringWithString:name];
		
		[mutableString deleteCharactersInRange:NSMakeRange(62, [name length] - 62)];
		volumeName = [NSString stringWithString:mutableString];
	}
	else
		volumeName = name;
	
	return volumeName;
}

- (NSString*)convertToUFSName:(NSString*)name
{
	NSMutableCharacterSet*	badChars;
	NSScanner*				scanner;
	NSString*				volumeName = nil;
	
	badChars = [[[NSMutableCharacterSet alloc] init] autorelease];
	[badChars addCharactersInString:@"/"];

	if (!name || ![name length])
		return nil;
	
	scanner = [NSScanner scannerWithString:name];
	[scanner setCaseSensitive:NO];
	
	if ([scanner scanUpToCharactersFromSet:badChars intoString:&volumeName])
	{
		if (![scanner isAtEnd] && [scanner scanCharactersFromSet:badChars intoString:nil])
			return nil;
		
		if ([volumeName length] > 63)
		{
			NSMutableString*	mutableString = [NSMutableString stringWithString:volumeName];
			
			[mutableString deleteCharactersInRange:NSMakeRange(63, [volumeName length] - 63)];
			volumeName = [NSString stringWithString:mutableString];
		}
	}
	
	return volumeName;
}

- (PGPUInt64)size
{
	PGPUInt32           multiplier = 0;
	PGPUInt64           returnValue;
	NSMenuItem*			selectedItem = [mSizeUnits selectedItem];
	NSDecimalNumber*    size;
	NSDecimalNumber*    temp;
	NSDecimalNumber*    result;
	NSString*           multiplierString; 
	
	if (![mSize stringValue] || ![[mSize stringValue] length])
		return 0;
	
	if ([selectedItem tag] == kPGPdiskCreationSizeUnits_KB)
		multiplier = 1024;
	else if ([selectedItem tag] == kPGPdiskCreationSizeUnits_MB)
		multiplier = 1024 * 1024;
	else
		multiplier = 1024 * 1024 * 1024;
	
	multiplierString = [NSString stringWithFormat:@"%u", multiplier];
	
	size = [NSDecimalNumber decimalNumberWithString:[mSize stringValue]];
	temp = [NSDecimalNumber decimalNumberWithString:multiplierString];
	result = [size decimalNumberByMultiplyingBy:temp];
	
	returnValue = [result doubleValue];
	return returnValue;
}

- (PGPdiskEncryptionAlgorithm)cipher
{
	return [[mCipher selectedItem] tag];
}

- (PGPdiskFilesystemType)format
{
	return [[mFormat selectedItem] tag];
}

- (const char*)filename
{
	if (mFilename)
		return [mFilename UTF8String];
	
	return NULL;
}

- (const char*)volumeName
{
	if (mVolumeName)
		return [mVolumeName UTF8String];
	
	return NULL;
}

- (const char*)username
{
	if ([[mUserType selectedItem] tag] == kPGPdiskCreationAuthType_Passphrase)
		return [mUsername stringValue] ? [[mUsername stringValue] UTF8String] : NULL;
	
	return NULL;
}

- (const char*)passphrase
{
	return mPassphrase;
}

- (PGPKeyDBObjRef)key
{
	if ([[mUserType selectedItem] tag] == kPGPdiskCreationAuthType_PubKey)
		return mKeyDBObj;
	
	return NULL;
}

- (PGPKeyID)keyID;
{
	return mKeyID;
}

- (BOOL)hideExtension
{
	return mHideExtension;
}

- (IBAction)userTypeChanged:(id)sender
{
	NSMenuItem*			selectedItem = [mUserType selectedItem];
	PGPdiskController*	diskController = [PGPdiskController sharedController];
	
	if ([selectedItem tag] != mLastSelectedAuthType)
	{
		if ([[mUserType selectedItem] tag] == kPGPdiskCreationAuthType_Passphrase)
		{
			[mName setStringValue:kPGPdiskString_Username];
			[mAuthButton setTitle:kPGPdiskString_SetPassphrase];
			[mAuthButton setTarget:self];
			[mAuthButton setAction:@selector(setPassphrase:)];
			
            mKeyDBObj = NULL;
            [mKeyBox removeFromSuperview];
			[mUsername setStringValue:@""];
			[mAuthButton setEnabled:NO];	/* Auto-enabled when the username is typed in */
			[[mWindow contentView] addSubview:mUsername];
			[self setupKeyViewLoop];
			
            [mWindow makeFirstResponder:mUsername];
		}
		else
		{
			pgpAssert([[mUserType selectedItem] tag] == kPGPdiskCreationAuthType_PubKey);
			
            if (!diskController || !PGPKeyDBRefIsValid([diskController pgpKeyDB]))
            {
                NSBeginAlertSheet(kPGPdiskString_MissingKeyrings,
                    kPGPdiskString_OK,      /* defaultButton */
                    nil,                    /* alternateButton */
                    nil,                    /* otherButton */
                    mWindow,				/* docWindow */
                    nil,                    /* modalDelegate */
                    nil,                    /* willEndSelector */
                    nil,                    /* didEndSelector */
                    nil,                    /* contextInfo */
                    kPGPdiskString_MissingKeyringsMsg);

                [mUserType removeItemWithTitle:[selectedItem title]];
            }
            else
            {
				[self clearPassphrase:nil];
				
				[mName setStringValue:kPGPdiskString_PublicKey];
				[mAuthButton setTitle:kPGPdiskString_SelectKey];
				[mAuthButton setTarget:self];
				[mAuthButton setAction:@selector(selectKey:)];
				
                [mUsername removeFromSuperview];
				[mAuthButton setEnabled:YES];
				[mKeyIcon setImage:nil];
				[mKeyUserID setStringValue:@""];
				[[mWindow contentView] addSubview:mKeyBox];
				[self setupKeyViewLoop];
            }
		}
		
		mLastSelectedAuthType = [selectedItem tag];
	}
}

- (IBAction)stepperPressed:(id)sender
{   
    NSTextField*    textField = nil;
    NSStepper*      stepper = nil;
    int             increment;
    int             currentSize;
    int             stepperValue;

	textField = mSize;
	stepper = mStepper;

    if (stepper && textField)
    {
        currentSize = [textField intValue];     
        stepperValue = [stepper intValue];
   
        if (stepperValue < 0)
            increment = -1;
        else
            increment = 1;

        [stepper setIntValue:0];
   
        if ((currentSize + increment) < 1)
            [textField setIntValue:1];
        else
            [textField setIntValue:(currentSize + increment)];
    }
}

- (IBAction)selectKey:(id)sender
{
	PGPError        	err = kPGPError_NoErr;
	PGPdiskController*	diskController = [PGPdiskController sharedController];
	PGPContextRef   	context = [diskController pgpContext];
	PGPKeyDBRef     	keyDB = [diskController pgpKeyDB];
	PGPPrefRef			prefRef = [diskController pgpPrefRef];
	PGPKeySetRef    	secretSet = kInvalidPGPKeySetRef;
	PGPFilterRef    	secretFilter = kInvalidPGPFilterRef;
		
	err = PGPNewKeyDBObjBooleanFilter(context, kPGPKeyProperty_IsSecret, TRUE, &secretFilter); CKERR;
	err = PGPFilterKeySet(PGPPeekKeyDBRootKeySet(keyDB), secretFilter, &secretSet); CKERR;
			
	PGPclBeginKeySelectSheet(kPGPdiskString_KeySelectPrompt,
		mWindow,
		prefRef,
		secretSet,  
		keyDB,      
		kPGPclKeySelect_SingleKey | kPGPclKeySelect_FreeKeySet,
		self,       
		@selector(keySelectDidEnd:selectedKeys:));
					
done:               
	if (PGPFilterRefIsValid(secretFilter))
		PGPFreeFilter(secretFilter);
}

- (IBAction)setPassphrase:(id)sender
{
	PGPError			err;
	PGPdiskController*	diskController = [PGPdiskController sharedController];
	PGPContextRef		context = [diskController pgpContext];
	char*				passphrase = NULL;
	
	err = PGPConfirmationPassphraseDialog(context,
				PGPOUIOutputPassphrase(context, &passphrase),
				PGPOUIMinimumPassphraseLength(context, kPGPdiskCreationMinPassphraseLen),
				PGPOUIShowPassphraseQuality(context, TRUE),
				PGPOLastOption(context));
	
	if (passphrase)
	{
		mPassphrase = passphrase;
		
		[mAuthButton setTitle:kPGPdiskString_ClearPassphrase];
		[mAuthButton setTarget:self];
		[mAuthButton setAction:@selector(clearPassphrase:)];
	}
}

- (IBAction)clearPassphrase:(id)sender
{
	if (mPassphrase)
	{
		PGPLog((@"Passphrase cleared"));
		
		PGPFreeData(mPassphrase);
		mPassphrase = NULL;
	}
	
	[mAuthButton setTitle:kPGPdiskString_SetPassphrase];
	[mAuthButton setTarget:self];
	[mAuthButton setAction:@selector(setPassphrase:)];
}

- (IBAction)create:(id)sender
{
	NSSavePanel*	savePanel;
	NSString*		filename;
	PGPBoolean		hideExtension = FALSE;
	
	if ([self validateFields])
	{
		[[PGPdiskController sharedController] setDiskEngineIsBusy:YES];
		
		savePanel = [NSSavePanel savePanel];
		
		PGPclHideFileExtensions(&hideExtension);
		
		if (hideExtension)
			filename = @"Untitled";
		else
			filename = @"Untitled.pgd";
		
		[savePanel setDelegate:self];
		[savePanel setRequiredFileType:@"pgd"];
		[savePanel setCanSelectHiddenExtension:YES];
		[savePanel beginSheetForDirectory:NSHomeDirectory()
						file:filename
						modalForWindow:mWindow
						modalDelegate:self
						didEndSelector:@selector(savePanelDidEnd:returnCode:contextInfo:)
						contextInfo:savePanel];
	}
}

- (IBAction)close:(id)sender
{
	[mWindow close];
}

- (IBAction)stopCreation:(id)sender
{
	sContinue = NO;
}

- (void)diskCreationThread
{
	NSAutoreleasePool*      pool = [[NSAutoreleasePool alloc] init];
	PGPdiskController*		diskController = [PGPdiskController sharedController];
	PGPError                err;
	PGPContextRef			context = kInvalidPGPContextRef;
	PGPKeyID				keyID;
	PGPKeyDBObjRef          key = [self key];
	PGPdiskContextRef		diskContext = kInvalidPGPdiskContextRef;
	PGPdiskDiskRef			newDisk = NULL;
	BOOL					haveKeyID = NO;
	
	if (IsPGPError(err = PGPNewContext(kPGPsdkAPIVersion, &context)))
	{
		PGPLog((@"PGPNewContext returned err: %d", err));
		goto done;
	}
	
	if (IsPGPError(err = PGPdiskNewContext(context, kPGPdiskAPIVersion, &diskContext)))
	{
		PGPLog((@"PGPdiskNewContext returned err: %d", err));
		goto done;
	}
	
	if ([self key])
	{
		keyID = [self keyID];
		haveKeyID = YES;
		
		if (IsPGPError(err = PGPdiskLoadKeyrings(diskContext, [diskController pgpPubRingPath], [diskController pgpSecRingPath])))
		{
			PGPLog((@"PGPdiskLoadKeyrings returned err: %d", err));
			goto done;
		}
	}
	
	err = PGPdiskCreateDisk(diskContext,
							[self filename],
							[self size],
							"",
							[self cipher],
							haveKeyID ? TRUE : FALSE,
							haveKeyID ? &keyID : NULL,
							[self username],
							key ? NULL : [self passphrase],
							newdisk_callback,
							mProgress,
							&newDisk);
	
	sContinue = YES;	/* Reset it for the next time */
	
	if (IsntPGPError(err))
	{
		[mStop setEnabled:NO];
		[mProgressCaption setStringValue:kPGPdiskString_FormattingPGPdisk];
		[mProgress setIndeterminate:YES];
		[mProgress startAnimation:nil];
		[mProgressWindow display];
		
		err = PGPdiskFormatDisk(newDisk, [self passphrase], [self volumeName], [self format]);
		
		[mProgress stopAnimation:nil];
		[mProgressWindow display];
		
		[NSThread sleepUntilDate:[NSDate dateWithTimeIntervalSinceNow:0.5]];
		
		err = PGPdiskCloseDisk(newDisk);
		
		[mProgressCaption setStringValue:kPGPdiskString_MountingPGPdisk];
		[mProgress startAnimation:nil];
		[mProgressWindow display];
	}
	
done:
	if (PGPdiskContextRefIsValid(diskContext))
		PGPdiskFreeContext(diskContext);
	
	if (PGPContextRefIsValid(context))
		PGPFreeContext(context);
	
	[NSApp endSheet:mProgressWindow returnCode:err];
	
	[pool release];
}

- (void)displayFailureAlert:(NSNumber*)errNum
{
	PGPError		err = kPGPError_UnknownError;
	
	if (errNum)
		err = [errNum intValue];

	if (err != kPGPError_UserAbort)
	{
		NSRunAlertPanel(kPGPdiskString_UnknownCreationFailure,
			kPGPdiskString_UnknownCreationFailureMsg,
			kPGPdiskString_OK,
			nil,
			nil);
	}
}

- (void)openDisk:(NSString*)path
{
	[[PGPdiskController sharedController] handleDocument:path ofType:kPGPDiskImageFileType];
}

- (void)closeWindow
{
	[self close:nil];
}

/*
This method will attempt to convert the specified filename (name) into a suitable
volume name for the specified format (supported formats: HFS, HFS+, UFS, MSDOS).

If a suitable conversion can't be made, the default volume name is returned.
*/

- (NSString*)convertToVolumeName:(NSString*)name forFormat:(PGPUInt32)format
{
	NSString*		volumeName = nil;
	NSScanner*		scanner;

	scanner = [NSScanner scannerWithString:name];
	
	/* Volume names beginning with '.' are reserved for system use. */
	if ([scanner scanString:@"." intoString:nil])
		return [NSString stringWithString:kPGPdiskString_DefaultVolumeName];
	
	switch (format)
	{
		case kPGPdiskFilesystemType_FAT16:
			volumeName = [self convertToMSDOSName:name];
			break;
		case kPGPdiskFilesystemType_HFS:
			volumeName = [self convertToHFSName:name];
			break;
		case kPGPdiskFilesystemType_HFSPlusJournaled:
			/* Fall through */
		case kPGPdiskFilesystemType_HFSPlus:
			volumeName = [self convertToHFSPlusName:name];
			break;
		case kPGPdiskFilesystemType_UFS:
			volumeName = [self convertToUFSName:name];
			break;
		default:
			volumeName = [NSString stringWithString:kPGPdiskString_DefaultVolumeName];
			break;
	}
	
	if (!volumeName)
		volumeName = [NSString stringWithString:kPGPdiskString_DefaultVolumeName];
	
	return volumeName;
}

- (void)mountNewDisk:(CPGPdiskObject*)disk
{
	PGPError		err;
	
	if (!disk)
		return;
	
	if (mPassphrase)
	{
		err = [disk mountWithPassphrase:mPassphrase];
		
		if (IsPGPError(err))
			[[PGPdiskController sharedController] displayMountFailedAlertWithError:err forDisk:disk];
	}
}

- (void)keySelectDidEnd:(PGPError)resultCode selectedKeys:(void*)keySetPtr
{
    PGPError        err;  
    PGPKeyIterRef   keyIter;
    PGPKeyDBObjRef  keyDBObj;
    PGPKeySetRef    keySet = (PGPKeySetRef) keySetPtr;
	NSString*		userID = nil;
   
    if (IsntPGPError(resultCode) && PGPKeySetRefIsValid(keySet))
    {
        if (IsntPGPError(err = PGPNewKeyIterFromKeySet(keySet, &keyIter)))
        {
            if (IsntPGPError(err = PGPKeyIterNextKeyDBObj(keyIter, kPGPKeyDBObjType_Key, &keyDBObj)))
            {
                mKeyDBObj = keyDBObj;

                [mKeyIcon setImage:[[CPGPIcons shared] getKeyDBObjIcon:mKeyDBObj]];

				userID = PGPclGetPrimaryUserIDName(keyDBObj);
				
				if (userID)
				{
					NSDictionary*   attributes = [NSDictionary dictionaryWithObjectsAndKeys:
										[NSColor blackColor], NSForegroundColorAttributeName, nil];

					[[mKeyUserID cell] setAttributedStringValue:[[[NSAttributedString alloc]
											initWithString:userID attributes:attributes] autorelease]];
				}
				else
					[mKeyUserID setStringValue:@""];
            }

            PGPFreeKeyIter(keyIter);
        }

        PGPFreeKeySet(keySet);
    }
}

- (void)alertDidEnd:(NSWindow*)sheet returnCode:(int)returnCode contextInfo:(void*)contextInfo
{
	/* If an NSView is stuffed into contextInfo, that view is made the first responder. */
	if (contextInfo)
		[mWindow makeFirstResponder:(id)contextInfo];
}

- (void)savePanelDidEnd:(NSWindow*)sheet returnCode:(int)returnCode contextInfo:(void*)contextInfo
{
	NSSavePanel*		savePanel = (NSSavePanel*) contextInfo;
	NSString*			filename;
	NSString*			proposedVolName;
	CPGPdiskObject*		selectedDisk;
	PGPdiskController*	diskController = [PGPdiskController sharedController];
	PGPContextRef		context = [diskController pgpContext];
	PGPError			err = kPGPError_NoErr;
	
	if (returnCode == NSOKButton)
	{
		mFilename = [[savePanel filename] retain];
		mHideExtension = [savePanel isExtensionHidden];
		
		proposedVolName = [[mFilename lastPathComponent] stringByDeletingPathExtension];
		mVolumeName = [[self convertToVolumeName:proposedVolName forFormat:[[mFormat selectedItem] tag]] retain];
		
		/* If the user chose to overwrite the currently selected disk, we first deselect the disk (to close it) */
		selectedDisk = [[PGPdiskController sharedController] selectedDisk];
		
		if (selectedDisk)
		{
			if ([[selectedDisk diskPath] isEqualToString:mFilename])
				[[PGPdiskController sharedController] selectDisk:nil];
		}
		
		[sheet orderOut:nil];
				
		filename = [mFilename lastPathComponent];
		
		if (mHideExtension)
			filename = [filename stringByDeletingPathExtension];
		
		filename = [NSString stringWithFormat:@"\"%@\"", filename];
		
		[mCancel setEnabled:NO];
		[mCreate setEnabled:NO];
		[mProgress setIndeterminate:NO];
		[mProgress setDoubleValue:0.0];
		[mProgressCaption setStringValue:kPGPdiskString_CreatingPGPdisk];
		[mFile setStringValue:filename];
		
		if ([self key])
		{
			err = PGPGetKeyID([self key], &mKeyID);
			pgpAssertNoErr(err);
		}
		
		if ([[mUserType selectedItem] tag] == kPGPdiskCreationAuthType_PubKey)
			err = PGPKeyPassphraseDialog(context, [self key], PGPOUIOutputPassphrase(context, &mPassphrase), PGPOLastOption(context));
		
		if (IsntPGPError(err))
		{
			[NSApp beginSheet:mProgressWindow
				modalForWindow:mWindow
				modalDelegate:self
				didEndSelector:@selector(creationDidEnd:returnCode:contextInfo:)
				contextInfo:nil];
			
			[NSThread detachNewThreadSelector:@selector(diskCreationThread) toTarget:self withObject:nil];
		}
		else
		{
			[mCancel setEnabled:YES];
			[mCreate setEnabled:YES];
			[[PGPdiskController sharedController] setDiskEngineIsBusy:NO];
		}
	}
	else
		[[PGPdiskController sharedController] setDiskEngineIsBusy:NO];
}

- (void)creationDidEnd:(NSWindow*)sheet returnCode:(int)returnCode contextInfo:(void*)contextInfo
{
	NSFileManager*		fileManager = [NSFileManager defaultManager];
	NSString*			filename = [NSString stringWithUTF8String:[self filename]];
	NSConnection*		connection;
	PGPdiskController*	diskController = [PGPdiskController sharedController];
	CPGPdiskObject*		disk;
	id					proxy;
	
	if (IsPGPError(returnCode))
		PGPLog((@"creationDidEnd with returnCode == %d", returnCode));
	
	pgpAssert(mServerConnection);
	
	[[PGPdiskController sharedController] setDiskEngineIsBusy:NO];
	
	if ([mProgress isIndeterminate])
		[mProgress stopAnimation:nil];
	else
		[mProgress setDoubleValue:0.0];
	
	[mProgressWindow close];
	
	connection = [NSConnection connectionWithReceivePort:[mServerConnection sendPort]
									sendPort:[mServerConnection receivePort]];
	
	proxy = [connection rootProxy];
	[proxy setProtocolForProxy:@protocol(PGPdiskCreationProtocol)];
	
	if (IsPGPError(returnCode))
	{
		if ([[mUserType selectedItem] tag] == kPGPdiskCreationAuthType_Passphrase)
			[self clearPassphrase:nil];
		
		if ([fileManager fileExistsAtPath:filename])
			[fileManager removeFileAtPath:filename handler:nil];
		
		[proxy displayFailureAlert:[NSNumber numberWithInt:returnCode]];
		
		[mCreate setEnabled:YES];
		[mCancel setEnabled:YES];
	}
	else
	{
		[fileManager changeFileAttributes:[NSDictionary dictionaryWithObject:[NSNumber numberWithBool:[self hideExtension]] forKey:NSFileExtensionHidden] atPath:filename];
		
		if (disk = [diskController diskWithPath:mFilename])
			[proxy mountNewDisk:disk];
		else
		{
			mDisk = [[CPGPdiskObject diskAtPath:mFilename] retain];
			[mDisk setController:diskController];
			[proxy mountNewDisk:mDisk];
		}
		
		[disk setDiskPath:filename];	/* Updates the hide extension status */
		
		[proxy mountNewDisk:disk];
		[proxy openDisk:filename];
		
		[mSize setEnabled:NO];
		[mStepper setEnabled:NO];
		[mSizeUnits setEnabled:NO];
		[mCipher setEnabled:NO];
		[mUserType setEnabled:NO];
		[mUsername setEnabled:NO];
		[mAuthButton setEnabled:NO];
		[mCreate setEnabled:NO];
		[mCancel setEnabled:NO];
				
		[[mWindow contentView] setNeedsDisplay:YES];
		[[mWindow contentView] displayIfNeeded];
		
		[proxy closeWindow];
	}
}

- (void)textDidChange:(NSNotification*)notification
{
	if ([mUsername superview])
	{
		if ([mUsername stringValue] && [[mUsername stringValue] length])
			[mAuthButton setEnabled:YES];
		else
			[mAuthButton setEnabled:NO];
	}
}

- (BOOL)panel:(id)sender isValidFilename:(NSString*)filename
{
	NSFileManager*      fileManager = [NSFileManager defaultManager];
	NSString*           dir;
	BOOL                isDirectory = NO;
	BOOL                isTooLarge = NO;
	struct statfs       buf;
	PGPInt32            fsType = 0;
	PGPUInt64           maxSize;
	
	memset(&buf, '\0', sizeof(struct statfs));
	
	dir = [filename stringByDeletingLastPathComponent];
	
	if (statfs([dir cString], &buf) == 0)
	{
		/*
		It's slightly difficult to tell the difference between an HFS and HFS+ volume from the statfs struct.
		From playing with it for a while, it seems that the key difference is in the f_bsize member. A value
		of 4096 indicates HFS+, whereas any other value would indicate an older HFS volume.
		*/
		if (!strncmp(buf.f_fstypename, "hfs", strlen("hfs")))
		{
			if (buf.f_bsize == 4096)
				fsType = kPGPdiskHFSExt;
			else
				fsType = kPGPdiskHFS;
		}
		else if (!strncmp(buf.f_fstypename, "ufs", strlen("ufs")))
			fsType = kPGPdiskUFS;
	}
	
	switch (fsType)
	{
		case kPGPdiskHFS:	/* Max HFS file size is 2GB */
			maxSize = 2 * ((PGPUInt64) 1024 * 1024 * 1024);
	
			if ([self size] > maxSize)
				isTooLarge = YES;
	
			break;
		case kPGPdiskUFS:	/* Max UFS file size is 4GB */
			maxSize = 4 * ((PGPUInt64) 1024 * 1024 * 1024);
	
			if ([self size] > maxSize)
				isTooLarge = YES;
	
			break;
		default:
			break;
	}
	
	if (isTooLarge)
	{
		NSRunAlertPanel(kPGPdiskString_DiskTooLarge,
			fsType == kPGPdiskHFS ? kPGPdiskString_DiskTooLargeHFSMsg : kPGPdiskString_DiskTooLargeUFSMsg,
			kPGPdiskString_OK,
			nil,
			nil);
	
		return NO;
	}
	
	if (fsType)     /* Only set if statfs didn't fail */
	{
		PGPUInt64   spaceAvailable = ((PGPUInt64) buf.f_bfree * buf.f_bsize);
		PGPUInt64   diskSize = [self size];
		double      reducedSize = 0;
		NSString*   sizeUnits = nil;
		BOOL        canCreateDisk = YES;
	
		if (diskSize > spaceAvailable)
		{
			if (spaceAvailable > ((PGPUInt64) 1024 * 1024 * 1024))
			{
				reducedSize = (double) ((double) spaceAvailable / ((PGPUInt64) 1024 * 1024 * 1024));
				sizeUnits = @"GB";
			}
			else if (spaceAvailable > (1024 * 1024))
			{
				reducedSize = (double) ((double) spaceAvailable / (1024 * 1024));
				sizeUnits = @"MB";
			}
			else
			{
				reducedSize = (double) ((double) spaceAvailable / 1024);
				sizeUnits = @"KB";
	
				if (reducedSize < kPGPdiskMinDiskKb)
					canCreateDisk = NO;
			}
	
			if (canCreateDisk)
			{
				NSRunAlertPanel(kPGPdiskString_VolumeNotLargeEnough,
					[NSString stringWithFormat:kPGPdiskString_VolumeNotLargeEnoughMsg, reducedSize, sizeUnits],
					kPGPdiskString_OK,
					nil,
					nil);
			}
			else
			{
				NSRunAlertPanel(kPGPdiskString_CantCreateMinSizeDisk,
					kPGPdiskString_CantCreateMinSizeDiskMsg,
					kPGPdiskString_OK,
					nil,
					nil);
			}
	
			return NO;
		}
	}
	
	if ([fileManager fileExistsAtPath:dir isDirectory:&isDirectory])
	{
		if ([fileManager isWritableFileAtPath:dir])
			return YES;
		else
		{
			NSRunAlertPanel(kPGPdiskString_CantWriteToLocation,
				kPGPdiskString_CantWriteToLocationMsg,
				kPGPdiskString_OK,
				nil,
				nil);
		}
	}
	else
	{
		NSRunAlertPanel(kPGPdiskString_InvalidDiskLocation,
			kPGPdiskString_InvalidDiskLocationMsg,
			kPGPdiskString_OK,
			nil,
			nil);
	}
	
	return NO;
}

- (BOOL)windowShouldClose:(id)sender
{
	return YES;
}

- (void)windowWillClose:(NSNotification*)notification
{
	if (mServerConnection)
	{
		[mServerConnection release];
		mServerConnection = nil;
	}
	
	[self autorelease];
}

- (void)dealloc
{
	PGPLog((@"CPGPdiskCreation deallocated"));
	
	if (self == sSharedInstance)
		sSharedInstance = nil;
	
	[[NSNotificationCenter defaultCenter] removeObserver:self];
	
	if (mDisk)
		[mDisk release];
	
	if (mVolumeName)
		[mVolumeName release];
	
	if (mFilename)
		[mFilename release];
	
	if (mPassphrase)
		PGPFreeData(mPassphrase);
	
	if (mIntFormatter)
		[mIntFormatter release];
	
	if (mUsername)
		[mUsername release];
	
	if (mKeyBox)
		[mKeyBox release];
	
	[super dealloc];
}

@end

@implementation PGPdiskController (PGPdiskCreationExtensions)
        
- (IBAction)newDisk:(id)sender
{   
    CPGPdiskCreation*   diskCreator;
	
	[NSApp activateIgnoringOtherApps:YES];
	
    if (!mDiskContext) 
        [self createDiskContext];

	diskCreator = [CPGPdiskCreation sharedCreator];
	
    /* The act of displaying the disk creation assistant retains the object */
	[diskCreator run];
}

@end