/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation
    All rights reserved.

    $Id: CPGPdiskReEncryption.m,v 1.4 2002/11/09 03:05:39 jeffc Exp $
____________________________________________________________________________*/

#import "CPGPdiskReEncryption.h"
#import "CPGPdiskProgressPanel.h"
#import "PGPdiskController.h"
#import <PGPclient/pgpClientLibCocoa.h>

#define kPGPdiskString_ReEncryptingDiskCaption	LSPD(@"Re-Encrypting PGPdisk", @"ReEncryption caption")

static BOOL						sReEncryptionAborted = NO;

static PGPError sReEncryptCallback(PGPUInt32 percent, void* userval)
{
    CPGPdiskProgressPanel*	panel = [CPGPdiskProgressPanel sharedPanel];
    NSProgressIndicator*	progressIndicator = [panel progressIndicator];
    
    if (sReEncryptionAborted)
        return kPGPError_UserAbort;
    
    if ([[panel window] isVisible])
	{
		if (progressIndicator)
		{
			[progressIndicator setDoubleValue:percent];
			[progressIndicator displayIfNeeded];
		}
	}
	
	return kPGPError_NoErr;
}

@implementation CPGPdiskReEncryption

- (id)init
{
	if (!mInited)
	{
		[super init];
		
		mInited = YES;
	}
	
	return self;
}

- (void)setAlgorithm:(PGPdiskEncryptionAlgorithm)algorithm
{
	mAlgorithm = algorithm;
}

- (void)setDiskPath:(NSString*)path
{
	if (mDiskPath)
	{
		[mDiskPath release];
		mDiskPath = nil;
	}
	
	if (path)
		mDiskPath = [path retain];
}

- (void)setPassphrase:(NSData*)passphrase
{
	if (mPassphrase)
	{
		[mPassphrase release];
		mPassphrase = nil;
	}
	
	if (passphrase)
		mPassphrase = [passphrase retain];
}

- (void)setServerConnection:(NSConnection*)connection
{
	if (mConnection)
	{
		[mConnection release];
		mConnection = nil;
	}
	
	if (connection)
		mConnection = [connection retain];
}

- (void)setPubRingPath:(char*)pubPath
{
	strncpy(mPubRingPath, pubPath, PATH_MAX);
}

- (void)setSecRingPath:(char*)secPath
{
	strncpy(mSecRingPath, secPath, PATH_MAX);
}

- (void)reEncryptDiskWithPorts:(NSArray*)ports
{
	NSAutoreleasePool*		pool = [[NSAutoreleasePool alloc] init];
	NSConnection*			connection;
    CPGPdiskProgressPanel*	panel = [CPGPdiskProgressPanel sharedPanel];
	NSString*				caption;
	id						proxy;
	PGPError				err = kPGPError_NoErr;
	PGPContextRef			context = kInvalidPGPContextRef;
	PGPdiskContextRef		diskContext = kInvalidPGPdiskContextRef;
	const char*				passphrase;
	
	if (!ports || (mAlgorithm == kPGPdiskInvalidAlgorithm) || !mPassphrase)
	{
		PGPLog((@"Failed to re-encrypt disk due to invalid arguments"));
		goto done;
	}
	
	if (IsPGPError(err = PGPNewContext(kPGPsdkAPIVersion, &context)))
	{
		PGPLog((@"PGPNewContext returned err: %d", err));
		goto done;
	}
	
	if (IsPGPError(err = PGPdiskNewContext(context, kPGPdiskAPIVersion, &diskContext)))
	{
		PGPLog((@"PGPdiskNewContext returned err: %d", err));
		goto done;
	}
	
	if (IsPGPError(err = PGPdiskLoadKeyrings(diskContext, mPubRingPath, mSecRingPath)))
	{
		PGPLog((@"PGPdiskLoadKeyrings returned err: %d", err));
		goto done;
	}
	
	if (IsPGPError(err = PGPdiskOpenDisk(diskContext, [mDiskPath UTF8String], FALSE, &mDiskRef)))
	{
		PGPLog((@"PGPdiskOpenDisk returned err: %d", err));
		goto done;
	}
	
	sReEncryptionAborted = NO;
	
	connection = [NSConnection connectionWithReceivePort:[ports objectAtIndex:0] sendPort:[ports objectAtIndex:1]];
	proxy = [connection rootProxy];
	[proxy setProtocolForProxy:@protocol(CPGPdiskReEncryptionClient)];
	passphrase = (const char*) [mPassphrase bytes];
	caption = [NSString stringWithFormat:@"%@ \"%@\"",
					kPGPdiskString_ReEncryptingDiskCaption,
					[[mDiskPath lastPathComponent] stringByDeletingPathExtension]];
					
	[panel setCaption:caption];
	
	err = PGPdiskReEncryptDisk(mDiskRef,
				passphrase,
				mAlgorithm,
				sReEncryptCallback,
				NULL);

	if ([[panel window] isVisible])
		[panel endSheetWithReturnCode:err];

	if (IsPGPError(err))
	{
		PGPLog((@"Failed to re-encrypt disk: %@ (err=%d)", mDiskPath, err));
		
		[proxy failedToReEncryptDisk:mDiskPath];
	}
	else
		[proxy completedReEncryptionOfDisk:mDiskPath];
	
    [panel autorelease];
    [self autorelease];
	
done:
	if (PGPdiskDiskRefIsValid(mDiskRef))
		PGPdiskCloseDisk(mDiskRef);
	
	if (PGPdiskContextRefIsValid(diskContext))
		PGPdiskFreeContext(diskContext);
	
	if (PGPContextRefIsValid(context))
		PGPFreeContext(context);
	
	[pool release];
}

- (void)abortReEncryption
{
	sReEncryptionAborted = YES;
}

- (void)dealloc
{
	PGPLog((@"Deallocated CPGPdiskReEncryption"));

	sReEncryptionAborted = NO;

	if (mDiskPath)
		[mDiskPath release];

	if (mPassphrase)
		[mPassphrase release];
	
	if (mConnection)
		[mConnection release];
	
	[super dealloc];	
}

@end
