/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation
    All rights reserved.

    $Id: CPGPdiskUserTable.m,v 1.21 2002/09/26 21:57:37 jeffc Exp $
____________________________________________________________________________*/

#import "CPGPdiskUserTable.h"
#import "CPGPdiskUsers.h"

@implementation CPGPdiskUserTable

- (id)init
{
	if (!mInited)
	{
		[super init];
		
		mInited = YES;
	}
	
	if (!mPubKeyIcon)
	{
		mPubKeyIcon = [[NSImage alloc] initWithContentsOfFile:
			[[NSBundle bundleForClass:[self class]] pathForImageResource:kPGPdiskPubKeyUserIcon]];
	}
	
	if (!mPassphraseIcon)
	{
		mPassphraseIcon = [[NSImage alloc] initWithContentsOfFile:
			[[NSBundle bundleForClass:[self class]] pathForImageResource:kPGPdiskPassphraseUserIcon]];
	}
	
	return self;
}

- (id)initWithController:(PGPdiskController*)controller
{
	mController = controller;
	
	return [self init];
}

- (id)table
{
	return mUserTable;
}

- (void)setTable:(id)table
{
	if (table)
		mUserTable = table;
}

- (void)tableViewSelectionDidChange:(NSNotification*)notification
{
	int					row = [mUserTable selectedRow];
	CPGPdiskUser*		user;
	
	if ([mController selectedDisk] && row == -1)
	{
		[[mController disableMenuItem] setEnabled:NO];
		[[mController changePassphraseMenuItem] setEnabled:NO];
		[mController setAddUserEnabled:YES];
		[mController setRemoveUserEnabled:NO];
	}
    else
	{
		if (user = [[mController users] objectAtIndex:row])
		{
			if ([user isSymmetricKeyUser])
				[[mController changePassphraseMenuItem] setEnabled:YES];
			else
				[[mController changePassphraseMenuItem] setEnabled:NO];
		
			if ([user isAdmin])
			{
				[[mController disableMenuItem] setTitle:kPGPdiskMenuItem_DisableUser];
				[[mController disableMenuItem] setEnabled:NO];
				[mController setRemoveUserEnabled:NO];
			}
			else
			{
				if ([user isDisabled])
					[[mController disableMenuItem] setTitle:kPGPdiskMenuItem_EnableUser];
				else
					[[mController disableMenuItem] setTitle:kPGPdiskMenuItem_DisableUser];
					
				[[mController disableMenuItem] setEnabled:YES];
				[mController setAddUserEnabled:YES];
				[mController setRemoveUserEnabled:YES];
			}
		}
	}
}

- (BOOL)tableView:(NSTableView*)table shouldEditTableColumn:(NSTableColumn*)column row:(int)row
{
	NSArray*		users;
    CPGPdiskObject*	disk = [mController selectedDisk];
	CPGPdiskUser*	user;
	
	[mController updateUsersForDisk:disk];
	users = [mController users];
	
	NSParameterAssert((row >= 0) && (row < [users count]));
	if ([disk isWritable] && (user = [users objectAtIndex:row]))
	{
		if (disk && ([disk diskFormat] != kPGPdiskDiskFormat_OS9) && [user isSymmetricKeyUser])
		{
            [mController setDiskEngineIsBusy:YES];
			mEditingCell = YES;
			
			return YES;
		}
		else if (disk && ([disk diskFormat] == kPGPdiskDiskFormat_OS9))
		{
            [mController convertDisk:disk displayAlert:YES];
            mEditingCell = NO;
			
			return NO;
		}
        else
        {
            mEditingCell = NO;
			
            return NO;
        }
	}
	
	return YES;
}

- (void)tableView:(NSTableView*)table willDisplayCell:(id)cell forTableColumn:(NSTableColumn*)column row:(int)row
{
	CPGPdiskObject*		disk = [mController selectedDisk];
	CPGPdiskUser*		user;
	NSString*			identifier = [column identifier];
	
	if (!disk || [disk diskIsBusy])
		return;
	
	if (user = [[mController users] objectAtIndex:row])
	{
		if ([identifier isEqualToString:kPGPdiskUserIconColumnIdentifier])
		{
			if ([user isSymmetricKeyUser])
				[cell setImage:mPassphraseIcon];
			else if ([user isPublicKeyUser])
				[cell setImage:mPubKeyIcon];
			else
				[cell setImage:nil];
		}
		else if ([identifier isEqualToString:kPGPdiskUsernameColumnIdentifier])
		{
			[cell setStringValue:[user name]];
			
			if ([user isDisabled])
				[cell setTextColor:[NSColor grayColor]];
			else
				[cell setTextColor:[NSColor blackColor]];
		}
		else if ([identifier isEqualToString:kPGPdiskReadOnlyColumnIdentifier])
		{
			if ([user isReadOnly])
			{
				[cell setImage:[[[CPGPIcons shared] images] objectAtIndex:kPGPclImage_DotRed]];
				[cell setAlternateImage:[[[CPGPIcons shared] images] objectAtIndex:kPGPclImage_DotSelect]];
				[cell setAction:@selector(readonlyPressed:)];
			}
			else
			{
				if ([user isAdmin])
					[cell setAction:nil];
				else
					[cell setAction:@selector(readonlyPressed:)];
					
				[cell setImage:nil];
				[cell setAlternateImage:nil];
			}
		}
		else if ([identifier isEqualToString:kPGPdiskAdminColumnIdentifier])
		{
			if ([user isAdmin])
				[cell setImage:[[[CPGPIcons shared] images] objectAtIndex:kPGPclImage_DotGreen]];
			else
			{
				if ([user isReadOnly])
					[cell setAction:nil];
				else
					[cell setAction:@selector(adminPressed:)];
				
				[cell setImage:[[[CPGPIcons shared] images] objectAtIndex:kPGPclImage_DotDimmed]];
			}
		}
	}
}

- (int)numberOfRowsInTableView:(NSTableView*)tableView
{
	NSArray*	users = [mController users];
	
	if (users)
		return [users count];
	
	return 0;
}

- (id)tableView:(NSTableView*)tableView objectValueForTableColumn:(NSTableColumn*)column row:(int)row
{
	NSArray*		users = [mController users];
	NSString*		identifier = [column identifier];
	CPGPdiskObject*	disk = [mController selectedDisk];
	CPGPdiskUser*	userEntry;
	id				value = nil;
	
	if (disk && ![disk diskIsBusy])
	{
		NSParameterAssert((row >= 0) && (row < [users count]));
		if (userEntry = [users objectAtIndex:row])
		{
			if ([identifier isEqualToString:kPGPdiskUsernameColumnIdentifier])
				value = [NSString stringWithString:@""];
			else if ([identifier isEqualToString:kPGPdiskReadOnlyColumnIdentifier])
				value = [[[CPGPIcons shared] images] objectAtIndex:kPGPclImage_DotRed];
			else if ([identifier isEqualToString:kPGPdiskAdminColumnIdentifier])
				value = [[[CPGPIcons shared] images] objectAtIndex:kPGPclImage_DotDimmed];
		}
	}
	
	return value;
}

- (void)tableView:(NSTableView*)tableView setObjectValue:(id)object forTableColumn:(NSTableColumn*)column row:(int)row
{
	NSArray*			users;
	NSString*			identifier = [column identifier];
	CPGPdiskObject*		disk = [mController selectedDisk];
	CPGPdiskUser*		user;
	unsigned char*		utf8Name;
	unsigned char		shortenedName[kPGPdiskMaxUserNameSize];
	BOOL				openedDisk = NO;
	
	if (mEditingCell)
	{
        [mController setDiskEngineIsBusy:NO];
		
		if (![disk diskRef])
		{
			[disk openDiskWithContext:[mController pgpDiskContext]];
			openedDisk = YES;
		}
		
		[mController updateUsersForDisk:disk];        
		
		users = [mController users];
		
		NSParameterAssert((row >= 0) && (row < [users count]));
		if (user = [users objectAtIndex:row])
		{
			if ([identifier isEqualToString:kPGPdiskUsernameColumnIdentifier])
			{
				utf8Name = (unsigned char*) [(NSString*) object UTF8String];
				
				if (strlen(utf8Name) > kPGPdiskMaxUserNameSize)
				{
					memcpy(shortenedName, utf8Name, 126);
					shortenedName[126] = '\0';
					[user setName:[NSString stringWithUTF8String:shortenedName]];
				}
				else
					[user setName:object];
			}
		}
		
		mEditingCell = NO;
		
		if (openedDisk)
			[disk closeDisk];
	}
}

@end
