/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation
    All rights reserved.

    $Id: PGPdiskController.h,v 1.90 2002/11/14 01:32:49 jeffc Exp $
____________________________________________________________________________*/

#import <Cocoa/Cocoa.h>
#import <PGP/pgpUtilities.h>
#import <PGPclient/pgpClientLibOSX.h>
#import <PGPclient/pgpClientLibCocoa.h>
#import <PGPclient/CImageButtonCell.h>
#import <PGPclient/CImageButton.h>
#import <PGPclient/CIntegerFormatter.h>

#import "PGPdiskCommon.h"
#import "PGPBundle.h"
#import "PGPBundleDisk.h"
#import "PGPController.h"
#import "pgpDiskEngine.h"

#import "CPGPdiskUserTable.h"
#import "CPGPdiskUsers.h"
#import "CPGPdiskObject.h"

#define kPGPdiskNib							@"PGPdisk"
#define kPGPdiskToolbarItemImage			@"pgpdisk"
#define kPGPdiskToolbarItemToolTip			LSPD(@"PGPdisk", @"Toolbar item tooltip")
#define kPGPdiskDiskCellIcon				@"pgpdisk-doc.icns"
#define kPGPdiskOS9DiskCellIcon				@"smdiskdoc"
#define kPGPdiskDiskColumnIdentifier		@"PGPdisk"
#define kPGPdiskUserIconColumnIdentifier	@"Icon"
#define kPGPdiskUsernameColumnIdentifier	@"User Name"
#define kPGPdiskReadOnlyColumnIdentifier	@"Read-Only"
#define kPGPdiskAdminColumnIdentifier		@"Admin"
#if PGP_DEBUG
	#define kPGPdiskDiskCheckInterval		0.2
#else
	#define kPGPdiskDiskCheckInterval		1.5
#endif
#define kPGPdiskEngineLaunchPath			@"Contents/Resources/PGPdiskEngine"
#define kPGPdiskEngineConnectRetries		10
#define kPGPdiskMinimumPassphraseLength		8
#define kPGPdiskDefaultUnmountTimeout		15
#define kPGPdiskUsesGlobalUnmountTimeout	-1
#define kPGPdiskDisableAutoUnmount			0

#define kPGPdisk_WindowTitle				LSPD(@"PGPdisk", @"Main window title")
#define kPGPdiskMenu_Disk					LSPD(@"Disk", @"Disk menu title")
#define kPGPdiskMenu_File					LSPD(@"File", @"File menu title")
#define kPGPdiskMenuItem_New				LSPD(@"New", @"New menu item title")
#define kPGPdiskMenuItem_Open				LSPD(@"Open...", @"Open menu item title")
#define kPGPdiskMenuItem_UnmountDisk		LSPD(@"Unmount Disk", @"Unmount disk menu item title")
#define kPGPdiskMenuItem_Properties			LSPD(@"Properties", @"Properties menu item title")
#define kPGPdiskMenuItem_AddUser			LSPD(@"Add User", @"Add User menu item title")
#define kPGPdiskMenuItem_RemoveUser			LSPD(@"Remove User", @"Remove User menu item title")
#define kPGPdiskMenuItem_DisableUser		LSPD(@"Disable User", @"Disable User menu item title")
#define kPGPdiskMenuItem_EnableUser			LSPD(@"Enable User", @"Enable User menu item title")
#define kPGPdiskMenuItem_ChangePassphrase	LSPD(@"Change Passphrase", @"Change Passphrase menu item title")
#define kPGPdiskMenuItem_Convert			LSPD(@"Convert...", @"Convert menu item title")

enum
{
	kPGPdiskCommand_Passphrase	= 4000,
	kPGPdiskCommand_PublicKey
};

@class NSDictionary, NSToolbarItem;
@class PGPdiskPreferences, CPGPdiskUserTable, CPGPdiskCreation, CPGPdiskObject;

@interface PGPdiskController : NSObject <PGPBundle, PGPBundleDisk>
{
	id <PGPController>				mController;
	
	PGPdiskPreferences*				mDiskPreferences;
	CPGPdiskUserTable*				mUserTableSource;
	PGPContextRef					mPGPContext;
	PGPdiskContextRef				mDiskContext;

	NSBundle*						mBundle;
	NSMutableArray*					mDisks;
	NSMutableArray*					mUsers;		/* Actually contained in mDisks, but brought forward when a disk is selected */
    NSMutableArray*					mMountedDisks;
	NSTask*							mDiskEngineTask;
	NSTimer*						mInstallTimer;
	NSTimer*						mEngineTimer;
	NSTimer*						mDiskCheckerTimer;
	NSImage*						mMountImage;
	NSImage*						mUnmountImage;
	NSImage*						mDiskIcon;
	NSImage*						mOS9DiskIcon;
	CIntegerFormatter*				mIntFormatter;
	char							mPubKeyringPath[PATH_MAX];
	char							mSecKeyringPath[PATH_MAX];
	NSResponder*					mLastResponder;
	BOOL							mAddUser;
	BOOL							mRemoveUser;
	
	IBOutlet NSWindow*				mWindow;
	IBOutlet NSPopUpButton*			mDiskPopup;
	
	/* PGPdisk Menu */
	IBOutlet NSMenu*				mDiskDockMenu;
	IBOutlet NSMenu*				mDiskMenu;
	IBOutlet NSMenuItem*			mDiskItemAddUser;
	IBOutlet NSMenuItem*			mDiskItemRemoveUser;
	IBOutlet NSMenuItem*			mDiskItemDisable;
	IBOutlet NSMenuItem*			mDiskItemChangePassphrase;
	
	/* Buttons */
	IBOutlet NSButton*				mMountButton;
	
	/* Tab View */
	IBOutlet NSTabView*				mTabView;
	
	/* General Info */
	IBOutlet NSTextField*			mLocation;
	IBOutlet NSButton*				mReveal;
	IBOutlet NSTextField*			mSize;
	IBOutlet NSPopUpButton*			mCipher;
	IBOutlet NSButton*				mReEncrypt;
	
	/* Mount Options */
	IBOutlet NSButton*				mOverrideInactivityTimer;
	IBOutlet NSButtonCell*			mUseCustomTimeout;
	IBOutlet NSTextField*			mCustomTimeout;
	IBOutlet NSStepper*				mStepper;
	IBOutlet NSButtonCell*			mDisableTimer;
	
	/* TableViews */
	IBOutlet NSTableView*			mUserTable;
	
    BOOL							mDiskEngineIsBusy;
	BOOL							mInited;
	BOOL							mLoadedNib;
	BOOL							mIsInstalled;
}

+ (id)sharedController;
+ (NSString*)identifier;

- (id)init;
- (id)initWithController:(id)controller;

- (id)loadNib:(NSString*)nibFile;

- (id)prefsController;
- (NSWindow*)window;
- (NSImage*)menuImage;
- (NSMenu*)dockMenu;
- (NSMenuItem*)newMenuItem;

- (void)activate:(BOOL)passive;
- (void)addMenus;
- (void)handleDocument:(NSString*)docFile ofType:(NSString*)docType;
- (void)openDisk:(NSString*)path mount:(BOOL)flag;

- (id)controller;
- (void)setController:(id)controller;

- (void)setupKeyViewLoop;
- (BOOL)keyViewIsValid:(NSResponder*)keyView;
- (void)registerForPrefNotifications;
- (BOOL)releaseIfReadyToShutdown;

- (void)keyDBChanged;
- (void)passphraseCacheChanged;

- (NSMutableArray*)disks;
- (void)setDisks:(NSMutableArray*)disks;
- (NSMutableArray*)users;
- (NSMutableArray*)mountedDisks;
- (PGPContextRef)pgpContext;
- (PGPdiskContextRef)pgpDiskContext;
- (PGPPrefRef)pgpPrefRef;
- (PGPKeyDBRef)pgpKeyDB;
- (char*)pgpPubRingPath;
- (char*)pgpSecRingPath;
- (NSTableView*)userTable;
- (CPGPdiskUserTable*)userTableSource;
- (NSMenuItem*)disableMenuItem;
- (NSMenuItem*)changePassphraseMenuItem;

- (void)setAddUserEnabled:(BOOL)flag;
- (void)setRemoveUserEnabled:(BOOL)flag;

- (IBAction)openDisk:(id)sender;
- (IBAction)convert:(id)sender;
- (IBAction)reveal:(id)sender;
- (IBAction)cipherChanged:(id)sender;
- (IBAction)reEncrypt:(id)sender;
- (IBAction)overrideGlobalInactivityTimer:(id)sender;
- (IBAction)useCustomTimeout:(id)sender;
- (IBAction)disableTimer:(id)sender;
- (IBAction)stepperPressed:(id)sender;

- (void)updateTabViewForDisk:(CPGPdiskObject*)disk;
- (void)dimTabViewContents:(BOOL)flag;
- (void)openSheetDidEnd:(NSWindow*)sheet returnCode:(int)returnCode contextInfo:(void*)contextInfo;
- (void)convertSheetDidEnd:(NSWindow*)sheet returnCode:(int)returnCode contextInfo:(void*)contextInfo;

- (void)setLocationForDisk:(CPGPdiskObject*)disk;
- (void)setSizeForDisk:(CPGPdiskObject*)disk;
- (void)setCipherForDisk:(CPGPdiskObject*)disk;
- (void)setAutoUnmountOptionsForDisk:(CPGPdiskObject*)disk;

- (void)populateDiskArray;
- (void)addDisk:(CPGPdiskObject*)disk;
- (void)removeDisk:(CPGPdiskObject*)disk;
- (void)saveKnownDisksList;
- (PGPError)findDisks;
- (PGPError)findDisksAtPaths:(NSArray*)paths;
- (CPGPdiskObject*)diskWithPath:(NSString*)path;
- (CPGPdiskObject*)selectedDisk;
- (CPGPdiskUser*)selectedUser;
- (void)selectDisk:(CPGPdiskObject*)disk;

- (void)setupDiskPopup;
- (void)updateDiskPopup;
- (void)populateDiskPopup;

- (IBAction)diskPopupSelectionChanged:(id)sender;

- (BOOL)diskEngineIsBusy;
- (void)setDiskEngineIsBusy:(BOOL)flag;

- (BOOL)validateMenuItem:(id <NSMenuItem>)menuItem;

- (void)reEncryptDisk:(CPGPdiskObject*)disk withCipher:(NSNumber*)cipherNumber;
- (void)continueReEncryptingDisk:(CPGPdiskObject*)disk;
- (void)reEncryptWarningDidEnd:(NSWindow*)sheet returnCode:(int)returnCode contextInfo:(void*)contextInfo;
- (void)convertDisk:(CPGPdiskObject*)disk displayAlert:(BOOL)flag;
- (void)conversionWarningDidEnd:(NSWindow*)sheet returnCode:(int)returnCode contextInfo:(void*)contextInfo;
- (void)completedReEncryptionOfDisk:(NSString*)filename;
- (void)failedToReEncryptDisk:(NSString*)filename;
- (void)completedConversionOfDisk:(NSString*)filename;
- (void)failedToConvertDisk:(NSString*)filename;

- (BOOL)panel:(id)sender isValidFilename:(NSString*)filename;
- (BOOL)isDiskEngineRunning;
- (BOOL)createDiskContext;
- (BOOL)launchDiskEngine;

- (void)setPassphraseCaching;

- (void)setGlobalDiskTimeout:(PGPUInt32)minutes;
- (PGPUInt32)globalDiskTimeout;

- (void)setUnmountOnLogout:(PGPBoolean)flag;
- (PGPBoolean)unmountOnLogout;

- (void)setUnmountOnSleep:(PGPBoolean)flag;
- (PGPBoolean)unmountOnSleep;

- (void)setForcedUnmount:(PGPBoolean)flag;
- (PGPBoolean)forcedUnmount;

- (void)setADK:(PGPKeyDBObjRef)adk;
- (PGPKeyDBObjRef)adk;

- (void)setPreventSleep:(PGPBoolean)flag;
- (PGPBoolean)preventSleep;

- (void)windowDidBecomeKey:(NSNotification*)notification;
- (void)windowDidResignKey:(NSNotification*)notification;

- (void)updateKeyrings;
- (void)unmountAllDisks:(NSMutableData*)success;
- (void)diskEnginePID:(NSMutableData*)pid;
- (void)displayOpenFailedAlertWithError:(PGPError)err forDisk:(CPGPdiskObject*)disk;
- (void)displayMountFailedAlertWithError:(PGPError)err forDisk:(CPGPdiskObject*)disk;
- (void)displayUnmountFailedAlertWithError:(PGPError)err forDisk:(CPGPdiskObject*)disk;
- (void)displayUserModAlertWithError:(PGPError)err forDisk:(CPGPdiskObject*)disk;
- (void)displayReEncryptionAlertWithError:(PGPError)err forDisk:(CPGPdiskObject*)disk;
- (void)displayConversionAlertWithError:(PGPError)err forDisk:(CPGPdiskObject*)disk;
- (void)displayAlert:(NSString*)caption message:(NSString*)message critical:(BOOL)critical;
- (void)installDiskCheckerTimer;
- (void)diskChecker:(NSTimer*)timer;
- (void)tabView:(NSTabView*)tabView didSelectTabViewItem:(NSTabViewItem*)tabViewItem;

/* Notification handlers */
- (void)passphraseCacheTypeDidChange:(NSNotification*)notification;
- (void)passphraseSecondsDidChange:(NSNotification*)notification;
- (void)passphraseShareCacheDidChange:(NSNotification*)notification;
- (void)allowForcibleUnmountDidChange:(NSNotification*)notification;
- (void)unmountOnInactivityDidChange:(NSNotification*)notification;
- (void)autoUnmountTimeoutDidChange:(NSNotification*)notification;
- (void)autoUnmountOnLogoutDidChange:(NSNotification*)notification;
- (void)autoUnmountOnSleepDidChange:(NSNotification*)notification;
- (void)preventSleepDidChange:(NSNotification*)notification;

- (void)finderUnmountedVolume:(NSNotification*)notification;

- (void)dealloc;

@end

@interface PGPdiskController (PGPdiskCreationExtensions)

- (IBAction)newDisk:(id)sender;

@end

@interface PGPdiskController (PGPdiskUserExtensions)

- (BOOL)updateUsersForDisk:(CPGPdiskObject*)disk;
- (IBAction)disablePressed:(id)sender;
- (IBAction)changePassphrase:(id)sender;
- (IBAction)addUserPressed:(id)sender;
- (IBAction)removeUserPressed:(id)sender;
- (IBAction)readonlyPressed:(id)sender;
- (IBAction)adminPressed:(id)sender;
- (PGPError)newUser:(NSString*)username keyDBObj:(PGPKeyDBObjRef)keyDBObj passphrase:(const char*)passphrase adminPassphrase:(const char*)adminPassphrase readOnly:(BOOL)flag;
- (void)removeUserWarningDidEnd:(NSWindow*)sheet returnCode:(int)returnCode contextInfo:(void*)contextInfo;
- (void)disableUserWarningDidEnd:(NSWindow*)sheet returnCode:(int)returnCode contextInfo:(void*)contextInfo;

@end

@interface PGPdiskController (PGPdiskMountExtensions)

- (IBAction)mountDiskPressed:(id)sender;
- (PGPError)mountDisk:(CPGPdiskObject*)disk;
- (PGPError)unmountDisk:(CPGPdiskObject*)disk;
- (void)setMountButtonForDiskAtIndex:(int)diskIndex;
- (void)setMountButtonForDisk:(CPGPdiskObject*)disk;
- (void)setUnmountButton:(CPGPdiskObject*)disk;
- (void)setMountButton:(CPGPdiskObject*)disk;

@end