/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.

	$Id: PGPdiskController.cpp,v 1.6 2002/10/17 06:59:30 wprice Exp $
____________________________________________________________________________*/
#include <IOKit/IOLib.h>
#include "PGPdiskDriver.h"
#include "PGPdiskController.h"
#include "PGPdiskControllerUserClient.h"


#define super IOService
OSDefineMetaClassAndStructors(PGPdiskController, IOService)

	bool
PGPdiskController::init(
	OSDictionary	*dictionary )
{
	bool res = super::init( dictionary );
	STATUS_LOG(( "PGPdiskController::init\n" ));
	fDiskCount = 0;
	return res;
}

	void
PGPdiskController::free( void )
{
	STATUS_LOG(( "PGPdiskController::free\n" ));
	super::free();
}

	bool
PGPdiskController::start(
	IOService	*provider )
{
	bool		res = false;
	
	if ( !super::start( provider ) )
		return false;
	STATUS_LOG(( "PGPdiskController::start\n" ));
	
     const OSSymbol *userClient = OSSymbol::withCStringNoCopy( kPGPdiskControllerUserClient_ClassName );
     const OSSymbol *vendor = OSSymbol::withCStringNoCopy( kPGPdiskDriver_VendorString );
     const OSSymbol *model = OSSymbol::withCStringNoCopy( kPGPdiskDriver_ModelString );
     const OSSymbol *revision = OSSymbol::withCStringNoCopy( kPGPdiskDriver_RevisionString );
	 
	if( userClient && vendor && model && revision )
	{
		setProperty( gIOUserClientClassKey, (OSObject *) userClient );
		setProperty( "vendor", (OSObject *) vendor );
		setProperty( "model", (OSObject *) model );
		setProperty( "revision", (OSObject *) revision );
		
		userClient->release();
		vendor->release();
		model->release();
		revision->release();
	
		registerService();
		res = true;
	}
	else
	{
		STATUS_LOG(( "PGPdiskController::start failed, no userClient\n" ));
	}
	return res;
}

	IOReturn 
PGPdiskController::message(
	UInt32			type,
	IOService		*provider,
	void			*argument )
{
	IOReturn		res = kIOReturnSuccess;
	
	switch (type)
	{         
		case kIOMessageServiceIsTerminated:
			if( provider )
			{
				STATUS_LOG(( "PGPdiskController::message: kIOMessageServiceIsTerminated from %s\n", provider->getName()) );
				provider->close(this);
			}
			break;
		default:
			res = super::message(type, provider, argument);
			break;
	}
	return res;
}

	void
PGPdiskController::stop(
	IOService	*provider )
{
	STATUS_LOG(( "PGPdiskController::stop\n" ));
	super::stop( provider );
}

	IOReturn 
PGPdiskController::createDisk(
	const PGPdiskCreateDiskParams	*diskParams )
{
	PGPdiskDriver					*driver;
	
	STATUS_LOG(( "PGPdiskController::createDisk\n" ));
	driver = new PGPdiskDriver;
	if( driver == NULL )
	{
		STATUS_LOG(( "PGPdiskController::createDisk: could not instantiate\n" ));
		return kIOReturnNoMemory;
	}
	if( !driver->initPGPdisk( diskParams, fDiskCount++ ) )
	{
		STATUS_LOG(( "PGPdiskController::createDisk: could not init\n" ));
		driver->release();
		return kIOReturnInternalError;
	}
	if( !driver->attach( this ) )
	{
		STATUS_LOG(( "PGPdiskController::createDisk: could not attach\n" ));
		driver->release();
		return kIOReturnInternalError;
	}

	if( !driver->start( this ) )
	{
		STATUS_LOG(( "PGPdiskController::createDisk: could not start\n" ));
		driver->release();
		return kIOReturnNotAttached;
	}

	// should happen AFTER we start the driver...
	STATUS_LOG(( "PGPdiskController:createDisk: registering PGPdiskDriver\n" ));
	driver->registerService();
	
	// now can release since we are owned by the registry...
	driver->release();
	
	STATUS_LOG(( "PGPdiskController::createDisk: exiting true\n" ));
	return kIOReturnSuccess;
}

