/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.

	$Id: PGPdiskDriverTest.c,v 1.4 2002/08/06 20:09:34 dallen Exp $
____________________________________________________________________________*/
#include <unistd.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sysexits.h>

#include <sys/errno.h>
 
#include <mach/mach.h>
#include <mach/mach_error.h>

#include <CoreFoundation/CoreFoundation.h>

#include <IOKit/IOKitLib.h>
#include <IOKit/iokitmig.h>

#include "PGPdiskControllerUserClient.h"

const char *cmdName;
kern_return_t createDisk(io_connect_t dataPort, const PGPdiskCreateDiskParams	*diskParams);

/* macro to test and report fatal errors when calling mach routines */
#define mach_test(expr, msg) do {                               \
     kern_return_t err = (expr);                                \
     if (KERN_SUCCESS != err) {                                 \
         fprintf(stderr, "%s: %s - %s(%x,%d)\n", cmdName,       \
                 msg, mach_error_string(err),                   \
                 err, err & 0xffffff);                          \
         fflush(stderr);                                        \
         *((char *) 0) = 0;                                     \
     }                                                          \
} while(0)

#define plain_test(expr, msg) do {                              \
     if (expr) {                                                \
         fprintf(stderr, "%s: %s\n", cmdName,  msg);            \
         fflush(stderr);                                        \
         *((char *) 0) = 0;                                     \
     }                                                          \
} while(0)

static void dosomething(io_object_t dataService)
{
	kern_return_t				ret;
	io_connect_t				dataPort = NULL;
	PGPdiskCreateDiskParams		diskParams;
	
	ret = IOServiceOpen(dataService, mach_task_self(), /* type */ 0, &dataPort);
	mach_test(ret, "Couldn't create a user client, why not?");
	printf("user client created\n");
	
	// We have a connection to our driver, YEH!
	// We can start using our io_connect_.* routines to communicate
	// with it.
	
	printf("calling createDisk\n");
	diskParams.diskSizeBytes = (2048 * 1024);
	diskParams.mountReadOnly = false;
	ret = createDisk(dataPort, &diskParams);
	printf("createDisk returned %d\n", ret);
	
	if (dataPort)
	{
		printf("closing user client\n");	 
		IOServiceClose(dataPort);
		dataPort = 0;
	}
}

// Search for our driver in the registry
static void talkToPGPdiskUserClient(void)
{
	mach_port_t			masterPort = 0;
	CFDictionaryRef		myMatch = 0;
	io_iterator_t		iter = 0;
	io_object_t			device,
						myDevice = 0;
	
	mach_test(IOMasterPort(NULL, &masterPort), "Couldn't create a master IOKit Port");
	printf("master port created\n");
	
	// For the curious IOServiceMatching creates a dictionary
	// with the input string as the value for IOProviderClass,
	// familiar, yes IOKit's complete matching system is used
	// for generating this iterator.  We can implement
	// as powerful a matching language as we want to rendezvous
	// with a particular instance of our service.
	myMatch = IOServiceMatching(kPGPdiskController_ClassName);
	plain_test(!myMatch, "Can't create a 'PGPdiskController' matching dictionary");
	printf("matching dictionary created\n");
	
	// Note the IOServiceGetMatchingServices is documented as
	// consuming a reference on the matching dictionary so
	// we don't have to release it here.  However it is so odd
	// that I think it is worth documenting.
	mach_test(IOServiceGetMatchingServices(masterPort, myMatch, &iter),
				"Can't create a PGPdiskController iterator");
	myMatch = 0;
	
	// Iterate over all matching drivers in the registry.
	// It is an error if we find more than one matching driver
	while ( (device = IOIteratorNext(iter)) )
	{
		plain_test(myDevice, "But we already have a device I'm confused");
		myDevice = device;
	}
	
	// Did we find the device then?
	plain_test(!myDevice, "Couldn't find a PGPdiskController in registry");

	if( myDevice )
	{
		dosomething(myDevice);
		// I always clean up after myself.  It isn't really
		// necessary as the task is about to terminate and we
		// can rely on the kernel to completely clean up behind us.
		IOObjectRelease(myDevice);
	}
	
	if( iter )
	{
		IOObjectRelease(iter);
		iter = 0;
	}
	
	if( myMatch )
	{
		CFRelease(myMatch);
		myMatch = 0;
	}

// 	I do not believe that you need to deallocate the master port...
#if 0
	if( masterPort )
	{
		mach_port_deallocate(mach_task_self(), masterPort);
		masterPort = 0;
	}
#endif	
}

int main(int argc, const char *argv[])
{
	cmdName = argv[0];
	
	talkToPGPdiskUserClient();
	
	exit(EX_OK);
}

	kern_return_t
createDisk(
	io_connect_t					dataPort,
	const PGPdiskCreateDiskParams	*diskParams )
{
	kern_return_t			ret;
	mach_msg_type_number_t	len = 0;
	
	ret = io_connect_method_structureI_structureO(dataPort,
		kPGPdiskControllerUserClient_CreateDisk, (void *)diskParams,
		sizeof(PGPdiskCreateDiskParams), NULL, &len);
	return ret;
}

