/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.

	$Id: PGPdiskDriverUserClient.cpp,v 1.8 2002/08/06 20:09:34 dallen Exp $
____________________________________________________________________________*/
#include <IOKit/IOService.h>
#include <IOKit/IOLib.h>
#include <IOKit/system.h>
#include "PGPdiskDriver.h"
#include "PGPdiskController.h"
#include "PGPdiskDriverUserClient.h"

#define super IOUserClient
OSDefineMetaClassAndStructors(PGPdiskDriverUserClient, IOUserClient)

	bool
PGPdiskDriverUserClient::initWithTask(
	task_t			owningTask,
	void			*securityToken,
	UInt32			type )
{
	STATUS_LOG(( "PGPdiskDriverUserClient::initWithTask\n" ));
	
	if( !super::init() )
		return false;
	
	STATUS_LOG(( "   tagging fClient as 0x%08lX\n", (UInt32)owningTask ));
	fClient = owningTask;
	task_reference(fClient);
	
	return true;
}

	IOReturn
PGPdiskDriverUserClient::clientClose( void )
{
	STATUS_LOG(( "PGPdiskDriverUserClient::clientClose\n" ));
	
	if( fService )
	{
		fService->close( this );  // Aborts any outstanding I/O getData
		
		// This should happen automatically but it doesn't yet.
		// be warned this detach will be unnecessary in the future and
		// will probably cause your driver to panic in the future.
		detach( fService );
	
		// the following is to force teardown of the PGPdiskDriver stack when the
		// user process dies.  Since the user process "serves" the disk data, when the
		// user process dies, this is equivalent to the network connection going away
		// from an AppleShare volume, or to a FireWire drive being removed.

		STATUS_LOG(( "PGPdiskDriverUserClient::clientClose - terminating provider with kIOServiceRequired\n" ));
		fService->terminate(kIOServiceRequired);
		fService = NULL;
	}
	
	// Better release our client task reference
	if( fClient )
	{
		task_deallocate( fClient );
		fClient = NULL;
	}
	return kIOReturnSuccess;
}

	IOReturn 
PGPdiskDriverUserClient::message(
	UInt32		type,
	IOService	*provider,
	void		*argument )
{   
	IOReturn	res = kIOReturnSuccess;
	
	switch (type)
	{
		case kIOMessageServiceIsTerminated:
			if( provider )
			{
				STATUS_LOG(( "PGPdiskDriverUserClient::message: kIOMessageServiceIsTerminated from %s\n", provider->getName() ));
				provider->close(this);
			}
			break;
		default:
			res = super::message(type, provider, argument);
			break;
	}
	return res;
}

	bool
PGPdiskDriverUserClient::start(
	IOService *provider)
{
	STATUS_LOG(( "PGPdiskDriverUserClient::start\n" ));
	if( !super::start( provider ) )
		return false;
	
	// make sure our provider is PGPdiskDriver class
	fService = OSDynamicCast( PGPdiskDriver, provider);
	if( !fService )	
		return false;
	if( !fService->open( this ) )
		goto abort;

	// set up the method table here
	sMethods[kPGPdiskDriverUserClient_ActivateDrive].object = fService;
	sMethods[kPGPdiskDriverUserClient_ActivateDrive].func = (IOMethod)&PGPdiskDriver::activateDrive;
	sMethods[kPGPdiskDriverUserClient_ActivateDrive].count0 = 0;
	sMethods[kPGPdiskDriverUserClient_ActivateDrive].count1 = sizeof(UInt32);
	sMethods[kPGPdiskDriverUserClient_ActivateDrive].flags = kIOUCScalarIScalarO;
	
	sMethods[kPGPdiskDriverUserClient_DiskControl].object = this;
	sMethods[kPGPdiskDriverUserClient_DiskControl].func = (IOMethod)&PGPdiskDriverUserClient::diskControl;
	sMethods[kPGPdiskDriverUserClient_DiskControl].count0 = sizeof(PGPdiskIOParamBlock);
	sMethods[kPGPdiskDriverUserClient_DiskControl].count1 = sizeof(PGPdiskIOParamBlock);
	sMethods[kPGPdiskDriverUserClient_DiskControl].flags = kIOUCStructIStructO;
	
	STATUS_LOG(( "PGPdiskDriverUserClient::start succeeded\n" ));
	return true;
abort:
	fService = NULL;
	return false;
}

	IOExternalMethod *
PGPdiskDriverUserClient::getTargetAndMethodForIndex(
	IOService		**target,
	UInt32			index )
{
	//STATUS_LOG(( "PGPdiskDriverUserClient::getTargetAndMethodForIndex(%ld)\n", index ));
	if( index < (UInt32) kPGPdiskDriverUserClient_NumCommands )
	{
		*target = sMethods[index].object;
		return &sMethods[index];
	}
	else
		return 0;
}

	void
PGPdiskDriverUserClient::free( void )
{
	STATUS_LOG(( "PGPdiskDriverUserClient::free\n" ));
	super::free();
}

	IOReturn
PGPdiskDriverUserClient::diskControl(
	const PGPdiskIOParamBlock		*diskIOIn,
	PGPdiskIOParamBlock				*diskIOOut )
{
    IOReturn						res = kIOReturnSuccess;
	IODirection						direction;
	IOMemoryDescriptor				*dataMD = NULL;
	
	switch( diskIOIn->controlCode )
	{
		case kPGPdiskDriverControlCode_GetRequest:
			direction = kIODirectionIn;
			break;
		case kPGPdiskDriverControlCode_ReadReply:
			direction = kIODirectionOut;
			break;
		default:
		case kPGPdiskDriverControlCode_WriteReply:
			direction = kIODirectionNone;
			break;
	}
	if( direction != kIODirectionNone )
	{
		dataMD = IOMemoryDescriptor::withAddress( ( vm_address_t )diskIOIn->data,
					kPGPdiskDriver_MaxTransferSize, direction, fClient );
		if( !dataMD )
		{
			STATUS_LOG(( "PGPdiskDriverUserClient::diskControl: IOMemoryDescriptor::withAddress failed\n" ));
		}
		res = dataMD->prepare();
	}
	if( res == kIOReturnSuccess )
	{
		res = fService->diskControl( diskIOIn, diskIOOut, dataMD );
	}
	else
	{
		STATUS_LOG(( "PGPdiskDriverUserClient::diskControl: IOMemoryDescriptor::prepare failed 0x%08lX\n", (unsigned long)res ));
	}
	if( dataMD )
	{
		dataMD->complete();
		dataMD->release();
	}
	return res;
}

