/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.

	$Id: PGPdiskDriverUserClient.h,v 1.9 2002/08/06 20:09:34 dallen Exp $
____________________________________________________________________________*/
#pragma once

#define kPGPdiskDriver_BlockSize				( 512 )
#define kPGPdiskDriver_MaxTransferBlocks		( 256 )
#define kPGPdiskDriver_MaxTransferSize			( kPGPdiskDriver_MaxTransferBlocks * kPGPdiskDriver_BlockSize )
#define kPGPdiskDriver_ClassName 				"PGPdiskDriver"
#define kPGPdiskDriverUserClient_ClassName 		"PGPdiskDriverUserClient"
#define kPGPdiskDriver_DiskIDProperty			"diskid"
#define kPGPdiskDriver_EjectOnLogoutProperty	"eject-upon-logout"
#define kPGPdiskDriver_IconPathProperty			"icon-path"
#define kPGPdiskDriver_AutoDiskMountProperty	"autodiskmount"


enum PGPdiskDriverUserClientCmdCodes
{
	kPGPdiskDriverUserClient_ActivateDrive	= 0,
	kPGPdiskDriverUserClient_DiskControl,
	kPGPdiskDriverUserClient_NumCommands
};

enum PGPdiskDriverControlCode
{
	kPGPdiskDriverControlCode_None	= 0,	/* driver -> engine */
	kPGPdiskDriverControlCode_Inactive,		/* driver -> engine */
	kPGPdiskDriverControlCode_ReadRequest,	/* driver -> engine */
	kPGPdiskDriverControlCode_WriteRequest,	/* driver -> engine */
	kPGPdiskDriverControlCode_GetRequest,	/* engine -> driver */
	kPGPdiskDriverControlCode_ReadReply,	/* engine -> driver */
	kPGPdiskDriverControlCode_WriteReply,	/* engine -> driver */
};

/*	The following parameter block is used for all reads and writes for a PGPdisk.
	The requestType tells you whether this is a read or a write. The startBlock
	and numBlocks together tell you where the IO operation should begin and how
	big it should be. data contains the actual data from the operation. This buffer
	must be allocated in the user space, and should be at least
	kPGPdiskDriver_MaxTransferSize big. For efficiency, this buffer should always
	remain allocated while a disk is mounted.
	
		kPGPdiskDriverControlCode_None:
			Sent from driver to userland when there is no work to do. Should
			only occur when the drive is being unmounted.
		kPGPdiskDriverControlCode_GetRequest:
			Sent from userland to driver to get a work request, blocks
		kPGPdiskDriverControlCode_ReadRequest:
			 This is returned from driver to app when a read is requested. Use
			 startBlock and numBlocks to read the correct data into data and
			 call back into the driver with a ReadReply.
		kPGPdiskDriverControlCode_WriteRequest:
			This is returned from driver to app when a write is requested. Use
			startBlock and numBlocks to write the data from data into
			the PGPdisk and then call back into the driver with a WriteReply
			to let the driver know about any errors.
		kPGPdiskDriverControlCode_Inactive:
			The driver is shutting down. You should close up shop.
*/

typedef struct PGPdiskIOParamBlock
{
	UInt32		controlCode;		// PGPdiskDriverControlCode
	UInt32		startBlock;
	UInt32		numBlocks;
	UInt32		refCon;				// make sure to set for replies the same as it was for request
	UInt32		err;				// set to 0 if successful
	void		*data;
} PGPdiskIOParamBlock;


#if KERNEL

#include <mach/mach_types.h>
#include <IOKit/IOUserClient.h>

class PGPdiskDriver;
class PGPdiskDriverUserClient : public IOUserClient
{
	OSDeclareDefaultStructors(PGPdiskDriverUserClient)
	
public:
	virtual bool				initWithTask( task_t owningTask, void *securityToken, UInt32 type );
	virtual bool				start( IOService *provider );
	virtual void				free( void );
	virtual IOExternalMethod	*getTargetAndMethodForIndex( IOService **target, UInt32 index );
	virtual IOReturn			clientClose( void );
	virtual IOReturn 			message(UInt32 type, IOService * provider, void * argument);
	
	virtual IOReturn			diskControl(	const PGPdiskIOParamBlock *diskIOIn,
												PGPdiskIOParamBlock *diskIOOut );
protected:
	IOExternalMethod			sMethods[ kPGPdiskDriverUserClient_NumCommands ];
	
	task_t						fClient;
	PGPdiskDriver				*fService;
};

#endif /* KERNEL */

