/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.

	$Id: main.c,v 1.20 2002/10/06 08:46:52 dallen Exp $
____________________________________________________________________________*/

#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <errno.h>
#include <unistd.h>
#include <syslog.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/resource.h>
#include "pgpPubTypes.h"
#include "pgpUtilities.h"
#include "pgpErrors.h"
#include "pgpDiskEngineAppWorker.h"
#include "pgpDiskEngineKernelWorker.h"
#include "pgpDiskEngineDisk.h"

#define CKERR		if( IsPGPError( err ) ) goto done
#define RETERR(x)	do { err = x; goto done; } while(0)

PGPBoolean		gbRunning		= TRUE;

    void
sigIntHandler(
    PGPInt32        signal )
{
    syslog( LOG_DEBUG, "Caught SIGINT.  Exiting.\n" );
	gbRunning = FALSE;
	return;
}

    void
sigTermHandler(
    PGPInt32        signal )
{
    syslog( LOG_DEBUG, "Caught SIGTERM.  Exiting.\n" );
	gbRunning = FALSE;
    return;
}

    void
sigHupHandler(
    PGPInt32        signal )
{
    syslog( LOG_DEBUG, "Caught SIGHUP\n" );
    return;
}

    void
sigPipeHandler(
    PGPInt32        signal )
{
    syslog( LOG_DEBUG, "Caught SIGPIPE\n" );
    return;
}

    void
sigSegvHandler(
    PGPInt32        signal )
{
    syslog( LOG_DEBUG, "Caught SIGSEGV.  Exiting.\n" );
    exit( EXIT_FAILURE );
    return;
}

	static PGPError
sSetSignalHandlers( void )
{
	PGPError		err		 = kPGPError_NoErr;
	
    if( signal( SIGINT, sigIntHandler ) == SIG_ERR )
        RETERR( kPGPError_UnknownError );

    if( signal( SIGTERM, sigTermHandler ) == SIG_ERR )
        RETERR( kPGPError_UnknownError );

    if( signal( SIGHUP, sigHupHandler ) == SIG_ERR )
        RETERR( kPGPError_UnknownError );

    if( signal( SIGPIPE, sigPipeHandler ) == SIG_ERR )
        RETERR( kPGPError_UnknownError );
		
    if( signal( SIGSEGV, sigSegvHandler ) == SIG_ERR )
        RETERR( kPGPError_UnknownError );

done:
	return err;
}

	int
main(
	int		argc,
	char *	argv[] )
{
	PGPError						err			= kPGPError_NoErr;
	char							errstr[256];
	PGPMemoryMgrRef					mgr			= kInvalidPGPMemoryMgrRef;
	PGPdiskKernelWorkerContextRef	kernCon		= kInvalidPGPdiskKernelWorkerContextRef;
	PGPUInt32						i			= 0;
#if PGP_DEBUG
	PGPBoolean						bDetach		= TRUE;
#endif
	PGPInt32						retval		= 0;
	struct rlimit					rlp;
	
	if( PGPdiskIsEngineRunning() )
	{
#if PGP_DEBUG
		printf( "PGPdiskEngine is already running\n" );
#endif
		exit( EXIT_SUCCESS );
	}
	
	for( i = 0; i < argc; i++ )
	{
#if PGP_DEBUG
		if( strcmp( argv[i], "--nodetach" ) == 0 )
			bDetach = FALSE;
		if( strcmp( argv[i], "-d" ) == 0 )
			bDetach = FALSE;
#endif
	}
	
#if !PGP_UNIX_DARWIN
	/* Be a daemon */
	if( bDetach )
	{
		if( fork() > 0 )
			exit( EXIT_SUCCESS );
		setsid();
	}
#endif
	
	/* Initial setup is done.  Now we can do the real stuff */
	openlog( "PGPdiskEngine", LOG_PID | LOG_NDELAY, LOG_USER );
	
	rlp.rlim_cur = 0;
	rlp.rlim_max = 0;
	retval = setrlimit( RLIMIT_CORE, &rlp );
	if( retval < 0 )
		syslog( LOG_WARNING, "Warning: Could not disable core dumps: %s\n", strerror( errno ) );

	err = PGPNewMemoryMgr( 0, &mgr ); CKERR;
	
	err = PGPdiskEngineInit(); CKERR;

	err = sSetSignalHandlers(); CKERR;
	
	/* Start kernel worker */
	err = PGPdiskEngineStartKernelWorker( mgr, &kernCon ); CKERR;
	
	/* Start App worker */
	err = PGPdiskEngineStartAppWorker( mgr, kernCon );
	
	(void) PGPdiskEngineRun( kernCon );

	(void) PGPdiskEngineStopKernelWorker( kernCon );
	
	(void) PGPdiskEngineCleanup();
	
done:
	if( IsPGPError( err ) )
	{
		if( err != kPGPError_UnknownError )
			(void) PGPdiskGetErrorString( err, sizeof( errstr ), errstr );
		else
			sprintf( errstr, "%s", strerror( errno ) );
		syslog( LOG_ERR, "Error: %s\n", errstr );
	}
	
	if( PGPMemoryMgrRefIsValid( mgr ) )
		(void) PGPFreeMemoryMgr( mgr );
	
	return IsPGPError( err ) ? EXIT_FAILURE : EXIT_SUCCESS;
}

/*__Editor_settings____

	Local Variables:
	tab-width: 4
	End:
	vi: ts=4 sw=4
	vim: si
_____________________*/
