/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.

	$Id: pgpDiskEngineTypes.h,v 1.29 2002/10/06 08:46:52 dallen Exp $
____________________________________________________________________________*/
#ifndef Included_PGPdiskEngineTypes_h	/* [ */
#define Included_PGPdiskEngineTypes_h

#include "pgpBase.h"
#include "pgpDiskHeaders.h"
#include "pgpDiskOS9Headers.h"
#include "pgpDiskCastCFB.h"
#include "pgpDiskAES.h"

#include <IOKit/IOKitLib.h>

typedef struct PGPdiskKernelWorkerContext
{
	mach_port_t		masterPort;
	io_object_t		controllerDevice;

	PGPBoolean		unmountOnSleep;
	PGPBoolean		forcedUnmount;

	/* Used for preventing system sleep */
	PGPBoolean		preventSleep;
	io_connect_t	port;
} PGPdiskKernelWorkerContext;

typedef struct PGPdiskKernelConnectionContext
{
	io_connect_t	driverPort;
	pthread_t		thread;
} PGPdiskKernelConnectionContext;

typedef struct PGPdiskIODiskUser
{
	struct PGPdiskIODisk *			disk;
	
	PGPdiskOnDiskUserHeader			userHdr;
	PGPBoolean						bSymUser;
	union
	{
		PGPdiskOnDiskUserWithSym	symUser;
		PGPdiskOnDiskUserWithPub	pubUser;
	} user;
	
	/* These fields only used for public key users */
	PGPByte *						encryptedKey;
	PGPSize							encryptedKeySize;
	
	struct PGPdiskIODiskUser *		next;
} PGPdiskIODiskUser;

typedef struct PGPdiskIODisk
{
	PGPMemoryMgrRef					mgr;
	PGPdiskKernelWorkerContext *	kernCon;
	PGPdiskKernelConnectionContext *driverConn;
	pthread_mutex_t					mutex;
	PGPInt32						fd;
	PGPUInt32						refCount;
	PGPBoolean						bMounted;
	PGPBoolean						bReadOnly;
	PGPBoolean						bTimedOut;
	char							filename[PATH_MAX];
	char							szDeviceName[128];
	
	PGPBoolean						bCustomTimeout;
	PGPUInt32						timeout;		/* in seconds, 0 = never */
	time_t							timeLastUsed;
	PGPBoolean						forcedUnmount;

	PGPUInt64						dataOffset;
	PGPUInt64						dataSize;
	
	PGPdiskOnDiskMainHeader			mainHdr;
	struct PGPdiskIODiskUser *		userList;
	PGPUInt32						numUsers;
	
	/* Set these when re-encrypting the disk */
	PGPdiskOnDiskMainHeader *		reencryptingMainHdr;
	PGPdiskIODiskUser *				reencryptingAdminUser;
	
	PGPdiskDiskFormat				diskFormat;
	PGPdiskOS9FileHeader			os9FileHeader;

	/*
	 * The following cipher contexts should be allocated in secure memory
	 * and periodically should be flipped to prevent memory burn-in
	 */
	PGPBoolean *					pbTimerStillRunning;
	PGPUInt32						whichContext;
	CASTCFBContext *				castCFB[2];
	PGPByte *						aesCFB[2];
} PGPdiskIODisk;

typedef struct PGPdiskBEContext
{
	PGPContextRef							context;		/* Thread-specific context */
	PGPMemoryMgrRef							mgr;			/* _THE_ Global (all threads) memory mgr */
	struct PGPdiskKernelWorkerContext *		kernCon;
	PGPInt32								sock;

	PGPKeyDBRef								keydb;
	PGPKeyDBObjRef							adkKey;

	PGPUInt32								globalTimeout;
	PGPBoolean								forcedUnmount;
	
	PGPBoolean								ejectOnLogout;
	
	PGPBoolean								cachePassphrases;
	PGPUInt32								cacheTimeout;
	PGPBoolean								shareCache;
	
	struct PGPdiskBEDisk *					diskList;
	struct PGPdiskBEUser *					userList;
} PGPdiskBEContext;

typedef struct PGPdiskBEDisk
{
	PGPdiskBEContext *		diskcon;
	char					filename[PATH_MAX];
	struct PGPdiskIODisk *	ioDisk;
	PGPBoolean				readonly;

	struct PGPdiskBEDisk *	next;
} PGPdiskBEDisk;

typedef struct PGPdiskBEUser
{
	PGPdiskBEContext *			diskcon;

	/*
	 * These fields get mirrored between the BEUser and the IODiskUser because
	 * we could create them and change them in the BEUser without ever creating
	 * a matching IODiskUser
	 */
	PGPBoolean					bSymUser;
	PGPBoolean					bReadOnly;
	PGPBoolean					bAdminUser;
	PGPBoolean					bDisabled;
	PGPBoolean					bLocked;
	
	/* Symmetric key users */
	char						username[kPGPdiskMaxUserNameSize + 1];
	char *						passphrase;
	
	/* Public key users */
	PGPKeyID					keyid;

	struct PGPdiskIODiskUser *	ioDiskUser;

	struct PGPdiskBEUser *		next;
} PGPdiskBEUser;


typedef PGPdiskKernelWorkerContext *	PGPdiskKernelWorkerContextRef;
typedef PGPdiskIODisk *					PGPdiskIODiskRef;
typedef PGPdiskBEContext *				PGPdiskBEContextRef;
typedef PGPdiskBEDisk *					PGPdiskBEDiskRef;
typedef PGPdiskBEUser *					PGPdiskBEUserRef;


#define kInvalidPGPdiskKernelWorkerContextRef		((PGPdiskKernelWorkerContextRef) NULL)
#define kInvalidPGPdiskIODiskRef					((PGPdiskIODiskRef) NULL)
#define kInvalidPGPdiskBEContextRef					((PGPdiskBEContextRef) NULL)
#define kInvalidPGPdiskBEDiskRef					((PGPdiskBEDiskRef) NULL)
#define kInvalidPGPdiskBEUserRef					((PGPdiskBEUserRef) NULL)


#define PGPdiskKernelWorkerContextRefIsValid(x)		((x) != kInvalidPGPdiskKernelWorkerContextRef)
#define PGPdiskIODiskRefIsValid(x)					((x) != kInvalidPGPdiskIODiskRef)
#define PGPdiskBEContextRefIsValid(x)				((x) != kInvalidPGPdiskBEContextRef)
#define PGPdiskBEDiskRefIsValid(x)					((x) != kInvalidPGPdiskBEDiskRef)
#define PGPdiskBEUserRefIsValid(x)					((x) != kInvalidPGPdiskBEUserRef)


#define PGPdiskValidateKernelWorkerContextRef(x)	PGPValidateParam(PGPdiskKernelWorkerContextRefIsValid(x))
#define PGPdiskValidateIODiskRef(x)					PGPValidateParam(PGPdiskIODiskRefIsValid(x))
#define PGPdiskValidateBEContextRef(x)				PGPValidateParam(PGPdiskBEContextRefIsValid(x))
#define PGPdiskValidateBEDiskRef(x)					PGPValidateParam(PGPdiskBEDiskRefIsValid(x))
#define PGPdiskValidateBEUserRef(x)					PGPValidateParam(PGPdiskBEUserRefIsValid(x))


#endif /* ] Included_PGPdiskEngineTypes_h */


/*__Editor_settings____

	Local Variables:
	tab-width: 4
	End:
	vi: ts=4 sw=4
	vim: si
_____________________*/
