/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.
	
	$Id: pgpDiskOS9Headers.h,v 1.2 2002/08/06 20:09:35 dallen Exp $
____________________________________________________________________________*/

#ifndef Included_pgpDiskOS9Headers_h	/* [ */
#define Included_pgpDiskOS9Headers_h

#include "pgpDiskBase.h"
#include "pgpPubTypes.h"


#define	kPGPdiskOS9HeaderMagic					'PGPD'

#define	kPGPdiskOS9PrimaryHeaderType			'MAIN'
#define	kPGPdiskOS9PrimaryHeaderMajorVersion 	2
#define	kPGPdiskOS9PrimaryHeaderMinorVersion 	1

#define	kPGPdiskOS9ReservedHeaderBlocks			12
#define	kPGPdiskOS9AlternateHeaderBlocks		1
#define	kPGPdiskOS9AlternateHeaderSize			(kPGPdiskOS9AlternateHeaderBlocks * kPGPdiskBlockSize)

typedef enum
{
	kPGPdiskOS9_InvalidEncryptionAlgorithm		= 0,
	kPGPdiskOS9_CopyDataEncryptionAlgorithm		= 1,
	kPGPdiskOS9_CASTEncryptionAlgorithm			= 2,

	kPGPdiskOS9_DefaultEncryptionAlgorithm		= kPGPdiskOS9_CASTEncryptionAlgorithm,
	// force size of enum to be 4 bytes
	kPGPdiskOS9_DummyEncryptionAlgorithm 		= ULONG_MAX

} PGPdiskOS9EncryptionAlgorithm;

typedef struct PGPdiskOS9FileHeaderInfo
{
	PGPUInt32		headerMagic;		// Always kPGPdiskOS9HeaderMagic
	PGPUInt32		headerType;			// kPGPdiskOS9PrimaryHeaderType + ???
	PGPUInt32		headerSize;			// Total size of this header, in bytes.
	PGPUInt32		headerCRC;			// CRC of this header
	PGPUInt64		nextHeaderOffset;	// Offset to next header from
										// beginning of file, in bytes
										// 0 = no additional headers
	PGPUInt32		reserved[ 2 ];
	
} PGPdiskOS9FileHeaderInfo;

typedef struct PGPdiskOS9CheckBytes
{
	PGPByte			bytes[8];

} PGPdiskOS9CheckBytes;

typedef union PGPdiskOS9CASTKey
{
	PGPByte			bytes[ 16 ];
	PGPUInt32		longs[ 16 / sizeof( PGPUInt32 ) ];
} PGPdiskOS9CASTKey;
typedef PGPdiskOS9CASTKey     PGPdiskOS9EncryptedCASTKey;

typedef union PGPdiskOS9PassphraseSalt
{
	PGPByte			bytes[ 8 ];
	PGPUInt32		longs[ 8 / sizeof( PGPUInt32 ) ];
} PGPdiskOS9PassphraseSalt;

typedef struct PGPdiskOS9PassphraseKey
{
	// session key encrypted with salted passphrase
	PGPdiskOS9EncryptedCASTKey	encryptedKey;
		
	// first 8 bytes of encrypted hashed salted user key
	PGPdiskOS9CheckBytes		checkBytes;
	
	// introduced in v1.1, # of hashes on passphrase
	PGPUInt16					hashReps;
	
	PGPUInt16					inUse 		: 1;
	PGPUInt16					readOnly	: 1;
	PGPUInt16					unused		: 14;
	PGPUInt32					unusedLong;
} PGPdiskOS9PassphraseKey;

#define	kPGPdiskOS9NumPassphrases			8		// master + 1 thru 7
#define	kPGPdiskOS9MasterPassphraseIndex	0


typedef struct PGPdiskOS9FileHeader
{
	PGPdiskOS9FileHeaderInfo	headerInfo;

	PGPByte				majorVersion;
	PGPByte				minorVersion;
	PGPUInt16			reserved;

	// Total size of the file, in blocks (eof == numFileBlocks * 512)
	PGPUInt32			numFileBlocks;		

	// Size of the header data at the beginning of the file, in blocks
	PGPUInt32			numHeaderBlocks;	

	// Total size of the encrypted data, in blocks
	PGPUInt32			numDataBlocks;		
	 
	// encryption method (4 bytes)
	PGPdiskOS9EncryptionAlgorithm 	algorithm;		

	PGPUInt32			numPassphrases;
	PGPUInt32			reservedLongs[ 15 ];
	
	// random data used to salt passphrase and IVs
	PGPdiskOS9PassphraseSalt	salt;
								
	// All of the individual passphrase info for one user
	PGPdiskOS9PassphraseKey	passphrases[kPGPdiskOS9NumPassphrases];	

} PGPdiskOS9FileHeader;

#if 0
const PGPUInt32	kPGPDiskPublicKeyHeaderType			= 'PKEY';
const PGPUInt32	kPGPDiskPublicKeyHeaderMajorVersion	= 1;
const PGPUInt32	kPGPDiskPublicKeyHeaderMinorVersion	= 0;

typedef struct PGPDiskPublicKeyHeader
{
	PGPDiskFileHeaderInfo	headerInfo;
	
	UInt8					majorVersion;
	UInt8					minorVersion;
	UInt16					reserved;

	UInt32					readOnly 	: 1;
	UInt32					locked 		: 1;
	UInt32					unused		: 30;
	
	CheckBytes				checkBytes;
	
	UInt32					keyIDOffset;
	UInt32					keyIDSize;
	
	UInt32					encryptedKeyOffset;
	UInt32					encryptedKeySize;
	
	PGPPublicKeyAlgorithm	algorithm;
	
	UInt32					reserved2[10];

	// Exported key ID here
	// Encrypted key here.
	
} PGPDiskPublicKeyHeader;
#endif

#endif /* ] Included_pgpDiskOS9Headers_h */


