/*____________________________________________________________________________
		Copyright (C) 2002 PGP Corporation
        All rights reserved.

        $Id: CPGPdiskDisk.h,v 1.8 2002/08/06 20:09:36 dallen Exp $
____________________________________________________________________________*/

#ifndef Included_CPGPdiskDisk_h	// [
#define Included_CPGPdiskDisk_h

#include "pgpDiskPubTypes.h"

#include "CList.h"
#include "CEngineImpFactory.h"

_PGP_BEGIN

// Types

class CPGPKey;

class CPGPdiskContext;
class CPGPdiskUser;
class CPGPdiskUserSet;


// Class CPGPdiskDisk

class CPGPdiskDisk : public CListableObject<CPGPdiskDisk>
{
	friend class CPGPdiskDiskSet;

	NOT_COPYABLE(CPGPdiskDisk)
	DECLARE_IMPHOST(CPGPdiskDisk)

public:
	PGPBoolean	operator==(const CPGPdiskDisk& disk) const;

	const CPGPdiskContext *	Context() const;

	PGPBoolean	IsMounted() const;
	PGPBoolean	IsUnmounted() const;
	PGPBoolean	IsOpened() const;
	PGPBoolean	IsReadOnly() const;
	PGPBoolean	IsLockedForFormat() const;
	PGPBoolean	IsLockedForIO() const;

	PGPUInt8		Drive() const;
	const char *	Path() const;
	const char *	Root() const;
	const char *	DeviceName() const;

	PGPBoolean	ComparePaths(const char *path) const;
	PGPBoolean	CompareRoots(const char *root) const;

	PGPBoolean		HasOldHeaders() const;
	PGPdiskEncryptionAlgorithm	Algorithm() const;
	PGPBoolean		IsBeingReEncrypted() const;
	PGPBoolean		AreUsersWiped() const;
	PGPBoolean		HasInactivityTimedOut() const;
	PGPBoolean		UsesCustomTimeout() const;
	const char *	DefaultRoot() const;
	PGPUInt32		CustomTimeout() const;
	PGPUInt64		BlocksDisk() const;

	void	AttachMounted(const CPGPdiskContext *pContext, const char *path, 
		const char *root, const char *deviceName, PGPBoolean readOnly, 
		PGPBoolean hasTimedOut);

	void	ExportUserSet(CPGPdiskUserSet& userSet) const;
	void	ImportUserSet(const CPGPdiskUserSet& userSet);

	void	EnableCipheringOnUser(CPGPdiskUser& user, const char *passphrase, int passkeylen);

	void	WipeUsers();
	void	SetUsesCustomTimeout(PGPBoolean usesCustom);
	void	SetDefaultRoot(const char *root);
	void	SetCustomTimeout(PGPUInt32 customTimeout);

	void	ReEncrypt(const char *masterPassphrase, 
		PGPdiskEncryptionAlgorithm newAlgorithm, 
		ReEncryptDiskStatusFuncType statusFunc, void *userValue = NULL);

	void	LockVolume(PGPBoolean forFormat);
	void	UnlockVolume();

	void	ReadVolume(void *buffer, PGPUInt64 blockPos, 
		PGPUInt32 blocksLength);
	void	WriteVolume(const void *buffer, PGPUInt64 blockPos, 
		PGPUInt32 blocksLength);

	void	Open(const CPGPdiskContext *pContext, const char *path, 
		PGPBoolean readOnly = FALSE);
	void	Close();

protected:
	void	ResetTimedOut();

	void	Mount(const CPGPdiskContext *pContext, const char *path, 
		const CPGPdiskUser &diskUser, const char *root, 
		PGPBoolean readOnly = FALSE);
	void	Unmount(PGPBoolean isForced = FALSE);

	void	Create(const CPGPdiskContext *pContext, const char *path, 
		PGPUInt64 blocksDisk, PGPdiskEncryptionAlgorithm algorithm, 
		const char *passphrase, const char *userName, 
		const char *defaultRoot, NewDiskStatusFuncType statusFunc, 
		void *userValue = NULL);

	void	Create(const CPGPdiskContext *pContext, const char *path, 
		PGPUInt64 blocksDisk, PGPdiskEncryptionAlgorithm algorithm, 
		const CPGPKey& pubKey, const char *defaultRoot, 
		NewDiskStatusFuncType statusFunc, void *userValue = NULL);
};

_PGP_END

#endif	// ] Included_CPGPdiskDisk_h
