/*____________________________________________________________________________
		Copyright (C) 2002 PGP Corporation
        All rights reserved.

        $Id: PGPdiskEngineExports.cpp,v 1.20 2002/08/06 20:09:36 dallen Exp $
____________________________________________________________________________*/

#include "pgpClassesConfig.h"
#include "pgpPflPriv.h"

#include "CString.h"
#include "UMath.h"
#include "UString.h"

#include "pgpClientErrors.h"
#include "pgpDiskEngine.h"

#include "CCipherContext.h"
#include "UCryptoFactory.h"

#include "CPGPKey.h"

#include "CDriverComm.h"
#include "CEngineSubsystems.h"
#include "CPGPdiskDisk.h"
#include "CPGPdiskDiskSet.h"
#include "CPGPdiskContext.h"
#include "CPGPdiskUser.h"
#include "CPGPdiskUserSet.h"

_USING_PGP

// Exported function definitions

/* PGPdiskContextRef functions */

PGPError 
PGPdiskEngineExport 
PGPdiskNewContext(PGPUInt32 clientAPIVersion, PGPdiskContextRef *pContext)
{
	PGPValidatePtr(pContext);

	PGPError	error	= kPGPError_NoErr;

	try
	{
		if ((PGPdiskMajorVersion(clientAPIVersion) > 
			PGPdiskMajorVersion(kPGPdiskAPIVersion)) ||
			(PGPdiskMajorVersion(clientAPIVersion) == 0))
		{
			THROW_PGPERROR(kPGPClientError_DiskEngineIncompat);
		}

		*pContext = new CPGPdiskContext;
	}
	catch (CComboError& caughtErr)
	{
		error = caughtErr.pgpErr;
	}

	return error;
}

void 
PGPdiskEngineExport 
PGPdiskFreeContext(PGPdiskContextRef context)
{
	pgpAssert(PGPdiskContextRefIsValid(context));

	try
	{
		delete context;
		context = NULL;
	}
	catch (CComboError&) { }
}

PGPError 
PGPdiskEngineExport 
PGPdiskOpenPeekContextPGPKeyDB(
	PGPdiskContextRef	context, 
	PGPKeyDBRef			*pPGPKeyDB)
{
	PGPValidateParam(PGPdiskContextRefIsValid(context));
	PGPValidatePtr(pPGPKeyDB);

	PGPError	error	= kPGPError_NoErr;

	try
	{
		*pPGPKeyDB = context->PGPKeyDB();
	}
	catch (CComboError& caughtErr)
	{
		error = caughtErr.pgpErr;
	}

	return error;
}

PGPError 
PGPdiskEngineExport 
PGPdiskReloadContextPGPKeyDB(PGPdiskContextRef context)
{
	PGPValidateParam(PGPdiskContextRefIsValid(context));

	PGPError	error	= kPGPError_NoErr;

	try
	{
		context->ReloadPGPKeyDB();
	}
	catch (CComboError& caughtErr)
	{
		error = caughtErr.pgpErr;
	}

	return error;
}

PGPError 
PGPdiskEngineExport 
PGPdiskDirtyPGPKeyDB(PGPdiskContextRef context)
{
	PGPValidateParam(PGPdiskContextRefIsValid(context));

	context->mKeyDBModified=TRUE;
	return kPGPError_NoErr;
}

PGPContextRef 
PGPdiskEngineExport 
PGPdiskPeekContextPGPContext(PGPdiskContextRef context)
{
	pgpAssert(PGPdiskContextRefIsValid(context));

	PGPContextRef	pgpContext;

	try
	{
		pgpContext = context->PGPContext();
	}
	catch (CComboError&) { }

	return pgpContext;
}

PGPPrefRef 
PGPdiskEngineExport 
PGPdiskPeekContextPGPclPrefs(PGPdiskContextRef context)
{
	pgpAssert(PGPdiskContextRefIsValid(context));

	PGPPrefRef	pgpCLPrefs;

	try
	{
		pgpCLPrefs = context->PGPclPrefs();
	}
	catch (CComboError&) { }

	return pgpCLPrefs;
}

PGPError 
PGPdiskEngineExport 
PGPdiskContextIsThereADK(PGPdiskContextRef context, PGPBoolean *pIsThereADK)
{
	PGPValidateParam(PGPdiskContextRefIsValid(context));
	PGPValidatePtr(pIsThereADK);

	PGPError	error	= kPGPError_NoErr;

	try
	{
		*pIsThereADK = context->IsTherePGPdiskADK();
	}
	catch (CComboError& caughtErr)
	{
		error = caughtErr.pgpErr;
	}

	return error;
}

PGPError 
PGPdiskEngineExport 
PGPdiskContextGetADK(PGPdiskContextRef context, PGPKeyDBObjRef *pADKKey)
{
	PGPValidateParam(PGPdiskContextRefIsValid(context));
	PGPValidatePtr(pADKKey);

	PGPError	error	= kPGPError_NoErr;

	try
	{
		CPGPKey	adkKey;

		context->GetPGPdiskADKPubKey(adkKey);
		*pADKKey = adkKey;
	}
	catch (CComboError& caughtErr)
	{
		error = caughtErr.pgpErr;
	}

	return error;
}

PGPError 
PGPdiskEngineExport 
PGPdiskContextGetADKUserID(
	PGPdiskContextRef	context, 
	char				*userID, 
	PGPSize				availLength)
{
	PGPValidateParam(PGPdiskContextRefIsValid(context));
	PGPValidatePtr(userID);

	PGPError	error	= kPGPError_NoErr;

	try
	{
		CString	cUserID;
		context->GetPGPdiskADKUserID(cUserID);

		UString::SmartStringCopy(userID, cUserID.Get(), availLength);
	}
	catch (CComboError& caughtErr)
	{
		error = caughtErr.pgpErr;
	}

	return error;
}

/* PGPdiskUserRef functions */

PGPError 
PGPdiskEngineExport 
PGPdiskNewUserWithSymmetricKey(
	PGPdiskUserRef	cipheringUser, 
	const char		*userName, 
	const char		*passphrase, 
	PGPBoolean		isReadOnly, 
	PGPdiskUserRef	*pNewUser)
{
	PGPValidateParam(PGPdiskUserRefIsValid(cipheringUser));
	PGPValidatePtr(userName);
	PGPValidatePtr(passphrase);
	PGPValidatePtr(pNewUser);

	PGPError	error	= kPGPError_NoErr;

	try
	{
		auto_ptr<CPGPdiskUser>	pTheNewUser(
			new CPGPdiskUser(cipheringUser->Context()));

		CCipherContext	cipher;
		cipheringUser->GetCipherContext(cipher);

		pTheNewUser->Create(cipher, userName, passphrase, isReadOnly);
		*pNewUser = pTheNewUser.release();
	}
	catch (CComboError& caughtErr)
	{
		error = caughtErr.pgpErr;
	}

	return error;
}

PGPError 
PGPdiskEngineExport 
PGPdiskNewUserWithPublicKey(
	PGPdiskUserRef	cipheringUser, 
	PGPKeyDBObjRef	pubKey, 
	PGPBoolean		isReadOnly, 
	PGPBoolean		isLocked, 
	PGPdiskUserRef	*pNewUser)
{
	PGPValidateParam(PGPdiskUserRefIsValid(cipheringUser));
	PGPValidateParam(PGPKeyDBObjRefIsValid(pubKey));
	PGPValidatePtr(pNewUser);

	PGPError	error	= kPGPError_NoErr;

	try
	{
		auto_ptr<CPGPdiskUser>	pTheNewUser(
			new CPGPdiskUser(cipheringUser->Context()));

		CCipherContext	cipher;
		cipheringUser->GetCipherContext(cipher);

		pTheNewUser->Create(cipher, pubKey, isReadOnly, isLocked);
		*pNewUser = pTheNewUser.release();
	}
	catch (CComboError& caughtErr)
	{
		error = caughtErr.pgpErr;
	}

	return error;
}

PGPError 
PGPdiskEngineExport 
PGPdiskDuplicateUser(PGPdiskUserRef user, PGPdiskUserRef *pNewUser)
{
	PGPValidateParam(PGPdiskUserRefIsValid(user));
	PGPValidatePtr(pNewUser);

	PGPError	error	= kPGPError_NoErr;

	try
	{
		*pNewUser = new CPGPdiskUser(*user);
	}
	catch (CComboError& caughtErr)
	{
		error = caughtErr.pgpErr;
	}

	return error;
}

void 
PGPdiskEngineExport 
PGPdiskFreeUser(PGPdiskUserRef user)
{
	pgpAssert(PGPdiskUserRefIsValid(user));

	try
	{
		delete user;
	}
	catch (CComboError&) { }
}

PGPdiskContextRef 
PGPdiskEngineExport
PGPdiskPeekUserContext(PGPdiskUserRef user)
{
	pgpAssert(PGPdiskUserRefIsValid(user));

	PGPdiskContextRef	result	= kInvalidPGPdiskContextRef;

	try
	{
		result = const_cast<CPGPdiskContext *>(user->Context());
	}
	catch (CComboError&) { }

	return result;
}

PGPError 
PGPdiskEngineExport 
PGPdiskUserEnableCiphering(
	PGPdiskUserRef	user, 
	PGPdiskDiskRef	disk, 
	const char		*passphrase,
	int				passkeylen)
{
	PGPValidateParam(PGPdiskUserRefIsValid(user));
	PGPValidateParam(PGPdiskDiskRefIsValid(disk));

	PGPError	error	= kPGPError_NoErr;

	try
	{
		disk->EnableCipheringOnUser(*user, passphrase, passkeylen);
	}
	catch (CComboError& caughtErr)
	{
		error = caughtErr.pgpErr;
	}

	return error;
}

void 
PGPdiskEngineExport 
PGPdiskUserDisableCiphering(PGPdiskUserRef user)
{
	pgpAssert(PGPdiskUserRefIsValid(user));

	try
	{
		user->DisableCiphering();
	}
	catch (CComboError&) { }
}

PGPError 
PGPdiskEngineExport 
PGPdiskUserGetPGPKeyID(PGPdiskUserRef user, PGPKeyID *pKeyID)
{
	PGPValidateParam(PGPdiskUserRefIsValid(user));
	PGPValidatePtr(pKeyID);

	PGPError	error	= kPGPError_NoErr;

	try
	{
		CPGPKeyID	keyID;
		user->GetPGPKeyID(keyID);

		*pKeyID = keyID;
	}
	catch (CComboError& caughtErr)
	{
		error = caughtErr.pgpErr;
	}

	return error;
}

PGPError 
PGPdiskEngineExport 
PGPdiskUserChangePassphrase(
	PGPdiskUserRef	symUserForCiphering, 
	const char		*newPassphrase)
{
	PGPValidateParam(PGPdiskUserRefIsValid(symUserForCiphering));
	PGPValidatePtr(newPassphrase);

	PGPError	error	= kPGPError_NoErr;

	try
	{
		symUserForCiphering->ChangePassphrase(newPassphrase);
	}
	catch (CComboError& caughtErr)
	{
		error = caughtErr.pgpErr;
	}

	return error;
}

PGPBoolean 
PGPdiskEngineExport 
PGPdiskUserAreUsersEqual(PGPdiskUserRef user1, PGPdiskUserRef user2)
{
	pgpAssert(PGPdiskUserRefIsValid(user1));
	pgpAssert(PGPdiskUserRefIsValid(user2));

	PGPBoolean	result	= FALSE;

	try
	{
		result = (*user1 == *user2);
	}
	catch (CComboError&) { }

	return result;
}

PGPError 
PGPdiskEngineExport 
PGPdiskGetUserBooleanProp(
	PGPdiskUserRef		user, 
	PGPdiskUserPropName	prop, 
	PGPBoolean			*pBool)
{
	PGPValidateParam(PGPdiskUserRefIsValid(user));
	PGPValidatePtr(pBool);

	PGPError	error	= kPGPError_NoErr;

	try
	{
		switch (prop)
		{
		case kPGPdiskUserProp_HasPublicKey:
			*pBool = user->HasPublicKey();
			break;

		case kPGPdiskUserProp_HasSymmetricKey:
			*pBool = user->HasSymmetricKey();
			break;

		case kPGPdiskUserProp_IsReadOnly:
			*pBool = user->IsReadOnly();
			break;

		case kPGPdiskUserProp_IsLocked:
			*pBool = user->IsLocked();
			break;

		case kPGPdiskUserProp_IsAdminUser:
			*pBool = user->IsAdminUser();
			break;

		case kPGPdiskUserProp_IsDisabled:
			*pBool = user->IsDisabled();
			break;

		case kPGPdiskUserProp_IsCipheringEnabled:
			*pBool = user->IsCipheringEnabled();
			break;

		case kPGPdiskUserProp_CanSetReadOnly:
			*pBool = user->CanSetReadOnly();
			break;

		case kPGPdiskUserProp_CanSetUserName:
			*pBool = user->CanSetUserName();
			break;

		default:
			THROW_PGPERROR(kPGPError_BadParams);
			break;
		}
	}
	catch (CComboError& caughtErr)
	{
		error = caughtErr.pgpErr;
	}

	return error;
}

PGPError 
PGPdiskEngineExport 
PGPdiskGetUserNumericProp(
	PGPdiskUserRef		user, 
	PGPdiskUserPropName	prop, 
	PGPInt32			*pNumber)
{
	PGPValidateParam(PGPdiskUserRefIsValid(user));
	PGPValidatePtr(pNumber);

	PGPError	error	= kPGPError_NoErr;

	try
	{
		switch (prop)
		{
		case 0:
		default:
			THROW_PGPERROR(kPGPError_BadParams);
			break;
		}
	}
	catch (CComboError& caughtErr)
	{
		error = caughtErr.pgpErr;
	}

	return error;
}

PGPError 
PGPdiskEngineExport 
PGPdiskGetUserDataProp(
	PGPdiskUserRef		user, 
	PGPdiskUserPropName	prop, 
	void				*pData, 
	PGPSize				availLength, 
	PGPSize				*pUsedLength)
{
	PGPValidateParam(PGPdiskUserRefIsValid(user));
	PGPValidatePtr(pData);

	PGPError	error	= kPGPError_NoErr;

	try
	{
		CString	temp;
		PGPSize	usedLength;

		switch (prop)
		{
		case kPGPdiskUserProp_UserName:
			user->GetUserName(temp);

			if (availLength < temp.Length() + 1)
				THROW_PGPERROR(kPGPError_BufferTooSmall);

			strcpy(static_cast<char *>(pData), temp);
			usedLength = temp.Length() + 1;
			break;

		default:
			THROW_PGPERROR(kPGPError_BadParams);
			break;
		}

		if (IsntNull(pUsedLength))
			*pUsedLength = usedLength;
	}
	catch (CComboError& caughtErr)
	{
		error = caughtErr.pgpErr;
	}

	return error;
}

PGPError 
PGPdiskEngineExport 
PGPdiskSetUserBooleanProp(
	PGPdiskUserRef		user, 
	PGPdiskUserPropName	prop, 
	PGPBoolean			boolVal)
{
	PGPValidateParam(PGPdiskUserRefIsValid(user));

	PGPError	error	= kPGPError_NoErr;

	try
	{
		switch (prop)
		{
		case kPGPdiskUserProp_IsReadOnly:
			user->SetIsReadOnly(boolVal);
			break;

		case kPGPdiskUserProp_IsLocked:
			user->SetIsLocked(boolVal);
			break;

		case kPGPdiskUserProp_IsDisabled:
			user->SetIsDisabled(boolVal);
			break;

		default:
			THROW_PGPERROR(kPGPError_BadParams);
			break;
		}
	}
	catch (CComboError& caughtErr)
	{
		error = caughtErr.pgpErr;
	}

	return error;
}

PGPError 
PGPdiskEngineExport 
PGPdiskSetUserNumericProp(
	PGPdiskUserRef		user, 
	PGPdiskUserPropName	prop, 
	PGPInt32			number)
{
	PGPValidateParam(PGPdiskUserRefIsValid(user));

	PGPError	error	= kPGPError_NoErr;

	try
	{
		switch (prop)
		{
		case 0:
		default:
			THROW_PGPERROR(kPGPError_BadParams);
			break;
		}
	}
	catch (CComboError& caughtErr)
	{
		error = caughtErr.pgpErr;
	}

	return error;
}

PGPError 
PGPdiskEngineExport 
PGPdiskSetUserDataProp(
	PGPdiskUserRef		user, 
	PGPdiskUserPropName	prop, 
	const void			*pData, 
	PGPSize				dataLength)
{
	PGPValidateParam(PGPdiskUserRefIsValid(user));
	PGPValidatePtr(pData);

	PGPError	error	= kPGPError_NoErr;

	try
	{
		switch (prop)
		{
		case kPGPdiskUserProp_UserName:
			user->SetUserName(static_cast<const char *>(pData));
			break;

		default:
			THROW_PGPERROR(kPGPError_BadParams);
			break;
		}
	}
	catch (CComboError& caughtErr)
	{
		error = caughtErr.pgpErr;
	}

	return error;
}

/* PGPdiskUserSetRef functions */

PGPError 
PGPdiskEngineExport 
PGPdiskNewUserSet(PGPdiskUserSetRef *pUserSet)
{
	PGPValidatePtr(pUserSet);

	PGPError	error	= kPGPError_NoErr;

	try
	{
		*pUserSet = new CPGPdiskUserSet;
	}
	catch (CComboError& caughtErr)
	{
		error = caughtErr.pgpErr;
	}

	return error;
}

PGPError 
PGPdiskEngineExport 
PGPdiskNewSingletonUserSet(PGPdiskUserRef user, PGPdiskUserSetRef *pUserSet)
{
	PGPValidateParam(PGPdiskUserRefIsValid(user));
	PGPValidatePtr(pUserSet);

	PGPError	error	= kPGPError_NoErr;

	try
	{
		*pUserSet = new CPGPdiskUserSet(*user);
	}
	catch (CComboError& caughtErr)
	{
		error = caughtErr.pgpErr;
	}

	return error;
}

void 
PGPdiskEngineExport 
PGPdiskFreeUserSet(PGPdiskUserSetRef userSet)
{
	pgpAssert(PGPdiskUserSetRefIsValid(userSet));

	try
	{
		delete userSet;
	}
	catch (CComboError&) { }
}

PGPUInt32 
PGPdiskEngineExport 
PGPdiskUserSetCount(PGPdiskUserSetRef userSet)
{
	pgpAssert(PGPdiskUserSetRefIsValid(userSet));

	PGPUInt32	count	= 0;

	try
	{
		count = userSet->Count();
	}
	catch (CComboError&) { }

	return count;
}

PGPError 
PGPdiskEngineExport 
PGPdiskUserSetAddUsers(
	PGPdiskUserSetRef	userSet, 
	PGPdiskUserSetRef	usersToAdd)
{
	PGPValidateParam(PGPdiskUserSetRefIsValid(userSet));
	PGPValidateParam(PGPdiskUserSetRefIsValid(usersToAdd));

	PGPError	error	= kPGPError_NoErr;

	try
	{
		userSet->Add(*usersToAdd);
	}
	catch (CComboError& caughtErr)
	{
		error = caughtErr.pgpErr;
	}

	return error;
}

PGPError 
PGPdiskEngineExport 
PGPdiskUserSetAddUser(
	PGPdiskUserSetRef	userSet, 
	PGPdiskUserRef		userToAdd)
{
	PGPValidateParam(PGPdiskUserSetRefIsValid(userSet));
	PGPValidateParam(PGPdiskUserRefIsValid(userToAdd));

	PGPError	error	= kPGPError_NoErr;

	try
	{
		userSet->Add(*userToAdd);
	}
	catch (CComboError& caughtErr)
	{
		error = caughtErr.pgpErr;
	}

	return error;
}

PGPError 
PGPdiskEngineExport 
PGPdiskUserSetRemoveUsers(
	PGPdiskUserSetRef	userSet, 
	PGPdiskUserSetRef	usersToRemove)
{
	PGPValidateParam(PGPdiskUserSetRefIsValid(userSet));
	PGPValidateParam(PGPdiskUserSetRefIsValid(usersToRemove));

	PGPError	error	= kPGPError_NoErr;

	try
	{
		userSet->Remove(*usersToRemove);
	}
	catch (CComboError& caughtErr)
	{
		error = caughtErr.pgpErr;
	}

	return error;
}

PGPError 
PGPdiskEngineExport 
PGPdiskUserSetRemoveUser(
	PGPdiskUserSetRef	userSet, 
	PGPdiskUserRef		userToRemove)
{
	PGPValidateParam(PGPdiskUserSetRefIsValid(userSet));
	PGPValidateParam(PGPdiskUserRefIsValid(userToRemove));

	PGPError	error	= kPGPError_NoErr;

	try
	{
		userSet->Remove(*userToRemove);
	}
	catch (CComboError& caughtErr)
	{
		error = caughtErr.pgpErr;
	}

	return error;
}

PGPError 
PGPdiskEngineExport 
PGPdiskUserSetEmptyUserSet(PGPdiskUserSetRef userSet)
{
	PGPValidateParam(PGPdiskUserSetRefIsValid(userSet));

	PGPError	error	= kPGPError_NoErr;

	try
	{
		userSet->Empty();
	}
	catch (CComboError& caughtErr)
	{
		error = caughtErr.pgpErr;
	}

	return error;
}

/* PGPdiskUserIterRef functions */

PGPError 
PGPdiskEngineExport 
PGPdiskNewUserIter(PGPdiskUserSetRef userSet, PGPdiskUserIterRef *pUserIter)
{
	PGPValidateParam(PGPdiskUserSetRefIsValid(userSet));
	PGPValidatePtr(pUserIter);

	PGPError	error	= kPGPError_NoErr;

	try
	{
		*pUserIter = new CPGPdiskUserIter(userSet);
	}
	catch (CComboError& caughtErr)
	{
		error = caughtErr.pgpErr;
	}

	return error;
}

void 
PGPdiskEngineExport 
PGPdiskFreeUserIter(PGPdiskUserIterRef userIter)
{
	pgpAssert(PGPdiskUserIterRefIsValid(userIter));

	try
	{
		delete userIter;
	}
	catch (CComboError&) { }
}

PGPdiskUserSetRef 
PGPdiskEngineExport 
PGPdiskPeekUserSet(PGPdiskUserIterRef userIter)
{
	pgpAssert(PGPdiskUserIterRefIsValid(userIter));

	PGPdiskUserSetRef	userSet	= kInvalidPGPdiskUserSetRef;

	try
	{
		userSet = const_cast<CPGPdiskUserSet *>(userIter->UserSet());
	}
	catch (CComboError&) { }

	return userSet;
}

PGPUInt32 
PGPdiskEngineExport 
PGPdiskUserIterCount(PGPdiskUserIterRef userIter)
{
	pgpAssert(PGPdiskUserIterRefIsValid(userIter));

	PGPUInt32	count	= 0;

	try
	{
		count = userIter->Count();
	}
	catch (CComboError&) { }

	return count;
}

PGPInt32 
PGPdiskEngineExport 
PGPdiskUserIterIndex(PGPdiskUserIterRef userIter)
{
	pgpAssert(PGPdiskUserIterRefIsValid(userIter));

	PGPInt32	index	= -1;

	try
	{
		index = userIter->Index();
	}
	catch (CComboError&) { }

	return index;
}

PGPBoolean 
PGPdiskEngineExport 
PGPdiskUserIterUser(PGPdiskUserIterRef userIter, PGPdiskUserRef *pUser)
{
	pgpAssert(PGPdiskUserIterRefIsValid(userIter));

	PGPBoolean	succeeded	= FALSE;

	try
	{
		*pUser = userIter->Current();
		succeeded = IsntNull(*pUser);
	}
	catch (CComboError&) { }

	return succeeded;
}

PGPBoolean 
PGPdiskEngineExport 
PGPdiskUserIterMove(
	PGPdiskUserIterRef	userIter, 
	PGPInt32			relOffset, 
	PGPBoolean			fromStart, 
	PGPdiskUserRef		*pUser)
{
	pgpAssert(PGPdiskUserIterRefIsValid(userIter));
	pgpAssertAddrValid(pUser, PGPdiskUserRef);

	PGPBoolean	succeeded	= FALSE;

	try
	{
		*pUser = userIter->Move(relOffset, fromStart);
		succeeded = IsntNull(*pUser);
	}
	catch (CComboError&) { }

	return succeeded;
}

PGPInt32 
PGPdiskEngineExport 
PGPdiskUserIterSeek(PGPdiskUserIterRef userIter, PGPdiskUserRef user)
{
	pgpAssert(PGPdiskUserIterRefIsValid(userIter));
	pgpAssert(PGPdiskUserRefIsValid(user));

	PGPInt32	index	= -1;

	try
	{
		index = userIter->Seek(user);
	}
	catch (CComboError&) { }

	return index;
}

PGPBoolean 
PGPdiskEngineExport 
PGPdiskUserIterNext(PGPdiskUserIterRef userIter, PGPdiskUserRef *pUser)
{
	pgpAssert(PGPdiskUserIterRefIsValid(userIter));
	pgpAssertAddrValid(pUser, PGPdiskUserRef);

	PGPBoolean	succeeded	= FALSE;

	try
	{
		*pUser = userIter->Next();
		succeeded = IsntNull(*pUser);
	}
	catch (CComboError&) { }

	return succeeded;
}

PGPBoolean 
PGPdiskEngineExport 
PGPdiskUserIterPrev(PGPdiskUserIterRef userIter, PGPdiskUserRef *pUser)
{
	pgpAssert(PGPdiskUserIterRefIsValid(userIter));
	pgpAssertAddrValid(pUser, PGPdiskUserRef);

	PGPBoolean	succeeded	= FALSE;

	try
	{
		*pUser = userIter->Prev();
		succeeded = IsntNull(*pUser);
	}
	catch (CComboError&) { }

	return succeeded;
}

void 
PGPdiskEngineExport 
PGPdiskUserIterRewind(PGPdiskUserIterRef userIter)
{
	pgpAssert(PGPdiskUserIterRefIsValid(userIter));

	try
	{
		userIter->Rewind();
	}
	catch (CComboError&) { }
}

PGPBoolean 
PGPdiskEngineExport 
PGPdiskUserIterSearchOnUserName(
	PGPdiskUserIterRef	userIter, 
	const char			*userName, 
	PGPdiskUserRef		*pUser)
{
	pgpAssert(PGPdiskUserIterRefIsValid(userIter));
	pgpAssertStrValid(userName);
	pgpAssertAddrValid(pUser, PGPdiskUserRef);

	PGPBoolean	succeeded	= FALSE;

	try
	{
		*pUser = userIter->SearchOnUserName(userName);
		succeeded = IsntNull(*pUser);
	}
	catch (CComboError&) { }

	return succeeded;
}

PGPBoolean 
PGPdiskEngineExport 
PGPdiskUserIterFindEqualUser(
	PGPdiskUserIterRef	userIter, 
	PGPdiskUserRef		user, 
	PGPdiskUserRef		*pEqualUser)
{
	pgpAssert(PGPdiskUserIterRefIsValid(userIter));
	pgpAssert(PGPdiskUserRefIsValid(user));
	pgpAssertAddrValid(pEqualUser, PGPdiskUserRef);

	PGPBoolean	succeeded	= FALSE;

	try
	{
		*pEqualUser = userIter->FindEqualUser(*user);
		succeeded = IsntNull(*pEqualUser);
	}
	catch (CComboError&) { }

	return succeeded;
}

PGPBoolean 
PGPdiskEngineExport 
PGPdiskUserIterFindAdminUser(
	PGPdiskUserIterRef	userIter, 
	PGPdiskUserRef		*pAdminUser)
{
	pgpAssert(PGPdiskUserIterRefIsValid(userIter));
	pgpAssertAddrValid(pAdminUser, PGPdiskUserRef);

	PGPBoolean	succeeded	= FALSE;

	try
	{
		*pAdminUser = userIter->FindAdminUser();
		succeeded = IsntNull(*pAdminUser);
	}
	catch (CComboError&) { }

	return succeeded;
}

PGPError 
PGPdiskEngineExport 
PGPdiskUserIterSetAdminUser(
	PGPdiskUserIterRef	userIter, 
	PGPdiskUserRef		userForCiphering, 
	PGPdiskUserRef		oldAdminForCiphering)
{
	PGPValidateParam(PGPdiskUserIterRefIsValid(userIter));
	PGPValidateParam(PGPdiskUserRefIsValid(userForCiphering));

	// oldAdminForCiphering may not be needed

	PGPError	error	= kPGPError_NoErr;

	try
	{
		userIter->SafeSetAdminUser(userForCiphering, oldAdminForCiphering);
	}
	catch (CComboError& caughtErr)
	{
		error = caughtErr.pgpErr;
	}

	return error;
}

/* PGPdiskDiskRef operations */

PGPError 
PGPdiskEngineExport 
PGPdiskOpenDisk(
	PGPdiskContextRef	context, 
	const char			*path, 
	PGPBoolean			readOnly, 
	PGPdiskDiskRef		*pDisk)
{
	PGPValidateParam(PGPdiskContextRefIsValid(context));
	PGPValidatePtr(path);
	PGPValidatePtr(pDisk);

	PGPError	error	= kPGPError_NoErr;

	try
	{
		auto_ptr<CPGPdiskDisk>	pNewDisk(new CPGPdiskDisk);
		pNewDisk->Open(context, path, readOnly);

		*pDisk = pNewDisk.release();
	}
	catch (CComboError& caughtErr)
	{
		error = caughtErr.pgpErr;
	}

	return error;
}

PGPError 
PGPdiskEngineExport 
PGPdiskCloseDisk(PGPdiskDiskRef disk)
{
	PGPValidateParam(PGPdiskDiskRefIsValid(disk));

	PGPError	error	= kPGPError_NoErr;

	try
	{
		disk->Close();
		delete disk;
	}
	catch (CComboError& caughtErr)
	{
		error = caughtErr.pgpErr;
	}

	return error;
}

PGPdiskContextRef 
PGPdiskEngineExport 
PGPdiskPeekDiskContext(PGPdiskDiskRef disk)
{
	pgpAssert(PGPdiskDiskRefIsValid(disk));

	PGPdiskContextRef	context	= kInvalidPGPdiskContextRef;

	try
	{
		context = const_cast<CPGPdiskContext *>(disk->Context());
	}
	catch (CComboError&) { }

	return context;
}

PGPError 
PGPdiskEngineExport 
PGPdiskDiskExportUserSet(PGPdiskDiskRef disk, PGPdiskUserSetRef userSet)
{
	PGPValidateParam(PGPdiskDiskRefIsValid(disk));
	PGPValidateParam(PGPdiskUserSetRefIsValid(userSet));

	PGPError	error	= kPGPError_NoErr;

	try
	{
		disk->ExportUserSet(*userSet);
	}
	catch (CComboError& caughtErr)
	{
		error = caughtErr.pgpErr;
	}

	return error;
}

PGPError 
PGPdiskEngineExport 
PGPdiskDiskImportUserSet(
	PGPdiskDiskRef		disk, 
	PGPdiskUserSetRef	userSet, 
	PGPdiskUserRef		adminUserForCiphering)
{
	PGPValidateParam(PGPdiskDiskRefIsValid(disk));
	PGPValidateParam(PGPdiskUserSetRefIsValid(userSet));
	PGPValidateParam(PGPdiskUserRefIsValid(adminUserForCiphering));

	PGPError	error	= kPGPError_NoErr;

	try
	{
		CPGPdiskUserIter	userIter(userSet);
		CPGPdiskUser		*pAdminUser	= userIter.FindAdminUser();

		if ((*adminUserForCiphering != *pAdminUser) || 
			(!adminUserForCiphering->IsCipheringEnabled()))
		{
			THROW_PGPERROR(kPGPError_BadParams);
		}

		disk->ImportUserSet(*userSet);
	}
	catch (CComboError& caughtErr)
	{
		error = caughtErr.pgpErr;
	}

	return error;
}

PGPError 
PGPdiskEngineExport 
PGPdiskDiskChangeOnePassphrase(
	PGPdiskDiskRef	disk, 
	const char		*userName, 
	const char		*oldPassphrase, 
	const char		*newPassphrase)
{
	PGPValidateParam(PGPdiskDiskRefIsValid(disk));
	PGPValidatePtr(userName);
	PGPValidatePtr(oldPassphrase);
	PGPValidatePtr(newPassphrase);

	PGPError	error	= kPGPError_NoErr;

	try
	{
		CPGPdiskUserSet	userSet;
		disk->ExportUserSet(userSet);

		CPGPdiskUserIter	userIter(&userSet);
		CPGPdiskUser		*pUser	= userIter.SearchOnUserName(userName);

		if (IsNull(pUser) || !pUser->HasSymmetricKey())
			THROW_PGPERROR(kPGPError_BadParams);

		// This is just conventional and the zero ok -wjb
		disk->EnableCipheringOnUser(*pUser, oldPassphrase, 0); 
		pUser->ChangePassphrase(newPassphrase);

		disk->ImportUserSet(userSet);
	}
	catch (CComboError& caughtErr)
	{
		error = caughtErr.pgpErr;
	}

	return error;
}

PGPError 
PGPdiskEngineExport 
PGPdiskDiskRemoveOneUser(PGPdiskDiskRef disk, PGPdiskUserRef userForCiphering)
{
	PGPValidateParam(PGPdiskDiskRefIsValid(disk));
	PGPValidateParam(PGPdiskUserRefIsValid(userForCiphering));

	PGPError	error	= kPGPError_NoErr;

	try
	{
		CPGPdiskUserSet	userSet;
		disk->ExportUserSet(userSet);

		CPGPdiskUserIter	userIter(&userSet);
		
		if (!userIter.Seek(userForCiphering) || 
			!userForCiphering->IsCipheringEnabled())
		{
			THROW_PGPERROR(kPGPError_BadParams);
		}

		CPGPdiskUserSet	usersToRemove(*userForCiphering);
		userSet.Remove(usersToRemove);

		disk->ImportUserSet(userSet);
	}
	catch (CComboError& caughtErr)
	{
		error = caughtErr.pgpErr;
	}

	return error;
}

PGPError 
PGPdiskEngineExport 
PGPdiskWipeUsersOnDisk(PGPdiskDiskRef disk)
{
	PGPValidateParam(PGPdiskDiskRefIsValid(disk));

	PGPError	error	= kPGPError_NoErr;

	try
	{
		disk->WipeUsers();
	}
	catch (CComboError& caughtErr)
	{
		error = caughtErr.pgpErr;
	}

	return error;
}

PGPError 
PGPdiskEngineExport 
PGPdiskReEncryptDisk(
	PGPdiskDiskRef				disk, 
	const char					*adminPassphrase, 
	PGPdiskEncryptionAlgorithm	newAlgorithm, 
	ReEncryptDiskStatusFuncType	statusFunc, 
	void						*userValue)
{
	PGPValidateParam(PGPdiskDiskRefIsValid(disk));
	PGPValidatePtr(adminPassphrase);
	PGPValidatePtr(statusFunc);

	PGPError	error	= kPGPError_NoErr;

	try
	{
		disk->ReEncrypt(adminPassphrase, newAlgorithm, statusFunc, userValue);
	}
	catch (CComboError& caughtErr)
	{
		error = caughtErr.pgpErr;
	}

	return error;
}

PGPError 
PGPdiskEngineExport 
PGPdiskGetDiskBooleanProp(
	PGPdiskDiskRef		disk, 
	PGPdiskDiskPropName	prop, 
	PGPBoolean			*pBool)
{
	PGPValidateParam(PGPdiskDiskRefIsValid(disk));
	PGPValidatePtr(pBool);

	PGPError	error	= kPGPError_NoErr;

	try
	{
		switch (prop)
		{
		case kPGPdiskDiskProp_IsMounted:
			*pBool = disk->IsMounted();
			break;

		case kPGPdiskDiskProp_IsOpened:
			*pBool = disk->IsOpened();
			break;

		case kPGPdiskDiskProp_IsReadOnly:
			*pBool = disk->IsReadOnly();
			break;

		case kPGPdiskDiskProp_IsLockedForFormat:
			*pBool = disk->IsLockedForFormat();
			break;

		case kPGPdiskDiskProp_IsLockedForIO:
			*pBool = disk->IsLockedForIO();
			break;

		case kPGPdiskDiskProp_IsBeingReEncrypted:
			*pBool = disk->IsBeingReEncrypted();
			break;

		case kPGPdiskDiskProp_HasOldHeaders:
			*pBool = disk->HasOldHeaders();
			break;

		case kPGPdiskDiskProp_AreUsersWiped:
			*pBool = disk->AreUsersWiped();
			break;

		case kPGPdiskDiskProp_HasTimedOut:
			*pBool = disk->HasInactivityTimedOut();
			break;

		case kPGPdiskDiskProp_UsesCustomTimeout:
			*pBool = disk->UsesCustomTimeout();
			break;

		default:
			THROW_PGPERROR(kPGPError_BadParams);
			break;
		}
	}
	catch (CComboError& caughtErr)
	{
		error = caughtErr.pgpErr;
	}

	return error;
}

PGPError 
PGPdiskEngineExport 
PGPdiskGetDiskNumericProp(
	PGPdiskDiskRef		disk, 
	PGPdiskDiskPropName	prop, 
	PGPInt32			*pNumber)
{
	PGPValidateParam(PGPdiskDiskRefIsValid(disk));
	PGPValidatePtr(pNumber);

	PGPError	error	= kPGPError_NoErr;

	try
	{
		switch (prop)
		{
		case kPGPdiskDiskProp_Algorithm:
			*pNumber = disk->Algorithm();
			break;

		case kPGPdiskDiskProp_CustomTimeout:
			*pNumber = disk->CustomTimeout();
			break;

		case kPGPdiskDiskProp_BlocksDiskLow:
			*pNumber = UMath::GetLowDWord(disk->BlocksDisk());
			break;

		case kPGPdiskDiskProp_BlocksDiskHigh:
			*pNumber = UMath::GetHighDWord(disk->BlocksDisk());
			break;

		default:
			THROW_PGPERROR(kPGPError_BadParams);
			break;
		}
	}
	catch (CComboError& caughtErr)
	{
		error = caughtErr.pgpErr;
	}

	return error;
}

PGPError 
PGPdiskEngineExport 
PGPdiskGetDiskDataProp(
	PGPdiskDiskRef		disk, 
	PGPdiskDiskPropName	prop, 
	void				*pData, 
	PGPSize				availLength, 
	PGPSize				*pUsedLength)
{
	PGPValidateParam(PGPdiskDiskRefIsValid(disk));
	PGPValidatePtr(pData);

	PGPError	error	= kPGPError_NoErr;

	try
	{
		PGPSize	usedLength;

		switch (prop)
		{
		case kPGPdiskDiskProp_Path:
			if (availLength < strlen(disk->Path()) + 1)
				THROW_PGPERROR(kPGPError_BufferTooSmall);

			strcpy(static_cast<char *>(pData), disk->Path());
			usedLength = strlen(disk->Path()) + 1;
			break;

		case kPGPdiskDiskProp_Root:
			if (availLength < strlen(disk->Root()) + 1)
				THROW_PGPERROR(kPGPError_BufferTooSmall);

			strcpy(static_cast<char *>(pData), disk->Root());
			usedLength = strlen(disk->Root()) + 1;
			break;
		
		case kPGPdiskDiskProp_DefaultRoot:
			if (availLength < strlen(disk->DefaultRoot()) + 1)
				THROW_PGPERROR(kPGPError_BufferTooSmall);

			strcpy(static_cast<char *>(pData), disk->DefaultRoot());
			usedLength = strlen(disk->DefaultRoot()) + 1;
			break;

		case kPGPdiskDiskProp_DeviceName:
			if (availLength < strlen(disk->DeviceName()) + 1)
				THROW_PGPERROR(kPGPError_BufferTooSmall);

			strcpy(static_cast<char *>(pData), disk->DeviceName());
			usedLength = strlen(disk->DeviceName()) + 1;
			break;

		default:
			THROW_PGPERROR(kPGPError_BadParams);
			break;
		}

		if (IsntNull(pUsedLength))
			*pUsedLength = usedLength;
	}
	catch (CComboError& caughtErr)
	{
		error = caughtErr.pgpErr;
	}

	return error;
}

PGPError 
PGPdiskEngineExport 
PGPdiskSetDiskBooleanProp(
	PGPdiskDiskRef		disk, 
	PGPdiskDiskPropName	prop, 
	PGPBoolean			boolVal)
{
	PGPValidateParam(PGPdiskDiskRefIsValid(disk));

	PGPError	error	= kPGPError_NoErr;

	try
	{
		switch (prop)
		{
		case kPGPdiskDiskProp_UsesCustomTimeout:
			disk->SetUsesCustomTimeout(boolVal);
			break;

		default:
			THROW_PGPERROR(kPGPError_BadParams);
			break;
		}
	}
	catch (CComboError& caughtErr)
	{
		error = caughtErr.pgpErr;
	}

	return error;
}

PGPError 
PGPdiskEngineExport 
PGPdiskSetDiskNumericProp(
	PGPdiskDiskRef		disk, 
	PGPdiskDiskPropName	prop, 
	PGPInt32			number)
{
	PGPValidateParam(PGPdiskDiskRefIsValid(disk));

	PGPError	error	= kPGPError_NoErr;

	try
	{
		switch (prop)
		{
		case kPGPdiskDiskProp_CustomTimeout:
			disk->SetCustomTimeout(number);
			break;

		default:
			THROW_PGPERROR(kPGPError_BadParams);
			break;
		}
	}
	catch (CComboError& caughtErr)
	{
		error = caughtErr.pgpErr;
	}

	return error;
}

PGPError 
PGPdiskEngineExport 
PGPdiskSetDiskDataProp(
	PGPdiskDiskRef		disk, 
	PGPdiskDiskPropName	prop, 
	const void			*pData, 
	PGPSize				dataLength)
{
	PGPValidateParam(PGPdiskDiskRefIsValid(disk));
	PGPValidatePtr(pData);

	PGPError	error	= kPGPError_NoErr;

	try
	{
		switch (prop)
		{
		case kPGPdiskDiskProp_DefaultRoot:
			disk->SetDefaultRoot(static_cast<const char *>(pData));
			break;

		default:
			THROW_PGPERROR(kPGPError_BadParams);
			break;
		}
	}
	catch (CComboError& caughtErr)
	{
		error = caughtErr.pgpErr;
	}

	return error;
}

/* Specialized PGPdiskDiskRef operations */

PGPError 
PGPdiskEngineExport 
PGPdiskLockDiskVolume(PGPdiskDiskRef disk, PGPBoolean forFormat)
{
	PGPValidateParam(PGPdiskDiskRefIsValid(disk));

	PGPError	error	= kPGPError_NoErr;

	try
	{
		disk->LockVolume(forFormat);
	}
	catch (CComboError& caughtErr)
	{
		error = caughtErr.pgpErr;
	}

	return error;
}

PGPError 
PGPdiskEngineExport 
PGPdiskUnlockDiskVolume(PGPdiskDiskRef disk)
{
	PGPValidateParam(PGPdiskDiskRefIsValid(disk));

	PGPError	error	= kPGPError_NoErr;

	try
	{
		disk->UnlockVolume();
	}
	catch (CComboError& caughtErr)
	{
		error = caughtErr.pgpErr;
	}

	return error;
}

PGPError 
PGPdiskEngineExport 
PGPdiskReadDiskVolume(
	PGPdiskDiskRef	disk, 
	void			*buffer, 
	PGPUInt64		blockPos, 
	PGPUInt32		blocksLength)
{
	PGPValidateParam(PGPdiskDiskRefIsValid(disk));
	PGPValidatePtr(buffer);

	PGPError	error	= kPGPError_NoErr;

	try
	{
		disk->ReadVolume(buffer, blockPos, blocksLength);
	}
	catch (CComboError& caughtErr)
	{
		error = caughtErr.pgpErr;
	}

	return error;
}

PGPError 
PGPdiskEngineExport 
PGPdiskWriteDiskVolume(
	PGPdiskDiskRef	disk, 
	const void		*buffer, 
	PGPUInt64		blockPos, 
	PGPUInt32		blocksLength)
{
	PGPValidateParam(PGPdiskDiskRefIsValid(disk));
	PGPValidatePtr(buffer);

	PGPError	error	= kPGPError_NoErr;

	try
	{
		disk->WriteVolume(buffer, blockPos, blocksLength);
	}
	catch (CComboError& caughtErr)
	{
		error = caughtErr.pgpErr;
	}

	return error;
}

/* PGPdiskDiskSetRef functions */

PGPError 
PGPdiskEngineExport 
PGPdiskNewDiskSet(PGPdiskContextRef context, PGPdiskDiskSetRef *pDiskSet)
{
	PGPValidateParam(PGPdiskContextRefIsValid(context));
	PGPValidatePtr(pDiskSet);

	PGPError	error	= kPGPError_NoErr;

	try
	{
		*pDiskSet = new CPGPdiskDiskSet(context);
	}
	catch (CComboError& caughtErr)
	{
		error = caughtErr.pgpErr;
	}

	return error;
}

void 
PGPdiskEngineExport 
PGPdiskFreeDiskSet(PGPdiskDiskSetRef diskSet)
{
	pgpAssert(PGPdiskDiskSetRefIsValid(diskSet));

	try
	{
		delete diskSet;
	}
	catch (CComboError&) { }
}

PGPdiskContextRef 
PGPdiskEngineExport 
PGPdiskPeekDiskSetContext(PGPdiskDiskSetRef diskSet)
{
	pgpAssert(PGPdiskDiskSetRefIsValid(diskSet));

	PGPdiskContextRef	context	= kInvalidPGPdiskContextRef;

	try
	{
		context = const_cast<CPGPdiskContext *>(diskSet->Context());
	}
	catch (CComboError&) { }

	return context;
}

PGPUInt32 
PGPdiskEngineExport 
PGPdiskDiskSetCount(PGPdiskDiskSetRef userSet)
{
	pgpAssert(PGPdiskDiskSetRefIsValid(userSet));

	PGPUInt32	count	= 0;

	try
	{
		count = userSet->Count();
	}
	catch (CComboError&) { }

	return count;
}

PGPError 
PGPdiskEngineExport 
PGPdiskDiskSetMountDisk(
	PGPdiskDiskSetRef	diskSet, 
	const char			*path, 
	PGPdiskUserRef		user, 
	const char			*root, 
	PGPBoolean			readOnly, 
	PGPdiskDiskRef		*pDisk)
{
	PGPValidateParam(PGPdiskDiskSetRefIsValid(diskSet));
	PGPValidatePtr(path);
	PGPValidateParam(PGPdiskUserRefIsValid(user));
	PGPValidatePtr(root);
	PGPValidatePtr(pDisk);

	PGPError	error	= kPGPError_NoErr;

	try
	{
		*pDisk = diskSet->Mount(path, *user, root, readOnly);
	}
	catch (CComboError& caughtErr)
	{
		error = caughtErr.pgpErr;
	}

	return error;
}

PGPError 
PGPdiskEngineExport 
PGPdiskDiskSetUnmountDisk(
	PGPdiskDiskSetRef	diskSet, 
	PGPdiskDiskRef		disk, 
	PGPBoolean			isForced)
{
	PGPValidateParam(PGPdiskDiskSetRefIsValid(diskSet));
	PGPValidateParam(PGPdiskDiskRefIsValid(disk));

	PGPError	error	= kPGPError_NoErr;

	try
	{
		diskSet->Unmount(disk, isForced);
	}
	catch (CComboError& caughtErr)
	{
		error = caughtErr.pgpErr;
	}

	return error;
}


PGPError 
PGPdiskEngineExport 
PGPdiskDiskSetCreateDiskWithSym(
	PGPdiskDiskSetRef			diskSet, 
	const char					*path, 
	PGPUInt64					blocksDisk, 
	PGPdiskEncryptionAlgorithm	algorithm, 
	const char					*passphrase, 
	const char					*userName, 
	const char					*defaultRoot, 
	NewDiskStatusFuncType		statusFunc, 
	void						*userValue, 
	PGPdiskDiskRef				*pDisk)
{
	PGPValidateParam(PGPdiskDiskSetRefIsValid(diskSet));
	PGPValidatePtr(path);
	PGPValidatePtr(passphrase);
	PGPValidatePtr(userName);
	PGPValidatePtr(defaultRoot);
	PGPValidatePtr(statusFunc);
	PGPValidatePtr(pDisk);

	PGPError	error	= kPGPError_NoErr;

	try
	{
		*pDisk = diskSet->CreateDisk(path, blocksDisk, algorithm, 
			passphrase, userName, defaultRoot, statusFunc, userValue);
	}
	catch (CComboError& caughtErr)
	{
		error = caughtErr.pgpErr;
	}

	return error;
}

PGPError 
PGPdiskEngineExport 
PGPdiskDiskSetCreateDiskWithPub(
	PGPdiskDiskSetRef			diskSet, 
	const char					*path, 
	PGPUInt64					blocksDisk, 
	PGPdiskEncryptionAlgorithm	algorithm, 
	PGPKeyDBObjRef				pubKey, 
	const char					*defaultRoot, 
	NewDiskStatusFuncType		statusFunc, 
	void						*userValue, 
	PGPdiskDiskRef				*pDisk)
{
	PGPValidateParam(PGPdiskDiskSetRefIsValid(diskSet));
	PGPValidatePtr(path);
	PGPValidateParam(PGPKeyDBObjRefIsValid(pubKey));
	PGPValidatePtr(defaultRoot);
	PGPValidatePtr(statusFunc);
	PGPValidatePtr(pDisk);

	PGPError	error	= kPGPError_NoErr;

	try
	{
		*pDisk = diskSet->CreateDisk(path, blocksDisk, algorithm, pubKey, 
			defaultRoot, statusFunc, userValue);
	}
	catch (CComboError& caughtErr)
	{
		error = caughtErr.pgpErr;
	}

	return error;
}

PGPError 
PGPdiskEngineExport 
PGPdiskDiskSetUnmountTimedOut(PGPdiskDiskSetRef diskSet, PGPBoolean isForced)
{
	PGPValidateParam(PGPdiskDiskSetRefIsValid(diskSet));

	PGPError	error	= kPGPError_NoErr;

	try
	{
		diskSet->UnmountTimedOut(isForced);
	}
	catch (CComboError& caughtErr)
	{
		error = caughtErr.pgpErr;
	}

	return error;
}

PGPError 
PGPdiskEngineExport 
PGPdiskDiskSetUnmountAll(PGPdiskDiskSetRef diskSet, PGPBoolean isForced)
{
	PGPValidateParam(PGPdiskDiskSetRefIsValid(diskSet));

	PGPError	error	= kPGPError_NoErr;

	try
	{
		diskSet->UnmountAll(isForced);
	}
	catch (CComboError& caughtErr)
	{
		error = caughtErr.pgpErr;
	}

	return error;
}

PGPError 
PGPdiskEngineExport 
PGPdiskDiskSetWipeAllUsers(PGPdiskDiskSetRef diskSet)
{
	PGPValidateParam(PGPdiskDiskSetRefIsValid(diskSet));

	PGPError	error	= kPGPError_NoErr;

	try
	{
		diskSet->WipeAllUsers();
	}
	catch (CComboError& caughtErr)
	{
		error = caughtErr.pgpErr;
	}

	return error;
}

PGPError 
PGPdiskEngineExport 
PGPdiskUpdateDiskSet(PGPdiskDiskSetRef diskSet)
{
	PGPValidateParam(PGPdiskDiskSetRefIsValid(diskSet));

	PGPError	error	= kPGPError_NoErr;

	try
	{
		diskSet->Update();
	}
	catch (CComboError& caughtErr)
	{
		error = caughtErr.pgpErr;
	}

	return error;
}

/* PGPdiskDiskIterRef functions */

PGPError 
PGPdiskEngineExport 
PGPdiskNewDiskIter(PGPdiskDiskSetRef diskSet, PGPdiskDiskIterRef *pDiskIter)
{
	PGPValidateParam(PGPdiskDiskSetRefIsValid(diskSet));
	PGPValidatePtr(pDiskIter);

	PGPError	error	= kPGPError_NoErr;

	try
	{
		*pDiskIter = new CPGPdiskDiskIter(diskSet);
	}
	catch (CComboError& caughtErr)
	{
		error = caughtErr.pgpErr;
	}

	return error;
}

void 
PGPdiskEngineExport 
PGPdiskFreeDiskIter(PGPdiskDiskIterRef diskIter)
{
	pgpAssert(PGPdiskDiskIterRefIsValid(diskIter));

	try
	{
		delete diskIter;
	}
	catch (CComboError&) { }
}

PGPdiskDiskSetRef 
PGPdiskEngineExport 
PGPdiskPeekDiskIterDiskSet(PGPdiskDiskIterRef diskIter)
{
	pgpAssert(PGPdiskDiskIterRefIsValid(diskIter));

	PGPdiskDiskSetRef	diskSet	= kInvalidPGPdiskDiskSetRef;

	try
	{
		diskSet = const_cast<CPGPdiskDiskSet *>(diskIter->DiskSet());
	}
	catch (CComboError&) { }

	return diskSet;
}

PGPUInt32 
PGPdiskEngineExport 
PGPdiskDiskIterCount(PGPdiskDiskIterRef diskIter)
{
	pgpAssert(PGPdiskDiskIterRefIsValid(diskIter));

	PGPUInt32	count	= 0;

	try
	{
		count = diskIter->Count();
	}
	catch (CComboError&) { }

	return count;
}

PGPInt32 
PGPdiskEngineExport 
PGPdiskDiskIterIndex(PGPdiskDiskIterRef diskIter)
{
	pgpAssert(PGPdiskDiskIterRefIsValid(diskIter));

	PGPInt32	index	= -1;

	try
	{
		index = diskIter->Index();
	}
	catch (CComboError&) { }

	return index;
}

PGPBoolean 
PGPdiskEngineExport 
PGPdiskDiskIterDisk(PGPdiskDiskIterRef diskIter, PGPdiskDiskRef *pDisk)
{
	pgpAssert(PGPdiskDiskIterRefIsValid(diskIter));

	PGPBoolean	succeeded	= FALSE;

	try
	{
		*pDisk = diskIter->Current();
		succeeded = IsntNull(*pDisk);
	}
	catch (CComboError&) { }

	return succeeded;
}

PGPBoolean 
PGPdiskEngineExport 
PGPdiskDiskIterMove(
	PGPdiskDiskIterRef	diskIter, 
	PGPInt32			relOffset, 
	PGPBoolean			fromStart, 
	PGPdiskDiskRef		*pDisk)
{
	pgpAssert(PGPdiskDiskIterRefIsValid(diskIter));
	pgpAssertAddrValid(pDisk, PGPdiskDiskRef);

	PGPBoolean	succeeded	= FALSE;

	try
	{
		*pDisk = diskIter->Move(relOffset, fromStart);
		succeeded = IsntNull(*pDisk);
	}
	catch (CComboError&) { }

	return succeeded;
}

PGPInt32 
PGPdiskEngineExport 
PGPdiskDiskIterSeek(PGPdiskDiskIterRef diskIter, PGPdiskDiskRef disk)
{
	pgpAssert(PGPdiskDiskIterRefIsValid(diskIter));
	pgpAssert(PGPdiskDiskRefIsValid(disk));

	PGPInt32	index	= -1;

	try
	{
		index = diskIter->Seek(disk);
	}
	catch (CComboError&) { }

	return index;
}

PGPBoolean 
PGPdiskEngineExport 
PGPdiskDiskIterNext(PGPdiskDiskIterRef diskIter, PGPdiskDiskRef *pDisk)
{
	pgpAssert(PGPdiskDiskIterRefIsValid(diskIter));
	pgpAssertAddrValid(pDisk, PGPdiskDiskRef);

	PGPBoolean	succeeded	= FALSE;

	try
	{
		*pDisk = diskIter->Next();
		succeeded = IsntNull(*pDisk);
	}
	catch (CComboError&) { }

	return succeeded;
}

PGPBoolean 
PGPdiskEngineExport 
PGPdiskDiskIterPrev(PGPdiskDiskIterRef diskIter, PGPdiskDiskRef *pDisk)
{
	pgpAssert(PGPdiskDiskIterRefIsValid(diskIter));
	pgpAssertAddrValid(pDisk, PGPdiskDiskRef);

	PGPBoolean	succeeded	= FALSE;

	try
	{
		*pDisk = diskIter->Prev();
		succeeded = IsntNull(*pDisk);
	}
	catch (CComboError&) { }

	return succeeded;
}

void 
PGPdiskEngineExport 
PGPdiskDiskIterRewind(PGPdiskDiskIterRef diskIter)
{
	pgpAssert(PGPdiskDiskIterRefIsValid(diskIter));

	try
	{
		diskIter->Rewind();
	}
	catch (CComboError&) { }
}

PGPBoolean 
PGPdiskEngineExport 
PGPdiskDiskIterSearchOnPath(
	PGPdiskDiskIterRef	diskIter, 
	const char			*path, 
	PGPdiskDiskRef		*pDisk)
{
	pgpAssert(PGPdiskDiskIterRefIsValid(diskIter));
	pgpAssertStrValid(path);
	pgpAssertAddrValid(pDisk, PGPdiskDiskRef);

	PGPBoolean	succeeded	= FALSE;

	try
	{
		*pDisk = diskIter->SearchOnPath(path);
		succeeded = IsntNull(*pDisk);
	}
	catch (CComboError&) { }

	return succeeded;
}

PGPBoolean 
PGPdiskEngineExport 
PGPdiskDiskIterSearchOnRoot(
	PGPdiskDiskIterRef	diskIter, 
	const char			*root, 
	PGPdiskDiskRef		*pDisk)
{
	pgpAssert(PGPdiskDiskIterRefIsValid(diskIter));
	pgpAssertStrValid(root);
	pgpAssertAddrValid(pDisk, PGPdiskDiskRef);

	PGPBoolean	succeeded	= FALSE;

	try
	{
		*pDisk = diskIter->SearchOnRoot(root);
		succeeded = IsntNull(*pDisk);
	}
	catch (CComboError&) { }

	return succeeded;
}

/* Miscellaneous functions */

void 
PGPdiskEngineExport 
PGPdiskPrefsChanged(PGPdiskContextRef context, PGPPrefRef prefs)
{
	pgpAssert(PGPdiskContextRefIsValid(context));
	pgpAssert(PGPPrefRefIsValid(prefs));

	try
	{
		CPGPdiskDiskSet	diskSet(context);
		diskSet.UpdateTimeouts();
	}
	catch (CComboError&) { }
}

PGPBoolean 
PGPdiskEngineExport 
PGPdiskAreDisksTimedOut()
{
	PGPBoolean	result	= FALSE;

	try
	{
		result = CEngineSubsystems::DriverComm().AreDisksTimedOut();
	}
	catch (CComboError&) { }

	return result;
}

PGPBoolean 
PGPdiskEngineExport 
PGPdiskIsPathMountedPGPdisk(const char *path)
{
	pgpAssertStrValid(path);

	PGPBoolean	result	= FALSE;

	try
	{
		CEngineSubsystems::DriverComm().QueryMountedPath(path, result);
	}
	catch (CComboError&) { }

	return result;
}

PGPBoolean 
PGPdiskEngineExport 
PGPdiskIsRootMountedPGPdisk(const char *root)
{
	pgpAssertStrValid(root);

	PGPBoolean	result	= FALSE;

	try
	{
		CEngineSubsystems::DriverComm().QueryMountedRoot(root, result);
	}
	catch (CComboError&) { }

	return result;
}

PGPUInt32 
PGPdiskEngineExport 
PGPdiskGetNumDiskAlgorithms()
{
	PGPUInt32	numAlgorithms	= 0;

	try
	{
		numAlgorithms = UCryptoFactory::GetNumAlgorithms();
	}
	catch (CComboError&) { }

	return numAlgorithms;
}

void 
PGPdiskEngineExport 
PGPdiskGetDiskAlgorithmIdString(
	PGPdiskEncryptionAlgorithm	algorithm, 
	char						*idString, 
	PGPUInt32					availLength)
{
	pgpAssertAddrValid(idString, char);

	try
	{
		CString	cidString;

		UCryptoFactory::GetAlgorithmIdString(algorithm, cidString);
		UString::SmartStringCopy(idString, cidString, availLength);
	}
	catch (CComboError&) { }
}

void 
PGPdiskEngineExport 
PGPdiskGetSupportedDiskAlgorithms(
	PGPdiskEncryptionAlgorithm	*pAlgArray,	
	PGPSize						availLength, 
	PGPSize						*pUsedLength)
{
	pgpAssertAddrValid(pAlgArray, PGPdiskEncryptionAlgorithm);

	try
	{
		CArray<PGPdiskEncryptionAlgorithm>	algArray;
		PGPUInt32							usedLength;

		UCryptoFactory::GetSupportedAlgorithms(algArray, usedLength);

		for (PGPUInt32 i = 0; i < usedLength; i++)
		{
			pAlgArray[i] = algArray[i];
		}

		if (IsntNull(pUsedLength))
			*pUsedLength = usedLength;
	}
	catch (CComboError&) { }
}

PGPUInt32 
PGPdiskEngineExport 
PGPdiskGetNeededRandomDataSize(PGPdiskEncryptionAlgorithm algorithm)
{
	PGPUInt32	result	= 0;

	try
	{
		CCipherContext	cipher(algorithm);
		result = cipher.GetNeededRandomDataSize();
	}
	catch (CComboError&) { }

	return result;
}

PGPError 
PGPdiskEngineExport 
PGPdiskNotifyUserLogoff()
{
	PGPError	error	= kPGPError_NoErr;

	try
	{
		CEngineSubsystems::DriverComm().NotifyUserLogoff();
	}
	catch (CComboError& caughtErr)
	{
		error = caughtErr.pgpErr;
	}

	return error;
}
