/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation
    All rights reserved.

    $Id: CKeySign.m,v 1.19 2002/11/11 05:26:30 wprice Exp $
____________________________________________________________________________*/
#import "CKeySign.h"
#import "PGPkeysCommon.h"
#import "PGPkeysController.h"
#import <PGPclient/CKeyDBItem.h>
#import <PGPclient/pflPrefTypes.h>
#import <PGPclient/pgpClientLibCocoa.h>
#import <PGPclient/pgpClientPrefs.h>
#import <PGPclient/pgpMem.h>
#import <PGPui/CBrowserCell.h>
#import <PGPui/CPGPIcons.h>


#define kCKeySignNib				@"CKeySign"

	void
DomainToRegExp(
	const char	*domain,
	char		*regExp )
{
	char 		*s = (char *)domain,
				*d = regExp;
	
	strcpy( d, "<[^>]+[@.]" );
	d += strlen( d );
	
	for( ; *s; s++ )
	{
		switch( *s )
		{
			case '*':
			case '+':
			case '?':
			case '.':
			case '^':
			case '$':
			case '\\':
			case '[':
			case ']':
			case '-':
				*d++ = '\\';
				*d++ = *s;
				break;
			default:
				*d++ = *s;
				break;
		}
	}
	*d++ = '>';
	*d++ = '$';
	*d++ = '\0';
}

@implementation CKeySign

- (id)init
{
	if( !mInited )
	{
		[super init];
		mInited = YES;
	}
	if( !mLoadedNib )
		[self initWithNib:kCKeySignNib];
    return self;
}

- (id)initWithNib:
	(NSString*)			nibFile
{
    if( nibFile && [nibFile length] )
        [NSBundle loadNibNamed:nibFile owner:self];
	mLoadedNib = YES;
    return [self init];
}

- (id)initWithController:(id)controller keysView:(id)keysView forArray:(NSMutableArray *)userIDArray
		forKeyDB:(PGPKeyDBRef)keyDB defaultSigner:(PGPKeyDBObjRef)signer
{
	int				userIDCount,
					uidIndex;
	CKeyDBItem		*item;
	CBrowserCell	*bCell;
	NSTextFieldCell	*tCell;
	
	[self init];
	mSignedKeys = NO;
	mOwnerController = controller;
	mKeysView = keysView;
	mKeyDB = keyDB;
	mDefaultSigner = signer;
	mUserIDArray = [userIDArray retain];
	userIDCount = [mUserIDArray count];
	for( uidIndex = 0; uidIndex < userIDCount; )
	{
		item = [mUserIDArray objectAtIndex:uidIndex];
		if( [item keyDBObjType] == kPGPKeyDBObjType_Signature )
		{
			[mUserIDArray removeObjectAtIndex:uidIndex];
			userIDCount--;
		}
		else
			uidIndex++;
	}
	
	[mDisclosureButton setContinuous:YES];
	[mFewerChoicesView removeFromSuperview];
	[mMoreChoicesView removeFromSuperview];
	[mChoicesView addSubview:mFewerChoicesView];
	[mChoicesView setFrameSize:[mFewerChoicesView frame].size];
	mFewerMode = YES;
	
	[mNEImage setImage:[[[CPGPIcons shared] images] objectAtIndex:kPGPclImage_Sig]];
	[mEImage setImage:[[[CPGPIcons shared] images] objectAtIndex:kPGPclImage_SigExport]];
	[mMINEImage setImage:[[[CPGPIcons shared] images] objectAtIndex:kPGPclImage_SigMeta]];
	[mTINEImage setImage:[[[CPGPIcons shared] images] objectAtIndex:kPGPclImage_SigTI]];
	
	[mExpirationDateField setFormatter:[[[NSDateFormatter alloc]
					initWithDateFormat:[[NSUserDefaults standardUserDefaults] stringForKey:NSShortDateFormatString]
					allowNaturalLanguage:YES] autorelease]];
	[mExpirationDateField setObjectValue:[NSDate dateWithTimeIntervalSinceNow:kPGPcl_OneYearInSeconds]];
	[self sigTypeChanged:self];

	[mUserIDTableView setDataSource:self];
    [mUserIDTableView setTarget:self];
    [mUserIDTableView setDelegate:self];
	bCell = [[CBrowserCell alloc] init];
	[bCell setLeaf:YES];
	[bCell setFont:[NSFont systemFontOfSize:[NSFont smallSystemFontSize]]];
	[[[mUserIDTableView tableColumns] objectAtIndex:0] setDataCell:bCell];
	[bCell release];
	
	tCell = [[NSTextFieldCell alloc] init];
	[tCell setFont:[NSFont systemFontOfSize:[NSFont smallSystemFontSize]]];
	[[[mUserIDTableView tableColumns] objectAtIndex:1] setDataCell:tCell];
	[tCell release];
	
	[mSignWindow center];
	[mSignWindow makeKeyAndOrderFront:self];
	return self;
}

- (NSWindow *)window
{
	return mSignWindow;
}

- (IBAction)keySignAction:(id)sender
{
	PGPError	err = kPGPError_NoErr;
	PGPBoolean	split = FALSE;
	char		*passphrase = NULL;
	PGPByte		*passKey = NULL;
	PGPSize		passKeySize = 0;
	BOOL		doSign = YES;
	
	if( ![sender tag] )
	{
		PGPContextRef	context = PGPPeekKeyDBContext( mKeyDB );
		PGPKeyDBObjRef	signingKey = kInvalidPGPKeyDBObjRef;
		PGPBoolean		serverSync;
		PGPUInt32		trustLevel = 0,
						expirationDays = 0;
		Boolean			exportable = FALSE,
						domainRestrict = FALSE;
		char			domainRegExp[256];
		NSString		*domainString;
		
		if( mFewerMode )
		{
			if( [mExportableCheckbox state] )
				exportable = TRUE;
		}
		else
		{
			switch( [mSigTypeMatrix selectedTag] )
			{
				case 0:
					break;
				case 1:
					exportable = TRUE;
					break;
				case 2:
					trustLevel = [mTrustDepthStepper intValue];
					break;
				case 3:
					trustLevel = [mTrustDepthStepper intValue];
					exportable = TRUE;
					domainString = [mDomainRestrictionField stringValue];
					if( ( [domainString length] > 0 ) && ( [domainString length] < 128 ) )
					{
						DomainToRegExp( [domainString cString], domainRegExp );
						domainRestrict = TRUE;
					}
					break;
			}
			if( [mExpirationMatrix selectedTag] )
			{
				NSDate		*endNSDate		= [mExpirationDateField objectValue];
				
				if( [endNSDate compare:[NSDate dateWithTimeIntervalSinceNow:0]] != NSOrderedDescending )
				{
					NSRunInformationalAlertPanel(
						LSPK( @"Invalid Expiration", @"Title of Bad Expiration informational" ),
						LSPK( @"The signature must expire after today.", @"Bad Expiration" ),
						kPGPkeysOKButton, nil, nil );
					doSign = NO;
				}
				else
				{
					expirationDays = ( [endNSDate timeIntervalSince1970] -
							[[NSDate dateWithTimeIntervalSinceNow:0] timeIntervalSince1970] ) / 86400 + 1;
				}
			}
		}
		if( doSign )
		{
			NSEnumerator	*userIDEnumerator = [mUserIDArray objectEnumerator];
			CKeyDBItem		*item;
			PGPKeyDBObjRef	userID;
			BOOL			dupeWarning = NO;
			
			[mSignWindow orderOut:self];
			
			err = PGPSigningPassphraseDialog( context, mKeyDB, &signingKey,
							PGPOUIDialogPrompt( context, 
							[LSPK( @"Enter your passphrase:", @"KeySign" ) cString] ),
							PGPOUIOutputPassphrase( context, &passphrase ),
							PGPOUIDefaultKey( context, mDefaultSigner ),
							PGPOLastOption( context ) );
			
			if( err == kPGPError_KeyUnusableForSignature )
			{
				PGPGetKeyDBObjBooleanProperty( signingKey, kPGPKeyProperty_IsSecretShared, &split );
				/* ***** if( split )
					err = PGPclReconstitutionDialog( signingKey, mKeyDB, gTLSContext,
							&passKey, &passKeySize);*/
			}
			else if (IsntPGPError(err))
			{
				while( item = [userIDEnumerator nextObject] )
				{
					if( [item keyDBObjType] == kPGPKeyDBObjType_Key )
						PGPGetPrimaryUserID( [item keyDBItem], &userID );
					else
						userID = [item keyDBItem];
					err = PGPclGuaranteeMinimumEntropy( context );
					if( IsntPGPError( err ) )
					{
						err = PGPCertifyUserID( userID, signingKey,
								PGPOExpiration( context, expirationDays ),
								PGPOExportable( context, exportable ),
								PGPOSigTrust( context, trustLevel, kPGPKeyTrust_Complete ),
								split ?
									PGPOPasskeyBuffer( context, passKey, passKeySize ):
									PGPOPassphrase( context, passphrase ),
								domainRestrict ?
									PGPOSigRegularExpression( context, domainRegExp ) :
									PGPONullOption( context ),
								PGPOLastOption( context ) );
					}	
					if( err == kPGPError_DuplicateCert )
					{
						if( !dupeWarning )
						{
							NSRunInformationalAlertPanel(
								LSPK( @"Duplicate Signature", @"Title of Dupe Sig informational" ),
								LSPK( @"Some signatures already existed and were not changed.", @"Dupe sig" ),
								LSPK( @"OK", @"" ), nil, nil );
							dupeWarning = YES;
						}
					}
					else if ( err == kPGPError_KeyRevoked )
					{
						NSRunInformationalAlertPanel(
							LSPK( @"Revoked Key", @"Title of Revoked Key Sig informational" ),
							LSPK( @"The key you attempted to sign is revoked. The key was not signed.", @"Revoked key sig" ),
							LSPK( @"OK", @"" ), nil, nil );
					}
					else
					{
						if( IsntPGPError( err ) )
							mSignedKeys = YES;
					}
				}
				PGPGetPrefBoolean( [mOwnerController pgpPrefRef], kPGPPrefKeyServerSyncOnKeySign, &serverSync );
				if( serverSync && exportable && mSignedKeys )
				{
					// ***** server sync
				}
			}
		}
	}
	if( IsntNull( passphrase ) )
		PGPFreeData( passphrase );
	if( IsntNull( passKey ) )
		PGPFreeData( passKey );
	
	if (doSign)
		[mSignWindow performClose:self];
}

- (IBAction)sigTypeChanged:(id)sender
{
	switch( [[mSigTypeMatrix selectedCell] tag] )
	{
		case 0:
		case 1:
			[mDomainRestrictionField setEnabled:NO];
			[mTrustDepthStepper setEnabled:NO];
			[mTrustDepthField setEnabled:NO];
			break;
		case 2:
			[mDomainRestrictionField setEnabled:NO];
			[mTrustDepthField setIntValue:2];
			[mTrustDepthStepper setIntValue:2];
			[mTrustDepthStepper setMinValue:2];
			[mTrustDepthStepper setEnabled:YES];
			[mTrustDepthField setEnabled:YES];
			break;
		case 3:
			[mDomainRestrictionField setEnabled:YES];
			[mTrustDepthStepper setMinValue:1];
			[mTrustDepthField setIntValue:1];
			[mTrustDepthStepper setIntValue:1];
			[mTrustDepthStepper setEnabled:YES];
			[mTrustDepthField setEnabled:YES];
			break;
	}
}

- (IBAction)toggleChoices:(id)sender
{
	NSRect	panelRect;
	float	moreDiff = [mMoreChoicesView frame].size.height - [mFewerChoicesView frame].size.height;
	
	if( [sender state] != mFewerMode )
		return;
	if( mFewerMode )
	{
		[mFewerChoicesView removeFromSuperview];
		panelRect = [mSignWindow frame];
		panelRect.origin.y -= moreDiff;
		panelRect.size.height += moreDiff;
		[mChoicesView addSubview:mMoreChoicesView];
		[mSignWindow setFrame:panelRect display:YES animate:YES];
	}
	else
	{
		[mMoreChoicesView removeFromSuperview];
		panelRect = [mSignWindow frame];
		panelRect.origin.y += moreDiff;
		panelRect.size.height -= moreDiff;
		[mChoicesView addSubview:mFewerChoicesView];
		[mSignWindow setFrame:panelRect display:YES animate:YES];
	}
	mFewerMode = !mFewerMode;
}

- (int)numberOfRowsInTableView:(NSTableView *)tableView
{
	return [mUserIDArray count];
}

- (id)tableView:(NSTableView *)tableView objectValueForTableColumn:(NSTableColumn *)tableColumn row:(int)row
{
	id				value = nil;
	char			cstr[kPGPcl_MaxPGPUserIDLength];
	CKeyDBItem		*dbItem;
	PGPKeyDBObjRef	userID;
	PGPBoolean		isAttributeID;

	switch( [[tableColumn identifier] intValue] )
	{
		case 0:
			dbItem = [mUserIDArray objectAtIndex:row];
			if( [dbItem keyDBObjType] == kPGPKeyDBObjType_Key )
				PGPGetPrimaryUserID( [dbItem keyDBItem], &userID );
			else
				userID = [dbItem keyDBItem];
			PGPGetKeyDBObjBooleanProperty( userID, kPGPUserIDProperty_IsAttribute, &isAttributeID );
			if( isAttributeID )
			{
				PGPInt32		attributeType;
				
				PGPGetKeyDBObjNumericProperty( userID, kPGPUserIDProperty_AttributeType, &attributeType );
				if( attributeType == kPGPAttribute_Image )
					value = LSPK(@"Photograph", @"CKeySign PhotoID");
				else
					value = LSPK(@"Unknown", @"CKeySign Unknown");
			}
			else
				value = PGPclGetUserIDName( userID );
			break;
		case 1:
			PGPclGetFingerprintString( PGPPeekKeyDBObjKey( [[mUserIDArray objectAtIndex:row] keyDBItem] ), cstr );
			value = [NSString stringWithCString:cstr];
			break;
	}
	return value;
}

- (void)tableView:(NSTableView *)tableView willDisplayCell:(id)cell forTableColumn:(NSTableColumn *)tableColumn row:(int)row
{
	if( ![[tableColumn identifier] intValue] )
		[cell setImage:[[CPGPIcons shared] getKeyDBObjIcon:[[mUserIDArray objectAtIndex:row] keyDBItem]]];
}

- (BOOL)tableView:(NSTableView *)tableView shouldSelectRow:(int)row
{
	return NO;
}

- (BOOL)windowShouldClose:(id)sender
{
	[mOwnerController endKeySign:mKeysView success:mSignedKeys];
	return YES;
}

- (void)dealloc
{
	[mUserIDArray release];
	[super dealloc];
	PGPLog(( @"Deallocated CKeySign" ));
}

@end

