/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation
    All rights reserved.

    $Id: CKeySplit.m,v 1.12 2002/09/23 08:47:41 wprice Exp $
____________________________________________________________________________*/
#import "CKeySplit.h"
#import "PGPkeysCommon.h"
#import "PGPkeysController.h"
#import <PGPclient/pgpClientLibCocoa.h>
#import <PGPclient/CShareholderItem.h>
#import <PGPclient/pflPrefTypes.h>
#import <PGPclient/pgpClientPrefs.h>
#import <PGPclient/pgpShare.h>
#import <PGPclient/pgpShareFile.h>
#import <PGPclient/pgpFileSpec.h>
#import <PGPclient/pgpMem.h>
#import <PGPui/CPGPIcons.h>

#define kCKeySplitNib				@"CKeySplit"
#define kCKeySplitIconColumn		0
#define kCKeySplitNameColumn		1
#define kCKeySplitSharesColumn		2

@implementation CKeySplit

- (id)init
{
	if( !mInited )
	{
		[super init];
		mInited = YES;
	}
	if( !mLoadedNib )
		[self initWithNib:kCKeySplitNib];
    return self;
}

- (id)initWithNib:
	(NSString*)			nibFile
{
	NSTextFieldCell		*tCell;
	NSImageCell			*iCell;
	NSFont				*smallFont = [NSFont systemFontOfSize:[NSFont smallSystemFontSize]];
	
	if( nibFile && [nibFile length] )
		[NSBundle loadNibNamed:nibFile owner:self];
	mLoadedNib = YES;

	mUserImage = [[NSImage alloc] initByReferencingFile:
		[[NSBundle bundleForClass:[CKeySplit class]] pathForImageResource:@"user"]];
	
	iCell = [[NSImageCell alloc] init];
	[iCell setImageScaling:NSScaleProportionally];
	[iCell setImageAlignment:NSImageAlignCenter];
	[iCell setImageFrameStyle:NSImageFrameNone];
	[[[mShareTableView tableColumns] objectAtIndex:kCKeySplitIconColumn] setDataCell:iCell];
	[iCell release];

	tCell = [[NSTextFieldCell alloc] init];
	[tCell setFont:smallFont];
	[tCell setEditable:YES];
	[tCell setWraps:NO];
	[[[mShareTableView tableColumns] objectAtIndex:kCKeySplitNameColumn] setDataCell:tCell];
	[tCell release];

	tCell = [[NSTextFieldCell alloc] init];
	[tCell setFont:smallFont];
	[tCell setEditable:YES];
	[tCell setFormatter:mShareFormatter];
	[[[mShareTableView tableColumns] objectAtIndex:kCKeySplitSharesColumn] setDataCell:tCell];
	[tCell release];

	[mShareTableView registerForDraggedTypes:[NSArray arrayWithObjects:kPGPKeySetPBoardType, nil]];
	
    return [self init];
}

- (id)initWithController:(id)controller keysView:(id)keysView splitKey:(PGPKeyDBObjRef)splitKey
{
	[self init];
	mController = controller;
	mKeysView = keysView;
	mSplitKey = splitKey;
	mShareArray = [[NSMutableArray alloc] init];
	mSplit = NO;
	mAddedCount = 1;
	mKeyDB = PGPPeekKeyDBObjKeyDB( splitKey );
	
	[mSplitKeyImage setImage:[[CPGPIcons shared] getKeyDBObjIcon:splitKey]];
	
	[mSplitKeyField setStringValue:PGPclGetPrimaryUserIDName( splitKey )];
	PGPGetKeyID( splitKey, &mKeyID );
	
	[mSharePathField setStringValue:[NSString stringWithFormat:@"%@%@", NSHomeDirectory(),
		LSPK( @"/Desktop", @"Path extension for Desktop" )]];
	
	[self selectionChanged:self];
	[mSplitKeyWindow center];
	[mSplitKeyWindow makeKeyAndOrderFront:self];
	return self;
}

- (PGPError)saveShareFile:(PGPShareRef)shareRef forItem:(CShareholderItem *)shItem atPath:(NSString *)path
{
	PGPError			err = kPGPError_NoErr;
	PGPContextRef		context = PGPPeekKeyDBContext( mKeyDB );
	PFLFileSpecRef		fileSpecRef = NULL;
	PGPShareFileRef		shareFileRef = kInvalidPGPShareFileRef;
	PGPShareRef			splitShareRef = kInvalidPGPShareRef;
	PGPOptionListRef	encOptions = kInvalidPGPOptionListRef;
	PGPKeyDBObjRef		holderKey = kInvalidPGPKeyDBObjRef;
	PGPKeyID			keyID;
	char				userID[kPGPcl_MaxPGPUserIDLength];
	char				rawstr[kPGPcl_MaxPath];
	NSString			*finalPath,
						*prePath;
	int					strInx,
						differentiator = 0;
	PGPSize				len;

	if( [shItem isPKItem] )
	{
		[shItem keyID:&keyID];
		err = PGPFindKeyByKeyID( mKeyDB, &keyID, &holderKey );										CKERR;
		PGPGetPrimaryUserIDName( holderKey, userID, sizeof(userID) - 1, &len );
	}
	else
		strncpy( userID, [[shItem name] cString], kPGPcl_MaxPGPUserIDLength );
	strcpy( rawstr, userID );
	strInx = strcspn( rawstr, "\\/:*?""<>|[]" );
	rawstr[strInx] = '\0';
	while( ( strInx > 0 ) && ( rawstr[strInx - 1] == ' ' ) )
	{
		strInx--;
		rawstr[strInx] = '\0';
	}
	if( [shItem numShares] > 1 )
		prePath = [NSString stringWithFormat:LSPK( @"%@/%s(%d Shares)",
			@"Share filename template > 1" ), path, rawstr, [shItem numShares]];
	else
		prePath = [NSString stringWithFormat:LSPK( @"%@/%s",
			@"Share filename template == 1" ), path, rawstr];
	do
	{
		if( !differentiator++ )
			finalPath = [NSString stringWithFormat:@"%@.shf", prePath];
		else
			finalPath = [NSString stringWithFormat:@"%@-%d.shf", prePath, differentiator];
	} while( [[NSFileManager defaultManager] fileExistsAtPath:finalPath] );
	err = PFLNewFileSpecFromFullPath( PGPPeekContextMemoryMgr( context ),
			[finalPath cString], &fileSpecRef);														CKERR;
	err = PFLFileSpecCreate( fileSpecRef );															CKERR;
	err = PGPNewShareFile( fileSpecRef, &shareFileRef );											CKERR;
	err = PGPSetShareFileUserID( shareFileRef, userID );											CKERR;
	err = PGPSplitShares( shareRef, [shItem numShares], &splitShareRef );							CKERR;
	if( [shItem isPKItem] )
	{
		err = PGPSetShareFileOwnerKeyID( shareFileRef, keyID );										CKERR;
		err = PGPBuildOptionList( context, &encOptions,
					PGPOEncryptToKeyDBObj( context, holderKey ),
					PGPOLastOption( context ) );													CKERR;
	}
	else
	{
		err = PGPBuildOptionList( context, &encOptions,
							PGPOConventionalEncrypt( context,
								PGPOPassphrase( context, [shItem phrase] ),
								PGPOLastOption( context ) ),
							PGPOLastOption( context ) );											CKERR;
	}
	err = PGPCopySharesToFile( context, shareFileRef, encOptions, splitShareRef );					CKERR;
	err = PGPSaveShareFile( shareFileRef );															CKERR;
done:
	if( PGPOptionListRefIsValid( encOptions ) )
		PGPFreeOptionList( encOptions );
	if( PGPShareRefIsValid( splitShareRef ) )
		PGPFreeShares( splitShareRef );
	if( PGPShareFileRefIsValid( shareFileRef ) )
		PGPFreeShareFile( shareFileRef );
	if( IsntNull( fileSpecRef ) )
		PFLFreeFileSpec( fileSpecRef );
	return err;
}

- (BOOL)split
{
	BOOL				split = NO;
	PGPError			err = kPGPError_NoErr;
	PGPContextRef		context = PGPPeekKeyDBContext( mKeyDB );
	PGPShareRef			shareRef = kInvalidPGPShareRef;
	PGPBoolean			wasSplit = FALSE;
	PGPKeyDBObjRef		targetKey;
	PGPByte				*passKey = NULL,
						*oldPassKey = NULL;
	PGPSize				passKeySize,
						oldPassKeySize;
	char				*passphrase;
	PGPUInt32			totalValue,
						reqdValue;
	PGPInt32			version;
	NSEnumerator		*shareEnum = [mShareArray objectEnumerator];
	CShareholderItem	*shItem;
	
	totalValue	= [mTotalSharesField intValue];
	reqdValue	= [mThresholdField intValue];
	err = PGPFindKeyByKeyID( mKeyDB, &mKeyID, &targetKey );											CKERR;
	err = PGPclGetKeyPassphrase( targetKey, [mController tlsContext], [mController pgpPrefRef],
			&wasSplit, &passphrase, &oldPassKey, &oldPassKeySize );									CKERR;
	err = PGPCreateShares( context, targetKey, reqdValue, totalValue, &shareRef );					CKERR;
	err = PGPGetPasskeyFromShares( shareRef, &passKey, &passKeySize );								CKERR;
	while( shItem = [shareEnum nextObject] )
	{
		err = [self saveShareFile:shareRef forItem:shItem atPath:[mSharePathField stringValue]];	CKERR;
	}
	err = PGPChangePassphrase( targetKey, wasSplit ?
				PGPOPasskeyBuffer( context, oldPassKey, oldPassKeySize ) :
				PGPOPassphrase( context, passphrase ),
				PGPOPasskeyBuffer( context, passKey, passKeySize ),
				PGPOLastOption( context ) );														CKERR;
	err = PGPGetKeyDBObjNumericProperty( targetKey, kPGPKeyProperty_Version, &version );			CKERR;
	if( version == kPGPKeyVersion_V4 )
	{
		PGPKeyIterRef		keyIter;
		PGPKeyDBObjRef		subKey;
		
		err = PGPNewKeyIterFromKeyDB( mKeyDB, &keyIter );											CKERR;
		PGPKeyIterSeek( keyIter, targetKey );
		while( IsntPGPError ( err ) &&
			IsntPGPError( PGPKeyIterNextKeyDBObj( keyIter, kPGPKeyDBObjType_SubKey, &subKey ) ) )
		{
			err = PGPChangePassphrase( subKey, wasSplit ?
				PGPOPasskeyBuffer( context, oldPassKey, oldPassKeySize ) :
				PGPOPassphrase( context, passphrase ),
				PGPOPasskeyBuffer( context, passKey, passKeySize ),
				PGPOLastOption( context ) );
		}
		PGPFreeKeyIter( keyIter );
	}
	PGPFlushKeyDB( mKeyDB );
	split = YES;
done:
	if( IsntNull( passphrase ) )
		PGPFreeData( passphrase );
	if( IsntNull( passKey ) )
		PGPFreeData( passKey );
	if( IsntNull( oldPassKey ) )
		PGPFreeData( oldPassKey );
	if( PGPShareRefIsValid( shareRef ) )
		PGPFreeShares( shareRef );
	if( IsPGPError( err ) )
		PGPclErrorBox( err );
	return split;
}

- (void)didEndSplitConfirmSheet:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo
{
    if( returnCode == NSAlertDefaultReturn )
	{
		if( [self split] )
			mSplit = YES;
		[mSplitKeyWindow performClose:self];
	}
}

- (IBAction)dialogButtonPressed:(id)sender
{
	if( ![sender tag] )
	{
		if( [mThresholdField intValue] <= [mTotalSharesField intValue] )
		{
			NSBeginAlertSheet(
				LSPK( @"Confirm Key Split", @"Title of KeySplit confirmation" ),
				kPGPkeysOKButton, kPGPkeysCancelButton, nil, mSplitKeyWindow, 
				self, NULL, @selector(didEndSplitConfirmSheet:returnCode:contextInfo:), NULL,
				LSPK( @"The key you are splitting will no longer be able to decrypt or sign without reconstituting the required shares.\n\nAre you sure you want to do this?",
				@"Confirm KeySplit msg" ) );
		}
		else
		{
			NSBeginInformationalAlertSheet( LSPK( @"Threshold Too High", @"Split Threshold Too High" ),
				kPGPkeysOKButton, nil, nil, mSplitKeyWindow, 
				self, NULL, NULL, NULL,
				LSPK( @"The threshold must be less than or equal to the total shares.",
				@"Split Threshold Too High msg" ) );
		}
	}
	else
		[mSplitKeyWindow performClose:self];
}

- (void)updateTotal
{
	NSEnumerator		*holderEnum = [mShareArray objectEnumerator];
	CShareholderItem	*shItem;
	PGPUInt32			total = 0;
	
	while( shItem = [holderEnum nextObject] )
		total += [shItem numShares];
	[mTotalSharesField setIntValue:total];
}

- (void)browseSheetDidEnd:(NSOpenPanel*)sheet returnCode:(int)code contextInfo:(void*)info
{
	[sheet orderOut:self];
	if( code == NSOKButton )
		[mSharePathField setStringValue:[[sheet filenames] objectAtIndex:0]];
}

- (IBAction)browseButtonPressed:(id)sender
{
	NSOpenPanel	*openPanel = [NSOpenPanel openPanel];
	
	[openPanel setCanChooseFiles:NO];
	[openPanel setCanChooseDirectories:YES];
	[openPanel setAllowsMultipleSelection:NO];
	[openPanel beginSheetForDirectory:[mSharePathField stringValue]
		file:nil types:nil
		modalForWindow:mSplitKeyWindow
		modalDelegate:self
		didEndSelector:@selector(browseSheetDidEnd:returnCode:contextInfo:)
		contextInfo:NULL];
}

- (IBAction)addButtonPressed:(id)sender;
{
	CShareholderItem	*newUser = [[[CShareholderItem alloc] initUserItem] autorelease];
	PGPContextRef		context = PGPPeekKeyDBContext( mKeyDB );
	PGPError			err = kPGPError_NoErr;
	char				*passphrase = NULL;
	PGPBoolean			enforceLength = FALSE,
						enforceQuality = FALSE;
	PGPUInt32			minLength,
						minQuality;
	
	if( PGPclEnterprise() )
	{
		PGPGetPrefBoolean( [mController pgpPrefRef], kPGPPrefEnforceMinChars, &enforceLength);
		PGPGetPrefBoolean( [mController pgpPrefRef], kPGPPrefEnforceMinQuality, &enforceQuality);
		PGPGetPrefNumber( [mController pgpPrefRef], kPGPPrefMinChars, &minLength);
		PGPGetPrefNumber( [mController pgpPrefRef], kPGPPrefMinQuality, &minQuality);
	}
	
	err = PGPConfirmationPassphraseDialog( context,
			PGPOUIDialogPrompt( context, [LSPK( @"Enter a shareholder passphrase:",
				@"Add Shareholder Passphrase prompt" ) cString] ),
			PGPOUIOutputPassphrase( context, &passphrase ),
			enforceLength ?
				PGPOUIMinimumPassphraseLength( context, minLength ) :
				PGPONullOption( context ),
			enforceQuality ?
				PGPOUIMinimumPassphraseQuality( context, minQuality ) :
				PGPONullOption( context ),
			PGPOLastOption( context ) );
	if( IsntPGPError( err ) && IsntNull(passphrase) && ( passphrase[0] != '\0' ) )
	{
		[newUser setName:[NSString stringWithFormat:
			LSPK( @"UnnamedUser%d", @"Default title for Share Split users" ), mAddedCount++]];
		[newUser setPhrase:passphrase];
		[mShareArray addObject:newUser];
		[mShareTableView noteNumberOfRowsChanged];
		[self updateTotal];
		passphrase = NULL;
	}
	if( IsntNull( passphrase ) )
		PGPFreeData( passphrase );
}

- (IBAction)removeButtonPressed:(id)sender;
{
	[mShareArray removeObjectAtIndex:[mShareTableView selectedRow]];
	[mShareTableView noteNumberOfRowsChanged];
	[mShareTableView deselectAll:self];
	[self selectionChanged:self];
	[self updateTotal];
}

- (IBAction)selectionChanged:(id)sender
{
	[mRemoveButton setEnabled:( [mShareTableView selectedRow] >= 0 )];
}

- (int)numberOfRowsInTableView:(NSTableView *)tableView
{
	return [mShareArray count];
}

- (id)tableView:(NSTableView *)tableView objectValueForTableColumn:(NSTableColumn *)tableColumn row:(int)row
{
	id					value = nil;
	CShareholderItem	*shItem = [mShareArray objectAtIndex:row];
	PGPKeyID			keyID;
	PGPKeyDBObjRef		key;

	switch( [[tableColumn identifier] intValue] )
	{
		case kCKeySplitIconColumn:
			if( [shItem isPKItem] )
			{
				[shItem keyID:&keyID];
				if( IsntPGPError( PGPFindKeyByKeyID( mKeyDB, &keyID, &key ) ) )
					value = [[CPGPIcons shared] getKeyDBObjIcon:key];
			}
			else
				value = mUserImage;
			break;
		case kCKeySplitNameColumn:
			if( [shItem isPKItem] )
			{
				[shItem keyID:&keyID];
				if( IsntPGPError( PGPFindKeyByKeyID( mKeyDB, &keyID, &key ) ) )
					value = PGPclGetPrimaryUserIDName( key );
			}
			else
				return [shItem name];
			break;
		case kCKeySplitSharesColumn:
			value = [NSNumber numberWithInt:[shItem numShares]];
			break;
	}
	return value;
}

- (BOOL)tableView:(NSTableView*)table shouldEditTableColumn:(NSTableColumn*)column row:(int)row
{
	if( [[column identifier] intValue] == kCKeySplitIconColumn )
		return NO;
	else if( [[column identifier] intValue] == kCKeySplitSharesColumn )
		return YES;
	return ![[mShareArray objectAtIndex:row] isPKItem];
}

- (void)tableView:(NSTableView*)tableView setObjectValue:(id)object forTableColumn:(NSTableColumn*)column row:(int)row
{
	CShareholderItem	*shItem = [mShareArray objectAtIndex:row];
	PGPUInt32			shareValue;
	
	switch( [[column identifier] intValue] )
	{
		case kCKeySplitNameColumn:
			[shItem setName:object];
			break;
		case kCKeySplitSharesColumn:
			shareValue = [object intValue];
			if( ( shareValue > 0 ) && ( shareValue <= 99 ) )
				[shItem setNumShares:[object intValue]];
			[self updateTotal];
			break;
	}
}

- (NSDragOperation)tableView:(NSTableView*)tv validateDrop:(id <NSDraggingInfo>)info proposedRow:(int)row proposedDropOperation:(NSTableViewDropOperation)op
{
	return NSDragOperationGeneric;
}

- (BOOL)tableView:(NSTableView*)tv acceptDrop:(id <NSDraggingInfo>)info row:(int)row dropOperation:(NSTableViewDropOperation)op
{
	PGPKeySetPBoardStruct	ckvpb;
	NSData					*keySetData;
	BOOL					accepted = NO;
	
	keySetData = [[info draggingPasteboard] dataForType:kPGPKeySetPBoardType];
	if( keySetData )
	{
		[keySetData getBytes:&ckvpb length:sizeof(PGPKeySetPBoardStruct)];
		if( ( PGPPeekKeyDBContext( mKeyDB ) == ckvpb.context ) && PGPKeySetRefIsValid( ckvpb.keySet ) )
		{
			PGPKeyIterRef	keyIter;
			PGPKeyDBObjRef	key;
			
			if( IsntPGPError( PGPNewKeyIterFromKeySet( ckvpb.keySet, &keyIter ) ) )
			{
				while( IsntPGPError( PGPKeyIterNextKeyDBObj( keyIter, kPGPKeyDBObjType_Key, &key ) ) )
				{
					PGPBoolean				expired,
											revoked,
											disabled,
											canEncrypt;
					
					PGPGetKeyDBObjBooleanProperty( key, kPGPKeyProperty_IsRevoked, &revoked );
					PGPGetKeyDBObjBooleanProperty( key, kPGPKeyProperty_IsExpired, &expired );
					PGPGetKeyDBObjBooleanProperty( key, kPGPKeyProperty_IsDisabled, &disabled );
					PGPGetKeyDBObjBooleanProperty( key, kPGPKeyProperty_CanEncrypt, &canEncrypt );
					if( !expired && !revoked && !disabled && canEncrypt )
						[mShareArray addObject:[[[CShareholderItem alloc] initPKItem:key] autorelease]];
				}
				PGPFreeKeyIter( keyIter );
				[mShareTableView noteNumberOfRowsChanged];
				[self updateTotal];
				accepted = YES;
			}
		}
	}
	return accepted;
}

- (BOOL)windowShouldClose:(id)sender
{
	[mController endKeySplit:mKeysView success:mSplit];
	return YES;
}

- (NSWindow *)window
{
	return mSplitKeyWindow;
}

- (void)dealloc
{
	if( mShareArray )
	{
		[mShareArray release];
		mShareArray = nil;
	}
	[mShareFormatter release];
	[mUserImage release];
	[super dealloc];
	PGPLog(( @"Deallocated CKeySplit" ));
}

@end

