/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation
    All rights reserved.

    $Id: CSearchPanel.m,v 1.12 2002/08/06 20:09:45 dallen Exp $
____________________________________________________________________________*/
#import "PGPkeysCommon.h"
#import "PGPkeysController.h"
#import <PGP/pgpDebug.h>
#import <PGPclient/pgpClientLibCocoa.h>
#import "CKeySearch.h"
#import "CSearchPanel.h"

#define kCSearchPanelNib				@"CSearchPanel"
#define kOperatorContainsString			LSPK( @"contains", @"Search operator:contains" )
#define kOperatorNotContainsString		LSPK( @"does not contain", @"Search operator:does not contain" )
#define kOperatorIsString				LSPK( @"is", @"Search operator:is" )
#define kOperatorIsNotString			LSPK( @"is not", @"Search operator:is not" )
#define kOperatorIsAtLeastString		LSPK( @"is at least", @"Search operator:is at least" )
#define kOperatorIsAtMostString			LSPK( @"is at most", @"Search operator:is at most" )
#define kOperatorIsOnOrBeforeString		LSPK( @"is on or before", @"Search operator:is on or before" )
#define kOperatorIsOnOrAfterString		LSPK( @"is on or after", @"Search operator:is on or after" )
#define kOperatorIsSignedByString		LSPK( @"is signed by", @"Search operator:is signed by" )
#define kOperatorNotSignedByString		LSPK( @"is not signed by", @"Search operator:is not signed by" )

enum
{
	kContainsOperator = 0,
	kNotContainsOperator,
	kIsOperator,
	kIsNotOperator,
	kIsAtLeastOperator,
	kIsAtMostOperator,
	kIsOnOrBeforeOperator,
	kIsOnOrAfterOperator,
	kIsSignedByOperator,
	kNotSignedByOperator,
	kOperatorCount
};

const int kCategoryOperators[kKeySearchTypeCount][kOperatorCount] =
{
	//kKeySearchType_UserID
	{	kContainsOperator, kNotContainsOperator, kIsOperator, kIsNotOperator,
		kIsSignedByOperator, kNotSignedByOperator, -1, -1, -1, -1 },
	//kKeySearchType_KeyID
	{	kIsOperator, kIsNotOperator, -1, -1, -1, -1, -1, -1, -1, -1 },
	//kKeySearchType_KeyType
	{	kIsOperator, -1, -1, -1, -1, -1, -1, -1, -1, -1	},
	//kKeySearchType_Creation
	{	kIsOperator, kIsOnOrBeforeOperator, kIsOnOrAfterOperator, -1, -1, -1, -1, -1, -1, -1	},
	//kKeySearchType_Expiration
	{	kIsOperator, kIsOnOrBeforeOperator, kIsOnOrAfterOperator, -1, -1, -1, -1, -1, -1, -1	},
	//kKeySearchType_Status
	{	kIsOperator, kIsNotOperator, -1, -1, -1, -1, -1, -1, -1, -1	},
	//kKeySearchType_Size
	{	kIsOperator, kIsAtLeastOperator, kIsAtMostOperator, -1, -1, -1, -1, -1, -1, -1	},
};


@implementation CSearchPanel

- (id)init
{
	if( !mInited )
	{
		[super init];
		mInited = YES;
	}
	if( !mLoadedNib )
		[self initWithNib:kCSearchPanelNib];
    return self;
}

- (id)initWithNib:
	(NSString*)			nibFile
{
    if( nibFile && [nibFile length] )
        [NSBundle loadNibNamed:nibFile owner:self];
	mLoadedNib = YES;
    return [self init];
}

- (id)initWithController:(id)controller defaultType:(PGPUInt32)searchType
{
	[self init];
	mController = controller;
	[mIDField retain];
	[mIDField removeFromSuperview];
	[mKeyTypePopup retain];
	[mKeyTypePopup removeFromSuperview];
	[mRevDisPopup retain];
	[mRevDisPopup removeFromSuperview];
	[mTypePopup selectItemAtIndex:searchType];
	[self setSearchType:searchType];

	return self;
}

- (void)setEnabled:(BOOL)enabled
{
	[mIDField setEnabled:enabled];
	[mKeyTypePopup setEnabled:enabled];
	[mModifierPopup setEnabled:enabled];
	[mRevDisPopup setEnabled:enabled];
	[mTypePopup setEnabled:enabled];
}

- (void)setSearchType:(PGPUInt32)searchType
{
	int			opIndex;
	NSString	*opString = nil;
	
	[mIDField removeFromSuperview];
	[mKeyTypePopup removeFromSuperview];
	[mRevDisPopup removeFromSuperview];
	[mIDField setFormatter:nil];
	[mIDField setStringValue:@""];
	switch( searchType )
	{
		case kKeySearchType_UserID:
			[mCriteriaView addSubview:mIDField];
			break;
		case kKeySearchType_KeyID:
			[mCriteriaView addSubview:mIDField];
			break;
		case kKeySearchType_KeyType:
			[mCriteriaView addSubview:mKeyTypePopup];
			break;
		case kKeySearchType_Creation:
		case kKeySearchType_Expiration:
			[mIDField setFormatter:mDateFormatter];
			[mIDField setObjectValue:[NSDate dateWithTimeIntervalSinceNow:0]];
			[mCriteriaView addSubview:mIDField];
			break;
		case kKeySearchType_Status:
			[mCriteriaView addSubview:mRevDisPopup];
			break;
		case kKeySearchType_Size:
			[mIDField setFormatter:mSizeFormatter];
			[mIDField setObjectValue:[NSNumber numberWithInt:1024]];
			[mCriteriaView addSubview:mIDField];
			break;
	}
	[mModifierPopup removeAllItems];
	for( opIndex = 0; kCategoryOperators[searchType][opIndex] != -1; opIndex++ )
	{
		switch( kCategoryOperators[searchType][opIndex] )
		{
			case kContainsOperator:
				opString = kOperatorContainsString;
				break;
			case kNotContainsOperator:
				opString = kOperatorNotContainsString;
				break;
			case kIsOperator:
				opString = kOperatorIsString;
				break;
			case kIsNotOperator:
				opString = kOperatorIsNotString;
				break;
			case kIsAtLeastOperator:
				opString = kOperatorIsAtLeastString;
				break;
			case kIsAtMostOperator:
				opString = kOperatorIsAtMostString;
				break;
			case kIsOnOrBeforeOperator:
				opString = kOperatorIsOnOrBeforeString;
				break;
			case kIsOnOrAfterOperator:
				opString = kOperatorIsOnOrAfterString;
				break;
			case kIsSignedByOperator:
				opString = kOperatorIsSignedByString;
				break;
			case kNotSignedByOperator:
				opString = kOperatorNotSignedByString;
				break;
		}
		[mModifierPopup addItemWithTitle:opString];
	}
}

- (PGPFilterRef)filterFromKeySet:(PGPKeySetRef)keySet
{
	PGPError		err		= kPGPError_NoErr;
	PGPKeyListRef	list	= kInvalidPGPKeyListRef;
	PGPKeyIterRef	iter	= kInvalidPGPKeyIterRef;
	PGPKeyDBObjRef	key		= kInvalidPGPKeyDBObjRef;
	PGPKeyID		keyID;
	PGPFilterRef	aFilter = kInvalidPGPFilterRef;
	PGPFilterRef	idFilter= kInvalidPGPFilterRef;

	err = PGPOrderKeySet( keySet, kPGPKeyOrdering_Any, FALSE, &list );
	if( IsntPGPError( err ) )
	{
		err = PGPNewKeyIter( list, &iter );
		if( IsntPGPError( err ) )
		{
			PGPError iterError = kPGPError_NoErr;

			do
			{
				iterError = PGPKeyIterNextKeyDBObj( iter, kPGPKeyDBObjType_Key, &key );
				if( IsntPGPError( iterError ) )
				{
					err = PGPGetKeyID( key, &keyID );
					if( IsntPGPError( err ) )
					{
						err = PGPNewKeyDBObjDataFilter( [mController pgpContext],
								kPGPSigProperty_KeyID, &keyID, sizeof(keyID),
								kPGPMatchCriterion_Equal, &aFilter );
						if( idFilter == kInvalidPGPFilterRef )
							idFilter = aFilter;
						else
						{
							PGPFilterRef combinedFilter = NULL;

							err = PGPUnionFilters(idFilter, aFilter,  &combinedFilter);
							idFilter = combinedFilter;
						}
					}
				}
			} while( IsntPGPError( iterError ) && IsntPGPError( err ) );
			PGPFreeKeyIter(iter);
		}
		PGPFreeKeyList(list);
	}
	if( IsntPGPError( err ) )
		return idFilter;
	else
		return nil;
}

- (PGPFilterRef)filter
{
	PGPError				err;
	PGPContextRef			context = [mController pgpContext];
	PGPFilterRef			filter = kInvalidPGPFilterRef;
	PGPUInt32				searchType = [mTypePopup indexOfSelectedItem];
	PGPUInt32				operator = kCategoryOperators[searchType][[mModifierPopup indexOfSelectedItem]];
	PGPMatchCriterion		match = kPGPMatchCriterion_Equal;
	PGPFilterRef			mergeFilter;
	PGPTime					boundaryTime;
	PGPBoolean				negate = FALSE;
	PGPPublicKeyAlgorithm	algorithm;
	NSDate					*date;
	char					cstr[kPGPcl_MaxPGPUserIDLength];
	
	switch( operator )
	{
		case kIsOperator:
			match = kPGPMatchCriterion_Equal;
			break;
		case kIsOnOrAfterOperator:
		case kIsAtLeastOperator:
			match = kPGPMatchCriterion_GreaterOrEqual;
			break;
		case kIsOnOrBeforeOperator:
		case kIsAtMostOperator:
			match = kPGPMatchCriterion_LessOrEqual;
			break;
	}
	switch( searchType )
	{
		case kKeySearchType_UserID:
		{
			const char *nameUTF8 = [[mIDField stringValue] UTF8String];
			
			switch( operator )
			{
				case kNotContainsOperator:
					negate = TRUE;
				case kContainsOperator:
					err = PGPNewKeyDBObjDataFilter( context, kPGPUserIDProperty_Name,
							nameUTF8, strlen(nameUTF8), kPGPMatchCriterion_SubString, &filter );
					pgpAssertNoErr( err );
					if( negate && IsntPGPError( err ) )
					{
						err = PGPNegateFilter( filter, &filter );
						pgpAssertNoErr( err );
					}
					break;
				case kIsNotOperator:
					negate = TRUE;
				case kIsOperator:
					err = PGPNewKeyDBObjDataFilter( context, kPGPUserIDProperty_Name,
							nameUTF8, strlen(nameUTF8), kPGPMatchCriterion_Equal, &filter );
					pgpAssertNoErr( err );
					if( negate && IsntPGPError( err ) )
					{
						err = PGPNegateFilter( filter, &filter );
						pgpAssertNoErr( err );
					}
					break;
				case kNotSignedByOperator:
					negate = TRUE;
				case kIsSignedByOperator:
					err = PGPNewKeyDBObjDataFilter( context,
							kPGPUserIDProperty_Name, nameUTF8, strlen(nameUTF8),
							kPGPMatchCriterion_SubString, &filter );
					pgpAssertNoErr( err );
					if( IsntPGPError( err ) && PGPFilterRefIsValid( filter ) )
					{
						PGPKeySetRef	filterSet = kInvalidPGPKeySetRef;
						
						err = PGPFilterKeyDB( [mController mainDB], filter, &filterSet );
						pgpAssertNoErr( err );
						PGPFreeFilter( filter );
						filter = kInvalidPGPFilterRef;
						if( IsntPGPError( err ) )
						{
							filter = [self filterFromKeySet:filterSet];
							PGPFreeKeySet( filterSet );
							if( !PGPFilterRefIsValid( filter ) )
							{
								NSBeginInformationalAlertSheet(
									LSPK( @"Key ID Not Found", @"No Signer Key error title" ),
									kPGPkeysOKButton, nil, nil, [mCriteriaView window], 
									self, NULL, NULL, NULL,
									LSPK( @"The Signing Key ID you entered could not be found.",
									@"No Signer Key error message" ) );
							}
						}
					}
					if( negate && IsntPGPError( err ) && PGPFilterRefIsValid( filter ) )
					{
						err = PGPNegateFilter( filter, &filter );
						pgpAssertNoErr( err );
					}
					break;
			}
			break;
		}
		case kKeySearchType_KeyID:
		{
			PGPKeyID		keyID;
			
			[[mIDField stringValue] getCString:cstr maxLength:kPGPcl_MaxPGPUserIDLength - 1];
			err = PGPNewKeyIDFromString( cstr, kPGPPublicKeyAlgorithm_Invalid, &keyID );
			if( IsPGPError(err) )
			{
				NSBeginInformationalAlertSheet( LSPK( @"Bad Key ID", @"Bad Key ID error title" ),
					kPGPkeysOKButton, nil, nil, [mCriteriaView window], 
					self, NULL, NULL, NULL,
					LSPK( @"The Key ID you entered is not valid.", @"Bad Key ID error message" ) );
			}
			else
			{
				err = PGPNewKeyDBObjDataFilter( context, kPGPKeyProperty_KeyID, &keyID, sizeof(keyID),
						kPGPMatchCriterion_Equal, &filter );
				if( ( operator == kIsNotOperator ) && IsntPGPError( err ) && PGPFilterRefIsValid( filter ) )
				{
					err = PGPNegateFilter( filter, &filter );
					pgpAssertNoErr( err );
				}
			}
			break;
		}
		case kKeySearchType_KeyType:
			switch( [mKeyTypePopup indexOfSelectedItem] )
			{
				default:
				case 0:
					algorithm = kPGPPublicKeyAlgorithm_ElGamal;
					break;
				case 1:
					algorithm = kPGPPublicKeyAlgorithm_RSA;
					break;
			}
			err = PGPNewKeyDBObjNumericFilter( context, kPGPKeyProperty_AlgorithmID,
						algorithm, kPGPMatchCriterion_Equal, &filter );
			pgpAssertNoErr( err );
			break;
		case kKeySearchType_Creation:
			date = [mIDField objectValue];
			boundaryTime = PGPGetPGPTimeFromStdTime( [date timeIntervalSince1970] );
			if( match == kPGPMatchCriterion_Equal )
			{
				err = PGPNewKeyDBObjTimeFilter( context, kPGPKeyProperty_Creation,
						boundaryTime, kPGPMatchCriterion_GreaterOrEqual, &mergeFilter );
				pgpAssertNoErr( err );
				
				boundaryTime += 86400;				
				err = PGPNewKeyDBObjTimeFilter( context, kPGPKeyProperty_Creation,
						boundaryTime, kPGPMatchCriterion_LessOrEqual, &filter );
				pgpAssertNoErr( err );
				
				err = PGPIntersectFilters( filter, mergeFilter, &filter );
				pgpAssertNoErr( err );
			}
			else
			{
				err = PGPNewKeyDBObjTimeFilter( context, kPGPKeyProperty_Creation,
						boundaryTime, match, &filter );
				pgpAssertNoErr( err );
			}
			break;
		case kKeySearchType_Expiration:
			date = [mIDField objectValue];
			boundaryTime = PGPGetPGPTimeFromStdTime( [date timeIntervalSince1970] );
			if( match == kPGPMatchCriterion_Equal )
			{
				err = PGPNewKeyDBObjTimeFilter( context, kPGPKeyProperty_Expiration,
						boundaryTime, kPGPMatchCriterion_GreaterOrEqual, &mergeFilter );
				pgpAssertNoErr( err );
				
				boundaryTime += 86400;				
				err = PGPNewKeyDBObjTimeFilter( context, kPGPKeyProperty_Expiration,
						boundaryTime, kPGPMatchCriterion_LessOrEqual, &filter );
				pgpAssertNoErr( err );
				
				err = PGPIntersectFilters( filter, mergeFilter, &filter );
				pgpAssertNoErr( err );
			}
			else
			{
				err = PGPNewKeyDBObjTimeFilter( context, kPGPKeyProperty_Expiration,
						boundaryTime, match, &filter );
				pgpAssertNoErr( err );
			}
			break;
		case kKeySearchType_Status:
		{
			PGPBoolean	boolValue = FALSE;
			
			switch( operator )
			{
				case kIsOperator:
					boolValue = TRUE;
					break;
				case kIsNotOperator:
					boolValue = FALSE;
					break;
			}
			switch( [mRevDisPopup indexOfSelectedItem] )
			{
				case 0:
					err = PGPNewKeyDBObjBooleanFilter( context, kPGPKeyProperty_IsRevoked, boolValue, &filter );
					pgpAssertNoErr( err );
					break;
				case 1:
					err = PGPNewKeyDBObjBooleanFilter( context, kPGPKeyProperty_IsDisabled, boolValue, &filter );
					pgpAssertNoErr( err );
					break;
			}
			break;
		}
		case kKeySearchType_Size:
		{
			PGPInt32	keySize = [[mIDField objectValue] intValue];

			err = PGPNewKeyDBObjNumericFilter( context, kPGPKeyProperty_Bits, keySize, match, &filter );
			pgpAssertNoErr(err);
			break;
		}
	}
	return filter;
}

- (NSView *)view
{
	return mCriteriaView;
}

- (NSMutableArray *)keyViews
{
	NSMutableArray	*keyViewArray = [NSMutableArray array];
	
	[keyViewArray addObject:mTypePopup];
	[keyViewArray addObject:mModifierPopup];
	if( [mIDField superview] )
		[keyViewArray addObject:mIDField];
	if( [mRevDisPopup superview] )
		[keyViewArray addObject:mRevDisPopup];
	if( [mKeyTypePopup superview] )
		[keyViewArray addObject:mKeyTypePopup];
	return keyViewArray;
}

- (IBAction)typeChanged:(id)sender
{
	[self setSearchType:[sender indexOfSelectedItem]];
	[mController fixKeyViews:NO];
}

- (void)dealloc
{
	[mIDField release];
	[mKeyTypePopup release];
	[mRevDisPopup release];
	[super dealloc];
	PGPLog(( @"Deallocated CSearchPanel" ));
}

@end

