/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation
    All rights reserved.

    $Id: CSendShares.m,v 1.5 2002/08/06 20:09:45 dallen Exp $
____________________________________________________________________________*/
#import "CSendShares.h"
#import "PGPkeysController.h"
#import <PGPclient/pgpFileSpec.h>
#import <PGPclient/pgpClientLibCocoa.h>
#import <PGPclient/pgpClientPrefs.h>
#import <PGPclient/pflPrefTypes.h>
#import <PGPclient/pgpShareFile.h>
#import <PGPclient/pgpMem.h>
#import <PGPui/CPGPIcons.h>

#define kCSendSharesNib				@"CSendShares"
#define kCSendSharesDoneString		LSPK(@"Done",@"SendShares Status:Done")

	static PGPError
decryptCallback(
	PGPContextRef 	context,
	PGPEvent 		*event,
	PGPUserValue 	userValue)
{
	CSendShares		*sendShares = (CSendShares *)userValue;
	
	return [sendShares decryptCallback:event];
}

	PGPError
skepCallback(
	PGPskepRef			skepRef,
	PGPskepEvent		*event,
	PGPUserValue		userValue )
{
	CSendShares			*sendShares = (CSendShares *)userValue;
	
	return [sendShares skepCallback:event];
}

@implementation CSendShares

- (id)init
{
	if( !mInited )
	{
		[super init];
		mInited = YES;
	}
	if( !mLoadedNib )
		[self initWithNib:kCSendSharesNib];
    return self;
}

- (id)initWithNib:
	(NSString*)			nibFile
{
	if( nibFile && [nibFile length] )
		[NSBundle loadNibNamed:nibFile owner:self];
	mLoadedNib = YES;
    return [self init];
}

- (id)initWithController:(PGPkeysController *)controller shareFile:(NSString*)docFile
{
	[self init];
	[mProgressIndicator retain];
	[mProgressIndicator removeFromSuperview];
	mController		= controller;
	mShareFile		= [docFile retain];
	mPassphrase		= NULL;
	mShares			= NULL;
	mRecipientSet	= kInvalidPGPKeySetRef;
	mSent			= NO;
	return self;
}

- (PGPKeyDBObjRef)defaultKey
{
	PGPKeyID		*keyID;
	PGPSize			keyIDLen;
	PGPKeyDBObjRef	defaultKey = kInvalidPGPKeyDBObjRef;
	
	if( IsntPGPError( PGPGetPrefData( [mController pgpPrefRef], kPGPPrefDefaultKeyID, &keyIDLen, (void **)&keyID ) ) )
	{
		PGPFindKeyByKeyID( [mController mainDB], keyID, &defaultKey );
		PGPFreeData( keyID );
	}
	return defaultKey;
}

- (void)run
{
	PGPContextRef		context = [mController pgpContext];
	PGPError			err			= kPGPError_NoErr;
	PFLFileSpecRef		fileSpecRef = NULL;
	PGPShareFileRef		sfRef		= NULL;
	PGPOptionListRef	decodeOpt	= kInvalidPGPOptionListRef;
	PGPUInt32			numShares	= 0;
	PGPSize				nameSize	= 0;
	char				name[kPGPcl_MaxPGPUserIDLength];
	
	err = PFLNewFileSpecFromFullPath( PGPPeekContextMemoryMgr( context ), [mShareFile cString], &fileSpecRef);	CKERR;
	err = PGPOpenShareFile( fileSpecRef, &sfRef );																CKERR;
	err = PGPGetShareFileUserID( sfRef, sizeof( name ), name, &nameSize );										CKERR;
	numShares = PGPGetNumSharesInFile( sfRef );
	err = PGPBuildOptionList( context, &decodeOpt,
			PGPOKeyDBRef( context, [mController mainDB] ),
			PGPOEventHandler( context, decryptCallback, self ),
			PGPOLastOption( context ) );																		CKERR;
	err = PGPCopySharesFromFile( context, sfRef, decodeOpt, &mShares );											CKERR;
	if( mConventional )
	{
		if( IsntNull( mPassphrase ) )
		{
			PGPFreeData( mPassphrase );
			mPassphrase = NULL;
		}
		err = PGPSigningPassphraseDialog( context,
					[mController mainDB], &mOwnerKey,
					PGPOUIDialogPrompt( context, 
					[LSPK(@"Please authenticate using a public key:", @"Send Shares PK Passphrase") cString] ),
					PGPOUIDefaultKey( context, [self defaultKey] ),
					PGPOUIOutputPassphrase( context, &mPassphrase ),
					PGPOLastOption( context ) );																CKERR;
	}
	[mNameField setStringValue:[NSString stringWithCString:name]];
	[mNumSharesField setIntValue:numShares];
	[mSendSharesWindow center];
	[mSendSharesWindow makeKeyAndOrderFront:self];
done:
	if( PGPOptionListRefIsValid( decodeOpt ) )
		PGPFreeOptionList( decodeOpt );
	if( IsntNull( sfRef ) )
		PGPFreeShareFile( sfRef );
	if( IsntNull( fileSpecRef ) )
		PFLFreeFileSpec( fileSpecRef );
	if( IsntNull( mKeyIDList ) )
		PGPFreeData( mKeyIDList );
	mKeyIDList = NULL;
	if( IsPGPError( err ) && ( err != kPGPError_UserAbort ) )
	{
		PGPclErrorBox( err );
		[mSendSharesWindow performClose:self];
	}
}

- (PGPError)decryptCallback:(PGPEvent *)event
{
	PGPError			err = kPGPError_NoErr;
	PGPContextRef		context = [mController pgpContext];
	
	switch( event->type )
	{
		case kPGPEvent_PassphraseEvent:
			if( IsntNull( mPassphrase ) )
			{
				PGPFreeData( mPassphrase );
				mPassphrase = NULL;
			}
			if( event->data.passphraseData.fConventional )
			{
				err = PGPConventionalDecryptionPassphraseDialog( context,
						PGPOUIDialogPrompt( context,
						[LSPK(@"Please enter the share passphrase:", @"Send Shares Passphrase") cString] ),
						PGPOUIOutputPassphrase( context, &mPassphrase ),
						PGPOLastOption( context ) );
				mConventional = TRUE;
			}
			else
			{
				PGPKeyDBRef	newKeys;
				
				err = PGPclDecryptionPassphraseDialog( context,
						[mController tlsContext], [mController pgpPrefRef],
						[LSPK(@"Please enter the key passphrase:", @"Send Shares Passphrase PK1") cString],
						mRecipientSet, mKeyIDList, mKeyIDCount,
						kInvalidPGPOptionListRef, &mPassphrase,
						&mOwnerKey, &newKeys );
				if( PGPKeyDBRefIsValid( newKeys ) )
					PGPFreeKeyDB( newKeys );
			}
			if( IsntPGPError( err ) )
			{
				err = PGPAddJobOptions( event->job,
							PGPOPassphrase( context, mPassphrase ),
							PGPOLastOption( context ) );
			}
			break;
		case kPGPEvent_RecipientsEvent:
			mRecipientSet = event->data.recipientsData.recipientSet;
			if( PGPKeySetRefIsValid( mRecipientSet ) )
				err = PGPIncKeySetRefCount( mRecipientSet );
			if( IsntPGPError( err ) && IsntNull( event->data.recipientsData.keyIDArray ) )
			{
				PGPSize	dataSize;
				
				mKeyIDCount 	= event->data.recipientsData.keyCount;
				dataSize		= mKeyIDCount * sizeof( PGPKeyID );
				mKeyIDList		= (PGPKeyID *)PGPNewData( PGPPeekContextMemoryMgr( context ), dataSize, 0 );
				if( IsntNull( mKeyIDList ) )
					pgpCopyMemory( event->data.recipientsData.keyIDArray, mKeyIDList, dataSize );
				else
					err = kPGPError_OutOfMemory;
			}
			break;
		default:
			break;
	}
	return err;
}

- (PGPError)skepCallback:(PGPskepEvent *)event
{
	PGPError			err = kPGPError_NoErr;
	
	if( mAbort )
		err = kPGPError_UserAbort;
	else switch( event->type )
	{
		case kPGPskepEvent_ConnectEvent:
			[mNetStatusField setStringValue:LSPK(@"Connected",@"SendShares Status:Connected")];
			break;
		case kPGPskepEvent_AuthenticateEvent:
		{
			PGPKeySetRef	keySet = kInvalidPGPKeySetRef;
			
			if( !PGPKeyDBObjRefIsValid( event->data.ad.remoteKey ) )
				break;
			[mAuthField setStringValue:PGPclGetPrimaryUserIDName( event->data.ad.remoteKey )];
			[mAuthImage setImage:[[CPGPIcons shared] getKeyDBObjIcon:event->data.ad.remoteKey]];
			keySet = PGPPeekKeyDBRootKeySet( PGPPeekKeyDBObjKeyDB( event->data.ad.remoteKey ) );
			PGPCheckKeyRingSigs( keySet, [mController mainDB], FALSE, NULL, NULL );
			PGPCalculateTrust( keySet, [mController mainDB] );
			err = PGPclConfirmRemoteKeyDialog(
					[LSPK(@"Please confirm the remote key to verify that your share(s) will be sent to the correct party.",
					@"SendShares Confirm") cString], [[mRemoteAddressField stringValue] cString], event->data.ad.remoteKey,
					event->data.ad.tlsCipher, TRUE );
			if( IsPGPError( err ) )
				break;
			[mNetStatusField setStringValue:LSPK(@"Authenticated",@"SendShares Status:Authenticated")];
			break;
		}
		case kPGPskepEvent_ProgressEvent:
			[mNetStatusField setStringValue:LSPK(@"Sending",@"SendShares Status:Sending")];
			break;
		case kPGPskepEvent_CloseEvent:
			[mNetStatusField setStringValue:LSPK(@"Finishing",@"SendShares Status:Finishing")];
			break;
		default:
			break;
	}
	return err;
}

- (void)sendShares:(id)sender
{
	NSAutoreleasePool	*pool	= [[NSAutoreleasePool alloc] init];
	PGPError			err		= kPGPError_NoErr;
	PGPskepRef			skepRef	= kInvalidPGPskepRef;
			
	[mNetStatusField setStringValue:LSPK(@"Connecting",@"SendShares Status:Connecting")];
	err = PGPNewSKEP( [mController pgpContext], [mController tlsContext], &skepRef );
	if( IsntPGPError( err ) )
	{
		err = PGPskepSetEventHandler( skepRef, skepCallback, self );
		if( IsntPGPError( err ) )
			err = PGPskepSendShares( skepRef, mOwnerKey, mPassphrase, mShares,
					[[mRemoteAddressField stringValue] cString] );
		PGPFreeSKEP( skepRef );
	}
	if( IsPGPError( err ) )
	{
		[mNetStatusField setStringValue:LSPK(@"Failed",@"SendShares Status:Failed")];
		PGPclErrorBox( err );
	}
	else
	{
		[mNetStatusField setStringValue:kCSendSharesDoneString];
		[mSendButton setTitle:kCSendSharesDoneString];
		[mCancelButton setEnabled:NO];
		mSent = YES;
	}
	[mProgressIndicator stopAnimation:self];
	[mProgressIndicator removeFromSuperview];
	[mSendButton setEnabled:YES];
	[pool release];
}

- (IBAction)dialogButtonPressed:(id)sender
{
	if( ![sender tag] && !mSent )
	{
		mAbort = NO;
		[sender setEnabled:NO];
		[[mSendSharesWindow contentView] addSubview:mProgressIndicator];
		[mProgressIndicator startAnimation:self];
		[NSThread detachNewThreadSelector:@selector(sendShares:) toTarget:self withObject:self];
	}
	else
	{
		if( ![mSendButton isEnabled] )
			mAbort = YES;
		else
			[mSendSharesWindow performClose:self];
	}
}

- (BOOL)windowShouldClose:(id)sender
{
	if( ![mSendButton isEnabled] )
		return NO;
	[mController sendSharesClosed];
	return YES;
}

- (NSWindow *)window
{
	return mSendSharesWindow;
}

- (void)dealloc
{
	if( IsntNull( mPassphrase ) )
		PGPFreeData( mPassphrase );
	mPassphrase = NULL;
	if( IsntNull( mShares ) )
		PGPFreeShares( mShares );
	mShares = NULL;
	if( PGPKeySetRefIsValid( mRecipientSet ) )
		PGPFreeKeySet( mRecipientSet );
	mRecipientSet = kInvalidPGPKeySetRef;
	[mShareFile release];
	[mProgressIndicator release];
	[super dealloc];
	PGPLog(( @"Deallocated CSendShares" ));
}

@end

