/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation
    All rights reserved.

    $Id: PGPkeysPreferences.m,v 1.25 2002/10/10 08:20:58 jeffc Exp $
____________________________________________________________________________*/

#import "PGPkeysPreferences.h"

@implementation PGPkeysPreferences

- (id)init
{
	if (!mInited)
	{
		[super init];
    
		mIdentifiers = [[NSArray alloc] initWithObjects:kPGPkeysPrefsFilesIdentifier,
													   kPGPkeysPrefsServersIdentifier,
													   kPGPkeysPrefsCAIdentifier,
													   nil];
	
		 mInited = YES;
	}
	
	if (!mLoadedNib)
		[self initWithNib:kPGPkeysPreferencesNib];
    
    return self;
}

- (id)initWithNib:(NSString*)nibFile
{
	NSTableColumn*		column;
	NSTextFieldCell*	textCell;
	NSCell*				cell;
	NSImageCell*		imageCell;
	
    if (nibFile && [nibFile length])
        [NSBundle loadNibNamed:nibFile owner:self];

	mAnyDomainIcon = [[NSImage alloc] initWithContentsOfFile:[[NSBundle bundleForClass:[self class]] pathForImageResource:kPGPkeysAnyDomainCellIcon]];

	[mServersTableView setDataSource:self];
	[mServersTableView setDelegate:self];
	[mServersTableView setTarget:self];
	[mServersTableView setDoubleAction:@selector(editServer:)];
    
	if (column = [mServersTableView tableColumnWithIdentifier:kPGPkeysPrefsServerColumnIdentifier])
	{
		if (textCell = [[[NSTextFieldCell alloc] init] autorelease])
		{
			[textCell setWraps:NO];
			[textCell setFont:[NSFont systemFontOfSize:11]];
			[column setDataCell:textCell];
		}
	}
	
	if (column = [mServersTableView tableColumnWithIdentifier:kPGPkeysPrefsDomainColumnIdentifier])
	{
		cell = [[[NSCell alloc] initTextCell:@""] autorelease];
		[cell setFont:[NSFont systemFontOfSize:11]];
		[cell setAlignment:NSCenterTextAlignment];
		[column setDataCell:cell];
	}
	
	if (column = [mServersTableView tableColumnWithIdentifier:kPGPkeysPrefsListedColumnIdentifier])
	{
		if (imageCell = [[[NSImageCell alloc] init] autorelease])
			[column setDataCell:imageCell];
	}
	
	mIntFormatter = [[CIntegerFormatter alloc] init];
	[mIntFormatter setFormat:@"##0"];
	[mIntFormatter setMinimum:[NSDecimalNumber zero]];
	
	[mServerPort setFormatter:mIntFormatter];
	
	[self updateServerButtons];
	
	mLoadedNib = YES;
	
    return [self init];
}

- (id)initWithController:(id)controller
{
	mController = controller;
	
	return [self init];
}

- (id)controller
{
	return mController;
}

- (id)identifiers
{
	return mIdentifiers;
}

- (id)prefsViewForIdentifier:(NSString*)identifier
{
	if (identifier)
	{
		if ([identifier isEqualToString:kPGPkeysPrefsFilesIdentifier])
			return mFilesPrefView;
		else if ([identifier isEqualToString:kPGPkeysPrefsServersIdentifier])
			return mServersPrefView;
		else if ([identifier isEqualToString:kPGPkeysPrefsCAIdentifier])
			return mCAPrefView;
	}
	
	return nil;
}

- (id)firstPrefsResponderForIdentifier:(NSString*)identifier
{
	if (!identifier)
		return nil;
	
	if ([identifier isEqualToString:kPGPkeysPrefsFilesIdentifier])
		return mChangePubKeyring;
	else if ([identifier isEqualToString:kPGPkeysPrefsServersIdentifier])
		return mServersTableView;
	else if ([identifier isEqualToString:kPGPkeysPrefsCAIdentifier])
		return mURL;
	
	return nil;
}

- (id)lastPrefsResponderForIdentifier:(NSString*)identifier
{
	if (!identifier)
		return nil;
	
	if ([identifier isEqualToString:kPGPkeysPrefsFilesIdentifier])
		return mChangeSecKeyring;
	else if ([identifier isEqualToString:kPGPkeysPrefsServersIdentifier])
		return mVerification;
	else if ([identifier isEqualToString:kPGPkeysPrefsCAIdentifier])
		return mSelectCert;
	
	return nil;
}

- (void)setupKeyViewLoopForIdentifier:(NSString*)identifier
{
	if (!identifier)
		return;
	
	if ([identifier isEqualToString:kPGPkeysPrefsFilesIdentifier])
		[mChangePubKeyring setNextKeyView:mChangeSecKeyring];
	else if ([identifier isEqualToString:kPGPkeysPrefsServersIdentifier])
	{
		[mServersTableView setNextKeyView:mNewServer];
		[mNewServer setNextKeyView:mRemoveServer];
		[mRemoveServer setNextKeyView:mEditServer];
		[mEditServer setNextKeyView:mSetAsRoot];
		[mSetAsRoot setNextKeyView:mMoveUp];
		[mMoveUp setNextKeyView:mMoveDown];
		[mMoveDown setNextKeyView:mEncryptToUnknown];
		[mEncryptToUnknown setNextKeyView:mSigningKeys];
		[mSigningKeys setNextKeyView:mAddingNamesPhotosRevokers];
		[mAddingNamesPhotosRevokers setNextKeyView:mRevocation];
		[mRevocation setNextKeyView:mVerification];
	}
	else if ([identifier isEqualToString:kPGPkeysPrefsCAIdentifier])
	{
		[mURL setNextKeyView:mRevocationURL];
		[mRevocationURL setNextKeyView:mCAType];
		[mCAType setNextKeyView:mClearCert];
		[mClearCert setNextKeyView:mSelectCert];
	}
}

- (void)restorePanelPrefs:(NSString*)panelIdentifier
{
	PGPError			err;
	PGPBoolean			bData;
	char*				szData;
	PGPUInt32			nData;
	PGPByte*            keyID;
	PGPByte*			iasn;
	PGPUInt32			size;
	PGPUInt32			iasnSize;
	PGPUInt32			prefFlags;
	PGPPrefRef			prefRef = [mController pgpPrefRef];
	PGPKeyDBObjRef		cert;
	PGPKeyServerEntry*	ksEntries;
	PGPUInt32			ksCount;
	NSData*				ksData;
	PGPUInt32			i;
	BOOL				enabled;
	BOOL				allocated;
	
	if (panelIdentifier && [panelIdentifier isEqualToString:kPGPkeysPrefsFilesIdentifier])
	{
		/* kPGPPrefPublicKeyringFile */
		allocated = NO;
		if (IsPGPError(err = PGPGetPrefStringAlloc(prefRef, kPGPPrefPublicKeyringFile, &szData)))
			PGPLog((@"Failed to retrieve value for prefid %d (err=%d)", kPGPPrefPublicKeyringFile, err));
		
		if (IsntPGPError(err))
		{
			allocated = YES;
			if (IsntPGPError(err = PGPGetPrefFlags(prefRef, kPGPPrefPublicKeyringFile, &prefFlags)))
				enabled = prefFlags & kPGPPrefFlags_DisableGUI ? NO : YES;
			else
			{
				PGPLog((@"Failed to retrieve flags for prefid %d (err=%d)", kPGPPrefPublicKeyringFile, err));
				enabled = NO;
			}
			
			[self setPubKeyringLocation:szData enabled:enabled];
	
			if (allocated)
			{
				PGPFreeData(szData);
				szData = NULL;
			}
		}
	
		/* kPGPPrefPrivateKeyringFile */
		allocated = NO;
		if (IsPGPError(err = PGPGetPrefStringAlloc(prefRef, kPGPPrefPrivateKeyringFile, &szData)))
			PGPLog((@"Failed to retrieve value for prefid %d (err=%d)", kPGPPrefPrivateKeyringFile, err));
		
		if (IsntPGPError(err))
		{
			allocated = YES;
			if (IsntPGPError(err = PGPGetPrefFlags(prefRef, kPGPPrefPrivateKeyringFile, &prefFlags)))
				enabled = prefFlags & kPGPPrefFlags_DisableGUI ? NO : YES;
			else
			{
				PGPLog((@"Failed to retrieve flags for prefid %d (err=%d)", kPGPPrefPrivateKeyringFile, err));
				enabled = NO;
			}
			
			[self setSecKeyringLocation:szData enabled:enabled];
	
			if (allocated)
			{
				PGPFreeData(szData);
				szData = NULL;
			}
		}
	}
	else if (panelIdentifier && [panelIdentifier isEqualToString:kPGPkeysPrefsServersIdentifier])
	{
		/* kPGPPrefKeyServerList */
		if (mKeyServerEntries)
			[mKeyServerEntries release];
				
		mKeyServerEntries = [[NSMutableArray alloc] init];
				
		err = PGPGetKeyServerPrefs(prefRef, &ksEntries, &ksCount);
		pgpAssertNoErr(err);
							
		for (i = 0; i < ksCount; ++i)
		{
			/* Copy each keyserver entry into an NSData buffer */
			ksData = [NSData dataWithBytes:&(ksEntries[i]) length:sizeof(PGPKeyServerEntry)];
			[mKeyServerEntries addObject:ksData];
		}
				
		[mServersTableView reloadData];
		PGPFreeKeyServerList(ksEntries);
		
		/* kPGPPrefKeyServerSyncUnknownKeys */
		if (IsPGPError(err = PGPGetPrefBoolean(prefRef, kPGPPrefKeyServerSyncUnknownKeys, &bData)))
			PGPLog((@"Failed to retrieve value for prefid %d (err=%d)", kPGPPrefKeyServerSyncUnknownKeys, err));
		
		if (IsntPGPError(err))
		{
			if (IsntPGPError(err = PGPGetPrefFlags(prefRef, kPGPPrefKeyServerSyncUnknownKeys, &prefFlags)))
				enabled = prefFlags & kPGPPrefFlags_DisableGUI ? NO : YES;
			else
			{
				PGPLog((@"Failed to retrieve flags for prefid %d (err=%d)", kPGPPrefKeyServerSyncUnknownKeys, err));
				enabled = NO;
			}
			
			[self setSyncOnUnknownKeys:bData enabled:enabled];
		}
	
		/* kPGPPrefKeyServerSyncOnKeySign */
		if (IsPGPError(err = PGPGetPrefBoolean(prefRef, kPGPPrefKeyServerSyncOnKeySign, &bData)))
			PGPLog((@"Failed to retrieve value for prefid %d (err=%d)", kPGPPrefKeyServerSyncOnKeySign, err));
		
		if (IsntPGPError(err))
		{
			if (IsntPGPError(err = PGPGetPrefFlags(prefRef, kPGPPrefKeyServerSyncOnKeySign, &prefFlags)))
				enabled = prefFlags & kPGPPrefFlags_DisableGUI ? NO : YES;
			else
			{
				PGPLog((@"Failed to retrieve flags for prefid %d (err=%d)", kPGPPrefKeyServerSyncOnKeySign, err));
				enabled = NO;
			}
			
			[self setSyncOnKeySign:bData enabled:enabled];
		}
	
		/* kPGPPrefKeyServerSyncOnAdd */
		if (IsPGPError(err = PGPGetPrefBoolean(prefRef, kPGPPrefKeyServerSyncOnAdd, &bData)))
			PGPLog((@"Failed to retrieve value for prefid %d (err=%d)", kPGPPrefKeyServerSyncOnAdd, err));
		
		if (IsntPGPError(err))
		{
			if (IsntPGPError(err = PGPGetPrefFlags(prefRef, kPGPPrefKeyServerSyncOnAdd, &prefFlags)))
				enabled = prefFlags & kPGPPrefFlags_DisableGUI ? NO : YES;
			else
			{
				PGPLog((@"Failed to retrieve flags for prefid %d (err=%d)", kPGPPrefKeyServerSyncOnAdd, err));
				enabled = NO;
			}
			
			[self setSyncOnAdd:bData enabled:enabled];
		}
	
		/* kPGPPrefKeyServerSyncOnRevocation */
		if (IsPGPError(err = PGPGetPrefBoolean(prefRef, kPGPPrefKeyServerSyncOnRevocation, &bData)))
			PGPLog((@"Failed to retrieve value for prefid %d (err=%d)", kPGPPrefKeyServerSyncOnRevocation, err));
		
		if (IsntPGPError(err))
		{
			if (IsntPGPError(err = PGPGetPrefFlags(prefRef, kPGPPrefKeyServerSyncOnRevocation, &prefFlags)))
				enabled = prefFlags & kPGPPrefFlags_DisableGUI ? NO : YES;
			else
			{
				PGPLog((@"Failed to retrieve flags for prefid %d (err=%d)", kPGPPrefKeyServerSyncOnRevocation, err));
				enabled = NO;
			}
			
			[self setSyncOnRevocation:bData enabled:enabled];
		}
	
		/* kPGPPrefKeyServerSyncOnVerify */
		if (IsPGPError(err = PGPGetPrefBoolean(prefRef, kPGPPrefKeyServerSyncOnVerify, &bData)))
			PGPLog((@"Failed to retrieve value for prefid %d (err=%d)", kPGPPrefKeyServerSyncOnVerify, err));
		
		if (IsntPGPError(err))
		{
			if (IsntPGPError(err = PGPGetPrefFlags(prefRef, kPGPPrefKeyServerSyncOnVerify, &prefFlags)))
				enabled = prefFlags & kPGPPrefFlags_DisableGUI ? NO : YES;
			else
			{
				PGPLog((@"Failed to retrieve flags for prefid %d (err=%d)", kPGPPrefKeyServerSyncOnVerify, err));
				enabled = NO;
			}
			
			[self setSyncOnVerify:bData enabled:enabled];
		}
	}
	else if (panelIdentifier && [panelIdentifier isEqualToString:kPGPkeysPrefsCAIdentifier])
	{
		/* kPGPPrefCAServerURL */
		allocated = NO;
		if (IsPGPError(err = PGPGetPrefStringAlloc(prefRef, kPGPPrefCAServerURL, &szData)))
			PGPLog((@"Failed to retrieve value for prefid %d (err=%d)", kPGPPrefCAServerURL, err));
		
		if (IsntPGPError(err))
		{
			allocated = YES;
			if (IsntPGPError(err = PGPGetPrefFlags(prefRef, kPGPPrefCAServerURL, &prefFlags)))
				enabled = prefFlags & kPGPPrefFlags_DisableGUI ? NO : YES;
			else
			{
				PGPLog((@"Failed to retrieve flags for prefid %d (err=%d)", kPGPPrefCAServerURL, err));
				enabled = NO;
			}
			
			[self setServerURL:szData enabled:enabled];
	
			if (allocated)
			{
				PGPFreeData(szData);
				szData = NULL;
			}
		}

		/* kPGPPrefCARevocationServerURL */
		allocated = NO;
		if (IsPGPError(err = PGPGetPrefStringAlloc(prefRef, kPGPPrefCARevocationServerURL, &szData)))
			PGPLog((@"Failed to retrieve value for prefid %d (err=%d)", kPGPPrefCARevocationServerURL, err));
		
		if (IsntPGPError(err))
		{
			allocated = YES;
			if (IsntPGPError(err = PGPGetPrefFlags(prefRef, kPGPPrefCARevocationServerURL, &prefFlags)))
				enabled = prefFlags & kPGPPrefFlags_DisableGUI ? NO : YES;
			else
			{
				PGPLog((@"Failed to retrieve flags for prefid %d (err=%d)", kPGPPrefCARevocationServerURL, err));
				enabled = NO;
			}
			
			[self setRevocationServerURL:szData enabled:enabled];
	
			if (allocated)
			{
				PGPFreeData(szData);
				szData = NULL;
			}
		}

		/* kPGPPrefCAType */
		if (IsPGPError(err = PGPGetPrefNumber(prefRef, kPGPPrefCAType, &nData)))
			PGPLog((@"Failed to retrieve value for prefid %d (err=%d)", kPGPPrefCAType, err));
		
		if (IsntPGPError(err))
		{
			if (IsntPGPError(err = PGPGetPrefFlags(prefRef, kPGPPrefCAType, &prefFlags)))
				enabled = prefFlags & kPGPPrefFlags_DisableGUI ? NO : YES;
			else
			{
				PGPLog((@"Failed to retrieve flags for prefid %d (err=%d)", kPGPPrefCAType, err));
				enabled = NO;
			}
	
			[self setCAType:nData enabled:enabled];
		}
		
		/* kPGPPrefCARootKeyID && kPGPPrefCARootX509IASN */
		keyID = NULL;
		iasn = NULL;
		cert = kInvalidPGPKeyDBObjRef;

		[self clearCert:self];
		
		PGPGetPrefData(prefRef, kPGPPrefCARootKeyID, (PGPSize*) &size, (void**) &keyID);
		PGPGetPrefData(prefRef, kPGPPrefCARootX509IASN, (PGPSize*) &iasnSize, (void**) &iasn);
			
		if (keyID && iasn)
		{
			if (IsntPGPError(err = PGPGetPrefFlags(prefRef, kPGPPrefCARootKeyID, &prefFlags)))
				enabled = prefFlags & kPGPPrefFlags_DisableGUI ? NO : YES;
			else
			{
				PGPLog((@"Failed to retrieve flags for prefid %d (err=%d)", kPGPPrefCARootKeyID, err));
				enabled = NO;
			}		
			
			if (IsntPGPError(err = PGPclX509CertFromExport([mController pgpContext],
						(PGPKeyID*) keyID,
						iasn,
						iasnSize,
						[mController mainDB],
						&cert)))
			{
				[self setRootCertificate:cert enabled:enabled];
			}
			else
				PGPLog((@"Failed to retrieve X.509 cert from exported representation (err=%d)", err));
			
			PGPFreeData(keyID);
			PGPFreeData(iasn);
        }
	}
}

- (void)restorePanelDefaults:(NSString*)panelIdentifier
{
	PGPError			err;
	PGPBoolean			bData;
	char*				szData;
	PGPUInt32			nData;
	PGPUInt32			prefFlags;
	PGPPrefRef			prefRef = [mController pgpPrefRef];
	PGPPrefDefinition	prefDef;
	PGPKeyServerEntry*	ksEntries;
	PGPUInt32			ksCount;
	NSData*				ksData;
	PGPUInt32			i;
	BOOL				enabled;
	
	if (panelIdentifier && [panelIdentifier isEqualToString:kPGPkeysPrefsFilesIdentifier])
	{
		/* kPGPPrefPublicKeyringFile */
		if (IsPGPError(err = PGPclGetPrefDefault(kPGPPrefPublicKeyringFile, &prefDef)))
			PGPLog((@"Failed to retrieve default for prefid %d (err=%d)", kPGPPrefPublicKeyringFile, err));
		else
		{
			szData = (char*)prefDef.data;
			
			if (IsntPGPError(err = PGPGetPrefFlags(prefRef, kPGPPrefPublicKeyringFile, &prefFlags)))
				enabled = prefFlags & kPGPPrefFlags_DisableGUI ? NO : YES;
			else
			{
				PGPLog((@"Failed to retrieve flags for prefid %d (err=%d)", kPGPPrefPublicKeyringFile, err));
				enabled = NO;
			}
			
			[self setPubKeyringLocation:szData enabled:enabled];
		}
	
		/* kPGPPrefPrivateKeyringFile */
		if (IsPGPError(err = PGPclGetPrefDefault(kPGPPrefPrivateKeyringFile, &prefDef)))
		{
			PGPLog((@"Failed to retrieve default for prefid %d (err=%d)", kPGPPrefPrivateKeyringFile, err));
		}
		else
		{
			szData = (char*)prefDef.data;
			
			if (IsntPGPError(err = PGPGetPrefFlags(prefRef, kPGPPrefPrivateKeyringFile, &prefFlags)))
				enabled = prefFlags & kPGPPrefFlags_DisableGUI ? NO : YES;
			else
			{
				PGPLog((@"Failed to retrieve flags for prefid %d (err=%d)", kPGPPrefPrivateKeyringFile, err));
				enabled = NO;
			}
			
			[self setSecKeyringLocation:szData enabled:enabled];
		}
	}
	else if (panelIdentifier && [panelIdentifier isEqualToString:kPGPkeysPrefsServersIdentifier])
	{
		/* kPGPPrefKeyServerList */
		if (mKeyServerEntries)
			[mKeyServerEntries release];
				
		mKeyServerEntries = [[NSMutableArray alloc] init];
	
		if (IsntPGPError(err = PGPGetKeyServerListFromPrefArray(prefRef, &DefaultKeyServer, &ksEntries, &ksCount)))
		{			
			for (i = 0; i < ksCount; ++i)
			{
				/* Copy each keyserver entry into an NSData buffer */
				ksData = [NSData dataWithBytes:&(ksEntries[i]) length:sizeof(PGPKeyServerEntry)];
				[mKeyServerEntries addObject:ksData];
			}
			
			PGPFreeKeyServerList(ksEntries);
		}
		else
			PGPLog((@"Failed to retrieve default for prefid %d (err=%d)", kPGPPrefKeyServerList, err));
				
		[mServersTableView reloadData];
	
		/* kPGPPrefKeyServerSyncUnknownKeys */
		if (IsPGPError(err = PGPclGetPrefDefault(kPGPPrefKeyServerSyncUnknownKeys, &prefDef)))
		{
			PGPLog((@"Failed to retrieve default for prefid %d (err=%d)", kPGPPrefKeyServerSyncUnknownKeys, err));
		}
		else
		{
			bData = (PGPBoolean)(size_t)prefDef.data;
			
			if (IsntPGPError(err = PGPGetPrefFlags(prefRef, kPGPPrefKeyServerSyncUnknownKeys, &prefFlags)))
				enabled = prefFlags & kPGPPrefFlags_DisableGUI ? NO : YES;
			else
			{
				PGPLog((@"Failed to retrieve flags for prefid %d (err=%d)", kPGPPrefKeyServerSyncUnknownKeys, err));
				enabled = NO;
			}
			
			[self setSyncOnUnknownKeys:bData enabled:enabled];
		}
	
		/* kPGPPrefKeyServerSyncOnKeySign */
		if (IsPGPError(err = PGPclGetPrefDefault(kPGPPrefKeyServerSyncOnKeySign, &prefDef)))
		{
			PGPLog((@"Failed to retrieve default for prefid %d (err=%d)", kPGPPrefKeyServerSyncOnKeySign, err));
		}
		else
		{
			bData = (PGPBoolean)(size_t)prefDef.data;
			
			if (IsntPGPError(err = PGPGetPrefFlags(prefRef, kPGPPrefKeyServerSyncOnKeySign, &prefFlags)))
				enabled = prefFlags & kPGPPrefFlags_DisableGUI ? NO : YES;
			else
			{
				PGPLog((@"Failed to retrieve flags for prefid %d (err=%d)", kPGPPrefKeyServerSyncOnKeySign, err));
				enabled = NO;
			}
			
			[self setSyncOnKeySign:bData enabled:enabled];
		}
	
		/* kPGPPrefKeyServerSyncOnAdd */
		if (IsPGPError(err = PGPclGetPrefDefault(kPGPPrefKeyServerSyncOnAdd, &prefDef)))
		{
			PGPLog((@"Failed to retrieve default for prefid %d (err=%d)", kPGPPrefKeyServerSyncOnAdd, err));
		}
		else
		{
			bData = (PGPBoolean)(size_t)prefDef.data;
			
			if (IsntPGPError(err = PGPGetPrefFlags(prefRef, kPGPPrefKeyServerSyncOnAdd, &prefFlags)))
				enabled = prefFlags & kPGPPrefFlags_DisableGUI ? NO : YES;
			else
			{
				PGPLog((@"Failed to retrieve flags for prefid %d (err=%d)", kPGPPrefKeyServerSyncOnAdd, err));
				enabled = NO;
			}
			
			[self setSyncOnAdd:bData enabled:enabled];
		}
	
		/* kPGPPrefKeyServerSyncOnRevocation */
		if (IsPGPError(err = PGPclGetPrefDefault(kPGPPrefKeyServerSyncOnRevocation, &prefDef)))
		{
			PGPLog((@"Failed to retrieve default for prefid %d (err=%d)", kPGPPrefKeyServerSyncOnRevocation, err));
		}
		else
		{
			bData = (PGPBoolean)(size_t)prefDef.data;
			
			if (IsntPGPError(err = PGPGetPrefFlags(prefRef, kPGPPrefKeyServerSyncOnRevocation, &prefFlags)))
				enabled = prefFlags & kPGPPrefFlags_DisableGUI ? NO : YES;
			else
			{
				PGPLog((@"Failed to retrieve flags for prefid %d (err=%d)", kPGPPrefKeyServerSyncOnRevocation, err));
				enabled = NO;
			}
			
			[self setSyncOnRevocation:bData enabled:enabled];
		}
	
		/* kPGPPrefKeyServerSyncOnVerify */
		if (IsPGPError(err = PGPclGetPrefDefault(kPGPPrefKeyServerSyncOnVerify, &prefDef)))
		{
			PGPLog((@"Failed to retrieve default for prefid %d (err=%d)", kPGPPrefKeyServerSyncOnVerify, err));
		}
		else
		{
			bData = (PGPBoolean)(size_t)prefDef.data;
			
			if (IsntPGPError(err = PGPGetPrefFlags(prefRef, kPGPPrefKeyServerSyncOnVerify, &prefFlags)))
				enabled = prefFlags & kPGPPrefFlags_DisableGUI ? NO : YES;
			else
			{
				PGPLog((@"Failed to retrieve flags for prefid %d (err=%d)", kPGPPrefKeyServerSyncOnVerify, err));
				enabled = NO;
			}
			
			[self setSyncOnVerify:bData enabled:enabled];
		}
	}
	else if (panelIdentifier && [panelIdentifier isEqualToString:kPGPkeysPrefsCAIdentifier])
	{
		/* kPGPPrefCAServerURL */
		if (IsPGPError(err = PGPclGetPrefDefault(kPGPPrefCAServerURL, &prefDef)))
		{
			PGPLog((@"Failed to retrieve default for prefid %d (err=%d)", kPGPPrefCAServerURL, err));
		}
		else
		{
			szData = (char*)prefDef.data;
			
			if (IsntPGPError(err = PGPGetPrefFlags(prefRef, kPGPPrefCAServerURL, &prefFlags)))
				enabled = prefFlags & kPGPPrefFlags_DisableGUI ? NO : YES;
			else
			{
				PGPLog((@"Failed to retrieve flags for prefid %d (err=%d)", kPGPPrefCAServerURL, err));
				enabled = NO;
			}
			
			[self setServerURL:szData enabled:enabled];
		}

		/* kPGPPrefCARevocationServerURL */
		if (IsPGPError(err = PGPclGetPrefDefault(kPGPPrefCARevocationServerURL, &prefDef)))
		{
			PGPLog((@"Failed to retrieve default for prefid %d (err=%d)", kPGPPrefCARevocationServerURL, err));
		}
		else
		{
			szData = (char*)prefDef.data;
			
			if (IsntPGPError(err = PGPGetPrefFlags(prefRef, kPGPPrefCARevocationServerURL, &prefFlags)))
				enabled = prefFlags & kPGPPrefFlags_DisableGUI ? NO : YES;
			else
			{
				PGPLog((@"Failed to retrieve flags for prefid %d (err=%d)", kPGPPrefCARevocationServerURL, err));
				enabled = NO;
			}
			
			[self setRevocationServerURL:szData enabled:enabled];
		}

		/* kPGPPrefCAType */
		if (IsPGPError(err = PGPclGetPrefDefault(kPGPPrefCAType, &prefDef)))
		{
			PGPLog((@"Failed to retrieve default for prefid %d (err=%d)", kPGPPrefCAType, err));
		}
		else
		{
			nData = (PGPUInt32)prefDef.data;
			
			if (IsntPGPError(err = PGPGetPrefFlags(prefRef, kPGPPrefCAType, &prefFlags)))
				enabled = prefFlags & kPGPPrefFlags_DisableGUI ? NO : YES;
			else
			{
				PGPLog((@"Failed to retrieve flags for prefid %d (err=%d)", kPGPPrefCAType, err));
				enabled = NO;
			}
			
			[self setCAType:nData enabled:enabled];
		}
		
		/* kPGPPrefCARootKeyID && kPGPPrefCARootX509IASN */
		[self clearCert:self];
	}
}

- (void)savePanelPrefs:(NSString*)panelIdentifier
{
	PGPError				err;
	PGPPrefRef				prefRef = [mController pgpPrefRef];
	PGPKeyServerEntry*		ksEntries;
	PGPUInt32				ksIndex = 0;
	PGPKeyID				keyID;
	PGPByte*				iasn;
	PGPUInt32				iasnSize;
	BOOL					good;
	NSEnumerator*			enumerator;
	NSData*					ksData;
	NSNotificationCenter*	center = [NSNotificationCenter defaultCenter];
	
	if (panelIdentifier && [panelIdentifier isEqualToString:kPGPkeysPrefsFilesIdentifier])
	{
		/* kPGPPrefPublicKeyringFile */
		if (IsPGPError(err = PGPSetPrefString(prefRef, kPGPPrefPublicKeyringFile, [self getPubKeyringLocation])))
		{
			PGPLog((@"Failed to set value for prefid %d (err=%d)", kPGPPrefPublicKeyringFile, err));
		}
		else
			[center postNotificationName:kPGPclPrefNotification_PubKeyring object:nil];
	
		/* kPGPPrefPrivateKeyringFile */
		if (IsPGPError(err = PGPSetPrefString(prefRef, kPGPPrefPrivateKeyringFile, [self getSecKeyringLocation])))
		{
			PGPLog((@"Failed to set value for prefid %d (err=%d)", kPGPPrefPrivateKeyringFile, err));
		}
		else
			[center postNotificationName:kPGPclPrefNotification_SecKeyring object:nil];
	
		[center postNotificationName:kPGPclPrefNotification_UpdateKeyrings object:nil];
	}
	else if (panelIdentifier && [panelIdentifier isEqualToString:kPGPkeysPrefsServersIdentifier])
	{
		/* kPGPPrefKeyServerList */
		ksEntries = (PGPKeyServerEntry*) PGPNewData(PGPPeekContextMemoryMgr([mController pgpContext]),
												[mKeyServerEntries count] * sizeof(PGPKeyServerEntry),
												0);
														
		pgpAssertAddrValid(ksEntries, PGPKeyServerEntry);
		if (ksEntries)
		{
			enumerator = [mKeyServerEntries objectEnumerator];
			while (ksData = [enumerator nextObject])
				[ksData getBytes:&ksEntries[ksIndex++] length:sizeof(PGPKeyServerEntry)];
				
			err = PGPSetKeyServerPrefs(prefRef, ksEntries, ksIndex);
			if (IsPGPError(err))
			{
				PGPLog((@"%s:%d: Failed to set keyserver prefs (err=%d)", __FILE__, __LINE__, err));
			}
			else
				[center postNotificationName:kPGPclPrefNotification_KeyServerList object:nil];
			
			PGPFreeData(ksEntries);
		}
	
		/* kPGPPrefKeyServerSyncUnknownKeys */
		if (IsPGPError(err = PGPSetPrefBoolean(prefRef, kPGPPrefKeyServerSyncUnknownKeys, [self getSyncOnUnknownKeys])))
		{
			PGPLog((@"Failed to set value for prefid %d (err=%d)", kPGPPrefKeyServerSyncUnknownKeys, err));
		}
		else
			[center postNotificationName:kPGPclPrefNotification_SyncUnknownKeys object:nil];
	
		/* kPGPPrefKeyServerSyncOnKeySign */
		if (IsPGPError(err = PGPSetPrefBoolean(prefRef, kPGPPrefKeyServerSyncOnKeySign, [self getSyncOnKeySign])))
		{
			PGPLog((@"Failed to set value for prefid %d (err=%d)", kPGPPrefKeyServerSyncOnKeySign, err));
		}
		else
			[center postNotificationName:kPGPclPrefNotification_SyncOnKeySign object:nil];
	
		/* kPGPPrefKeyServerSyncOnAdd */
		if (IsPGPError(err = PGPSetPrefBoolean(prefRef, kPGPPrefKeyServerSyncOnAdd, [self getSyncOnAdd])))
		{
			PGPLog((@"Failed to set value for prefid %d (err=%d)", kPGPPrefKeyServerSyncOnAdd, err));
		}
		else
			[center postNotificationName:kPGPclPrefNotification_SyncOnAdd object:nil];
	
		/* kPGPPrefKeyServerSyncOnRevocation */
		if (IsPGPError(err = PGPSetPrefBoolean(prefRef, kPGPPrefKeyServerSyncOnRevocation, [self getSyncOnRevocation])))
		{
			PGPLog((@"Failed to set value for prefid %d (err=%d)", kPGPPrefKeyServerSyncOnRevocation, err));
		}
		else
			[center postNotificationName:kPGPclPrefNotification_SyncOnRevoke object:nil];
	
		/* kPGPPrefKeyServerSyncOnVerify */
		if (IsPGPError(err = PGPSetPrefBoolean(prefRef, kPGPPrefKeyServerSyncOnVerify, [self getSyncOnVerify])))
		{
			PGPLog((@"Failed to set value for prefid %d (err=%d)", kPGPPrefKeyServerSyncOnVerify, err));
		}
		else
			[center postNotificationName:kPGPclPrefNotification_SyncOnVerify object:nil];
	}
	else if (panelIdentifier && [panelIdentifier isEqualToString:kPGPkeysPrefsCAIdentifier])
	{
		/* kPGPPrefCAServerURL */
		if (IsPGPError(err = PGPSetPrefString(prefRef, kPGPPrefCAServerURL, [self getServerURL])))
		{
			PGPLog((@"Failed to set value for prefid %d (err=%d)", kPGPPrefCAServerURL, err));
		}
		else
			[center postNotificationName:kPGPclPrefNotification_ServerURL object:nil];
			
		/* kPGPPrefCARevocationServerURL */
		if (IsPGPError(err = PGPSetPrefString(prefRef, kPGPPrefCARevocationServerURL, [self getRevocationServerURL])))
		{
			PGPLog((@"Failed to set value for prefid %d (err=%d)", kPGPPrefCARevocationServerURL, err));
		}
		else
			[center postNotificationName:kPGPclPrefNotification_RevocationServerURL object:nil];
			
		/* kPGPPrefCAType */
		if (IsPGPError(err = PGPSetPrefNumber(prefRef, kPGPPrefCAType, [self getCAType])))
		{
			PGPLog((@"Failed to set value for prefid %d (err=%d)", kPGPPrefCAType, err));
		}
		else
			[center postNotificationName:kPGPclPrefNotification_CAType object:nil];

		/* kPGPPrefCARootKeyID && kPGPPrefCARootX509IASN */
		good = NO;
		
		if (PGPKeyDBObjRefIsValid(mX509AuthCert))
        {
			if(IsntPGPError(PGPclX509CertToExport([mController pgpContext], mX509AuthCert, &keyID, &iasn, &iasnSize)))
			{
				PGPSetPrefData(prefRef, kPGPPrefCARootX509IASN, iasnSize, iasn);
				PGPSetPrefData(prefRef, kPGPPrefCARootKeyID, sizeof(PGPKeyID), &keyID);
				
				PGPFreeData(iasn);
                good = TRUE;
			}
        }
		
        if (!good)
			PGPSetPrefData(prefRef, kPGPPrefCARootKeyID, 0, NULL);
		
		[center postNotificationName:kPGPclPrefNotification_RootCertificate object:nil];
	}
}

- (void)setPubKeyringLocation:(const char*)value enabled:(BOOL)enabled
{
	[mPubKeyringLocation setStringValue:@""];
	[mPubKeyringLocation setStringValue:[NSString stringWithCString:value]];
	[mPubKeyringLocation setEnabled:enabled];
}

- (void)setSecKeyringLocation:(const char*)value enabled:(BOOL)enabled
{
	[mSecKeyringLocation setStringValue:@""];
	[mSecKeyringLocation setStringValue:[NSString stringWithCString:value]];
	[mSecKeyringLocation setEnabled:enabled];
}

- (void)setSyncOnUnknownKeys:(PGPBoolean)value enabled:(BOOL)enabled
{
	[self setCheckbox:mEncryptToUnknown state:value enabled:enabled];
}

- (void)setSyncOnKeySign:(PGPBoolean)value enabled:(BOOL)enabled
{
	[self setCheckbox:mSigningKeys state:value enabled:enabled];
}

- (void)setSyncOnAdd:(PGPBoolean)value enabled:(BOOL)enabled
{
	[self setCheckbox:mAddingNamesPhotosRevokers state:value enabled:enabled];
}

- (void)setSyncOnRevocation:(PGPBoolean)value enabled:(BOOL)enabled
{
	[self setCheckbox:mRevocation state:value enabled:enabled];
}

- (void)setSyncOnVerify:(PGPBoolean)value enabled:(BOOL)enabled
{
	[self setCheckbox:mVerification state:value enabled:enabled];
}

- (void)setServerURL:(const char*)value enabled:(BOOL)enabled
{
	[mURL setStringValue:@""];
	[mURL setStringValue:[NSString stringWithCString:value]];
	[mURL setEnabled:enabled];
}

- (void)setRevocationServerURL:(const char*)value enabled:(BOOL)enabled
{
	[mRevocationURL setStringValue:@""];
	[mRevocationURL setStringValue:[NSString stringWithCString:value]];
	[mRevocationURL setEnabled:enabled];
}

- (void)setCAType:(PGPUInt32)value enabled:(BOOL)enabled
{
	[mCAType selectItemAtIndex:[mCAType indexOfItemWithTag:value]];
	[mCAType setEnabled:enabled];
}

- (void)setRootCertificate:(PGPKeyDBObjRef)cert enabled:(BOOL)enabled
{
	PGPError		err;
	char			cstr[256];
	PGPSize			size;
	
	[mSelectCert setEnabled:enabled];
	
	[mCertIcon setImage:nil];
	[mCertName setStringValue:@""];
	
	if (PGPKeyDBObjRefIsValid(cert))
	{
		mX509AuthCert = cert;
		
		err = PGPGetKeyDBObjDataProperty(cert, kPGPSigProperty_X509LongName, cstr, sizeof(cstr) - 1, &size);
		cstr[size] = '\0';
	
		if (IsPGPError(err))
		{
			PGPLog((@"Failed to retrieve X.509 cert long name (err=%d)", err));
		}
		else
			[mCertName setStringValue:[NSString stringWithCString:cstr]];
		
		[mCertIcon setImage:[[CPGPIcons shared] getKeyDBObjIcon:cert]];
		
		[mClearCert setEnabled:enabled];
	}
}

- (const char*)getPubKeyringLocation
{
	return [[mPubKeyringLocation stringValue] cString];
}

- (const char*)getSecKeyringLocation
{
	return [[mSecKeyringLocation stringValue] cString];
}

- (PGPBoolean)getSyncOnUnknownKeys
{
	return [self getCheckbox:mEncryptToUnknown];
}

- (PGPBoolean)getSyncOnKeySign
{
	return [self getCheckbox:mSigningKeys];
}

- (PGPBoolean)getSyncOnAdd
{
	return [self getCheckbox:mAddingNamesPhotosRevokers];
}

- (PGPBoolean)getSyncOnRevocation
{
	return [self getCheckbox:mRevocation];
}

- (PGPBoolean)getSyncOnVerify
{
	return [self getCheckbox:mVerification];
}

- (const char*)getServerURL
{
	return [[mURL stringValue] cString];
}

- (const char*)getRevocationServerURL
{
	return [[mRevocationURL stringValue] cString];
}

- (PGPUInt32)getCAType
{
	return [[mCAType selectedItem] tag];
}

- (void)setCheckbox:(NSButton*)checkBox state:(BOOL)state enabled:(BOOL)enabled
{
	if (checkBox)
	{
		if (state)
			[checkBox setState:NSOnState];
		else
			[checkBox setState:NSOffState];
		
		if (enabled)
			[checkBox setEnabled:YES];
		else
			[checkBox setEnabled:NO];
	}
}

- (PGPBoolean)getCheckbox:(NSButton*)checkBox
{
	if (!checkBox)
		return FALSE;
	
	return [checkBox state] == NSOnState ? TRUE : FALSE;
}

- (IBAction)changePubKeyring:(id)sender
{
	int					result;
	NSOpenPanel*		openPanel = [NSOpenPanel openPanel];
	NSString*			existingFile = [mPubKeyringLocation stringValue];
	NSString*			hfsFileType = NSFileTypeForHFSTypeCode(kPGPkeysPubringOSType);
	
	if ([[NSFileManager defaultManager] fileExistsAtPath:existingFile])
		result = [openPanel runModalForDirectory:( mLastDirectory ? mLastDirectory : 
							[existingFile stringByDeletingLastPathComponent] )
							file:[existingFile lastPathComponent]
							types:[NSArray arrayWithObjects:@"pkr", hfsFileType, nil]];
	else
		result = [openPanel runModalForDirectory:( mLastDirectory ? mLastDirectory : NSHomeDirectory() )
							file:nil
							types:[NSArray arrayWithObjects:@"pkr", hfsFileType, nil]];
	
	if (result == NSOKButton)
	{
		if( mLastDirectory )
			[mLastDirectory release];
		mLastDirectory = [openPanel directory];
		[mLastDirectory retain];
		[mPubKeyringLocation setStringValue:[openPanel filename]];
	}
}

- (IBAction)changeSecKeyring:(id)sender
{
	int					result;
	NSOpenPanel*		openPanel = [NSOpenPanel openPanel];
	NSString*			existingFile = [mSecKeyringLocation stringValue];
	NSString*			hfsFileType = NSFileTypeForHFSTypeCode(kPGPkeysSecringOSType);

	if ([[NSFileManager defaultManager] fileExistsAtPath:existingFile])
		result = [openPanel runModalForDirectory:( mLastDirectory ? mLastDirectory :
							[existingFile stringByDeletingLastPathComponent] )
							file:[existingFile lastPathComponent]
							types:[NSArray arrayWithObjects:@"skr", hfsFileType, nil]];
	else
		result = [openPanel runModalForDirectory:( mLastDirectory ? mLastDirectory : NSHomeDirectory() )
							file:nil
							types:[NSArray arrayWithObjects:@"skr", hfsFileType, nil]];
	
	if (result == NSOKButton)
	{
		if( mLastDirectory )
			[mLastDirectory release];
		mLastDirectory = [openPanel directory];
		[mLastDirectory retain];
		[mSecKeyringLocation setStringValue:[openPanel filename]];
	}
}

- (IBAction)newServer:(id)sender
{
	mServerEditMode = kPGPkeysServerEditModeNew;
	
	[self setupServerEditWindow:NULL];
	
	[NSApp beginSheet:mServerEditWindow modalForWindow:[NSApp keyWindow] modalDelegate:self didEndSelector:nil contextInfo:nil];
}

- (IBAction)removeServer:(id)sender
{
	NSBeginAlertSheet(LSPK(@"Confirm Keyserver Removal", @""),
					  LSPK(@"Cancel", @""), LSPK(@"Remove", @""),
					  nil, [NSApp keyWindow], self, nil,
					  @selector(remove:returnCode:contextInfo:),
					  nil, LSPK(@"Are you sure that you wish to remove the selected keyserver entry?", @""));
}

- (IBAction)editServer:(id)sender
{
	PGPKeyServerEntry*		ksEntry;
	PGPInt32				row = [mServersTableView selectedRow];
	NSData*					ksData;
	
	if ((row != -1) && mKeyServerEntries)
	{
		ksData = [mKeyServerEntries objectAtIndex:row];
		ksEntry = (PGPKeyServerEntry*) [ksData bytes];
		
		mServerEditMode = kPGPkeysServerEditModeEdit;
	
		[self setupServerEditWindow:ksEntry];
	
		[NSApp beginSheet:mServerEditWindow modalForWindow:[NSApp keyWindow] modalDelegate:self didEndSelector:nil contextInfo:nil];
	}
}

- (IBAction)setServerAsRoot:(id)sender
{
	PGPUInt32			row = [mServersTableView selectedRow];
	PGPKeyServerEntry*	entry;
	NSData*				ksData;
	NSEnumerator*		enumerator;
	
	if (row != -1)
	{
		enumerator = [mKeyServerEntries objectEnumerator];
		while (ksData = [enumerator nextObject])
		{
			entry = (PGPKeyServerEntry*) [ksData bytes];
			entry->flags &= ~kKeyServerIsRoot;
		}

		ksData = [mKeyServerEntries objectAtIndex:row];
		entry = (PGPKeyServerEntry*) [ksData bytes];
		entry->flags |= kKeyServerIsRoot;
		
		[mServersTableView reloadData];
	}
}

- (IBAction)moveServerUp:(id)sender
{
	PGPUInt32		row = [mServersTableView selectedRow];
	PGPUInt32		proposedRow = row - 1;
	NSData*			ksData;
	
	if ((row != -1) && (proposedRow != -1))
	{
		ksData = [[mKeyServerEntries objectAtIndex:row] retain];
		[mKeyServerEntries removeObjectAtIndex:row];
		
		[mKeyServerEntries insertObject:ksData atIndex:proposedRow];
		
		[ksData release];
		[mServersTableView reloadData];
		[mServersTableView selectRow:proposedRow byExtendingSelection:NO];
	}
}

- (IBAction)moveServerDown:(id)sender
{
	PGPUInt32		row = [mServersTableView selectedRow];
	PGPUInt32		proposedRow = row + 1;
	NSData*			ksData;
	
	if (row != -1)
	{
		ksData = [[mKeyServerEntries objectAtIndex:row] retain];
		[mKeyServerEntries removeObjectAtIndex:row];
		
		if (proposedRow > [mKeyServerEntries count])
			[mKeyServerEntries addObject:ksData];
		else
			[mKeyServerEntries insertObject:ksData atIndex:proposedRow];
		
		[ksData release];
		[mServersTableView reloadData];
		[mServersTableView selectRow:proposedRow byExtendingSelection:NO];
	}
}

- (IBAction)anyDomain:(id)sender
{
	[mDomain setStringValue:@""];
	[mDomain setEditable:NO];
	[mServerEditWindow makeFirstResponder:mDomain];
}

- (IBAction)specificDomain:(id)sender
{
	[mDomain setEditable:YES];
	[mServerEditWindow makeFirstResponder:mDomain];
}

- (IBAction)serverEditOK:(id)sender
{
	PGPError		err = kPGPError_NoErr;
	
	if ([self validateServerEditWindow])
	{	
		[NSApp endSheet:mServerEditWindow];
		[mServerEditWindow close];
		
		if (mServerEditMode == kPGPkeysServerEditModeNew)
		{
			PGPKeyServerEntry		newEntry;
			NSData*					newKSData;
			PGPInt32				row;
			
			memset(&newEntry, '\0', sizeof(PGPKeyServerEntry));
			
			if (IsntPGPError(err = [self saveServerEditEntry:&newEntry]))
			{
				newKSData = [NSData dataWithBytes:&newEntry length:sizeof(PGPKeyServerEntry)];
				[mKeyServerEntries addObject:newKSData];
				[mServersTableView reloadData];
				
				row = [mKeyServerEntries indexOfObject:newKSData];
				[mServersTableView selectRow:row byExtendingSelection:NO];
				[mServersTableView scrollRowToVisible:row];
			}
		}
		else if (mServerEditMode == kPGPkeysServerEditModeEdit)
		{
			PGPKeyServerEntry*		ksEntry;
			PGPUInt32				row = [mServersTableView selectedRow];
			NSData*					ksData;
			
			if (row != -1)
			{
				ksData = [mKeyServerEntries objectAtIndex:row];
				ksEntry = (PGPKeyServerEntry*) [ksData bytes];
				
				if (IsntPGPError(err = [self saveServerEditEntry:ksEntry]))
					[mServersTableView reloadData];
			}
		}
		
		if (IsPGPError(err))
		{
			NSBeginAlertSheet(LSPK(@"Keyserver Edit Failed", @""),
								LSPK(@"OK", @""),	/* defaultButton */
								nil,	/* alternateButton */
								nil,	/* otherButton */
								[NSApp keyWindow],	/* docWindow */
								self,	/* modalDelegate */
								nil,	/* willEndSelector */
								nil,	/* didEndSelector */
								nil,	/* contextInfo */
								LSPK(@"Failed to save the edited keyserver entry.", @""));
		}
		
		mServerEditMode = kPGPkeysServerEditModeNone;
	}
}

- (IBAction)serverEditCancel:(id)sender
{
	[NSApp endSheet:mServerEditWindow];
	[mServerEditWindow close];
	
	mServerEditMode = kPGPkeysServerEditModeNone;
}

- (IBAction)selectCert:(id)sender
{
	PGPclBeginCertSelectSheet(LSPK(@"Select root certificate for CA", @""),
		[NSApp keyWindow],
		[mController mainDB],
		kPGPclCertSelect_Default,
		self,
		@selector(certSelectDidEnd:selectedCert:));
}

- (IBAction)clearCert:(id)sender
{
	mX509AuthCert = kInvalidPGPKeyDBObjRef;
	
	[mCertIcon setImage:nil];
	[mCertName setStringValue:@""];
	
	[mClearCert setEnabled:NO];
}

- (void)remove:(NSWindow*)sheet returnCode:(int)returnCode contextInfo:(void*)contextInfo
{
	int			row = [mServersTableView selectedRow];
	
	/* If 'Remove' was pressed */
	if (returnCode == NSAlertAlternateReturn)
	{
		[mKeyServerEntries removeObjectAtIndex:row];
		[mServersTableView noteNumberOfRowsChanged];
		
		if ([mKeyServerEntries count] > 0)
		{
			if (row == [mKeyServerEntries count])
				--row;
			
			[mServersTableView selectRow:row byExtendingSelection:NO];
		}
		else
			[mServersTableView deselectAll:nil];
		
		[self tableViewSelectionDidChange:nil];
	}
}

- (void)certSelectDidEnd:(PGPError)resultCode selectedCert:(PGPKeyDBObjRef)cert
{
	if (IsntPGPError(resultCode) && PGPKeyDBObjRefIsValid(cert))
		[self setRootCertificate:cert enabled:YES];
}

- (BOOL)validateServerEditWindow
{
	if (![[mServerDNS stringValue] length])
	{
		NSRunAlertPanel(LSPK(@"Invalid Server Name", @""),
						LSPK(@"You must specify the server name in order \
for keyserver operations to work correctly.", @""),
						LSPK(@"OK", @""),
						nil,
						nil);
		
		[mServerEditWindow makeFirstResponder:mServerDNS];
		return NO;
	}
	
	if ([[mServerPort stringValue] length] && ![mServerPort intValue] && ([mServerPort intValue] < 65536))
	{
		NSRunAlertPanel(LSPK(@"Invalid Server Port", @""),
						LSPK(@"Please specify a port number greater than 0 and less than 65536. \
If you choose to leave the port field empty, the protocol-specific default port will be used.", @""),
						LSPK(@"OK", @""),
						nil,
						nil);
		[mServerEditWindow makeFirstResponder:mServerPort];
		return NO;
	}
	
	if (([mSpecificDomain state] == NSOnState) && ![[mDomain stringValue] length])
	{
		NSRunAlertPanel(LSPK(@"Invalid Domain", @""),
						LSPK(@"Please specify a domain for which the keyserver will service requests.", @""),
						LSPK(@"OK", @""),
						nil,
						nil);
		[mServerEditWindow makeFirstResponder:mDomain];
		return NO;
	}
	
	return YES;
}

- (void)updateServerButtons
{
	PGPKeyServerEntry*	ksEntry;
	int					row = [mServersTableView selectedRow];
	BOOL				newFlag;
	BOOL				removeFlag;
	BOOL				editFlag;
	BOOL				setRootFlag;
	BOOL				moveUpFlag;
	BOOL				moveDownFlag;
	
	if (row == -1)
	{
		newFlag = YES;
		removeFlag = NO;
		editFlag = NO;
		setRootFlag = NO;
		moveUpFlag = NO;
		moveDownFlag = NO;
	}
	else
	{
		ksEntry = (PGPKeyServerEntry*) [[mKeyServerEntries objectAtIndex:row] bytes];
		
		pgpAssert(ksEntry != NULL);
		
		newFlag = YES;
		removeFlag = YES;
		editFlag = YES;
		setRootFlag = YES;
		moveUpFlag = YES;
		moveDownFlag = YES;

		if (row == 0)
			moveUpFlag = NO;
		else if (row == ([mServersTableView numberOfRows] - 1))
			moveDownFlag = NO;
		
		if (ksEntry && IsKeyServerRoot(ksEntry->flags))
			setRootFlag = NO;
		
		if ([mServersTableView numberOfRows] == 1)
		{
			removeFlag = NO;
			moveDownFlag = NO;
		}
	}
	
	[mNewServer setEnabled:newFlag];
	[mRemoveServer setEnabled:removeFlag];
	[mEditServer setEnabled:editFlag];
	[mSetAsRoot setEnabled:setRootFlag];
	[mMoveUp setEnabled:moveUpFlag];
	[mMoveDown setEnabled:moveDownFlag];
}

- (void)setupServerEditWindow:(PGPKeyServerEntry*)entry
{
	PGPError		err = kPGPError_NoErr;
	PGPUInt32		tag;
	PGPKeyID		authKeyID;
	PGPKeyDBObjRef	authKey = kInvalidPGPKeyDBObjRef;
	
	if (!entry)
		tag = kPGPkeysServerType_PGPKeyserverLDAP;
	else
	{
		pgpAssert(entry->protocol != kPGPKeyServerProtocol_Invalid);
		pgpAssert(entry->type != kPGPKeyServerClass_Invalid);
		
		switch (entry->protocol)
		{
			case kPGPKeyServerProtocol_HTTP:
				switch (entry->type)
				{
					case kPGPKeyServerClass_PGP:
						tag = kPGPkeysServerType_PGPKeyserverHTTP;
						break;
					default:
						tag = kPGPkeysServerType_Invalid;
						break;
				}
				break;
			case kPGPKeyServerProtocol_LDAP:
				switch (entry->type)
				{
					case kPGPKeyServerClass_LDAPPGP:
					case kPGPKeyServerClass_PGP:
						tag = kPGPkeysServerType_PGPKeyserverLDAP;
						break;
					case kPGPKeyServerClass_LDAPX509:
						tag = kPGPkeysServerType_X509DirectoryLDAP;
						break;
					default:
						tag = kPGPkeysServerType_Invalid;
						break;
				}
				break;
			case kPGPKeyServerProtocol_LDAPS:
				switch (entry->type)
				{
					case kPGPKeyServerClass_LDAPPGP:
					case kPGPKeyServerClass_PGP:
						tag = kPGPkeysServerType_PGPKeyserverLDAPS;
						break;
					case kPGPKeyServerClass_LDAPX509:
						tag = kPGPkeysServerType_X509DirectoryLDAPS;
						break;
					default:
						tag = kPGPkeysServerType_Invalid;
						break;
				}
				break;
			default:
				tag = kPGPkeysServerType_Invalid;
				break;
		}
	}
	
	pgpAssert(tag != kPGPkeysServerType_Invalid);
	[mServerType selectItemAtIndex:[mServerType indexOfItemWithTag:tag]];
	
	[mServerDNS setStringValue:@""];
	
	if (entry)
		[mServerDNS setStringValue:[NSString stringWithCString:entry->serverDNS]];
	
	[mServerPort setStringValue:@""];

	if (entry && entry->serverPort)
		[mServerPort setIntValue:entry->serverPort];
	
	[mAuthKeyID setStringValue:LSPK(@"<Unknown>", @"")];

	if (entry && entry->authKeyIDString[0])
	{
		if (PGPKeyDBRefIsValid([mController mainDB]))
		{
			if (IsntPGPError(err = PGPNewKeyIDFromString(entry->authKeyIDString, entry->authAlg, &authKeyID)))
			{
				err = PGPFindKeyByKeyID([mController mainDB], &authKeyID, &authKey);
			}
		}
		if( PGPKeyDBObjRefIsValid( authKey ) )
			[mAuthKeyID setStringValue:PGPclGetPrimaryUserIDName(authKey)];
		else
			[mAuthKeyID setStringValue:LSPK(@"<Unknown>", @"")];
	}
	
	[mDomain setStringValue:@""];
	[mAnyDomain setState:NSOnState];
	[mSpecificDomain setState:NSOffState];

	if (entry && entry->domain[0])
	{
		[mDomain setStringValue:[NSString stringWithCString:entry->domain]];		
		[mAnyDomain setState:NSOffState];
		[mSpecificDomain setState:NSOnState];
		[self specificDomain:self];
	}
	
	[mListInSearchWindow setState:NSOnState];
	
	if (entry && !IsKeyServerListed(entry->flags))
		[mListInSearchWindow setState:NSOffState];
}

- (PGPError)saveServerEditEntry:(PGPKeyServerEntry*)entry
{
	if (!entry)
		return kPGPError_BadParams;
	
	switch ([[mServerType selectedItem] tag])
	{
		case kPGPkeysServerType_PGPKeyserverHTTP:
			entry->protocol = kPGPKeyServerProtocol_HTTP;
			entry->type = kPGPKeyServerClass_PGP;
			break;
		case kPGPkeysServerType_PGPKeyserverLDAP:
			entry->protocol = kPGPKeyServerProtocol_LDAP;
			entry->type = kPGPKeyServerClass_PGP;
			break;
		case kPGPkeysServerType_PGPKeyserverLDAPS:
			entry->protocol = kPGPKeyServerProtocol_LDAPS;
			entry->type = kPGPKeyServerClass_PGP;
			break;
		case kPGPkeysServerType_X509DirectoryLDAP:
			entry->protocol = kPGPKeyServerProtocol_LDAP;
			entry->type = kPGPKeyServerClass_LDAPX509;
			break;
		case kPGPkeysServerType_X509DirectoryLDAPS:
			entry->protocol = kPGPKeyServerProtocol_LDAPS;
			entry->type = kPGPKeyServerClass_LDAPX509;
			break;
		default:
			entry->protocol = kPGPKeyServerProtocol_Invalid;
			entry->type = kPGPKeyServerClass_Invalid;
			break;
	}
	
	[[mServerDNS stringValue] getCString:entry->serverDNS maxLength:kMaxServerNameLength];
	
	entry->serverPort = [mServerPort intValue];
	
	if ([mAnyDomain state] == NSOnState)
		entry->domain[0] = '\0';
	else
		[[mDomain stringValue] getCString:entry->domain maxLength:kMaxServerNameLength];
	
	if ([mListInSearchWindow state] == NSOnState)
		entry->flags |= kKeyServerListed;
	else
		entry->flags &= ~kKeyServerListed;
	
	return kPGPError_NoErr;
}

- (void)tableViewSelectionDidChange:(NSNotification*)notification
{
	[self updateServerButtons];
}

- (void)tableView:(NSTableView*)table willDisplayCell:(id)cell forTableColumn:(NSTableColumn*)column row:(int)row
{
	NSData*					ksData;
	PGPKeyServerEntry*		ksEntry;
	NSMutableDictionary*	attributes;
	NSAttributedString*		attrString;
	
	pgpAssert(mKeyServerEntries != nil);
	
	if (ksData = [mKeyServerEntries objectAtIndex:row])
	{
		ksEntry = (PGPKeyServerEntry*) [ksData bytes];
		
		if ([[column identifier] isEqualToString:kPGPkeysPrefsServerColumnIdentifier])
		{
			attributes = [[NSMutableDictionary alloc] init];
			
			if (IsKeyServerRoot(ksEntry->flags))
				[attributes setObject:[NSFont boldSystemFontOfSize:11] forKey:NSFontAttributeName];
			else
				[attributes setObject:[NSFont systemFontOfSize:11] forKey:NSFontAttributeName];
				
			attrString = [[NSAttributedString alloc] initWithString:[cell stringValue] attributes:attributes];
			
			[cell setAttributedStringValue:attrString];
			
			[attributes release];
			[attrString release];
		}
		else if ([[column identifier] isEqualToString:kPGPkeysPrefsDomainColumnIdentifier])
		{
			if (ksEntry->domain[0])
			{
				[cell setFont:[NSFont systemFontOfSize:11]];
				[cell setStringValue:[NSString stringWithCString:ksEntry->domain]];
			}
			else
				[cell setImage:mAnyDomainIcon];
		}
		else if ([[column identifier] isEqualToString:kPGPkeysPrefsListedColumnIdentifier])
		{
			if (IsKeyServerListed(ksEntry->flags))
				[cell setImage:[[[CPGPIcons shared] images] objectAtIndex:kPGPclImage_DotGreen]];
			else
				[cell setImage:[[[CPGPIcons shared] images] objectAtIndex:kPGPclImage_DotDimmed]];
		}
	}
}

- (int)numberOfRowsInTableView:(NSTableView*)tableView
{
	if (mKeyServerEntries)
		return [mKeyServerEntries count];
		
	return 0;
}

- (id)tableView:(NSTableView*)tableView objectValueForTableColumn:(NSTableColumn*)column row:(int)row
{
	NSData*				ksData;
	PGPKeyServerEntry*	ksEntry;
	NSMutableString*	serverDNS;
	id					value = nil;

	if (!mKeyServerEntries)
		return nil;

	NSParameterAssert((row >= 0) && (row < [mKeyServerEntries count]));
	if (ksData = [mKeyServerEntries objectAtIndex:row])
	{
		ksEntry = (PGPKeyServerEntry*) [ksData bytes];
		
		if ([[column identifier] isEqualToString:kPGPkeysPrefsServerColumnIdentifier])
		{
			serverDNS = [[[NSMutableString alloc] init] autorelease];
			
			switch (ksEntry->protocol)
			{
				case kPGPKeyServerProtocol_HTTP:
					[serverDNS appendString:@"http://"];
					break;
				case kPGPKeyServerProtocol_HTTPS:
					[serverDNS appendString:@"https://"];
					break;
				case kPGPKeyServerProtocol_LDAP:
					[serverDNS appendString:@"ldap://"];
					break;
				case kPGPKeyServerProtocol_LDAPS:
					[serverDNS appendString:@"ldaps://"];
					break;
				default:
					break;
			}
			
			[serverDNS appendFormat:@"%s", ksEntry->serverDNS];
			
			if (ksEntry->serverPort)
				[serverDNS appendFormat:@":%d", ksEntry->serverPort];
			
			value = serverDNS;
		}
	}
	
	return value;
}

- (void)dealloc
{		
	if (mIdentifiers)
		[mIdentifiers release];
		
	if (mKeyServerEntries)
		[mKeyServerEntries release];

	if (mIntFormatter)
		[mIntFormatter release];
		
	[super dealloc];
}

@end
