/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation
    All rights reserved.

    $Id: CPGPmailController.h,v 1.49 2002/10/16 04:44:09 jeffc Exp $
____________________________________________________________________________*/
#ifndef Included_CPGPmailController_h	/* [ */
#define Included_CPGPmailController_h

#import <PGP/PGP.h>
#import <Cocoa/Cocoa.h>

#import "PGPBundle.h"
#import "PGPBundleMail.h"
#import "PGPController.h"

#import "PGPmailCommon.h"
#import "CPGPmailButton.h"
#import <PGPclient/CImageButton.h>

#define kPGPmailNib						@"CPGPmail"
#define kPGPmailToolbarItemImage		@"pgpmail"
#define kPGPmailMailMenuTitle			LSPM(@"Mail", @"Mail menu title")

@class NSDictionary, NSToolbarItem, NSMenuItem;
@class CPGPmailPreferences, CPGPVerificationDataSource, CPGPmailButton;

@interface CPGPmailController : NSObject <PGPBundle,PGPBundleMail>
{
    id <PGPController>			mAppController;
    
    CPGPmailPreferences			*mPGPMailPreferences;
    PGPContextRef				mPGPContext;
    PGPtlsContextRef			mtlsContext;
    PGPPrefRef					mPrefRef;
	NSResponder*				mLastResponder;
    
	IBOutlet NSWindow*			mWindow;
	IBOutlet NSPanel*			mSmWindow;
	IBOutlet NSWindow*			mVerificationWindow;
	
    IBOutlet NSDateFormatter	*mDateFormatter;
    
    IBOutlet CPGPmailButton		*mDecryptVerifyButton;
    IBOutlet CPGPmailButton		*mEncryptButton;
    IBOutlet CPGPmailButton		*mEncryptSignButton;
    IBOutlet CPGPmailButton		*mSignButton;
    IBOutlet CPGPmailButton		*mWipeButton;
	
	IBOutlet CPGPmailButton		*mSmDecryptVerifyButton;
	IBOutlet CPGPmailButton		*mSmEncryptButton;
	IBOutlet CPGPmailButton		*mSmEncryptSignButton;
	IBOutlet CPGPmailButton		*mSmSignButton;
	IBOutlet CPGPmailButton		*mSmWipeButton;
	
	IBOutlet NSMenu				*mMailDockMenu;
    IBOutlet NSMenu				*mMailMenu;

    IBOutlet NSView				*mMailView;
    
    IBOutlet NSTableView 		*mVerificationTable;
    IBOutlet CPGPVerificationDataSource	*mVerificationDataSource;
    
    NSString					*mDefaultOpenDirectory;
    
    BOOL						mInited;
    BOOL						mLoadedNib;
    BOOL						mInitedViewer;
	BOOL						mMenusAdded;
	BOOL						mUseMiniWindow;
	BOOL						mCheckEntourage;
}

+ (NSString*)identifier;

- (id)init;
- (id)initWithNib:(NSString*)nibFile;
- (id)initWithController:(id)controller;

- (void)enableMailViewer;
- (void)checkEntourageInstall;
- (void)removeEntourageScripts;

- (id)prefsController;
- (NSWindow*)window;
- (NSImage*)menuImage;
- (NSMenu*)dockMenu;
- (NSMenuItem*)newMenuItem;
- (void)activate:(BOOL)passive;
- (void)deactivate;
- (void)addMenus;
- (void)removeMenus;
- (void)handleDocument:(NSString*)docFile ofType:(NSString*)docType;
- (id)controller;
- (void)setController:(id)controller;
- (PGPContextRef)pgpContext;
- (PGPtlsContextRef)tlsContext;
- (PGPPrefRef)pgpPrefRef;
- (PGPKeyDBRef)pgpKeyDB;
- (void)newVerificationResult:(id)verification;

- (void)setupKeyViewLoop;
- (BOOL)keyViewIsValid:(NSResponder*)keyView;
- (BOOL)releaseIfReadyToShutdown;
- (BOOL)jobsPending;

- (void)keyDBChanged;
- (void)passphraseCacheChanged;

- (IBAction)fileDecryptVerify:(id)sender;
- (IBAction)clipboardDecryptVerify:(id)sender;
- (NSString *)decryptVerifyText:(NSString *)data error:(NSString **)error showDialog:(BOOL)showDialog;
- (void)decryptVerifyFiles:(NSArray *)files;
- (void)decryptVerifySelectedFiles:(NSArray*)files;

- (IBAction)fileEncrypt:(id)sender;
- (IBAction)clipboardEncrypt:(id)sender;
- (NSString *)encryptText:(NSString *)data toRecipients:(NSArray *)toRecipients error:(NSString **)error;
- (void)encryptFiles:(NSArray *)files;
- (void)encryptSelectedFiles:(NSArray*)files;

- (IBAction)fileEncryptSign:(id)sender;
- (IBAction)clipboardEncryptSign:(id)sender;
- (NSString *)signEncryptText:(NSString *)data toRecipients:(NSArray *)toRecipients error:(NSString **)error;
- (void)encryptSignFiles:(NSArray *)files;
- (void)encryptSignSelectedFiles:(NSArray*)files;

- (IBAction)fileSign:(id)sender;
- (IBAction)clipboardSign:(id)sender;
- (NSString *)signText:(NSString *)data error:(NSString **)error;
- (void)signFiles:(NSArray *)files;
- (void)signSelectedFiles:(NSArray*)files;

- (IBAction)fileWipe:(id)sender;
- (void)wipeFiles:(NSArray *)files;
- (void)wipeSelectedFiles:(NSArray*)files;

- (IBAction)verificationResults:(id)sender;

- (void)transformClipboard:(PGPmailOperation)inOperation;
- (NSString *)transformBuffer:(NSString *)inBuffer toRecipients:(NSArray *)toRecipients operation:(PGPmailOperation)inOperation;

- (void)selectFilesToTransform:(PGPmailOperation)inOperation;
- (void)transformFiles:(NSArray *)inFiles operation:(PGPmailOperation)inOperation lock:(NSLock*)lock;

- (void)setDefaultOpenDirectory:(NSString *)directory;
- (NSString *)defaultOpenDirectory;

- (void)setUseMiniWindow:(BOOL)flag;
- (BOOL)useMiniWindow;

- (void)transformSheetDidEnd:(NSOpenPanel*)sheet returnCode:(int)code contextInfo:(void*)info;

/* NSWindow delegate methods */
- (void)windowDidBecomeKey:(NSNotification*)notification;
- (void)windowDidResignKey:(NSNotification*)notification;
- (void)windowDidUpdate:(NSNotification*)notification;
- (void)windowWillBeginSheet:(NSNotification*)notification;
- (void)windowDidEndSheet:(NSNotification*)notification;
- (BOOL)windowShouldZoom:(NSWindow*)sender toFrame:(NSRect)frame;

- (void)dealloc;

@end

#endif /* ] Included_CPGPmailController_h */


/*__Editor_settings____

	Local Variables:
	tab-width: 4
	End:
	vi: ts=4 sw=4
	vim: si
_____________________*/