/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation
    All rights reserved.

    $Id: CPGPmailController.m,v 1.92 2002/11/15 23:43:32 jeffc Exp $
____________________________________________________________________________*/

#import "CPGPmailController.h"
#import "CPGPmailPreferences.h"

#import <PGP/PGP.h>
#import <PGPui/CBrowserCell.h>
#import <PGPclient/pgpClientLibOSX.h>
#import <PGPclient/pgpClientLibCocoa.h>

#include <sys/types.h>
#include <sys/stat.h>

#import "PGPmailCommon.h"

#import "CPGPVerification.h"
#import "CPGPVerificationDataSource.h"
#import "CPGPmailEncodeDecodeFile.h"
#import "CPGPmailEncodeDecodeBuffer.h"
#import "CPGPmailWipeFile.h"

#define kPGPmailBadParams				LSPM(@"PGPmail has not been initalized properly", @"Bad Params")
#define kPGPmailMissingKeys				LSPM(@"There are no keys in the Key Database, you cannot sign.", @"Missing Keys")

#define kPGPmailEntourageScriptsDir		@"~/Documents/Microsoft User Data/Entourage Script Menu Items"
#define kPGPmailDecryptVerifyScript 	@"Decrypt & Verify\\mod"
#define kPGPmailEncryptSignScript		@"Encrypt & Sign\\moc"
#define kPGPmailEncryptScript			@"Encrypt\\moe"
#define kPGPmailSignScript				@"Sign\\mos"
#define kPGPmailBundleName				@"PGPmail.bundle"
#define kMailAppPrefsDomain				@"com.apple.Mail"
#define kMailAppEnableBundlesKey		@"EnableBundles"
#define kPGPmailSaveFrameName			@"PGPmail Main Window"

@implementation CPGPmailController

+ (NSString*)identifier
{
    return( kPGPmailBundleName );
}

- (id)init;
{
    if (!mInited)
    {
        [super init];
        mInited = YES;
    }
	
	if (!mLoadedNib)
		[self initWithNib:kPGPmailNib];
	
	if (!mPGPContext && mAppController)
		mPGPContext = [mAppController pgpContext];
	
	if (!mtlsContext && mAppController)
		mtlsContext = [mAppController tlsContext];
	
	if (!mPrefRef)
		mPrefRef = [self pgpPrefRef];
	
	if (!mPGPMailPreferences)
		mPGPMailPreferences = [[CPGPmailPreferences alloc] initWithController:self];
	
	if (PGPclIsAuthorized() && PGPclPGPmailEnabled())
	{
		if (!mInitedViewer)
			[self enableMailViewer];
		
		if (!mCheckEntourage)
			[self checkEntourageInstall];
	}
	else
		[self removeEntourageScripts];
	
	return self;
}

- (id)initWithNib:(NSString*)nibFile
{
	CBrowserCell*		bCell = nil;
	NSTextFieldCell*	tCell = nil;
    NSImageCell*		iCell = nil;
	NSImage*			image;
	NSFont*				smallFont = [NSFont systemFontOfSize:[NSFont smallSystemFontSize]];
	NSBundle*			bundle = [NSBundle bundleForClass:[self class]];

	[NSBundle loadNibNamed:nibFile owner:self];
	
    [mMailMenu setTitle:kPGPmailMailMenuTitle];

	bCell = [[CBrowserCell alloc] init];
	[bCell setLeaf:YES];
	[bCell setWraps:NO];
	[bCell setScrollable:NO];
	[bCell setFont:smallFont];
	[[[mVerificationTable tableColumns] objectAtIndex:0] setDataCell:bCell];
 	[bCell release];
    bCell = nil;
    
    tCell = [[NSTextFieldCell alloc] init];
	[tCell setFont:smallFont];
	[tCell setWraps:NO];
	[[[mVerificationTable tableColumns] objectAtIndex:1] setDataCell:tCell];
    [tCell release];
    tCell = nil;
    
    iCell = [[NSImageCell alloc] init];
    [[[mVerificationTable tableColumns] objectAtIndex:2] setDataCell:iCell];
    [iCell release];
    iCell = nil;
    
    tCell = [[NSTextFieldCell alloc] init];
	[tCell setFont:smallFont];
	[tCell setWraps:NO];
    [tCell setFormatter:mDateFormatter];
	[[[mVerificationTable tableColumns] objectAtIndex:3] setDataCell:tCell];
	[[[mVerificationTable tableColumns] objectAtIndex:4] setDataCell:tCell];
	[tCell release];
    tCell = nil;
    
	[mVerificationTable setDelegate:mVerificationDataSource];
	[mVerificationTable setDataSource:mVerificationDataSource];
	
	[mEncryptButton registerForDraggedTypes:[NSArray arrayWithObject:NSFilenamesPboardType]];
	[mEncryptButton setDragSelector:@selector(encryptFiles:)];
	[mEncryptButton setImage:[[[NSImage alloc] initWithContentsOfFile:[bundle pathForImageResource:@"encrypt.icns"]] autorelease]];
	[mSignButton registerForDraggedTypes:[NSArray arrayWithObject:NSFilenamesPboardType]];
	[mSignButton setDragSelector:@selector(signFiles:)];
	[mSignButton setImage:[[[NSImage alloc] initWithContentsOfFile:[bundle pathForImageResource:@"sign.icns"]] autorelease]];
	[mEncryptSignButton registerForDraggedTypes:[NSArray arrayWithObject:NSFilenamesPboardType]];
	[mEncryptSignButton setDragSelector:@selector(encryptSignFiles:)];
	[mEncryptSignButton setImage:[[[NSImage alloc] initWithContentsOfFile:[bundle pathForImageResource:@"signandencrypt.icns"]] autorelease]];
    [mDecryptVerifyButton registerForDraggedTypes:[NSArray arrayWithObject:NSFilenamesPboardType]];
	[mDecryptVerifyButton setDragSelector:@selector(decryptVerifyFiles:)];
	[mDecryptVerifyButton setSupportedFileExtensions:[NSArray arrayWithObjects:@"sig", @"pgp", @"gpg", @"asc", @"shf", nil]];
	[mDecryptVerifyButton setSupportedFileTypes:[NSArray arrayWithObjects:@"pgDS", @"pgEF", @"pgSK", nil]];
	[mDecryptVerifyButton setImage:[[[NSImage alloc] initWithContentsOfFile:[bundle pathForImageResource:@"decrypt.icns"]] autorelease]];
	[mWipeButton registerForDraggedTypes:[NSArray arrayWithObject:NSFilenamesPboardType]];
	[mWipeButton setDragSelector:@selector(wipeFiles:)];
	[mWipeButton setImage:[[[NSImage alloc] initWithContentsOfFile:[bundle pathForImageResource:@"wipe.icns"]] autorelease]];
	
	image = [[[NSImage alloc] initWithContentsOfFile:[bundle pathForImageResource:@"encrypt.icns"]] autorelease];
	[image setSize:NSMakeSize(16, 16)];
	[mSmEncryptButton registerForDraggedTypes:[NSArray arrayWithObject:NSFilenamesPboardType]];
	[mSmEncryptButton setDragSelector:@selector(encryptFiles:)];
	[mSmEncryptButton setImage:image];
	
	image = [[[NSImage alloc] initWithContentsOfFile:[bundle pathForImageResource:@"sign.icns"]] autorelease];
	[image setSize:NSMakeSize(16, 16)];
	[mSmSignButton registerForDraggedTypes:[NSArray arrayWithObject:NSFilenamesPboardType]];
	[mSmSignButton setDragSelector:@selector(signFiles:)];
	[mSmSignButton setImage:image];
	
	image = [[[NSImage alloc] initWithContentsOfFile:[bundle pathForImageResource:@"signandencrypt.icns"]] autorelease];
	[image setSize:NSMakeSize(16, 16)];
	[mSmEncryptSignButton registerForDraggedTypes:[NSArray arrayWithObject:NSFilenamesPboardType]];
	[mSmEncryptSignButton setDragSelector:@selector(encryptSignFiles:)];
	[mSmEncryptSignButton setImage:image];
	
	image = [[[NSImage alloc] initWithContentsOfFile:[bundle pathForImageResource:@"decrypt.icns"]] autorelease];
	[image setSize:NSMakeSize(16, 16)];
	[mSmDecryptVerifyButton registerForDraggedTypes:[NSArray arrayWithObject:NSFilenamesPboardType]];
	[mSmDecryptVerifyButton setDragSelector:@selector(decryptVerifyFiles:)];
	[mSmDecryptVerifyButton setSupportedFileExtensions:[NSArray arrayWithObjects:@"sig", @"pgp", @"gpg", @"asc", @"shf", nil]];
	[mSmDecryptVerifyButton setSupportedFileTypes:[NSArray arrayWithObjects:@"pgDS", @"pgEF", @"pgSK", nil]];
	[mSmDecryptVerifyButton setImage:image];
	
	image = [[[NSImage alloc] initWithContentsOfFile:[bundle pathForImageResource:@"wipe.icns"]] autorelease];
	[image setSize:NSMakeSize(16, 16)];
	[mSmWipeButton registerForDraggedTypes:[NSArray arrayWithObject:NSFilenamesPboardType]];
	[mSmWipeButton setDragSelector:@selector(wipeFiles:)];
	[mSmWipeButton setImage:image];
    
	[mWindow setDelegate:self];
	[mSmWindow setDelegate:self];
	[mSmWindow setLevel:[mWindow level]];
	[mSmWindow setHidesOnDeactivate:NO];
	
	if (mUseMiniWindow)
	{
		[mSmWindow setFrameUsingName:kPGPmailSaveFrameName];
		mLastResponder = mSmEncryptButton;
	}
	else
	{
		[mWindow setFrameUsingName:kPGPmailSaveFrameName];
		mLastResponder = mEncryptButton;
	}
	
	mLoadedNib = YES;
	
    return( [self init] );
}

- (id)initWithController:(id)controller
{
    mAppController = controller;
    return( [self init] );
}

- (void)awakeFromNib
{
    NSUserDefaults		*defaults = [NSUserDefaults standardUserDefaults];
    NSDictionary		*appDefaults = [defaults persistentDomainForName:kPGPPrefsDomain];
	NSString*			string;
	
	string = [appDefaults objectForKey:kPGPmailDefaultUseMiniWindow];
	
	if (string)
	{
		if ([string isEqualToString:@"Yes"])
			[self setUseMiniWindow:YES];
		else
			[self setUseMiniWindow:NO];
	}
	
    [self setDefaultOpenDirectory:[appDefaults objectForKey:kPGPmailDefaultOpenDirectoryKey]];
    
    if( nil == [self defaultOpenDirectory] )
    {
        [self setDefaultOpenDirectory:NSHomeDirectory()];
    }
}

- (void)enableMailViewer
{
    NSUserDefaults*			defaults = [NSUserDefaults standardUserDefaults];
    NSMutableDictionary*	appDefaults = [[defaults persistentDomainForName:kMailAppPrefsDomain] mutableCopy];
    
	[appDefaults setObject:@"Yes" forKey:kMailAppEnableBundlesKey];
	[defaults removePersistentDomainForName:kMailAppPrefsDomain];
	[defaults setPersistentDomain:appDefaults forName:kMailAppPrefsDomain];
	[defaults synchronize];
	[appDefaults release];
	
	mInitedViewer = YES;
}

- (void)checkEntourageInstall
{
	NSFileManager*		manager = [NSFileManager defaultManager];
	NSString*			scriptsDir = [kPGPmailEntourageScriptsDir stringByExpandingTildeInPath];
	NSString*			path;
	NSString*			bundlePath = [[NSBundle bundleForClass:[self class]] bundlePath];
	BOOL				isDir;
	NSEnumerator*		enumerator;
	NSArray*			scripts = [NSArray arrayWithObjects:kPGPmailDecryptVerifyScript,
															kPGPmailEncryptSignScript,
															kPGPmailEncryptScript,
															kPGPmailSignScript,
															nil];
	
	/* Check if the Entourage scripts directory is present. If not, we assume that Entourage isn't installed. */
	if (!([manager fileExistsAtPath:scriptsDir isDirectory:&isDir] && isDir))
		return;	/* Entourage not installed */
	
	path = [scriptsDir stringByAppendingPathComponent:@"PGP"];
	
	if (!([manager fileExistsAtPath:path isDirectory:&isDir] && isDir))
	{
		/* Remove any prior PGP script install */
		[manager removeFileAtPath:path handler:nil];

		if (mkdir([path UTF8String], 0775) == -1)
		{
			PGPLog((@"Failed to create %@", path));
			return;
		}
	}
	
	enumerator = [scripts objectEnumerator];
	
	/* Copy the scripts */
	while (path = [enumerator nextObject])
	{
		NSString*	scriptPath = [NSString pathWithComponents:[NSArray arrayWithObjects:bundlePath, @"Contents/Resources/Entourage", path, nil]];
		NSString*	scriptDest = [NSString pathWithComponents:[NSArray arrayWithObjects:scriptsDir, @"PGP", path, nil]];
		
		PGPLog((@"scriptPath: %@", scriptPath));
		
		/* Only delete the old scripts if we have the scripts internally */
		if ([manager fileExistsAtPath:scriptPath])
			[manager removeFileAtPath:scriptDest handler:nil];
		
		if (![manager copyPath:scriptPath toPath:scriptDest handler:nil])
			PGPLog((@"PGPmail: Failed to copy entourage script %@", scriptDest));
	}
}

- (void)removeEntourageScripts
{
	NSFileManager*		manager = [NSFileManager defaultManager];
	NSString*			scriptsDir = [kPGPmailEntourageScriptsDir stringByExpandingTildeInPath];
	NSString*			path;
	BOOL				isDir;
	
	/* Check if the Entourage scripts directory is present. If not, we assume that Entourage isn't installed. */
	if (!([manager fileExistsAtPath:scriptsDir isDirectory:&isDir] && isDir))
		return;	/* Entourage not installed */
	
	path = [scriptsDir stringByAppendingPathComponent:@"PGP"];
	
	if (!([manager fileExistsAtPath:path isDirectory:&isDir] && isDir))
	{
		/* Remove any prior PGP script install */
		[manager removeFileAtPath:path handler:nil];
	}
}

- (id)prefsController;
{
    return( mPGPMailPreferences );
}

- (NSWindow*)window
{
	return mUseMiniWindow ? (NSWindow*) mSmWindow : mWindow;
}

- (NSImage*)menuImage
{
	NSImage*	image;
	
	image = [[[NSImage alloc] initWithContentsOfFile:[[NSBundle bundleForClass:[self class]] pathForImageResource:kPGPmailToolbarItemImage]] autorelease];
	[image setScalesWhenResized:YES];
	[image setSize:NSMakeSize(16, 16)];
	
	return( image );
}

- (NSMenu*)dockMenu
{
	return mMailDockMenu;
}

- (NSMenuItem*)newMenuItem
{
	return nil;
}

- (void)activate:(BOOL)passive
{
	if (!passive)
	{
		if (mUseMiniWindow)
			[mSmWindow makeKeyAndOrderFront:nil];
		else
			[mWindow makeKeyAndOrderFront:nil];
	}
}

- (void)deactivate
{
	if (mUseMiniWindow)
		mLastResponder = [mSmWindow firstResponder];
	else
		mLastResponder = [mWindow firstResponder];
}

- (void)addMenus
{
	[mAppController addMenu:mMailMenu withIdentifier:[[self class] identifier]
		atIndex:[[mAppController menuItems] count] - 2];
}

- (void)removeMenus
{
}

- (void)handleDocument:(NSString*)docFile ofType:(NSString*)docType
{
    NSMutableArray*		files = [NSMutableArray array];
    
    [files addObject:docFile];
	[self transformFiles:files operation:kPGPmailDecryptVerifyOperation lock:nil];
}

- (id)controller
{
    return( mAppController );
}

- (void)setController:(id)controller
{
    if( mAppController )
        mAppController = controller;
}

- (PGPContextRef)pgpContext
{
    return( mPGPContext );
}

- (PGPtlsContextRef)tlsContext
{
    return( mtlsContext );
}

- (PGPPrefRef)pgpPrefRef
{
    if( mAppController )
        return( [mAppController pgpPrefRef] );
    
    return( NULL );
}

- (PGPKeyDBRef)pgpKeyDB
{
    return( [mAppController mainKeyDB] );
}

- (void)newVerificationResult:(id)verification
{
    [mVerificationDataSource addRow:verification];
    [mVerificationTable noteNumberOfRowsChanged];
	
	[self verificationResults:nil];
}

- (void)setupKeyViewLoop
{
	[mEncryptButton setNextKeyView:mSignButton];
	[mSignButton setNextKeyView:mEncryptSignButton];
	[mEncryptSignButton setNextKeyView:mDecryptVerifyButton];
	[mDecryptVerifyButton setNextKeyView:mWipeButton];
	[mWipeButton setNextKeyView:mVerificationTable];
	[mVerificationTable setNextKeyView:mEncryptButton];
}

- (BOOL)keyViewIsValid:(NSResponder*)keyView
{
	return ((keyView == mEncryptButton) ||
			(keyView == mSignButton) ||
			(keyView == mEncryptSignButton) ||
			(keyView == mDecryptVerifyButton) ||
			(keyView == mWipeButton) ||
			(keyView == mVerificationTable));
}

- (BOOL)releaseIfReadyToShutdown
{
	NSUserDefaults*			defaults = [NSUserDefaults standardUserDefaults];
	NSMutableDictionary*	dict = [[defaults persistentDomainForName:kPGPPrefsDomain] mutableCopy];
	
	if (mUseMiniWindow)
		[mSmWindow saveFrameUsingName:kPGPmailSaveFrameName];
	else
		[mWindow saveFrameUsingName:kPGPmailSaveFrameName];
	
	[dict setObject:(mUseMiniWindow ? @"Yes" : @"No") forKey:kPGPmailDefaultUseMiniWindow];
	
    if( [self defaultOpenDirectory] )
	{
        [dict setObject:[self defaultOpenDirectory] forKey:kPGPmailDefaultOpenDirectoryKey];
						
        [self setDefaultOpenDirectory:nil];
	}
	
	[defaults removePersistentDomainForName:kPGPPrefsDomain];
	[defaults setPersistentDomain:dict forName:kPGPPrefsDomain];
	[defaults synchronize];
	[dict release];
	
    PGPLog(( @"Deallocated PGPmail" ));	
    return( YES );
}

- (BOOL)jobsPending
{
	return [[CPGPmailProgress sharedInstance] jobsPending];
}

- (void)keyDBChanged
{
	PGPLog((@"PGPmail: keyDBChanged"));
}

- (void)passphraseCacheChanged
{
	PGPLog((@"PGPmail: passphraseCacheChanged"));
}

- (IBAction)fileDecryptVerify:(id)sender
{
	[NSApp activateIgnoringOtherApps:YES];
    [self selectFilesToTransform:kPGPmailDecryptVerifyOperation];
}

- (IBAction)clipboardDecryptVerify:(id)sender
{
	[NSThread detachNewThreadSelector:@selector(clipboardDecryptVerify) toTarget:self withObject:nil];
}

- (void)clipboardDecryptVerify
{
	ProcessSerialNumber		frontPSN;
	OSErr					osError;
	NSAutoreleasePool*		pool = [[NSAutoreleasePool alloc] init];
	
	osError = GetFrontProcess(&frontPSN);
	
	[NSApp activateIgnoringOtherApps:YES];
    
	[self transformClipboard:kPGPmailDecryptVerifyOperation];
    
    osError = SetFrontProcess(&frontPSN);
	
	[pool release];
}

- (NSString *)decryptVerifyText:(NSString *)data error:(NSString **)error showDialog:(BOOL)showDialog
{
	NSString*				decryptedText = nil;
	CPGPmailProgress*		progress = [CPGPmailProgress sharedInstance];
	CPGPmailProgressJob*	job = nil;
	
	job = [[CPGPmailProgressJob alloc] initWithFile:LSPM(@"Selected Text", @"") operation:kPGPmailDecryptVerifyOperation selector:nil target:nil];
	[progress startProgressForJob:job];
	[progress updateProgressForJob:job];
	
	decryptedText = [self transformBuffer:data toRecipients:nil operation:kPGPmailDecryptVerifyOperation];
	
	if (job)
	{
		[progress finishedJob:job];
		[job release];
	}
	
	if (decryptedText && showDialog)
		PGPclOutputDialog(mPGPContext, mPrefRef, [decryptedText UTF8String]);
	
    return decryptedText;
}

- (IBAction)fileEncrypt:(id)sender
{
	[NSApp activateIgnoringOtherApps:YES];
    [self selectFilesToTransform:kPGPmailEncryptOperation];
}

- (IBAction)clipboardEncrypt:(id)sender
{
	[NSThread detachNewThreadSelector:@selector(clipboardEncrypt) toTarget:self withObject:nil];
}

- (void)clipboardEncrypt
{
	ProcessSerialNumber		frontPSN;
	OSErr					osError;
	NSAutoreleasePool*		pool = [[NSAutoreleasePool alloc] init];
	
	osError = GetFrontProcess(&frontPSN);
	
	[NSApp activateIgnoringOtherApps:YES];
    
	[self transformClipboard:kPGPmailEncryptOperation];
    
    osError = SetFrontProcess(&frontPSN);
	
	[pool release];
}

- (NSString *)encryptText:(NSString *)data toRecipients:(NSArray *)toRecipients error:(NSString **)error
{
	NSString*				encryptedText = nil;
	CPGPmailProgress*		progress = [CPGPmailProgress sharedInstance];
	CPGPmailProgressJob*	job = nil;
	
	job = [[CPGPmailProgressJob alloc] initWithFile:LSPM(@"Selected Text", @"") operation:kPGPmailEncryptOperation selector:nil target:nil];
	[progress startProgressForJob:job];
	[progress updateProgressForJob:job];
	
	encryptedText = [self transformBuffer:data toRecipients:toRecipients operation:kPGPmailEncryptOperation];
	
	if (job)
	{
		[progress finishedJob:job];
		[job release];
	}
	
    return encryptedText;
}

- (IBAction)fileEncryptSign:(id)sender
{
	[NSApp activateIgnoringOtherApps:YES];
    [self selectFilesToTransform:kPGPmailEncryptSignOperation];
}

- (IBAction)clipboardEncryptSign:(id)sender
{
	[NSThread detachNewThreadSelector:@selector(clipboardEncryptSign) toTarget:self withObject:nil];
}

- (void)clipboardEncryptSign
{
	ProcessSerialNumber		frontPSN;
	OSErr					osError;
	NSAutoreleasePool*		pool = [[NSAutoreleasePool alloc] init];
	
	osError = GetFrontProcess(&frontPSN);
	
	[NSApp activateIgnoringOtherApps:YES];
    
	[self transformClipboard:kPGPmailEncryptSignOperation];
    
    osError = SetFrontProcess(&frontPSN);
	
	[pool release];
}

- (NSString *)signEncryptText:(NSString *)data toRecipients:(NSArray *)toRecipients error:(NSString **)error
{
	NSString*				encryptedText = nil;
	CPGPmailProgress*		progress = [CPGPmailProgress sharedInstance];
	CPGPmailProgressJob*	job = nil;
	
	job = [[CPGPmailProgressJob alloc] initWithFile:LSPM(@"Selected Text", @"") operation:kPGPmailEncryptSignOperation selector:nil target:nil];
	[progress startProgressForJob:job];
	[progress updateProgressForJob:job];
	
	encryptedText = [self transformBuffer:data toRecipients:toRecipients operation:kPGPmailEncryptSignOperation];
	
	if (job)
	{
		[progress finishedJob:job];
		[job release];
	}
	
    return encryptedText;
}

- (IBAction)fileSign:(id)sender
{
    PGPUInt32	keyCount = 0;
    PGPError 	theError = kPGPError_NoErr;
    
	[NSApp activateIgnoringOtherApps:YES];
	
    theError = PGPCountKeysInKeyDB( [self pgpKeyDB], &keyCount );
    
    if( keyCount > 0 )
        [self selectFilesToTransform:kPGPmailSignOperation];
    else
    {
        if( [mWindow isVisible] )
            NSBeginCriticalAlertSheet(kErrorStr, kOKStr, nil, nil, mWindow, self, nil,
                nil, nil, kPGPmailMissingKeys);
        else
            NSRunCriticalAlertPanel(kErrorStr, kPGPmailMissingKeys, kOKStr, nil, nil);
    }
}

- (IBAction)clipboardSign:(id)sender
{
	[NSThread detachNewThreadSelector:@selector(clipboardSign) toTarget:self withObject:nil];
}

- (void)clipboardSign
{
	ProcessSerialNumber		frontPSN;
	OSErr					osError;
	NSAutoreleasePool*		pool = [[NSAutoreleasePool alloc] init];
	
	osError = GetFrontProcess(&frontPSN);
	
	[NSApp activateIgnoringOtherApps:YES];
    
	[self transformClipboard:kPGPmailSignOperation];
    
    osError = SetFrontProcess(&frontPSN);
	
	[pool release];
}

- (NSString *)signText:(NSString *)data error:(NSString **)error
{
	NSString*				signedText = nil;
	CPGPmailProgress*		progress = [CPGPmailProgress sharedInstance];
	CPGPmailProgressJob*	job = nil;
	
	job = [[CPGPmailProgressJob alloc] initWithFile:LSPM(@"Selected Text", @"") operation:kPGPmailSignOperation selector:nil target:nil];
	[progress startProgressForJob:job];
	[progress updateProgressForJob:job];
	
	signedText = [self transformBuffer:data toRecipients:nil operation:kPGPmailSignOperation];
	
	if (job)
	{
		[progress finishedJob:job];
		[job release];
	}
	
    return signedText;
}

- (IBAction)fileWipe:(id)sender
{
	[NSApp activateIgnoringOtherApps:YES];
    [self selectFilesToTransform:kPGPmailWipeOperation];
}

- (void)signFiles:(NSArray *)files
{
	[NSApp activateIgnoringOtherApps:YES];
	[self transformFiles:files operation:kPGPmailSignOperation lock:nil];
}

- (void)signSelectedFiles:(NSArray *)files
{
	NSAutoreleasePool*	pool = [[NSAutoreleasePool alloc] init];
	NSLock*				lock = [[NSLock alloc] init];
	
	[lock lock];
	[self transformFiles:files operation:kPGPmailSignOperation lock:lock];
	
	[lock lock];
	[lock release];
	
	[pool release];
}

- (void)encryptFiles:(NSArray *)files
{
	[NSApp activateIgnoringOtherApps:YES];
	[self transformFiles:files operation:kPGPmailEncryptOperation lock:nil];
}

- (void)encryptSelectedFiles:(NSArray *)files
{
	NSAutoreleasePool*	pool = [[NSAutoreleasePool alloc] init];
	NSLock*				lock = [[NSLock alloc] init];
	
	[lock lock];
	[self transformFiles:files operation:kPGPmailEncryptOperation lock:lock];
	
	[lock lock];
	[lock release];
	
	[pool release];
}

- (void)encryptSignFiles:(NSArray *)files
{
	[NSApp activateIgnoringOtherApps:YES];
	[self transformFiles:files operation:kPGPmailEncryptSignOperation lock:nil];
}

- (void)encryptSignSelectedFiles:(NSArray *)files
{
	NSAutoreleasePool*	pool = [[NSAutoreleasePool alloc] init];
	NSLock*				lock = [[NSLock alloc] init];
	
	[lock lock];
	[self transformFiles:files operation:kPGPmailEncryptSignOperation lock:lock];
	
	[lock lock];
	[lock release];
	
	[pool release];
}

- (void)decryptVerifyFiles:(NSArray *)files
{
	[NSApp activateIgnoringOtherApps:YES];
	[self transformFiles:files operation:kPGPmailDecryptVerifyOperation lock:nil];
}

- (void)decryptVerifySelectedFiles:(NSArray *)files
{
	NSAutoreleasePool*	pool = [[NSAutoreleasePool alloc] init];
	NSLock*				lock = [[NSLock alloc] init];
	
	[lock lock];
	[self transformFiles:files operation:kPGPmailDecryptVerifyOperation lock:lock];
	
	[lock lock];
	[lock release];
	
	[pool release];
}

- (void)wipeFiles:(NSArray *)files
{
	[NSApp activateIgnoringOtherApps:YES];
	[self transformFiles:files operation:kPGPmailWipeOperation lock:nil];
}

- (void)wipeSelectedFiles:(NSArray *)files
{
	NSAutoreleasePool*	pool = [[NSAutoreleasePool alloc] init];
	NSLock*				lock = [[NSLock alloc] init];
	
	[lock lock];
	[self transformFiles:files operation:kPGPmailWipeOperation lock:lock];
	
	[lock lock];
	[lock release];
	
	[pool release];
}

- (IBAction)verificationResults:(id)sender
{
	[mVerificationWindow makeKeyAndOrderFront:nil];
}

- (void)transformClipboard:(PGPmailOperation)inOperation
{
    if( kInvalidPGPKeyDBRef == [self pgpKeyDB] )
    {
        NSRunAlertPanel(kStopStr, kPGPmailBadParams, kStopStr, nil, nil);
    }
    else
    {
        NSPasteboard	*pboard = [NSPasteboard generalPasteboard];
        NSArray			*pBoardTypes = [pboard types];
    
        if( [pBoardTypes containsObject:NSStringPboardType] )
        {
            NSString*				inData = [pboard stringForType:NSStringPboardType];
            NSString*				outData = nil;
            NSArray*				types = [NSArray arrayWithObject:NSStringPboardType];
            CPGPmailProgress*		progress = [CPGPmailProgress sharedInstance];
			CPGPmailProgressJob*	job = nil;
			
			job = [[CPGPmailProgressJob alloc] initWithFile:LSPM(@"Clipboard", @"") operation:inOperation selector:nil target:nil];
			[progress startProgressForJob:job];
			[progress updateProgressForJob:job];
			
			outData = [self transformBuffer:inData toRecipients:nil operation:inOperation];
            
            if (outData)
            {
                [pboard declareTypes:types owner:nil];
                [pboard setString:outData forType:NSStringPboardType];
            }
			
			if (job)
			{
				[progress finishedJob:job];
				[job release];
			}
        }
    }
}

- (NSString *)transformBuffer:(NSString *)inBuffer toRecipients:(NSArray *)toRecipients operation:(PGPmailOperation)inOperation
{
	NSString						*outData = nil;
    
    if( kInvalidPGPKeyDBRef == [self pgpKeyDB] )
    {
        NSRunAlertPanel(kStopStr, kPGPmailBadParams, kStopStr, nil, nil);
    }
    else
    {
         CPGPmailEncodeDecodeBuffer		*transformer = [[CPGPmailEncodeDecodeBuffer alloc] initWithController:self];
        
        switch( inOperation )
        {
            case kPGPmailEncryptOperation:
            case kPGPmailSignOperation:
            case kPGPmailEncryptSignOperation:
                outData = [transformer encryptSign:inOperation withData:inBuffer toRecipients:toRecipients];
            break;
            case kPGPmailDecryptVerifyOperation:
                outData = [transformer decryptVerify:inBuffer];
            break;
            default:
            break;
        }
        
        [transformer release];
    }
    return( outData );
}

- (void)selectFilesToTransform:(PGPmailOperation)inOperation
{
	NSOpenPanel*		openPanel = [NSOpenPanel openPanel];
	NSMutableArray*		theTypes = nil;
	NSString*			prompt = nil;
	int					result = 0;
    
	switch( inOperation )
	{
		case kPGPmailDecryptVerifyOperation:
			prompt = kPGPmailDecryptVerify1Str;
			theTypes = [NSMutableArray array];
			[theTypes addObject:@"asc"];
			[theTypes addObject:@"sig"];
			[theTypes addObject:@"pgp"];
			[theTypes addObject:@"shf"];
			[theTypes addObject:NSFileTypeForHFSTypeCode(kPGPMacFileType_EncryptedData)];
			[theTypes addObject:NSFileTypeForHFSTypeCode(kPGPMacFileType_SignedData)];
			[theTypes addObject:NSFileTypeForHFSTypeCode(kPGPMacFileType_DetachedSig)];
			[theTypes addObject:NSFileTypeForHFSTypeCode(kPGPMacFileType_KeyShares)];
			break;
		case kPGPmailEncryptOperation:
			prompt = kPGPmailEncrypt1Str;
			break;
		case kPGPmailSignOperation:
			prompt = kPGPmailSign1Str;
			break;
		case kPGPmailEncryptSignOperation:
			prompt = kPGPmailEncryptSign1Str;
			break;
		case kPGPmailWipeOperation:
			prompt = kPGPmailWipeFile1Str;
			break;
		default:
			break;
	}
	if( prompt )
		[openPanel setPrompt:prompt];

	[openPanel setAllowsMultipleSelection:YES];
	[openPanel setCanChooseDirectories:YES];
    
	if( [mWindow isVisible] )
    {	// start the sheet
        [openPanel beginSheetForDirectory:[self defaultOpenDirectory]
            file:nil types:theTypes
            modalForWindow:mWindow
            modalDelegate:self
            didEndSelector:@selector(transformSheetDidEnd:returnCode:contextInfo:)
            contextInfo:(void *)inOperation];
    }
    else
    {	// start the open panel
        result = [openPanel runModalForDirectory:[self defaultOpenDirectory] file:nil types:theTypes];
        if( result == NSOKButton )
        {
            NSArray		*files = [openPanel filenames];
            
            [self setDefaultOpenDirectory:[[files objectAtIndex:0] stringByDeletingLastPathComponent]];
            [self transformFiles:files operation:inOperation lock:nil];
        }
    }
}

- (void)transformSheetDidEnd:(NSOpenPanel*)sheet returnCode:(int)code contextInfo:(void*)info
{
	[sheet orderOut:self];
	if( code == NSOKButton )
	{
        NSArray		*files = [sheet filenames];
        
        [self setDefaultOpenDirectory:[[files objectAtIndex:0] stringByDeletingLastPathComponent]];
        [self transformFiles:files operation:(PGPmailOperation)info lock:nil];
    }
}

- (void)transformFiles:(NSArray *)inFiles operation:(PGPmailOperation)inOperation lock:(NSLock*)lock
{
    switch( inOperation )
    {
        case kPGPmailEncryptOperation:
        case kPGPmailSignOperation:
        case kPGPmailEncryptSignOperation:
        case kPGPmailDecryptVerifyOperation:
        {
            if( [self pgpKeyDB] == kInvalidPGPKeyDBRef )
            {
                NSRunAlertPanel(kStopStr, kPGPmailBadParams, kStopStr, nil, nil);
            }
            else
            {
                CPGPmailEncodeDecodeFile 	*transformer =
                                            [[CPGPmailEncodeDecodeFile alloc] initWithController:self];
        
                [transformer processFiles:inFiles operation:inOperation lock:lock];
            }
        }
        break;
        case kPGPmailWipeOperation:
        {
            CPGPmailWipeFile*	wiper = [[CPGPmailWipeFile alloc] initWithController:self];
			
			[wiper setLock:lock];
            [wiper processFiles:inFiles];
        }
        break;
        default:
        break;
    }
}

- (void)setDefaultOpenDirectory:(NSString *)directory
{
    [mDefaultOpenDirectory autorelease];
    mDefaultOpenDirectory = [directory copy];
}

- (NSString *)defaultOpenDirectory
{
    return( mDefaultOpenDirectory );
}

- (void)setUseMiniWindow:(BOOL)flag
{
	mUseMiniWindow = flag;
}

- (BOOL)useMiniWindow
{
	return mUseMiniWindow;
}

- (void)windowDidBecomeKey:(NSNotification*)notification
{
	[mAppController setCurrentBundle:self passive:YES];
	
	if (mUseMiniWindow)
		[mSmWindow makeFirstResponder:mLastResponder];
	else
		[mWindow makeFirstResponder:mLastResponder];
}

- (void)windowDidResignKey:(NSNotification*)notification
{
	if (mUseMiniWindow)
		mLastResponder = [mSmWindow firstResponder];
	else
		mLastResponder = [mWindow firstResponder];
}

- (void)windowDidUpdate:(NSNotification*)notification
{
	if ([mWindow isVisible] && ![mWindow attachedSheet])
	{
		[[mWindow standardWindowButton:NSWindowZoomButton] setEnabled:YES];
		[[mWindow standardWindowButton:NSWindowMiniaturizeButton] setEnabled:YES];
	}
	else if ([mSmWindow isVisible] && ![mSmWindow attachedSheet])
	{
		[[mSmWindow standardWindowButton:NSWindowZoomButton] setEnabled:YES];
		[[mSmWindow standardWindowButton:NSWindowMiniaturizeButton] setEnabled:YES];
	}
}

- (void)windowWillBeginSheet:(NSNotification*)notification
{
	if ([mWindow isVisible])
	{
		[mWindow setHidesOnDeactivate:NO];
		[[mWindow standardWindowButton:NSWindowZoomButton] setEnabled:NO];
	}
	
	if ([mSmWindow isVisible])
	{
		[mSmWindow setHidesOnDeactivate:NO];
		[[mSmWindow standardWindowButton:NSWindowZoomButton] setEnabled:NO];
	}
}

- (void)windowDidEndSheet:(NSNotification*)notification
{
	[self windowDidUpdate:nil];
}

- (BOOL)windowShouldClose:(id)sender
{
	if (mUseMiniWindow)
		[mSmWindow close];
	else
		[mWindow close];
	
	return YES;
}

- (BOOL)windowShouldZoom:(NSWindow*)sender toFrame:(NSRect)frame
{
	NSRect		oldFrame;
	NSRect		newFrame;
	
	if (mUseMiniWindow)
	{
		oldFrame = [mSmWindow frame];
		newFrame = [mWindow frame];
		
		newFrame.origin.x = oldFrame.origin.x;
		newFrame.origin.y = oldFrame.origin.y - (NSHeight(newFrame) - NSHeight(oldFrame));
		
		[mWindow setFrame:newFrame display:NO];
		[mWindow makeKeyAndOrderFront:nil];
		[mSmWindow orderOut:nil];
	}
	else
	{
		oldFrame = [mWindow frame];
		newFrame = [mSmWindow frame];
		
		newFrame.origin.x = oldFrame.origin.x;
		newFrame.origin.y = oldFrame.origin.y + (NSHeight(oldFrame) - NSHeight(newFrame));
		
		[mSmWindow setFrame:newFrame display:NO];
		[mSmWindow makeKeyAndOrderFront:nil];
		[mWindow orderOut:nil];
	}
	
	[self setUseMiniWindow:!mUseMiniWindow];
	
	return NO;
}

- (void)dealloc
{    
    [super dealloc];
}

@end

/*__Editor_settings____

	Local Variables:
	tab-width: 4
	End:
	vi: ts=4 sw=4
	vim: si
_____________________*/
