/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.

	$Id: CPGPmailEncodeDecodeBuffer.m,v 1.14.2.1 2002/11/27 01:17:16 jeffc Exp $
____________________________________________________________________________*/

#import <PGPclient/pgpClientLibOSX.h>
#import <PGPclient/pgpClientLibCocoa.h>
#import <Foundation/Foundation.h>

#import "PGPmailCommon.h"
#import "CPGPmailEncodeDecodeBuffer.h"

@implementation CPGPmailEncodeDecodeBuffer

- (id)init;
{
    PGPError 		theError = kPGPError_NoErr;
    
    if( !mInited )
    {
        [super init];
        mInited = YES;
    }
    if( !mAppController )
        PGPLog((@"%s:%d: Warning: mAppController has not been set.", __FILE__, __LINE__));
        
    if( !mContext && mAppController )
        mContext = [mAppController pgpContext];
        
    if( !mTLSContext && mAppController )
        mTLSContext = [mAppController tlsContext];
    
    if( !mPrefRef && mAppController )
        mPrefRef = [mAppController pgpPrefRef];
    
    if( !mAllKeys && mAppController )
        mAllKeys = [mAppController pgpKeyDB];
    
    if( !mMemoryMgr && mContext )
        mMemoryMgr = PGPPeekContextMemoryMgr( mContext );
    
    theError = PGPclInitEncodeDecodeData( mContext, mTLSContext, mAllKeys, mPrefRef, &mDataRef );

    return( self );
}

- (id)initWithController:(id)controller
{
    mAppController = controller;
    return ( [self init] );
}

- (void)dealloc
{
    PGPclCleanupEncodeDecodeData( &mDataRef );
    
    [super dealloc];
}

- (NSString *)encryptSign:(PGPmailOperation)inOperation withData:(NSString *)inData toRecipients:(NSArray *)toRecipients
{
    PGPError			theError = kPGPError_NoErr;
    NSString 			*outData = nil;
    PGPByte 			*theData = NULL;
    PGPSize				outDataSize = 0;
    PGPBoolean			forceDialogs = FALSE;		
	PGPUInt32			numRecipients = [toRecipients count];
	PGPRecipientSpec *	recipients = NULL;
	NSMutableString*	mutableString = [[inData mutableCopy] autorelease];
    
    // Search the string to make sure that data is in \n format and not \r.
	
	[mutableString replaceOccurrencesOfString:@"\r" withString:@"\n" options:NSLiteralSearch range:NSMakeRange(0, [inData length])];
	inData = mutableString;
	
    if (numRecipients > 0) {
		recipients = (PGPRecipientSpec *) PGPNewData(mMemoryMgr, numRecipients * sizeof(PGPRecipientSpec), kPGPMemoryMgrFlags_Clear);
		if (recipients) {
			int i;
			
			for (i = 0; i < numRecipients; i++) {
				recipients[i].type = kPGPRecipientSpecType_UserID;
				[[toRecipients objectAtIndex:i] getCString:recipients[i].u.userIDStr maxLength:255];
			}
		}
	}
    switch( inOperation )
    {
        case kPGPmailEncryptSignOperation:
        theError = PGPclEncode(&mDataRef, TRUE, kPGPUISettingsTextOutput,
				kPGPGetPassphraseOptionsHideFileOptions, ! forceDialogs, 
				FALSE, TRUE, TRUE, kPGPUISettingsTextOutput,
				kPGPRecipientOptionsHideFileOptions | 
					kPGPRecipientOptionsShowFYEO | ((forceDialogs) ?
					kPGPRecipientOptionsAlwaysShowDialog : 0),
				numRecipients, recipients, TRUE, [inData UTF8String], strlen([inData UTF8String]),
                &theData, &outDataSize);
        break;
        case kPGPmailEncryptOperation:
            theError = PGPclEncode( &mDataRef, FALSE, 0, 0, FALSE, FALSE, TRUE, TRUE,
                kPGPUISettingsTextOutput,
                kPGPRecipientOptionsHideFileOptions | kPGPRecipientOptionsShowFYEO |
                ((forceDialogs) ? kPGPRecipientOptionsAlwaysShowDialog : FALSE),
				numRecipients, recipients, TRUE, [inData UTF8String], strlen([inData UTF8String]),
                &theData, &outDataSize );
        break;
        case kPGPmailSignOperation:
            theError = PGPclEncode( &mDataRef, TRUE, kPGPUISettingsTextOutput,
				kPGPGetPassphraseOptionsHideFileOptions, ! forceDialogs, 
				TRUE, TRUE, FALSE, 0, 0, 0, 0, FALSE, [inData UTF8String], strlen([inData UTF8String]),
                &theData, &outDataSize );
        break;
        default:
        break;
    }

    if( theData != NULL )
    {
		if (IsntPGPError(theError)) {
			outData = [NSString stringWithUTF8String:theData];
		}
        PGPFreeData( theData );
    }
    if (recipients != NULL) {
		PGPFreeData(recipients);
	}
	
    return( outData );
}

- (NSString *)decryptVerify:(NSString *)inData
{
    PGPByte *		theData = NULL;
    PGPSize			outDataSize;
    PGPError 		theError;
    NSString		*outData = nil;
    
    theError = PGPclDecode( &mDataRef, [inData UTF8String], [inData length], &theData, &outDataSize );

    if( theData != NULL )
    {
		if (IsntPGPError(theError)) {
			outData = [NSString stringWithUTF8String:theData];
		}
          PGPFreeData( theData );
    }

    return( outData );
}
@end
