/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.

	$Id: CPGPmailEncodeDecodeFile.m,v 1.78 2002/11/15 10:17:32 wprice Exp $
____________________________________________________________________________*/

#import <PGP/PGP.h>
#import <PGPui/CPGPIcons.h>
#import <pgpMem.h>
#import <PGPclient/pflPrefs.h>
#import <PGPclient/pgpClientLibOSX.h>
#import <PGPclient/pgpClientLibCocoa.h>
#import <PGPclient/CPGPPrefObj.h>
#import <PGPclient/pgpClientPrefs.h>
#import <PGPclient/pgpVersionHeader.h>
#import <PGPclient/pgpClientLibOSX.h>
#import <PGPclient/PGPSharedEncryptDecrypt.h>
#import <PGPclient/pgpClientErrors.h>
#import <PGPclient/pgpShareFile.h>
#import <pgpFileSpec.h>

#import "PGPmailCommon.h"
#import "CPGPVerification.h"
#import "PGPmailFileUtilities.h"
#import "CPGPmailWipeFile.h"
#import "CPGPmailEncodeDecodeFile.h"

#define kSaveAsPromptStr                    LSPM(@"Save As:", @"Save As")
#define kSaveEncryptedFileAsPromptStr       LSPM(@"Save Encrypted File As:", @"Save Encrypted File As")
#define kSaveSignedFileAsPromptStr          LSPM(@"Save Signed File As:", @"Save Signed File As")
#define kSaveDecryptedFileAsPromptStr       LSPM(@"Save Decrypted File As:", @"Save Decrypted File As")
#define kFileCannotBeEncryptedStr			LSPM(@"The file could not be Encrypted.", @"")
#define kFileCannotBeSignedStr				LSPM(@"The file could not be Signed.", @"")
#define kFileCannotBeDecryptedVerifiedStr   LSPM(@"The file could not be Decrypted/Verified.", @"")

#define kIncorrectPassphrasePrefixStr		LSPM(@"Incorrect passphrase.", @"Incorrect Passphrase")
#define kGetPassphraseForFilePromptStr		LSPM(@"Please enter the passphrase for the file: ", @"Passphrase")
#define kUnableToMixMacBinaryAndFYEOStr		LSPM(@"MacBinary encoding must be turned off in order to use the Secure Viewer option.", @"MacBinary/FYEO")
#define kConfirmMacBinaryOffStr				LSPM(@"Encrypting or signing files without MacBinary on is intended for sending files to non-MacOS users. The resulting files will not be the same as the original files.  You should be sure to retain the original files.", @"Confirm MacBinary")
#define kMacBinaryFileSizeTooLargeStr		LSPM(@"You cannot encode to MacBinary files larger than 2G.", @"MacBinary File Size")
#define kConfirmNotEncryptingToSelfStr		LSPM(@"You have chosen not to encrypt to yourself and you are wiping the original item(s). You will not be able to decrypt the resulting item(s) nor will you have the original files. Are you sure you want to do this?", @"Encrypt to self")
#define kSDAPassphrasePromptStr				LSPM(@"Please enter a passphrase to protect the self-decrypting archive:", @"Self Decrypting Passphrase")
#define kConventionalPassphrasePromptStr	LSPM(@"Please enter the encryption passphrase:", @"Conventional Passphrase")
#define kConfirmSEACreationStr				LSPM(@"Are you sure you want to create this archive without a passphrase? The contents will not be encrypted.", @"Confirm SEA Creation")

#define kSelectiveImportFoundKeysPromptStr	LSPM(@"Please select those keys, if any, that you wish to add to the local keyring:", @"")
#define kSigningPassphrasePromptStr			LSPM(@"Enter the passphrase for the key shown above.", @"")
#define kOperationCouldNotBeCompletedStr	LSPM(@"The operation could not be completed because", @"")
#define kAnErrorOccurredStr					LSPM(@"An error occurred: ", @"")

#define kConfirmWipingItemsAfterSigningStr	LSPM(@"After signing the items, are you sure you want to permanently erase the originals?", @"")
#define kConfirmWipingItemsAfterEncryptingStr LSPM(@"After encrypting the items, are you sure you want to permanently erase the originals?", @"")
#define kConfirmWipingItemsAfterEncryptingSigningStr LSPM(@"After encrypting and signing the items, are you sure you want to permanently erase the originals?", @"")
#define kConfirmWipingItemAfterSigningStr 	LSPM(@"After signing , are you sure you want to permanently erase ", @"")
#define kConfirmWipingItemAfterEncryptingStr LSPM(@"After encrypting , are you sure you want to permanently erase ", @"")
#define kConfirmWipingItemAfterEncryptingSigningStr LSPM(@"After encrypting and signing , are you sure you want to permanently erase ", @"")
#define kConfirmWipingItemsStr				LSPM(@"Are you sure you want to permanently erase the items?", @"")
#define kConfirmWipingItemStr				LSPM(@"Are you sure you want to permanently erase ", @"")
#define kDecryptKeyShareFileMsgStr 			LSPM(@"This file is an encrypted share file. Share files should be stored in encrypted form until needed. This file must be re-encrypted to one or more public keys or conventionally encrypted with a passphrase. ", @"")

#define kOperationNotSupported				LSPM(@"Operation Not Supported", @"")

#define kUnableToSave						LSPM(@"Unable to Save.", @"Alert title when unable to save.")
#define kDirectoryNotWritable				LSPM(@"The directory is not writable.", @"Alert message when destination directory is not writable.")

static BOOL sPGPmailEncodeDecodeStop = NO;

	PGPError
sEncodeEventHandlerProc(
    PGPContextRef		context,
	PGPEvent*			event,
   	PGPUserValue		userValue);

	PGPError
sDecodeEventHandlerProc(
    PGPContextRef		context,
	PGPEvent*			event,
	PGPUserValue		userValue);

//____________________________________________________________________________
	static PGPLocalEncodingFlags
PGPUISettingsToLocalEncoding( PGPUISettings	settings )
{
	PGPLocalEncodingFlags	encodingFlags = kPGPLocalEncoding_None;
	
	if( ( settings & kPGPUISettingsSmartMacBinary ) != 0 )
	{
		encodingFlags = kPGPLocalEncoding_Auto;
	}
	else if( ( settings & kPGPUISettingsForceMacBinary ) != 0 )
	{
		encodingFlags = kPGPLocalEncoding_Force;
	}
	
	return( encodingFlags );
}

//____________________________________________________________________________
    void
PGPmailInitEncodeDecodeParams(
    PGPmailEncodeDecodeParams*		data )
{
    data->allKeys					= kInvalidPGPKeyDBRef;
    data->encodeOptions				= kInvalidPGPOptionListRef;
    data->encrypting				= FALSE;
    data->conventionalEncrypt		= FALSE;
    data->signing					= FALSE;
    data->detachedSignature			= FALSE;
    data->textOutput				= FALSE;
    data->wipeOriginal				= FALSE;
    data->encryptingToSelf			= FALSE;
    data->signingWithSplitKey		= FALSE;
    data->fyeo						= FALSE;
    data->selfDecryptingArchive		= FALSE;
    data->signingKey				= kInvalidPGPKeyDBObjRef;
    data->encodingFlags				= kPGPLocalEncoding_None;
    data->conventionalPassphrase[0] = NULL;
    data->signingPassphrase[0]		= NULL;
    data->signingPasskey			= NULL;
    data->signingPasskeySize		= 0;
}

//____________________________________________________________________________
    void
PGPmailCleanupEncodeDecodeParams(
    PGPmailEncodeDecodeParams*		data )
{
	if( PGPOptionListRefIsValid( data->encodeOptions ) )
        PGPFreeOptionList( data->encodeOptions );
        
    if( IsntNull( data->signingPasskey ) )
        PGPFreeData( data->signingPasskey );
}

//____________________________________________________________________________
    PGPError
PGPmailInitEventHandlerData(
    PGPContextRef 				context,
	PGPtlsContextRef			tlsContext,
	PGPPrefRef					prefRef,
	CPGPmailEncodeDecodeFile*	task,
	PGPmailEncodeDecodeParams	params,
    PGPmailEventHandlerData*	data )
{
    PGPError	theError = kPGPError_NoErr;
    
    data->mTask						= task;
    data->mParams					= params;
    data->retryingSigVerification	= FALSE;
    data->isFYEO					= FALSE;
    data->outputData				= NULL;
    data->outputDataSize			= 0;
    
    theError = PGPclInitDecodeEventHandlerData(
                context, tlsContext,
                params.allKeys,
                prefRef,
                &data->clientHandlerData );
    pgpAssertNoErr( theError );
    
    return( theError );
}

//____________________________________________________________________________
	void
PGPmailCleanupEventHandlerData(
    PGPmailEventHandlerData* 	data )
{
    PGPclCleanupDecodeEventHandlerData( &data->clientHandlerData );
}

@interface CPGPmailEncodeDecodeFile( PrivateMethods )

- (void)processThread;

- (void)processEncryptSign;

- (void)processDecryptVerify;

- (void)processWipe;

- (PGPError)setupEncryptionOptions:
    (PGPKeyDBRef)recipients
    params:(PGPmailEncodeDecodeParams *)encodeParams;

- (PGPError)setupSigningOptions:
    (PGPGetPassphraseSettings)settings
	options:(PGPGetPassphraseOptions)options
	optionList:(PGPOptionListRef)encodeOptions;

- (PGPError)setupCommonOptions:
    (PGPOptionListRef)encodeOptions;

- (BOOL)recipientsIncludeSelf:
    (PGPKeyDBRef)recipients;

- (PGPError)confirmationDialogs:
    (PGPmailEncodeDecodeParams *)params;

- (PGPError)setMacBinaryOption:
    (PGPmailEncodeDecodeParams *)params;

- (PGPError)confirmWipe:
    (BOOL)encrypting
    signing:(BOOL)signing;

- (PGPError)confirmNotEncryptingToSelf;

- (PGPError)genericEventHandler:
    (PGPEvent *)event
    data:(PGPmailEventHandlerData *)data;

- (PGPError)encodeEventHandler:
    (PGPEvent *)event
    data:(PGPmailEventHandlerData *)data;

- (PGPError)nullEvent:
    (PGPEvent *)event
    data:(PGPmailEventHandlerData *)data;

- (PGPError)decodePassphraseEvent:
    (PGPEvent *)event
    data:(PGPmailEventHandlerData *)data;

- (PGPError)decodeRecipientsEvent:
    (PGPEvent *)event
    data:(PGPmailEventHandlerData *)data;

- (PGPError)decodeSignatureEvent:
    (PGPEvent *)event
    data:(PGPmailEventHandlerData *)data;

- (PGPError)decodeAnalyzeEvent:
    (PGPEvent *)event
    data:(PGPmailEventHandlerData *)data;

- (PGPError)decodeEventHandler:
    (PGPEvent *)event
    data:(PGPmailEventHandlerData *)data;

- (PGPError)encryptSignFileInternal:
    (PGPmailEncodeDecodeParams *)params
    withInfo:(const ProcessingInfo *)srcInfo
    dest:(NSString *)dest;

- (PGPError)encryptSign:
    (PGPmailEncodeDecodeParams *)params;

- (PGPError)encryptSignFolder:
    (PGPmailEncodeDecodeParams *)params;

- (BOOL)shouldEncryptSignFile:
    (BOOL)encrypting
    file:(NSString *)file;

- (PGPError)getOutputFolderName:
    (PGPBoolean)encryptSign
    src:(NSString *)src
    dest:(NSString **)dest;

- (PGPError)decryptVerifyEncryptedFile:
    (PGPmailEncodeDecodeParams *)params
    info:(ProcessingInfo *)srcInfo
    dest:(NSString *)dest;

- (PGPError)decryptKeyShareFile:
    (PGPmailEncodeDecodeParams *)params
    info:(ProcessingInfo *)srcInfo
    dest:(NSString *)dest;

- (PGPError)decryptVerifyFileInternal:
    (PGPmailEncodeDecodeParams *)params
    info:(ProcessingInfo *)srcInfo
    dest:(NSString *)dest;

- (PGPError)decryptVerifyFolder:
    (PGPmailEncodeDecodeParams *)params;

- (PGPError)decryptVerify:
    (PGPmailEncodeDecodeParams *)params;

- (PGPError)calcProgressBytes:
    (ByteCount *)progressBytes;

- (PGPError)decodeOutputEvent:
    (PGPEvent *)event
    data:(PGPmailEventHandlerData *)data;

- (PGPError)decodeDetachedSignatureEvent:
    (PGPEvent *)event
    data:(PGPmailEventHandlerData *)data;

- (void)getDecryptPassphrasePromptString:
    (NSString **)prompt;

- (PGPError)canUseFYEO:
    (BOOL *)outCanUseFYEO;

- (void)getProcessingInfo:
    (NSString *)file
    info:(ProcessingInfo *)info;

- (void)getIndFile:
    (UInt32)index
    info:(ProcessingInfo *)info;

- (void)getErrorString:(PGPError)inErr
    value:(NSString* *)errorStr;

- (void)newResult:(NSString *)fileName sigData:(PGPmailSignatureData *)signatureData;

- (void)getConfirmWipeStr:(BOOL)encrypting
    sign:(BOOL)signing
	multiple:(BOOL)multipleFiles
    string:(NSString **)str;

- (void)setCurrentPrefs;

- (void)process;

@end

//____________________________________________________________________________
@implementation CPGPmailEncodeDecodeFile

//____________________________________________________________________________
- (id)init
{
    if( !mInited )
    {
        [super init];
        mInited = YES;
    }
    if( !mAppController )
        PGPLog((@"%s:%d: Warning: mAppController has not been set.", __FILE__, __LINE__));
    
    if( !mContext && mAppController )
        mContext = [mAppController pgpContext];
        
    if( !mTLSContext && mAppController )
        mTLSContext = [mAppController tlsContext];
    
    if( !mPrefRef && mAppController )
        mPrefRef = [mAppController pgpPrefRef];
    
    if( !mAllKeys && mAppController )
        mAllKeys = [mAppController pgpKeyDB];
    
    if( !mMemoryMgr && mContext )
        mMemoryMgr = PGPPeekContextMemoryMgr( mContext );

	// setup default prefs
	pgpClearMemory( &mTaskPrefs, sizeof( mTaskPrefs ) );
	
	mTaskPrefs.warnOnWipe		= TRUE;
	mTaskPrefs.comment[ 0 ]		= 0;
	mTaskPrefs.convAlgorithm	= kPGPCipherAlgorithm_None;
	mTaskListHasFileTasks		= NO;
	mBadPassphraseCount			= 0;
	mDataType					= kPGPAnalyze_Unknown;
	mSignatureDataValid 		= FALSE;
    mWipeTasks					= [[NSMutableArray array] retain];
    sPGPmailEncodeDecodeStop	= NO;
    
	pgpClearMemory( &mSignatureData, sizeof( mSignatureData ) );
    
    mFileManager 				= [[NSFileManager defaultManager] retain];
    
    mProgressDialog				= [CPGPmailProgress sharedInstance];
    
    return( self );
}

//____________________________________________________________________________
- (id)initWithController:
    (CPGPmailController *)inController
{
    mAppController 	= inController;
    
    return( [self init] );
}

//____________________________________________________________________________
- (void)dealloc
{
    if( IsntNull( mTaskPrefs.allowedAlgorithms ) )
		(void) PGPFreeData( mTaskPrefs.allowedAlgorithms );
        
    [super dealloc];
}

- (void)processFiles:(NSArray*)inFiles operation:(PGPmailOperation)inOperation lock:(NSLock*)lock
{
    mOperation = inOperation;
    mTaskList = [inFiles copy];
    mTaskListHasFileTasks = YES;
    
    mModifierFlags = [[NSApp currentEvent] modifierFlags];
    
    [NSThread detachNewThreadSelector:@selector(processThread:) toTarget:self withObject:lock];
}

- (void)processThread:(NSLock*)lock
{
    NSAutoreleasePool*	pool = [[NSAutoreleasePool alloc] init];
    NSEnumerator*		enumerator;
    NSString*			file;
    
    [self process];
    
    if (lock)
        [lock unlock];
    
    enumerator = [mTaskList objectEnumerator];
    
    while (file = [enumerator nextObject])
        [[mAppController controller] finishedHandlingDocument:file];
    
    [mTaskList release];
	mTaskList = nil;
    [mWipeTasks release];
    mWipeTasks = nil;
    [mFileManager release];
    mFileManager = nil;
    
    [pool release];
    [self release];
}

//____________________________________________________________________________
- (PGPError)setupEncryptionOptions:
    (PGPKeyDBRef)recipients
    params:(PGPmailEncodeDecodeParams *)encodeParams
{
	PGPError		theError = kPGPError_NoErr;
	
	// Check for conventional encryption first.
	
	if( encodeParams->conventionalEncrypt )
	{
		char		*passphrase = NULL;
		NSString*	prompt = nil;
		BOOL		repeatPassphrase = YES;
		
        prompt = ( encodeParams->selfDecryptingArchive ) ?
            kSDAPassphrasePromptStr : kConventionalPassphrasePromptStr;

		while( repeatPassphrase )
		{
			repeatPassphrase = NO;
			
			theError = PGPConventionalEncryptionPassphraseDialog( mContext,
								PGPOUIDialogPrompt( mContext, [prompt UTF8String] ),
								PGPOUIOutputPassphrase( mContext, &passphrase ),
								encodeParams->selfDecryptingArchive ?
									PGPONullOption( mContext ) :
									PGPOUIMinimumPassphraseLength( mContext, 1 ),
								PGPOLastOption( mContext ) );
		
            if( IsntPGPError( theError ) )
                (void)strcpy(encodeParams->conventionalPassphrase, passphrase);
			
			if( IsntPGPError( theError ) )
			{
				if( encodeParams->selfDecryptingArchive )
				{
					if( passphrase[0] == 0 )
					{
						// Verify SEA creation
						int			result = 0;
                        
						NSBeep();
                        
                        result = NSRunAlertPanel(kCautionStr, kConfirmSEACreationStr,
                                    kOKStr, kCancelStr, nil);
                        if( result == NSAlertDefaultReturn )
                            repeatPassphrase = TRUE;
					}
				}
				else
				{
					theError = PGPAppendOptionList( encodeParams->encodeOptions,
							PGPOConventionalEncrypt( mContext,
								PGPOPassphrase( mContext, passphrase ),
								PGPOLastOption( mContext ) ),
							mTaskPrefs.convAlgorithm != kPGPCipherAlgorithm_None ?
								PGPOCipherAlgorithm( mContext,
									mTaskPrefs.convAlgorithm) :
								PGPONullOption( mContext ),
							PGPOLastOption( mContext ) );
				}
			}
		
			if( IsntNull( passphrase ) )
				PGPFreeData( passphrase );
		}
	}
	else
	{
		PGPUInt32	numKeys	= 0;

		// Verify that there is at least one recipient
		
		theError	= PGPCountKeysInKeyDB( recipients, &numKeys );
		if( IsntPGPError( theError ) && numKeys == 0 )
		{
			theError = kPGPError_UserAbort;
		}
		else
		{
			theError = PGPAppendOptionList( encodeParams->encodeOptions,
					PGPOEncryptToKeySet( mContext,
						PGPPeekKeyDBRootKeySet( recipients ) ),
					PGPOLastOption( mContext ) );
		}
	}

	// Setup preferred algorithms
	if( IsntPGPError( theError ) && IsntNull( mTaskPrefs.allowedAlgorithms ) )
	{
		theError = PGPAppendOptionList( encodeParams->encodeOptions,
						PGPOPreferredAlgorithms( mContext,
								mTaskPrefs.allowedAlgorithms,
								mTaskPrefs.numAllowedAlgorithms ),
						PGPOLastOption( mContext ) );
	}
	
	return( theError );
}

//____________________________________________________________________________
- (PGPError)setupSigningOptions:
    (PGPGetPassphraseSettings)settings
	options:(PGPGetPassphraseOptions)options
	optionList:(PGPOptionListRef)encodeOptions
{
	PGPError	theError = kPGPError_NoErr;
    
	(void) settings;
	(void) options;
	(void) encodeOptions;
        
	return( theError );
}

//____________________________________________________________________________
- (PGPError)setupCommonOptions:
    (PGPOptionListRef)encodeOptions
{
	PGPError		theError = kPGPError_NoErr;

	if( IsntPGPError( theError ) )
	{
		theError = PGPAppendOptionList( encodeOptions,
					PGPOVersionString( mContext, pgpVersionHeaderString()),
					PGPOCommentString( mContext, mTaskPrefs.comment ),
					PGPOLastOption( mContext ) );
	}
	
	return( theError );
}

//____________________________________________________________________________
//	Return TRUE if at least one of the recipients has a corresponding private
//	key.
//____________________________________________________________________________
- (BOOL)recipientsIncludeSelf:
    (PGPKeyDBRef)recipients
{
	PGPError		theError = kPGPError_NoErr;
	BOOL			havePrivateKey	= NO;
	PGPKeyListRef	keyListRef;
	
	theError = PGPOrderKeySet( PGPPeekKeyDBRootKeySet( recipients ),
						kPGPKeyOrdering_Any, FALSE, &keyListRef );
	if( IsntPGPError( theError ) )
	{
		PGPKeyIterRef	keyIterator;
	
		theError = PGPNewKeyIter( keyListRef, &keyIterator );
		
		if( IsntPGPError( theError ) )
		{
			PGPKeyDBObjRef	theKey;
			
			theError = PGPKeyIterNextKeyDBObj( keyIterator,
							kPGPKeyDBObjType_Key, &theKey );
			while( IsntPGPError( theError ) )
			{
				PGPBoolean	isSecret;
				
				theError	= PGPGetKeyDBObjBooleanProperty( theKey,
								kPGPKeyProperty_IsSecret, &isSecret );
				if( IsPGPError( theError ) )
					break;
				if( isSecret )
				{
					havePrivateKey	= YES;
					break;
				}
					
				theError = PGPKeyIterNextKeyDBObj( keyIterator,
							kPGPKeyDBObjType_Key, &theKey );
			}
			if( theError == kPGPError_EndOfIteration )
				theError = kPGPError_NoErr;
				
			PGPFreeKeyIter( keyIterator );
		}
		
		PGPFreeKeyList( keyListRef );
	}
	
	return( havePrivateKey );
}

//____________________________________________________________________________
- (PGPError)confirmationDialogs:
    (PGPmailEncodeDecodeParams *)params
{
	PGPError		theError = kPGPError_NoErr;
	
	if( ! mTaskListHasFileTasks )
		return( theError );
    
    theError = [self setMacBinaryOption:params];

	if( mTaskPrefs.warnOnWipe )
	{
		//if( IsntPGPError( theError ) )
		//	if( params->wipeOriginal )
		//		theError = [self confirmWipe:params->encrypting signing:params->signing];
		
		if( IsntPGPError( theError ) )
			if( (!params->encryptingToSelf) && params->wipeOriginal && (!params->conventionalEncrypt) )
				theError = [self confirmNotEncryptingToSelf];
	}
	
	return( theError );
}

//____________________________________________________________________________
- (void)processEncryptSign
{
	PGPKeyDBRef					recipients	= kInvalidPGPKeyDBRef;
    CPGPmailProgressJob*		job = nil;
    NSString*					filename;
    
    if ([mTaskList count] != 0)
    {
        if ([mTaskList count] > 1)
            filename = [NSString stringWithFormat:@"%@, ", [mFileManager displayNameAtPath:[mTaskList objectAtIndex:0]]];
        else
            filename = [mFileManager displayNameAtPath:[mTaskList objectAtIndex:0]];
        
        job = [[CPGPmailProgressJob alloc] initWithFile:filename operation:mOperation selector:@selector(stopProcessing) target:self];
        [mProgressDialog startProgressForJob:job];
    }
	
	//	Blocked as a quick fix to the optionlist being freed before the keydb
	{
		PGPError					theError = kPGPError_NoErr;
		PGPRecipientSettings		recipientSettings;
		PGPRecipientOptions			recipientOptions;
		PGPGetPassphraseSettings	getPassphraseSettings;
		PGPGetPassphraseOptions		getPassphraseOptions;
		PGPmailEncodeDecodeParams	state;
		BOOL						haveEncodingFlags	= NO;
		
		pgpAssert( 	mOperation == kPGPmailsEncryptOperation ||
					mOperation == kPGPmailSignOperation ||
					mOperation == kPGPmailEncryptSignOperation );
		
		recipientOptions 		= kPGPRecipientOptionsDefault;
		getPassphraseOptions	= kPGPGetPassphraseOptionsDefault;
		getPassphraseSettings	= 0;
		
		if( ! mTaskListHasFileTasks )
		{
			recipientOptions 		|= kPGPRecipientOptionsHideFileOptions;
			recipientOptions		|= kPGPRecipientOptionsShowFYEO;
			getPassphraseOptions	|= kPGPGetPassphraseOptionsHideFileOptions;
		}
        else if( ( [mTaskList count] == 1 ) )
		{
			BOOL	canFYEO = NO;
			
            mSource = [mTaskList objectAtIndex:0];
            
			theError = [self canUseFYEO:&canFYEO];
			if( IsntPGPError( theError ) && canFYEO )
			{
				recipientOptions |= kPGPRecipientOptionsShowFYEO;
			}
		}
        		
		if( IsntPGPError( theError ) )
		{
            PGPmailInitEncodeDecodeParams( &state );
         			
			state.allKeys			= mAllKeys;
			state.encrypting		= ( mOperation == kPGPmailEncryptOperation ||
										mOperation == kPGPmailEncryptSignOperation );
			state.signing			= ( mOperation == kPGPmailSignOperation ||
										mOperation == kPGPmailEncryptSignOperation );
			state.encodingFlags		= kPGPLocalEncoding_None;

			theError = PGPNewOptionList( mContext, &state.encodeOptions);
		}
		
		if( IsntPGPError( theError ) )
		{
			if( state.encrypting)
			{
				PGPKeyDBRef		newKeys = kInvalidPGPKeyDBRef;
				
				// Hide the passphrase file options if we're encrypting
				getPassphraseOptions |= kPGPGetPassphraseOptionsHideFileOptions;

				// Show the SDA checkbox if we're encryting (and not signing) files.
				//if( mTaskListHasFileTasks && ! state.signing )
				//	recipientOptions |= kPGPRecipientOptionsShowSDA;
					
				// Show the PGPRecipientDialog
                theError = PGPclRecipientDialog( mContext, mPrefRef, mTLSContext,
								mAllKeys, 0, nil, recipientOptions,
								kPGPRecipientSettingsDefault,
								&recipientSettings, &recipients, &newKeys );
                                
				if( IsntPGPError( theError ) && PGPKeyDBRefIsValid( newKeys ) )
				{
					PGPKeySetRef	importSet;

					theError = PGPclSelectKeysDialog( mContext, mPrefRef,
							kPGPclKeySelect_ImportVariation,
							[kSelectiveImportFoundKeysPromptStr UTF8String],
                            newKeys, mAllKeys, &importSet );
					if( IsntPGPError( theError ) )
					{
                        theError = PGPCopyKeys( importSet, mAllKeys, NULL );
						
						(void) PGPFreeKeySet( importSet );
					}
					else if( theError == kPGPError_UserAbort )
					{
						theError = kPGPError_NoErr;
					}
					
					(void) PGPFreeKeyDB( newKeys );
				}
				
				if( IsntPGPError( theError ) )
				{
					if( ( ( recipientSettings & kPGPRecipientSettingsFYEO ) != 0 )
					&& ( ( recipientSettings & kPGPUISettingsNoMacBinary ) == 0 )
					&& ( ( recipientOptions & kPGPRecipientOptionsHideFileOptions ) == 0 ) )
					{
						NSBeep();
                        NSRunAlertPanel(kWarningStr, kUnableToMixMacBinaryAndFYEOStr, kOKStr, kCancelStr, nil);
						theError = kPGPError_IllegalFileOp;
					}
				}
				
				if( IsntPGPError( theError ) )
				{
					if( ( recipientSettings & kPGPRecipientSettingsSDA ) != 0 )
					{
						state.selfDecryptingArchive 	= TRUE;
						state.conventionalEncrypt		= TRUE;
						state.encodingFlags				= kPGPLocalEncoding_Force;
					}
					else
					{
						state.encodingFlags	=
							PGPUISettingsToLocalEncoding( recipientSettings );

						state.conventionalEncrypt = ( recipientSettings &
							kPGPRecipientSettingsConvEncrypt ) != 0;
						
					}
					
					haveEncodingFlags = TRUE;
					
					state.wipeOriginal	=
					(recipientSettings & kPGPRecipientSettingsWipeOriginal) != 0;
					
					state.fyeo = ( recipientSettings & kPGPRecipientSettingsFYEO ) != 0;
						
					theError = [self setupEncryptionOptions:recipients params:&state];
				}
				
				if( IsntPGPError( theError ) && ! state.conventionalEncrypt )
				{
					state.encryptingToSelf	= [self recipientsIncludeSelf:recipients];
				}
            }
		}
		
		if( IsntPGPError( theError ) && state.signing )
		{
			PGPSize			keyIDSize;
			PGPByte *		keyID;
			PGPKeyDBObjRef	defaultKey = kInvalidPGPKeyDBObjRef;
				
			theError = PGPGetPrefData( mPrefRef, kPGPPrefDefaultKeyID, &keyIDSize, (void *)&keyID );
			if( IsntPGPError( theError ) )
			{
				(void) PGPFindKeyByKeyID( mAllKeys, (PGPKeyID *)keyID, &defaultKey );
				
				PGPFreeData( keyID );
			}
            
			theError = PGPclSigningPassphraseDialog( mContext, mPrefRef,
						mAllKeys, [kSigningPassphrasePromptStr UTF8String],
						getPassphraseOptions,
						kPGPGetPassphraseSettingsDetachedSig,
						defaultKey,
						state.signingPassphrase,
						&getPassphraseSettings,
						&state.signingKey );
			
			if( theError == kPGPError_KeyUnusableForSignature )
			{
				// User has chosen split key. Show reconstitution dialog
				theError = PGPclReconstitutionDialog( state.signingKey, mAllKeys,
										mTLSContext, mPrefRef, &state.signingPasskey,
										&state.signingPasskeySize);
				
				state.signingWithSplitKey = TRUE;
			}
			
			if( IsntPGPError( theError ) )
			{
				PGPUInt32					cacheType,
											cacheSeconds;
				PGPBoolean					cacheGlobal;
				
				(void) PGPGetPrefNumber( mPrefRef, kPGPPrefCacheType, &cacheType );
				(void) PGPGetPrefNumber( mPrefRef, kPGPPrefCacheSeconds, &cacheSeconds );
				(void) PGPGetPrefBoolean( mPrefRef, kPGPPrefShareCache, &cacheGlobal );
			
				if( cacheType != kPGPPrefCacheTypeNone )
				{
					PGPPassphraseIsValid( state.signingKey,
						( state.signingPassphrase[0] != '\0' ) ?
							PGPOPassphrase( mContext, state.signingPassphrase ) :
							PGPOPasskeyBuffer( mContext, state.signingPasskey,
								state.signingPasskeySize ),
						PGPOCachePassphrase( mContext,
							( cacheType == kPGPPrefCacheTypeLogon ) ?
							kPGPMaxTimeInterval : cacheSeconds, cacheGlobal ),
						PGPOLastOption( mContext ) );
				}
			
				if( ! haveEncodingFlags )
				{
					state.encodingFlags	=
						PGPUISettingsToLocalEncoding( getPassphraseSettings );
					haveEncodingFlags	= TRUE;
				}
						
				theError = [self setupSigningOptions:getPassphraseSettings
							options:getPassphraseOptions optionList:state.encodeOptions];
			}
		}
		
		if( IsntPGPError( theError ) )
		{
			if( ( recipientSettings & kPGPUISettingsTextOutput ) != 0 ||
				( getPassphraseSettings & kPGPUISettingsTextOutput ) != 0 )
				state.textOutput = TRUE;
			
			if( (getPassphraseSettings &
					kPGPGetPassphraseSettingsDetachedSig ) != 0 )
			{
				pgpAssert( state.signing );
				state.detachedSignature = TRUE;
			}
			
			theError = [self setupCommonOptions:state.encodeOptions];
		}
		
		if( IsntPGPError( theError ) )
			theError = [self confirmationDialogs:&state];
		
		if( IsntPGPError( theError ) )
		{
			UInt32					numTasks = 0;
			
			numTasks = [mTaskList count];
			
			if( state.selfDecryptingArchive )
                NSRunAlertPanel(kErrorStr, kOperationNotSupported, kOKStr, nil, nil);
			
			if( numTasks != 0 )
			{
                UInt32	taskIndex = 0;
                
				for( taskIndex = 0; taskIndex < numTasks; taskIndex++ )
				{
                    mSource = [mTaskList objectAtIndex:taskIndex];
                    
                    [job setFile:[mFileManager displayNameAtPath:mSource]];
                    [mProgressDialog updateProgressForJob:job];
                    [mFileManager fileExistsAtPath:mSource isDirectory:&mSourceIsFolder];
					theError = [self encryptSign:&state];
					if( IsPGPError( theError ) )
						break;
				}
 			}
        }

		if( IsPGPError(theError ) && (theError != kPGPError_UserAbort) )
		{
			NSString*	errorStr;
                        
			[self getErrorString:theError value:&errorStr];
			
			NSBeep();
            NSRunAlertPanel(kOperationCouldNotBeCompletedStr, errorStr, kOKStr, nil, nil);
            theError = kPGPError_UserAbort;
		}
        else if( 0 < [mWipeTasks count] )
        {
            if (job)
            {
                [mProgressDialog finishedJob:job];
                [job release];
                job = nil;
            }
            
            [mAppController performSelectorOnMainThread:@selector(wipeFiles:) withObject:mWipeTasks waitUntilDone:NO];
        }
        PGPmailCleanupEncodeDecodeParams( &state );
	}	
    
    if (job)
    {
        [mProgressDialog finishedJob:job];
        [job release];
    }
    
	if( PGPKeyDBRefIsValid( recipients ) )
		PGPFreeKeyDB( recipients );
}

//____________________________________________________________________________
- (void)processDecryptVerify
{
	PGPError					theError = kPGPError_NoErr;
	UInt32						numTasks;
	UInt32 						taskIndex;
    PGPmailEncodeDecodeParams	state;
    CPGPmailProgressJob*		job = nil;
    NSString*					filename;
    
	pgpAssert( mOperation == kPGPtoolsDecryptVerifyOperation );
        
    if ([mTaskList count] != 0)
    {
        if ([mTaskList count] > 1)
            filename = [NSString stringWithFormat:@"%@, ", [mFileManager displayNameAtPath:[mTaskList objectAtIndex:0]]];
        else
            filename = [mFileManager displayNameAtPath:[mTaskList objectAtIndex:0]];
        
        job = [[CPGPmailProgressJob alloc] initWithFile:filename operation:mOperation selector:@selector(stopProcessing) target:self];
        [mProgressDialog startProgressForJob:job];
    }
    
    PGPmailInitEncodeDecodeParams( &state );
    
    state.allKeys			= mAllKeys;
    
	numTasks = [mTaskList count];
    
	for( taskIndex = 0; taskIndex < numTasks; taskIndex++ )
	{
        mSource = [mTaskList objectAtIndex:taskIndex];
        [job setFile:[mFileManager displayNameAtPath:mSource]];
        [mProgressDialog updateProgressForJob:job];
        [mFileManager fileExistsAtPath:mSource isDirectory:&mSourceIsFolder];
		theError = [self decryptVerify:&state];
		if( IsPGPError( theError ) )
			break;
	}

	if( IsPGPError( theError ) && ( theError != kPGPError_UserAbort ) )
	{
		NSString*	errorStr = nil;
        
		[self getErrorString:theError value:&errorStr];
		NSBeep();
        NSRunAlertPanel(kErrorStr, errorStr, kOKStr, nil, nil);
        theError = kPGPError_UserAbort;
	}
    
    PGPmailCleanupEncodeDecodeParams( &state );
    
    if (job)
    {
        [mProgressDialog finishedJob:job];
        [job release];
    }
}

//____________________________________________________________________________
// Check to see if the user has selected the macbinary options correctly and
// warn against problems.
//____________________________________________________________________________
- (PGPError)setMacBinaryOption:
    (PGPmailEncodeDecodeParams *)params
{
    PGPError		theError = kPGPError_NoErr;
    
    if( !params->detachedSignature )
    {
        ProcessingInfo	fileInfo;
        [self getProcessingInfo:mSource info:&fileInfo];
        
        switch( params->encodingFlags )
        {
            case kPGPLocalEncoding_None:
            default:
                params->encodingFlags = kPGPLocalEncoding_None;
            break;
            case kPGPLocalEncoding_Auto:
                if( fileInfo.resSize == 0 ) // No resource fork
                    params->encodingFlags = kPGPLocalEncoding_None;
                else if( fileInfo.resSize > 0 && fileInfo.resSize < kPGPmailMacBinaryFileLimit )
                    params->encodingFlags = kPGPLocalEncoding_Force;
                else
                {
                    NSRunAlertPanel(LSPM(@"MacBinary Warning", @""),
                        kMacBinaryFileSizeTooLargeStr, kOKStr, nil, nil);
                    theError = kPGPError_UserAbort;
                    // size larger than 2G display warning
                }
            break;
            case kPGPLocalEncoding_Force:
                if( fileInfo.resSize > kPGPmailMacBinaryFileLimit )
                {
                    NSRunAlertPanel(LSPM(@"MacBinary Warning", @""),
                        kMacBinaryFileSizeTooLargeStr, kOKStr, nil, nil);
                    theError = kPGPError_UserAbort;
                }
            break;
        }
    }
    
    return( theError );
}

//____________________________________________________________________________
//	The user has requested a wipe operation either directly via the wipe
//	command or indirectly by encrypting or signing.
//____________________________________________________________________________
- (PGPError)confirmWipe:
    (BOOL)encrypting
    signing:(BOOL)signing
{
	PGPError	theError = kPGPError_NoErr;
	int			alertResult;
	NSString*	str;
			
	if( [mTaskList count] > 1 )
	{
		[self getConfirmWipeStr:encrypting sign:signing multiple:YES string:&str];
        alertResult = NSRunAlertPanel(str, kCannotUndoStr, kOKStr, kCancelStr, nil);
	}
	else
	{
		NSString*			itemName;
		
		itemName = [mTaskList objectAtIndex:0];

		[self getConfirmWipeStr:encrypting sign:signing multiple:NO string:&str];
		
        alertResult = NSRunAlertPanel(str, kCannotUndoStr, kOKStr, kCancelStr, nil);
	}
	
	theError	= (alertResult == NSAlertDefaultReturn) ? kPGPError_NoErr :kPGPError_UserAbort;
    
	return( theError );
}

//____________________________________________________________________________
- (void)getConfirmWipeStr:(BOOL)encrypting
    sign:(BOOL)signing
	multiple:(BOOL)multipleFiles
    string:(NSString **)str
{
    NSString			*error = nil;
    
	if( encrypting && signing )
		error = multipleFiles?kConfirmWipingItemsAfterEncryptingSigningStr:kConfirmWipingItemAfterEncryptingSigningStr;
	else if( encrypting )
		error = multipleFiles?kConfirmWipingItemsAfterEncryptingStr:kConfirmWipingItemAfterEncryptingSigningStr;
	else if( signing )
		error = multipleFiles?kConfirmWipingItemsAfterSigningStr:kConfirmWipingItemAfterSigningStr;
	else
		error = multipleFiles?kConfirmWipingItemsStr:kConfirmWipingItemStr;
    
   *str = multipleFiles ? error : 
        [[error stringByAppendingString:
            [[mTaskList objectAtIndex:0] lastPathComponent]] stringByAppendingString:@"?"];    
}

//____________________________________________________________________________
//	Confirm that the user doesn't want to encrypt to himself.
//	Should be called if the originals are being wiped.
//____________________________________________________________________________
- (PGPError)confirmNotEncryptingToSelf
{
	PGPError		theError = kPGPError_NoErr;
	int				alertResult;
	
    alertResult = NSRunAlertPanel(LSPM(@"Encryption Warning", @""),
                        kConfirmNotEncryptingToSelfStr, kOKStr, kCancelStr, nil);

	theError = (alertResult == NSAlertDefaultReturn ) ? kPGPError_NoErr : kPGPError_UserAbort;
	
	return( theError );
}

//____________________________________________________________________________
- (void)processWipe
{
	PGPError		theError = kPGPError_NoErr;
	
	pgpAssert( mOperation == kPGPmailWipeOperation );
    
	if( mTaskPrefs.warnOnWipe )
		theError = [self confirmWipe:FALSE signing:FALSE];
	if( IsntPGPError( theError ) )
	{
        CPGPmailWipeFile*		wiper = [[CPGPmailWipeFile alloc] initWithController:mAppController];
        
        [wiper processFiles:mWipeTasks];
        
        [wiper release];
    }
}

//____________________________________________________________________________
- (void)process
{
	PGPError 		theError = kPGPError_NoErr;

	if( mOperation == kPGPmailWipeFreeSpaceOperation )
	{
        NSRunAlertPanel(kErrorStr, kOperationNotSupported, kOKStr, nil, nil);
    }
	else
	{
		ByteCount				progressBytes;
        
		theError = [self calcProgressBytes:&progressBytes];
        
        [self setCurrentPrefs];
        
		if( IsntPGPError( theError ) )
		{
			if( mOperation != kPGPmailWipeOperation )
			{
                if( IsntPGPError( theError ) )
                {
                    switch( mOperation )
                    {
                        case kPGPmailEncryptOperation:
                        case kPGPmailSignOperation:
                        case kPGPmailEncryptSignOperation:
                            [self processEncryptSign];
                            break;
                            
                        case kPGPmailDecryptVerifyOperation:
                            [self processDecryptVerify];
                            break;
                            
                        default:
                            pgpDebugMsg( "process: Invalid operation" );
                            break;
                    }
                }
			}
			else
			{
                if( 0 < [mWipeTasks count] )
                {
                    mOperation = kPGPmailWipeOperation;
                    [self processWipe];
                }
			}
		}
    }
}

//____________________________________________________________________________
- (void)stopProcessing
{
    sPGPmailEncodeDecodeStop = YES;
}

//____________________________________________________________________________
- (PGPError)genericEventHandler:
    (PGPEvent *)event
    data:(PGPmailEventHandlerData *)data
{
	PGPError	theError	= kPGPError_NoErr;
	
	if( event->type == kPGPEvent_NullEvent )
		theError	= [self nullEvent:event data:data];
        
	return( theError );
}

//____________________________________________________________________________
- (PGPError)encodeEventHandler:
    (PGPEvent *)event
    data:(PGPmailEventHandlerData *)data
{
	PGPError	theError	= kPGPError_NoErr;
	
    switch( event->type )
    {
		default:
		{
            theError	= [self genericEventHandler:event data:data];
			break;
		}
		
		case kPGPEvent_InitialEvent:
        case kPGPEvent_FinalEvent:
        {
            theError = [self nullEvent:event data:data];
            break;
        }
    }
	
	return( theError );
}

//____________________________________________________________________________
- (PGPError)nullEvent:
    (PGPEvent *)event
    data:(PGPmailEventHandlerData *)data
{
	PGPError	theError	= kPGPError_NoErr;
    
	if( IsntNull( mProgressDialog ) )
	{
		PGPSize		bytesProcessed;
		PGPSize		totalBytes;
		
		bytesProcessed	= event->data.nullData.bytesWritten;
		totalBytes		= event->data.nullData.bytesTotal;
		
		[mProgressDialog doProgress:bytesProcessed total:totalBytes];
	}
    
	return( theError );
}

//____________________________________________________________________________
- (PGPError)decodePassphraseEvent:
    (PGPEvent *)event
    data:(PGPmailEventHandlerData *)data
{
	PGPError			theError = kPGPError_NoErr;
    NSMutableString*	tmpString = [NSMutableString string];
    
	if( mBadPassphraseCount > 0 )
	{
        NSString*  temp;
        
		NSBeep();

        [tmpString appendString:kIncorrectPassphrasePrefixStr];
		[self getDecryptPassphrasePromptString:&temp];
        [tmpString appendString:temp];
	}
	else
 		[self getDecryptPassphrasePromptString:&tmpString];

	(void)strcpy( data->clientHandlerData.prompt, [tmpString UTF8String] );
	
    theError = PGPclDecodeEventHandler( mContext, event, &data->clientHandlerData );
		
	++mBadPassphraseCount;

	return( theError );
}

//____________________________________________________________________________
- (PGPError)decodeRecipientsEvent:
    (PGPEvent *)event
    data:(PGPmailEventHandlerData *)data
{
	PGPError	theError = kPGPError_NoErr;
	
	theError = PGPclDecodeEventHandler( mContext, event, &data->clientHandlerData );
	
	return( theError );
}

//____________________________________________________________________________
- (PGPError)decodeSignatureEvent:
    (PGPEvent *)event
    data:(PGPmailEventHandlerData *)data
{
	PGPError	theError = kPGPError_NoErr;

	theError = PGPclDecodeEventHandler( mContext, event, &data->clientHandlerData );
	if( IsntPGPError( theError ) )
	{
		mSignatureData.data = data->clientHandlerData.signatureData;
		mSignatureDataValid = data->clientHandlerData.signatureDataValid;
		
		mSignatureData.signingKeyIDStr[0] = 0;
		
		if( mSignatureDataValid )
			(void)PGPGetKeyIDString( 
					&mSignatureData.data.signingKeyID,
					kPGPKeyIDString_Abbreviated,
					mSignatureData.signingKeyIDStr );
	}

	return( theError );
}

//____________________________________________________________________________
- (PGPError)decodeAnalyzeEvent:
    (PGPEvent *)event
    data:(PGPmailEventHandlerData *)data
{
	(void) data;
	
	if( mDataType == kPGPAnalyze_Unknown &&
            event->data.analyzeData.sectionType != kPGPAnalyze_Unknown &&
            event->data.analyzeData.sectionType != kPGPAnalyze_Key )
		mDataType = event->data.analyzeData.sectionType;
	
	return( kPGPError_NoErr );
}

//____________________________________________________________________________
- (PGPError)decodeEventHandler:
    (PGPEvent *)event
    data:(PGPmailEventHandlerData *)data
{
	PGPError				theError = kPGPError_NoErr;
    PGPError				clientHandlerErr = kPGPError_NoErr;
	
	switch( event->type )
	{
		default:
		{
			theError = PGPclDecodeEventHandler( mContext, event, &data->clientHandlerData );
			if( IsntPGPError( theError ) )
				theError	= [self genericEventHandler:event data:data];
			
			break;
		}
		
		case kPGPEvent_InitialEvent:
		{
			mBadPassphraseCount = 0;
			mDataType			= kPGPAnalyze_Unknown;
			break;
		}

		case kPGPEvent_AnalyzeEvent:
		{
			theError = [self decodeAnalyzeEvent:event data:data];
			break;
		}

		case kPGPEvent_PassphraseEvent:
		{
			theError = [self decodePassphraseEvent:event data:data];
			break;
		}

		case kPGPEvent_RecipientsEvent:
		{
			theError = [self decodeRecipientsEvent:event data:data];
			break;
		}
		
		case kPGPEvent_SignatureEvent:
		{
			theError = [self decodeSignatureEvent:event data:data];
			break;
		}

		case kPGPEvent_FinalEvent:
		{
            theError = [self nullEvent:event data:data];
			break;
		}
		
		case kPGPEvent_OutputEvent:
		{
			if( event->data.outputData.forYourEyesOnly )
			{
				data->isFYEO = true;
	           
                theError = PGPclDecodeEventHandler( mContext, event, &data->clientHandlerData );
    
                if( IsntPGPError( theError ) )
                    theError = [self genericEventHandler:event data:data];
            }
            else
                [self decodeOutputEvent:event data:data];
 
			break;
		}
		
		case  kPGPEvent_DetachedSignatureEvent:
		{
			theError = [self decodeDetachedSignatureEvent:event data:data];
			break;
		}

		case kPGPEvent_EndLexEvent:
		{
			if( data->clientHandlerData.outputBufferSize != 0 )
			{
				if( data->outputDataSize == 0 )
				{
					data->outputDataSize = data->clientHandlerData.outputBufferSize;
					data->outputData = PGPNewSecureData(
											PGPPeekContextMemoryMgr( mContext ),
											data->outputDataSize + 1,
											kPGPMemoryMgrFlags_None );
					if( IsNull( data->outputData ) )
						theError = kPGPError_OutOfMemory;
					pgpCopyMemory( data->clientHandlerData.outputBuffer,
								   data->outputData,
								   data->outputDataSize );
					( (char *) data->outputData )[data->outputDataSize] = 0;
				}
				else
				{
					PGPSize	newSize = data->outputDataSize +
								data->clientHandlerData.outputBufferSize;
					
					theError = PGPReallocData( PGPPeekContextMemoryMgr( mContext ),
										  &data->outputData,
										  newSize + 1,
										  kPGPMemoryMgrFlags_None );
					if( IsntPGPError( theError ) )
					{
						pgpCopyMemory( data->clientHandlerData.outputBuffer,
									   ( (char *) data->outputData) + data->outputDataSize,
									   data->clientHandlerData.outputBufferSize );
						( (char *) data->outputData )[newSize] = 0;
						data->outputDataSize = newSize;
					}
				}
			}
			
			
			clientHandlerErr = PGPclDecodeEventHandler( mContext, event, &data->clientHandlerData );
			if( IsntPGPError( clientHandlerErr ) )
				clientHandlerErr	= [self genericEventHandler:event data:data];
			
			if( IsntPGPError( theError ) )
				theError = clientHandlerErr;
			
			break;
		}
	}
	
	return( theError );
}

//____________________________________________________________________________
	PGPError
sEncodeEventHandlerProc(
    PGPContextRef		context,
	PGPEvent*			event,
   	PGPUserValue		userValue)
{
	PGPError	theError = kPGPError_NoErr;
	
	if( sPGPmailEncodeDecodeStop )
    {
        theError = kPGPError_UserAbort;
        sPGPmailEncodeDecodeStop = NO;	/* reset */
    }
    
    if( IsntPGPError( theError ) )
    {
        PGPmailEventHandlerData		*data = (PGPmailEventHandlerData *)userValue;
        
        pgpAssert( IsntNull( data )  );
    
        theError = [data->mTask encodeEventHandler:event data:data];
    }
    return( theError );
}

//____________________________________________________________________________
	PGPError
sDecodeEventHandlerProc(
    PGPContextRef		context,
	PGPEvent*			event,
	PGPUserValue		userValue)
{
	PGPError	theError = kPGPError_NoErr;
	
	if( sPGPmailEncodeDecodeStop )
    {
        theError = kPGPError_UserAbort;
        sPGPmailEncodeDecodeStop = NO; /* reset */
    }
    
    if( IsntPGPError( theError ) )
    {
        PGPmailEventHandlerData		*data = (PGPmailEventHandlerData *)userValue;
        
        pgpAssert( IsntNull( data )  );
	
        theError = [data->mTask decodeEventHandler:event data:data];
	}
	return( theError );
}

//____________________________________________________________________________
- (PGPError)encryptSignFileInternal:
    (PGPmailEncodeDecodeParams *)params
    withInfo:(const ProcessingInfo *)srcInfo
    dest:(NSString *)dest
{
	PGPError				theError = kPGPError_NoErr;
	PGPOptionListRef		signingOptions	= kInvalidPGPOptionListRef;
	PGPBoolean				clearSign 	= FALSE;
	PGPBoolean				armorOutput = FALSE;
	PGPLocalEncodingFlags	encodingFlags;
		
	pgpAssert( PGPContextRefIsValid( mContext ) );
	pgpAssertAddrValid( params, PGPmailEncodeDecodeParams );
        
	encodingFlags = params->encodingFlags;
	if( encodingFlags == kPGPLocalEncoding_None && srcInfo->dataSize == 0 && srcInfo->resSize != 0 )
		// MacBinary is off, but the file is a resource fork-only
		// file. This case is illogical. Turn on MacBinary for this file.
		encodingFlags = kPGPLocalEncoding_Force;
	
	if( params->textOutput )
	{
		armorOutput = TRUE;
		
		if( srcInfo->typeCode == kPGPMacFileType_Text &&
			params->signing &&
			! params->encrypting &&
			encodingFlags != kPGPLocalEncoding_Force &&
			! params->detachedSignature )
		{
			// Clear sign TEXT files in the non-MacBinary,
			// non-detached sig case
			
			clearSign 	= TRUE;
			armorOutput = FALSE;
		}
	}
	
	if( params->signing )
	{
		theError = PGPBuildOptionList( mContext, &signingOptions, 
				PGPOSignWithKey( mContext, params->signingKey,
					params->signingWithSplitKey ?
						PGPOPasskeyBuffer( mContext, params->signingPasskey,
							 params->signingPasskeySize ) :
						PGPOPassphrase( mContext, params->signingPassphrase ),
					PGPOLastOption( mContext ) ),
				PGPOClearSign( mContext, clearSign ),
				PGPOLastOption( mContext ) );
		
		if( IsntPGPError( theError ) && params->detachedSignature )
			theError = PGPAppendOptionList( signingOptions,
						PGPODetachedSig( mContext,
							PGPOLastOption( mContext ) ),
						PGPOLastOption( mContext ) );
			// text output should have alreday been set up
	}
	else
	{
		// no signing options; create dummy list
		theError	= PGPBuildOptionList( mContext, &signingOptions, PGPOLastOption( mContext ) );
	}
	
	if( IsntPGPError( theError ) )
		theError = PGPclGuaranteeMinimumEntropy( mContext );
	
    if (IsntPGPError(theError)) {
        NSString *			destDir;
        NSString *			destName;
        NSFileManager *		fileMgr = [NSFileManager defaultManager];
        NSSavePanel *		sp = [NSSavePanel savePanel];
        int					runResult;
        
        destDir = [dest stringByDeletingLastPathComponent];
        destName = [dest lastPathComponent];
        while (! [fileMgr isWritableFileAtPath:destDir]) {
            NSRunAlertPanel(kUnableToSave, kDirectoryNotWritable, nil, nil, nil);
            runResult = [sp runModalForDirectory:destDir file:destName];
            if (runResult == NSFileHandlingPanelCancelButton) {
                theError = kPGPError_UserAbort;
                break;
            } else {
                dest = [sp filename];
                destDir = [dest stringByDeletingLastPathComponent];
                destName = [dest lastPathComponent];
            }
        }
    }
    
	if( IsntPGPError( theError ) )
	{
		PGPmailEventHandlerData		data;
        PGPFileSpecRef				inputFile = kInvalidPGPFileSpecRef;
        PGPFileSpecRef				outputFile = kInvalidPGPFileSpecRef;

        theError = PGPmailInitEventHandlerData( mContext, mTLSContext, mPrefRef, self, *params, &data );
        theError = PGPNewFileSpecFromFullPath( mContext, [srcInfo->src UTF8String], &inputFile );
        theError = PGPNewFileSpecFromFullPath( mContext, [dest UTF8String], &outputFile );
        
		theError = PGPEncode( mContext,
					PGPOInputFile( mContext, inputFile ),
					PGPOOutputFile( mContext, outputFile ),
					PGPODataIsASCII( mContext, srcInfo->typeCode == kPGPMacFileType_Text ),
					PGPOArmorOutput( mContext, armorOutput ),
					signingOptions,
					PGPOptionListRefIsValid( params->encodeOptions ) ?
						params->encodeOptions : PGPONullOption( mContext ),
					PGPOEventHandler( mContext,
						sEncodeEventHandlerProc, &data ),
					PGPOSendNullEvents( mContext, TRUE ),
					PGPOLocalEncoding( mContext, encodingFlags ),
					PGPOForYourEyesOnly( mContext, params->fyeo ),
					PGPOLastOption( mContext  ) );
        
        if( PGPFileSpecRefIsValid( inputFile ) )
        {
            PGPFreeFileSpec( inputFile );
            inputFile = kInvalidPGPFileSpecRef;
        }
        
        if( PGPFileSpecRefIsValid( outputFile ) )
        {
            PGPFreeFileSpec( outputFile );
            outputFile = kInvalidPGPFileSpecRef;
        }
        PGPmailCleanupEventHandlerData( &data );
	}

	if( PGPOptionListRefIsValid( signingOptions ) )
		PGPFreeOptionList( signingOptions );

    if( IsPGPError( theError ) && (theError != kPGPError_UserAbort) )
    {
        NSBeep();
        NSRunAlertPanel( kWarningStr, 
            params->encrypting ? kFileCannotBeEncryptedStr :
			kFileCannotBeSignedStr, kOKStr, nil, nil);
        theError = kPGPError_UserAbort;
    }
	return( theError );
}

//____________________________________________________________________________
- (PGPError)encryptSign:(PGPmailEncodeDecodeParams*)params
{
    NSString*		dest = nil;
    NSString*		prompt = nil;
    NSSavePanel*	sp = nil;
    ProcessingInfo	info;
    PGPError		err = kPGPError_NoErr;
    int				result = 0;
    
	pgpAssert((!params->wipeOriginal) || params->encrypting);
	
	/* Send output to a duplicate folder hierarchy unless we are creating detached sigs */
	mProcessInPlace	= params->detachedSignature;
    
	if (params->wipeOriginal)
        [mWipeTasks addObject:mSource];
    
	if (mSourceIsFolder)
		err = [self encryptSignFolder:params];
	else
	{
		[self getProcessingInfo:mSource info:&info];
        
        mWorkingFile = info.src;
		GetDefaultEncryptSignName(info.src, params->textOutput, params->detachedSignature, &dest);
        
		if (NSAlternateKeyMask & mModifierFlags)
		{
			if ((params->encrypting && params->signing) || params->detachedSignature)
				prompt = kSaveAsPromptStr;
			else if (params->encrypting)
				prompt = kSaveEncryptedFileAsPromptStr;
			else
				prompt = kSaveSignedFileAsPromptStr;
			
            sp = [NSSavePanel savePanel];
            
            [sp setRequiredFileType:[dest pathExtension]];
            [sp setCanSelectHiddenExtension:YES];
            
            result = [sp runModalForDirectory:[dest stringByDeletingLastPathComponent] file:[dest lastPathComponent]];
            
			if (result == NSOKButton)
				dest = [sp filename];
			else
				err = kPGPError_UserAbort;
		}
		else
			GetUniqueFileName(dest, &dest);
		
		if (IsntPGPError(err))
			err = [self encryptSignFileInternal:params withInfo:&info dest:dest];
	}
    
	return err;
}

//____________________________________________________________________________
- (PGPError)encryptSignFolder:
    (PGPmailEncodeDecodeParams *)params
{
	PGPError				theError = kPGPError_NoErr;
	NSString				*destTree = nil;
	NSDirectoryEnumerator	*theEnumerator = nil;
    
	(void)params;
	
	if( IsntPGPError( theError ) && !mProcessInPlace)
	{
		theError = [self getOutputFolderName:TRUE src:mSource dest:&destTree];
		if( IsntPGPError( theError ) )
            [mFileManager copyPath:mSource toPath:destTree handler:nil];
	}
    else
        destTree = mSource;

    theEnumerator = [mFileManager enumeratorAtPath:destTree];
    
	if( IsntPGPError( theError ) )
	{
        BOOL			isDir = NO;
        
        while( mSource = [theEnumerator nextObject] )
        {
            mWorkingFile = [destTree stringByAppendingPathComponent:mSource];
            
            if( [mFileManager fileExistsAtPath:mWorkingFile isDirectory:&isDir] && !isDir )
            {
                ProcessingInfo	info;
                NSString		*destFile = nil;
			
                [self getProcessingInfo:mWorkingFile info:&info];
                
                if( [self shouldEncryptSignFile:params->encrypting file:info.src] )
                {
                    GetDefaultEncryptSignName( info.src, params->textOutput, params->detachedSignature, &destFile );
                    [mProgressDialog setFileName:[mFileManager displayNameAtPath:info.src]];
                    theError = [self encryptSignFileInternal:params withInfo:&info dest:destFile];
                    if( IsntPGPError(theError)  && !mProcessInPlace )
                        [mFileManager removeFileAtPath:info.src handler:nil];
                }
            }
            
            isDir = NO;
            if( IsPGPError( theError ) )
				break;
        }
    }
                
    return( theError );
}

/*____________________________________________________________________________
	Determine if the file should be processed.
	
	While it might make sense to not encrypt already encrypted items or
	encrypt detached sigs, there are also cases where you might want
	to do this. so don't prevent it.
	
	The same idea applies when signing already signed or encrypted items.
____________________________________________________________________________*/
- (BOOL)shouldEncryptSignFile:
    (BOOL)encrypting
    file:(NSString *)file
{
	BOOL	shouldProcess = NO;
    
    if( ![[file lastPathComponent] isEqualToString:kFileDisplayServerStore] )
    {
        NSDictionary*		fattr = [mFileManager fileAttributesAtPath:file traverseLink:NO];
                
        if( nil != fattr)
        {
            // Don't do sockets, pipes, links, etc.
            if( [[fattr fileType] isEqualToString:NSFileTypeRegular] ||
                [[fattr fileType] isEqualToString:NSFileTypeSymbolicLink] )
                shouldProcess = YES;
        }
	}
    
	return( shouldProcess );
}

//____________________________________________________________________________
- (PGPError)getOutputFolderName:
    (PGPBoolean)encryptSign
    src:(NSString *)src
    dest:(NSString **)dest
{
	PGPError			theError = kPGPError_NoErr;
	NSString*			kPGPSuffix = @" PGP";			// Folder suffix
	NSMutableString*	newFile = [NSMutableString string];
    NSString *			destDir;
    NSString *			destName;
    NSFileManager *		fileMgr = [NSFileManager defaultManager];
    NSSavePanel *		sp = [NSSavePanel savePanel];
    int					runResult;
    
	[newFile setString:src];
	
	if( encryptSign )
	{
		if( ([[newFile lastPathComponent] length] + [kPGPSuffix length]) > kPGPMaxFileNameLength )
            [newFile setString:[src substringToIndex:[src length] - [kPGPSuffix length]]];            
		[newFile appendString:kPGPSuffix];
	}
	else
	{	// decrypting/verifying
		if( [src hasSuffix:kPGPSuffix] )
			// strip the ".pgp" off
            [newFile setString:[src substringToIndex:[src length] - [kPGPSuffix length]]];
	}
    destDir = [newFile stringByDeletingLastPathComponent];
    destName = [newFile lastPathComponent];
    while (! [fileMgr isWritableFileAtPath:destDir]) {
        NSRunAlertPanel(kUnableToSave, kDirectoryNotWritable, nil, nil, nil);
        runResult = [sp runModalForDirectory:destDir file:destName];
        if (runResult == NSFileHandlingPanelCancelButton) {
            theError = kPGPError_UserAbort;
            break;
        } else {
            newFile = [NSMutableString stringWithString:[sp filename]];
            destDir = [newFile stringByDeletingLastPathComponent];
            destName = [newFile lastPathComponent];
        }
    }
    if (IsntPGPError(theError)) {
        GetUniqueFileName( newFile, dest );
    }
	return( theError );
}

//____________________________________________________________________________
- (PGPError)decryptVerifyEncryptedFile:
    (PGPmailEncodeDecodeParams *)params
    info:(ProcessingInfo *)srcInfo
    dest:(NSString *)dest
{
	PGPmailEventHandlerData		data;
	PGPError					theError = kPGPError_NoErr;
	PGPKeyDBRef					newKeysDB = kInvalidPGPKeyDBRef;
    
	mWorkingFile = srcInfo->src;

    PGPmailInitEventHandlerData( mContext, mTLSContext, mPrefRef, self, *params, &data );
    
	// Assume both forks are verified
	mSignatureData.didNotVerifyResourceFork = FALSE;
	
	theError = PGPNewKeyDB( mContext, &newKeysDB );
	if( IsntPGPError( theError ) )
	{
		PGPOptionListRef		decodeOptions = kInvalidPGPOptionListRef;

		theError = PGPBuildOptionList( mContext, &decodeOptions,
						PGPOKeyDBRef( mContext, params->allKeys ),
						PGPOImportKeysTo( mContext, newKeysDB ),
						PGPOEventHandler( mContext, sDecodeEventHandlerProc,
							&data ),
						PGPOSendNullEvents( mContext, TRUE ),
						PGPOLastOption( mContext ) );
		if( IsntPGPError( theError ) )
		{
            PGPFileSpecRef		inputFile = kInvalidPGPFileSpecRef;
            PGPFileSpecRef		outputFile = kInvalidPGPFileSpecRef;
            
            if (! [srcInfo->src hasSuffix:kDetachedSigFileSuffixStr]) {
                NSString *			destDir;
                NSString *			destName;
                NSFileManager *		fileMgr = [NSFileManager defaultManager];
                NSSavePanel *		sp = [NSSavePanel savePanel];
                int					runResult;
                
                if (dest == nil) {
                    destDir = [srcInfo->src stringByDeletingLastPathComponent];
                    GetDefaultDecryptVerifyName([srcInfo->src lastPathComponent], &destName);
                } else {
                    destDir = [dest stringByDeletingLastPathComponent];
                    destName = [dest lastPathComponent];
                }
                while (! [fileMgr isWritableFileAtPath:destDir]) {
                    NSRunAlertPanel(kUnableToSave, kDirectoryNotWritable, nil, nil, nil);
                    runResult = [sp runModalForDirectory:destDir file:destName];
                    if (runResult == NSFileHandlingPanelCancelButton) {
                        theError = kPGPError_UserAbort;
                        break;
                    } else {
                        dest = [sp filename];
                        destDir = [dest stringByDeletingLastPathComponent];
                        destName = [dest lastPathComponent];
                    }
                }
            }
			if (IsntPGPError(theError)) {
				theError = PGPNewFileSpecFromFullPath( mContext, [srcInfo->src UTF8String], &inputFile );
				if( dest != nil && IsntPGPError(theError))
					theError = PGPNewFileSpecFromFullPath( mContext, [dest UTF8String], &outputFile );
			}
			if (IsntPGPError(theError)) {
				theError = PGPDecode( mContext,
						PGPOInputFile( mContext, inputFile ),
						IsntNull( outputFile ) ?
							PGPOOutputFile( mContext, outputFile ) :
							PGPONullOption( mContext ),
						decodeOptions,
						PGPOLastOption( mContext ) );
			}
			
			if( IsntPGPError( theError ) && mSignatureDataValid && mSignatureData.data.checked &&
                 ( !mSignatureData.data.verified ) )
			{
				/*
				sig checked, but didn't verify.  Output should have been
				created. Try again with no output, existing passphrase,
				and local encoding off (controlled by 
				data.retryingSigVerification ).
				*/
                PGPFileSpecRef		sigFile = kInvalidPGPFileSpecRef;

                theError = PGPNewFileSpecFromFullPath( mContext, [srcInfo->src UTF8String], &sigFile );

				data.retryingSigVerification			= TRUE;
				mSignatureData.didNotVerifyResourceFork = TRUE;
				mSignatureDataValid						= FALSE;
				
				theError = PGPDecode( mContext,
						PGPOInputFile( mContext, sigFile ),
						PGPODiscardOutput( mContext, TRUE ),
						decodeOptions,
						PGPOLastOption( mContext ) );
						
				pgpAssert( mSignatureDataValid );
                
                if( PGPFileSpecRefIsValid( inputFile ) )
                {
                    PGPFreeFileSpec( inputFile );
                    inputFile = kInvalidPGPFileSpecRef;
                }
			}
			
			if( IsntPGPError( theError ) && ( data.outputData != NULL ) )
			{
				theError = PGPclForYourEyesOnlyDialog( mContext, mPrefRef, (char *) data.outputData );
				pgpClearMemory( data.outputData, data.outputDataSize );
				PGPFreeData( data.outputData );
				data.outputData = NULL;
			}
			
			if( IsntPGPError( theError ) && PGPKeyDBRefIsValid( data.clientHandlerData.newKeys ) )
				theError = PGPCopyKeys( PGPPeekKeyDBRootKeySet(data.clientHandlerData.newKeys ), newKeysDB, NULL );
			
			if( IsntPGPError( theError ) )
			{
				PGPUInt32	numNewKeys;
			
				theError = PGPCountKeysInKeyDB( newKeysDB, &numNewKeys );
				if( IsntPGPError( theError ) && numNewKeys > 0 )
				{
					// Offer to import the keys
                    PGPKeySetRef			importSet;
					PGPKeySetRef*			importSetRef = &importSet;
                    NSMutableDictionary*	args = [[NSMutableDictionary alloc] init];
                    
                    [args setObject:[NSData dataWithBytes:&newKeysDB length:sizeof(PGPKeyDBRef)] forKey:@"PGPNewKeysDB"];
                    [args setObject:[NSData dataWithBytes:&params->allKeys length:sizeof(PGPKeyDBRef)] forKey:@"PGPValidityKeysDB"];
                    [args setObject:[NSData dataWithBytes:&importSetRef length:sizeof(PGPKeySetRef)] forKey:@"PGPImportedKeySet"];
                    
                    [self performSelectorOnMainThread:@selector(runKeySelectDialog:) withObject:args waitUntilDone:YES];
                    
                    theError = *((PGPError*) [(NSData*) [args objectForKey:@"PGPError"] bytes]);
                    
                    [args release];
				}
			}
            
            if( PGPFileSpecRefIsValid( mOutputFile ) )
            {
                MacBinaryConversion( mOutputFile );
                
                PGPFreeFileSpec( mOutputFile );
                mOutputFile = kInvalidPGPFileSpecRef;
            }
           
            if( PGPFileSpecRefIsValid( inputFile ) )
            {
                PGPFreeFileSpec( inputFile );
                inputFile = kInvalidPGPFileSpecRef;
            }
            
            if( PGPFileSpecRefIsValid( outputFile ) )
            {
                PGPFreeFileSpec( outputFile );
                outputFile = kInvalidPGPFileSpecRef;
            }

			PGPFreeOptionList( decodeOptions );
		}
		
		PGPFreeKeyDB( newKeysDB );
	}

    PGPmailCleanupEventHandlerData( &data );
	return( theError );
}

- (void)runKeySelectDialog:(NSMutableDictionary*)args
{
    PGPError			err;
    PGPKeyDBRef			newKeysDB;
    PGPKeyDBRef			validityKeyDB;
    PGPKeySetRef*		importSet;
    
    newKeysDB = *((PGPKeyDBRef*) [(NSData*) [args objectForKey:@"PGPNewKeysDB"] bytes]);
    validityKeyDB = *((PGPKeyDBRef*) [(NSData*) [args objectForKey:@"PGPValidityKeysDB"] bytes]);
    importSet = *((PGPKeySetRef**) [(NSData*) [args objectForKey:@"PGPImportedKeySet"] bytes]);
    
    err = PGPclSelectKeysDialog(mContext,
                mPrefRef,
                kPGPclKeySelect_ImportVariation,
                [kSelectiveImportFoundKeysPromptStr UTF8String],
                newKeysDB,
                validityKeyDB,
                importSet);
    
    if (IsntPGPError(err))
    {
        err = PGPCopyKeys(*importSet, validityKeyDB, NULL);
        PGPFreeKeySet(*importSet);
    }
    
    [args setObject:[NSData dataWithBytes:&err length:sizeof(PGPError)] forKey:@"PGPError"];
}

//____________________________________________________________________________
- (PGPError)decryptKeyShareFile:
    (PGPmailEncodeDecodeParams *)params
    info:(ProcessingInfo *)srcInfo
    dest:(NSString *)dest
{
	PGPError			theError = kPGPError_NoErr;
	NSMutableString*	message = [NSMutableString string];
	int					dialogResult;
    
	[message setString:kDecryptKeyShareFileMsgStr];
	[message appendString:@" "];
    [message appendString:srcInfo->src];
	
    dialogResult = mSourceIsFolder ?
            NSRunAlertPanel(kAlertStr, message, kOKStr, nil, nil) :
            NSRunAlertPanel(kAlertStr, message, kOKStr, kCancelStr, nil);
	
	if( dialogResult == NSOKButton )
	{
		NSString		*shareFile;
			
		if( mSourceIsFolder )
		{
			// Must copy the file first as the conversion is done in place.
			BOOL	 isError;

			pgpAssert( dest != nil );
            
            isError = [mFileManager copyPath:srcInfo->src toPath:dest handler:nil];
            theError = isError ? kPGPError_NoErr : kPGPError_FileOpFailed;
            				
			shareFile = dest;
		}
		else
			shareFile = srcInfo->src;
        
		mWorkingFile = shareFile;
        
		if( IsntPGPError( theError ) )
		{
			PFLFileSpecRef	shareFileSpecRef;

			theError = PFLNewFileSpecFromFullPath(
								PGPPeekContextMemoryMgr( mContext ),
								[shareFile UTF8String], &shareFileSpecRef );

			if( IsntPGPError( theError ) )
			{
				PGPShareFileRef	shareFileRef;
				
				theError = PGPOpenShareFile( shareFileSpecRef, &shareFileRef );
				if( IsntPGPError( theError ) )
				{
					PGPOptionListRef			optionsDecode = kInvalidPGPOptionListRef;
					PGPmailEventHandlerData		data;
					PGPShareRef					shares = kInvalidPGPShareRef;
                    
                    theError = PGPmailInitEventHandlerData(
                            mContext, mTLSContext, mPrefRef, self, *params, &data );

					// decrypt specified share file
					theError = PGPBuildOptionList( mContext, &optionsDecode,
										PGPOKeyDBRef( mContext,
											params->allKeys ),
										PGPOEventHandler( mContext,
											sDecodeEventHandlerProc, &data ),
										PGPOLastOption( mContext ) );
					if( IsntPGPError( theError ) )
						theError = PGPCopySharesFromFile( mContext,
											shareFileRef, optionsDecode, &shares );
					
					if( IsntPGPError( theError ) )
					{
						PGPRecipientSettings	recipientSettings;
						PGPKeyDBRef				recipients = kInvalidPGPKeyDBRef;
						PGPKeyDBRef				newKeys = kInvalidPGPKeyDBRef;
						PGPOptionListRef		optionsEncode = kInvalidPGPOptionListRef;
							
						theError = PGPclRecipientDialog( mContext, mPrefRef,
										mTLSContext, params->allKeys, 0, nil,
										kPGPRecipientOptionsHideFileOptions,
										kPGPRecipientSettingsDefault,
										&recipientSettings, &recipients,
										&newKeys );

						if( IsntPGPError( theError ) && PGPKeyDBRefIsValid( newKeys ) )
						{
							PGPKeySetRef	importSet;
							
							theError = PGPclSelectKeysDialog( mContext, mPrefRef,
									kPGPclKeySelect_ImportVariation,
									[kSelectiveImportFoundKeysPromptStr UTF8String],
                                    newKeys, params->allKeys, &importSet );
							if( IsntPGPError( theError ) )
							{								
                                theError = PGPCopyKeys( importSet, params->allKeys, NULL );
                                            								
								(void) PGPFreeKeySet( importSet );
							}
							else if( theError == kPGPError_UserAbort )
								theError = kPGPError_NoErr;
							
							(void) PGPFreeKeyDB( newKeys );
						}
						
						if( IsntPGPError( theError ) )
						{
							if( ( recipientSettings & kPGPRecipientSettingsConvEncrypt ) != 0 )
							{
								char		*passphrase = NULL;
										
								theError =
									PGPConventionalEncryptionPassphraseDialog(
											mContext,
											PGPOUIDialogPrompt( mContext,
                                                [kConventionalPassphrasePromptStr UTF8String] ),
											PGPOUIOutputPassphrase( mContext,
												&passphrase ),
											PGPOLastOption( mContext ) );
								
								if( IsntPGPError( theError ) )
								{
									theError = PGPBuildOptionList(
													mContext, &optionsEncode,
													PGPOConventionalEncrypt(
														mContext,
														PGPOPassphrase(
															mContext,
															passphrase ),
														PGPOLastOption(
															mContext ) ),
													PGPOLastOption( mContext ) );
								}
								
								if( IsntPGPError( passphrase ) )
									PGPFreeData( passphrase );
							}
							else
							{
								theError = PGPBuildOptionList( mContext, &optionsEncode,
												PGPOEncryptToKeySet( mContext,
													PGPPeekKeyDBRootKeySet( recipients ) ),
												PGPOLastOption( mContext ) );
							}
						}
	
						if( IsntPGPError( theError ) )
							theError = PGPclGuaranteeMinimumEntropy( mContext );
						
						if( IsntPGPError( theError ) )
						{
							theError = PGPCopySharesToFile( mContext,
											shareFileRef, optionsEncode,
											shares );

							if( IsntPGPError( theError ) )
								theError = PGPSaveShareFile( shareFileRef );
						}

						if( PGPKeyDBRefIsValid( recipients ) )
							PGPFreeKeyDB( recipients );
							
						if( PGPOptionListRefIsValid( optionsEncode ) )
							PGPFreeOptionList( optionsEncode );
              
                        PGPmailCleanupEventHandlerData( &data );
					}

					if( PGPOptionListRefIsValid( optionsDecode ) )
						PGPFreeOptionList( optionsDecode );
						
					if( PGPShareRefIsValid( shares ) )
						PGPFreeShares( shares );

					PGPFreeShareFile( shareFileRef );
				}
			}
			
			PFLFreeFileSpec( shareFileSpecRef );
			
			// Delete copied share file on error.
			if( IsPGPError( theError ) && mSourceIsFolder )
                [mFileManager removeFileAtPath:dest handler:nil];
		}
	}
	else if( dialogResult != NSOKButton )
	{
		theError = kPGPError_UserAbort;
	}
	else
	{
		// Skip button. Copy to destination if we're processing a folder.
		
		if( mSourceIsFolder )
		{
            BOOL	isError = NO;
            
            isError = [mFileManager copyPath:srcInfo->src toPath:dest handler:nil];
            theError = isError ? kPGPError_NoErr : kPGPError_FileOpFailed;
		}
	}
	return( theError );
}

//____________________________________________________________________________
- (PGPError)decryptVerifyFileInternal:
    (PGPmailEncodeDecodeParams *)params
    info:(ProcessingInfo *)srcInfo
    dest:(NSString *)dest
{
	PGPError			theError = kPGPError_NoErr;
	FileKindInfo		info;
     
#if PGP_DEBUG
	pgpAssert( PGPContextRefIsValid( mContext ) );
	pgpAssertAddrValid( params, PGPmailEncodeDecodeParams );
	if( dest != nil] )
		pgpAssert( ![srcInfo->src isEqualToString:dest] );
#endif
	
	(void) GetPGPFileKindInfo( srcInfo->src, &info );
	
	if( ( info.maybePGPFile || info.isPGPFile ) || !mSourceIsFolder )	// Always try to decrypt "top level" files
	{
		if( info.isKeyShareFile )
			theError = [self decryptKeyShareFile:params info:srcInfo dest:dest];
		else
		{
			theError = [self decryptVerifyEncryptedFile:params info:srcInfo dest:dest];
			if( IsntPGPError( theError ) &&
				mSourceIsFolder &&
				IsntNull( dest ) &&
				! [mFileManager fileExistsAtPath:dest] )
			{
				// "Decrypting" the file did not really do so i.e. there was no
				// data to decrypt. Copy the file instead
                 BOOL	error;
                
                error = [mFileManager copyPath:srcInfo->src toPath:dest handler:nil];
                theError = error ? kPGPError_NoErr : kPGPError_FileOpFailed;
 			}
            else if (!dest && (theError == kPGPError_UserAbort))
                theError = kPGPError_NoErr;
		}
	}
	else
	{
		// if it's a file job, don't copy it
		// but if it's a folder, copy non-relevant items
		if( mSourceIsFolder )
		{
            BOOL	error;
			pgpAssert( IsntNull( dest ) );
            
            error = [mFileManager copyPath:srcInfo->src toPath:dest handler:nil];
            theError = error ? kPGPError_NoErr : kPGPError_FileOpFailed;
 		}
	}
	
	if( IsntPGPError( theError ) )
	{
		if( mSignatureDataValid )
            [self newResult:(mDetachedSigInput == nil) ? srcInfo->src : mDetachedSigInput  sigData:&mSignatureData];
	}

	if( IsPGPError( theError ) && (theError != kPGPError_UserAbort) )
    {
        NSBeep();
        NSRunAlertPanel(kErrorStr, kFileCannotBeDecryptedVerifiedStr, kOKStr, kCancelStr, nil);
        theError = kPGPError_UserAbort;
    }

	return( theError );
}

//____________________________________________________________________________
- (PGPError)decryptVerifyFolder:
    (PGPmailEncodeDecodeParams *)params
{
	PGPError	theError = kPGPError_NoErr;
    NSString	*destTree = nil;
	NSDirectoryEnumerator	*theEnumerator = nil;
    
	if(IsntPGPError( theError ) && ! mProcessInPlace)
	{
		theError = [self getOutputFolderName:FALSE src:mSource dest:&destTree];
		if( IsntPGPError( theError ) )
            [mFileManager copyPath:mSource toPath:destTree handler:nil];
	}
    else
        destTree = mSource;
        
    theEnumerator = [mFileManager enumeratorAtPath:destTree];
    
	if( IsntPGPError( theError ) )
	{
        BOOL			isDir = NO;
        
        while( mSource = [theEnumerator nextObject] )
        {
            mWorkingFile = [destTree stringByAppendingPathComponent:mSource];
            
            if( [mFileManager fileExistsAtPath:mWorkingFile isDirectory:&isDir] && !isDir )
            {
                ProcessingInfo	info;
                FileKindInfo	fileInfo;
                NSString		*destFile = nil;
			
                [self getProcessingInfo:mWorkingFile info:&info];

                (void) GetPGPFileKindInfo( info.src, &fileInfo );
	
                if( fileInfo.maybePGPFile || fileInfo.isPGPFile )
                {
                    GetDefaultDecryptVerifyName( info.src, &destFile );
                    [mProgressDialog setFileName:[mFileManager displayNameAtPath:info.src]];
                    theError = [self decryptVerifyFileInternal:params info:&info dest:destFile];
                    if( IsntPGPError(theError)  && !mProcessInPlace )
                        [mFileManager removeFileAtPath:info.src handler:nil];
                }
            }
            
            isDir = NO;
            if( IsPGPError( theError ) )
				break;
        }
    }
    
	return( theError );
}

//____________________________________________________________________________
- (PGPError)decryptVerify:
    (PGPmailEncodeDecodeParams *)params
{
	PGPError		theError = kPGPError_NoErr;
	
	pgpAssertAddrValid( params, PGPmailEncodeDecodeParams );
	
	mProcessInPlace	= FALSE;
	
	if( mSourceIsFolder )
		theError = [self decryptVerifyFolder:params];
	else
	{
		ProcessingInfo	info;
		
        [self getProcessingInfo:mSource info:&info];
		theError = [self decryptVerifyFileInternal:params info:&info dest:nil];
	}

	return( theError );
}

//____________________________________________________________________________
- (PGPError)calcProgressBytes:
    (ByteCount *)progressBytes
{
	PGPError		theError = kPGPError_NoErr;
 
    *progressBytes	= 0;
	
    if( 0 < [mTaskList count] )
	{
		PGPUInt32	numFiles = [mTaskList count];
        PGPUInt32	index = 0;
		NSString*	sigFileSuffix;
		
		sigFileSuffix = kDetachedSigFileSuffixStr;

		for( index = 0; index < numFiles; ++index )
		{
			ProcessingInfo	info;
			
            [self getIndFile:index info:&info];
			*progressBytes += info.resSize + info.dataSize;

			// Account for bytes in target of detached signature
			
			if( info.typeCode == kPGPMacFileType_DetachedSig ||
                [info.src hasSuffix:sigFileSuffix] )
			{                				
				NSString* 		testFile = [info.src stringByDeletingPathExtension];
                BOOL			isDir;
                NSDictionary	*fattrs = nil, *rattrs = nil;
                
                [mFileManager fileExistsAtPath:testFile isDirectory:&isDir];
                fattrs = [mFileManager fileAttributesAtPath:testFile traverseLink:NO];
                rattrs = [mFileManager fileAttributesAtPath:
                        [testFile stringByAppendingPathComponent:kFileResourceFork] traverseLink:YES];
               
				if( fattrs != nil && !isDir)
					*progressBytes += [fattrs fileSize] + [rattrs fileSize];
			}
		}
	}
	return( theError );
}

//____________________________________________________________________________
- (PGPError)decodeOutputEvent:
    (PGPEvent *)event
    data:(PGPmailEventHandlerData *)data
{
	PGPError	theError = kPGPError_NoErr;
	NSString*	outFile = nil;

	(void)data;
		
	GetDefaultDecryptVerifyName( mWorkingFile, &outFile );
	
	if( ( NSAlternateKeyMask & mModifierFlags ) && !mSourceIsFolder )
	{
        NSSavePanel*	sp = [NSSavePanel savePanel];
		int				result = 0;
        
        [sp setRequiredFileType:[outFile pathExtension]];
        [sp setCanSelectHiddenExtension:YES];
        result = [sp runModalForDirectory:[outFile stringByDeletingLastPathComponent] 
                    file:[outFile lastPathComponent]];
        
		if( result = NSOKButton )
			outFile = [sp filename];
		else
			theError = kPGPError_UserAbort;
	}
	else
		GetUniqueFileName( outFile, &outFile );
	
	if( IsntPGPError( theError ) )
	{
        theError = PGPNewFileSpecFromFullPath( mContext, [outFile UTF8String], &mOutputFile );

		theError = PGPAddJobOptions( event->job,
					PGPOOutputFile( mContext, mOutputFile ),
					PGPOLastOption( mContext ) );
    }
	
	return( theError );
}

//____________________________________________________________________________
- (PGPError)decodeDetachedSignatureEvent:
    (PGPEvent *)event
    data:(PGPmailEventHandlerData *)data
{
	PGPError		theError = kPGPError_NoErr;
	
	// see if there is an appropriately named source file
	if( !data->retryingSigVerification )
	{
		if( GetDetachedSigFileSourceName( mWorkingFile, &mDetachedSigInput ) )
		{
            BOOL	exist = NO;
            
            exist = [mFileManager fileExistsAtPath:mDetachedSigInput];
            if( !exist )
                theError = kPGPError_FileNotFound;
        }
		if( IsPGPError( theError ) )
		{
            BOOL		err;
            
			err = PromptForSignatureSourceFile( mWorkingFile, &mDetachedSigInput );
            theError = err ? kPGPError_NoErr : kPGPError_UserAbort ;
		}
	}
	
	if( IsntPGPError( theError ) )
	{
		PGPLocalEncodingFlags	encodingFlags;
        PGPFileSpecRef			inputFile = kInvalidPGPFileSpecRef;
        NSDictionary*			fattr = nil;
        
		// first we'll try MacBinary on (Force),
        // then if we're doing a retry, we'll try it off
		if( data->retryingSigVerification )
			encodingFlags	= kPGPLocalEncoding_None;
		else
            encodingFlags	= kPGPLocalEncoding_Force;

        fattr = [mFileManager fileAttributesAtPath:mWorkingFile traverseLink:YES];

		// require CRC to be valid, unless it's not created by us
		// at least one brain-dead program doesn't set the CRC
		if( kPGPMacFileCreator_PGPtools != [fattr fileHFSCreatorCode])
            encodingFlags	|= kPGPLocalEncoding_NoMacBinCRCOkay;
		
        fattr = [mFileManager fileAttributesAtPath:
            [mDetachedSigInput stringByAppendingPathComponent:kFileResourceFork] traverseLink:YES];

		mSignatureData.haveResourceFork = ( 0 != [fattr fileSize] );
         
        theError = PGPNewFileSpecFromFullPath( mContext, [mDetachedSigInput UTF8String], &inputFile );

		theError = PGPAddJobOptions( event->job,
			PGPODetachedSig( mContext, 
				PGPOInputFile( mContext, inputFile),
				PGPOLocalEncoding( mContext, encodingFlags),
				PGPOLastOption( mContext ) ),
			PGPOLastOption( mContext ) );
            
        if( PGPFileSpecRefIsValid( inputFile ) )
        {
            PGPFreeFileSpec( inputFile );
            inputFile = kInvalidPGPFileSpecRef;
        }
	}
	
	return( theError );
}

//____________________________________________________________________________
- (void)getDecryptPassphrasePromptString:
    (NSString **)prompt
{
    NSMutableString*	tmpString = [NSMutableString string];
    
	[tmpString appendString:kGetPassphraseForFilePromptStr];
    if( nil != mWorkingFile )
        [tmpString appendString:[mFileManager displayNameAtPath:mWorkingFile]];
    *prompt = tmpString;
}

//____________________________________________________________________________
- (PGPError)canUseFYEO:
    (BOOL *)outCanUseFYEO
{
	PGPError		result = kPGPError_NoErr;
	
	*outCanUseFYEO = NO;
    
	if( !mSourceIsFolder )
	{
        NSDictionary*	attributes = [mFileManager fileAttributesAtPath:mSource traverseLink:YES];
                
         if( kPGPMacFileType_Text == [attributes fileHFSTypeCode] )
            *outCanUseFYEO = YES;
    }
	
	return( result );
}

//____________________________________________________________________________
- (void)getProcessingInfo:
    (NSString *)file
    info:(ProcessingInfo *)info
{
    NSDictionary*		fattrs = nil;
    NSDictionary*		rattrs= nil;
    BOOL				hasResource = NO;
    NSString			*resourceFork = nil;
    
	pgpClearMemory( info, sizeof( *info ) );
    
	info->src = file;

    [mFileManager fileExistsAtPath:info->src isDirectory:&info->isFolder];
    fattrs = [mFileManager fileAttributesAtPath:info->src traverseLink:YES];
    
	info->dataSize = [fattrs fileSize];
    info->fileType = [fattrs fileType];
    info->typeCode = [fattrs fileHFSTypeCode];
    info->creatorCode = [fattrs fileHFSCreatorCode];
    
    if( [[info->src pathExtension] isEqualTo:@"txt"] )
        info->typeCode = kPGPMacFileType_Text;
    else if( info->typeCode == kPGPMacFileType_Text )
        info->typeCode = kPGPMacFileType_Unknown;
    
    resourceFork = [info->src stringByAppendingString:kFileResourceFork];
    
    if( [mFileManager fileExistsAtPath:resourceFork] )
    {
        rattrs = [mFileManager fileAttributesAtPath:resourceFork traverseLink:YES];
        hasResource = ( [rattrs fileSize] > 0 );

        if( hasResource )
        {
            info->resFork = [info->src stringByAppendingString:kFileResourceFork];
            info->resSize = [rattrs fileSize];
        }
    }
}

//____________________________________________________________________________
- (void)getIndFile:
    (UInt32)index
    info:(ProcessingInfo *)info
{
	pgpAssert( index < [mTaskList count] );

    mSource = [mTaskList objectAtIndex:index];
    [self getProcessingInfo:mSource info:info];
}

//____________________________________________________________________________
- (void)getErrorString:(PGPError)inErr
    value:(NSString* *)errorStr
{
    NSMutableString*	theStr = [NSMutableString string];
    char				cStr[ 256 ];

    pgpAssert( IsPGPError( inErr ) && (! kPGPError_UserAbort == inErr ) );
    
	*errorStr = nil;
	
    [theStr appendString:kAnErrorOccurredStr];
	
	if( IsPGPError( inErr ) )
	{
		PGPGetClientErrorString( inErr, sizeof( cStr ), cStr );
		[theStr appendString:[NSString stringWithCString:cStr]];
	}
	else
		pgpDebugMsg( "impossible case" );
    
    *errorStr = theStr;
}

//____________________________________________________________________________
- (void)setCurrentPrefs
{
	PGPError			theError = kPGPError_NoErr;
	
	// Comments are not always present. Don't check for errors.
	(void) PGPGetPrefStringBuffer( mPrefRef, kPGPPrefComment,
		sizeof( mTaskPrefs.comment ), mTaskPrefs.comment );
	
	theError	= PGPGetPrefBoolean( mPrefRef,
		kPGPPrefWarnOnWipe, &mTaskPrefs.warnOnWipe );

	if( IsntPGPError( theError ) )
	{
		PGPUInt32	algorithm = 0;
		
		theError = PGPGetPrefNumber( mPrefRef, kPGPPrefPreferredAlgorithm, &algorithm );

		mTaskPrefs.convAlgorithm = (PGPCipherAlgorithm) algorithm;
	}

	if( IsntPGPError( theError ) )
	{
		PGPPrefArray	*prefArray = NULL;
		PGPUInt32		i = 0;
		
		if( IsntPGPError( PGPGetPrefArray( mPrefRef, kPGPPrefAllowedAlgorithmsList, &prefArray ) ) )
		{
			PGPBoolean aes256 = FALSE;
            
            mTaskPrefs.allowedAlgorithms = (PGPCipherAlgorithm *) PGPNewData( mMemoryMgr,
                        ( prefArray->numElements + 2 ) * sizeof( PGPCipherAlgorithm ), 0);
			if( IsntNull( mTaskPrefs.allowedAlgorithms ) )
			{
                mTaskPrefs.numAllowedAlgorithms = prefArray->numElements;
                for( i = 0; i < mTaskPrefs.numAllowedAlgorithms; i++ )
                {
                    mTaskPrefs.allowedAlgorithms[i] = (PGPCipherAlgorithm) prefArray->elements[i].data;
                    if( mTaskPrefs.allowedAlgorithms[i] == kPGPCipherAlgorithm_AES256 )
                        aes256 = TRUE;
                }
                if( aes256 )
                {
                    mTaskPrefs.allowedAlgorithms[mTaskPrefs.numAllowedAlgorithms++] = kPGPCipherAlgorithm_AES128;
                    mTaskPrefs.allowedAlgorithms[mTaskPrefs.numAllowedAlgorithms++] = kPGPCipherAlgorithm_AES192;
                }
			}
			else
				theError = kPGPError_OutOfMemory;
			
			(void) PGPFreePrefArray( prefArray );
		}
	}
}

//____________________________________________________________________________
- (void)newResult:(NSString *)fileName sigData:(PGPmailSignatureData *)signatureData
{
    CPGPVerification		*verification = [[CPGPVerification alloc] init];
    NSImage*				validityImage = nil;
    time_t					unixTime;
    BOOL					validSignature = NO;
	PGPError				theError = kPGPError_NoErr;
	char					userID[256];
	PGPSharedKeyProperties	keyProperties;
    PGPBoolean				isAxiomatic = NO;

	pgpAssertAddrValid( signatureData, PGPEventSignatureData );

	if( PGPKeyDBObjRefIsValid( signatureData->data.signingKey ) )
	{
		PGPSize	userIDLength;
		
		// Get the primary user ID and key properties for the signing key:
		userID[0] = 0;
		
		theError = PGPGetPrimaryUserIDName(signatureData->data.signingKey, userID, sizeof( userID ) - 1, &userIDLength );
		pgpAssertNoErr( theError );
		
		userID[userIDLength] = 0;
		
		theError	= PGPSharedGetKeyProperties( signatureData->data.signingKey, userID, &keyProperties );
		pgpAssertNoErr( theError );
	}
    
    validSignature	= ( signatureData->data.checked && signatureData->data.verified );
    
	theError = PGPGetKeyDBObjBooleanProperty(signatureData->data.signingKey, kPGPKeyProperty_IsAxiomatic, &isAxiomatic);
    if( isAxiomatic )
        validityImage = [[[CPGPIcons shared] images] objectAtIndex:kPGPclImage_DotAxiom];
    else
    {
        PGPBoolean				marginalValidity = FALSE;
        switch( keyProperties.userValidity )
        {
            case kPGPValidity_Unknown:
            case kPGPValidity_Invalid:
                validityImage = [[[CPGPIcons shared] images] objectAtIndex:kPGPclImage_DotDimmed];
                break;
            case kPGPValidity_Marginal:
                theError = PGPGetPrefBoolean(mPrefRef, kPGPPrefMarginalIsInvalid, &marginalValidity);
                validityImage = marginalValidity ?	[[[CPGPIcons shared] images] objectAtIndex:kPGPclImage_DotDimmed] :
                                                    [[[CPGPIcons shared] images] objectAtIndex:kPGPclImage_DotGreen];
                break;
            case kPGPValidity_Complete:
                validityImage = [[[CPGPIcons shared] images] objectAtIndex:kPGPclImage_DotGreen];
                break;
            case kPGPValidity_force:
                validityImage = [[[CPGPIcons shared] images] objectAtIndex:kPGPclImage_DotRed];
                break;
        }
    }
    
    unixTime = PGPGetStdTimeFromPGPTime(signatureData->data.creationTime);
    
    [verification setName:[mFileManager displayNameAtPath:fileName] validity:validSignature]; // name of the file
    [verification setSigner:PGPclGetPrimaryUserIDName( signatureData->data.signingKey )];	// user id
	[verification setValidity:validityImage validity:(keyProperties.userValidity == kPGPValidity_Complete)]; // validity
    [verification setSigned:[NSCalendarDate dateWithTimeIntervalSince1970:unixTime]]; 		// signed time
    [verification setVerified:[NSCalendarDate date]];			// now
    
    [mAppController newVerificationResult:verification];
    [verification release];
}

@end
