/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation
    All rights reserved.

    $Id: CPGPmailPreferences.m,v 1.13 2002/10/21 08:31:47 jeffc Exp $
____________________________________________________________________________*/

#import "CPGPmailPreferences.h"

@implementation CPGPmailPreferences

- (id)init
{
    if( !mInited )
    {
        [super init];
        
        identifiers = [[NSArray alloc] initWithObjects:kPGPmailPrefsIdentifier, nil];

        mInited = YES;
    }
    
    if( !mLoadedNib )
		[self initWithNib:kPGPmailPreferencesNib];
    
    return( self );
}

- (id)initWithNib:(NSString*)nibFile
{
    if( nibFile && [nibFile length] )
        [NSBundle loadNibNamed:nibFile owner:self];
    
	mLoadedNib = YES;
	
    return( [self init] );
}

- (id)initWithController:(id)controller
{
	mMailController = controller;
	
	return( [self init] );
}

- (id)controller
{
    return( mMailController );
}

- (NSArray*)identifiers
{
    return( identifiers );
}

- (id)prefsViewForIdentifier:(NSString*)identifier
{
	if( identifier && [identifier isEqualToString:kPGPmailPrefsIdentifier] )
		return mEmailPrefsView;
	
	return nil;
}

- (id)firstPrefsResponderForIdentifier:(NSString*)identifier
{
	return mPGPMIMECheckButton;
}

- (id)lastPrefsResponderForIdentifier:(NSString*)identifier
{
	return mColumnWidthField;
}

- (void)setupKeyViewLoopForIdentifier:(NSString*)identifier
{
	[mPGPMIMECheckButton setNextKeyView:mEncryptNewCheckButton];
	[mEncryptNewCheckButton setNextKeyView:mSignNewCheckButton];
	[mSignNewCheckButton setNextKeyView:mAutoDecryptCheckButton];
	[mAutoDecryptCheckButton setNextKeyView:mWordWrapCheckButton];
	[mWordWrapCheckButton setNextKeyView:mColumnWidthField];
}

- (void)restorePanelPrefs:(NSString*)panelIdentifier
{
	PGPError			err;
	PGPBoolean			bData;
	PGPUInt32			nData;
	PGPUInt32			prefFlags;
	PGPPrefRef			prefRef = [mMailController pgpPrefRef];
	BOOL				enabled;
	
	if( panelIdentifier && [panelIdentifier isEqualToString:kPGPmailPrefsIdentifier] )
	{
		// kPGPPrefMailSignPGPMIME
		if( IsPGPError(err = PGPGetPrefBoolean(prefRef, kPGPPrefMailSignPGPMIME, &bData)))
			PGPLog((@"Failed to retrieve value for prefid %d (err=%d)", kPGPPrefMailSignPGPMIME, err) );
		
		if( IsntPGPError(err) )
		{
			if( IsntPGPError(err = PGPGetPrefFlags(prefRef, kPGPPrefMailSignPGPMIME, &prefFlags)) )
				enabled = prefFlags & kPGPPrefFlags_DisableGUI ? NO : YES;
			else
			{
				PGPLog((@"Failed to retrieve flags for prefid %d (err=%d)", kPGPPrefMailSignPGPMIME, err));
				enabled = NO;
			}
			
			[self setUsePGPMIME:bData enabled:enabled];
		}
			
		// kPGPPrefMailEncryptDefault
		if( IsPGPError(err = PGPGetPrefBoolean(prefRef, kPGPPrefMailEncryptDefault, &bData)) )
			PGPLog((@"Failed to retrieve value for prefid %d (err=%d)", kPGPPrefMailEncryptDefault, err));
		
		if( IsntPGPError(err) )
		{
			if( IsntPGPError(err = PGPGetPrefFlags(prefRef, kPGPPrefMailEncryptDefault, &prefFlags)) )
				enabled = prefFlags & kPGPPrefFlags_DisableGUI ? NO : YES;
			else
			{
				PGPLog((@"Failed to retrieve flags for prefid %d (err=%d)", kPGPPrefMailEncryptDefault, err));
				enabled = NO;
			}
	
			[self setEncryptByDefault:bData enabled:enabled];
		}
	
		// kPGPPrefMailSignDefault
		if( IsPGPError(err = PGPGetPrefBoolean(prefRef, kPGPPrefMailSignDefault, &bData)) )
			PGPLog((@"Failed to retrieve value for prefid %d (err=%d)", kPGPPrefMailSignDefault, err));
		
		if( IsntPGPError(err) )
		{
			if( IsntPGPError(err = PGPGetPrefFlags(prefRef, kPGPPrefMailSignDefault, &prefFlags)) )
				enabled = prefFlags & kPGPPrefFlags_DisableGUI ? NO : YES;
			else
			{
				PGPLog((@"Failed to retrieve flags for prefid %d (err=%d)", kPGPPrefMailSignDefault, err));
				enabled = NO;
			}
			
			[self setSignByDefault:bData enabled:enabled];
		}
		
		// kPGPPrefAutoDecrypt
		if( IsPGPError(err = PGPGetPrefBoolean(prefRef, kPGPPrefAutoDecrypt, &bData)) )
			PGPLog((@"Failed to retrieve value for prefid %d (err=%d)", kPGPPrefAutoDecrypt, err));
		
		if( IsntPGPError(err) )
		{
			if( IsntPGPError(err = PGPGetPrefFlags(prefRef, kPGPPrefAutoDecrypt, &prefFlags)) )
				enabled = prefFlags & kPGPPrefFlags_DisableGUI ? NO : YES;
			else
			{
				PGPLog((@"Failed to retrieve flags for prefid %d (err=%d)", kPGPPrefAutoDecrypt, err));
				enabled = NO;
			}
			
			[self setAutoDecryptVerify:bData enabled:enabled];
		}
		
		// kPGPPrefWordWrapEnable
		if( IsPGPError(err = PGPGetPrefBoolean(prefRef, kPGPPrefWordWrapEnable, &bData)) )
			PGPLog((@"Failed to retrieve value for prefid %d (err=%d)", kPGPPrefWordWrapEnable, err));
		
		if( IsntPGPError(err) )
		{
			if( IsntPGPError(err = PGPGetPrefFlags(prefRef, kPGPPrefWordWrapEnable, &prefFlags)) )
				enabled = prefFlags & kPGPPrefFlags_DisableGUI ? NO : YES;
			else
			{
				PGPLog((@"Failed to retrieve flags for prefid %d (err=%d)", kPGPPrefWordWrapEnable, err));
				enabled = NO;
			}
			
			[self setWordWrapEnabled:bData enabled:enabled];
		}
		
		// kPGPPrefWordWrapWidth
		if( IsPGPError(err = PGPGetPrefNumber(prefRef, kPGPPrefWordWrapWidth, &nData)) )
			PGPLog((@"Failed to retrieve value for prefid %d (err=%d)", kPGPPrefWordWrapWidth, err));
		
		if( IsntPGPError(err) )
		{
			if( IsntPGPError(err = PGPGetPrefFlags(prefRef, kPGPPrefWordWrapWidth, &prefFlags)) )
				enabled = prefFlags & kPGPPrefFlags_DisableGUI ? NO : YES;
			else
			{
				PGPLog((@"Failed to retrieve flags for prefid %d (err=%d)", kPGPPrefWordWrapWidth, err));
				enabled = NO;
			}
			
			[self setWordWrapWidth:nData enabled:enabled];
		}
	}
}

- (void)restorePanelDefaults:(NSString*)panelIdentifier
{
	PGPError			err;
	PGPBoolean			bData;
	PGPUInt32			nData;
	PGPUInt32			prefFlags;
	PGPPrefRef			prefRef = [mMailController pgpPrefRef];
	PGPPrefDefinition	prefDef;
	BOOL				enabled;
	
	if( panelIdentifier && [panelIdentifier isEqualToString:kPGPmailPrefsIdentifier] )
	{
		// kPGPPrefMailSignPGPMIME
		if( IsPGPError(err = PGPclGetPrefDefault(kPGPPrefMailSignPGPMIME, &prefDef)) )
			PGPLog((@"Failed to retrieve default for prefid %d (err=%d)", kPGPPrefMailSignPGPMIME, err));
		else
		{
			bData = (PGPBoolean)(size_t)prefDef.data;
			
			if( IsntPGPError(err = PGPGetPrefFlags(prefRef, kPGPPrefMailSignPGPMIME, &prefFlags)) )
				enabled = prefFlags & kPGPPrefFlags_DisableGUI ? NO : YES;
			else
			{
				PGPLog((@"Failed to retrieve flags for prefid %d (err=%d)", kPGPPrefMailSignPGPMIME, err));
				enabled = NO;
			}
			
			[self setUsePGPMIME:bData enabled:enabled];
		}
	
		// kPGPPrefMailEncryptDefault
		if( IsPGPError(err = PGPclGetPrefDefault(kPGPPrefMailEncryptDefault, &prefDef)) )
			PGPLog((@"Failed to retrieve default for prefid %d (err=%d)", kPGPPrefMailEncryptDefault, err));
		else
		{
			bData = (PGPBoolean)(size_t)prefDef.data;
			
			if( IsntPGPError(err = PGPGetPrefFlags(prefRef, kPGPPrefMailEncryptDefault, &prefFlags)) )
				enabled = prefFlags & kPGPPrefFlags_DisableGUI ? NO : YES;
			else
			{
				PGPLog((@"Failed to retrieve flags for prefid %d (err=%d)", kPGPPrefMailEncryptDefault, err));
				enabled = NO;
			}
			
			[self setEncryptByDefault:bData enabled:enabled];
		}
		
		// kPGPPrefMailSignDefault
		if( IsPGPError(err = PGPclGetPrefDefault(kPGPPrefMailSignDefault, &prefDef)) )
			PGPLog((@"Failed to retrieve default for prefid %d (err=%d)", kPGPPrefMailSignDefault, err));
		else
		{
			bData = (PGPBoolean)(size_t)prefDef.data;
			
			if( IsntPGPError(err = PGPGetPrefFlags(prefRef, kPGPPrefMailSignDefault, &prefFlags)) )
				enabled = prefFlags & kPGPPrefFlags_DisableGUI ? NO : YES;
			else
			{
				PGPLog((@"Failed to retrieve flags for prefid %d (err=%d)", kPGPPrefMailSignDefault, err));
				enabled = NO;
			}
			
			[self setSignByDefault:bData enabled:enabled];
		}
		
		// kPGPPrefAutoDecrypt
		if( IsPGPError(err = PGPclGetPrefDefault(kPGPPrefAutoDecrypt, &prefDef)) )
			PGPLog((@"Failed to retrieve default for prefid %d (err=%d)", kPGPPrefAutoDecrypt, err));
		else
		{
			bData = (PGPBoolean)(size_t)prefDef.data;
			
			if( IsntPGPError(err = PGPGetPrefFlags(prefRef, kPGPPrefAutoDecrypt, &prefFlags)) )
				enabled = prefFlags & kPGPPrefFlags_DisableGUI ? NO : YES;
			else
			{
				PGPLog((@"Failed to retrieve flags for prefid %d (err=%d)", kPGPPrefAutoDecrypt, err));
				enabled = NO;
			}
			
			[self setAutoDecryptVerify:bData enabled:enabled];
		}
	
		// kPGPPrefWordWrapEnable
		if( IsPGPError(err = PGPclGetPrefDefault(kPGPPrefWordWrapEnable, &prefDef)) )
			PGPLog((@"Failed to retrieve default for prefid %d (err=%d)", kPGPPrefWordWrapEnable, err));
		else
		{
			bData = (PGPBoolean)(size_t)prefDef.data;
			
			if( IsntPGPError(err = PGPGetPrefFlags(prefRef, kPGPPrefWordWrapEnable, &prefFlags)) )
				enabled = prefFlags & kPGPPrefFlags_DisableGUI ? NO : YES;
			else
			{
				PGPLog((@"Failed to retrieve flags for prefid %d (err=%d)", kPGPPrefWordWrapEnable, err));
				enabled = NO;
			}
			
			[self setWordWrapEnabled:bData enabled:enabled];
		}
	
		// kPGPPrefWordWrapWidth
		if( IsPGPError(err = PGPclGetPrefDefault(kPGPPrefWordWrapWidth, &prefDef)) )
			PGPLog((@"Failed to retrieve default for prefid %d (err=%d)", kPGPPrefWordWrapWidth, err));
		else
		{
			nData = (PGPUInt32)prefDef.data;
			
			if( IsntPGPError(err = PGPGetPrefFlags(prefRef, kPGPPrefWordWrapWidth, &prefFlags)) )
				enabled = prefFlags & kPGPPrefFlags_DisableGUI ? NO : YES;
			else
			{
				PGPLog((@"Failed to retrieve flags for prefid %d (err=%d)", kPGPPrefWordWrapWidth, err));
				enabled = NO;
			}
			
			[self setWordWrapWidth:nData enabled:enabled];
		}
	}
}

- (void)savePanelPrefs:(NSString*)panelIdentifier
{
	PGPError				err;
	PGPPrefRef				prefRef = [mMailController pgpPrefRef];
	NSNotificationCenter*	center = [NSNotificationCenter defaultCenter];

	if( panelIdentifier && [panelIdentifier isEqualToString:kPGPmailPrefsIdentifier] )
	{
		// kPGPPrefMailSignPGPMIME
		if( IsPGPError(err = PGPSetPrefBoolean(prefRef, kPGPPrefMailSignPGPMIME, [self getUsePGPMIME])) )
			PGPLog((@"Failed to set value for prefid %d (err=%d)", kPGPPrefMailSignPGPMIME, err));
		else
			[center postNotificationName:kPGPclPrefNotification_UsePGPMIME object:nil];
		
		// kPGPPrefMailEncryptDefault
		if( IsPGPError(err = PGPSetPrefBoolean(prefRef, kPGPPrefMailEncryptDefault, [self getEncryptByDefault])) )
			PGPLog((@"Failed to set value for prefid %d (err=%d)", kPGPPrefMailEncryptDefault, err));
		else
			[center postNotificationName:kPGPclPrefNotification_EncryptByDefault object:nil];
	
		// kPGPPrefMailSignDefault
		if( IsPGPError(err = PGPSetPrefBoolean(prefRef, kPGPPrefMailSignDefault, [self getSignByDefault])) )
			PGPLog((@"Failed to set value for prefid %d (err=%d)", kPGPPrefMailSignDefault, err));
		else
			[center postNotificationName:kPGPclPrefNotification_SignByDefault object:nil];
	
		// kPGPPrefAutoDecrypt
		if( IsPGPError(err = PGPSetPrefBoolean(prefRef, kPGPPrefAutoDecrypt, [self getAutoDecryptVerify])) )
			PGPLog((@"Failed to set value for prefid %d (err=%d)", kPGPPrefAutoDecrypt, err));
		else
			[center postNotificationName:kPGPclPrefNotification_AutoDecrypt object:nil];
	
		// kPGPPrefWordWrapEnable
		if( IsPGPError(err = PGPSetPrefBoolean(prefRef, kPGPPrefWordWrapEnable, [self getWordWrapEnabled])) )
			PGPLog((@"Failed to set value for prefid %d (err=%d)", kPGPPrefWordWrapEnable, err));
		else
			[center postNotificationName:kPGPclPrefNotification_WordWrapEnabled object:nil];
	
		// kPGPPrefWordWrapWidth
		if( IsPGPError(err = PGPSetPrefNumber(prefRef, kPGPPrefWordWrapWidth, [self getWordWrapWidth])) )
			PGPLog((@"Failed to set value for prefid %d (err=%d)", kPGPPrefWordWrapWidth, err));
		else
			[center postNotificationName:kPGPclPrefNotification_WordWrapWidth object:nil];
	}
}

- (void)setUsePGPMIME:(PGPBoolean)value enabled:(BOOL)enabled
{
	[self setCheckbox:mPGPMIMECheckButton state:value enabled:enabled];
}

- (void)setEncryptByDefault:(PGPBoolean)value enabled:(BOOL)enabled
{
	[self setCheckbox:mEncryptNewCheckButton state:value enabled:enabled];
}

- (void)setSignByDefault:(PGPBoolean)value enabled:(BOOL)enabled
{
	[self setCheckbox:mSignNewCheckButton state:value enabled:enabled];
}

- (void)setAutoDecryptVerify:(PGPBoolean)value enabled:(BOOL)enabled
{
	[self setCheckbox:mAutoDecryptCheckButton state:value enabled:enabled];
}

- (void)setWordWrapEnabled:(PGPBoolean)value enabled:(BOOL)enabled
{
    [self setCheckbox:mWordWrapCheckButton state:value enabled:enabled];
    [self wordWrap:self];
}

- (void)setWordWrapWidth:(PGPUInt32)value enabled:(BOOL)enabled
{
	[mColumnWidthField setStringValue:@""];
	[mColumnWidthField setIntValue:value];
	
	if( enabled )
		[mColumnWidthField setEnabled:YES];
	else
		[mColumnWidthField setEnabled:NO];
}

- (PGPBoolean)getUsePGPMIME
{
    return [self getCheckbox:mPGPMIMECheckButton];
}

- (PGPBoolean)getEncryptByDefault
{
    return [self getCheckbox:mEncryptNewCheckButton];
}

- (PGPBoolean)getSignByDefault
{
    return [self getCheckbox:mSignNewCheckButton];
}

- (PGPBoolean)getAutoDecryptVerify
{
    return [self getCheckbox:mAutoDecryptCheckButton];
}

- (PGPBoolean)getWordWrapEnabled
{
    return [self getCheckbox:mWordWrapCheckButton];
}

- (PGPUInt32)getWordWrapWidth
{
	if ([mColumnWidthField intValue] < 30)
		return 30;
	else
		return [mColumnWidthField intValue];
}

- (void)setCheckbox:(NSButton*)checkBox state:(BOOL)state enabled:(BOOL)enabled
{
	if( checkBox )
	{
		if( state )
			[checkBox setState:NSOnState];
		else
			[checkBox setState:NSOffState];
		
		if( enabled )
			[checkBox setEnabled:YES];
		else
			[checkBox setEnabled:NO];
	}
}

- (PGPBoolean)getCheckbox:(NSButton*)checkBox
{
	if( !checkBox )
		return FALSE;
	
	return [checkBox state] == NSOnState ? TRUE : FALSE;
}

- (IBAction)wordWrap:(id)sender
{
    if( [mWordWrapCheckButton state] == NSOnState )
        [mColumnWidthField setEnabled:YES];
    else
        [mColumnWidthField setEnabled:NO];
}

@end

/*__Editor_settings____

	Local Variables:
	tab-width: 4
	End:
	vi: ts=4 sw=4
	vim: si
_____________________*/