/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation.
    All rights reserved.

    $Id: MessageStore.h,v 1.2 2002/08/24 01:32:31 wmalik Exp $
____________________________________________________________________________*/

#import <Foundation/Foundation.h>

@class MailboxUid;
@class MailAccount;
@class MboxIndex;
@class ObjectCache;

@interface MessageStore:NSObject
{
@public
    //struct ? _flags;
    long _flags;
    MailboxUid *_mailboxUid;
    MailAccount *_account;
    MboxIndex *_index;
    NSMutableArray *_allMessages;
    NSMutableArray *_messagesToBeAddedToIndex;
    NSMutableArray *_messagesToBeRemovedFromIndex;
    unsigned int _allMessagesSize;
    unsigned int _deletedMessagesSize;
    unsigned int _deletedMessageCount;
    unsigned int _unreadMessageCount;
    id _updateIndexMonitor;
    int _state;
    ObjectCache *_headerCache;
    ObjectCache *_bodyDataCache;
    ObjectCache *_bodyCache;
    NSTimer *_timer;
}

+ (void)initialize;
+ (unsigned int)numberOfCurrentlyOpenStores;
+ descriptionOfOpenStores;
+ currentlyAvailableStoreForUid:fp12;
+ currentlyAvailableStoresForAccount:fp12;
+ registerAvailableStore:fp12;
+ (void)removeStoreFromCache:fp12;
+ (char)createEmptyStoreIfNeededForPath:fp12;
+ (char)createEmptyStoreForPath:fp12;
+ (char)storeAtPathIsWritable:fp12;
+ (char)cheapStoreAtPathIsEmpty:fp12;
- (void)release;
- initWithMailboxUid:fp12 readOnly:(char)fp16;
- copyWithZone:(struct _NSZone *)fp12;
- (void)dealloc;
- (void)openAsynchronouslyUpdatingIndex:(char)fp12 andOtherMetadata:(char)fp16;
- (void)openAsynchronously;
- (void)openSynchronously;
- (void)openSynchronouslyUpdatingIndex:(char)fp12 andOtherMetadata:(char)fp16;
- (void)updateMetadataAsynchronously;
- (void)updateMetadata;
- (void)didOpen;
- (void)writeUpdatedMessageDataToDisk;
- (void)invalidateSavingChanges:(char)fp12;
- account;
- mailboxUid;
- (char)isOpened;
- storePathRelativeToAccount;
- displayName;
- (char)isReadOnly;
- description;
- (char)isTrash;
- (char)isDrafts;
- (void)messageFlagsDidChange:fp12 flags:fp16;
- (void)structureDidChange;
- (void)messagesWereAdded:fp12;
- (void)messagesWereCompacted:fp12;
- (void)updateUserInfoToLatestValues;
- (void)totalCount:(unsigned int *)fp12 andSize:(unsigned int *)fp16;
- (void)deletedCount:(unsigned int *)fp12 andSize:(unsigned int *)fp16;
- (unsigned int)totalCount;
- (unsigned int)unreadCount;
- (unsigned int)indexOfMessage:fp12;
- copyOfAllMessages;
- (void)addMessageToAllMessages:fp12;
- _defaultRouterDestination;
- routeMessages:fp12;
- finishRoutingMessages:fp12 routed:fp16;
- (char)canRebuild;
- (void)rebuildTableOfContentsAsynchronously;
- (char)canCompact;
- (void)doCompact;
- (void)deleteMessagesOlderThanNumberOfDays:(int)fp12 compact:(char)fp16;
- (void)deleteMessages:fp12 moveToTrash:(char)fp16;
- (void)undeleteMessages:fp12;
- (void)deleteLastMessageWithHeader:fp12 forHeaderKey:fp16 compactWhenDone:(char)fp20;
- (char)allowsAppend;
- (int)appendMessages:fp12 unsuccessfulOnes:fp16;
- messageWithValue:fp12 forHeader:fp16 options:(unsigned int)fp20;
- messageForMessageID:fp12;
- (void)_setHeaderDataInCache:fp12 forMessage:fp16;
- headerDataForMessage:fp12;
- bodyDataForMessage:fp12;
- fullBodyDataForMessage:fp12;
- bodyForMessage:fp12 fetchIfNotAvailable:(char)fp16;
- dataForMimePart:fp12;
- (char)hasCachedDataForMimePart:fp12;
- (unsigned int)flagsForMessage:fp12;
- colorForMessage:fp12;
- (char)_shouldChangeComponentMessageFlags;
- setFlagsFromDictionary:fp12 forMessages:fp16;
- setFlagsFromDictionary:fp12 forMessage:fp16;
- (void)setFlag:fp12 state:(char)fp16 forMessages:fp20;
- (void)setColor:fp12 highlightTextOnly:(char)fp16 forMessages:fp20;
- (void)messageColorsNeedToBeReevaluated;
- (void)startSynchronization;
- performBruteForceSearchWithString:fp12 options:(int)fp16;
- (STR)createSerialNumberStringFrom:(STR)fp12 colorCode:(unsigned short)fp16;
- _getSerialNumberString;
- (void)setNumberOfAttachments:(unsigned int)fp12 forMessage:fp16;
- (void)updateNumberOfAttachmentsForMessages:fp12;
- (void)updateMessageColorsSynchronouslyForMessages:fp12;
- (void)updateMessageColorsAsynchronouslyForMessages:fp12;
- (void)setJunkMailLevel:(int)fp12 forMessages:fp16;
- (void)setJunkMailLevel:(int)fp12 forMessages:fp16 trainJunkMailDatabase:(char)fp20;
- status;
- (void)fetchSynchronously;
- (char)setPreferredEncoding:(unsigned int)fp12 forMessage:fp16;
@end

@interface MessageStore(MessageFrameworkOnly)
+ (void)_autosaveMessageStore:(void *)fp12;
- (void)_cancelAutosave;
- (void)_setNeedsAutosave;
- _fetchHeaderDataForMessage:fp12;
- _fetchBodyDataForMessage:fp12 andHeaderDataIfReadilyAvailable:(id *)fp16;
- _fetchBodyForMessage:fp12;
- (char)_shouldCallCompactWhenClosing;
- (void)_compactMessageAtIndex:(int)fp12;
- (char)_isReadyToBeInvalidated;
- (void)_saveChanges;
- (void)_resetStateToOpenedIfNeeded;
- (void)_invalidate;
- (void)_invalidateInBackground;
- (void)_countUnreadAndDeleted;
- (void)_cleanUpStaleAttachments;
- _lockFilePath;
- (char)_acquireLockFile:fp12;
- (void)_removeLockFile:fp12;
- (void)_flushAllMessageData;
- (void)_rebuildTableOfContentsSynchronously;
- _setOrGetBody:fp12 forMessage:fp16;
@end

@interface MessageStore(Indexing)
- (char)allowsIndexing;
- (char)isIndexed;
- (char)indexIsValid;
- (void)updateIndex;
- _index;
- (void)openIndexWithWriteAccess:(char)fp12;
- (void)checkForValidIndex;
- (void)closeIndex;
- (void)_fullUpdateIndexAsynchronously;
- (void)_setNeedsToUpdateIndex;
- (void)_addMessagesToIndex:fp12;
- (void)_removeMessagesFromIndex:fp12;
- (void)stopUpdatingIndex;
- (void)handleUpdatingFinished;
- (void)_prepareIndexForUpdate;
- (void)_updateIndex;
- (void)_fullUpdateOfIndex;
- (void)_writeUpdatedIndexToDisk;
- indexedSearchFor:fp12 ranks:(id *)fp16 errorString:(id *)fp20;
- cheapIndexedSearchFor:fp12 ranks:(id *)fp16 errorString:(id *)fp20;
- (void)_invalidateIndex;
- (char)_isUpdatingIndex;
@end

@interface MessageStore(ScriptingSupport)
- objectSpecifier;
- objectSpecifierForMessage:fp12;
@end
