/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation.
    All rights reserved.

    $Id: PGPMailDecodeBuffer.m,v 1.7 2002/10/17 06:56:24 jason Exp $
____________________________________________________________________________*/

#import <PGP/PGP.h>
#import <PGP/PGPPrivate.h>
#import <PGPclient/pgpClientPrefs.h>
#import <PGPclient/CPGPPrefObj.h>
#import <pgpPFLPriv.h>
#import <pgpMemoryIO.h>
#import <pgpWordWrap.h>
#import <PGPclient/pgpClientLibOSX.h>
#import <PGPclient/pgpClientLibCocoa.h>
#import <PGPclient/pgpEncodeDecodeData.h>
#import <PGPclient/pgpVersionHeader.h>
#import <PGPclient/SignatureStatusMessage.h>
#import "PGPMailBundle.h"
#import "PGPMailDecodeBuffer.h"

static BOOL sPGPmailDecodeBufferStop = NO;

@interface PGPMailDecodeBuffer ( Private )
- (PGPError)initialEvent:(PGPEvent *)inEvent;
- (PGPError)outputEvent:(PGPEvent *)inEvent;
- (PGPError)endLexEvent:(PGPEvent *)inEvent;
- (PGPError)detachedSignatureEvent:(PGPEvent *)inEvent;
- (PGPError)decodeEventHandler:(PGPEvent *)inEvent;
- (PGPError)signatureEvent:(PGPEvent *)inEvent;
@end

	 static PGPError
sPGPmailDecodeBufferEventHandler(
	PGPContextRef	inContext,
	PGPEvent		*inEvent,
	PGPUserValue	userValue);

@implementation PGPMailDecodeBuffer
+ (void)pgpCancelOperation
{
    sPGPmailDecodeBufferStop = YES;
}

- (id)init
{
    PGPError		theError = kPGPError_NoErr;

    if( !mInited )
    {
        [super init];
        mInited = YES;
    }

    if( !mPGPController )
        mPGPController = [PGPMailBundle sharedInstance];

    theError = PGPclInitEncodeDecodeData(
        [mPGPController pgpContext],
        [mPGPController pgpTLSContext],
        [mPGPController pgpMainKeyDB],
        [mPGPController pgpPrefRef],
        &mData );

	mHaveSignatureData = NO;
	mBuffer = nil;

    return self;
}

- (id)initWithController:(PGPMailBundle *)inController
{
    mPGPController = inController;
    return [self init];
}

- (void)dealloc
{
    PGPclCleanupEncodeDecodeData( &mData );
    [super dealloc];
}

- (NSData *)pgpDecodeData:(NSData *)inData signature:(NSData *)inSignature options:(PGPMailDecodeOptions *)ioOptions
{
    NSData				*theData = nil;
    PGPError 			theError = kPGPError_NoErr;    
	PGPBoolean			cleanupClientDecodeEventHandler = FALSE;
	PGPByte 			*decodeData = NULL;
	PGPByte				*decryptBuffer = PGPNewSecureData( mData.memoryMgr, kPGPcl_MaxPGPUserIDLength, 0);
	PGPByte				*outData = NULL;
	PGPSize				outDataSize = 0;

    mOptions = *ioOptions;

    PGPclUpdateEncodeDecodeData( &mData );

    if( ioOptions->mIsOpenPGPSigned )
        mBuffer = inData;

	decodeData = (PGPByte *)( PGPNewData(mData.memoryMgr, 1, kPGPMemoryMgrFlags_Clear) );
	mData.decodeData = &decodeData;

	theError = PGPclInitDecodeEventHandlerData( mData.context, mData.tlsContext,
				mData.allKeys, mData.prefRef, &mData.clientDecodeEventHandlerData );

	cleanupClientDecodeEventHandler = TRUE;
    if( IsntPGPError( theError ) )
        theError = PGPDecode( mData.context,
                    (ioOptions->mIsOpenPGPSigned) ?
                    PGPOInputBuffer(mData.context, [inSignature bytes], [inSignature length]):
                    PGPOInputBuffer(mData.context, [inData bytes], [inData length]),
                    PGPOEventHandler(mData.context,
                        (PGPEventHandlerProcPtr)(sPGPmailDecodeBufferEventHandler), (PGPUserValue)self),
                    PGPOSendNullEvents(mData.context, 100),
                    PGPOKeyDBRef(mData.context, mData.allKeys),
                    PGPOSendEventIfKeyFound(mData.context, TRUE),
                    PGPOPassThroughIfUnrecognized(mData.context, TRUE),
                    PGPOLastOption(mData.context));

    if(IsntPGPError(theError) && ioOptions->mIsOpenPGPSigned )
        GetSignatureStatusMessage( mData.context, mData.prefRef, &mSignatureData, &mOptions.mSignature );

	if( IsntPGPError( theError ) && PGPKeyDBRefIsValid( mData.clientDecodeEventHandlerData.newKeys ) )
    {
		PGPUInt32		count = 0;
		PGPKeySetRef	selectedSet = kInvalidPGPKeySetRef;

		theError = PGPCountKeysInKeyDB( mData.clientDecodeEventHandlerData.newKeys, &count );

		if( IsntPGPError( theError ) && count > 0 )
        {
 			theError = PGPclSelectKeysDialog( mData.context, mData.prefRef,
						kPGPclKeySelect_ImportVariation,
                        [kUtilImportKeysStr cString],
                        mData.clientDecodeEventHandlerData.newKeys,
						mData.allKeys, &selectedSet );
                        
			if( kPGPError_UserAbort != theError )
                theError = PGPCopyKeys( selectedSet, mData.allKeys, NULL );
            else if( kPGPError_UserAbort == theError )
                theError = kPGPError_NoErr;
		}
	}

	if( IsntPGPError( theError ) && (!mData.sigChecked) || (!mData.sigVerified) )
        NSRunAlertPanel(kPGPmailUtilSignatureWarningStr, (mData.sigChecked) ? kPGPmailUtilBadSignatureStr :
			kPGPmailUtilSignatureNotVerifiedStr, kPGPmailOKButton, nil, nil);

    if( IsntPGPError( theError ) && !ioOptions->mIsOpenPGPSigned)
    {
        if( mData.fyeo )
            theError = PGPclForYourEyesOnlyDialog(mData.context, mData.prefRef, decodeData);
        else
        {
            outData = decodeData;
            outDataSize = strlen(decodeData);
            decodeData = 0;
        }
    }

    if( cleanupClientDecodeEventHandler )
		PGPclCleanupDecodeEventHandlerData( &mData.clientDecodeEventHandlerData );

    if( IsntNull( decryptBuffer ) )
        PGPFreeData( decryptBuffer );

	if (IsntPGPError(theError)) {
		if( ioOptions->mIsOpenPGPSigned )
			theData = inData;
		else if( NULL != outData )
			theData = [NSData dataWithBytes:outData length:outDataSize];
	} else {
		theData = nil;
	}

	*ioOptions = mOptions;
	ioOptions->mDecodeError = theError;

     return theData;
}

- (PGPError)decodeEventHandler:(PGPEvent *)inEvent
{
	PGPError	theError = kPGPError_NoErr;

    switch (inEvent->type)
    {
        case kPGPEvent_InitialEvent:
            theError = [self initialEvent:inEvent];
        break;

        case kPGPEvent_OutputEvent:
            theError = [self outputEvent:inEvent];
        break;

        case kPGPEvent_EndLexEvent:
            theError = [self endLexEvent:inEvent];
        break;

        case kPGPEvent_SignatureEvent:
            theError = [self signatureEvent:inEvent];
        break;

        case kPGPEvent_DetachedSignatureEvent:
            theError = [self detachedSignatureEvent:inEvent];
        break;

        default:
        break;
    }

    theError = PGPclDecodeEventHandler( [mPGPController pgpContext], inEvent,
            &mData.clientDecodeEventHandlerData);

	if( IsPGPError(theError) )
		mData.abort = TRUE;

	return theError;
}

- (PGPError)initialEvent:(PGPEvent *)inEvent
{
    mData.frame = 0;
    mData.fyeo = FALSE;
    mData.sigChecked = TRUE;
    mData.sigVerified = TRUE;
    mData.abort = FALSE;

    return kPGPError_NoErr;
}

- (PGPError)outputEvent:(PGPEvent *)inEvent
{
    if( inEvent->data.outputData.forYourEyesOnly && ( !mData.fyeo) )
    {
        PGPByte	*secureData = (PGPByte *)(PGPNewSecureData(mData.memoryMgr,
                                strlen((char *)(*mData.decodeData)) + 1, kPGPMemoryMgrFlags_None));
    
        strcpy((char *)(secureData), (char *)(*mData.decodeData));
        PGPFreeData(*mData.decodeData);
        *mData.decodeData = secureData;
        mData.fyeo = TRUE;
    }
    return kPGPError_NoErr;
}

- (PGPError)endLexEvent:(PGPEvent *)inEvent
{
    PGPError		theError = kPGPError_NoErr;

    if( mData.clientDecodeEventHandlerData.outputBuffer != 0 )
    {
        NSString		*prefix = nil;
        NSString 		*suffix = nil;
        PGPSize			newSize = mData.clientDecodeEventHandlerData.outputBufferSize
                                    + strlen((char *)(*mData.decodeData)) + 1;

        if( mData.clientDecodeEventHandlerData.signatureDataValid )
        {
            SignatureStatusInfo	status;

            status.sigData = mData.clientDecodeEventHandlerData.signatureData;
            if( !status.sigData.checked)
                mData.sigChecked = FALSE;

            if( !status.sigData.verified)
                mData.sigVerified = FALSE;

            theError = PGPGetKeyIDString( &status.sigData.signingKeyID, kPGPKeyIDString_Abbreviated, status.keyIDString );
            GetSignatureStatusMessage( mData.context, mData.prefRef, &status, &mOptions.mSignature );

            if( mData.clientDecodeEventHandlerData.sectionType == kPGPAnalyze_Encrypted )
            {
                if( !mOptions.mIsOpenPGPEncrypted)
                {
                    prefix = mData.beginEncryptedAndSignedMessage;
                    suffix = mData.endEncryptedAndSignedMessage;
                }
            }
            else
            {
                if( !mOptions.mIsOpenPGPSigned )
                {
                    prefix = mData.beginSignedMessage;
                    suffix = mData.endSignedMessage;
                }
            }

            newSize += [kPGPmailSigLabelBeginStr cStringLength] +[mOptions.mSignature cStringLength] +
                            [prefix cStringLength] + [suffix cStringLength];
        }
        theError = PGPReallocData(mData.memoryMgr, (void *)mData.decodeData, newSize, kPGPMemoryMgrFlags_None);

        if( prefix != 0 )
        {
            strcat((char *)(*mData.decodeData), [kPGPmailSigLabelBeginStr cString]);
            strcat((char *)(*mData.decodeData), [mOptions.mSignature cString]);
            strcat((char *)(*mData.decodeData), [prefix cString]);
        }

        strcat((char *)(*mData.decodeData), (char *)(mData.clientDecodeEventHandlerData.outputBuffer));

        if( suffix != 0 )
            strcat((char *)(*mData.decodeData), [suffix cString]);
    }

    return theError;
}

- (PGPError)detachedSignatureEvent:(PGPEvent *)inEvent
{
	PGPError		theError = kPGPError_NoErr;
	
    theError = PGPAddJobOptions( inEvent->job,
			PGPODetachedSig( mData.context, 
				PGPOInputBuffer( mData.context, [mBuffer bytes], [mBuffer length]),
				PGPOLastOption( mData.context ) ),
			PGPOLastOption( mData.context ) );

	return theError;
}

- (PGPError)signatureEvent:(PGPEvent *)inEvent
{
    PGPError		theError = kPGPError_NoErr;
    
    theError = PGPclDecodeEventHandler( mData.context, inEvent, &mData.clientDecodeEventHandlerData );
    if( IsntPGPError( theError ) )
    {
        mSignatureData.sigData = mData.clientDecodeEventHandlerData.signatureData;
        mHaveSignatureData =  mData.clientDecodeEventHandlerData.signatureDataValid;
        
        mSignatureData.keyIDString[0] = 0;
        
        if( mHaveSignatureData )
            (void)PGPGetKeyIDString(&mSignatureData.sigData.signingKeyID, kPGPKeyIDString_Abbreviated, mSignatureData.keyIDString );
    }
    
    return theError;
}

	PGPError
sPGPmailDecodeBufferEventHandler(
	PGPContextRef	inContext,
	PGPEvent		*inEvent,
	PGPUserValue	userValue)
{
	PGPError		theError = kPGPError_NoErr;

    if( sPGPmailDecodeBufferStop )
        theError = kPGPError_UserAbort;

    if( IsntPGPError( theError ) )
        theError = [(PGPMailDecodeBuffer *)userValue decodeEventHandler:inEvent];

    return theError;
}
@end
