/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation.
    All rights reserved.

    $Id: PGPMailEncodeBuffer.m,v 1.8 2002/10/23 09:52:07 jason Exp $
____________________________________________________________________________*/

#import <PGP/PGP.h>
#import <PGP/PGPPrivate.h>
#import <PGPclient/pgpClientPrefs.h>
#import <PGPclient/CPGPPrefObj.h>
#import <pgpPFLPriv.h>
#import <pgpMemoryIO.h>
#import <pgpWordWrap.h>
#import <PGPclient/pgpClientLibOSX.h>
#import <PGPclient/pgpClientLibCocoa.h>
#import <PGPclient/pgpEncodeDecodeData.h>
#import <PGPclient/pgpVersionHeader.h>
#import <PGPclient/SignatureStatusMessage.h>

#import "PGPMailBundle.h"
#import "PGPMailEncodeBuffer.h"

static BOOL sPGPmailEncodeStop = NO;

	static  PGPError
sPGPmailEncodeEventHandler(
	PGPContextRef	inContext,
	PGPEvent *		inEvent,
	PGPUserValue	inUserValue);

@implementation PGPMailEncodeBuffer
+ (void)pgpCancelOperation
{
    sPGPmailEncodeStop = YES;
}

- (id)init
{
    PGPError		theError = kPGPError_NoErr;

    if( !mInited )
    {
        [super init];
        mInited = YES;
    }

    if( !mPGPController )
        mPGPController = [PGPMailBundle sharedInstance];

    theError = PGPclInitEncodeDecodeData(
        [mPGPController pgpContext],
        [mPGPController pgpTLSContext],
        [mPGPController pgpMainKeyDB],
        [mPGPController pgpPrefRef],
        &mData );

    return self;
}

- (id)initWithController:(PGPMailBundle *)inController
{
    mPGPController = inController;
    return [self init];
}

- (void)dealloc
{
    PGPclCleanupEncodeDecodeData( &mData );
    [super dealloc];
}

- (NSData *)pgpEncodeData:(NSData *)inData options:(PGPMailEncodeOptions *)ioOptions recipients:(NSArray *)inRecipients
{
    NSData					*encodedData = nil;
    PGPError 				theError = kPGPError_NoErr;
	PGPRecipientSettings	theEncryptSettings = 0;
	PGPKeySetRef			recipientSet = kInvalidPGPKeySetRef;
	PGPByte					*convEncryptPassphrase;
	PGPKeyDBObjRef			signingKey = kInvalidPGPKeyDBObjRef;
    PGPKeyDBRef				recipientDB = kInvalidPGPKeyDBRef;
	PGPByte					*signingBuffer = PGPNewSecureData( mData.memoryMgr, kPGPcl_MaxPGPUserIDLength, 0);
	PGPSize					signingPasskeySize = 0;
	PGPBoolean				isPassphrase = FALSE;
	PGPBoolean				askUser = TRUE;
	PGPByte					*wrappedData = NULL;
	PGPSize					wrappedDataSize = 0;
	PGPByte					*outData = NULL;
	PGPSize					outDataSize = 0;
    
	PGPclUpdateEncodeDecodeData( &mData );

    if( ioOptions->mEncrypt )
    {
		PGPKeyDBRef					newKeys = kInvalidPGPKeyDBRef;
        PGPUInt32					numOfRecipients = 0;
        PGPRecipientSpec			*theRecipients = NULL;
		PGPRecipientOptions			theDialogOptions = kPGPRecipientOptionsHideFileOptions;

        if( inRecipients )
        {
            NSEnumerator	*theAddresses = [inRecipients objectEnumerator];
            NSString		*theAddress = nil;
            int				index = 0;

            numOfRecipients = [inRecipients count];

            theRecipients = (PGPRecipientSpec *) PGPNewData( mData.memoryMgr, 
								numOfRecipients * sizeof( PGPRecipientSpec ),
								kPGPMemoryMgrFlags_Clear );
			if( IsNull( theRecipients ) )
				theError = kPGPError_OutOfMemory;

            while( theAddress = [theAddresses nextObject] )
            {
                theRecipients[index].type = kPGPRecipientSpecType_UserID;
                (void)strcpy( theRecipients[index].u.userIDStr, [theAddress cString]);
                index++;
            }
        }

        if( !ioOptions->mUseMIME )
            theDialogOptions |= kPGPRecipientOptionsShowFYEO;

        if( IsntPGPError( theError  ) )
            theError = PGPclRecipientDialog(mData.context, mData.prefRef, mData.tlsContext,
                    mData.allKeys, numOfRecipients, theRecipients, theDialogOptions,
					theEncryptSettings, &theEncryptSettings, &recipientDB, &newKeys);

		if( IsntPGPError( theError ) && PGPKeyDBRefIsValid( recipientDB ) )
			theError = PGPNewKeySet( recipientDB, &recipientSet );

		if( IsntPGPError( theError ) && PGPKeyDBRefIsValid( newKeys ) )
		{
			PGPKeySetRef	selectedSet = kInvalidPGPKeySetRef;

            theError = kPGPError_UserAbort;

 			theError = PGPclSelectKeysDialog(mData.context, mData.prefRef, kPGPclKeySelect_ImportVariation,
						[kUtilImportKeysStr cString], newKeys, mData.allKeys, &selectedSet);

			if( kPGPError_UserAbort != theError )
                theError = PGPCopyKeys( selectedSet, mData.allKeys, NULL );

            if( PGPKeySetRefIsValid(selectedSet) )
            {
                PGPFreeKeySet(selectedSet);
                selectedSet = kInvalidPGPKeySetRef;
            }
		}

		if( 0 != (theEncryptSettings & kPGPRecipientSettingsConvEncrypt))
			theError = PGPConventionalEncryptionPassphraseDialog(mData.context,
						PGPOUIOutputPassphrase(mData.context, (char **)(&convEncryptPassphrase)),
						PGPOLastOption(mData.context));
        else
        {
			PGPUInt32	numKeys = 0;

			theError = PGPCountKeys(recipientSet, &numKeys);
			if( numKeys == 0 )
				if( ioOptions->mSign )
					ioOptions->mEncrypt = NO;
		}

        if( theRecipients )
        {
          	PGPFreeData( theRecipients );
            theRecipients = NULL;
        }

        if( PGPKeyDBRefIsValid(newKeys) )
        {
            PGPFreeKeyDB(newKeys);
            newKeys = kInvalidPGPKeyDBRef;
        }
    }

	if( ioOptions->mSign )
	{
		PGPSize			keyIDSize = 0;
		PGPKeyID *		keyID = NULL;
        
		theError = PGPGetPrefData(mData.prefRef, kPGPPrefDefaultKeyID, &keyIDSize, (void *)(&keyID));
        
		if( IsntPGPError(theError) && (keyIDSize == sizeof(PGPKeyID)) )
		{
			theError = PGPFindKeyByKeyID(mData.allKeys, keyID, &signingKey);
			if( IsntPGPError(theError) )
			{
					theError = PGPGetKeyDBObjBooleanProperty(signingKey,
								kPGPKeyProperty_NeedsPassphrase, &askUser);
					if( IsntPGPError(theError) && askUser )
						if( PGPPassphraseIsValid(signingKey, PGPOLastOption(mData.context)) )
							askUser = FALSE;
			}
            else if( kPGPError_ItemNotFound == theError )
                theError = kPGPError_NoErr;
		}
        // if the default key is not set then we still want to ask the signing passphrase 
        else if( kPGPError_PrefNotFound == theError )
            theError = kPGPError_NoErr;
                
		if( IsntPGPError( theError) && IsntNull( keyID ) )
    	{
			PGPFreeData(keyID);
			keyID = NULL;
		}
        
        if( IsntPGPError( theError ) && askUser )
		{
            PGPGetPassphraseSettings	theSignSettings = kPGPUISettingsTextOutput;
            
			theError = PGPclSigningPassphraseDialog(mData.context, mData.prefRef, mData.allKeys,
						0, kPGPGetPassphraseOptionsHideFileOptions, kPGPGetPassphraseOptionsHideFileOptions,
                        signingKey, (char *)(signingBuffer), &theSignSettings, &signingKey);
                        
			if( kPGPError_KeyUnusableForSignature != theError )
				isPassphrase = TRUE;
			else
			{
				PGPByte		*passkey = NULL;
				
				theError = PGPclReconstitutionDialog(signingKey, mData.allKeys,
							mData.tlsContext, mData.prefRef, &passkey, &signingPasskeySize);
                            
				pgpCopyMemory(passkey, signingBuffer, signingPasskeySize);
				isPassphrase = FALSE;
            }
		}
        
		if( IsntPGPError( theError) && (!ioOptions->mUseMIME) && (!ioOptions->mEncrypt) && (!ioOptions->mFlowed) && (mData.wordWrapWidth > 0))
		{
			PGPIORef			inputIO;
			PGPIORef			outputIO;
			PGPFileOffset		bufSize;
			
			theError = PGPNewMemoryIOFixedBuffer(mData.memoryMgr, (PGPByte *)([inData bytes]),
                                        [inData length], (PGPMemoryIORef *)(&inputIO));
			theError = PGPNewMemoryIO(mData.memoryMgr,(PGPMemoryIORef *)(&outputIO));
			theError = pgpWordWrapIO(inputIO, outputIO, mData.wordWrapWidth, "\r\n");
			PGPFreeIO( inputIO );
			theError = PGPIOSetPos(outputIO, 0);
			theError = PGPIOGetEOF(outputIO, &bufSize);
			wrappedData = (PGPByte *)(PGPNewData(mData.memoryMgr, bufSize, kPGPMemoryMgrFlags_None));
			theError = PGPIORead(outputIO, bufSize, wrappedData, &wrappedDataSize);
            PGPFreeIO( outputIO );
		}	
    }
            
    if( IsntPGPError( theError ) )
        theError= PGPclGuaranteeMinimumEntropy(mData.context);
    
    if( IsntPGPError( theError ) )
        theError = PGPEncode( mData.context,
                    (wrappedData != 0) ? PGPOInputBuffer(mData.context, wrappedData,
                        wrappedDataSize) : PGPOInputBuffer(mData.context, [inData bytes], [inData length]),
                    PGPOAllocatedOutputBuffer(mData.context, (void *)&outData, MAX_PGPSize, &outDataSize),
                    (ioOptions->mUseMIME && !ioOptions->mEncrypt) ?
                        PGPOOmitMIMEVersion( mData.context, TRUE),
                        PGPOOutputLineEndType( mData.context, kPGPLineEnd_CRLF),
                        PGPODetachedSig( mData.context, PGPOLastOption( mData.context )):
                        PGPONullOption(mData.context),
                    PGPOEventHandler(mData.context,
                        (PGPEventHandlerProcPtr)(sPGPmailEncodeEventHandler), (PGPUserValue)self),
                    PGPOSendNullEvents(mData.context, 100),
                    PGPOForYourEyesOnly(mData.context, (theEncryptSettings & kPGPRecipientSettingsFYEO) != 0),
                    ([mData.commentString cStringLength] != 0) ? PGPOCommentString(mData.context,
                        [mData.commentString cString]) : PGPONullOption(mData.context),
                    PGPOVersionString(mData.context, pgpVersionHeaderString()),
                    (ioOptions->mEncrypt && PGPKeySetRefIsValid(recipientSet)) ?
                        PGPOEncryptToKeySet(mData.context, recipientSet) : PGPONullOption(mData.context),
                    (mData.allowedAlgorithms != 0) ? PGPOPreferredAlgorithms(mData.context,
                        mData.allowedAlgorithms, mData.numAlgorithms) : PGPONullOption(mData.context),
                    (ioOptions->mEncrypt && ((theEncryptSettings & kPGPRecipientSettingsConvEncrypt) != 0)) ?
                        PGPOConventionalEncrypt(mData.context, PGPOPassphrase(mData.context,
                        convEncryptPassphrase), PGPOLastOption(mData.context)) :
                        PGPONullOption(mData.context),
                    (ioOptions->mEncrypt && ((theEncryptSettings & kPGPRecipientSettingsConvEncrypt) != 0)) ?
                        PGPOCipherAlgorithm(mData.context, mData.preferredAlgorithm) :
                        PGPONullOption(mData.context),
                    (ioOptions->mSign) ? PGPOSignWithKey(mData.context, signingKey,
                        (askUser) ? ((isPassphrase) ? PGPOPassphrase(mData.context,
                        (char *)(signingBuffer)) : PGPOPasskeyBuffer(mData.context, signingBuffer,
                        signingPasskeySize)) : PGPONullOption(mData.context),
                        (askUser) ? PGPOCachePassphrase(mData.context,
                        (mData.cacheType == kPGPPrefCacheTypeLogon) ? kPGPMaxTimeInterval :
                        mData.cacheSeconds, mData.cacheGlobal) : PGPONullOption(mData.context), 
                        PGPOLastOption(mData.context)) : PGPONullOption(mData.context),
                    PGPOArmorOutput(mData.context, (ioOptions->mEncrypt ? TRUE : FALSE)),
                    PGPODataIsASCII(mData.context, TRUE),
                    PGPOClearSign(mData.context, (ioOptions->mEncrypt ? FALSE : TRUE)),
                    PGPOLastOption(mData.context));
                
    if( IsntNull( signingBuffer ) )
        PGPFreeData( signingBuffer );
    
    if( IsntNull( wrappedData ) )
        PGPFreeData( wrappedData );
        
    if( NULL != outData )
    {
        encodedData = [NSData dataWithBytes:outData length:outDataSize];
        PGPFreeData(outData);
    }
    
    if( PGPKeyDBRefIsValid(recipientDB) )
    {
        PGPFreeKeyDB(recipientDB);
        recipientDB = kInvalidPGPKeyDBRef;
    }

	ioOptions->mEncodeError = theError;
    return encodedData;
}

    PGPError
sPGPmailEncodeEventHandler(
	PGPContextRef	inContext,
	PGPEvent *		inEvent,
	PGPUserValue	inUserValue)
{
    PGPError		theError = kPGPError_NoErr;

    if( sPGPmailEncodeStop )
        theError = kPGPError_UserAbort;

	return theError;
}
@end
