/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation.
    All rights reserved.

    $Id: PGPMailEncodeDecode.m,v 1.3 2002/10/12 10:56:13 jason Exp $
____________________________________________________________________________*/

#import <PGP.h>
#import <PGPclient/pgpClientPrefs.h>
#import <PGPclient/CPGPPrefObj.h>
#import <PGPclient/pgpClientLibCocoa.h>
#import <PGPclient/pgpClientLibOSX.h>
#import <PGPclient/SignatureStatusMessage.h>
#import "PGPMailBundle.h"
#import "PGPMailNSData.h"
#import "PGPMailEncodeDecode.h"
#import "PGPMailEncodeBuffer.h"
#import "PGPMailDecodeBuffer.h"

#define kPGPmailBeginSignedMessage			@"-----BEGIN PGP SIGNED MESSAGE-----"
#define kPGPmailEndSignedMessage			@"-----END PGP SIGNATURE-----"
#define kPGPmailBeginSignatureMessage		@"-----BEGIN PGP SIGNATURE-----"
#define kPGPmailEndSignatureMessage			@"-----END PGP SIGNATURE-----"
#define kPGPmailBeginEncryptedMessage		@"-----BEGIN PGP MESSAGE-----"
#define kPGPmailEndEncryptedMessage			@"-----END PGP MESSAGE-----"
#define kPGPmailBeginPublicKey				@"-----BEGIN PGP PUBLIC KEY BLOCK-----"
#define kPGPmailEndPublicKey				@"-----END PGP PUBLIC KEY BLOCK-----"
#define kPGPmailBeginPrivateKey				@"-----BEGIN PGP PRIVATE KEY BLOCK-----"
#define kPGPmailEndPrivateKey				@"-----END PGP PRIVATE KEY BLOCK-----"

#define kPGPmailDefaultHashAlgorithm		@"sha1"

@interface PGPMailEncodeDecode( Private )
+ (NSRange)pgpBlockRangeInData:(NSData *)inData start:(NSString *)inStartString end:(NSString *)inEndString;
@end

@implementation PGPMailEncodeDecode

+ (id)handler
{
    PGPMailEncodeDecode 	*theEncoder = [[PGPMailEncodeDecode alloc] init];
    
    return [theEncoder autorelease];
}

+ (NSRange)pgpBlockRangeInData:(NSData *)inData start:(NSString *)inStartString end:(NSString *)inEndString
{
    NSRange			theRange = NSMakeRange(0, 0);
    NSString		*theString = nil;
    NSRange			startRange;
    
    theString = [[NSString alloc] initWithData:inData encoding:NSASCIIStringEncoding];
    
    if( ![theString hasPrefix:inStartString] )
    {
//        inStartString = [@"\n" stringByAppendingString:inStartString];
        startRange = [theString rangeOfString:inStartString];
		if (startRange.length > 0) {
			while ((startRange.location != 0) && ([theString characterAtIndex:startRange.location] != '\n')) {
				startRange.location--;
				startRange.length++;
			}
		}
    }
    else
        startRange = NSMakeRange(0, [inStartString length] );
    
    if( startRange.length > 0 )
    {
        NSRange		endRange;
        
//        inEndString = [@"\n" stringByAppendingString:inEndString];
        endRange = [theString rangeOfString:inEndString options:0
            range:NSMakeRange(NSMaxRange(startRange), [theString length] - NSMaxRange(startRange))];
            
        if( endRange.length > 0 )
        {
            NSAssert(endRange.location > startRange.location, @"End is before start");
            theRange = NSUnionRange( startRange, endRange );
        }
    }
    
    [theString release];
    
    return theRange;
}

+ (NSRange)pgpSignedBlockRangeInData:(NSData *)inData
{
    return [self pgpBlockRangeInData:inData start:kPGPmailBeginSignedMessage end:kPGPmailEndSignedMessage];
}

+ (NSRange)pgpSignatureBlockRangeInData:(NSData *)inData
{
    return [self pgpBlockRangeInData:inData start:kPGPmailBeginSignatureMessage end:kPGPmailEndSignatureMessage];
}

+ (NSRange)pgpEncryptionBlockRangeInData:(NSData *)inData
{
    return [self pgpBlockRangeInData:inData start:kPGPmailBeginEncryptedMessage end:kPGPmailEndEncryptedMessage];
}

+ (NSRange)pgpKeyBlockRangeInData:(NSData *)inData
{
	NSRange	range = [self pgpBlockRangeInData:inData start:kPGPmailBeginPublicKey end:kPGPmailEndPublicKey];
	
	if (range.length == 0) {
		range = [self pgpBlockRangeInData:inData start:kPGPmailBeginPrivateKey end:kPGPmailBeginPrivateKey];
	}
	
	return range;
}


+ (NSString *)pgpDefaultHashAlgorithm
{
    return kPGPmailDefaultHashAlgorithm;
}

+ (NSArray *)pgpKnownHashAlgorithms
{
    return [NSArray arrayWithObjects:kPGPmailDefaultHashAlgorithm, nil];
}

+ (NSData *)pgpConvertedStringData:(NSData *)inData fromEncoding:(CFStringEncoding)inOrigEncoding toEncoding:(CFStringEncoding *)ioNewEncoding
{
    NSParameterAssert( NULL != ioNewEncoding );
    
    switch( inOrigEncoding )
    {
        case kCFStringEncodingISOLatin1:
        case kCFStringEncodingISOLatin2:
        case kCFStringEncodingKOI8_R:
        case kCFStringEncodingUTF8:
        case kCFStringEncodingASCII:
            *ioNewEncoding = inOrigEncoding;
            return inData;
        break;
        default:
        {
            CFStringRef 	theString = CFStringCreateFromExternalRepresentation( NULL, 
                                            (CFDataRef)inData, inOrigEncoding );
            CFDataRef		outData;
            
            NSAssert( NULL != theString, @"Unable to convert back to string" );
            outData = CFStringCreateExternalRepresentation( NULL, theString,
                            kCFStringEncodingISOLatin1, 0 );
            if( NULL == outData )
            {
                outData = CFStringCreateExternalRepresentation( NULL, theString,
                                kCFStringEncodingISOLatin2, 0 );
                if( NULL == outData )
                {
                    outData = CFStringCreateExternalRepresentation( NULL, theString,
                                    kCFStringEncodingKOI8_R, 0 );
                    if( NULL == outData )
                    {
                        outData = CFStringCreateExternalRepresentation( NULL, theString,
                                        kCFStringEncodingUTF8, 0 );
                        NSAssert( NULL !=  outData, @"Unable to convert string to UTF-8" );
                        *ioNewEncoding = kCFStringEncodingUTF8;
                    }
                    else
                       *ioNewEncoding = kCFStringEncodingKOI8_R;
                }
                else
                    *ioNewEncoding = kCFStringEncodingISOLatin2;
            }
            else
                *ioNewEncoding = kCFStringEncodingISOLatin1;

            CFRelease( theString );

            return [(NSData *)outData autorelease];
        }
    }
}

+ (void)pgpCancelEncodeOperation
{
    [PGPMailEncodeBuffer pgpCancelOperation];
}

+ (void)pgpCancelDecodeOperation
{
    [PGPMailDecodeBuffer pgpCancelOperation];
}

- (id)init
{
    if( !mInited )
    {
        [super init];
        mInited = YES;
    }

    if( !mController )
        mController = [PGPMailBundle sharedInstance];

    return self;
}

- (id)initWithController:(PGPMailBundle *)inController
{
    mController = inController;
    
    return [self init];
}

- (void)dealloc
{
    [[NSNotificationCenter defaultCenter] removeObserver:self];
    
    [super dealloc];
}

- (NSData *)pgpEncodeData:(NSData *)inData options:(PGPMailEncodeOptions *)ioOptions recipients:(NSArray *)inRecipients
{
    PGPMailEncodeBuffer		*theEncoder = nil;
    NSData					*outData = nil;
    
    NSParameterAssert(nil != inData);
    
    theEncoder = [[PGPMailEncodeBuffer alloc] initWithController:mController];
    
    outData = [theEncoder pgpEncodeData:inData options:ioOptions recipients:inRecipients];
    
    [theEncoder release];
    
    return outData;
}

- (NSData *)pgpDecryptVerifyData:(NSData *)inData options:(PGPMailDecodeOptions *)ioOptions
{
    PGPMailDecodeBuffer		*theDecoder = nil;
    NSData					*outData = nil;
    
    NSParameterAssert(nil != inData);
    
    theDecoder = [[PGPMailDecodeBuffer alloc] initWithController:mController];
    
    outData = [theDecoder pgpDecodeData:inData signature:nil options:ioOptions];
    
    [theDecoder release];
    
    return outData;
}

- (NSData *)pgpVerifyMIMEData:(NSData *)inData signature:(NSData *)inSignature options:(PGPMailDecodeOptions *)ioOptions
{
    PGPMailDecodeBuffer		*theDecoder = nil;
    NSData					*outData = nil;
    
    NSParameterAssert(nil != inData);
    NSParameterAssert(nil != inSignature);
    
    theDecoder = [[PGPMailDecodeBuffer alloc] initWithController:mController];
    
    outData = [theDecoder pgpDecodeData:inData signature:inSignature options:ioOptions];
    
    [theDecoder release];
    
    return outData;
}

- (NSString *)pgpDefaultHashAlgorithm
{
    return [[self class] pgpDefaultHashAlgorithm];
}

- (NSArray *)pgpKnownHashAlgorithms
{
    return [[self class] pgpKnownHashAlgorithms];
}
@end
