/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation.
    All rights reserved.

    $Id: PGPMailMessage.m,v 1.5 2002/08/21 22:33:22 wmalik Exp $
____________________________________________________________________________*/

#import <PGP/PGP.h>
#import <PGPclient/pgpClientPrefs.h>
#import "PGPMailMessage.h"

#import "PGPMailMessageBody.h"
#import "PGPMailMessageHeaders.h"
#import "PGPMailMimeBody.h"

#import <NSStringMessage.h>
#import <MessageHeaders.h>
#import <MimeBody.h>
#import <MutableMessageHeaders.h>
#import <MessageStore.h>
#import <MimeBody.h>
#import <MimePart.h>
#import <ObjectCache.h>
#import "PGPMailBundle.h"

#import <Foundation/Foundation.h>

#define kPGPmailErrorEncoding					LSPV(@"-[%@ %@] Error encrypting & signing", @"ERROR_ENCRYPTING_AND_SIGNING")
#define kPGPmailErrorMimeParsing				LSPV(@"Error parsing MIME body", @"ERROR_PARSING_MIME_BODY")

@implementation Message(PGPMail)
static NSString *pgpFilteredRecipient(NSString *recipient)
{
    NSMutableString	*filteredRecipient = [NSMutableString stringWithString:recipient];
    int				i = 0, j = 0, length = [filteredRecipient length];

    for(i = 0 ; i < length; i++)
    {
        if([recipient characterAtIndex:i] == ' ')
        {
            int	k = i;
            
            for(++i; i < length; i++)
                if([recipient characterAtIndex:i] != ' ')
                    break;
            [filteredRecipient deleteCharactersInRange:NSMakeRange(k - j, i - k)];
            j += i - k;
            length -= j;
        }
    }
    
	return filteredRecipient;
}

- (NSArray *)pgpRecipients
{
    NSArray			*recipients = [[self headers] recipients];
    int				i = 0, count = [recipients count];
    NSMutableArray	*recipientsAddresses = [NSMutableArray arrayWithCapacity:count];

    for(i = 0; i < count; i++)
        [recipientsAddresses addObject:pgpFilteredRecipient([[[recipients objectAtIndex:i] decodeMimeHeaderValue] uncommentedAddress])];

    return recipientsAddresses;
}

- (BOOL)isPGPMessage
{
    return [[self messageBody] isPGPMessage];
}

- (BOOL)isPGPMessage:(PGPMailDecodeOptions *)ioOptions
{
    return [[self messageBody] isPGPMessage:ioOptions];
}

- (void)pgpEncodeMessage:(PGPMailEncodeOptions *)ioOptions
{
    if([[[self messageBody] rawData] length] > 0)
    {
        NSAutoreleasePool	*localPool;
        NSMutableData		*theData;
        Message				*tmpMessage = nil;
        NSArray				*theRecipients = [self pgpRecipients];

        NSAssert2( [[self messageBody] respondsToSelector:@selector(mutableData)], kPGPmailErrorEncoding, NSStringFromClass([self class]), NSStringFromSelector(_cmd));
        
        localPool = [[NSAutoreleasePool alloc] init];
        theData = [NSMutableData dataWithData:[[self headers] encodedHeadersIncludingFromSpace:NO]];
        [theData appendData:[[self messageBody] rawData]];
        tmpMessage = [Message messageWithRFC822Data:theData];
        
        NS_DURING
            NSData	*encodedData = [(MessageBody *)[tmpMessage messageBody] pgpEncode:ioOptions recipients:theRecipients];
            
            [theData setData:encodedData];
        NS_HANDLER
            [localException retain];
            [localPool release];
            [[localException autorelease] raise];
        NS_ENDHANDLER
        [[self headers] flushCachedData];
        [self setHeaders:[tmpMessage headers]];
        [[self headers] setMessage:self];
        [[[self messageBody] performSelector:@selector(mutableData)] setData:theData];
        [localPool release];
    }
}

- (Message *)pgpDecryptVerifyMessage:(PGPMailDecodeOptions *)ioOptions
{
    MessageHeaders	*decryptedMessageHeaders;
    MessageBody		*decryptedBody = [(MimeBody *)[self messageBody] pgpDecryptedBody:&decryptedMessageHeaders options:ioOptions];
    
    if(decryptedBody)
    {
        Message		*decryptedMessage = [[[self class] alloc] init];
        MimePart	*newPart;
    
        [decryptedBody setMessage:decryptedMessage];
        [decryptedMessage setHeaders:decryptedMessageHeaders];
        [[self messageStore] _setHeaderDataInCache:[((MessageStore *)[self messageStore])->_headerCache objectForKey:decryptedBody] forMessage:decryptedMessage];
        [((MessageStore *)[self messageStore])->_headerCache removeObjectForKey:decryptedBody];
        [decryptedMessage setMessageFlags:[self messageFlags]];
        [((MessageStore *)[self messageStore])->_bodyCache setObject:decryptedBody forKey:decryptedMessage];
        [((MessageStore *)[self messageStore])->_bodyDataCache setObject:[((MessageStore *)[self messageStore])->_bodyDataCache objectForKey:decryptedBody] forKey:decryptedMessage];
        [((MessageStore *)[self messageStore])->_bodyDataCache removeObjectForKey:decryptedBody];
        [decryptedMessage setMessageStore:[self messageStore]];
        
        newPart = [[MimePart alloc] init];
        [(MimeBody *)decryptedBody setTopLevelPart:newPart];
        [newPart setMimeBody:decryptedBody];
        [newPart release];
        NSAssert([[(MimeBody *)decryptedBody topLevelPart] parseMimeBody], kPGPmailErrorMimeParsing);
        
        return [decryptedMessage autorelease];
    }
    else
        return nil;
}
@end
