/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation.
    All rights reserved.

    $Id: PGPMailMimeBody1.m,v 1.1 2002/10/16 04:18:16 jason Exp $
____________________________________________________________________________*/

#import <PGP/PGP.h>
#import <PGPclient/pgpClientPrefs.h>

#import "PGPMailMimeBody.h"
#import "PGPMailMimePart.h"
#import "PGPMailMessageBody.h"
#import "PGPMailMessageHeaders.h"
#import "PGPMailNSData.h"
#import "PGPMailBundle.h"

#import <Message.h>
#import <MutableMessageHeaders.h>
#import <MessageStore.h>
#import <MessageWriter.h>
#import <MimeTextAttachment.h>
#import <ObjectCache.h>
#import <NSDataMessage.h>
#import <NSStringMessage.h>

#import <Foundation/Foundation.h>

#define kPGPmailOnlySupport7Bit		LSPV(@"-[%@ %@]: only 7bit content-transfer-encoding", @"MESSAGE_ONLY_SUPPORT_7BIT")

@interface MimeBody(PGPMailPrivate)
- (NSData *)pgpOpenPGPEncrypt:(PGPMailEncodeOptions *)ioOptions recipients:(NSArray *)inRecipients;
- (NSData *)pgpOpenPGPSign:(PGPMailEncodeOptions *)ioOptions;
@end

@implementation MimeBody(PGPMail)

- (BOOL)isPGPMessage
{
    (void)[self mimeType];
    return [_topLevelPart isPGPMessage];
}

- (BOOL)isPGPMessage:(PGPMailDecodeOptions *)ioOptions;
{
    (void)[self mimeType];
    return [_topLevelPart isPGPMessage:ioOptions];
}

- (MessageBody *)pgpDecryptedBody:(MessageHeaders **)ioDecryptedMessageHeaders options:(PGPMailDecodeOptions *)ioOptions
{
    NSData					*decryptedData;
	MimePart				*part = _topLevelPart;

	if (([[_topLevelPart type] caseInsensitiveCompare:kPGPmailHeaderMultipart] == NSOrderedSame) && ([[_topLevelPart subtype] caseInsensitiveCompare:kPGPmailHeaderAlternative] == NSOrderedSame)) {
		NSArray *		parts = [_topLevelPart subparts];
		NSEnumerator *	n = [parts objectEnumerator];
		MimePart *		thePart;
		
		while (thePart = [n nextObject]) {
			if (([[thePart type] caseInsensitiveCompare:kPGPmailHeaderText] == NSOrderedSame) && ([[thePart subtype] caseInsensitiveCompare:kPGPmailHeaderPlain] == NSOrderedSame)) {
				part = thePart;
				break;
			}
		}
	}
    decryptedData = [part pgpDecryptedVerifiedData:ioOptions];
//    decryptedData = [_topLevelPart pgpDecryptedVerifiedData:ioOptions];
    if(nil != decryptedData)
    {
        MutableMessageHeaders	*headers = [[[self message] headers] mutableCopy];
        NSData					*headerData;
        NSRange					headerBodySeparationRange;

        decryptedData = [NSMutableData dataWithData:decryptedData];
        [(NSMutableData *)decryptedData convertNetworkLineEndingsToUnix];
		
		if (ioOptions->mIsOpenPGPSigned || (ioOptions->mIsOpenPGPEncrypted && ([ioOptions->mSignature cStringLength] > 0))) {
			NSString *		boundaryString = [NSString stringWithFormat:@"======PGPMail_Plugin_%@======", [[NSProcessInfo processInfo] globallyUniqueString]];
			NSString *		multipartMixed = [NSString stringWithFormat:@"Content-Type: multipart/mixed; boundary=\"%@\"\n\n", boundaryString];
			NSString *		boundary = [NSString stringWithFormat:@"\n--%@\n", boundaryString];
			NSString *		endBoundary = [NSString stringWithFormat:@"\n--%@--\n", boundaryString];
			NSString *		textPlain = @"Content-Type: text/plain; charset=us-ascii\n\n";
			NSString *		prefix = (ioOptions->mIsOpenPGPEncrypted) ? kPGPmailSigBeginDecryptStr : kPGPmailSigBeginVerifiedStr;
			NSString *		suffix = (ioOptions->mIsOpenPGPEncrypted) ? kPGPmailSigEndDecryptStr : kPGPmailSigEndVerifiedStr;
			NSMutableData *	tmpData = [NSMutableData data];
			
			[tmpData appendData:[NSData dataWithBytes:[multipartMixed cString] length:[multipartMixed cStringLength]]];
			[tmpData appendData:[NSData dataWithBytes:[boundary cString] length:[boundary cStringLength]]];
			[tmpData appendData:[NSData dataWithBytes:[textPlain cString] length:[textPlain cStringLength]]];
			[tmpData appendData:[NSData dataWithBytes:[kPGPmailSigLabelBeginStr cString] length:[kPGPmailSigLabelBeginStr cStringLength]]];
			[tmpData appendData:[NSData dataWithBytes:[ioOptions->mSignature cString] length:[ioOptions->mSignature cStringLength]]];
			[tmpData appendData:[NSData dataWithBytes:[prefix cString] length:[prefix cStringLength]]];
			[tmpData appendData:[NSData dataWithBytes:[boundary cString] length:[boundary cStringLength]]];
			[tmpData appendData:decryptedData];
			[tmpData appendData:[NSData dataWithBytes:[boundary cString] length:[boundary cStringLength]]];
			[tmpData appendData:[NSData dataWithBytes:[textPlain cString] length:[textPlain cStringLength]]];
			[tmpData appendData:[NSData dataWithBytes:[suffix cString] length:[suffix cStringLength]]];
			[tmpData appendData:[NSData dataWithBytes:[endBoundary cString] length:[endBoundary cStringLength]]];
			decryptedData = tmpData;
/*		} else if (([[_topLevelPart type] caseInsensitiveCompare:@"multipart"] == NSOrderedSame) && ([[_topLevelPart subtype] caseInsensitiveCompare:@"alternative"] == NSOrderedSame)) {
			NSString *		boundary = [NSString stringWithFormat:@"\n--%@\n", [_topLevelPart bodyParameterForKey:@"boundary"]];
			NSArray *		parts = [[NSString stringWithCString:[decryptedData bytes] length:[decryptedData length]] componentsSeparatedByString:boundary];
			NSEnumerator *	n = [parts objectEnumerator];
			NSString *		part;
			NSString *		s;
			NSRange			r;
			NSMutableData *	tmpData = [NSMutableData data];
			
			while (part = [n nextObject]) {
				r = [part rangeOfString:@"\n\n"];
				if (r.length > 0) {
					s = [part substringToIndex:r.location];
					r = [s rangeOfString:@"content-type:" options:NSCaseInsensitiveSearch];
					if (r.length > 0) {
						s = [[s substringFromIndex:r.length] stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceCharacterSet]];
						if ([s hasPrefix:@"text/plain"]) {
							[tmpData appendData:[NSData dataWithBytes:[boundary cString] length:[boundary cStringLength]]];
							[tmpData appendData:[NSData dataWithBytes:[part cString] length:[part cStringLength]]];
							[tmpData appendData:[NSData dataWithBytes:[boundary cString] length:[boundary cStringLength]]];
						}
						continue;
					}
				}
				[tmpData appendData:[NSData dataWithBytes:[part cString] length:[part cStringLength]]];
			}
			decryptedData = tmpData;
*/		}
        headerBodySeparationRange = [decryptedData pgpHeaderBodySeparationRange];
                    
        NSAssert2(headerBodySeparationRange.location != NSNotFound, kPGPmailUnexpectedCase, NSStringFromClass([self class]), NSStringFromSelector(_cmd) );
        if(headerBodySeparationRange.location != 0)
        {
            NSString		*aKey;
            NSEnumerator	*orderEnum;
            
            headerData = [decryptedData subdataWithRange:NSMakeRange(0, headerBodySeparationRange.location)];
            orderEnum = [[headers _decodeHeaderKeysFromData:headerData] objectEnumerator];
            while(aKey = [orderEnum nextObject])
                [headers setHeader:[headers _headerValueForKey:aKey fromData:headerData] forKey:aKey];
        }
        else
        {
            headerData = [NSData dataWithData:[[[self message] messageStore] headerDataForMessage:[self message]]];
        }
        
        decryptedData = [decryptedData subdataWithRange:NSMakeRange(headerBodySeparationRange.location, [decryptedData length] - headerBodySeparationRange.location)];
        
        *ioDecryptedMessageHeaders = [headers autorelease];
        {
            MimeBody	*decryptedBody = [[MimeBody alloc] init];
            
            [((MessageStore *)[[self message] messageStore])->_bodyDataCache setObject:decryptedData forKey:decryptedBody];
            [((MessageStore *)[[self message] messageStore])->_headerCache setObject:headerData forKey:decryptedBody];
            return [decryptedBody autorelease];
        }
    }
    return nil;
}

- (BOOL)pgpAllAttachmentsAreAvailable
{
    return [_topLevelPart pgpAllAttachmentsAreAvailable];
}

- (NSData *)pgpEncode:(PGPMailEncodeOptions *)ioOptions recipients:(NSArray *)inRecipients
{
    if(!ioOptions->mUseMIME && [[self mimeType] isEqualToString:kPGPmailHeaderText] && [[self mimeSubtype] isEqualToString:kPGPmailHeaderPlain])
        return [super pgpEncode:ioOptions recipients:inRecipients];
    else
    {
		ioOptions->mUseMIME = YES;
        if( ioOptions->mEncrypt )
            return [self pgpOpenPGPEncrypt:ioOptions recipients:inRecipients];
        else
        {
            ioOptions->mEncapsulatedSignature = NO;
            return [self pgpOpenPGPSign:ioOptions];
        }
    }
}

- (NSData *)pgpOpenPGPEncrypt:(PGPMailEncodeOptions *)ioOptions recipients:(NSArray *)inRecipients
{
    NSMutableData			*dataToEncrypt = [NSMutableData data];
    NSData					*encryptedData = nil;
    MutableMessageHeaders	*newHeaders = [[[self message] headers] mutableCopy];
    MutableMessageHeaders	*headersToEncrypt = [[MutableMessageHeaders alloc] init];
    NSString				*newBoundary = [MimeBody createMimeBoundary];
    BOOL					usesQuotedPrintable = NO;

    [headersToEncrypt setHeader:[newHeaders firstHeaderForKey:kPGPmailHeaderContentType] forKey:kPGPmailHeaderContentType];
    [headersToEncrypt setHeader:[newHeaders firstHeaderForKey:kPGPmailHeaderTransferEncoding] forKey:kPGPmailHeaderTransferEncoding];

    NSAssert2([[self rawData] length] > 0, kPGPmailDataMissing, NSStringFromClass([self class]), NSStringFromSelector(_cmd));

    NSAssert2(![[self rawData] pgpContainsNonASCIICharacter], kPGPmailOnlySupport7Bit, NSStringFromClass([self class]), NSStringFromSelector(_cmd) );
    
    if(ioOptions->mSignatureType != PGPNoSignature)
    {
        if(![[PGPMailBundle sharedInstance] pgpUsesEncapsulatedSignature])
        {
            [dataToEncrypt appendData:[headersToEncrypt encodedHeadersIncludingFromSpace:NO]];
            [dataToEncrypt appendData:[self rawData]];
        }
        else
        {
            NSData	*tempData = [self pgpOpenPGPSign:ioOptions];
            
            ioOptions->mSignatureType = PGPNoSignature;
            [dataToEncrypt appendData:tempData];
        }
    }
    else{
        [dataToEncrypt appendData:[headersToEncrypt encodedHeadersIncludingFromSpace:NO]];
        [dataToEncrypt appendData:[self rawData]];
    }

    NS_DURING
        ioOptions->mStringEncoding = kCFStringEncodingISOLatin1;
        encryptedData = [[PGPMailEncodeDecode handler] pgpEncodeData:dataToEncrypt options:ioOptions recipients:inRecipients];
    NS_HANDLER
        [newHeaders release];
        [headersToEncrypt release];
        [localException raise];
    NS_ENDHANDLER
    [headersToEncrypt release];

    if([encryptedData pgpContainsNonASCIICharacter])
    {
        encryptedData = [encryptedData encodeQuotedPrintableForText:YES];
        usesQuotedPrintable = YES;
    }

    [newHeaders removeHeaderForKey:kPGPmailHeaderContentType];
    [newHeaders removeHeaderForKey:kPGPmailHeaderTransferEncoding];
    [newHeaders setHeader:kPGPmailHeader7Bit forKey:kPGPmailHeaderTransferEncoding];
    [newHeaders setHeader:[NSString stringWithFormat:@"multipart/encrypted; protocol=\"application/pgp-encrypted\";\n\tboundary=\"%@\"", newBoundary] forKey:kPGPmailHeaderContentType];
    [[[self message] headers] flushCachedData];
    [[self message] setHeaders:newHeaders];
    [newHeaders release];
    
    dataToEncrypt = [NSMutableData data];
    [dataToEncrypt appendData:[@"--" dataUsingEncoding:NSASCIIStringEncoding]];
    [dataToEncrypt appendData:[newBoundary dataUsingEncoding:NSASCIIStringEncoding]];
    [dataToEncrypt appendData:[@"\n" dataUsingEncoding:NSASCIIStringEncoding]];
    [dataToEncrypt appendData:[@"content-type: application/pgp-encrypted; x-mac-type=54455854;\n\tname=Version.txt\n" dataUsingEncoding:NSASCIIStringEncoding]];
    [dataToEncrypt appendData:[@"content-disposition: inline\n" dataUsingEncoding:NSASCIIStringEncoding]];
    [dataToEncrypt appendData:[@"content-transfer-encoding: 7bit\n\n" dataUsingEncoding:NSASCIIStringEncoding]];
    [dataToEncrypt appendData:[@"Version: 1\n\n--" dataUsingEncoding:NSASCIIStringEncoding]];
    [dataToEncrypt appendData:[newBoundary dataUsingEncoding:NSASCIIStringEncoding]];
    [dataToEncrypt appendData:[@"\n" dataUsingEncoding:NSASCIIStringEncoding]];
    [dataToEncrypt appendData:[@"content-type: application/octet-stream; x-mac-type=70674546;\n\tname=Message.pgp" dataUsingEncoding:NSASCIIStringEncoding]];
    if(usesQuotedPrintable)
        [dataToEncrypt appendData:[[@"; charset=" stringByAppendingString:(NSString *)CFStringConvertEncodingToIANACharSetName(kCFStringEncodingUTF8)] dataUsingEncoding:NSASCIIStringEncoding]];
    [dataToEncrypt appendData:[@"\ncontent-disposition: inline\n" dataUsingEncoding:NSASCIIStringEncoding]];
    if(usesQuotedPrintable)
        [dataToEncrypt appendData:[@"content-transfer-encoding: quoted-printable\n\n" dataUsingEncoding:NSASCIIStringEncoding]];
    else
        [dataToEncrypt appendData:[@"content-transfer-encoding: 7bit\n\n" dataUsingEncoding:NSASCIIStringEncoding]];
    [dataToEncrypt appendData:encryptedData];
    [dataToEncrypt appendData:[@"\n--" dataUsingEncoding:NSASCIIStringEncoding]];
    [dataToEncrypt appendData:[newBoundary dataUsingEncoding:NSASCIIStringEncoding]];
    [dataToEncrypt appendData:[@"--\n" dataUsingEncoding:NSASCIIStringEncoding]];
    
    return [[dataToEncrypt retain] autorelease];
}

- (NSData *)pgpOpenPGPSign:(PGPMailEncodeOptions *)ioOptions
{
    NSMutableData			*dataToSign = [NSMutableData data], *signedData;
    NSData					*signatureData = nil;
    MutableMessageHeaders	*newHeaders = [[[self message] headers] mutableCopy];
    MutableMessageHeaders	*headersToSign = [[MutableMessageHeaders alloc] init];
    NSString				*newBoundary = [MimeBody createMimeBoundary];
    PGPMailEncodeDecode		*theEncoder;
    BOOL					signatureDataIsOnlyASCII;

    [headersToSign setHeader:[newHeaders firstHeaderForKey:kPGPmailHeaderContentType] forKey:kPGPmailHeaderContentType];
    [headersToSign setHeader:[newHeaders firstHeaderForKey:kPGPmailHeaderTransferEncoding] forKey:kPGPmailHeaderTransferEncoding];

    NSAssert2([[self rawData] length] > 0, kPGPmailDataMissing, NSStringFromClass([self class]), NSStringFromSelector(_cmd));

    [dataToSign appendData:[headersToSign encodedHeadersIncludingFromSpace:NO]];
    [dataToSign appendData:[self rawData]];
    [dataToSign pgpNormalizeDataForSigning];

    NSAssert2(![dataToSign pgpContainsNonASCIICharacter], kPGPmailOnlySupport7Bit, NSStringFromClass([self class]), NSStringFromSelector(_cmd) );

    theEncoder = [PGPMailEncodeDecode handler];
    NS_DURING
        ioOptions->mStringEncoding = kCFStringEncodingISOLatin1;
        ioOptions->mDetachedSignature = YES;
        signatureData = [theEncoder pgpEncodeData:dataToSign options:ioOptions recipients:nil];
    NS_HANDLER
        [newHeaders release];
        [headersToSign release];
        [localException raise];
    NS_ENDHANDLER

    if(!ioOptions->mEncapsulatedSignature)
    {
        [newHeaders removeHeaderForKey:kPGPmailHeaderContentType];
        [newHeaders removeHeaderForKey:kPGPmailHeaderTransferEncoding];
        [newHeaders setHeader:[NSString stringWithFormat:@"multipart/signed; protocol=\"application/pgp-signature\";\n\tmicalg=%@; boundary=\"%@\"", [@"pgp-" stringByAppendingString:[theEncoder pgpDefaultHashAlgorithm]], newBoundary] forKey:kPGPmailHeaderContentType];
        [newHeaders setHeader:kPGPmailHeader7Bit forKey:kPGPmailHeaderTransferEncoding];
        [[[self message] headers] flushCachedData];
        [[self message] setHeaders:newHeaders];
    }
    [newHeaders release];
    
    signedData = [NSMutableData data];
    if(!ioOptions->mEncapsulatedSignature)
        [signedData appendData:[@"--" dataUsingEncoding:NSASCIIStringEncoding]];
    else{
        [signedData appendData:[[NSString stringWithFormat:@"Content-type: multipart/signed; protocol=\"application/pgp-signature\";\n\tmicalg=%@; boundary=\"%@\"\n", [@"pgp-" stringByAppendingString:[theEncoder pgpDefaultHashAlgorithm]], newBoundary] dataUsingEncoding:NSASCIIStringEncoding]];
        [signedData appendData:[@"Content-transfer-encoding: 7bit\n" dataUsingEncoding:NSASCIIStringEncoding]];
        [signedData appendData:[[@"X-PGP-Agent: PGPMail " stringByAppendingString:[[PGPMailBundle sharedInstance] pgpVersion]] dataUsingEncoding:NSASCIIStringEncoding]];
        
        [signedData appendData:[@"\n\n--" dataUsingEncoding:NSASCIIStringEncoding]];
    }
    [signedData appendData:[newBoundary dataUsingEncoding:NSASCIIStringEncoding]];
    [signedData appendData:[@"\n" dataUsingEncoding:NSASCIIStringEncoding]];
    {
        NSMutableData	*convertedData = [NSMutableData dataWithData:dataToSign];

        [convertedData convertNetworkLineEndingsToUnix];
        [signedData appendData:convertedData];
    }
    [signedData appendData:[[NSString stringWithFormat:@"\n--%@\n", newBoundary] dataUsingEncoding:NSASCIIStringEncoding]];
    [headersToSign release];

    signatureDataIsOnlyASCII = ![signatureData pgpContainsNonASCIICharacter];
    [signedData appendData:[@"content-type: application/pgp-signature; x-mac-type=70674453;\n\tname=PGP.sig" dataUsingEncoding:NSASCIIStringEncoding]];
    if(!signatureDataIsOnlyASCII)
        [signedData appendData:[[@"; charset=" stringByAppendingString:(NSString *)CFStringConvertEncodingToIANACharSetName(kCFStringEncodingUTF8)] dataUsingEncoding:NSASCIIStringEncoding]];
    [signedData appendData:[@"\ncontent-disposition: inline\n" dataUsingEncoding:NSASCIIStringEncoding]];
    if(!signatureDataIsOnlyASCII)
    {
        signatureData = [signatureData encodeQuotedPrintableForText:YES];
        [signedData appendData:[@"content-transfer-encoding: quoted-printable\n\n" dataUsingEncoding:NSASCIIStringEncoding]];
    }
    else
        [signedData appendData:[@"content-transfer-encoding: 7bit\n\n" dataUsingEncoding:NSASCIIStringEncoding]];
    [signedData appendData:signatureData];
    [signedData appendData:[[NSString stringWithFormat:@"\n--%@--\n", newBoundary] dataUsingEncoding:NSASCIIStringEncoding]];
    
    return [[signedData retain] autorelease];
}

@end
