/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation.
    All rights reserved.

    $Id: PGPMailMimePart.m,v 1.10 2002/10/30 12:14:49 jason Exp $
____________________________________________________________________________*/

#import <PGP/PGP.h>
#import <PGPclient/pgpClientPrefs.h>

#import "PGPMailMimePart.h"
#import "PGPMailNSData.h"

#import "PGPMailEncodeDecode.h"

#import <MimeBody.h>
#import <NSDataMessage.h>
#import <Message.h>
#import <MessageStore.h>

#import <Cocoa/Cocoa.h>
#import "PGPMailBundle.h"

#define kPGPmailMessageContentUnavailable		LSPV(@"The message content not available at this time.", @"MESSAGE_CONTENT_UNAVAILABLE")
#define kPGPmailMessageUnableToFind				LSPV(@"Unable to find application/octet-stream.", @"MESSAGE_MISSING_OCTET_STREAM")
#define kPGPmailMessageUnableToDecryptVerify	LSPV(@"PGPMail is unable to decrypt/verify this message.", @"MESSAGE_UNABLE_TO_DERYPT_VERIFY")

@interface MimePart(PGPMailPrivate)
- (BOOL)pgpIsEncrypted;
- (BOOL)pgpIsSigned;
- (BOOL)pgpIsEncrypted:(PGPMailDecodeOptions *)ioOptions;
- (BOOL)pgpIsSigned:(PGPMailDecodeOptions *)ioOptions;
- (NSData *)pgpDecryptedData:(PGPMailDecodeOptions *)ioOptions;
- (NSData *)pgpVerifiedData:(PGPMailDecodeOptions *)ioOptions;
- (NSData *)pgpFullBodyPartData;
@end

@implementation MimePart(PGPMail)

- (BOOL)isPGPMessage
{
    return ([self pgpIsEncrypted] || [self pgpIsSigned]);
}

- (BOOL)isPGPMessage:(PGPMailDecodeOptions *)ioOptions
{
    return ([self pgpIsEncrypted:ioOptions] || [self pgpIsSigned:ioOptions]);
}

- (NSData *)pgpBodyPartData
{
    NSData	*bodyPartData;
    
    if([[self mimeBody] topLevelPart] == self)
    {
        bodyPartData = [[[[self mimeBody] message] messageStore] fullBodyDataForMessage:[[self mimeBody] message]];
        if([[[self contentTransferEncoding] lowercaseString] isEqualToString:kPGPmailHeaderQuotedPrintable])
            bodyPartData = [bodyPartData decodeQuotedPrintableForText:YES];
        else if([[[self contentTransferEncoding] lowercaseString] isEqualToString:kPGPmailHeaderBase64])
            bodyPartData = [bodyPartData decodeBase64];
    }
    else
    {
        NSString	*boundary = [NSString stringWithFormat:@"--%@\r\n", [[self parentPart] bodyParameterForKey:kPGPmailHeaderBoundry]];

        bodyPartData = [[[[[self mimeBody] rawData] pgpConvertEndOfLines] componentsSeparatedByData:[boundary dataUsingEncoding:NSASCIIStringEncoding]] objectAtIndex:[[[self parentPart] subparts] indexOfObject:self] + 1];
        {
            NSRange	range = {0, [bodyPartData length] - 2};
            bodyPartData = [bodyPartData subdataWithRange:range];
        }
        if([[[self contentTransferEncoding] lowercaseString] isEqualToString:kPGPmailHeaderQuotedPrintable])
            bodyPartData = [bodyPartData decodeQuotedPrintableForText:YES];
        else if([[[self contentTransferEncoding] lowercaseString] isEqualToString:kPGPmailHeaderBase64])
            bodyPartData = [bodyPartData decodeBase64];
    }
    
    return bodyPartData;
}

- (NSData *)pgpBodyPartSubdata
{
    if([[self subparts] count] == 0)
    {
        NSData	*bodyPartData = [[[[self mimeBody] message] messageStore] dataForMimePart:self];

        if([[[self contentTransferEncoding] lowercaseString] isEqualToString:kPGPmailHeaderQuotedPrintable])
            bodyPartData = [bodyPartData decodeQuotedPrintableForText:YES];
        else if([[[self contentTransferEncoding] lowercaseString] isEqualToString:kPGPmailHeaderBase64])
            bodyPartData = [bodyPartData decodeBase64];

        return bodyPartData;
    }
    else
    {
        return nil;
    }
}

- (BOOL)pgpIsOpenPGPEncryptedContainerPart
{
    if([[[self type] lowercaseString] isEqualToString:kPGPmailHeaderMultipart] && [[[self subtype] lowercaseString] isEqualToString:kPGPmailHeaderEncrypted])
        if([self bodyParameterForKey:kPGPmailHeaderProtocol] == nil || [[[self bodyParameterForKey:kPGPmailHeaderProtocol] lowercaseString] isEqualToString:kPGPmailHeaderPGPAppEncrypted])
            if([[self subparts] count] == 2)
            {
                BOOL			recognizesVersion = NO;
                MimePart	*thePart = [self subpartAtIndex:0];

                if([[[thePart type] lowercaseString] isEqualToString:kPGPmailHeaderApplication] && [[[thePart subtype] lowercaseString] isEqualToString:kPGPmailHeaderPGPEncrypted])
                {
                    NSData	*bodyData = [thePart bodyData];

                    if(bodyData)
                    {
                        NSString	*string = [[NSString alloc] initWithData:bodyData encoding:NSASCIIStringEncoding];
                        if([[string lowercaseString] isEqualToString:@"version: 1\n"] || [[string lowercaseString] isEqualToString:@"version : 1\n"] || [[string lowercaseString] isEqualToString:kPGPmailHeaderVersion])
                            recognizesVersion = YES;
                        [string release];
                    }
                }

                if(recognizesVersion)
                    return YES;
            }

    return NO;
}

- (BOOL)pgpIsNonOpenPGPEncryptedPart
{
    NSData	*bodyData;

    if ((([self type] && ![[[self type] lowercaseString] isEqualToString:kPGPmailHeaderText]) || ([self subtype] && ![[[self subtype] lowercaseString] isEqualToString:kPGPmailHeaderPlain])))
		return NO;
    
    bodyData = [self bodyData];
    if(bodyData != nil)
		if ([PGPMailEncodeDecode pgpEncryptionBlockRangeInData:[bodyData pgpConvertEndOfLines]].length > 0)
			return YES;
		else
			return [PGPMailEncodeDecode pgpKeyBlockRangeInData:[bodyData pgpConvertEndOfLines]].length > 0;
    else
        return NO;
}


- (BOOL)pgpIsOpenPGPSignedContainerPart
{
    MimePart	*thePart;
    NSArray		*parts;
    NSString	*hashAlgorithmName;

    if(![[[self type] lowercaseString] isEqualToString:kPGPmailHeaderMultipart] && ![[[self subtype] lowercaseString] isEqualToString:kPGPmailHeaderSigned])
        return NO;

    if([self bodyParameterForKey:kPGPmailHeaderProtocol] && ![[[self bodyParameterForKey:kPGPmailHeaderProtocol] lowercaseString] isEqualToString:KPGPmailHeaderPGPAppSignature])
        return NO;
    hashAlgorithmName = [[self bodyParameterForKey:kPGPmailHeaderAlgorithm] lowercaseString];
    parts = [self subparts];
    if([parts count] != 2)
        return NO;

    thePart = [parts objectAtIndex:1];
    if([[[thePart type] lowercaseString] isEqualToString:kPGPmailHeaderApplication] && [[[thePart subtype] lowercaseString] isEqualToString:kPGPmailHeaderPGPSignature])
        return YES;
    else
        return NO;
}

- (BOOL)pgpIsNonOpenPGPSignedPart
{
    NSData	*bodyData;

	if(([self type] && ![[[self type] lowercaseString] isEqualToString:kPGPmailHeaderText]) || ([self subtype] && ![[[self subtype] lowercaseString] isEqualToString:kPGPmailHeaderPlain]))
		return NO;
    bodyData = [[self bodyData] pgpConvertEndOfLines];
    if(bodyData != nil)
        return [PGPMailEncodeDecode pgpSignatureBlockRangeInData:bodyData].length > 0;
    else
        return NO;
}

- (NSData *)pgpDecryptedSubdata:(PGPMailDecodeOptions *)ioOptions
{
    NSData	*bodyPartData = [self pgpBodyPartSubdata];
    
    if(bodyPartData != nil)
        return [[PGPMailEncodeDecode handler] pgpDecryptVerifyData:bodyPartData options:ioOptions];
    else{
        [NSException raise:NSGenericException format:kPGPmailMessageContentUnavailable];
        return nil;
    }
}

- (NSData *)pgpDecryptedInlineData:(PGPMailDecodeOptions *)ioOptions
{
    NSData			*data = [[self bodyData] pgpConvertEndOfLines];
    NSRange			pgpRange;
    NSData			*pgpData, *decryptedData;
    NSMutableData	*newEncodedBody;
    
	pgpRange = [PGPMailEncodeDecode pgpEncryptionBlockRangeInData:data];
    if( 0 == pgpRange.length )
        pgpRange = [PGPMailEncodeDecode pgpSignedBlockRangeInData:data];
	if (0 == pgpRange.length)
		pgpRange = [PGPMailEncodeDecode pgpKeyBlockRangeInData:data];
    NSAssert(pgpRange.length > 0, @"Not encrypted!");

    pgpData = [data subdataWithRange:pgpRange];
    decryptedData = [[PGPMailEncodeDecode handler] pgpDecryptVerifyData:pgpData options:ioOptions];

    if( nil != decryptedData )
    {
        newEncodedBody = [NSMutableData dataWithCapacity:[data length]];
        [newEncodedBody appendData:[data subdataWithRange:NSMakeRange(0, pgpRange.location)]];
        [newEncodedBody appendData:decryptedData];
        [newEncodedBody appendData:[data subdataWithRange:NSMakeRange(NSMaxRange(pgpRange), [data length] - NSMaxRange(pgpRange))]];
        [newEncodedBody convertNetworkLineEndingsToUnix];
        if([[[self contentTransferEncoding] lowercaseString] isEqualToString:kPGPmailHeaderQuotedPrintable])
            [newEncodedBody setData:[newEncodedBody encodeQuotedPrintableForText:YES]];
        else if([[[self contentTransferEncoding] lowercaseString] isEqualToString:kPGPmailHeaderBase64])
            [newEncodedBody setData:[newEncodedBody encodeBase64]];
        {
            NSMutableData	*prefixData = [[NSMutableData alloc] initWithData:[@"\n" dataUsingEncoding:NSASCIIStringEncoding]];
    
            [prefixData appendData:newEncodedBody];
            [newEncodedBody setData:prefixData];
            [prefixData release];
        }
        return newEncodedBody;
    }
    return nil;
}

- (void) pgpDecryptInlineData:(NSMutableData *)decryptedData fromData:(NSData *)fullBodyData options:(PGPMailDecodeOptions *)ioOptions
{
    NSData	*partDecryptedData = [self pgpDecryptedInlineData:ioOptions];

    if(nil != partDecryptedData)
    {
        NSRange	partRange = [self range];

        [decryptedData setData:[fullBodyData subdataWithRange:NSMakeRange(0, partRange.location)]];
        [decryptedData appendData:partDecryptedData];
        [decryptedData appendData:[fullBodyData subdataWithRange:NSMakeRange(NSMaxRange(partRange), [fullBodyData length] - (NSMaxRange(partRange)))]];
    }
}

- (NSData *)pgpDecryptedVerifiedData:(PGPMailDecodeOptions *)ioOptions
{
    if( ioOptions->mIsEncrypted )
        return [self pgpDecryptedData:ioOptions];
    else if( ioOptions->mIsSigned )
        return [self pgpVerifiedData:ioOptions];
    else
        return nil;
}

- (NSData *)pgpDecryptedData:(PGPMailDecodeOptions *)ioOptions;
{
    NSArray	*parts = [self subparts];
    int		i = [parts count];

    if( ioOptions->mIsOpenPGPEncrypted )
    {
        MimePart	*thePart = [parts objectAtIndex:1];

        if([[[thePart type] lowercaseString] isEqualToString:kPGPmailHeaderApplication] && [[[thePart subtype] lowercaseString] isEqualToString:kPGPmailHeaderOctetStream])
        {
            NSData	*decryptedData = [thePart pgpDecryptedSubdata:ioOptions];
            return decryptedData;
        }

        [NSException raise:NSInternalInconsistencyException format:kPGPmailMessageUnableToFind];
        
        return nil;
    }
    else if(i > 0)
    {
        int				index;
        NSMutableData	*decryptedData = nil;
        NSData			*fullBodyData = nil;

        for(index = 0; index < i; index++)
        {
            MimePart	*thePart = [parts objectAtIndex:index];

            if([thePart pgpIsNonOpenPGPEncryptedPart])
            {
                if(!fullBodyData)
                {
                    fullBodyData = [[[[self mimeBody] message] messageStore] fullBodyDataForMessage:[[self mimeBody] message]];
                    decryptedData = [NSMutableData dataWithData:fullBodyData];
                }
                [thePart pgpDecryptInlineData:decryptedData fromData:fullBodyData options:ioOptions];
				if (IsPGPError(ioOptions->mDecodeError)) {
					decryptedData = nil;
					break;
				}
            }
        }
        
		if (ioOptions->mDecodeError != kPGPError_UserAbort) {
			if(!decryptedData)
			{
				NSRunAlertPanel(kPGPmailUtilDecodeErrorStr, kPGPmailUtilNotPlaintextStr, kPGPmailCancelButton, nil, nil);
				NSLog(kPGPmailMessageUnableToDecryptVerify);
				[NSException raise:NSInternalInconsistencyException format:kPGPmailMessageUnableToDecryptVerify];
			}
			else
			{
				NSMutableData	*prefixData = [[NSMutableData alloc] initWithData:[@"\n" dataUsingEncoding:NSASCIIStringEncoding]];
	
				[prefixData appendData:decryptedData];
				[decryptedData setData:prefixData];
				[prefixData release];
			}
		}
        
        return decryptedData;
    }
    else if( ioOptions->mIsEncrypted )
        return [self pgpDecryptedInlineData:ioOptions];
    else
        return nil;
}

- (NSData *)pgpVerifiedData:(PGPMailDecodeOptions *)ioOptions
{
    NSArray		*parts = [self subparts];
    int			count = [parts count];

    if( ioOptions->mIsOpenPGPSigned )
    {
        MimePart			*thePart = nil;
        NSMutableData		*dataToDecode = [NSMutableData data];
        NSMutableData		*signatureData = [NSMutableData data];
       	NSData				*signature = nil;

        thePart = [parts objectAtIndex:0];
        [dataToDecode appendData:[[thePart pgpFullBodyPartData] pgpNormalizedDataForVerifying]];

        thePart = [parts objectAtIndex:1];
        [signatureData appendData:[thePart bodyData]];
        [signatureData pgpNormalizeDataForVerifying];
        
        ioOptions->mStringEncoding = [self textEncoding];
        NS_DURING
            signature = [[PGPMailEncodeDecode handler] pgpVerifyMIMEData:dataToDecode signature:signatureData options:ioOptions];
         NS_HANDLER
            [localException raise];
        NS_ENDHANDLER
        
        return signature;
    }
    else if(count > 0)
    {
        int				index;
        NSMutableData	*theSignature = nil;
        NSData			*fullBodyData = nil;

        for(index = 0; index < count; index++)
        {
            MimePart	*thePart = [parts objectAtIndex:index];

            if([thePart pgpIsNonOpenPGPSignedPart])
            {
                if(!fullBodyData)
                {
                    fullBodyData = [[[[self mimeBody] message] messageStore] fullBodyDataForMessage:[[self mimeBody] message]];
                    theSignature = [NSMutableData dataWithData:fullBodyData];
                }
                [thePart pgpDecryptInlineData:theSignature fromData:fullBodyData options:ioOptions];
				if (IsPGPError(ioOptions->mDecodeError)) {
					theSignature = nil;
					break;
				}
            }
        }
        
		if (ioOptions->mDecodeError != kPGPError_UserAbort) {
			if(!theSignature)
			{
				NSRunAlertPanel(kPGPmailUtilDecodeErrorStr, kPGPmailUtilNotPlaintextStr, kPGPmailCancelButton, nil, nil);
				NSLog(kPGPmailMessageUnableToDecryptVerify);
				[NSException raise:NSInternalInconsistencyException format:kPGPmailMessageUnableToDecryptVerify];
			}
			else
			{
				NSMutableData	*prefixData = [[NSMutableData alloc] initWithData:[@"\n" dataUsingEncoding:NSASCIIStringEncoding]];
	
				[prefixData appendData:theSignature];
				[theSignature setData:prefixData];
				[prefixData release];
			}
		}
        
        return theSignature;


/*        for(count = count - 1; count >= 0; count--)
        {
            NSData	*theSignature = [[parts objectAtIndex:count] pgpVerifiedData:ioOptions];
            
            if(theSignature != nil)
                return theSignature;
        }
        return nil;
*/    }
    else if( ioOptions->mIsSigned )
        return [self pgpDecryptedInlineData:ioOptions];
    else
        return nil;
}

- (BOOL)pgpAllAttachmentsAreAvailable
{
    NSArray	*subparts = [self subparts];
    int		subpartsCount = [subparts count];
    
    if(subpartsCount == 0)
        return [[[[self mimeBody] message] messageStore] hasCachedDataForMimePart:self];
    else{
        for(--subpartsCount; subpartsCount >= 0; subpartsCount--)
            if(![[subparts objectAtIndex:subpartsCount] pgpAllAttachmentsAreAvailable])
                return NO;
        return YES;
    }
}

- (BOOL)pgpIsEncrypted
{
    int			count = 0;
    NSArray		*parts = nil;

    if([self pgpIsOpenPGPEncryptedContainerPart])
        return YES;

    parts = [self subparts];
    count = [parts count];
    if(count > 0)
    {
        for(count = count - 1; count >= 0; count--)
            if([[parts objectAtIndex:count] pgpIsEncrypted])
                return YES;
    }
    else if([self pgpIsNonOpenPGPEncryptedPart])
        return YES;

    return NO;
}

- (BOOL)pgpIsEncrypted:(PGPMailDecodeOptions *)ioOptions
{
    int			count = 0;
    NSArray		*parts = nil;

    if([self pgpIsOpenPGPEncryptedContainerPart])
    {
        ioOptions->mIsEncrypted = YES;
        ioOptions->mIsOpenPGPEncrypted = YES;
        return ioOptions->mIsEncrypted;
    }
    
    parts = [self subparts];
    count = [parts count];
    if(count > 0)
    {
        for(count = count - 1; count >= 0; count--)
            if([[parts objectAtIndex:count] pgpIsEncrypted:ioOptions])
                return YES;
    }
    else if([self pgpIsNonOpenPGPEncryptedPart])
    {
        ioOptions->mIsEncrypted = YES;
        ioOptions->mIsOpenPGPEncrypted = NO;
        return ioOptions->mIsEncrypted;
    }
    
    return NO;
}

- (BOOL)pgpIsSigned:(PGPMailDecodeOptions *)ioOptions
{
    int			count = 0;
    NSArray		*parts = nil;

    if([self pgpIsOpenPGPSignedContainerPart])
    {
        ioOptions->mIsSigned = YES;
        ioOptions->mIsOpenPGPSigned = YES;
        return ioOptions->mIsSigned;
    }

    parts = [self subparts];
    count = [parts count];
    if(count > 0)
    {
        for(count = count - 1; count >= 0; count--)
            if([[parts objectAtIndex:count] pgpIsSigned:ioOptions])
                return YES;
    }
    else if([self pgpIsNonOpenPGPSignedPart])
    {
        ioOptions->mIsSigned = YES;
        ioOptions->mIsOpenPGPSigned = NO;
        return ioOptions->mIsSigned;
    }

    return NO;
}

- (BOOL)pgpIsSigned
{
    int			count = 0;
    NSArray 	*parts = nil;

    if([self pgpIsOpenPGPSignedContainerPart])
        return YES;

    parts = [self subparts];
    count = [parts count];
    if(count > 0)
    {
        for(count = count - 1; count >= 0; count--)
            if([[parts objectAtIndex:count] pgpIsSigned])
                return YES;
    }
    else if([self pgpIsNonOpenPGPSignedPart])
        return YES;

    return NO;
}

- (NSData *)pgpFullBodyPartData
{
    NSData	*bodyPartData;
    
    if([[self mimeBody] topLevelPart] == self)
    {
        bodyPartData = [[[[self mimeBody] message] messageStore] fullBodyDataForMessage:[[self mimeBody] message]];
    }
    else
    {
        NSString	*boundary = [NSString stringWithFormat:@"--%@\r\n", [[self parentPart] bodyParameterForKey:kPGPmailHeaderBoundry]];
        char		bytes[2];
        
        bodyPartData = [[[[[self mimeBody] message] messageStore] fullBodyDataForMessage:[[self mimeBody] message]] pgpConvertEndOfLines];
        
        [bodyPartData getBytes:bytes length:2];
        if(bytes[0] != '\r' && bytes[1] != '\n')
        {
            NSMutableData	*newData;
            
            bytes[0] = '\r';
            bytes[1] = '\n';
            newData = [NSMutableData dataWithBytes:bytes length:2];
            
            [newData appendData:bodyPartData];
            bodyPartData = newData;
        }
        bodyPartData = [[bodyPartData componentsSeparatedByData:[boundary dataUsingEncoding:NSASCIIStringEncoding]] objectAtIndex:[[[self parentPart] subparts] indexOfObject:self] + 1];
        {
            NSRange	range = {0, [bodyPartData length] - 2};
            bodyPartData = [bodyPartData subdataWithRange:range];
        }
    }
    
    return bodyPartData;
}
@end
