/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation.
    All rights reserved.

    $Id: PGPMailNSData.m,v 1.4 2002/10/16 04:18:16 jason Exp $
____________________________________________________________________________*/

#import "PGPMailNSData.h"
#import <Foundation/NSException.h>
#import <Foundation/NSString.h>
#import <NSDataMessage.h>

@implementation NSData(PGPMail)
- (NSData *)pgpConvertEndOfLines
{
#define kPGPmailCR	'\r'
#define kPGPmailLF	'\n'
    unsigned	length = [self length];
    NSData		*result;

    if(length > 0)
    {
        unsigned			index = 0, newLength = 0;
        BOOL				foundCR = NO;
        const unsigned char	*oldBytes;
        unsigned char		theByte, *newBytes;

        oldBytes = [self bytes];
        newBytes = NSZoneMalloc(NSDefaultMallocZone(), 2 * length);

        for(index = 0; index < length; index++)
        {
            theByte = oldBytes[index];
            if(theByte == kPGPmailLF)
            {
                if(!foundCR)
                {
                    newBytes[newLength++] = kPGPmailCR;
                    newBytes[newLength++] = kPGPmailLF;
                }
                else
                {
                    newBytes[newLength++] = kPGPmailLF;
                    foundCR = NO;
                }
            }
            else
            {
                if(foundCR)
                    newBytes[newLength++] = kPGPmailLF;
                newBytes[newLength++] = theByte;
                foundCR = (theByte == kPGPmailCR);
            }
        }

        result = [NSData dataWithBytes:newBytes length:newLength];
        NSZoneFree(NSDefaultMallocZone(), newBytes);
    }
    else
        result = [NSData data];

    return result;
#undef kPGPmailLF
#undef kPGPmailCR
}

- (BOOL)pgpContainsNonASCIICharacter
{
    unsigned	index = [self length];

    if(index > 0)
    {
        const char	*bytes = [self bytes];

        do
        {
            if(bytes[--index] & 0x80)
                return YES;
        }while(index != 0);
    }
    return NO;
}

- (NSData *)pgpNormalizedDataForVerifying
{
    NSMutableData	*result = [NSMutableData dataWithData:self];
    
    [result pgpNormalizeDataForVerifying];
    
    return result;
}

- (NSRange)pgpHeaderBodySeparationRange
{
#define kPGPmailCR	'\r'
#define kPGPmailLF	'\n'
    const unsigned char	*bytes = [self bytes];
    const int			length = [self length];
    int					index = 0;
    NSRange				result = NSMakeRange(NSNotFound, 0);
    
    for(index = 0; index < length; index++)
    {
        if(bytes[index] == kPGPmailCR)
        {
            if(index + 1 < length)
            {
                if(bytes[index + 1] == kPGPmailCR)
                {
                    result.location = index + 1;
                    result.length = 1;
                    break;
                }
                if(bytes[index + 1] == kPGPmailLF)
                    if(index + 3 < length)
                        if(bytes[index + 2] == kPGPmailCR && bytes[index + 3] == kPGPmailLF)
                        {
                            result.location = index + 2;
                            result.length = 2;
                            break;
                        }
            }
            if(index == 0)
            {
                result.location = 0;
                result.length = 1;
                break;
            }
        }
        else if(bytes[index] == kPGPmailLF)
        {
            if(index + 1 < length)
                if(bytes[index + 1] == kPGPmailLF)
                {
                    result.location = index + 1;
                    result.length = 1;
                    break;
                }
            if(index == 0)
            {
                result.location = 0;
                result.length = 1;
                break;
            }
        }
    }
    
    return result;
#undef kPGPmailLF
#undef kPGPmailCR
}
@end

@implementation NSMutableData(PGPMail)

- (void)pgpNormalizeDataForSigning
{
    [self setData:[self pgpConvertEndOfLines]];
}

- (void)pgpNormalizeDataForVerifying
{
    [self setData:[self pgpConvertEndOfLines]];
}

- (BOOL)pgpApplyQuotedPrintableIfNeeded:(BOOL)alreadyUsesQuotedPrintable
{
    BOOL	needsQuotedPrintable = NO;
    
    if([self pgpContainsNonASCIICharacter])
    {
        NSParameterAssert(alreadyUsesQuotedPrintable == NO);
        needsQuotedPrintable = YES;
        NSLog(@"PGPMail: signature will be invalid. Cannot sign 8bit data in OpenPGP.");
    }
    else
    {
        NSString	*aString = [[NSString alloc] initWithData:self encoding:NSASCIIStringEncoding];
    
        needsQuotedPrintable = ([aString hasPrefix:@"From "] || [aString rangeOfString:@"\r\nFrom " options:NSLiteralSearch].length > 0);
        [aString release];
    }
    
    if(needsQuotedPrintable)
    {
        NSLog(@"PGPMail: Verification might not work.");
        needsQuotedPrintable = NO;
    }
    
    return needsQuotedPrintable && !alreadyUsesQuotedPrintable;
}

@end
