/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation.
    All rights reserved.

    $Id: PGPMailPreferences.m,v 1.2 2002/11/15 10:17:32 wprice Exp $
____________________________________________________________________________*/

#import <PGP/PGP.h>
#import <PGPOpenPrefs.h>
#import <PGPclient/pgpClientLibOSX.h>
#import <PGPclient/pgpClientLibCocoa.h>

#import "PGPMailCommon.h"
#import "PGPMailPreferences.h"
#import "PGPMailBundle.h"

@implementation PGPMailPreferences
- (id)init
{
    if( !mInited )
    {
        [super init];
        mInited = YES;
    }
    	
	if( !mPrefRef )
		[self pgpOpenPrefs];
        
	return self;
}

- (id)initWithNib:(NSString *)inNibFile
{
    if( inNibFile && [inNibFile length] )
        [NSBundle loadNibNamed:inNibFile owner:self];

	mLoaded = YES;
	
    return [self init];
}

- (id)initWithController:(PGPMailBundle *)inController
{
    mAppController = inController;
    
    return [self init];
}

- (void)dealloc
{
	[self pgpClosePrefs];
	[super dealloc];
}

- (PGPPrefRef)pgpPrefRef
{
    return mPrefRef;
}

- (void)pgpSetPrefRef:(PGPPrefRef)inPrefRef
{
    if( PGPPrefRefIsValid(inPrefRef) )
    {
        if( PGPPrefRefIsValid(mPrefRef) )
             PGPFreePrefs(mPrefRef);
		
        mPrefRef = inPrefRef;
    }
}

- (PGPBoolean)pgpAlwaysEncryptToDefaultKey
{
    PGPBoolean			theData = FALSE;
    PGPError			theError = kPGPError_NoErr;
    
    if( IsPGPError(theError = PGPGetPrefBoolean(mPrefRef, kPGPPrefEncryptToSelf, &theData)) )
        PGPLog((kPGPmailErrorRetrieve, kPGPPrefEncryptToSelf, theError));
    
    return theData;
}

- (PGPUInt32)pgpCacheType
{
    PGPUInt32			theData = 0;
    PGPError			theError = kPGPError_NoErr;
    
    if( IsPGPError(theError = PGPGetPrefNumber(mPrefRef, kPGPPrefCacheType, &theData)) )
        PGPLog((kPGPmailErrorRetrieve, kPGPPrefCacheType, theError));
    
    return theData;
}

- (PGPUInt32)pgpCacheSeconds
{
    PGPUInt32			theData = 0;
    PGPError			theError = kPGPError_NoErr;
    
    if( IsPGPError(theError = PGPGetPrefNumber(mPrefRef, kPGPPrefCacheSeconds, &theData)) )
        PGPLog((kPGPmailErrorRetrieve, kPGPPrefCacheSeconds, theError));
    
    return theData;
}

- (PGPBoolean)pgpSharePassphraseCacheAmongModules
{
    PGPBoolean			theData = FALSE;
    PGPError			theError = kPGPError_NoErr;
    
    if( IsPGPError(theError = PGPGetPrefBoolean(mPrefRef, kPGPPrefShareCache, &theData)) )
        PGPLog((kPGPmailErrorRetrieve, kPGPPrefShareCache, theError));
    
    return theData;
}

- (PGPBoolean)pgpUsePGPMimeWhenSendingMail
{
    PGPBoolean			theData = FALSE;
    PGPError			theError = kPGPError_NoErr;
    
    if( IsPGPError(theError = PGPGetPrefBoolean(mPrefRef, kPGPPrefMailSignPGPMIME, &theData)) )
        PGPLog((kPGPmailErrorRetrieve, kPGPPrefMailSignPGPMIME, theError));
    
    return theData;
}

- (PGPBoolean)pgpEncryptNewMessagesByDefault
{
    PGPBoolean			theData = FALSE;
    PGPError			theError = kPGPError_NoErr;
    
    if( IsPGPError(theError = PGPGetPrefBoolean(mPrefRef, kPGPPrefMailEncryptDefault, &theData)) )
        PGPLog((kPGPmailErrorRetrieve, kPGPPrefMailEncryptDefault, theError));
    
    return theData;
}

- (PGPBoolean)pgpSignNewMessagesByDefault
{
    PGPBoolean			theData = FALSE;
    PGPError			theError = kPGPError_NoErr;
    
    if( IsPGPError(theError = PGPGetPrefBoolean(mPrefRef, kPGPPrefMailSignDefault, &theData)) )
        PGPLog((kPGPmailErrorRetrieve, kPGPPrefMailSignDefault, theError));
    
    return theData;
}

- (PGPBoolean)pgpUsesEncapsulatedSignature
{
    return TRUE;
}

- (PGPBoolean)pgpAutoDecryptVerifyWhenOpeningMessges
{
    PGPBoolean			theData = FALSE;
    PGPError			theError = kPGPError_NoErr;
    
    if( IsPGPError(theError = PGPGetPrefBoolean(mPrefRef, kPGPPrefAutoDecrypt, &theData)) )
        PGPLog((kPGPmailErrorRetrieve, kPGPPrefAutoDecrypt, theError));
    
    return theData;
}

- (PGPBoolean)pgpWordWrapCleanSignedEnabled
{
    PGPBoolean			theData = FALSE;
    PGPError			theError = kPGPError_NoErr;
    
    if( IsPGPError(theError = PGPGetPrefBoolean(mPrefRef, kPGPPrefWordWrapEnable, &theData)) )
        PGPLog((kPGPmailErrorRetrieve, kPGPPrefWordWrapEnable, theError));
    
    return theData;
}

- (PGPUInt32)pgpWordWrapWidth
{
    PGPUInt32			theData = 0;
    PGPError			theError = kPGPError_NoErr;
    
    if( IsPGPError(theError = PGPGetPrefNumber(mPrefRef, kPGPPrefWordWrapWidth, &theData)) )
        PGPLog((kPGPmailErrorRetrieve, kPGPPrefWordWrapWidth, theError));
    
    return theData;
}

- (PGPUInt32)pgpPreferredAlgorithm
{
    PGPUInt32			theData = 0;
    PGPError			theError = kPGPError_NoErr;
    
    if( IsPGPError(theError = PGPGetPrefNumber(mPrefRef, kPGPPrefPreferredAlgorithm, &theData)) )
        PGPLog((kPGPmailErrorRetrieve, kPGPPrefPreferredAlgorithm, theError));
    
    return theData;
}

- (PGPPrefArray*)pgpAllowedAlgorithms
{
	PGPPrefArray		*theData = NULL;
	PGPError			theError = kPGPError_NoErr;
	
	if( IsPGPError(theError = PGPGetPrefArray(mPrefRef, kPGPPrefAllowedAlgorithmsList, &theData)) )
		PGPLog((kPGPmailErrorRetrieve, kPGPPrefAllowedAlgorithmsList, theError));
	
	return theData;
}

- (PGPError)pgpOpenPrefs
{
	PGPError		theError = kPGPError_NoErr;
	PGPPrefRef		thePrefRef = kInvalidPGPPrefRef;
	
	if( !mAppController )
		return kPGPError_BadParams;

	theError = PGPclOpenPrefs(PGPPeekContextMemoryMgr([mAppController pgpContext]),
                        kPGPclPrefsClientKey, nil, &thePrefRef, TRUE);

	if( IsPGPError(theError) )
		return theError;
	else
		[self pgpSetPrefRef:thePrefRef];
		
	return kPGPError_NoErr;
}

- (PGPError)pgpClosePrefs
{
	PGPError		theError = kPGPError_NoErr;
	
	if( !PGPPrefRefIsValid(mPrefRef) )
		return kPGPError_BadParams;
	
	PGPFreePrefs(mPrefRef);
	mPrefRef = kInvalidPGPPrefRef;
	
	return theError;
}

@end
