/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation.
    All rights reserved.

    $Id: TableViewManager.h,v 1.2 2002/08/24 01:32:31 wmalik Exp $
____________________________________________________________________________*/

#import <Foundation/NSObject.h>

@class ASExtendedTableView;
@class MessageStore;
@class NSFont;
@class Message;
@class NSArray;
@class NSMutableArray;
@class NSMutableAttributedString;
@class NSDictionary;
@class MessageMall;
@class NSMutableDictionary;

@protocol MVSelectionOwner
- (char)transferSelectionToMailbox:fp12 deleteOriginals:(char)fp16;
- messageStore;
- currentDisplayedMessage;
- (void)selectMessages:fp12;
- selection;
@end

@interface TableViewManager:NSObject <MVSelectionOwner>
{
    ASExtendedTableView *_tableView;
    MessageMall *_messageMall;
    id _delegate;
    NSArray *_tableColumns;
    NSFont *_font;
    NSArray *_selection;
    char _suspendChangesToScrollingAndSelection;
    Message *_currentDisplayedMessage;
    struct _NSMapTable *_storeStateTable;
    NSMutableArray *_draggedMessages;
    NSMutableAttributedString *_unreadAttrString;
    NSMutableAttributedString *_flaggedAttrString;
    NSMutableAttributedString *_junkMailFlagAttrString;
    NSMutableAttributedString *_attachmentAttrString;
    NSMutableAttributedString *_attachmentSelectedAttrString;
    NSMutableAttributedString *_repliedAttrString;
    NSMutableAttributedString *_forwardedAttrString;
    NSMutableAttributedString *_redirectedAttrString;
    NSMutableAttributedString *_repliedSelectedAttrString;
    NSMutableAttributedString *_forwardedSelectedAttrString;
    NSMutableAttributedString *_redirectedSelectedAttrString;
    NSMutableAttributedString *_onlinePresenceString;
    //struct ? _flags;
    long _flags;
    NSDictionary *_searchRanks;
    NSDictionary *_messageIDsToSelectWhenOpened;
    int colorHighlightLeftEdge;
    int colorHighlightWidth;
    int locationColumnPosition;
    int locationColumnWidth;
    int senderColumnPosition;
    int senderColumnWidth;
    int receiverColumnPosition;
    int receiverColumnWidth;
    NSMutableDictionary *_columnWidthsBeforeSearch;
    int _windowWidthBeforeSearch;
}

- (void)awakeFromNib;
- (void)tableViewScrolled:fp12;
- (void)_setupColumnHeaderIcon:fp12 inColumnWithIdentifier:fp16 withOffset:(float)fp40 alignment:(int)fp24;
- (void)_setupTableColumnWidths;
- (void)_configureColumnForEndTruncation:fp12;
- (void)_configureColumnForDateCell:fp12;
- (void)_setupColumnsForTableView;
- (void)dealloc;
- (void)windowWillClose:fp12;
- (void)windowDidActivateOrDeactivate:fp12;
- delegate;
- (void)setMessageMall:fp12;
- (void)readDefaultsFromDictionary:fp12;
- (void)writeDefaultsToDictionary:fp12;
- (void)writeStoreSpecificDefaultsToMailboxUid:fp12;
- (void)readStoreSpecificDefaultsFromMailboxUid:fp12;
- (void)readStoreSpecificDefaultsFromDictionary:fp12;
- (char)canSelectNextMessage;
- (char)canSelectPreviousMessage;
- (void)rulePreferencesChanged:fp12;
- (void)viewerPreferencesChanged:fp12;
- (void)presencePreferenceChanged:fp12;
- (void)presenceChanged:fp12;
- (void)presenceIconChanged:fp12;
- (void)_updateTableHeaderToMatchCurrentSort;
- (void)tableView:fp12 didClickTableColumn:fp16;
- _columnWithIdentifierTag:(int)fp12;
- (char)isColumnVisible:(int)fp12;
- (void)setColumn:(int)fp12 toVisible:(char)fp16;
- (void)_setColumn:(int)fp12 toVisible:(char)fp16;
- (char)displaysMessageSizes;
- (void)setDisplaysReceiver:(char)fp12;
- (void)setFont:fp12;
- font;
- (void)selectNextMessageMovingDownward;
- (void)selectNextMessageMovingUpward;
- (void)selectNextMessage;
- (void)selectPreviousMessage;
- (void)showSelectionAndCenter:(char)fp12;
- (char)isSelectionVisible;
- (void)scrollRowToVisible:(int)fp12 position:(int)fp16;
- (void)_redisplayTableView;
- (char)_isMessageInSelectedThread:fp12;
- (void)_updateSubjectSet;
- (void)_reallyUpdateTextViewerToSelection;
- (void)updateTextViewerToSelection;
- (void)selectionIsChanging:fp12;
- (void)selectionChanged:fp12;
- (void)receiveMidnightNotification;
- (void)refreshAtMidnight;
- _createAttributedStringWithImageNamed:fp12 font:fp16 alignment:(int)fp20;
- (int)numberOfRowsInTableView:fp12;
- _colorForMessage:fp12 inRow:(int)fp16;
- tableView:fp12 objectValueForTableColumn:fp16 row:(int)fp20;
- (void)_updateColorHighlightEdges;
- (void)tableView:fp12 didDragTableColumn:fp16;
- (void)tableViewColumnDidMove:fp12;
- (void)tableViewColumnDidResize:fp12;
- (void)tableView:fp12 willDisplayCell:fp16 forTableColumn:fp20 row:(int)fp24;
- tableViewHighlightColor:fp12;
- (int)tableView:fp12 highlightStyleForRow:(int)fp16 inRect:(struct _NSRect *)fp20 color:(id *)fp24;
- (void)doubleClickedMessage:fp12;
- (void)deleteKeyPressed;
- selection;
- (void)_selectMessages:fp12 scrollIfNeeded:(char)fp16;
- (void)selectMessages:fp12;
- currentDisplayedMessage;
- messageStore;
- tableView;
- (char)tableView:fp12 doCommandBySelector:(SEL)fp16;
- _undoActionNameForMessages:fp12;
- (void)_undeleteMessages:fp12;
- (void)_redeleteMessages:fp12 messagesToSelect:fp16;
- (void)deleteMessages:fp12 allowMoveToTrash:(char)fp16;
- (void)undeleteMessages;
- (char)shouldDeleteGivenCurrentState;
- (char)_synchronouslyTransferMessages:fp12 fromStore:fp16 toMailbox:fp20 deleteOriginals:(char)fp24 retainingStores:fp28;
- (char)_transferMessages:fp12 toMailbox:fp16 deleteOriginals:(char)fp20;
- (char)_transferMessages:fp12 toSpecialMailbox:fp16 deleteOriginals:(char)fp20;
- (unsigned int)tableView:fp12 draggingSourceOperationMaskForLocal:(char)fp16;
- (char)transferSelectionToMailbox:fp12 deleteOriginals:(char)fp16;
- (void)tableViewDragWillEnd:fp12 operation:(unsigned int)fp16;
- (char)tableView:fp12 writeRows:fp16 toPasteboard:fp20;
- tableView:fp12 dragImageForRows:fp16 event:fp20 dragImageOffset:(struct _NSPoint *)fp24;
- messageMall;
@end

@interface TableViewManager(FilteredList_Management)
- (void)mallStructureWillChange:fp12;
- (void)setMailboxUids:fp12;
- (void)mallMailboxesChanged:fp12;
- (char)allMailboxesAreOutgoing;
- (void)storeStructureChanged:fp12;
- (void)messagesAdded:fp12;
- (void)messagesCompacted:fp12;
- (void)messageFlagsChanged:fp12;
- (void)_scrollToHappyPlace;
- (void)_recordVisibleState:(struct visible_state *)fp12 deletedAreGoingAway:(char)fp16 considerPinToEnd:(char)fp20;
- (char)_restoreVisibleState:(struct visible_state *)fp12 forceSelToVisible:(char)fp16;
- (void)dumpVisibleState;
- _messageToSelectIfEntireSelectionRemoved;
- (int)indexOfTopmostVisibleMessageMustBeSelected:(char)fp12 okayIfDeleted:(char)fp16;
- (int)indexOfLastReadMessage;
- (int)indexOfFirstNonDeletedNonSelectedMessage:(int)fp12 goUp:(char)fp16;
- persistentVisibleStateCreateIfNecessary:(char)fp12;
- (void)saveScrollAndSelection;
- (char)restoreScrollAndSelection;
- selectedMessageIDs;
- (void)setSelectedMessageIDs:fp12;
- (void)addRanks;
- (void)removeRanks;
- searchRanks;
- (void)clearSearch;
- (char)isShowingSearchResults;
- (void)searchForString:fp12 in:(int)fp16 withOptions:(int)fp20;
- (char)isFocused;
- (void)setFocusedMessages:fp12;
- (char)includeDeleted;
- (void)setIncludeDeleted:(char)fp12;
- (char)isSortedByRank;
- (char)isSortedByDateReceived;
- (char)isSortedChronologically;
- (int)sortColumn;
- (void)setSortColumn:(int)fp12;
- (char)isSortedAscending;
- (void)setIsSortedAscending:(char)fp12;
- (int)state;
- filteredMessages;
- (void)restoreLocationColumnIfAppropriate;
@end