/*____________________________________________________________________________
		Copyright (C) 2002 PGP Corporation
		All rights reserved.

		$Id: pgpDiskEngine.h,v 1.36 2002/11/13 21:01:58 dallen Exp $
____________________________________________________________________________*/

#ifndef Included_PGPdiskEngine_h	/* [ */
#define Included_PGPdiskEngine_h

#include "pflPrefs.h"
#include "pgpPubTypes.h"
#include "pgpDiskBase.h"

#include "pgpDiskEngineMessages.h"

/* Constants */
#define	kPGPdiskAPIVersion	((PGPUInt32) 0x01000000)

#define PGPdiskMajorVersion(v)	((((PGPUInt32)(v)) & 0xFF000000) >> 24)
#define PGPdiskMinorVersion(v)	((((PGPUInt32)(v)) & 0x00FFF000) >> 16)
#define PGPdiskRevVersion(v)	((((PGPUInt32)(v)) & 0x00000FF0) >> 4)

PGP_BEGIN_C_DECLARATIONS

/* Types */

typedef struct PGPdiskContextPriv *		PGPdiskContextRef;
typedef struct PGPdiskDiskPriv *		PGPdiskDiskRef;
typedef struct PGPdiskUserPriv *		PGPdiskUserRef;
typedef struct PGPdiskUserIterPriv *	PGPdiskUserIterRef;
typedef struct PGPdiskUserSetPriv *		PGPdiskUserSetRef;

#define	kInvalidPGPdiskContextRef		((PGPdiskContextRef) NULL)
#define	kInvalidPGPdiskDiskRef			((PGPdiskDiskRef) NULL)
#define	kInvalidPGPdiskUserRef			((PGPdiskUserRef) NULL)
#define	kInvalidPGPdiskUserSetRef		((PGPdiskUserSetRef) NULL)
#define	kInvalidPGPdiskUserIterRef		((PGPdiskUserIterRef) NULL)

#define PGPdiskContextRefIsValid(ref)	((ref) != kInvalidPGPdiskContextRef)
#define PGPdiskDiskRefIsValid(ref)		((ref) != kInvalidPGPdiskDiskRef)
#define PGPdiskUserRefIsValid(ref)		((ref) != kInvalidPGPdiskUserRef)
#define PGPdiskUserSetRefIsValid(ref)	((ref) != kInvalidPGPdiskUserSetRef)
#define PGPdiskUserIterRefIsValid(ref)	((ref) != kInvalidPGPdiskUserIterRef)

#define PGPdiskValidateContextRef(ref)	PGPValidateParam(PGPdiskContextRefIsValid(ref))
#define PGPdiskValidateDiskRef(ref)		PGPValidateParam(PGPdiskDiskRefIsValid(ref))
#define PGPdiskValidateUserRef(ref)		PGPValidateParam(PGPdiskUserRefIsValid(ref))
#define PGPdiskValidateUserSetRef(ref)	PGPValidateParam(PGPdiskUserSetRefIsValid(ref))
#define PGPdiskValidateUserIterRef(ref)	PGPValidateParam(PGPdiskUserIterRefIsValid(ref))


/* Enums */

/*
 * These numbers should not conflict with any of the standard
 * PGPError values.  These error codes are meant to supplement
 * the normal PGPError codes with PGPdisk-specific errors.
 * We might eventually want to move these into pgpErrors.h in
 * the SDK.
 */
typedef enum
{
	kPGPdiskError_First						= -10000,
	
	kPGPdiskError_NotInstalledCorrectly		= -9999,
	kPGPdiskError_DiskAlreadyMounted		= -9998,
	kPGPdiskError_DiskNotMounted			= -9997,
	kPGPdiskError_CantUnmountDisk			= -9996,
	kPGPdiskError_DiskNeedsReEncryption		= -9995,
	kPGPdiskError_OS9DiskNeedsConversion	= -9994,
	kPGPdiskError_DiskIsReadOnly			= -9993,
	kPGPdiskError_CantFormatDisk			= -9992,
	
	kPGPdiskError_IsAdminUser				= -9989,
	kPGPdiskError_IsLocked					= -9988,
	kPGPdiskError_IsDisabled				= -9987,
	
	kPGPdiskError_Last
} PGPdiskError;
#define IsPGPdiskError(err)		((err) > kPGPdiskError_First && (err) < kPGPdiskError_Last)

typedef enum
{
	kPGPdiskDiskFormat_V6			= 0,			// Version 6 disk (PGP 7.0, 7.1, 8.0 Win32; PGP 8.0 Mac OS X)
	kPGPdiskDiskFormat_OS9			= 1,			// (PGP any version Mac OS 9)

	kPGPdiskDiskFormat_Invalid		= 0xFFFFFFFF
} PGPdiskDiskFormat;

typedef enum
{
	kPGPdiskUserProp_HasPublicKey		= 100,		// boolean
	kPGPdiskUserProp_HasSymmetricKey	= 200,		// boolean
	kPGPdiskUserProp_IsReadOnly			= 300,		// boolean, settable
	kPGPdiskUserProp_IsLocked			= 400,		// boolean, settable
	kPGPdiskUserProp_IsAdminUser		= 500,		// boolean
	kPGPdiskUserProp_IsDisabled			= 600,		// boolean, settable
	kPGPdiskUserProp_CanSetReadOnly		= 700,		// boolean
	kPGPdiskUserProp_CanSetLocked		= 800,		// boolean
	kPGPdiskUserProp_CanSetUserName		= 900,		// boolean

	kPGPdiskUserProp_UserName			= 1000		// string, settable

} PGPdiskUserPropName;

typedef enum
{
	kPGPdiskDiskProp_IsMounted					= 100,		// boolean
	kPGPdiskDiskProp_IsReadOnly					= 200, 		// boolean
	kPGPdiskDiskProp_IsBeingReEncrypted			= 300, 		// boolean
	kPGPdiskDiskProp_HasTimedOut				= 400,		// boolean
	kPGPdiskDiskProp_UsesCustomTimeout			= 500,		// boolean, settable
	kPGPdiskDiskProp_UsesCustomEjectOnLogout	= 600,		// boolean, settable	
	kPGPdiskDiskProp_EjectOnLogout				= 700,		// boolean, settable

	kPGPdiskDiskProp_DiskFormat					= 800,		// number
	kPGPdiskDiskProp_Algorithm					= 900, 		// number
	kPGPdiskDiskProp_CustomTimeout				= 1000, 		// number, settable	(timeout, in minutes)
	kPGPdiskDiskProp_BlocksDiskLow				= 1100,		// number	(low 32 bits of number of data blocks)
	kPGPdiskDiskProp_BlocksDiskHigh				= 1200,		// number	(high 32 bits of number of data blocks)

	kPGPdiskDiskProp_Path						= 1300,		// string	(filename of PGPdisk file)
	kPGPdiskDiskProp_Root						= 1400, 	// string	(mount point)
	kPGPdiskDiskProp_DefaultRoot				= 1500, 	// string, settable	(default mount point)
	kPGPdiskDiskProp_DeviceName					= 1600		// string	(device, e.g. "/dev/disk1")

} PGPdiskDiskPropName;

typedef enum
{
	kPGPdiskFilesystemType_FAT12			= 0,
	kPGPdiskFilesystemType_FAT16			= 1,
	kPGPdiskFilesystemType_FAT12Or16		= 2,
	kPGPdiskFilesystemType_FAT32			= 3,
	kPGPdiskFilesystemType_NTFS				= 4,

	kPGPdiskFilesystemType_HFS				= 5,
	kPGPdiskFilesystemType_HFSPlus			= 6,
	kPGPdiskFilesystemType_HFSPlusJournaled	= 7,
	kPGPdiskFilesystemType_UFS				= 8,
	
	kPGPdiskFilesystemType_Invalid		= 0xFFFFFFFF
} PGPdiskFilesystemType;

typedef PGPError (*PGPdiskReEncryptDiskStatusFunc)(
	PGPUInt32 percent, 
	void *userVal);

typedef PGPError (*PGPdiskConvertDiskStatusFunc)(
	PGPUInt32 percent, 
	void *userVal);

typedef PGPError (*PGPdiskNewDiskStatusFunc)(
	PGPBoolean areFormatting, 
	PGPUInt32 percent,
	void *userVal);


/*____________________________________________________________
	DLL init/exit functions 
  ____________________________________________________________*/

PGPError 
PGPdiskEngineInitLibrary();

PGPError 
PGPdiskEngineCleanupLibrary();

/*____________________________________________________________
	PGPdiskContextRef functions
  ____________________________________________________________*/

PGPError 
PGPdiskNewContext(
	PGPContextRef context,
	PGPUInt32 diskAPIVersion,		/* should always be kPGPdiskAPIVersion */
   	PGPdiskContextRef *pContext );

PGPError 
PGPdiskFreeContext(
	PGPdiskContextRef context );
	
PGPError
PGPdiskLoadKeyrings(
	PGPdiskContextRef context,
	char *publicFilename,
	char *privateFilename );

PGPContextRef 
PGPdiskPeekContextPGPContext(
	PGPdiskContextRef context );

PGPError 
PGPdiskContextSetADK(
	PGPdiskContextRef context,
   	PGPKeyID *adkKeyID );
	
PGPError
PGPdiskContextSetGlobalDiskTimeout(
	PGPdiskContextRef context,
	PGPUInt32 timeout );	/* in minutes, 0 = no timeout */
	
PGPError
PGPdiskContextSetGlobalEjectOnLogout(
	PGPdiskContextRef context,
	PGPBoolean ejectOnLogout );
	
PGPError
PGPdiskContextSetPassphraseCaching(
	PGPdiskContextRef context,
	PGPBoolean cache,			/* TRUE if we should cache the passphrases */
	PGPUInt32 howlong,			/* How long to cache the passphrase for (in seconds) */
	PGPBoolean shareCache );	/* The "Share passphrase cache among modules" pref */
	
PGPError
PGPdiskContextSetUnmountOnSleep(
	PGPdiskContextRef context,
	PGPBoolean unmountOnSleep );

PGPError
PGPdiskContextSetForcedAutoDiskUnmount(
	PGPdiskContextRef context,
	PGPBoolean forced );
	
PGPError
PGPdiskContextSetPreventSleep(
	PGPdiskContextRef context,
	PGPBoolean preventSleep );

/*____________________________________________________________
	PGPdiskUserRef functions
  ____________________________________________________________*/
  
/*
 * A PGPdiskUserRef is associated with a PGPdiskUserSetRef.  You don't
 * need to worry about freeing each user because it will be freed when
 * you free the UserSet that the user belongs to.
 */

PGPError 
PGPdiskNewUserWithSymmetricKey(
	PGPdiskUserSetRef userSet, 
	const char *userName,
   	const char *passphrase,
   	PGPBoolean isReadOnly, 
	PGPdiskUserRef *pNewUser );

PGPError 
PGPdiskNewUserWithPublicKey(
	PGPdiskUserSetRef userSet, 
	PGPKeyID *pubKeyID,
   	PGPBoolean isReadOnly,
   	PGPBoolean isLocked, 
	PGPdiskUserRef *pNewUser );

PGPError 
PGPdiskDuplicateUser(
	PGPdiskUserRef user,
   	PGPdiskUserRef *pNewUser );

PGPError 
PGPdiskFreeUser(
	PGPdiskUserRef user );

PGPdiskContextRef 
PGPdiskPeekUserContext(
	PGPdiskUserRef user );
	
PGPError 
PGPdiskUserGetPGPKeyID(
	PGPdiskUserRef user,
   	PGPKeyID *pKeyID );

PGPError
PGPdiskIsUserPassphraseCorrect(
	PGPdiskUserRef user,
	const char *passphrase,
	PGPBoolean * isCorrect );

PGPError 
PGPdiskUserChangePassphrase(
	PGPdiskUserRef user, 
	const char *oldPassphrase,
	const char *newPassphrase );

PGPBoolean 
PGPdiskUserAreUsersEqual(
	PGPdiskUserRef user1,
   	PGPdiskUserRef user2);

PGPError 
PGPdiskGetUserBooleanProp(
	PGPdiskUserRef user,
   	PGPdiskUserPropName prop, 
	PGPBoolean *pBool );

PGPError 
PGPdiskGetUserNumericProp(
	PGPdiskUserRef user,
   	PGPdiskUserPropName prop, 
	PGPInt32 *pNumber );

PGPError 
PGPdiskGetUserDataProp(
	PGPdiskUserRef user,
   	PGPdiskUserPropName prop, 
	void *pData,
   	PGPSize availLength,
   	PGPSize *pUsedLength );

PGPError 
PGPdiskSetUserBooleanProp(
	PGPdiskUserRef user,
   	PGPdiskUserPropName prop, 
	PGPBoolean boolVal );

PGPError 
PGPdiskSetUserNumericProp(
	PGPdiskUserRef user,
   	PGPdiskUserPropName prop, 
	PGPInt32 number );

PGPError 
PGPdiskSetUserDataProp(
	PGPdiskUserRef user,
   	PGPdiskUserPropName prop, 
	const void *pData,
   	PGPSize length );

/*____________________________________________________________
	PGPdiskUserSetRef functions
  ____________________________________________________________*/

PGPError 
PGPdiskNewUserSet(
	PGPdiskContextRef context,
	PGPdiskUserSetRef *pUserSet );

PGPError 
PGPdiskFreeUserSet(
	PGPdiskUserSetRef userSet );

PGPUInt32 
PGPdiskUserSetCount(
	PGPdiskUserSetRef userSet );

PGPError 
PGPdiskUserSetRemoveUser(
	PGPdiskUserSetRef userSet, 
	PGPdiskUserRef userToRemove );

/*____________________________________________________________
	PGPdiskUserIterRef functions
  ____________________________________________________________*/

PGPError 
PGPdiskNewUserIter(
	PGPdiskUserSetRef userSet,
   	PGPdiskUserIterRef *pUserIter );

PGPError 
PGPdiskFreeUserIter(
	PGPdiskUserIterRef userIter );

PGPdiskUserSetRef 
PGPdiskPeekUserIterUserSet(
	PGPdiskUserIterRef userIter );

PGPInt32 
PGPdiskUserIterIndex(
	PGPdiskUserIterRef userIter );

PGPError 
PGPdiskUserIterMove(
	PGPdiskUserIterRef userIter,
   	PGPInt32 relOffset, 
	PGPBoolean fromStart,
   	PGPdiskUserRef *pUser );

PGPInt32 
PGPdiskUserIterSeek(
	PGPdiskUserIterRef userIter,
   	PGPdiskUserRef user );

PGPError 
PGPdiskUserIterNext(
	PGPdiskUserIterRef userIter,
   	PGPdiskUserRef *pUser );

PGPError 
PGPdiskUserIterPrev(
	PGPdiskUserIterRef userIter,
   	PGPdiskUserRef *pUser );

PGPError 
PGPdiskUserIterRewind(
	PGPdiskUserIterRef userIter );

/*____________________________________________________________
	PGPdiskDiskRef functions
  ____________________________________________________________*/

PGPError 
PGPdiskOpenDisk(
	PGPdiskContextRef context,
   	const char *path, 
	PGPBoolean readOnly,
   	PGPdiskDiskRef *pDisk );

PGPError
PGPdiskCreateDisk(
	PGPdiskContextRef	context,
	const char *		path,
	PGPUInt64 			sizeBytes,
	const char *		defaultMountPoint,
	PGPdiskEncryptionAlgorithm	alg,
	PGPBoolean 			bPubKeyUser,		// True if user is a public key user, false if passphrase user
	PGPKeyID *	 		keyid,
	const char *		username,
	const char *		passphrase,
	PGPdiskNewDiskStatusFunc	callback,
	void *				userValue,
	PGPdiskDiskRef *	pDisk );

PGPError 
PGPdiskCloseDisk(
	PGPdiskDiskRef disk );
	
PGPError
PGPdiskFormatDisk(
	PGPdiskDiskRef disk,
	const char * adminPassphrase,
	const char * volumeName,
	PGPdiskFilesystemType type );
	
PGPdiskContextRef 
PGPdiskPeekDiskContext(
	PGPdiskDiskRef disk );
	
PGPError
PGPdiskCanPassphraseMountDisk(
	PGPdiskDiskRef disk,
	const char *passphrase,
	PGPBoolean *canMount );

PGPError 
PGPdiskMountDiskAsUser(
	PGPdiskDiskRef disk,
   	const char *root,
	PGPdiskUserRef user,
	const char *passphrase,
   	PGPBoolean readOnly );

PGPError 
PGPdiskMountDisk(
	PGPdiskDiskRef disk,
   	const char *root,
	const char *passphrase,
   	PGPBoolean readOnly );

PGPError 
PGPdiskUnmountDisk(
	PGPdiskDiskRef disk, 
	PGPBoolean isForced );
	
PGPError 
PGPdiskDiskExportUserSet(
	PGPdiskDiskRef disk,
   	PGPdiskUserSetRef userSet );

PGPError 
PGPdiskDiskImportUserSet(
	PGPdiskDiskRef disk,
	const char *adminPassphrase,	/* Passphrase of current admin user */
   	PGPdiskUserSetRef userSet, 
	PGPdiskUserRef newAdminUser );

PGPError 
PGPdiskReEncryptDisk(
	PGPdiskDiskRef disk,
   	const char *adminPassphrase, 
	PGPdiskEncryptionAlgorithm newAlgorithm, 
	PGPdiskReEncryptDiskStatusFunc statusFunc,
   	void *userValue );
	
PGPError
PGPdiskConvertDisk(
	PGPdiskContextRef context,
	PGPdiskDiskFormat format,
	const char * originalFilename,
	const char * adminPassphrase,
	const char * newFilename,
	PGPdiskConvertDiskStatusFunc statusFunc,
	void *userValue );

PGPError 
PGPdiskGetDiskBooleanProp(
	PGPdiskDiskRef disk,
   	PGPdiskDiskPropName prop, 
	PGPBoolean *pBool );

PGPError 
PGPdiskGetDiskNumericProp(
	PGPdiskDiskRef disk,
   	PGPdiskDiskPropName prop, 
	PGPInt32 *pNumber );

PGPError 
PGPdiskGetDiskDataProp(
	PGPdiskDiskRef disk,
   	PGPdiskDiskPropName prop, 
	void *pData,
   	PGPSize availLength,
   	PGPSize *pUsedLength );
	
PGPError 
PGPdiskSetDiskBooleanProp(
	PGPdiskDiskRef disk,
   	PGPdiskDiskPropName prop, 
	PGPBoolean boolVal );

PGPError 
PGPdiskSetDiskNumericProp(
	PGPdiskDiskRef disk,
   	PGPdiskDiskPropName prop, 
	PGPInt32 number );

PGPError 
PGPdiskSetDiskDataProp(
	PGPdiskDiskRef disk,
   	PGPdiskDiskPropName prop, 
	const void *pData,
   	PGPSize length );

/*____________________________________________________________
	Miscellaneous functions
  ____________________________________________________________*/

PGPError
PGPdiskGetSockPath( char * sockPath );

PGPBoolean
PGPdiskIsEngineRunning( void );

PGPError
PGPdiskGetEnginePID( PGPdiskContextRef context,
	pid_t *pid );

PGPError
PGPdiskGetErrorString( PGPError theError,
	PGPSize bufferSize,
	char * theString );

PGP_END_C_DECLARATIONS

#endif	/* ] Included_PGPdiskEngine_h */
