/*____________________________________________________________________________
    Copyright (C) 2002 PGP Corporation
    All rights reserved.

    $Id: PGPDictionaryTraversal.m,v 1.7 2002/08/06 20:10:27 dallen Exp $
____________________________________________________________________________*/

#import <stdlib.h>
#import <CoreFoundation/CFString.h>
#import "PGPDictionaryTraversal.h"

@implementation NSDictionary (PGPDictionaryTraversal)

- (id)objectForKeyPath:(NSString*)keyPath
{
	NSAutoreleasePool*	pool = [[NSAutoreleasePool alloc] init];
    NSArray*		keyElements = 0;
    
    if (!keyPath)
        return nil;
    
    keyElements = [keyPath componentsSeparatedByString:kPGPDictionaryKeyPathSeparator];
    
    return [self objectForKeyPathArray:keyElements];
	[pool release];	
}

- (id)objectForKeyPathArray:(NSArray*)keyPathArray
{
	NSAutoreleasePool*	pool = [[NSAutoreleasePool alloc] init];
    NSDictionary*	dictionary;
    NSString*		element;
    id				object;
    BOOL			failure = NO;
    int				numElements;
    int				i;
    
    if (!keyPathArray)
        return nil;
        
    numElements = [keyPathArray count];
    
    dictionary = self;
    
    for (i = 0; (i < numElements) && !failure; ++i)
    {
        if (!(element = [keyPathArray objectAtIndex:i]))
        {
            NSLog(@"PGPDictionaryTraversal: keyPathArray has been modified!");
            failure = YES;
            continue;
        }
        
        if (!(object = [dictionary objectForKey:element]))
        {
            NSLog(@"PGPDictionaryTraversal: nil value for %s", [element cString]);
            failure = YES;
            continue;
        }
        
        dictionary = object;
    }
    if (failure)
        NSLog(@"PGPDictionaryTraversal: Failed to retrieve object at path");
    
    return object;
	[pool release];	
}

@end



@implementation NSMutableDictionary (PGPDictionaryTraversal)

- (void)setObject:(id)newObject forKeyPath:(NSString*)keyPath
{
	NSAutoreleasePool*	pool = [[NSAutoreleasePool alloc] init];
    NSArray*		keyElements = 0;
    
    if (newObject && keyPath)
    {
        keyElements = [keyPath componentsSeparatedByString:kPGPDictionaryKeyPathSeparator];
        
        [self setObject:newObject forKeyPathArray:keyElements];
    }
	[pool release];	
}

- (void)setObject:(id)newObject forKeyPathArray:(NSArray*)keyPathArray
{
	NSAutoreleasePool*	pool = [[NSAutoreleasePool alloc] init];
    NSMutableDictionary*	dictionary;
    NSMutableDictionary*	newDictionary;
    NSString*				element = 0;
    id						object;
    BOOL					failure = NO;    
    int						numElements;
    int						i;
    
    if (newObject && keyPathArray)
    {
        numElements = [keyPathArray count];
    
        dictionary = self;
    
        for (i = 0; (i < (numElements - 1)) && !failure; ++i)
        {
            if (!(element = [keyPathArray objectAtIndex:i]))
            {
                NSLog(@"PGPDictionaryTraversal: keyPathArray has been modified!");
                failure = YES;
                continue;
            }
        
            if (!(object = [dictionary objectForKey:element]))
            {
                NSLog(@"PGPDictionaryTraversal: Creating dictionary %s with capacity %d",
                    [element cString],
                    kPGPDictionaryDefaultCapacity);
                
				newDictionary = [[NSMutableDictionary alloc] initWithCapacity:kPGPDictionaryDefaultCapacity];
                
                if (!newDictionary)
                {
                    NSLog(@"PGPDictionaryTraversal: Failed to create new dictionary!");
                    failure = YES;
                    continue;
                }
                
                [dictionary setObject:newDictionary forKey:[NSString stringWithString:element]];
                
                object = newDictionary;
            }
            
            dictionary = object;
        }
        
        if (!failure && (element = [keyPathArray objectAtIndex:(numElements - 1)]))
        {
            NSLog(@"PGPDictionaryTraversal: Setting object for key %s", [element cString]);
            [dictionary setObject:newObject forKey:[NSString stringWithString:element]];
        }
            
        if (failure)
            NSLog(@"PGPDictionaryTraversal: Failed to set object for keypath");;
    }
	[pool release];	
}

- (void)releaseChildDictionaries
{
	NSAutoreleasePool*	pool = [[NSAutoreleasePool alloc] init];
	NSEnumerator*		keyEnumerator;
	id					key;
	id					object;
	//BOOL				deleteObject;
	//BOOL				isValidEntry;
	
	keyEnumerator = [self keyEnumerator];
	
	while (key = [keyEnumerator nextObject])
	{
		object = [self objectForKey:key];
		
		if ([object isKindOfClass:[NSDictionary class]])
		{
			[object releaseChildDictionaries];
			[object release];
		}
	}
	
	[pool release];
}

@end