/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.

	$Id: PGPOpenPrefs.m,v 1.15 2002/10/19 06:55:47 jeffc Exp $
____________________________________________________________________________*/

#import "PGPClientPrefs.h"
#import "PGPNetPrefs.h"
#import "PGPOpenPrefs.h"
#import "pgpClientLibCocoa.h"
#import <Cocoa/Cocoa.h>

NSString *		kPGPPrefsDomain = @"com.pgp.PGP";
const char *	kPGPclPrefsClientKey = "client";
const char * 	kPGPclPrefsNetKey = "net";
NSString *		kPrefsUpdated = @"com.pgp.PGP:kPrefsUpdated";

static PGPPrefRef		sNetPrefs = kInvalidPGPPrefRef;


@interface CPGPPrefRef : NSObject
{
@public
	PGPPrefRef				mPrefRef;
	const char *			mKey;
	PGPclPrefNotifierProc	mProc;
	PGPBoolean				mUseNotification;
}

- (id)initWithPrefRef:(PGPPrefRef)inPrefRef key:(const char *)inKey notifier:(PGPclPrefNotifierProc)inProc useNotification:(PGPBoolean)inUseNotification;
- (void)dealloc;

- (void)prefsUpdated:(NSNotification *)inNotification;

@end



@implementation CPGPPrefRef

- (id)initWithPrefRef:(PGPPrefRef)inPrefRef key:(const char *)inKey notifier:(PGPclPrefNotifierProc)inProc useNotification:(PGPBoolean)inUseNotification;
{
	if (self = [super init])
	{
		mPrefRef = inPrefRef;
		mKey = inKey;
		mProc = inProc;
		mUseNotification = inUseNotification;
		
		if (mUseNotification)
		{
			[[NSDistributedNotificationCenter defaultCenter] addObserver:self
				selector:@selector(prefsUpdated:) name:kPrefsUpdated
				object:[NSString stringWithCString:inKey]
				suspensionBehavior:NSNotificationSuspensionBehaviorDeliverImmediately];
		}
	}
	
	return self;
}



- (void)dealloc
{
	if (mUseNotification)
	{
		[[NSDistributedNotificationCenter defaultCenter] removeObserver:self name:kPrefsUpdated
			object:[NSString stringWithCString:mKey]];
	}
	
	[super dealloc];
}


- (void)prefsUpdated:(NSNotification *)	inNotification
{
	PGPMemoryMgrRef	memoryMgr;
	PGPError		pgpErr = PGPPeekPrefMemoryMgr(mPrefRef, &memoryMgr);
	PGPPrefRef		prefRef;
	
	if (IsntPGPError(pgpErr)) {
		[[NSUserDefaults standardUserDefaults] synchronize];
		pgpErr = PGPclOpenPrefs(memoryMgr, mKey, NULL, &prefRef, mUseNotification);
		if (IsntPGPError(pgpErr)) {
			PGPAddPrefs(prefRef, mPrefRef);
			PGPFreePrefs(prefRef);
			if (mProc) {
				mProc(kPGPclPrefNotification_PrefsUpdated, mPrefRef);
			}
		}
	}
}
@end

	void
sReleaseCPGPPrefRef(
	PGPUserValue	inObject)
{
	NSAutoreleasePool *			pool = [[NSAutoreleasePool alloc] init];

	[((CPGPPrefRef *) inObject) release];
	[pool release];
}

	PGPError
PGPclOpenPrefs(
	PGPMemoryMgrRef			inMemoryMgr,
	const char *			inKey,
	PGPclPrefNotifierProc	inProc,
	PGPPrefRef *			outPrefRef,
	PGPBoolean 				inUseNotification)
{
	PGPError					result = kPGPError_NoErr;
	NSAutoreleasePool *			pool = [[NSAutoreleasePool alloc] init];
	NSUserDefaults *			defaults = [NSUserDefaults standardUserDefaults];
	NSDictionary *				dict = [defaults persistentDomainForName:kPGPPrefsDomain];
	NSData *					data = NULL;
	const PGPPrefDefinition *	prefs = NULL;
	PGPUInt32					numPrefs = 0;
	
	if (inKey == kPGPclPrefsClientKey) {
		prefs = clientDefaults;
		numPrefs = clientDefaultsSize;
	} else if (inKey == kPGPclPrefsNetKey) {
		prefs = netDefaults;
		numPrefs = netDefaultsSize;
	} else {
		result = kPGPError_BadParams;
	}
	if (IsntPGPError(result)) {
		if (dict) {
			data = [dict objectForKey:[NSString stringWithCString:inKey]];
		}
		if (data) {
			result = PGPImportBufferToMemoryPrefs(inMemoryMgr, [data bytes], [data length], prefs,
						numPrefs, outPrefRef);
		} else {
			result = PGPNewMemoryPrefs(inMemoryMgr, prefs, numPrefs, outPrefRef);
		}
		if (IsntPGPError(result)) {
			PGPSetPrefUserValue(*outPrefRef, [[CPGPPrefRef alloc] initWithPrefRef:*outPrefRef
				key:inKey notifier:inProc useNotification:inUseNotification], sReleaseCPGPPrefRef);
		}
	}
	[pool release];

	return result;
}

	PGPError
PGPclWritePrefs(
	PGPPrefRef	inPrefRef)
{
	PGPError				result = kPGPError_NoErr;
	NSAutoreleasePool *		pool = [[NSAutoreleasePool alloc] init];
	CPGPPrefRef *			prefRef = (CPGPPrefRef *)PGPGetPrefUserValue(inPrefRef);
	NSUserDefaults *		defaults = [NSUserDefaults standardUserDefaults];
	NSMutableDictionary *	dict = [[[defaults persistentDomainForName:kPGPPrefsDomain] mutableCopy] autorelease];
	NSString *				keyString = [NSString stringWithCString:prefRef->mKey];
	PGPSize					size;
	void *					buffer;
	
	if (! dict) {
		dict = [NSMutableDictionary dictionary];
	}
	result = PGPExportPrefsToBuffer(inPrefRef, &size, &buffer);
	if (IsntPGPError(result)) {
		[dict setObject:[NSData dataWithBytes:buffer length:size] forKey:keyString];
		[defaults removePersistentDomainForName:kPGPPrefsDomain];
		[defaults setPersistentDomain:dict forName:kPGPPrefsDomain];
		[defaults synchronize];
		[[NSDistributedNotificationCenter defaultCenter] postNotificationName:kPrefsUpdated
			object:keyString];
		PGPFreeData(buffer);
	}
	[pool release];
	
	return result;
}

	PGPError
PGPclPeekNetPrefs(
	PGPnetContextRef		netContext,
	PGPPrefRef *			outPrefRef)
{
	PGPError				result = kPGPError_NoErr;
	
	if( PGPPrefRefIsValid( sNetPrefs ) )
		*outPrefRef =  sNetPrefs;
	else
	{
		result = PGPnetGetPrefs(netContext, &sNetPrefs);
		if (IsntPGPError(result))
			*outPrefRef = sNetPrefs;
	}
	
	return result;
}

	PGPError
PGPclFlushNetPrefs(
	PGPnetContextRef	netContext,
	PGPPrefRef			inPrefRef)
{
	return PGPnetSetPrefs(netContext, inPrefRef);
}

	void
PGPclSetNetPrefs(
	PGPPrefRef	inPrefRef)
{
	PGPclFreeNetPrefs();
	sNetPrefs = inPrefRef;
}

	void
PGPclFreeNetPrefs()
{
	if( PGPPrefRefIsValid( sNetPrefs ) )
		PGPFreePrefs( sNetPrefs );
	sNetPrefs = kInvalidPGPPrefRef;
}

