/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.

	$Id: SignatureStatusMessage.m,v 1.15 2002/08/06 20:10:27 dallen Exp $
____________________________________________________________________________*/

#import <PGP/PGP.h>
#import <PGPclient/pgpClientPrefs.h>
#import <PGPclient/CPGPPrefObj.h>
#import <PGPclient/pgpClientLibCocoa.h>
#import <PGPclient/CRemoteAuth.h>

#import "SignatureStatusMessage.h"
#import "PGPSharedEncryptDecrypt.h"

#define kSigDateTimeFormatStr					LSCL(@"%m/%d/%Y %I:%M:%S %p", @"Date/Time Format")

#define kSigLabelStatusStr						LSCL(@"*** Status:   ", @"")
#define kSigLabelSignerStr 						LSCL(@"*** Signer:   ", @"")
#define kSigLabelSignedStr 						LSCL(@"*** Signed:   ", @"")
#define kSigLabelVerifiedStr 					LSCL(@"*** Verified: ", @"")

#define kSigStatusAlgNotSupportedStr			LSCL(@"Signing Algorithm Not Supported.", @"")

#define kSigStatusNotVerifiedStr				LSCL(@"Not Verified (Signing Key is Missing)", @"")
#define	kSigStatusUnknownStr					LSCL(@"Unknown Signature", @"")
#define kSigStatusGoodStr						LSCL(@"Good Signature", @"")
	
#define kSigStatusGoodButNoValidityStr			LSCL(@" from Invalid Key\n", @"")
#define kSigStatusGoodButMarginalValidityStr	LSCL(@" from Partialy Valid Key\n", @"")
#define kSigStatusGoodButUnknownValidityStr		LSCL(@" from unknown validity\n", @"")
#define kSigStatusGoodButExpiredStr				LSCL(@" from Expired Key\n", @"")
#define kSigStatusGoodButDisabledStr			LSCL(@" from Disabled Key\n", @"")
#define kSigStatusGoodButRevokedStr				LSCL(@" from Revoked Key\n", @"")
#define kSigStatusGoodButProblemStr				LSCL(@"but key has an unknown problem\n", @"")
#define kSigStatusKeyCorruptStr					LSCL(@" from Damaged Key\n", @"")				
#define kSigStatusKeyUnknownStr					LSCL(@"Unknown Key", @"")

#define kSigStatusBadStr 						LSCL(@"Bad Signature", @"")

#define kSigStatusAlertVerifyStr				LSCL(@"*** Alert:    Verify signer's key before trusting.\n", @"")
#define kSigStatusAlertNotVerifyStr				LSCL(@"*** Alert:    Signature did not verify. Message has been altered.\n", @"")
 
	PGPError
GetSignatureStatusMessage(
    PGPContextRef				inContext,
    PGPPrefRef					inPrefRef,
	SignatureStatusInfo const 	*inInfo,
	NSString 				  	**ioMsg )
{
    PGPBoolean						isAxiomatic = FALSE;
    PGPBoolean						keyCanSign = TRUE;
    PGPBoolean						invalidAlert = FALSE;
    PGPBoolean						sigAlert = FALSE;
    PGPUInt32						validity = 0;
    time_t							unixTime;
	PGPError						theError = kPGPError_NoErr;
	PGPEventSignatureData const 	*sigData = &inInfo->sigData;
    NSMutableString					*theString = nil;
             
    theString = [NSMutableString string];
    
    [theString appendString:kSigLabelStatusStr];
    if( sigData->signingKey != 0 )
	    PGPGetKeyDBObjBooleanProperty(sigData->signingKey, kPGPKeyProperty_IsSigningKey, &keyCanSign);

	if( !keyCanSign )
        [theString appendString:kSigStatusAlgNotSupportedStr];
    else
    {
        NSString 		*tempString = nil;
        
       	if( sigData->verified )
			[theString appendString:kSigStatusGoodStr];
		else if( sigData->signingKey == 0 )
			[theString appendString:kSigStatusUnknownStr];
		else
		{
			sigAlert=TRUE;
			[theString appendString:kSigStatusBadStr];
		}
        
        tempString = @"\n";
        
        if( sigData->signingKey != 0 )
		{
			PGPGetKeyDBObjNumericProperty(sigData->signingKey, kPGPKeyProperty_Validity, &validity);

			PGPGetKeyDBObjBooleanProperty(sigData->signingKey, kPGPKeyProperty_IsAxiomatic, &isAxiomatic);

			if( sigData->keyRevoked )
                tempString = kSigStatusGoodButRevokedStr;
			else if( sigData->keyExpired )
                tempString = kSigStatusGoodButExpiredStr;
			else if( sigData->keyDisabled )
				tempString = kSigStatusGoodButDisabledStr;
            else if( !isAxiomatic )
			{
                PGPBoolean			marginal = FALSE;
                
                theError = PGPGetPrefBoolean(inPrefRef, kPGPPrefMarginalIsInvalid, &marginal);
                
				if( (validity == kPGPValidity_Invalid) ||
					(validity == kPGPValidity_Unknown) ||
                    ((validity == kPGPValidity_Marginal) && marginal) )
				{
					tempString = kSigStatusGoodButNoValidityStr;
					invalidAlert=TRUE;
				}
 			}
 		}
        [theString appendString:tempString];
    }
    
    if( sigAlert )
        [theString appendString:kSigStatusAlertNotVerifyStr];
        
    if( invalidAlert )
        [theString appendString:kSigStatusAlertVerifyStr];
        
    [theString appendString:kSigLabelSignerStr];
    
     // Get name and raw Validity number
	if( sigData->signingKey == 0 )
        [theString appendString:kSigStatusKeyUnknownStr];
	else
    {
        PGPInt32	keyType = 0;
        NSString	*theName = nil;

        PGPGetKeyDBObjNumericProperty( sigData->signingKey, kPGPKeyDBObjProperty_ObjectType, &keyType );

        if( kPGPKeyDBObjType_SubKey == keyType )
        {
            // if signing key is a subkey (as allowed by RFC2440
            // and emitted by GPG) then get the name from the
            // parent master key
            PGPKeyDBObjRef  theKey = PGPPeekKeyDBObjKey( sigData->signingKey );
            theName = PGPclGetPrimaryUserIDName( theKey );
        }
        else
            theName = PGPclGetPrimaryUserIDName( sigData->signingKey );

        [theString appendString:theName];
    }
    
    [theString appendString:@" ("];
    [theString appendString:[NSString stringWithCString:inInfo->keyIDString]];
    [theString appendString:@")\n"];
    
    [theString appendString:kSigLabelSignedStr];
    
    unixTime = PGPGetStdTimeFromPGPTime(sigData->creationTime);
    [theString appendString:[[NSCalendarDate dateWithTimeIntervalSince1970:unixTime]
                    descriptionWithCalendarFormat:kSigDateTimeFormatStr]];
    [theString appendString:@"\n"];
    
    [theString appendString:kSigLabelVerifiedStr];
    [theString appendString:[[NSCalendarDate date]
                    descriptionWithCalendarFormat:kSigDateTimeFormatStr]];
    [theString appendString:@"\n"];

    *ioMsg = [NSString stringWithString:theString];
    
    return theError;
}