/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.

	$Id: pgpReconstruct.c,v 1.37 2002/08/06 20:10:16 dallen Exp $
____________________________________________________________________________*/
#include <string.h>

#include "pflPrefTypes.h"
#include "pgpMem.h"
#include "pgpClientPrefs.h"
#include "pgpPFLPriv.h"
#include "pgpUtilities.h"
#include "pgpEndianConversion.h"
#include "pgpCBC.h"
#include "pgpLDAP.h"
#include "pgpReconstructPriv.h"
#include "pgpStrings.h"
#include "pgpKeys.h"

#define noPGPRECONSELFTEST
#define CKERR						if( IsPGPError( err ) ) goto done

/* This is are table of 6-bit value -> character */
	static char const armorTable[65] =
"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";

	PGPError
PGPNewReconstruct(
	PGPKeyDBObjRef				inTargetKey,
	PGPPrefRef					inClientPrefRef,
	char						*inAuthUser,
	char						*inAuthPass,
	PGPReconstructEventHandler	inHandler,
	PGPUserValue				inUserValue,
	PGPReconContextRef			*outRef )
{
	PGPError					err = kPGPError_NoErr;
	PGPReconContextPriv *		pContext;
	PGPContextRef				context;
	PGPMemoryMgrRef				memMgr;
	PGPBoolean					splitKey;

	*outRef = NULL;
	PGPValidatePtr( inTargetKey );
	(void)PGPGetKeyDBObjBooleanProperty( inTargetKey,
			kPGPKeyProperty_IsSecretShared, &splitKey );
	if( splitKey )
		return kPGPError_SecretKeyNotFound;
	
	context = PGPPeekKeyDBObjContext( inTargetKey );
	memMgr = PGPPeekContextMemoryMgr( context );
	
	pContext = (PGPReconContextPriv *) PGPNewSecureData( memMgr,
											sizeof(PGPReconContextPriv),
											kPGPMemoryMgrFlags_Clear );
	if( IsntNull( pContext ) )
	{
		pContext->context			= context;
		pContext->memMgr			= memMgr;
		pContext->targetKey			= inTargetKey;
		pContext->clientPrefs		= inClientPrefRef;
		pContext->handler			= inHandler;
		pContext->userValue			= inUserValue;
		if( IsntNull( inAuthUser ) )
			strcpy( pContext->authUser, inAuthUser );
		if( IsntNull( inAuthPass ) )
			strcpy( pContext->authPass, inAuthPass );
		pContext->hashReps			= 0;
		pContext->blobData			= NULL;
		pContext->blobDataSize		= 0;

		*outRef = ( PGPReconContextRef ) pContext;
	}
	else
		err = kPGPError_OutOfMemory;
	return err;
}

	PGPError
PGPFreeReconstruct(
	PGPReconContextRef			reconRef )
{
	PGPError					err = kPGPError_NoErr;
	PGPReconContextPriv *		con = (PGPReconContextPriv *)reconRef;
	
	PGPValidatePtr( reconRef );
	
	if( IsntNull( con->blobData ) )
		(void)PGPFreeData( con->blobData );
	err = PGPFreeData( reconRef );
	return err;
}

	static PGPError
sSendTLSEvent(
	PGPContextRef				context,
	PGPEventHandlerProcPtr		handler,
	PGPUserValue				userValue,
	PGPKeyServerState			state,
	PGPtlsSessionRef			tlsSession)
{
	PGPEvent					event;

	if( IsNull( handler ) )
		return kPGPError_NoErr;

	pgpClearMemory( &event, sizeof( PGPEvent ) );

	event.type = kPGPEvent_KeyServerTLSEvent;
	event.data.keyServerTLSData.state = state;
	event.data.keyServerTLSData.tlsSession = tlsSession;

	return handler( context, &event, userValue );
}

	static PGPError
sEstablishTLSSession(
	PGPContextRef		pgpContext,
	PGPldapContextRef	ldap,
	PGPtlsContextRef *	tlsContext,
	PGPtlsSessionRef *	tlsSession )
{
	PGPSocketRef		sock		= kInvalidPGPSocketRef;
	PGPError			err			= kPGPError_NoErr;

	err = PGPldapGetOption( ldap, kPGPldapOpt_Desc, &sock ); CKERR;

	err = PGPNewTLSContext( pgpContext, tlsContext ); CKERR;

	err = PGPNewTLSSession( *tlsContext, tlsSession ); CKERR;

	err = PGPSocketsEstablishTLSSession( sock, *tlsSession ); CKERR;

done:
	return err;
}

	static PGPError
sGetCertserverLocation(
	PGPldapContextRef			ldap,
	char *						dn )
{
	PGPError					err		= kPGPError_NoErr;
	char *						attr[]	= { kPGPBaseKeyspaceDN,
											kPGPCertserverVersion,
											NULL };
	char **						values	= NULL;
	PGPldapMessageRef			message	= kInvalidPGPldapMessageRef;

	err = PGPNewLDAPMessage( ldap, &message );							CKERR;

	/*
	 * First check to see if the server can accept recon data.  We check this
	 * by searching for a baseReconDN in the PGPServerInfo.  If this exists,
	 * the server can accept recon data.
	 */
	err = PGPldapSearchSync( ldap, 
			kPGPServerInfoCN,
			kPGPldapScope_Base,
			kPGPObjectclassAny,
			attr,
			FALSE,
			message );	CKERR;

	err = PGPldapGetValues( ldap, message, kPGPBaseReconDN, &values );
	if( err == kPGPError_LDAPNoSuchAttribute )
	{
		err = kPGPError_ServerOperationNotSupported;
		goto done;
	}

	if( IsntNull( values ) )
	{
		(void) PGPFreeLDAPValues( values );
		values = NULL;
	}

	/* Server supports it.  Now find where to put it */
	err = PGPldapGetValues( ldap, message, kPGPBaseKeyspaceDN, &values );	CKERR;

	pgpCopyMemory( values[0], dn, strlen( values[0] ) + 1 );


done:
	if( PGPldapMessageRefIsValid( message ) )
		(void) PGPFreeLDAPMessage( message );

	if( IsntNull( values ) )
		(void) PGPFreeLDAPValues( values );

	return err;
}

	static PGPError
sCountLDAPVariables(
	char *						inDN,
	PGPSize *					num )
{
	PGPError					err				= kPGPError_NoErr;
	char *						userid			= NULL;
	char *						useridVars[]	= kPGPUserIDVariables;
	PGPUInt32					whichVar		= 0;

	PGPValidatePtr( inDN );
	PGPValidatePtr( num );

	*num = 0;

	for( whichVar = 0; whichVar < sizeof( useridVars ) / sizeof( useridVars[0] ); whichVar++ )
	{
		for( userid = inDN; 
			userid <= inDN + strlen( inDN ) - strlen( useridVars[whichVar] );
			userid++ )
		{
			if( pgpCompareStringsIgnoreCaseN( userid, 
					useridVars[whichVar], 
					strlen( useridVars[whichVar] ) ) == 0 )
			{
				(*num)++;
				userid += strlen( useridVars[whichVar] );
			}
		}
	}

	return err;
}

	static PGPError
sReplaceLDAPVariables(
	char *						inDN,
	char *						username,
	char *						outDN )
{
	PGPError					err				= kPGPError_NoErr;
	PGPSize						len				= 0;
	PGPUInt32					i				= 0;
	PGPUInt32					j				= 0;
	PGPUInt32					k				= 0;
	char *						useridVars[]	= kPGPUserIDVariables;
	PGPUInt32					whichVar		= 0;
	PGPBoolean					found			= FALSE;

	PGPValidatePtr( inDN );
	PGPValidatePtr( outDN );

	if( IsNull( username ) )
		username = "";

	len = strlen( inDN );

	while( i < len )
	{
		for( whichVar = 0; whichVar < sizeof( useridVars ) / sizeof( useridVars[0] ); whichVar++ )
		{
			if( i <= strlen( inDN ) - strlen( useridVars[whichVar] ) )
			{
				if( pgpCompareStringsIgnoreCaseN( inDN + i, 
						useridVars[whichVar], 
						strlen( useridVars[whichVar] ) ) == 0 )
				{
					for( k = 0; k < strlen( username ); k++ )
						outDN[j++] = username[k];
					i += strlen( useridVars[whichVar] );
					found = TRUE;
					break;
				}
			}
		}

		if( !found )
			outDN[j++] = inDN[i++];
		found = FALSE;
	}
	outDN[j] = '\0';

	return err;
}


	static PGPError
sSendToServer(
	PGPldapContextRef			ldap,
	char *						dn,
	PGPReconContextPriv *		con,
	PGPByte *					keyBlock,
	PGPSize						keyBlockLen )
{
	PGPldapMod					reconData;
	PGPldapMod					objectclassData;
	PGPldapMod					ownerData;
	PGPldapMod					keyData;
	char *						ownerAttr[2]	= { 0 };
	PGPldapMod *				mod[4]			= { NULL, NULL, NULL, NULL };
	PGPberValue					berData			= { 0 };
	PGPberValue *				bvalue[2]		= { &berData, NULL };
	char *						attr[2]			= { kPGPReconElementObjectclass, NULL };
	char						certserverDN[256]	= { 0 };
	PGPUInt32					serverType		= kPGPKeyServerClass_PGP;
	char						virtDN[256];
	char *						exportedKey		= NULL;
	PGPSize						bufSize			= 0;
	char *						keyAttr[2]		= { NULL, NULL };
	PGPError					err				= kPGPError_NoErr;

#ifndef PGP_CMDLINE
	err = PGPGetPrefNumber( con->clientPrefs, kPGPPrefLDAPReconServerType,
				&serverType );	CKERR;
#else /* PGP_CMDLINE */
	serverType = (PGPKeyServerClass)con->serverType;
#endif /* PGP_CMDLINE */
	if( serverType == kPGPKeyServerClass_PGP )
	{
		/* Query cn=PGPServerInfo for certserver location */
		err = sGetCertserverLocation( ldap, certserverDN );				CKERR;
		strcat( dn, certserverDN );
	}

	/*
	 * If using standard LDAP server, we try to modify it first,
	 * then if that fails, we add it.  PGP certserver doesn't do
	 * modifies, so we just add it unconditionally.
	 */
	if( serverType == kPGPKeyServerClass_PGP )
	{
		/* PGP Keyserver */
		sprintf( virtDN, "%s=%s, %s", kPGPReconCertIDAttr, kPGPCertIDVirtual, certserverDN );

		err = PGPExport( con->context,
			PGPOExportKeyDBObj( con->context, con->targetKey ),
			PGPOAllocatedOutputBuffer( con->context,
								(void **) &exportedKey,
								MAX_PGPSize,
								&bufSize ),
			PGPOLastOption( con->context ) );						CKERR;

		keyAttr[0] = exportedKey;

		mod[0] = &keyData;
		mod[0]->op = kPGPldapModOp_Add;
		mod[0]->type = kPGPKeyAttr;
		mod[0]->value = keyAttr;

		mod[1] = &reconData;
		mod[1]->op = (PGPldapModOp)( kPGPldapModOp_Replace | kPGPldapModOpMask_UseBERValues );
		mod[1]->type = kPGPReconDataAttr;

		bvalue[0]->value = keyBlock;
		bvalue[0]->length = keyBlockLen;
		bvalue[1] = NULL;
		mod[1]->bvalue = bvalue;

		mod[2] = NULL;

		err = PGPldapAddSync( ldap, virtDN, mod );			CKERR;
	}
	else
	{
		/* Generic LDAP server */
		mod[0] = &reconData;
		mod[0]->op = (PGPldapModOp)( kPGPldapModOp_Replace | kPGPldapModOpMask_UseBERValues );
		mod[0]->type = kPGPReconDataAttr;

		bvalue[0]->value = keyBlock;
		bvalue[0]->length = keyBlockLen;
		bvalue[1] = NULL;
		mod[0]->bvalue = bvalue;

		mod[1] = &ownerData;
		mod[1]->op = kPGPldapModOp_Replace;
		mod[1]->type = kPGPOwnerAttr;
		
		ownerAttr[0] = dn;
		while( ( *ownerAttr[0] != ' ' ) && ( *ownerAttr[0] != '\0' ) )
			ownerAttr[0]++;
		while( ( *ownerAttr[0] == ' ' ) && ( *ownerAttr[0] != '\0' ) )
			ownerAttr[0]++;
		mod[1]->value = ownerAttr;

		mod[2] = NULL;

		err = PGPldapModifySync( ldap, dn, mod );

		if( err == kPGPError_LDAPNoSuchObject )
		{
			/* Object didn't exist, we need to add it */
			mod[0]->op = (PGPldapModOp)( kPGPldapModOp_Add | kPGPldapModOpMask_UseBERValues );
			/* mod[0] is already set */
			
			mod[1]->op = kPGPldapModOp_Add;
			/* mod[1] is already set */

			mod[2] = &objectclassData;
			mod[2]->op = kPGPldapModOp_Add;
			mod[2]->type = kPGPObjectclassAttr;
			mod[2]->value = attr;

			mod[3] = NULL;

			err = PGPldapAddSync( ldap, dn, mod );
		}
	}
	CKERR;

done:

	if( IsntNull( exportedKey ) )
	{
		(void) PGPFreeData( exportedKey );
		exportedKey = NULL;
	}

	return err;
}


	PGPError
PGPSetReconstructionEventHandler(
	PGPReconContextRef			reconRef,
	PGPReconstructEventHandler	inHandler,
	PGPUserValue				inUserValue )
{
	PGPError					err = kPGPError_NoErr;
	PGPReconContextPriv *		con = (PGPReconContextPriv *)reconRef;
	
	PGPValidatePtr( reconRef );
	
	con->handler = inHandler;
	con->userValue = inUserValue;
	
	return err;
}


	PGPError
PGPMakeReconstruction(
	PGPReconContextRef			reconRef,
	PGPReconPrompts 			inPromptInfo,
	PGPReconPasses				inPassInfo,
	char						*inPassphrase )
{
	PGPError					err = kPGPError_NoErr;
	PGPReconContextPriv *		con = (PGPReconContextPriv *)reconRef;
	PGPKeyDBRef					unlockDB = kInvalidPGPKeyDBRef;
	PGPKeyDBObjRef				udbTarget = kInvalidPGPKeyDBObjRef;
	PGPKeySetRef				udbSet = kInvalidPGPKeySetRef; /* do not free */
	PGPKeyListRef				udbList = kInvalidPGPKeyListRef;
	PGPKeyIterRef				udbIter = kInvalidPGPKeyIterRef;
	PGPKeyDBObjRef				udbObj = kInvalidPGPKeyDBObjRef;
	PGPHashContextRef			hashCon = kInvalidPGPHashContextRef;
	PGPSymmetricCipherContextRef symmCon = kInvalidPGPSymmetricCipherContextRef;
	PGPCBCContextRef			cbcCon = kInvalidPGPCBCContextRef;
	PGPUInt32					shareIndex;
	PGPUInt16					hashReps;
	PGPByte						*rawExport = NULL;
	PGPByte						*blockKey = NULL;
	PGPByte						*blockShares = NULL;
	PGPByte						*innerBlock = NULL;
	PGPByte						*outerBlock = NULL;
	PGPByte						*blockWalk = NULL;	/* do not free */
	PGPByte						*shareWalk = NULL;	/* do not free */
	PGPSize						rawExportSize = 0,
								innerBlockLen = 0,
								innerKeyBlockLen = 0,
								innerBlockPadLen = 0,
								outerBlockLen = 0;
	PGPByte						zeroIV[kReconCipherBlockSize];
	PGPUInt16					hashIndex;
	PGPByte						hashMagic[sizeof(PGPUInt32)];
	PGPByte						hashOut[20];
	PGPByte						tempShare[kReconShareSize];
	PGPBoolean					secretKey;
	
	pgpClearMemory( zeroIV, kReconCipherBlockSize );
	(void)PGPGetKeyDBObjBooleanProperty( con->targetKey,
			kPGPKeyProperty_IsSecret, &secretKey );
	if( !secretKey )
		return kPGPError_SecretKeyNotFound;
	
	/*	First Wrap */
	err = PGPNewKeyDB( con->context, &unlockDB );						CKERR;
	err = PGPCopyKeyDBObj( con->targetKey, unlockDB, &udbTarget );		CKERR;
	err = PGPChangePassphrase( udbTarget, 
			PGPOPassphrase( con->context, inPassphrase ),
			PGPOPassphrase( con->context, "" ),
			PGPOLastOption( con->context ) );							CKERR;
	udbSet = PGPPeekKeyDBRootKeySet( unlockDB );
	err = PGPOrderKeySet( udbSet, kPGPKeyOrdering_Any, FALSE, &udbList );CKERR;
	err = PGPNewKeyIter( udbList, &udbIter );							CKERR;
	PGPKeyIterSeek( udbIter, udbTarget );
	while( IsntPGPError( err ) && IsntPGPError(
		PGPKeyIterNextKeyDBObj( udbIter, kPGPKeyDBObjType_SubKey, &udbObj ) ) &&
		PGPKeyDBObjRefIsValid( udbObj ) )
	{
		err = PGPChangePassphrase( udbObj, 
				PGPOPassphrase( con->context, inPassphrase ),
				PGPOPassphrase( con->context, "" ),
				PGPOLastOption( con->context ) );						CKERR;
	}
	err = PGPExport( con->context,
			PGPOExportKeyDBObj( con->context, udbTarget ),
			PGPOAllocatedOutputBuffer( con->context, (void **) &rawExport,
					MAX_PGPUInt32, &rawExportSize ),
			PGPOExportPrivateKeys( con->context, TRUE ),
			PGPOExportFormat( con->context, kPGPExportFormat_Complete ),
			PGPOArmorOutput( con->context, FALSE ),
			PGPOLastOption( con->context ) );							CKERR;
	innerBlockLen = rawExportSize + sizeof(PGPUInt32);
	while( innerBlockLen % kReconCipherBlockSize )
		innerBlockLen++;
	innerKeyBlockLen = innerBlockLen;
	innerBlockLen += sizeof(PGPKeyID) + sizeof(PGPUInt16) +
						( kReconShareSize * kPGPRecon_NumShares );
	innerBlock = (PGPByte *)PGPNewSecureData( con->memMgr,
						innerBlockLen, kPGPMemoryMgrFlags_Clear);		CKERR;
	blockKey = (PGPByte *)PGPNewSecureData( con->memMgr,
						kReconstructWrapKeySize,
						kPGPMemoryMgrFlags_Clear);						CKERR;
	blockShares = (PGPByte *)PGPNewSecureData( con->memMgr,
						kReconNaturalShareSize * kPGPRecon_NumShares,
						kPGPMemoryMgrFlags_Clear);						CKERR;
	err = PGPGetKeyID( con->targetKey, (PGPKeyID *)innerBlock );		CKERR;
	err = PGPContextGetRandomBytes( con->context, blockKey,
						kReconstructWrapKeySize );						CKERR;
	err = PGPSecretShareData( con->context, blockKey, kReconstructWrapKeySize,
			kPGPRecon_Threshold, kPGPRecon_NumShares, blockShares );	CKERR;
	do
	{
		err = PGPContextGetRandomBytes( con->context, &hashReps,
							sizeof(PGPUInt16) );						CKERR;
	} while ( !hashReps || ( hashReps > kReconMaxHashReps ) );
	blockWalk = innerBlock + sizeof(PGPKeyID);
	*blockWalk++ = ( hashReps >> 8 ) & 0xFF;
	*blockWalk++ = hashReps & 0xFF;
	shareWalk = blockShares;
	err = PGPNewHashContext( con->context,
			kPGPHashAlgorithm_SHA, &hashCon );							CKERR;
	err = PGPNewSymmetricCipherContext( con->context,
			kPGPCipherAlgorithm_CAST5, &symmCon );						CKERR;
	err = PGPNewCBCContext( symmCon, &cbcCon );							CKERR;
	PGPUInt32ToEndian( kReconInnerMagic, kPGPBigEndian, hashMagic );
	for( shareIndex = 0; shareIndex < kPGPRecon_NumShares; shareIndex++ )
	{
		for( hashIndex = 0; hashIndex < hashReps; hashIndex++ )
		{
			(void) PGPContinueHash( hashCon, inPassInfo[shareIndex],
					strlen( inPassInfo[shareIndex] ) );
			(void) PGPContinueHash( hashCon, hashMagic, sizeof(hashMagic) );
		}
		err = PGPFinalizeHash( hashCon, hashOut );							CKERR;
		err = PGPResetHash( hashCon );										CKERR;
		pgpClearMemory( tempShare, sizeof(tempShare) );
		pgpCopyMemory( shareWalk, tempShare, kReconNaturalShareSize );
		shareWalk += kReconNaturalShareSize;
		err = PGPInitCBC( cbcCon, hashOut, zeroIV );						CKERR;
		err = PGPCBCEncrypt( cbcCon, tempShare, kReconShareSize, blockWalk );CKERR;
		blockWalk += kReconShareSize;
		pgpClearMemory( hashOut, sizeof(hashOut) );
	}
	PGPUInt32ToEndian( rawExportSize, kPGPBigEndian, blockWalk );
	pgpCopyMemory( rawExport, blockWalk + sizeof(PGPUInt32), rawExportSize );
	err = PGPInitCBC( cbcCon, blockKey, zeroIV );							CKERR;
	err = PGPCBCEncrypt( cbcCon, blockWalk, innerKeyBlockLen, blockWalk );	CKERR;
	/* innerBlock First Wrap complete */
	
	/*	Second Wrap */
	innerBlockPadLen = innerBlockLen;
	while( innerBlockPadLen % kReconCipherBlockSize )
		innerBlockPadLen++;
	outerBlockLen = sizeof(PGPKeyID) + sizeof(PGPReconPrompts) + sizeof(PGPUInt16) +
		sizeof(PGPUInt32) + kReconShareSize * kPGPRecon_NumShares + innerBlockPadLen;
	outerBlock = (PGPByte *)PGPNewSecureData( con->memMgr,
						outerBlockLen, kPGPMemoryMgrFlags_Clear);		CKERR;
	err = PGPGetKeyID( con->targetKey, (PGPKeyID *)outerBlock );		CKERR;
	blockWalk = outerBlock + sizeof(PGPKeyID);
	pgpCopyMemory( inPromptInfo, blockWalk, sizeof(PGPReconPrompts) );
	blockWalk += sizeof(PGPReconPrompts);
	*blockWalk++ = ( hashReps >> 8 ) & 0xFF;
	*blockWalk++ = hashReps & 0xFF;
	PGPUInt32ToEndian( innerBlockPadLen, kPGPBigEndian, blockWalk );
	blockWalk += sizeof(PGPUInt32);
	err = PGPContextGetRandomBytes( con->context, blockKey,
						kReconstructWrapKeySize );						CKERR;
	err = PGPSecretShareData( con->context, blockKey, kReconstructWrapKeySize,
			kPGPRecon_Threshold, kPGPRecon_NumShares, blockShares );	CKERR;
	PGPUInt32ToEndian( kReconOuterMagic, kPGPBigEndian, hashMagic );
	shareWalk = blockShares;
	for( shareIndex = 0; shareIndex < kPGPRecon_NumShares; shareIndex++ )
	{
		for( hashIndex = 0; hashIndex < hashReps; hashIndex++ )
		{
			(void) PGPContinueHash( hashCon, inPassInfo[shareIndex],
					strlen( inPassInfo[shareIndex] ) );
			(void) PGPContinueHash( hashCon, hashMagic, sizeof(hashMagic) );
		}
		err = PGPFinalizeHash( hashCon, hashOut );							CKERR;
		err = PGPResetHash( hashCon );										CKERR;
		pgpClearMemory( tempShare, sizeof(tempShare) );
		pgpCopyMemory( shareWalk, tempShare, kReconNaturalShareSize );
		shareWalk += kReconNaturalShareSize;
		err = PGPInitCBC( cbcCon, hashOut, zeroIV );						CKERR;
		err = PGPCBCEncrypt( cbcCon, tempShare, kReconShareSize, blockWalk );CKERR;
		blockWalk += kReconShareSize;
		pgpClearMemory( hashOut, sizeof(hashOut) );
	}
	pgpCopyMemory( innerBlock, blockWalk, innerBlockLen );
	err = PGPInitCBC( cbcCon, blockKey, zeroIV );							CKERR;
	err = PGPCBCEncrypt( cbcCon, blockWalk, innerBlockPadLen, blockWalk );	CKERR;

	con->blobData = outerBlock;
	con->blobDataSize = outerBlockLen;

#ifdef PGPRECONSELFTEST
	PGPKeyDBRef	testKeyDB = kInvalidPGPKeyDBRef;
	
	err = PGPReconstruct( reconRef, inPromptInfo, inPassInfo, &testKeyDB );
	if( PGPKeyDBRefIsValid( testKeyDB ) )
		(void)PGPFreeKeyDB( testKeyDB );
#endif
	

done:
	if( PGPCBCContextRefIsValid( cbcCon ) )
		(void) PGPFreeCBCContext( cbcCon );
	if( PGPHashContextRefIsValid( hashCon ) )
		(void) PGPFreeHashContext( hashCon );
	if( IsntNull( blockShares ) )
		(void) PGPFreeData( blockShares );
	if( IsntNull( blockKey ) )
		(void) PGPFreeData( blockKey );
	if( IsntNull( innerBlock ) )
		(void) PGPFreeData( innerBlock );
	if( IsntNull( rawExport ) )
	{
		pgpClearMemory( rawExport, rawExportSize );
		(void) PGPFreeData( rawExport );
	}
	if( PGPKeyIterRefIsValid( udbIter ) )
		(void) PGPFreeKeyIter( udbIter );
	if( PGPKeyListRefIsValid( udbList ) )
		(void) PGPFreeKeyList( udbList );
	if( PGPKeyDBRefIsValid( unlockDB ) )
		(void) PGPFreeKeyDB( unlockDB );

	return err;
}

	PGPError
PGPGetReconstruction(
	PGPReconContextRef			reconRef,
	PGPByte 					**reconData,
	PGPSize 					*reconDataSize )
{
	PGPError					err = kPGPError_NoErr;
	PGPReconContextPriv *		con = (PGPReconContextPriv *)reconRef;

	PGPValidatePtr( reconRef );
	PGPValidatePtr( reconData );
	PGPValidatePtr( reconDataSize );

	pgpAssert( con->blobDataSize > 0 );
	PGPValidatePtr( con->blobData );
	
	*reconData = (PGPByte *)PGPNewSecureData( con->memMgr,
		con->blobDataSize,
		kPGPMemoryMgrFlags_Clear );
	if( IsNull( *reconData ) )
	{
		err = kPGPError_OutOfMemory;
		goto done;
	}

	pgpCopyMemory( con->blobData, reconData, con->blobDataSize );
	*reconDataSize = con->blobDataSize;

done:
	return err;
}

	PGPError
PGPSendReconstruction(
	PGPReconContextRef			reconRef )
{
	PGPError					err = kPGPError_NoErr;
	PGPReconContextPriv *		con = (PGPReconContextPriv *)reconRef;
	PGPUInt32					numValue;
	char						serverURL[256];
	PGPKeyServerClass			serverClass;
	PGPKeyID					keyid;
	PGPEvent					event;
	char						szKeyID[kPGPMaxKeyIDStringSize] = { 0 };
	char *						dn = NULL;
	char *						userDN = NULL;
	PGPSize						dnLength = 0;
	PGPSize						numVars = 0;
	PGPldapContextRef			ldap = kInvalidPGPldapContextRef;
	PGPldapURLDesc				*lud = NULL;
	PGPBoolean					bSecure = FALSE;
	PGPtlsContextRef			tlsContext = kInvalidPGPtlsContextRef;
	PGPtlsSessionRef			tlsSession = kInvalidPGPtlsSessionRef;

	/*	Check Server Info */
#ifndef PGP_CMDLINE
	err = PGPGetPrefStringBuffer( con->clientPrefs, kPGPPrefLDAPReconServer,
				sizeof(serverURL), serverURL );	CKERR;
	err = PGPGetPrefNumber( con->clientPrefs, kPGPPrefLDAPReconServerType,
				&numValue );	CKERR;
#else /* PGP_CMDLINE */
	numValue = con->serverType;
	strcpy(serverURL, con->szServerURL);
#endif /* PGP_CMDLINE */
	serverClass = (PGPKeyServerClass) numValue;
	if( !strlen( serverURL ) )
	{
		err = kPGPError_BadParams;
		goto done;
	}

	/*	Send Wrapper to Server */
	err = PGPNewLDAPContext( con->context, &ldap );						CKERR;

	/*
	 * PGPldapURLParse needs the server URL to be an actual URL, meaning
	 * that it has at the minimum "ldap://x.x.x.x/".  We add the trailing
	 * '/' to make the URL fit this form.
	 */
	if( serverClass == kPGPKeyServerClass_PGP )
		strcat( serverURL, "/" );
	
	err = PGPldapURLParse( ldap, serverURL, &lud );						CKERR;

	if( strstr( serverURL, "ldaps://" ) != NULL )
		bSecure = TRUE;

	if( lud->port == 0 )
		lud->port = bSecure ? kPGPldap_DefaultSecurePort : kPGPldap_DefaultPort;
	
	pgpClearMemory( &event, sizeof(PGPEvent) );
	event.type = kPGPEvent_KeyServerEvent;
	event.data.keyServerData.state = kPGPKeyServerState_Opening;
	if( IsntNull( con->handler ) )
		err = con->handler( con->context, &event, con->userValue );		CKERR;

	err = sCountLDAPVariables( lud->dn, &numVars );						CKERR;

	userDN = (char *) PGPNewData( con->memMgr, 
			strlen( lud->dn ) + 
			( IsntNull( con->authUser ) ? strlen( con->authUser ) : 0 ) * numVars + 
			1,
			kPGPMemoryMgrFlags_Clear );
	if( IsNull( userDN ) )
	{
		err = kPGPError_OutOfMemory;
		goto done;
	}
	err = sReplaceLDAPVariables( lud->dn, con->authUser, userDN );CKERR;
	
	err = PGPldapOpen( ldap, lud->host, lud->port );					CKERR;

	if( bSecure )
	{
		err = sEstablishTLSSession( con->context,
			ldap, 
			&tlsContext, 
			&tlsSession );
		if( IsPGPError( err ) )
		{
			(void) sSendTLSEvent( con->context, con->handler, con->userValue,
				kPGPKeyServerState_TLSUnableToSecureConnection, tlsSession );
			goto done;
		}
		else
		{
			err = sSendTLSEvent( con->context, con->handler, con->userValue,
				kPGPKeyServerState_TLSConnectionSecured, tlsSession ); CKERR;
		}
	}

	err = PGPldapSimpleBindSync( ldap, userDN, con->authPass );			CKERR;

	err = PGPGetKeyID( con->targetKey, &keyid );						CKERR;
	err = PGPGetKeyIDString( &keyid, kPGPKeyIDString_Full, szKeyID );	CKERR;

	/*
	 * If using PGP certserver, we need to allocate enough extra
	 * room for the DN we get later from querying cn=PGPServerInfo.
	 */
	if( serverClass == kPGPKeyServerClass_PGP )
		dnLength = sizeof( serverURL );
	else
		dnLength = strlen( userDN );

	dn = (char *)PGPNewData( con->memMgr, 
			strlen( kPGPReconCertIDAttr ) + sizeof( "=" ) + strlen( szKeyID ) +
			sizeof( ", " ) +  dnLength + 1, 
			kPGPMemoryMgrFlags_Clear );
	if( IsNull( dn ) )
	{
		err = kPGPError_OutOfMemory;
		goto done;
	}
	sprintf( dn, "%s=%s, %s", kPGPReconCertIDAttr, szKeyID + 2, userDN );

	event.data.keyServerData.state = kPGPKeyServerState_Uploading;
	if( IsntNull( con->handler ) )
		err = con->handler( con->context, &event, con->userValue );		CKERR;

	err = sSendToServer( ldap, dn, con, con->blobData, con->blobDataSize );	CKERR;

	event.data.keyServerData.state = kPGPKeyServerState_Closing;
	if( IsntNull( con->handler ) )
		err = con->handler( con->context, &event, con->userValue );		CKERR;

	err = PGPldapUnbind( ldap );										CKERR;

done:
	if( PGPtlsContextRefIsValid( tlsContext ) )
		(void) PGPFreeTLSContext( tlsContext );
	if( PGPtlsSessionRefIsValid( tlsSession ) )
		(void) PGPFreeTLSSession( tlsSession );
	if( IsntNull( userDN ) )
		(void) PGPFreeData( userDN );
	if( IsntNull( dn ) )
		(void) PGPFreeData( dn );
	if( IsntNull( lud ) )
		(void) PGPFreeLDAPURLDesc( lud );
	if( PGPldapContextRefIsValid( ldap ) )
		(void) PGPFreeLDAPContext( ldap );

	return err;
}

	PGPError
PGPGetReconstructionPrompts(
	PGPReconContextRef			reconRef,
	PGPReconPrompts				outPromptInfo )
{
	PGPError					err				= kPGPError_NoErr;
	PGPReconContextPriv *		con				= (PGPReconContextPriv *)reconRef;
	PGPUInt32					serverType		= kPGPKeyServerClass_PGP;
	char						serverURL[256]	= { 0 };
	char						serverDN[256]	= { 0 };
	char *						dn				= NULL;
	char *						userDN			= NULL;
	PGPldapContextRef			ldap			= kInvalidPGPldapContextRef;
	PGPldapURLDesc *			lud				= NULL;
	PGPBoolean					bSecure			= FALSE;
	PGPtlsContextRef			tlsContext		= kInvalidPGPtlsContextRef;
	PGPtlsSessionRef			tlsSession		= kInvalidPGPtlsSessionRef;
	PGPKeyID					keyid;
	char						szKeyID[kPGPMaxKeyIDStringSize] = { 0 };
	char *						attrs[]			= { kPGPReconDataAttr, NULL };
	PGPldapMessageRef			message			= kInvalidPGPldapMessageRef;
	PGPberValue **				berValues		= NULL;
	char **						values			= NULL;
	PGPByte *					blockWalk		= NULL;
	PGPSize						numVars			= 0;
#ifndef PGP_CMDLINE
	PGPUInt32					numValue		= 0;
#endif
	PGPEvent					event;
	PGPKeyID					targetKeyID;

	/*	CertServer: Search for Prompts based on KeyID
		LDAPServer: Retrieve blob based on LDAP URL/username/pass */
		
#ifndef PGP_CMDLINE
	err = PGPGetPrefStringBuffer( con->clientPrefs, kPGPPrefLDAPReconServer,
				sizeof( serverURL ), serverURL );	CKERR;
	if( !strlen( serverURL ) )
	{
		err = kPGPError_BadParams;
		goto done;
	}
#else /* PGP_CMDLINE */
	strcpy(serverURL, con->szServerURL);
#endif /* PGP_CMDLINE */

	err = PGPNewLDAPContext( con->context, &ldap );						CKERR;

#ifndef PGP_CMDLINE
	err = PGPGetPrefNumber( con->clientPrefs, kPGPPrefLDAPReconServerType,
				&numValue );	CKERR;
	serverType = (PGPKeyServerClass) numValue;
#else /* PGP_CMDLINE */
	serverType = (PGPKeyServerClass) con->serverType;
#endif /* PGP_CMDLINE */

	if( serverType == kPGPKeyServerClass_PGP )
		strcat( serverURL, "/" );
	err = PGPldapURLParse( ldap, serverURL, &lud );						CKERR;

	if( strstr( serverURL, "ldaps://" ) != NULL )
		bSecure = TRUE;

	if( lud->port == 0 )
		lud->port = bSecure ? kPGPldap_DefaultSecurePort : kPGPldap_DefaultPort;

	pgpClearMemory( &event, sizeof( PGPEvent ) );
	event.type = kPGPEvent_KeyServerEvent;
	event.data.keyServerData.state = kPGPKeyServerState_Opening;
	if( IsntNull( con->handler ) )
		err = con->handler( con->context, &event, con->userValue );		CKERR;

	err = PGPldapOpen( ldap, lud->host, lud->port );					CKERR;
	if( bSecure )
	{
		err = sEstablishTLSSession( con->context,
			ldap, 
			&tlsContext, 
			&tlsSession );
		if( IsPGPError( err ) )
		{
			(void) sSendTLSEvent( con->context, con->handler, con->userValue,
				kPGPKeyServerState_TLSUnableToSecureConnection, tlsSession );
			goto done;
		}
		else
		{
			err = sSendTLSEvent( con->context, con->handler, con->userValue,
				kPGPKeyServerState_TLSConnectionSecured, tlsSession ); CKERR;
		}
	}

	err = PGPGetKeyID( con->targetKey, &keyid );						CKERR;
	err = PGPGetKeyIDString( &keyid, kPGPKeyIDString_Full, szKeyID );	CKERR;

#ifndef PGP_CMDLINE
	err = PGPGetPrefNumber( con->clientPrefs, kPGPPrefLDAPReconServerType,
				&serverType );	CKERR;
#else /* PGP_CMDLINE */
	serverType = (PGPKeyServerClass)con->serverType;
#endif /* PGP_CMDLINE */
	if( serverType == kPGPKeyServerClass_PGP )
	{
		/* PGP Keyserver */
		/* Query cn=PGPServerInfo for certserver location */
		err = sGetCertserverLocation( ldap, serverDN );					CKERR;
		dn = serverDN;
		dn = (char *)PGPNewData( con->memMgr, 
				strlen( kPGPReconCertIDAttr ) + sizeof( "=" ) + strlen( szKeyID ) +
				sizeof( ", " ) + strlen( serverDN ) + 1, 
				kPGPMemoryMgrFlags_Clear );
		if( IsNull( dn ) )
		{
			err = kPGPError_OutOfMemory;
			goto done;
		}
		sprintf( dn, "%s=%s, %s", kPGPReconCertIDAttr, szKeyID + 2, serverDN );

		/* Also get hashReps  */
		attrs[0] = kPGPReconPromptsAttr;
		attrs[1] = kPGPReconHashRepsAttr;
		attrs[2] = NULL;
	}
	else
	{
		/* Generic LDAP server */
		err = sCountLDAPVariables( lud->dn, &numVars );					CKERR;

		userDN = (char *) PGPNewData( con->memMgr, 
				strlen( lud->dn ) + 
				( IsntNull( con->authUser ) ? strlen( con->authUser ) : 0 ) * numVars + 
				1,
				kPGPMemoryMgrFlags_Clear );
		if( IsNull( userDN ) )
		{
			err = kPGPError_OutOfMemory;
			goto done;
		}
		err = sReplaceLDAPVariables( lud->dn, con->authUser, userDN );	CKERR;
		
		err = PGPldapSimpleBindSync( ldap, userDN, con->authPass );		CKERR;
		
		dn = (char *)PGPNewData( con->memMgr, 
				strlen( kPGPReconCertIDAttr ) + sizeof( "=" ) + strlen( szKeyID ) +
				sizeof( ", " ) + strlen( userDN ) + 1, kPGPMemoryMgrFlags_Clear );
		if( IsNull( dn ) )
		{
			err = kPGPError_OutOfMemory;
			goto done;
		}
		sprintf( dn, "%s=%s, %s", kPGPReconCertIDAttr, szKeyID + 2, userDN );

		attrs[0] = kPGPReconDataAttr;
		attrs[1] = NULL;
	}

	/* dn now contains the DN we need to search for the key stuff */

	err = PGPNewLDAPMessage( ldap, &message );						CKERR;

	event.data.keyServerData.state = kPGPKeyServerState_Querying;
	if( IsntNull( con->handler ) )
		err = con->handler( con->context, &event, con->userValue );	CKERR;

	err = PGPldapSearchSync( ldap, dn, kPGPldapScope_Base, kPGPObjectclassAny,
				attrs, FALSE, message );
	if( err == kPGPError_LDAPNoSuchObject )
		err = kPGPError_ItemNotFound;
	CKERR;

	event.data.keyServerData.state = kPGPKeyServerState_ProcessingResults;
	if( IsntNull( con->handler ) )
		err = con->handler( con->context, &event, con->userValue );	CKERR;

	/*	Compare KeyID for match and copy retrieved prompts to outPromptInfo,
		and save entire blob in con->blobData,blobDataSize if this is an LDAPServer */

	if( serverType == kPGPKeyServerClass_PGP )
	{
		/* PGP Keyserver */
		err = PGPldapGetValuesLen( ldap, message, kPGPReconPromptsAttr, &berValues );CKERR;

		if( IsNull( berValues ) )
		{
			err = kPGPError_ItemNotFound;
			goto done;
		}
	
		blockWalk = berValues[0]->value;

		/* Also get hashReps */
		err = PGPldapGetValues( ldap, message, kPGPReconHashRepsAttr, &values ); CKERR;

		con->hashReps = atoi( values[0] );
	}
	else
	{
		/* Generic LDAP server */
		err = PGPldapGetValuesLen( ldap, message, kPGPReconDataAttr, &berValues );CKERR;

		if( IsNull( berValues ) )
		{
			err = kPGPError_ItemNotFound;
			goto done;
		}
	
		con->blobDataSize = berValues[0]->length;
		con->blobData = (PGPByte *)PGPNewSecureData( con->memMgr, con->blobDataSize,
			kPGPMemoryMgrFlags_Clear );
		if( IsNull( con->blobData ) )
		{
			con->blobDataSize = 0;
			err = kPGPError_OutOfMemory;
			goto done;
		}
		pgpCopyMemory( berValues[0]->value, con->blobData, con->blobDataSize );

		blockWalk = con->blobData;

		if( con->blobDataSize < kReconMinBlobSize )
		{
			err = kPGPError_KeyInvalid;
			goto done;
		}
		err = PGPGetKeyID( con->targetKey, &targetKeyID );						CKERR;
		if( PGPCompareKeyIDs( (PGPKeyID *)blockWalk, &targetKeyID ) )
		{
			err = kPGPError_KeyInvalid;
			goto done;
		}
		blockWalk += sizeof( PGPKeyID );
		con->hashReps = PGPEndianToUInt16( kPGPBigEndian, blockWalk + sizeof( PGPReconPrompts ) );
	}

	pgpCopyMemory( blockWalk, outPromptInfo, sizeof( PGPReconPrompts ) );

	event.data.keyServerData.state = kPGPKeyServerState_Closing;
	if( IsntNull( con->handler ) )
		err = con->handler( con->context, &event, con->userValue );		CKERR;

	
done:
	if( PGPtlsContextRefIsValid( tlsContext ) )
		(void) PGPFreeTLSContext( tlsContext );
	if( PGPtlsSessionRefIsValid( tlsSession ) )
		(void) PGPFreeTLSSession( tlsSession );
	if( PGPldapContextRefIsValid( ldap ) )
		(void) PGPFreeLDAPContext( ldap );
	if( IsntNull( lud ) )
		(void) PGPFreeLDAPURLDesc( lud );
	if( PGPldapMessageRefIsValid( message ) )
		(void) PGPFreeLDAPMessage( message );
	if( IsntNull( values ) )
		(void) PGPFreeLDAPValues( values );
	if( IsntNull( berValues ) )
		(void) PGPFreeLDAPValuesLen( berValues );
	if( IsntNull( userDN ) )
		(void) PGPFreeData( userDN );
	if( IsntNull( dn ) )
		(void) PGPFreeData( dn );

	return err;
}


/*
 * Armor 3 raw bytes into 4
 * If armoring n < 3 bytes, make the trailers zero, and
 * then overwrite the trailing 3-n bytes with '='
 */
	static void
sArmorMorsel(PGPByte const raw[3], char armor[4])
{
	armor[0] = armorTable[raw[0] >> 2 & 0x3f];
	armor[1] = armorTable[(raw[0] << 4 & 0x30) + (raw[1] >> 4 & 0x0f)];
	armor[2] = armorTable[(raw[1] << 2 & 0x3c) + (raw[2] >> 6 & 0x03)];
	armor[3] = armorTable[raw[2] & 0x3f];
}

	static int
sArmorLine (PGPByte const *in, unsigned inlen, char *out)
{
	unsigned len;
	int t;
	char const *out0 = out;

	/* Fill the output buffer from the input buffer */
	for (len = 0; len < inlen; len += 3)
	{
		sArmorMorsel (in, out);
		in += 3;
		out += 4;
	}

	/* Now back up and erase any overrun */
	t = (int)(inlen - len); 		/* Zero or negative */
	while (t)
		out[t++] = '=';
		  
	return (out - out0);
}


	PGPError
PGPGetReconstructionData(
	PGPReconContextRef			reconRef,
	PGPReconPasses				inPassInfo,
	PGPByte						**outReconData,
	PGPSize						*outReconSize )
{
	PGPError					err = kPGPError_NoErr;
	PGPReconContextPriv *		con				= (PGPReconContextPriv *)reconRef;
	PGPldapContextRef			ldap = kInvalidPGPldapContextRef;
	PGPUInt32					numValue;
	PGPKeyServerClass			serverClass;
	char						serverURL[256];
	char						szCertserverDN[256];
	char *						dn = NULL;
	PGPldapMessageRef			message = kInvalidPGPldapMessageRef;
	char						szKeyID[kPGPMaxKeyIDStringSize];
	char						szArmoredPassKey[
									( ( ( kPGPRecon_NumShares * kReconstructWrapKeySize / 3 ) + 1 ) * 4 ) + 1
									];
	char *						attrs[2];
	PGPberValue **				berValues = NULL;
	PGPldapURLDesc *			lud = NULL;
	PGPBoolean					bSecure = FALSE;
	PGPtlsContextRef			tlsContext = kInvalidPGPtlsContextRef;
	PGPtlsSessionRef			tlsSession = kInvalidPGPtlsSessionRef;
	PGPKeyID					keyid;
	PGPSize						len = 0;
	PGPByte						passKey[kPGPRecon_NumShares][kReconstructWrapKeySize];
	PGPHashContextRef			hashCon = kInvalidPGPHashContextRef;
	PGPByte						hashMagic[sizeof(PGPUInt32)];
	PGPUInt32					shareIndex;
	PGPUInt32					hashIndex;
	PGPByte						hashOut[20];

	PGPValidatePtr( outReconData );
	PGPValidatePtr( outReconSize );

	/* We need to have already called PGPGetReconstructionPrompts and
	 * gotten the number of hashReps */
	pgpAssert( con->hashReps != 0 );

	/*	Check Server Info */
#ifndef PGP_CMDLINE
	err = PGPGetPrefStringBuffer( con->clientPrefs, kPGPPrefLDAPReconServer,
				sizeof(serverURL), serverURL );	CKERR;
	err = PGPGetPrefNumber( con->clientPrefs, kPGPPrefLDAPReconServerType,
				&numValue );	CKERR;
#else /* PGP_CMDLINE */
	numValue = con->serverType;
	strcpy(serverURL, con->szServerURL);
#endif /* PGP_CMDLINE */
	serverClass = (PGPKeyServerClass) numValue;
	if( !strlen( serverURL ) )
	{
		err = kPGPError_BadParams;
		goto done;
	}

	if( serverClass == kPGPKeyServerClass_PGP )
	{
		/*
		 * We have the hashed passphrases in inPassInfo[...] and the
		 * number of hash repetitions in con->hashReps.
		 */

		err = PGPNewLDAPContext( con->context, &ldap );						CKERR;
		if( IsNull( ldap ) )
		{
			err = kPGPError_OutOfMemory;
			goto done;
		}

		strcat( serverURL, "/" );
		
		err = PGPldapURLParse( ldap, serverURL, &lud );						CKERR;

		if( strstr( serverURL, "ldaps://" ) != NULL )
			bSecure = TRUE;

		if( lud->port == 0 )
			lud->port = bSecure ? kPGPldap_DefaultSecurePort : kPGPldap_DefaultPort;

		err = PGPldapOpen( ldap, lud->host, lud->port );					CKERR;

		if( bSecure )
		{
			err = sEstablishTLSSession( con->context,
				ldap, 
				&tlsContext, 
				&tlsSession );
			if( IsPGPError( err ) )
			{
				(void) sSendTLSEvent( con->context, con->handler, con->userValue,
					kPGPKeyServerState_TLSUnableToSecureConnection, tlsSession );
				goto done;
			}
			else
			{
				err = sSendTLSEvent( con->context, con->handler, con->userValue,
					kPGPKeyServerState_TLSConnectionSecured, tlsSession ); CKERR;
			}
		}

		err = PGPNewHashContext( con->context,
			kPGPHashAlgorithm_SHA, &hashCon );									CKERR;

		/* Hash all 5 outer keys */
		PGPUInt32ToEndian( kReconOuterMagic, kPGPBigEndian, hashMagic );
		for( shareIndex = 0; shareIndex < kPGPRecon_NumShares; shareIndex++ )
		{
			for( hashIndex = 0; hashIndex < con->hashReps; hashIndex++ )
			{
				(void) PGPContinueHash( hashCon, inPassInfo[shareIndex],
						strlen( inPassInfo[shareIndex] ) );
				(void) PGPContinueHash( hashCon, hashMagic, sizeof(hashMagic) );
			}
			err = PGPFinalizeHash( hashCon, hashOut );							CKERR;
			err = PGPResetHash( hashCon );										CKERR;
			pgpCopyMemory( hashOut, passKey[shareIndex], kReconstructWrapKeySize );
			pgpClearMemory( hashOut, sizeof(hashOut) );
		}

		len = sArmorLine( (PGPByte *) passKey, 
			kPGPRecon_NumShares * kReconstructWrapKeySize,
			(char *) szArmoredPassKey );
		szArmoredPassKey[len] = '\0';

		err = sGetCertserverLocation( ldap, szCertserverDN );				CKERR;

		err = PGPNewLDAPMessage( ldap, &message );							CKERR;
		if( IsNull( message ) )
		{
			err = kPGPError_OutOfMemory;
			goto done;
		}

		err = PGPGetKeyID( con->targetKey, &keyid );						CKERR;
		err = PGPGetKeyIDString( &keyid, kPGPKeyIDString_Full, szKeyID );	CKERR;

		dn = (char *)PGPNewData( con->memMgr, 
				strlen( kPGPReconCertIDAttr ) + sizeof( "=" ) + strlen( szKeyID ) +
				sizeof( ", " ) +  strlen( szCertserverDN ) + 1, 
				kPGPMemoryMgrFlags_Clear );
		if( IsNull( dn ) )
		{
			err = kPGPError_OutOfMemory;
			goto done;
		}
		
		sprintf( dn, "%s=%s, %s", kPGPReconCertIDAttr, szKeyID + 2, szCertserverDN );

		err = PGPldapBindSync( ldap, dn, szArmoredPassKey, kPGPldapAuth_Simple );CKERR;

		attrs[0] = kPGPReconDataAttr;
		attrs[1] = NULL;

		err = PGPldapSearchSync( ldap, 
			dn, 
			kPGPldapScope_Base,
			kPGPObjectclassAny,
			attrs,
			FALSE,
			message );
		if( err == kPGPError_LDAPNoSuchObject )
			err = kPGPError_ItemNotFound;
		CKERR;

		err = PGPldapGetValuesLen( ldap, message, kPGPReconDataAttr, &berValues );CKERR;

		if( IsNull( berValues ) )
		{
			err = kPGPError_ItemNotFound;
			goto done;
		}

		con->blobDataSize = berValues[0]->length;
		con->blobData = (PGPByte *)PGPNewSecureData( con->memMgr, con->blobDataSize,
			kPGPMemoryMgrFlags_Clear );
		if( IsNull( con->blobData ) )
		{
			con->blobDataSize = 0;
			err = kPGPError_OutOfMemory;
			goto done;
		}
		pgpCopyMemory( berValues[0]->value, con->blobData, con->blobDataSize );
	}
	
	*outReconData = (PGPByte *)PGPNewSecureData( con->memMgr,
		con->blobDataSize, kPGPMemoryMgrFlags_Clear );
	if( IsNull( *outReconData ) )
	{
		err = kPGPError_OutOfMemory;
		goto done;
	}

	pgpCopyMemory( con->blobData, *outReconData, con->blobDataSize );
	*outReconSize = con->blobDataSize;

done:
	if( PGPtlsContextRefIsValid( tlsContext ) )
		(void) PGPFreeTLSContext( tlsContext );
	if( PGPtlsSessionRefIsValid( tlsSession ) )
		(void) PGPFreeTLSSession( tlsSession );
	if( IsntNull( dn ) )
		(void) PGPFreeData( dn );
	if( IsntNull( lud ) )
		(void) PGPFreeLDAPURLDesc( lud );
	if( IsntNull( berValues ) )
		(void) PGPFreeLDAPValuesLen( berValues );
	if( PGPldapMessageRefIsValid( message ) )
		(void) PGPFreeLDAPMessage( message );
	if( PGPldapContextRefIsValid( ldap ) )
		(void) PGPFreeLDAPContext( ldap );
	if( PGPHashContextRefIsValid( hashCon ) )
		(void) PGPFreeHashContext( hashCon );

	return err;
}


	PGPError
PGPReconstruct(
	PGPReconContextRef			reconRef,
	PGPReconPasses				inPassInfo,
	PGPByte						*inReconData,
	PGPSize						inReconDataSize,
	PGPKeyDBRef					*outReconstructedKey )
{
	PGPError					err = kPGPError_NoErr;
	PGPReconContextPriv *		con = (PGPReconContextPriv *)reconRef;
	PGPByte						zeroIV[kReconCipherBlockSize];
	PGPByte						*blockWalk;
	PGPKeyID					targetKeyID;
	PGPUInt16					hashReps;
	PGPUInt32					innerBlockPadLen,
								innerBlockLen;
	PGPByte						*innerBlock = NULL;
	PGPUInt16					matrixIndex[kPGPRecon_Threshold];
	PGPUInt16					matrixIndex2[kPGPRecon_Threshold];
	PGPUInt32					hashIndex,
								shareIndex;
	PGPByte						passKey[kPGPRecon_NumShares][kReconstructWrapKeySize];
	PGPByte						shares[kPGPRecon_Threshold][kReconNaturalShareSize];
	PGPSymmetricCipherContextRef symmCon = kInvalidPGPSymmetricCipherContextRef;
	PGPCBCContextRef			cbcCon = kInvalidPGPCBCContextRef;
	PGPHashContextRef			hashCon = kInvalidPGPHashContextRef;
	PGPByte						hashMagic[sizeof(PGPUInt32)];
	PGPByte						hashOut[20];
	PGPByte						shareOut[kReconShareSize];
	PGPByte						blockKey[kReconstructWrapKeySize];
	
	*outReconstructedKey = kInvalidPGPKeyDBRef;
	pgpClearMemory( zeroIV, kReconCipherBlockSize );

	err = PGPNewHashContext( con->context,
			kPGPHashAlgorithm_SHA, &hashCon );									CKERR;

	blockWalk = inReconData;
	if( inReconDataSize < kReconMinBlobSize )
	{
		err = kPGPError_KeyInvalid;
		goto done;
	}
	err = PGPGetKeyID( con->targetKey, &targetKeyID );							CKERR;
	if( PGPCompareKeyIDs( (PGPKeyID *)blockWalk, &targetKeyID ) )
	{
		err = kPGPError_KeyInvalid;
		goto done;
	}
	blockWalk += sizeof(PGPKeyID) + sizeof(PGPReconPrompts);
	hashReps = PGPEndianToUInt16( kPGPBigEndian, blockWalk );

	/* Hash all 5 outer keys */
	PGPUInt32ToEndian( kReconOuterMagic, kPGPBigEndian, hashMagic );
	for( shareIndex = 0; shareIndex < kPGPRecon_NumShares; shareIndex++ )
	{
		for( hashIndex = 0; hashIndex < hashReps; hashIndex++ )
		{
			(void) PGPContinueHash( hashCon, inPassInfo[shareIndex],
					strlen( inPassInfo[shareIndex] ) );
			(void) PGPContinueHash( hashCon, hashMagic, sizeof(hashMagic) );
		}
		err = PGPFinalizeHash( hashCon, hashOut );							CKERR;
		err = PGPResetHash( hashCon );										CKERR;
		pgpCopyMemory( hashOut, passKey[shareIndex], kReconstructWrapKeySize );
		pgpClearMemory( hashOut, sizeof(hashOut) );
	}
	
	/*	Unwrap outer layer */
	blockWalk = inReconData;
	blockWalk += sizeof(PGPKeyID) + sizeof(PGPReconPrompts);
	blockWalk += sizeof(PGPUInt16);
	innerBlockPadLen = PGPEndianToUInt32( kPGPBigEndian, blockWalk );
	blockWalk += sizeof(PGPUInt32);
	
	err = PGPNewSymmetricCipherContext( con->context,
			kPGPCipherAlgorithm_CAST5, &symmCon );							CKERR;
	err = PGPNewCBCContext( symmCon, &cbcCon );								CKERR;
	innerBlock = (PGPByte *)PGPNewSecureData( con->memMgr,
						innerBlockPadLen, kPGPMemoryMgrFlags_Clear);		CKERR;
	
	/* Try all possible combinations of 3 keys against innerBlock		*/
	/* Use only 3 so that some wrong answers don't penalize				*/
	/* Going 6 levels deep, redented for readability...					*/
		for( matrixIndex[0] = 0; matrixIndex[0] < kPGPRecon_NumShares; matrixIndex[0]++ )
	{	for( matrixIndex[1] = 0; matrixIndex[1] < kPGPRecon_NumShares; matrixIndex[1]++ )
	{	for( matrixIndex[2] = 0; matrixIndex[2] < kPGPRecon_NumShares; matrixIndex[2]++ )
	{
		if( ( matrixIndex[0] == matrixIndex[1] ) || ( matrixIndex[1] == matrixIndex[2] ) ||
			( matrixIndex[2] == matrixIndex[0] ) )
			continue;
		for( matrixIndex2[0] = 0; matrixIndex2[0] < kPGPRecon_NumShares; matrixIndex2[0]++ )
	{	for( matrixIndex2[1] = 0; matrixIndex2[1] < kPGPRecon_NumShares; matrixIndex2[1]++ )
	{	for( matrixIndex2[2] = 0; matrixIndex2[2] < kPGPRecon_NumShares; matrixIndex2[2]++ )
	{
		if( ( matrixIndex2[0] == matrixIndex2[1] ) || ( matrixIndex2[1] == matrixIndex2[2] ) ||
			( matrixIndex2[2] == matrixIndex2[0] ) )
			continue;
		for( shareIndex = 0; shareIndex < kPGPRecon_Threshold; shareIndex++ )
		{
			err = PGPInitCBC( cbcCon, passKey[matrixIndex[shareIndex]], zeroIV );	CKERR;
			err = PGPCBCDecrypt( cbcCon, blockWalk +
					kReconShareSize * matrixIndex2[shareIndex],
					kReconShareSize, shareOut );									CKERR;
			pgpCopyMemory( shareOut, shares[shareIndex], kReconNaturalShareSize );
		}
		if( ( shares[0][0] != kPGPShare_Version1 ) ||
			( shares[1][0] != kPGPShare_Version1 ) ||
			( shares[2][0] != kPGPShare_Version1 ) )
			continue;
		err = PGPSecretReconstructData( con->context, shares, 
				kReconstructWrapKeySize, kPGPRecon_Threshold, blockKey );
		if( IsPGPError( err ) )
		{
			err = kPGPError_NoErr;
			continue;
		}
		err = PGPInitCBC( cbcCon, blockKey, zeroIV );								CKERR;
		err = PGPCBCDecrypt( cbcCon, blockWalk + kReconShareSize * kPGPRecon_NumShares,
				innerBlockPadLen, innerBlock );										CKERR;
		if( !PGPCompareKeyIDs( (PGPKeyID *)innerBlock, &targetKeyID ) )
		{
			/* Outer layer has been removed */
			goto foundCombo;
		}
	}}}}}}
	err = kPGPError_BadPassphrase;
	goto done;
foundCombo:
	/*	Unwrap inner layer */
	blockWalk = innerBlock + sizeof(PGPKeyID);
	if( hashReps != PGPEndianToUInt16( kPGPBigEndian, blockWalk ) )
	{
		err = kPGPError_BadPassphrase;
		goto done;
	}
	blockWalk += sizeof(PGPUInt16);
	/* Hash all 5 inner keys */
	PGPUInt32ToEndian( kReconInnerMagic, kPGPBigEndian, hashMagic );
	for( shareIndex = 0; shareIndex < kPGPRecon_NumShares; shareIndex++ )
	{
		for( hashIndex = 0; hashIndex < hashReps; hashIndex++ )
		{
			(void) PGPContinueHash( hashCon, inPassInfo[shareIndex],
					strlen( inPassInfo[shareIndex] ) );
			(void) PGPContinueHash( hashCon, hashMagic, sizeof(hashMagic) );
		}
		err = PGPFinalizeHash( hashCon, hashOut );								CKERR;
		err = PGPResetHash( hashCon );											CKERR;
		pgpCopyMemory( hashOut, passKey[shareIndex], kReconstructWrapKeySize );
		pgpClearMemory( hashOut, sizeof(hashOut) );
	}
	/* Decrypt the shares */
	for( shareIndex = 0; shareIndex < kPGPRecon_Threshold; shareIndex++ )
	{
		err = PGPInitCBC( cbcCon, passKey[matrixIndex[shareIndex]], zeroIV );	CKERR;
		err = PGPCBCDecrypt( cbcCon, blockWalk +
				kReconShareSize * matrixIndex2[shareIndex],
				kReconShareSize, shareOut );									CKERR;
		pgpCopyMemory( shareOut, shares[shareIndex], kReconNaturalShareSize );
	}
	err = PGPSecretReconstructData( con->context, shares, 
			kReconstructWrapKeySize, kPGPRecon_Threshold, blockKey );
	if( IsPGPError( err ) )
	{
		err = kPGPError_BadPassphrase;
		goto done;
	}
	blockWalk += kReconShareSize * kPGPRecon_NumShares;
	innerBlockPadLen -= sizeof(PGPKeyID) + sizeof(PGPUInt16) +
						kReconShareSize * kPGPRecon_NumShares;
	while( innerBlockPadLen % kReconCipherBlockSize )
		innerBlockPadLen--;
	err = PGPInitCBC( cbcCon, blockKey, zeroIV );								CKERR;
	err = PGPCBCDecrypt( cbcCon, blockWalk, innerBlockPadLen, blockWalk );		CKERR;
	innerBlockLen = PGPEndianToUInt32( kPGPBigEndian, blockWalk );
	blockWalk += sizeof(PGPUInt32);
	if( innerBlockLen > innerBlockPadLen )
	{
		err = kPGPError_CorruptData;
		goto done;
	}
	/*	Import private key */
	err = PGPImport( con->context, outReconstructedKey,
			PGPOInputBuffer( con->context, blockWalk, innerBlockLen ),
			PGPOLastOption( con->context ) );									CKERR;
	
done:
	pgpClearMemory( blockKey, sizeof(blockKey) );
	pgpClearMemory( passKey, sizeof(passKey) );
	pgpClearMemory( shares, sizeof(shares) );
	if( IsntNull( innerBlock ) )
		(void) PGPFreeData( innerBlock );
	if( PGPCBCContextRefIsValid( cbcCon ) )
		(void) PGPFreeCBCContext( cbcCon );
	if( PGPHashContextRefIsValid( hashCon ) )
		(void) PGPFreeHashContext( hashCon );
	return err;
}

#ifdef PGP_CMDLINE
	PGPError
PGPSetReconstructionServerURL(
	PGPReconContextRef			reconRef,
	char						*pszServerURL,
	PGPUInt32					dwServerType )
{
	PGPError					err = kPGPError_NoErr;
	PGPReconContextPriv *		con = (PGPReconContextPriv *)reconRef;
	
	PGPValidatePtr( reconRef );
	
	strcpy(con->szServerURL, pszServerURL);
	con->serverType  = dwServerType;
	
	return err;
}
#endif /* PGP_CMDLINE */

/*__Editor_settings____

	Local Variables:
	tab-width: 4
	End:
	vi: ts=4 sw=4
	vim: si
_____________________*/
