/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.

	$Id: pgpPassphraseUtils.c,v 1.8 2002/08/30 23:38:12 jeffc Exp $
____________________________________________________________________________*/
#include <math.h>
#include <string.h>

#include "pgpMem.h"

#include "pgpPassphraseUtils.h"

#if defined(PGP_WIN32) || defined(PGP_OSX)
#include "pgpUnicode.h"
#endif

#define OUT_OF_RANGE_LENGTH 1	/* length of the unrecognized character interval (could be less conservative) */

#define SEP /*,*/

/* Group IDs with unique Language and Category */

#define Pd_Armenian /* 0 */
#define Nd_Gurmukhi /* 1 */
#define Lm_Katakana /* 2 */
#define Mn_Buhid /* 3 */
#define Ll_LatinExtended_A /* 4 */
#define Ll_LatinExtended_B /* 5 */
#define Lu_Deseret /* 6 */
#define Lu_GreekExtended /* 7 */
#define Mn_Mongolian /* 8 */
#define Mn_Syriac /* 9 */
#define So_Kanbun /* 10 */
#define Lo_Khmer /* 11 */
#define Po_GreekandCoptic /* 12 */
#define Zl_GeneralPunctuation /* 13 */
#define Ll_CyrillicSupplementary /* 14 */
#define No_SuperscriptsandSubscripts /* 15 */
#define Lo_KatakanaPhoneticExtensions /* 16 */
#define Pd_GeneralPunctuation /* 17 */
#define Lo_Ethiopic /* 18 */
#define So_Specials /* 19 */
#define Pc_CJKCompatibilityForms /* 20 */
#define So_ControlPictures /* 21 */
#define Mn_Bengali /* 22 */
#define So_YiRadicals /* 23 */
#define Po_Tibetan /* 24 */
#define Ll_Latin_1Supplement /* 25 */
#define Lm_Lao /* 26 */
#define No_Latin_1Supplement /* 27 */
#define So_KangxiRadicals /* 28 */
#define Po_Hanunoo /* 29 */
#define No_EnclosedAlphanumerics /* 30 */
#define Lu_GreekandCoptic /* 31 */
#define Mn_VariationSelectors /* 32 */
#define Po_Khmer /* 33 */
#define Pd_HalfwidthandFullwidthForms /* 34 */
#define Nl_NumberForms /* 35 */
#define Lm_CJKSymbolsandPunctuation /* 36 */
#define Lo_Devanagari /* 37 */
#define Lo_Myanmar /* 38 */
#define Po_GeneralPunctuation /* 39 */
#define Sc_Bengali /* 40 */
#define Nl_Gothic /* 41 */
#define Mc_MusicalSymbols /* 42 */
#define Lu_Georgian /* 43 */
#define Mc_Bengali /* 44 */
#define Lm_Arabic /* 45 */
#define Lo_Georgian /* 46 */
#define Lo_Tagbanwa /* 47 */
#define Lu_LetterlikeSymbols /* 48 */
#define Lu_LatinExtendedAdditional /* 49 */
#define Ps_Tibetan /* 50 */
#define Po_Armenian /* 51 */
#define Pc_GeneralPunctuation /* 52 */
#define Zs_CJKSymbolsandPunctuation /* 53 */
#define Lo_YiSyllables /* 54 */
#define So_BraillePatterns /* 55 */
#define Lo_Tagalog /* 56 */
#define Mn_Arabic /* 57 */
#define Lo_BopomofoExtended /* 58 */
#define Sk_HalfwidthandFullwidthForms /* 59 */
#define Pe_CJKSymbolsandPunctuation /* 60 */
#define No_Bengali /* 61 */
#define Mc_Tamil /* 62 */
#define No_EnclosedCJKLettersandMonths /* 63 */
#define Mn_Hiragana /* 64 */
#define No_OldItalic /* 65 */
#define Ps_Dingbats /* 66 */
#define So_LetterlikeSymbols /* 67 */
#define Sm_Latin_1Supplement /* 68 */
#define Mc_Gujarati /* 69 */
#define So_ByzantineMusicalSymbols /* 70 */
#define Ll_AlphabeticPresentationForms /* 71 */
#define Mn_CombiningDiacriticalMarksforSymbols /* 72 */
#define Lo_Sinhala /* 73 */
#define Sm_GeometricShapes /* 74 */
#define Cf_Syriac /* 75 */
#define Nd_Bengali /* 76 */
#define Sc_BasicLatin /* 77 */
#define Nd_Malayalam /* 78 */
#define Lo_Gurmukhi /* 79 */
#define Mc_Devanagari /* 80 */
#define Cf_Specials /* 81 */
#define So_Dingbats /* 82 */
#define Lo_Telugu /* 83 */
#define Cs_HighSurrogates /* 84 */
#define Ll_Cyrillic /* 85 */
#define Pe_SmallFormVariants /* 86 */
#define Cs_LowSurrogates /* 87 */
#define Nd_HalfwidthandFullwidthForms /* 88 */
#define Mn_CombiningHalfMarks /* 89 */
#define Ps_CJKSymbolsandPunctuation /* 90 */
#define Lo_Cherokee /* 91 */
#define Ll /* 92 */
#define Ll_BasicLatin /* 93 */
#define Nd_Lao /* 94 */
#define Nd_Tamil /* 95 */
#define Lo /* 96 */
#define Me_Arabic /* 97 */
#define Lt_LatinExtended_B /* 98 */
#define Nd_Devanagari /* 99 */
#define So_GeometricShapes /* 100 */
#define So_Oriya /* 101 */
#define Sc_Thai /* 102 */
#define Lu_Armenian /* 103 */
#define Lo_Hebrew /* 104 */
#define Ll_GreekandCoptic /* 105 */
#define Lu_MathematicalAlphanumericSymbols /* 106 */
#define Ps_HalfwidthandFullwidthForms /* 107 */
#define Po_BasicLatin /* 108 */
#define Mn_Tibetan /* 109 */
#define Ps_Ogham /* 110 */
#define Mc_Telugu /* 111 */
#define Mn_Hanunoo /* 112 */
#define Lo_Tamil /* 113 */
#define Mn_AlphabeticPresentationForms /* 114 */
#define Sc_CurrencySymbols /* 115 */
#define So_Bengali /* 116 */
#define Lo_Mongolian /* 117 */
#define Sm_MiscellaneousTechnical /* 118 */
#define Mn_Kannada /* 119 */
#define Lo_Lao /* 120 */
#define Nd_Telugu /* 121 */
#define Pd_Katakana /* 122 */
#define Po_CJKCompatibilityForms /* 123 */
#define Mn_Gujarati /* 124 */
#define Mn_Cyrillic /* 125 */
#define Cf_Arabic /* 126 */
#define Mn_Oriya /* 127 */
#define Sm_MiscellaneousMathematicalSymbols_A /* 128 */
#define Mc_Tibetan /* 129 */
#define Sm_MiscellaneousMathematicalSymbols_B /* 130 */
#define Sm_AlphabeticPresentationForms /* 131 */
#define Ps_SuperscriptsandSubscripts /* 132 */
#define Po_Hebrew /* 133 */
#define Lm_Khmer /* 134 */
#define Pf_Latin_1Supplement /* 135 */
#define Cf_MusicalSymbols /* 136 */
#define Ps_SmallFormVariants /* 137 */
#define So_MiscellaneousSymbols /* 138 */
#define Pe_Tibetan /* 139 */
#define Mc_Kannada /* 140 */
#define Lo_Thai /* 141 */
#define Lo_Bopomofo /* 142 */
#define So_EnclosedCJKLettersandMonths /* 143 */
#define Lo_CJKUnifiedIdeographs /* 144 */
#define Lo_HangulJamo /* 145 */
#define No_Tibetan /* 146 */
#define Lm_Mongolian /* 147 */
#define Sk_BasicLatin /* 148 */
#define Sm_MiscellaneousSymbols /* 149 */
#define Co_SupplementaryPrivateUseArea_A /* 150 */
#define Co_SupplementaryPrivateUseArea_B /* 151 */
#define Pc_HalfwidthandFullwidthForms /* 152 */
#define Lt_GreekExtended /* 153 */
#define Sk_Hiragana /* 154 */
#define Nl_Runic /* 155 */
#define Zs_GeneralPunctuation /* 156 */
#define Po_Myanmar /* 157 */
#define Lo_CJKCompatibilityIdeographsSupplement /* 158 */
#define Nd_Tibetan /* 159 */
#define Mn_CJKSymbolsandPunctuation /* 160 */
#define Nd_MathematicalAlphanumericSymbols /* 161 */
#define Sm_SuperscriptsandSubscripts /* 162 */
#define Mn_Khmer /* 163 */
#define Co_PrivateUseArea /* 164 */
#define Sc_Khmer /* 165 */
#define Mn_CombiningDiacriticalMarks /* 166 */
#define Lo_Thaana /* 167 */
#define Po_SmallFormVariants /* 168 */
#define Nd_Kannada /* 169 */
#define Nd_Gujarati /* 170 */
#define Lm_Thai /* 171 */
#define Lm_HalfwidthandFullwidthForms /* 172 */
#define Lo_Bengali /* 173 */
#define Pe_SuperscriptsandSubscripts /* 174 */
#define Lo_AlphabeticPresentationForms /* 175 */
#define Sk_GreekandCoptic /* 176 */
#define Lo_HangulSyllables /* 177 */
#define Pd_SmallFormVariants /* 178 */
#define Pc_Katakana /* 179 */
#define So_Arrows /* 180 */
#define Lo_Syriac /* 181 */
#define Sm_HalfwidthandFullwidthForms /* 182 */
#define Pc_BasicLatin /* 183 */
#define Po_MiscellaneousTechnical /* 184 */
#define So_Arabic /* 185 */
#define Sc_SmallFormVariants /* 186 */
#define So_CJKCompatibility /* 187 */
#define Pe_Dingbats /* 188 */
#define Po_Sinhala /* 189 */
#define Lo_Hiragana /* 190 */
#define Pi_Latin_1Supplement /* 191 */
#define Pe_MiscellaneousTechnical /* 192 */
#define Lo_Buhid /* 193 */
#define So_EnclosedAlphanumerics /* 194 */
#define So_Tibetan /* 195 */
#define Sm_GreekandCoptic /* 196 */
#define Nd_Mongolian /* 197 */
#define Po_HalfwidthandFullwidthForms /* 198 */
#define Po_Syriac /* 199 */
#define Lu_Latin_1Supplement /* 200 */
#define Lu_BasicLatin /* 201 */
#define Ll_IPAExtensions /* 202 */
#define Ll_Deseret /* 203 */
#define Nd_Thai /* 204 */
#define Po_UnifiedCanadianAboriginalSyllabics /* 205 */
#define Lo_LatinExtended_B /* 206 */
#define Ll_LetterlikeSymbols /* 207 */
#define Mc_Malayalam /* 208 */
#define So_OpticalCharacterRecognition /* 209 */
#define Pi_GeneralPunctuation /* 210 */
#define So_IdeographicDescriptionCharacters /* 211 */
#define Cc /* 212 */
#define Lm_Armenian /* 213 */
#define So_Latin_1Supplement /* 214 */
#define Ps_ArabicPresentationForms_A /* 215 */
#define Lo_Arabic /* 216 */
#define Mn_Lao /* 217 */
#define Lo_Katakana /* 218 */
#define Lo_CJKUnifiedIdeographsExtensionA /* 219 */
#define Lo_CJKUnifiedIdeographsExtensionB /* 220 */
#define Mn_Malayalam /* 221 */
#define Cs /* 222 */
#define Pd_BasicLatin /* 223 */
#define Mn_Myanmar /* 224 */
#define So_CJKRadicalsSupplement /* 225 */
#define No_Kanbun /* 226 */
#define Sm_MathematicalAlphanumericSymbols /* 227 */
#define Zp_GeneralPunctuation /* 228 */
#define Ll_GreekExtended /* 229 */
#define So_MusicalSymbols /* 230 */
#define Mc_Gurmukhi /* 231 */
#define Po_Thai /* 232 */
#define Pe_MiscellaneousMathematicalSymbols_A /* 233 */
#define Pe_MiscellaneousMathematicalSymbols_B /* 234 */
#define Sm_BasicLatin /* 235 */
#define Po_Devanagari /* 236 */
#define Sk_SpacingModifierLetters /* 237 */
#define Lm_SpacingModifierLetters /* 238 */
#define Ll_SuperscriptsandSubscripts /* 239 */
#define Ps_MiscellaneousTechnical /* 240 */
#define Sc_HalfwidthandFullwidthForms /* 241 */
#define Lu_Cyrillic /* 242 */
#define Mn_Tagalog /* 243 */
#define Ll_MathematicalAlphanumericSymbols /* 244 */
#define Sm /* 245 */
#define Nl_CJKSymbolsandPunctuation /* 246 */
#define Po_Arabic /* 247 */
#define Lo_Gujarati /* 248 */
#define So /* 249 */
#define Cf_GeneralPunctuation /* 250 */
#define Mn_Telugu /* 251 */
#define Mc_Myanmar /* 252 */
#define Sm_LetterlikeSymbols /* 253 */
#define Lo_Tibetan /* 254 */
#define Lo_HangulCompatibilityJamo /* 255 */
#define Sk_Latin_1Supplement /* 256 */
#define Pe_Ogham /* 257 */
#define Lo_Hanunoo /* 258 */
#define Mn_Tamil /* 259 */
#define Lu_CyrillicSupplementary /* 260 */
#define Mn_Tagbanwa /* 261 */
#define Sm_SupplementalArrows_A /* 262 */
#define So_CJKSymbolsandPunctuation /* 263 */
#define Mc_Oriya /* 264 */
#define Nd_Arabic /* 265 */
#define Ps_GeneralPunctuation /* 266 */
#define Mn_Thai /* 267 */
#define Lo_Kannada /* 268 */
#define Mn_Sinhala /* 269 */
#define Sm_SmallFormVariants /* 270 */
#define Pe_CJKCompatibilityForms /* 271 */
#define Ps_CJKCompatibilityForms /* 272 */
#define Pe_ArabicPresentationForms_A /* 273 */
#define Sm_GeneralPunctuation /* 274 */
#define Pe_HalfwidthandFullwidthForms /* 275 */
#define Lo_LetterlikeSymbols /* 276 */
#define Po_CJKSymbolsandPunctuation /* 277 */
#define Pd_CJKSymbolsandPunctuation /* 278 */
#define Mn_Hebrew /* 279 */
#define Lu_LatinExtended_A /* 280 */
#define No_NumberForms /* 281 */
#define Ll_Armenian /* 282 */
#define Lu_LatinExtended_B /* 283 */
#define Cc_BasicLatin /* 284 */
#define Ll_HalfwidthandFullwidthForms /* 285 */
#define Lo_UnifiedCanadianAboriginalSyllabics /* 286 */
#define Pe_BasicLatin /* 287 */
#define Sk_GreekExtended /* 288 */
#define Nd_Ethiopic /* 289 */
#define Mc_Sinhala /* 290 */
#define Nd_Myanmar /* 291 */
#define Lm_GreekandCoptic /* 292 */
#define Lo_CJKSymbolsandPunctuation /* 293 */
#define Lo_CJKCompatibilityIdeographs /* 294 */
#define Sc_ArabicPresentationForms_A /* 295 */
#define No_Tamil /* 296 */
#define Mn_Gurmukhi /* 297 */
#define Lo_HalfwidthandFullwidthForms /* 298 */
#define Nd_Oriya /* 299 */
#define Mn_Devanagari /* 300 */
#define Cf_Mongolian /* 301 */
#define Me_Cyrillic /* 302 */
#define Nd_BasicLatin /* 303 */
#define Pd_Mongolian /* 304 */
#define Lo_Runic /* 305 */
#define Lo_OldItalic /* 306 */
#define So_HalfwidthandFullwidthForms /* 307 */
#define Mc_Khmer /* 308 */
#define Lo_Gothic /* 309 */
#define Lm_Hiragana /* 310 */
#define Pf_GeneralPunctuation /* 311 */
#define Po_Ethiopic /* 312 */
#define Ps_BasicLatin /* 313 */
#define Po_Latin_1Supplement /* 314 */
#define Zs_BasicLatin /* 315 */
#define Lo_Oriya /* 316 */
#define Pd_CJKCompatibilityForms /* 317 */
#define Po_Mongolian /* 318 */
#define So_MiscellaneousTechnical /* 319 */
#define Pd_Latin_1Supplement /* 320 */
#define Me_CombiningDiacriticalMarksforSymbols /* 321 */
#define No_Dingbats /* 322 */
#define Lu_HalfwidthandFullwidthForms /* 323 */
#define Cf_ArabicPresentationForms_B /* 324 */
#define Zs_Latin_1Supplement /* 325 */
#define Po_Runic /* 326 */
#define Ps_MiscellaneousMathematicalSymbols_A /* 327 */
#define So_Cyrillic /* 328 */
#define Lo_Ogham /* 329 */
#define Ps_MiscellaneousMathematicalSymbols_B /* 330 */
#define Sc_Latin_1Supplement /* 331 */
#define Ll_LatinExtendedAdditional /* 332 */
#define Mn_MusicalSymbols /* 333 */
#define Po_Georgian /* 334 */
#define Zs_Ogham /* 335 */
#define Lo_ArabicPresentationForms_A /* 336 */
#define Lo_ArabicPresentationForms_B /* 337 */
#define Nd_Khmer /* 338 */
#define Pe_GeneralPunctuation /* 339 */
#define Lo_Malayalam /* 340 */
#define No_Ethiopic /* 341 */
#define Cf_Tags /* 342 */
#define Mn_Thaana /* 343 */
#define Sm_Arrows /* 344 */

typedef struct pgpUnicodeRange_ {
		PGPUInt32 base;
		PGPUInt16 length;
		PGPUInt16 total_length;
#if 0
		PGPUInt32 type;
#endif
} pgpUnicodeRange;

static const pgpUnicodeRange sUnicodeSpace[] = {
   { 0x000000, 32, 32 SEP Cc_BasicLatin },
   { 0x000020, 1, 1 SEP Zs_BasicLatin },
   { 0x000021, 3, 15 SEP Po_BasicLatin },
   { 0x000024, 1, 1 SEP Sc_BasicLatin },
   { 0x000025, 3, 15 SEP Po_BasicLatin },
   { 0x000028, 1, 3 SEP Ps_BasicLatin },
   { 0x000029, 1, 3 SEP Pe_BasicLatin },
   { 0x00002a, 1, 15 SEP Po_BasicLatin },
   { 0x00002b, 1, 6 SEP Sm_BasicLatin },
   { 0x00002c, 1, 15 SEP Po_BasicLatin },
   { 0x00002d, 1, 1 SEP Pd_BasicLatin },
   { 0x00002e, 2, 15 SEP Po_BasicLatin },
   { 0x000030, 10, 10 SEP Nd_BasicLatin },
   { 0x00003a, 2, 15 SEP Po_BasicLatin },
   { 0x00003c, 3, 6 SEP Sm_BasicLatin },
   { 0x00003f, 2, 15 SEP Po_BasicLatin },
   { 0x000041, 26, 26 SEP Lu_BasicLatin },
   { 0x00005b, 1, 3 SEP Ps_BasicLatin },
   { 0x00005c, 1, 15 SEP Po_BasicLatin },
   { 0x00005d, 1, 3 SEP Pe_BasicLatin },
   { 0x00005e, 1, 2 SEP Sk_BasicLatin },
   { 0x00005f, 1, 1 SEP Pc_BasicLatin },
   { 0x000060, 1, 2 SEP Sk_BasicLatin },
   { 0x000061, 26, 26 SEP Ll_BasicLatin },
   { 0x00007b, 1, 3 SEP Ps_BasicLatin },
   { 0x00007c, 1, 6 SEP Sm_BasicLatin },
   { 0x00007d, 1, 3 SEP Pe_BasicLatin },
   { 0x00007e, 1, 6 SEP Sm_BasicLatin },
   { 0x00007f, 33, 33 SEP Cc },
   { 0x0000a0, 1, 1 SEP Zs_Latin_1Supplement },
   { 0x0000a1, 1, 3 SEP Po_Latin_1Supplement },
   { 0x0000a2, 4, 4 SEP Sc_Latin_1Supplement },
   { 0x0000a6, 2, 6 SEP So_Latin_1Supplement },
   { 0x0000a8, 1, 4 SEP Sk_Latin_1Supplement },
   { 0x0000a9, 1, 6 SEP So_Latin_1Supplement },
   { 0x0000aa, 1, 35 SEP Ll_Latin_1Supplement },
   { 0x0000ab, 1, 1 SEP Pi_Latin_1Supplement },
   { 0x0000ac, 1, 4 SEP Sm_Latin_1Supplement },
   { 0x0000ad, 1, 1 SEP Pd_Latin_1Supplement },
   { 0x0000ae, 1, 6 SEP So_Latin_1Supplement },
   { 0x0000af, 1, 4 SEP Sk_Latin_1Supplement },
   { 0x0000b0, 1, 6 SEP So_Latin_1Supplement },
   { 0x0000b1, 1, 4 SEP Sm_Latin_1Supplement },
   { 0x0000b2, 2, 6 SEP No_Latin_1Supplement },
   { 0x0000b4, 1, 4 SEP Sk_Latin_1Supplement },
   { 0x0000b5, 1, 35 SEP Ll_Latin_1Supplement },
   { 0x0000b6, 1, 6 SEP So_Latin_1Supplement },
   { 0x0000b7, 1, 3 SEP Po_Latin_1Supplement },
   { 0x0000b8, 1, 4 SEP Sk_Latin_1Supplement },
   { 0x0000b9, 1, 6 SEP No_Latin_1Supplement },
   { 0x0000ba, 1, 35 SEP Ll_Latin_1Supplement },
   { 0x0000bb, 1, 1 SEP Pf_Latin_1Supplement },
   { 0x0000bc, 3, 6 SEP No_Latin_1Supplement },
   { 0x0000bf, 1, 3 SEP Po_Latin_1Supplement },
   { 0x0000c0, 23, 30 SEP Lu_Latin_1Supplement },
   { 0x0000d7, 1, 4 SEP Sm_Latin_1Supplement },
   { 0x0000d8, 7, 30 SEP Lu_Latin_1Supplement },
   { 0x0000df, 24, 35 SEP Ll_Latin_1Supplement },
   { 0x0000f7, 1, 4 SEP Sm_Latin_1Supplement },
   { 0x0000f8, 8, 35 SEP Ll_Latin_1Supplement },
   { 0x000100, 1, 63 SEP Lu_LatinExtended_A },
   { 0x000101, 1, 63 SEP Ll_LatinExtended_A },
   { 0x000102, 1, 63 SEP Lu_LatinExtended_A },
   { 0x000103, 1, 63 SEP Ll_LatinExtended_A },
   { 0x000104, 1, 63 SEP Lu_LatinExtended_A },
   { 0x000105, 1, 63 SEP Ll_LatinExtended_A },
   { 0x000106, 1, 63 SEP Lu_LatinExtended_A },
   { 0x000107, 1, 63 SEP Ll_LatinExtended_A },
   { 0x000108, 1, 63 SEP Lu_LatinExtended_A },
   { 0x000109, 1, 63 SEP Ll_LatinExtended_A },
   { 0x00010a, 1, 63 SEP Lu_LatinExtended_A },
   { 0x00010b, 1, 63 SEP Ll_LatinExtended_A },
   { 0x00010c, 1, 63 SEP Lu_LatinExtended_A },
   { 0x00010d, 1, 63 SEP Ll_LatinExtended_A },
   { 0x00010e, 1, 63 SEP Lu_LatinExtended_A },
   { 0x00010f, 1, 63 SEP Ll_LatinExtended_A },
   { 0x000110, 1, 63 SEP Lu_LatinExtended_A },
   { 0x000111, 1, 63 SEP Ll_LatinExtended_A },
   { 0x000112, 1, 63 SEP Lu_LatinExtended_A },
   { 0x000113, 1, 63 SEP Ll_LatinExtended_A },
   { 0x000114, 1, 63 SEP Lu_LatinExtended_A },
   { 0x000115, 1, 63 SEP Ll_LatinExtended_A },
   { 0x000116, 1, 63 SEP Lu_LatinExtended_A },
   { 0x000117, 1, 63 SEP Ll_LatinExtended_A },
   { 0x000118, 1, 63 SEP Lu_LatinExtended_A },
   { 0x000119, 1, 63 SEP Ll_LatinExtended_A },
   { 0x00011a, 1, 63 SEP Lu_LatinExtended_A },
   { 0x00011b, 1, 63 SEP Ll_LatinExtended_A },
   { 0x00011c, 1, 63 SEP Lu_LatinExtended_A },
   { 0x00011d, 1, 63 SEP Ll_LatinExtended_A },
   { 0x00011e, 1, 63 SEP Lu_LatinExtended_A },
   { 0x00011f, 1, 63 SEP Ll_LatinExtended_A },
   { 0x000120, 1, 63 SEP Lu_LatinExtended_A },
   { 0x000121, 1, 63 SEP Ll_LatinExtended_A },
   { 0x000122, 1, 63 SEP Lu_LatinExtended_A },
   { 0x000123, 1, 63 SEP Ll_LatinExtended_A },
   { 0x000124, 1, 63 SEP Lu_LatinExtended_A },
   { 0x000125, 1, 63 SEP Ll_LatinExtended_A },
   { 0x000126, 1, 63 SEP Lu_LatinExtended_A },
   { 0x000127, 1, 63 SEP Ll_LatinExtended_A },
   { 0x000128, 1, 63 SEP Lu_LatinExtended_A },
   { 0x000129, 1, 63 SEP Ll_LatinExtended_A },
   { 0x00012a, 1, 63 SEP Lu_LatinExtended_A },
   { 0x00012b, 1, 63 SEP Ll_LatinExtended_A },
   { 0x00012c, 1, 63 SEP Lu_LatinExtended_A },
   { 0x00012d, 1, 63 SEP Ll_LatinExtended_A },
   { 0x00012e, 1, 63 SEP Lu_LatinExtended_A },
   { 0x00012f, 1, 63 SEP Ll_LatinExtended_A },
   { 0x000130, 1, 63 SEP Lu_LatinExtended_A },
   { 0x000131, 1, 63 SEP Ll_LatinExtended_A },
   { 0x000132, 1, 63 SEP Lu_LatinExtended_A },
   { 0x000133, 1, 63 SEP Ll_LatinExtended_A },
   { 0x000134, 1, 63 SEP Lu_LatinExtended_A },
   { 0x000135, 1, 63 SEP Ll_LatinExtended_A },
   { 0x000136, 1, 63 SEP Lu_LatinExtended_A },
   { 0x000137, 2, 63 SEP Ll_LatinExtended_A },
   { 0x000139, 1, 63 SEP Lu_LatinExtended_A },
   { 0x00013a, 1, 63 SEP Ll_LatinExtended_A },
   { 0x00013b, 1, 63 SEP Lu_LatinExtended_A },
   { 0x00013c, 1, 63 SEP Ll_LatinExtended_A },
   { 0x00013d, 1, 63 SEP Lu_LatinExtended_A },
   { 0x00013e, 1, 63 SEP Ll_LatinExtended_A },
   { 0x00013f, 1, 63 SEP Lu_LatinExtended_A },
   { 0x000140, 1, 63 SEP Ll_LatinExtended_A },
   { 0x000141, 1, 63 SEP Lu_LatinExtended_A },
   { 0x000142, 1, 63 SEP Ll_LatinExtended_A },
   { 0x000143, 1, 63 SEP Lu_LatinExtended_A },
   { 0x000144, 1, 63 SEP Ll_LatinExtended_A },
   { 0x000145, 1, 63 SEP Lu_LatinExtended_A },
   { 0x000146, 1, 63 SEP Ll_LatinExtended_A },
   { 0x000147, 1, 63 SEP Lu_LatinExtended_A },
   { 0x000148, 2, 63 SEP Ll_LatinExtended_A },
   { 0x00014a, 1, 63 SEP Lu_LatinExtended_A },
   { 0x00014b, 1, 63 SEP Ll_LatinExtended_A },
   { 0x00014c, 1, 63 SEP Lu_LatinExtended_A },
   { 0x00014d, 1, 63 SEP Ll_LatinExtended_A },
   { 0x00014e, 1, 63 SEP Lu_LatinExtended_A },
   { 0x00014f, 1, 63 SEP Ll_LatinExtended_A },
   { 0x000150, 1, 63 SEP Lu_LatinExtended_A },
   { 0x000151, 1, 63 SEP Ll_LatinExtended_A },
   { 0x000152, 1, 63 SEP Lu_LatinExtended_A },
   { 0x000153, 1, 63 SEP Ll_LatinExtended_A },
   { 0x000154, 1, 63 SEP Lu_LatinExtended_A },
   { 0x000155, 1, 63 SEP Ll_LatinExtended_A },
   { 0x000156, 1, 63 SEP Lu_LatinExtended_A },
   { 0x000157, 1, 63 SEP Ll_LatinExtended_A },
   { 0x000158, 1, 63 SEP Lu_LatinExtended_A },
   { 0x000159, 1, 63 SEP Ll_LatinExtended_A },
   { 0x00015a, 1, 63 SEP Lu_LatinExtended_A },
   { 0x00015b, 1, 63 SEP Ll_LatinExtended_A },
   { 0x00015c, 1, 63 SEP Lu_LatinExtended_A },
   { 0x00015d, 1, 63 SEP Ll_LatinExtended_A },
   { 0x00015e, 1, 63 SEP Lu_LatinExtended_A },
   { 0x00015f, 1, 63 SEP Ll_LatinExtended_A },
   { 0x000160, 1, 63 SEP Lu_LatinExtended_A },
   { 0x000161, 1, 63 SEP Ll_LatinExtended_A },
   { 0x000162, 1, 63 SEP Lu_LatinExtended_A },
   { 0x000163, 1, 63 SEP Ll_LatinExtended_A },
   { 0x000164, 1, 63 SEP Lu_LatinExtended_A },
   { 0x000165, 1, 63 SEP Ll_LatinExtended_A },
   { 0x000166, 1, 63 SEP Lu_LatinExtended_A },
   { 0x000167, 1, 63 SEP Ll_LatinExtended_A },
   { 0x000168, 1, 63 SEP Lu_LatinExtended_A },
   { 0x000169, 1, 63 SEP Ll_LatinExtended_A },
   { 0x00016a, 1, 63 SEP Lu_LatinExtended_A },
   { 0x00016b, 1, 63 SEP Ll_LatinExtended_A },
   { 0x00016c, 1, 63 SEP Lu_LatinExtended_A },
   { 0x00016d, 1, 63 SEP Ll_LatinExtended_A },
   { 0x00016e, 1, 63 SEP Lu_LatinExtended_A },
   { 0x00016f, 1, 63 SEP Ll_LatinExtended_A },
   { 0x000170, 1, 63 SEP Lu_LatinExtended_A },
   { 0x000171, 1, 63 SEP Ll_LatinExtended_A },
   { 0x000172, 1, 63 SEP Lu_LatinExtended_A },
   { 0x000173, 1, 63 SEP Ll_LatinExtended_A },
   { 0x000174, 1, 63 SEP Lu_LatinExtended_A },
   { 0x000175, 1, 63 SEP Ll_LatinExtended_A },
   { 0x000176, 1, 63 SEP Lu_LatinExtended_A },
   { 0x000177, 1, 63 SEP Ll_LatinExtended_A },
   { 0x000178, 2, 63 SEP Lu_LatinExtended_A },
   { 0x00017a, 1, 63 SEP Ll_LatinExtended_A },
   { 0x00017b, 1, 63 SEP Lu_LatinExtended_A },
   { 0x00017c, 1, 63 SEP Ll_LatinExtended_A },
   { 0x00017d, 1, 63 SEP Lu_LatinExtended_A },
   { 0x00017e, 3, 3 SEP Ll },
   { 0x000181, 2, 90 SEP Lu_LatinExtended_B },
   { 0x000183, 1, 79 SEP Ll_LatinExtended_B },
   { 0x000184, 1, 90 SEP Lu_LatinExtended_B },
   { 0x000185, 1, 79 SEP Ll_LatinExtended_B },
   { 0x000186, 2, 90 SEP Lu_LatinExtended_B },
   { 0x000188, 1, 79 SEP Ll_LatinExtended_B },
   { 0x000189, 3, 90 SEP Lu_LatinExtended_B },
   { 0x00018c, 2, 79 SEP Ll_LatinExtended_B },
   { 0x00018e, 4, 90 SEP Lu_LatinExtended_B },
   { 0x000192, 1, 79 SEP Ll_LatinExtended_B },
   { 0x000193, 2, 90 SEP Lu_LatinExtended_B },
   { 0x000195, 1, 79 SEP Ll_LatinExtended_B },
   { 0x000196, 3, 90 SEP Lu_LatinExtended_B },
   { 0x000199, 3, 79 SEP Ll_LatinExtended_B },
   { 0x00019c, 2, 90 SEP Lu_LatinExtended_B },
   { 0x00019e, 1, 79 SEP Ll_LatinExtended_B },
   { 0x00019f, 2, 90 SEP Lu_LatinExtended_B },
   { 0x0001a1, 1, 79 SEP Ll_LatinExtended_B },
   { 0x0001a2, 1, 90 SEP Lu_LatinExtended_B },
   { 0x0001a3, 1, 79 SEP Ll_LatinExtended_B },
   { 0x0001a4, 1, 90 SEP Lu_LatinExtended_B },
   { 0x0001a5, 1, 79 SEP Ll_LatinExtended_B },
   { 0x0001a6, 2, 90 SEP Lu_LatinExtended_B },
   { 0x0001a8, 1, 79 SEP Ll_LatinExtended_B },
   { 0x0001a9, 1, 90 SEP Lu_LatinExtended_B },
   { 0x0001aa, 2, 79 SEP Ll_LatinExtended_B },
   { 0x0001ac, 1, 90 SEP Lu_LatinExtended_B },
   { 0x0001ad, 1, 79 SEP Ll_LatinExtended_B },
   { 0x0001ae, 2, 90 SEP Lu_LatinExtended_B },
   { 0x0001b0, 1, 79 SEP Ll_LatinExtended_B },
   { 0x0001b1, 3, 90 SEP Lu_LatinExtended_B },
   { 0x0001b4, 1, 79 SEP Ll_LatinExtended_B },
   { 0x0001b5, 1, 90 SEP Lu_LatinExtended_B },
   { 0x0001b6, 1, 79 SEP Ll_LatinExtended_B },
   { 0x0001b7, 2, 90 SEP Lu_LatinExtended_B },
   { 0x0001b9, 2, 79 SEP Ll_LatinExtended_B },
   { 0x0001bb, 1, 5 SEP Lo_LatinExtended_B },
   { 0x0001bc, 1, 90 SEP Lu_LatinExtended_B },
   { 0x0001bd, 3, 79 SEP Ll_LatinExtended_B },
   { 0x0001c0, 4, 5 SEP Lo_LatinExtended_B },
   { 0x0001c4, 1, 90 SEP Lu_LatinExtended_B },
   { 0x0001c5, 1, 4 SEP Lt_LatinExtended_B },
   { 0x0001c6, 1, 79 SEP Ll_LatinExtended_B },
   { 0x0001c7, 1, 90 SEP Lu_LatinExtended_B },
   { 0x0001c8, 1, 4 SEP Lt_LatinExtended_B },
   { 0x0001c9, 1, 79 SEP Ll_LatinExtended_B },
   { 0x0001ca, 1, 90 SEP Lu_LatinExtended_B },
   { 0x0001cb, 1, 4 SEP Lt_LatinExtended_B },
   { 0x0001cc, 1, 79 SEP Ll_LatinExtended_B },
   { 0x0001cd, 1, 90 SEP Lu_LatinExtended_B },
   { 0x0001ce, 1, 79 SEP Ll_LatinExtended_B },
   { 0x0001cf, 1, 90 SEP Lu_LatinExtended_B },
   { 0x0001d0, 1, 79 SEP Ll_LatinExtended_B },
   { 0x0001d1, 1, 90 SEP Lu_LatinExtended_B },
   { 0x0001d2, 1, 79 SEP Ll_LatinExtended_B },
   { 0x0001d3, 1, 90 SEP Lu_LatinExtended_B },
   { 0x0001d4, 1, 79 SEP Ll_LatinExtended_B },
   { 0x0001d5, 1, 90 SEP Lu_LatinExtended_B },
   { 0x0001d6, 1, 79 SEP Ll_LatinExtended_B },
   { 0x0001d7, 1, 90 SEP Lu_LatinExtended_B },
   { 0x0001d8, 1, 79 SEP Ll_LatinExtended_B },
   { 0x0001d9, 1, 90 SEP Lu_LatinExtended_B },
   { 0x0001da, 1, 79 SEP Ll_LatinExtended_B },
   { 0x0001db, 1, 90 SEP Lu_LatinExtended_B },
   { 0x0001dc, 2, 79 SEP Ll_LatinExtended_B },
   { 0x0001de, 1, 90 SEP Lu_LatinExtended_B },
   { 0x0001df, 1, 79 SEP Ll_LatinExtended_B },
   { 0x0001e0, 1, 90 SEP Lu_LatinExtended_B },
   { 0x0001e1, 1, 79 SEP Ll_LatinExtended_B },
   { 0x0001e2, 1, 90 SEP Lu_LatinExtended_B },
   { 0x0001e3, 1, 79 SEP Ll_LatinExtended_B },
   { 0x0001e4, 1, 90 SEP Lu_LatinExtended_B },
   { 0x0001e5, 1, 79 SEP Ll_LatinExtended_B },
   { 0x0001e6, 1, 90 SEP Lu_LatinExtended_B },
   { 0x0001e7, 1, 79 SEP Ll_LatinExtended_B },
   { 0x0001e8, 1, 90 SEP Lu_LatinExtended_B },
   { 0x0001e9, 1, 79 SEP Ll_LatinExtended_B },
   { 0x0001ea, 1, 90 SEP Lu_LatinExtended_B },
   { 0x0001eb, 1, 79 SEP Ll_LatinExtended_B },
   { 0x0001ec, 1, 90 SEP Lu_LatinExtended_B },
   { 0x0001ed, 1, 79 SEP Ll_LatinExtended_B },
   { 0x0001ee, 1, 90 SEP Lu_LatinExtended_B },
   { 0x0001ef, 2, 79 SEP Ll_LatinExtended_B },
   { 0x0001f1, 1, 90 SEP Lu_LatinExtended_B },
   { 0x0001f2, 1, 4 SEP Lt_LatinExtended_B },
   { 0x0001f3, 1, 79 SEP Ll_LatinExtended_B },
   { 0x0001f4, 1, 90 SEP Lu_LatinExtended_B },
   { 0x0001f5, 1, 79 SEP Ll_LatinExtended_B },
   { 0x0001f6, 3, 90 SEP Lu_LatinExtended_B },
   { 0x0001f9, 1, 79 SEP Ll_LatinExtended_B },
   { 0x0001fa, 1, 90 SEP Lu_LatinExtended_B },
   { 0x0001fb, 1, 79 SEP Ll_LatinExtended_B },
   { 0x0001fc, 1, 90 SEP Lu_LatinExtended_B },
   { 0x0001fd, 1, 79 SEP Ll_LatinExtended_B },
   { 0x0001fe, 1, 90 SEP Lu_LatinExtended_B },
   { 0x0001ff, 1, 79 SEP Ll_LatinExtended_B },
   { 0x000200, 1, 90 SEP Lu_LatinExtended_B },
   { 0x000201, 1, 79 SEP Ll_LatinExtended_B },
   { 0x000202, 1, 90 SEP Lu_LatinExtended_B },
   { 0x000203, 1, 79 SEP Ll_LatinExtended_B },
   { 0x000204, 1, 90 SEP Lu_LatinExtended_B },
   { 0x000205, 1, 79 SEP Ll_LatinExtended_B },
   { 0x000206, 1, 90 SEP Lu_LatinExtended_B },
   { 0x000207, 1, 79 SEP Ll_LatinExtended_B },
   { 0x000208, 1, 90 SEP Lu_LatinExtended_B },
   { 0x000209, 1, 79 SEP Ll_LatinExtended_B },
   { 0x00020a, 1, 90 SEP Lu_LatinExtended_B },
   { 0x00020b, 1, 79 SEP Ll_LatinExtended_B },
   { 0x00020c, 1, 90 SEP Lu_LatinExtended_B },
   { 0x00020d, 1, 79 SEP Ll_LatinExtended_B },
   { 0x00020e, 1, 90 SEP Lu_LatinExtended_B },
   { 0x00020f, 1, 79 SEP Ll_LatinExtended_B },
   { 0x000210, 1, 90 SEP Lu_LatinExtended_B },
   { 0x000211, 1, 79 SEP Ll_LatinExtended_B },
   { 0x000212, 1, 90 SEP Lu_LatinExtended_B },
   { 0x000213, 1, 79 SEP Ll_LatinExtended_B },
   { 0x000214, 1, 90 SEP Lu_LatinExtended_B },
   { 0x000215, 1, 79 SEP Ll_LatinExtended_B },
   { 0x000216, 1, 90 SEP Lu_LatinExtended_B },
   { 0x000217, 1, 79 SEP Ll_LatinExtended_B },
   { 0x000218, 1, 90 SEP Lu_LatinExtended_B },
   { 0x000219, 1, 79 SEP Ll_LatinExtended_B },
   { 0x00021a, 1, 90 SEP Lu_LatinExtended_B },
   { 0x00021b, 1, 79 SEP Ll_LatinExtended_B },
   { 0x00021c, 1, 90 SEP Lu_LatinExtended_B },
   { 0x00021d, 1, 79 SEP Ll_LatinExtended_B },
   { 0x00021e, 1, 90 SEP Lu_LatinExtended_B },
   { 0x00021f, 1, 79 SEP Ll_LatinExtended_B },
   { 0x000220, 1, 90 SEP Lu_LatinExtended_B },
   { 0x000222, 1, 90 SEP Lu_LatinExtended_B },
   { 0x000223, 1, 79 SEP Ll_LatinExtended_B },
   { 0x000224, 1, 90 SEP Lu_LatinExtended_B },
   { 0x000225, 1, 79 SEP Ll_LatinExtended_B },
   { 0x000226, 1, 90 SEP Lu_LatinExtended_B },
   { 0x000227, 1, 79 SEP Ll_LatinExtended_B },
   { 0x000228, 1, 90 SEP Lu_LatinExtended_B },
   { 0x000229, 1, 79 SEP Ll_LatinExtended_B },
   { 0x00022a, 1, 90 SEP Lu_LatinExtended_B },
   { 0x00022b, 1, 79 SEP Ll_LatinExtended_B },
   { 0x00022c, 1, 90 SEP Lu_LatinExtended_B },
   { 0x00022d, 1, 79 SEP Ll_LatinExtended_B },
   { 0x00022e, 1, 90 SEP Lu_LatinExtended_B },
   { 0x00022f, 1, 79 SEP Ll_LatinExtended_B },
   { 0x000230, 1, 90 SEP Lu_LatinExtended_B },
   { 0x000231, 1, 79 SEP Ll_LatinExtended_B },
   { 0x000232, 1, 90 SEP Lu_LatinExtended_B },
   { 0x000233, 1, 79 SEP Ll_LatinExtended_B },
   { 0x000250, 94, 94 SEP Ll_IPAExtensions },
   { 0x0002b0, 9, 24 SEP Lm_SpacingModifierLetters },
   { 0x0002b9, 2, 39 SEP Sk_SpacingModifierLetters },
   { 0x0002bb, 7, 24 SEP Lm_SpacingModifierLetters },
   { 0x0002c2, 14, 39 SEP Sk_SpacingModifierLetters },
   { 0x0002d0, 2, 24 SEP Lm_SpacingModifierLetters },
   { 0x0002d2, 14, 39 SEP Sk_SpacingModifierLetters },
   { 0x0002e0, 5, 24 SEP Lm_SpacingModifierLetters },
   { 0x0002e5, 9, 39 SEP Sk_SpacingModifierLetters },
   { 0x0002ee, 1, 24 SEP Lm_SpacingModifierLetters },
   { 0x000300, 80, 96 SEP Mn_CombiningDiacriticalMarks },
   { 0x000360, 16, 96 SEP Mn_CombiningDiacriticalMarks },
   { 0x000374, 2, 4 SEP Sk_GreekandCoptic },
   { 0x00037a, 1, 1 SEP Lm_GreekandCoptic },
   { 0x00037e, 1, 2 SEP Po_GreekandCoptic },
   { 0x000384, 2, 4 SEP Sk_GreekandCoptic },
   { 0x000386, 1, 49 SEP Lu_GreekandCoptic },
   { 0x000387, 1, 2 SEP Po_GreekandCoptic },
   { 0x000388, 3, 49 SEP Lu_GreekandCoptic },
   { 0x00038c, 1, 49 SEP Lu_GreekandCoptic },
   { 0x00038e, 2, 49 SEP Lu_GreekandCoptic },
   { 0x000390, 1, 58 SEP Ll_GreekandCoptic },
   { 0x000391, 17, 49 SEP Lu_GreekandCoptic },
   { 0x0003a3, 9, 49 SEP Lu_GreekandCoptic },
   { 0x0003ac, 35, 58 SEP Ll_GreekandCoptic },
   { 0x0003d0, 2, 58 SEP Ll_GreekandCoptic },
   { 0x0003d2, 3, 49 SEP Lu_GreekandCoptic },
   { 0x0003d5, 3, 58 SEP Ll_GreekandCoptic },
   { 0x0003d8, 1, 49 SEP Lu_GreekandCoptic },
   { 0x0003d9, 1, 58 SEP Ll_GreekandCoptic },
   { 0x0003da, 1, 49 SEP Lu_GreekandCoptic },
   { 0x0003db, 1, 58 SEP Ll_GreekandCoptic },
   { 0x0003dc, 1, 49 SEP Lu_GreekandCoptic },
   { 0x0003dd, 1, 58 SEP Ll_GreekandCoptic },
   { 0x0003de, 1, 49 SEP Lu_GreekandCoptic },
   { 0x0003df, 1, 58 SEP Ll_GreekandCoptic },
   { 0x0003e0, 1, 49 SEP Lu_GreekandCoptic },
   { 0x0003e1, 1, 58 SEP Ll_GreekandCoptic },
   { 0x0003e2, 1, 49 SEP Lu_GreekandCoptic },
   { 0x0003e3, 1, 58 SEP Ll_GreekandCoptic },
   { 0x0003e4, 1, 49 SEP Lu_GreekandCoptic },
   { 0x0003e5, 1, 58 SEP Ll_GreekandCoptic },
   { 0x0003e6, 1, 49 SEP Lu_GreekandCoptic },
   { 0x0003e7, 1, 58 SEP Ll_GreekandCoptic },
   { 0x0003e8, 1, 49 SEP Lu_GreekandCoptic },
   { 0x0003e9, 1, 58 SEP Ll_GreekandCoptic },
   { 0x0003ea, 1, 49 SEP Lu_GreekandCoptic },
   { 0x0003eb, 1, 58 SEP Ll_GreekandCoptic },
   { 0x0003ec, 1, 49 SEP Lu_GreekandCoptic },
   { 0x0003ed, 1, 58 SEP Ll_GreekandCoptic },
   { 0x0003ee, 1, 49 SEP Lu_GreekandCoptic },
   { 0x0003ef, 5, 58 SEP Ll_GreekandCoptic },
   { 0x0003f4, 1, 49 SEP Lu_GreekandCoptic },
   { 0x0003f5, 1, 58 SEP Ll_GreekandCoptic },
   { 0x0003f6, 1, 1 SEP Sm_GreekandCoptic },
   { 0x000400, 48, 120 SEP Lu_Cyrillic },
   { 0x000430, 48, 119 SEP Ll_Cyrillic },
   { 0x000460, 1, 120 SEP Lu_Cyrillic },
   { 0x000461, 1, 119 SEP Ll_Cyrillic },
   { 0x000462, 1, 120 SEP Lu_Cyrillic },
   { 0x000463, 1, 119 SEP Ll_Cyrillic },
   { 0x000464, 1, 120 SEP Lu_Cyrillic },
   { 0x000465, 1, 119 SEP Ll_Cyrillic },
   { 0x000466, 1, 120 SEP Lu_Cyrillic },
   { 0x000467, 1, 119 SEP Ll_Cyrillic },
   { 0x000468, 1, 120 SEP Lu_Cyrillic },
   { 0x000469, 1, 119 SEP Ll_Cyrillic },
   { 0x00046a, 1, 120 SEP Lu_Cyrillic },
   { 0x00046b, 1, 119 SEP Ll_Cyrillic },
   { 0x00046c, 1, 120 SEP Lu_Cyrillic },
   { 0x00046d, 1, 119 SEP Ll_Cyrillic },
   { 0x00046e, 1, 120 SEP Lu_Cyrillic },
   { 0x00046f, 1, 119 SEP Ll_Cyrillic },
   { 0x000470, 1, 120 SEP Lu_Cyrillic },
   { 0x000471, 1, 119 SEP Ll_Cyrillic },
   { 0x000472, 1, 120 SEP Lu_Cyrillic },
   { 0x000473, 1, 119 SEP Ll_Cyrillic },
   { 0x000474, 1, 120 SEP Lu_Cyrillic },
   { 0x000475, 1, 119 SEP Ll_Cyrillic },
   { 0x000476, 1, 120 SEP Lu_Cyrillic },
   { 0x000477, 1, 119 SEP Ll_Cyrillic },
   { 0x000478, 1, 120 SEP Lu_Cyrillic },
   { 0x000479, 1, 119 SEP Ll_Cyrillic },
   { 0x00047a, 1, 120 SEP Lu_Cyrillic },
   { 0x00047b, 1, 119 SEP Ll_Cyrillic },
   { 0x00047c, 1, 120 SEP Lu_Cyrillic },
   { 0x00047d, 1, 119 SEP Ll_Cyrillic },
   { 0x00047e, 1, 120 SEP Lu_Cyrillic },
   { 0x00047f, 1, 119 SEP Ll_Cyrillic },
   { 0x000480, 1, 120 SEP Lu_Cyrillic },
   { 0x000481, 1, 119 SEP Ll_Cyrillic },
   { 0x000482, 1, 1 SEP So_Cyrillic },
   { 0x000483, 4, 4 SEP Mn_Cyrillic },
   { 0x000488, 2, 2 SEP Me_Cyrillic },
   { 0x00048a, 1, 120 SEP Lu_Cyrillic },
   { 0x00048b, 1, 119 SEP Ll_Cyrillic },
   { 0x00048c, 1, 120 SEP Lu_Cyrillic },
   { 0x00048d, 1, 119 SEP Ll_Cyrillic },
   { 0x00048e, 1, 120 SEP Lu_Cyrillic },
   { 0x00048f, 1, 119 SEP Ll_Cyrillic },
   { 0x000490, 1, 120 SEP Lu_Cyrillic },
   { 0x000491, 1, 119 SEP Ll_Cyrillic },
   { 0x000492, 1, 120 SEP Lu_Cyrillic },
   { 0x000493, 1, 119 SEP Ll_Cyrillic },
   { 0x000494, 1, 120 SEP Lu_Cyrillic },
   { 0x000495, 1, 119 SEP Ll_Cyrillic },
   { 0x000496, 1, 120 SEP Lu_Cyrillic },
   { 0x000497, 1, 119 SEP Ll_Cyrillic },
   { 0x000498, 1, 120 SEP Lu_Cyrillic },
   { 0x000499, 1, 119 SEP Ll_Cyrillic },
   { 0x00049a, 1, 120 SEP Lu_Cyrillic },
   { 0x00049b, 1, 119 SEP Ll_Cyrillic },
   { 0x00049c, 1, 120 SEP Lu_Cyrillic },
   { 0x00049d, 1, 119 SEP Ll_Cyrillic },
   { 0x00049e, 1, 120 SEP Lu_Cyrillic },
   { 0x00049f, 1, 119 SEP Ll_Cyrillic },
   { 0x0004a0, 1, 120 SEP Lu_Cyrillic },
   { 0x0004a1, 1, 119 SEP Ll_Cyrillic },
   { 0x0004a2, 1, 120 SEP Lu_Cyrillic },
   { 0x0004a3, 1, 119 SEP Ll_Cyrillic },
   { 0x0004a4, 1, 120 SEP Lu_Cyrillic },
   { 0x0004a5, 1, 119 SEP Ll_Cyrillic },
   { 0x0004a6, 1, 120 SEP Lu_Cyrillic },
   { 0x0004a7, 1, 119 SEP Ll_Cyrillic },
   { 0x0004a8, 1, 120 SEP Lu_Cyrillic },
   { 0x0004a9, 1, 119 SEP Ll_Cyrillic },
   { 0x0004aa, 1, 120 SEP Lu_Cyrillic },
   { 0x0004ab, 1, 119 SEP Ll_Cyrillic },
   { 0x0004ac, 1, 120 SEP Lu_Cyrillic },
   { 0x0004ad, 1, 119 SEP Ll_Cyrillic },
   { 0x0004ae, 1, 120 SEP Lu_Cyrillic },
   { 0x0004af, 1, 119 SEP Ll_Cyrillic },
   { 0x0004b0, 1, 120 SEP Lu_Cyrillic },
   { 0x0004b1, 1, 119 SEP Ll_Cyrillic },
   { 0x0004b2, 1, 120 SEP Lu_Cyrillic },
   { 0x0004b3, 1, 119 SEP Ll_Cyrillic },
   { 0x0004b4, 1, 120 SEP Lu_Cyrillic },
   { 0x0004b5, 1, 119 SEP Ll_Cyrillic },
   { 0x0004b6, 1, 120 SEP Lu_Cyrillic },
   { 0x0004b7, 1, 119 SEP Ll_Cyrillic },
   { 0x0004b8, 1, 120 SEP Lu_Cyrillic },
   { 0x0004b9, 1, 119 SEP Ll_Cyrillic },
   { 0x0004ba, 1, 120 SEP Lu_Cyrillic },
   { 0x0004bb, 1, 119 SEP Ll_Cyrillic },
   { 0x0004bc, 1, 120 SEP Lu_Cyrillic },
   { 0x0004bd, 1, 119 SEP Ll_Cyrillic },
   { 0x0004be, 1, 120 SEP Lu_Cyrillic },
   { 0x0004bf, 1, 119 SEP Ll_Cyrillic },
   { 0x0004c0, 2, 120 SEP Lu_Cyrillic },
   { 0x0004c2, 1, 119 SEP Ll_Cyrillic },
   { 0x0004c3, 1, 120 SEP Lu_Cyrillic },
   { 0x0004c4, 1, 119 SEP Ll_Cyrillic },
   { 0x0004c5, 1, 120 SEP Lu_Cyrillic },
   { 0x0004c6, 1, 119 SEP Ll_Cyrillic },
   { 0x0004c7, 1, 120 SEP Lu_Cyrillic },
   { 0x0004c8, 1, 119 SEP Ll_Cyrillic },
   { 0x0004c9, 1, 120 SEP Lu_Cyrillic },
   { 0x0004ca, 1, 119 SEP Ll_Cyrillic },
   { 0x0004cb, 1, 120 SEP Lu_Cyrillic },
   { 0x0004cc, 1, 119 SEP Ll_Cyrillic },
   { 0x0004cd, 1, 120 SEP Lu_Cyrillic },
   { 0x0004ce, 1, 119 SEP Ll_Cyrillic },
   { 0x0004d0, 1, 120 SEP Lu_Cyrillic },
   { 0x0004d1, 1, 119 SEP Ll_Cyrillic },
   { 0x0004d2, 1, 120 SEP Lu_Cyrillic },
   { 0x0004d3, 1, 119 SEP Ll_Cyrillic },
   { 0x0004d4, 1, 120 SEP Lu_Cyrillic },
   { 0x0004d5, 1, 119 SEP Ll_Cyrillic },
   { 0x0004d6, 1, 120 SEP Lu_Cyrillic },
   { 0x0004d7, 1, 119 SEP Ll_Cyrillic },
   { 0x0004d8, 1, 120 SEP Lu_Cyrillic },
   { 0x0004d9, 1, 119 SEP Ll_Cyrillic },
   { 0x0004da, 1, 120 SEP Lu_Cyrillic },
   { 0x0004db, 1, 119 SEP Ll_Cyrillic },
   { 0x0004dc, 1, 120 SEP Lu_Cyrillic },
   { 0x0004dd, 1, 119 SEP Ll_Cyrillic },
   { 0x0004de, 1, 120 SEP Lu_Cyrillic },
   { 0x0004df, 1, 119 SEP Ll_Cyrillic },
   { 0x0004e0, 1, 120 SEP Lu_Cyrillic },
   { 0x0004e1, 1, 119 SEP Ll_Cyrillic },
   { 0x0004e2, 1, 120 SEP Lu_Cyrillic },
   { 0x0004e3, 1, 119 SEP Ll_Cyrillic },
   { 0x0004e4, 1, 120 SEP Lu_Cyrillic },
   { 0x0004e5, 1, 119 SEP Ll_Cyrillic },
   { 0x0004e6, 1, 120 SEP Lu_Cyrillic },
   { 0x0004e7, 1, 119 SEP Ll_Cyrillic },
   { 0x0004e8, 1, 120 SEP Lu_Cyrillic },
   { 0x0004e9, 1, 119 SEP Ll_Cyrillic },
   { 0x0004ea, 1, 120 SEP Lu_Cyrillic },
   { 0x0004eb, 1, 119 SEP Ll_Cyrillic },
   { 0x0004ec, 1, 120 SEP Lu_Cyrillic },
   { 0x0004ed, 1, 119 SEP Ll_Cyrillic },
   { 0x0004ee, 1, 120 SEP Lu_Cyrillic },
   { 0x0004ef, 1, 119 SEP Ll_Cyrillic },
   { 0x0004f0, 1, 120 SEP Lu_Cyrillic },
   { 0x0004f1, 1, 119 SEP Ll_Cyrillic },
   { 0x0004f2, 1, 120 SEP Lu_Cyrillic },
   { 0x0004f3, 1, 119 SEP Ll_Cyrillic },
   { 0x0004f4, 1, 120 SEP Lu_Cyrillic },
   { 0x0004f5, 1, 119 SEP Ll_Cyrillic },
   { 0x0004f8, 1, 120 SEP Lu_Cyrillic },
   { 0x0004f9, 1, 119 SEP Ll_Cyrillic },
   { 0x000500, 1, 8 SEP Lu_CyrillicSupplementary },
   { 0x000501, 1, 8 SEP Ll_CyrillicSupplementary },
   { 0x000502, 1, 8 SEP Lu_CyrillicSupplementary },
   { 0x000503, 1, 8 SEP Ll_CyrillicSupplementary },
   { 0x000504, 1, 8 SEP Lu_CyrillicSupplementary },
   { 0x000505, 1, 8 SEP Ll_CyrillicSupplementary },
   { 0x000506, 1, 8 SEP Lu_CyrillicSupplementary },
   { 0x000507, 1, 8 SEP Ll_CyrillicSupplementary },
   { 0x000508, 1, 8 SEP Lu_CyrillicSupplementary },
   { 0x000509, 1, 8 SEP Ll_CyrillicSupplementary },
   { 0x00050a, 1, 8 SEP Lu_CyrillicSupplementary },
   { 0x00050b, 1, 8 SEP Ll_CyrillicSupplementary },
   { 0x00050c, 1, 8 SEP Lu_CyrillicSupplementary },
   { 0x00050d, 1, 8 SEP Ll_CyrillicSupplementary },
   { 0x00050e, 1, 8 SEP Lu_CyrillicSupplementary },
   { 0x00050f, 1, 8 SEP Ll_CyrillicSupplementary },
   { 0x000531, 38, 38 SEP Lu_Armenian },
   { 0x000559, 1, 1 SEP Lm_Armenian },
   { 0x00055a, 6, 7 SEP Po_Armenian },
   { 0x000561, 39, 39 SEP Ll_Armenian },
   { 0x000589, 1, 7 SEP Po_Armenian },
   { 0x00058a, 1, 1 SEP Pd_Armenian },
   { 0x000591, 17, 47 SEP Mn_Hebrew },
   { 0x0005a3, 23, 47 SEP Mn_Hebrew },
   { 0x0005bb, 3, 47 SEP Mn_Hebrew },
   { 0x0005be, 1, 5 SEP Po_Hebrew },
   { 0x0005bf, 1, 47 SEP Mn_Hebrew },
   { 0x0005c0, 1, 5 SEP Po_Hebrew },
   { 0x0005c1, 2, 47 SEP Mn_Hebrew },
   { 0x0005c3, 1, 5 SEP Po_Hebrew },
   { 0x0005c4, 1, 47 SEP Mn_Hebrew },
   { 0x0005d0, 27, 30 SEP Lo_Hebrew },
   { 0x0005f0, 3, 30 SEP Lo_Hebrew },
   { 0x0005f3, 2, 5 SEP Po_Hebrew },
   { 0x00060c, 1, 8 SEP Po_Arabic },
   { 0x00061b, 1, 8 SEP Po_Arabic },
   { 0x00061f, 1, 8 SEP Po_Arabic },
   { 0x000621, 26, 141 SEP Lo_Arabic },
   { 0x000640, 1, 3 SEP Lm_Arabic },
   { 0x000641, 10, 141 SEP Lo_Arabic },
   { 0x00064b, 11, 31 SEP Mn_Arabic },
   { 0x000660, 10, 20 SEP Nd_Arabic },
   { 0x00066a, 4, 8 SEP Po_Arabic },
   { 0x00066e, 2, 141 SEP Lo_Arabic },
   { 0x000670, 1, 31 SEP Mn_Arabic },
   { 0x000671, 99, 141 SEP Lo_Arabic },
   { 0x0006d4, 1, 8 SEP Po_Arabic },
   { 0x0006d5, 1, 141 SEP Lo_Arabic },
   { 0x0006d6, 7, 31 SEP Mn_Arabic },
   { 0x0006dd, 1, 1 SEP Cf_Arabic },
   { 0x0006de, 1, 1 SEP Me_Arabic },
   { 0x0006df, 6, 31 SEP Mn_Arabic },
   { 0x0006e5, 2, 3 SEP Lm_Arabic },
   { 0x0006e7, 2, 31 SEP Mn_Arabic },
   { 0x0006e9, 1, 3 SEP So_Arabic },
   { 0x0006ea, 4, 31 SEP Mn_Arabic },
   { 0x0006f0, 10, 20 SEP Nd_Arabic },
   { 0x0006fa, 3, 141 SEP Lo_Arabic },
   { 0x0006fd, 2, 3 SEP So_Arabic },
   { 0x000700, 14, 14 SEP Po_Syriac },
   { 0x00070f, 1, 1 SEP Cf_Syriac },
   { 0x000710, 1, 28 SEP Lo_Syriac },
   { 0x000711, 1, 28 SEP Mn_Syriac },
   { 0x000712, 27, 28 SEP Lo_Syriac },
   { 0x000730, 27, 28 SEP Mn_Syriac },
   { 0x000780, 38, 39 SEP Lo_Thaana },
   { 0x0007a6, 11, 11 SEP Mn_Thaana },
   { 0x0007b1, 1, 39 SEP Lo_Thaana },
   { 0x000901, 2, 18 SEP Mn_Devanagari },
   { 0x000903, 1, 8 SEP Mc_Devanagari },
   { 0x000905, 53, 65 SEP Lo_Devanagari },
   { 0x00093c, 1, 18 SEP Mn_Devanagari },
   { 0x00093d, 1, 65 SEP Lo_Devanagari },
   { 0x00093e, 3, 8 SEP Mc_Devanagari },
   { 0x000941, 8, 18 SEP Mn_Devanagari },
   { 0x000949, 4, 8 SEP Mc_Devanagari },
   { 0x00094d, 1, 18 SEP Mn_Devanagari },
   { 0x000950, 1, 65 SEP Lo_Devanagari },
   { 0x000951, 4, 18 SEP Mn_Devanagari },
   { 0x000958, 10, 65 SEP Lo_Devanagari },
   { 0x000962, 2, 18 SEP Mn_Devanagari },
   { 0x000964, 2, 3 SEP Po_Devanagari },
   { 0x000966, 10, 10 SEP Nd_Devanagari },
   { 0x000970, 1, 3 SEP Po_Devanagari },
   { 0x000981, 1, 9 SEP Mn_Bengali },
   { 0x000982, 2, 10 SEP Mc_Bengali },
   { 0x000985, 8, 51 SEP Lo_Bengali },
   { 0x00098f, 2, 51 SEP Lo_Bengali },
   { 0x000993, 22, 51 SEP Lo_Bengali },
   { 0x0009aa, 7, 51 SEP Lo_Bengali },
   { 0x0009b2, 1, 51 SEP Lo_Bengali },
   { 0x0009b6, 4, 51 SEP Lo_Bengali },
   { 0x0009bc, 1, 9 SEP Mn_Bengali },
   { 0x0009be, 3, 10 SEP Mc_Bengali },
   { 0x0009c1, 4, 9 SEP Mn_Bengali },
   { 0x0009c7, 2, 10 SEP Mc_Bengali },
   { 0x0009cb, 2, 10 SEP Mc_Bengali },
   { 0x0009cd, 1, 9 SEP Mn_Bengali },
   { 0x0009d7, 1, 10 SEP Mc_Bengali },
   { 0x0009dc, 2, 51 SEP Lo_Bengali },
   { 0x0009df, 3, 51 SEP Lo_Bengali },
   { 0x0009e2, 2, 9 SEP Mn_Bengali },
   { 0x0009e6, 10, 10 SEP Nd_Bengali },
   { 0x0009f0, 2, 51 SEP Lo_Bengali },
   { 0x0009f2, 2, 2 SEP Sc_Bengali },
   { 0x0009f4, 6, 6 SEP No_Bengali },
   { 0x0009fa, 1, 1 SEP So_Bengali },
   { 0x000a02, 1, 11 SEP Mn_Gurmukhi },
   { 0x000a05, 6, 51 SEP Lo_Gurmukhi },
   { 0x000a0f, 2, 51 SEP Lo_Gurmukhi },
   { 0x000a13, 22, 51 SEP Lo_Gurmukhi },
   { 0x000a2a, 7, 51 SEP Lo_Gurmukhi },
   { 0x000a32, 2, 51 SEP Lo_Gurmukhi },
   { 0x000a35, 2, 51 SEP Lo_Gurmukhi },
   { 0x000a38, 2, 51 SEP Lo_Gurmukhi },
   { 0x000a3c, 1, 11 SEP Mn_Gurmukhi },
   { 0x000a3e, 3, 3 SEP Mc_Gurmukhi },
   { 0x000a41, 2, 11 SEP Mn_Gurmukhi },
   { 0x000a47, 2, 11 SEP Mn_Gurmukhi },
   { 0x000a4b, 3, 11 SEP Mn_Gurmukhi },
   { 0x000a59, 4, 51 SEP Lo_Gurmukhi },
   { 0x000a5e, 1, 51 SEP Lo_Gurmukhi },
   { 0x000a66, 10, 10 SEP Nd_Gurmukhi },
   { 0x000a70, 2, 11 SEP Mn_Gurmukhi },
   { 0x000a72, 3, 51 SEP Lo_Gurmukhi },
   { 0x000a81, 2, 11 SEP Mn_Gujarati },
   { 0x000a83, 1, 7 SEP Mc_Gujarati },
   { 0x000a85, 7, 50 SEP Lo_Gujarati },
   { 0x000a8d, 1, 50 SEP Lo_Gujarati },
   { 0x000a8f, 3, 50 SEP Lo_Gujarati },
   { 0x000a93, 22, 50 SEP Lo_Gujarati },
   { 0x000aaa, 7, 50 SEP Lo_Gujarati },
   { 0x000ab2, 2, 50 SEP Lo_Gujarati },
   { 0x000ab5, 5, 50 SEP Lo_Gujarati },
   { 0x000abc, 1, 11 SEP Mn_Gujarati },
   { 0x000abd, 1, 50 SEP Lo_Gujarati },
   { 0x000abe, 3, 7 SEP Mc_Gujarati },
   { 0x000ac1, 5, 11 SEP Mn_Gujarati },
   { 0x000ac7, 2, 11 SEP Mn_Gujarati },
   { 0x000ac9, 1, 7 SEP Mc_Gujarati },
   { 0x000acb, 2, 7 SEP Mc_Gujarati },
   { 0x000acd, 1, 11 SEP Mn_Gujarati },
   { 0x000ad0, 1, 50 SEP Lo_Gujarati },
   { 0x000ae0, 1, 50 SEP Lo_Gujarati },
   { 0x000ae6, 10, 10 SEP Nd_Gujarati },
   { 0x000b01, 1, 8 SEP Mn_Oriya },
   { 0x000b02, 2, 9 SEP Mc_Oriya },
   { 0x000b05, 8, 51 SEP Lo_Oriya },
   { 0x000b0f, 2, 51 SEP Lo_Oriya },
   { 0x000b13, 22, 51 SEP Lo_Oriya },
   { 0x000b2a, 7, 51 SEP Lo_Oriya },
   { 0x000b32, 2, 51 SEP Lo_Oriya },
   { 0x000b36, 4, 51 SEP Lo_Oriya },
   { 0x000b3c, 1, 8 SEP Mn_Oriya },
   { 0x000b3d, 1, 51 SEP Lo_Oriya },
   { 0x000b3e, 1, 9 SEP Mc_Oriya },
   { 0x000b3f, 1, 8 SEP Mn_Oriya },
   { 0x000b40, 1, 9 SEP Mc_Oriya },
   { 0x000b41, 3, 8 SEP Mn_Oriya },
   { 0x000b47, 2, 9 SEP Mc_Oriya },
   { 0x000b4b, 2, 9 SEP Mc_Oriya },
   { 0x000b4d, 1, 8 SEP Mn_Oriya },
   { 0x000b56, 1, 8 SEP Mn_Oriya },
   { 0x000b57, 1, 9 SEP Mc_Oriya },
   { 0x000b5c, 2, 51 SEP Lo_Oriya },
   { 0x000b5f, 3, 51 SEP Lo_Oriya },
   { 0x000b66, 10, 10 SEP Nd_Oriya },
   { 0x000b70, 1, 1 SEP So_Oriya },
   { 0x000b82, 1, 3 SEP Mn_Tamil },
   { 0x000b83, 1, 35 SEP Lo_Tamil },
   { 0x000b85, 6, 35 SEP Lo_Tamil },
   { 0x000b8e, 3, 35 SEP Lo_Tamil },
   { 0x000b92, 4, 35 SEP Lo_Tamil },
   { 0x000b99, 2, 35 SEP Lo_Tamil },
   { 0x000b9c, 1, 35 SEP Lo_Tamil },
   { 0x000b9e, 2, 35 SEP Lo_Tamil },
   { 0x000ba3, 2, 35 SEP Lo_Tamil },
   { 0x000ba8, 3, 35 SEP Lo_Tamil },
   { 0x000bae, 8, 35 SEP Lo_Tamil },
   { 0x000bb7, 3, 35 SEP Lo_Tamil },
   { 0x000bbe, 2, 11 SEP Mc_Tamil },
   { 0x000bc0, 1, 3 SEP Mn_Tamil },
   { 0x000bc1, 2, 11 SEP Mc_Tamil },
   { 0x000bc6, 3, 11 SEP Mc_Tamil },
   { 0x000bca, 3, 11 SEP Mc_Tamil },
   { 0x000bcd, 1, 3 SEP Mn_Tamil },
   { 0x000bd7, 1, 11 SEP Mc_Tamil },
   { 0x000be7, 9, 9 SEP Nd_Tamil },
   { 0x000bf0, 3, 3 SEP No_Tamil },
   { 0x000c01, 3, 7 SEP Mc_Telugu },
   { 0x000c05, 8, 51 SEP Lo_Telugu },
   { 0x000c0e, 3, 51 SEP Lo_Telugu },
   { 0x000c12, 23, 51 SEP Lo_Telugu },
   { 0x000c2a, 10, 51 SEP Lo_Telugu },
   { 0x000c35, 5, 51 SEP Lo_Telugu },
   { 0x000c3e, 3, 12 SEP Mn_Telugu },
   { 0x000c41, 4, 7 SEP Mc_Telugu },
   { 0x000c46, 3, 12 SEP Mn_Telugu },
   { 0x000c4a, 4, 12 SEP Mn_Telugu },
   { 0x000c55, 2, 12 SEP Mn_Telugu },
   { 0x000c60, 2, 51 SEP Lo_Telugu },
   { 0x000c66, 10, 10 SEP Nd_Telugu },
   { 0x000c82, 2, 14 SEP Mc_Kannada },
   { 0x000c85, 8, 52 SEP Lo_Kannada },
   { 0x000c8e, 3, 52 SEP Lo_Kannada },
   { 0x000c92, 23, 52 SEP Lo_Kannada },
   { 0x000caa, 10, 52 SEP Lo_Kannada },
   { 0x000cb5, 5, 52 SEP Lo_Kannada },
   { 0x000cbe, 1, 14 SEP Mc_Kannada },
   { 0x000cbf, 1, 4 SEP Mn_Kannada },
   { 0x000cc0, 5, 14 SEP Mc_Kannada },
   { 0x000cc6, 1, 4 SEP Mn_Kannada },
   { 0x000cc7, 2, 14 SEP Mc_Kannada },
   { 0x000cca, 2, 14 SEP Mc_Kannada },
   { 0x000ccc, 2, 4 SEP Mn_Kannada },
   { 0x000cd5, 2, 14 SEP Mc_Kannada },
   { 0x000cde, 1, 52 SEP Lo_Kannada },
   { 0x000ce0, 2, 52 SEP Lo_Kannada },
   { 0x000ce6, 10, 10 SEP Nd_Kannada },
   { 0x000d02, 2, 12 SEP Mc_Malayalam },
   { 0x000d05, 8, 52 SEP Lo_Malayalam },
   { 0x000d0e, 3, 52 SEP Lo_Malayalam },
   { 0x000d12, 23, 52 SEP Lo_Malayalam },
   { 0x000d2a, 16, 52 SEP Lo_Malayalam },
   { 0x000d3e, 3, 12 SEP Mc_Malayalam },
   { 0x000d41, 3, 4 SEP Mn_Malayalam },
   { 0x000d46, 3, 12 SEP Mc_Malayalam },
   { 0x000d4a, 3, 12 SEP Mc_Malayalam },
   { 0x000d4d, 1, 4 SEP Mn_Malayalam },
   { 0x000d57, 1, 12 SEP Mc_Malayalam },
   { 0x000d60, 2, 52 SEP Lo_Malayalam },
   { 0x000d66, 10, 10 SEP Nd_Malayalam },
   { 0x000d82, 2, 15 SEP Mc_Sinhala },
   { 0x000d85, 18, 59 SEP Lo_Sinhala },
   { 0x000d9a, 24, 59 SEP Lo_Sinhala },
   { 0x000db3, 9, 59 SEP Lo_Sinhala },
   { 0x000dbd, 1, 59 SEP Lo_Sinhala },
   { 0x000dc0, 7, 59 SEP Lo_Sinhala },
   { 0x000dca, 1, 5 SEP Mn_Sinhala },
   { 0x000dcf, 3, 15 SEP Mc_Sinhala },
   { 0x000dd2, 3, 5 SEP Mn_Sinhala },
   { 0x000dd6, 1, 5 SEP Mn_Sinhala },
   { 0x000dd8, 8, 15 SEP Mc_Sinhala },
   { 0x000df2, 2, 15 SEP Mc_Sinhala },
   { 0x000df4, 1, 1 SEP Po_Sinhala },
   { 0x000e01, 48, 56 SEP Lo_Thai },
   { 0x000e31, 1, 16 SEP Mn_Thai },
   { 0x000e32, 2, 56 SEP Lo_Thai },
   { 0x000e34, 7, 16 SEP Mn_Thai },
   { 0x000e3f, 1, 1 SEP Sc_Thai },
   { 0x000e40, 6, 56 SEP Lo_Thai },
   { 0x000e46, 1, 1 SEP Lm_Thai },
   { 0x000e47, 8, 16 SEP Mn_Thai },
   { 0x000e4f, 1, 3 SEP Po_Thai },
   { 0x000e50, 10, 10 SEP Nd_Thai },
   { 0x000e5a, 2, 3 SEP Po_Thai },
   { 0x000e81, 2, 39 SEP Lo_Lao },
   { 0x000e84, 1, 39 SEP Lo_Lao },
   { 0x000e87, 2, 39 SEP Lo_Lao },
   { 0x000e8a, 1, 39 SEP Lo_Lao },
   { 0x000e8d, 1, 39 SEP Lo_Lao },
   { 0x000e94, 4, 39 SEP Lo_Lao },
   { 0x000e99, 7, 39 SEP Lo_Lao },
   { 0x000ea1, 3, 39 SEP Lo_Lao },
   { 0x000ea5, 1, 39 SEP Lo_Lao },
   { 0x000ea7, 1, 39 SEP Lo_Lao },
   { 0x000eaa, 2, 39 SEP Lo_Lao },
   { 0x000ead, 4, 39 SEP Lo_Lao },
   { 0x000eb1, 1, 15 SEP Mn_Lao },
   { 0x000eb2, 2, 39 SEP Lo_Lao },
   { 0x000eb4, 6, 15 SEP Mn_Lao },
   { 0x000ebb, 2, 15 SEP Mn_Lao },
   { 0x000ebd, 1, 39 SEP Lo_Lao },
   { 0x000ec0, 5, 39 SEP Lo_Lao },
   { 0x000ec6, 1, 1 SEP Lm_Lao },
   { 0x000ec8, 6, 15 SEP Mn_Lao },
   { 0x000ed0, 10, 10 SEP Nd_Lao },
   { 0x000edc, 2, 39 SEP Lo_Lao },
   { 0x000f00, 1, 47 SEP Lo_Tibetan },
   { 0x000f01, 3, 32 SEP So_Tibetan },
   { 0x000f04, 15, 16 SEP Po_Tibetan },
   { 0x000f13, 5, 32 SEP So_Tibetan },
   { 0x000f18, 2, 71 SEP Mn_Tibetan },
   { 0x000f1a, 6, 32 SEP So_Tibetan },
   { 0x000f20, 10, 10 SEP Nd_Tibetan },
   { 0x000f2a, 10, 10 SEP No_Tibetan },
   { 0x000f34, 1, 32 SEP So_Tibetan },
   { 0x000f35, 1, 71 SEP Mn_Tibetan },
   { 0x000f36, 1, 32 SEP So_Tibetan },
   { 0x000f37, 1, 71 SEP Mn_Tibetan },
   { 0x000f38, 1, 32 SEP So_Tibetan },
   { 0x000f39, 1, 71 SEP Mn_Tibetan },
   { 0x000f3a, 1, 2 SEP Ps_Tibetan },
   { 0x000f3b, 1, 2 SEP Pe_Tibetan },
   { 0x000f3c, 1, 2 SEP Ps_Tibetan },
   { 0x000f3d, 1, 2 SEP Pe_Tibetan },
   { 0x000f3e, 2, 3 SEP Mc_Tibetan },
   { 0x000f40, 8, 47 SEP Lo_Tibetan },
   { 0x000f49, 34, 47 SEP Lo_Tibetan },
   { 0x000f71, 14, 71 SEP Mn_Tibetan },
   { 0x000f7f, 1, 3 SEP Mc_Tibetan },
   { 0x000f80, 5, 71 SEP Mn_Tibetan },
   { 0x000f85, 1, 16 SEP Po_Tibetan },
   { 0x000f86, 2, 71 SEP Mn_Tibetan },
   { 0x000f88, 4, 47 SEP Lo_Tibetan },
   { 0x000f90, 8, 71 SEP Mn_Tibetan },
   { 0x000f99, 36, 71 SEP Mn_Tibetan },
   { 0x000fbe, 8, 32 SEP So_Tibetan },
   { 0x000fc6, 1, 71 SEP Mn_Tibetan },
   { 0x000fc7, 6, 32 SEP So_Tibetan },
   { 0x000fcf, 1, 32 SEP So_Tibetan },
   { 0x001000, 34, 47 SEP Lo_Myanmar },
   { 0x001023, 5, 47 SEP Lo_Myanmar },
   { 0x001029, 2, 47 SEP Lo_Myanmar },
   { 0x00102c, 1, 5 SEP Mc_Myanmar },
   { 0x00102d, 4, 10 SEP Mn_Myanmar },
   { 0x001031, 1, 5 SEP Mc_Myanmar },
   { 0x001032, 1, 10 SEP Mn_Myanmar },
   { 0x001036, 2, 10 SEP Mn_Myanmar },
   { 0x001038, 1, 5 SEP Mc_Myanmar },
   { 0x001039, 1, 10 SEP Mn_Myanmar },
   { 0x001040, 10, 10 SEP Nd_Myanmar },
   { 0x00104a, 6, 6 SEP Po_Myanmar },
   { 0x001050, 6, 47 SEP Lo_Myanmar },
   { 0x001056, 2, 5 SEP Mc_Myanmar },
   { 0x001058, 2, 10 SEP Mn_Myanmar },
   { 0x0010a0, 38, 38 SEP Lu_Georgian },
   { 0x0010d0, 41, 41 SEP Lo_Georgian },
   { 0x0010fb, 1, 1 SEP Po_Georgian },
   { 0x001100, 90, 240 SEP Lo_HangulJamo },
   { 0x00115f, 68, 240 SEP Lo_HangulJamo },
   { 0x0011a8, 82, 240 SEP Lo_HangulJamo },
   { 0x001200, 7, 317 SEP Lo_Ethiopic },
   { 0x001208, 63, 317 SEP Lo_Ethiopic },
   { 0x001248, 1, 317 SEP Lo_Ethiopic },
   { 0x00124a, 4, 317 SEP Lo_Ethiopic },
   { 0x001250, 7, 317 SEP Lo_Ethiopic },
   { 0x001258, 1, 317 SEP Lo_Ethiopic },
   { 0x00125a, 4, 317 SEP Lo_Ethiopic },
   { 0x001260, 39, 317 SEP Lo_Ethiopic },
   { 0x001288, 1, 317 SEP Lo_Ethiopic },
   { 0x00128a, 4, 317 SEP Lo_Ethiopic },
   { 0x001290, 31, 317 SEP Lo_Ethiopic },
   { 0x0012b0, 1, 317 SEP Lo_Ethiopic },
   { 0x0012b2, 4, 317 SEP Lo_Ethiopic },
   { 0x0012b8, 7, 317 SEP Lo_Ethiopic },
   { 0x0012c0, 1, 317 SEP Lo_Ethiopic },
   { 0x0012c2, 4, 317 SEP Lo_Ethiopic },
   { 0x0012c8, 7, 317 SEP Lo_Ethiopic },
   { 0x0012d0, 7, 317 SEP Lo_Ethiopic },
   { 0x0012d8, 23, 317 SEP Lo_Ethiopic },
   { 0x0012f0, 31, 317 SEP Lo_Ethiopic },
   { 0x001310, 1, 317 SEP Lo_Ethiopic },
   { 0x001312, 4, 317 SEP Lo_Ethiopic },
   { 0x001318, 7, 317 SEP Lo_Ethiopic },
   { 0x001320, 39, 317 SEP Lo_Ethiopic },
   { 0x001348, 19, 317 SEP Lo_Ethiopic },
   { 0x001361, 8, 8 SEP Po_Ethiopic },
   { 0x001369, 9, 9 SEP Nd_Ethiopic },
   { 0x001372, 11, 11 SEP No_Ethiopic },
   { 0x0013a0, 85, 85 SEP Lo_Cherokee },
   { 0x001401, 620, 628 SEP Lo_UnifiedCanadianAboriginalSyllabics },
   { 0x00166d, 2, 2 SEP Po_UnifiedCanadianAboriginalSyllabics },
   { 0x00166f, 8, 628 SEP Lo_UnifiedCanadianAboriginalSyllabics },
   { 0x001680, 1, 1 SEP Zs_Ogham },
   { 0x001681, 26, 26 SEP Lo_Ogham },
   { 0x00169b, 1, 1 SEP Ps_Ogham },
   { 0x00169c, 1, 1 SEP Pe_Ogham },
   { 0x0016a0, 75, 75 SEP Lo_Runic },
   { 0x0016eb, 3, 3 SEP Po_Runic },
   { 0x0016ee, 3, 3 SEP Nl_Runic },
   { 0x001700, 13, 17 SEP Lo_Tagalog },
   { 0x00170e, 4, 17 SEP Lo_Tagalog },
   { 0x001712, 3, 3 SEP Mn_Tagalog },
   { 0x001720, 18, 18 SEP Lo_Hanunoo },
   { 0x001732, 3, 3 SEP Mn_Hanunoo },
   { 0x001735, 2, 2 SEP Po_Hanunoo },
   { 0x001740, 18, 18 SEP Lo_Buhid },
   { 0x001752, 2, 2 SEP Mn_Buhid },
   { 0x001760, 13, 16 SEP Lo_Tagbanwa },
   { 0x00176e, 3, 16 SEP Lo_Tagbanwa },
   { 0x001772, 2, 2 SEP Mn_Tagbanwa },
   { 0x001780, 52, 53 SEP Lo_Khmer },
   { 0x0017b4, 3, 13 SEP Mc_Khmer },
   { 0x0017b7, 7, 19 SEP Mn_Khmer },
   { 0x0017be, 8, 13 SEP Mc_Khmer },
   { 0x0017c6, 1, 19 SEP Mn_Khmer },
   { 0x0017c7, 2, 13 SEP Mc_Khmer },
   { 0x0017c9, 11, 19 SEP Mn_Khmer },
   { 0x0017d4, 3, 6 SEP Po_Khmer },
   { 0x0017d7, 1, 1 SEP Lm_Khmer },
   { 0x0017d8, 3, 6 SEP Po_Khmer },
   { 0x0017db, 1, 1 SEP Sc_Khmer },
   { 0x0017dc, 1, 53 SEP Lo_Khmer },
   { 0x0017e0, 10, 10 SEP Nd_Khmer },
   { 0x001800, 6, 10 SEP Po_Mongolian },
   { 0x001806, 1, 1 SEP Pd_Mongolian },
   { 0x001807, 4, 10 SEP Po_Mongolian },
   { 0x00180b, 3, 4 SEP Mn_Mongolian },
   { 0x00180e, 1, 1 SEP Cf_Mongolian },
   { 0x001810, 10, 10 SEP Nd_Mongolian },
   { 0x001820, 35, 128 SEP Lo_Mongolian },
   { 0x001843, 1, 1 SEP Lm_Mongolian },
   { 0x001844, 52, 128 SEP Lo_Mongolian },
   { 0x001880, 41, 128 SEP Lo_Mongolian },
   { 0x0018a9, 1, 4 SEP Mn_Mongolian },
   { 0x001e00, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001e01, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001e02, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001e03, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001e04, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001e05, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001e06, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001e07, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001e08, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001e09, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001e0a, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001e0b, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001e0c, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001e0d, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001e0e, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001e0f, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001e10, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001e11, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001e12, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001e13, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001e14, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001e15, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001e16, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001e17, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001e18, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001e19, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001e1a, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001e1b, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001e1c, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001e1d, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001e1e, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001e1f, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001e20, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001e21, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001e22, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001e23, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001e24, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001e25, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001e26, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001e27, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001e28, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001e29, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001e2a, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001e2b, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001e2c, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001e2d, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001e2e, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001e2f, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001e30, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001e31, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001e32, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001e33, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001e34, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001e35, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001e36, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001e37, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001e38, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001e39, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001e3a, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001e3b, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001e3c, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001e3d, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001e3e, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001e3f, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001e40, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001e41, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001e42, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001e43, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001e44, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001e45, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001e46, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001e47, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001e48, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001e49, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001e4a, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001e4b, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001e4c, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001e4d, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001e4e, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001e4f, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001e50, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001e51, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001e52, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001e53, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001e54, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001e55, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001e56, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001e57, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001e58, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001e59, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001e5a, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001e5b, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001e5c, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001e5d, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001e5e, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001e5f, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001e60, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001e61, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001e62, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001e63, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001e64, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001e65, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001e66, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001e67, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001e68, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001e69, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001e6a, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001e6b, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001e6c, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001e6d, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001e6e, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001e6f, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001e70, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001e71, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001e72, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001e73, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001e74, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001e75, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001e76, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001e77, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001e78, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001e79, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001e7a, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001e7b, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001e7c, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001e7d, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001e7e, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001e7f, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001e80, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001e81, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001e82, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001e83, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001e84, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001e85, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001e86, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001e87, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001e88, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001e89, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001e8a, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001e8b, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001e8c, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001e8d, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001e8e, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001e8f, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001e90, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001e91, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001e92, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001e93, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001e94, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001e95, 7, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001ea0, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001ea1, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001ea2, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001ea3, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001ea4, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001ea5, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001ea6, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001ea7, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001ea8, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001ea9, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001eaa, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001eab, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001eac, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001ead, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001eae, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001eaf, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001eb0, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001eb1, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001eb2, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001eb3, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001eb4, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001eb5, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001eb6, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001eb7, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001eb8, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001eb9, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001eba, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001ebb, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001ebc, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001ebd, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001ebe, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001ebf, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001ec0, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001ec1, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001ec2, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001ec3, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001ec4, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001ec5, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001ec6, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001ec7, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001ec8, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001ec9, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001eca, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001ecb, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001ecc, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001ecd, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001ece, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001ecf, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001ed0, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001ed1, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001ed2, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001ed3, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001ed4, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001ed5, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001ed6, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001ed7, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001ed8, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001ed9, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001eda, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001edb, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001edc, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001edd, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001ede, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001edf, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001ee0, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001ee1, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001ee2, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001ee3, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001ee4, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001ee5, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001ee6, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001ee7, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001ee8, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001ee9, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001eea, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001eeb, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001eec, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001eed, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001eee, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001eef, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001ef0, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001ef1, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001ef2, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001ef3, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001ef4, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001ef5, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001ef6, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001ef7, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001ef8, 1, 120 SEP Lu_LatinExtendedAdditional },
   { 0x001ef9, 1, 126 SEP Ll_LatinExtendedAdditional },
   { 0x001f00, 8, 122 SEP Ll_GreekExtended },
   { 0x001f08, 8, 69 SEP Lu_GreekExtended },
   { 0x001f10, 6, 122 SEP Ll_GreekExtended },
   { 0x001f18, 6, 69 SEP Lu_GreekExtended },
   { 0x001f20, 8, 122 SEP Ll_GreekExtended },
   { 0x001f28, 8, 69 SEP Lu_GreekExtended },
   { 0x001f30, 8, 122 SEP Ll_GreekExtended },
   { 0x001f38, 8, 69 SEP Lu_GreekExtended },
   { 0x001f40, 6, 122 SEP Ll_GreekExtended },
   { 0x001f48, 6, 69 SEP Lu_GreekExtended },
   { 0x001f50, 8, 122 SEP Ll_GreekExtended },
   { 0x001f59, 1, 69 SEP Lu_GreekExtended },
   { 0x001f5b, 1, 69 SEP Lu_GreekExtended },
   { 0x001f5d, 1, 69 SEP Lu_GreekExtended },
   { 0x001f5f, 1, 69 SEP Lu_GreekExtended },
   { 0x001f60, 8, 122 SEP Ll_GreekExtended },
   { 0x001f68, 8, 69 SEP Lu_GreekExtended },
   { 0x001f70, 14, 122 SEP Ll_GreekExtended },
   { 0x001f80, 8, 122 SEP Ll_GreekExtended },
   { 0x001f88, 8, 27 SEP Lt_GreekExtended },
   { 0x001f90, 8, 122 SEP Ll_GreekExtended },
   { 0x001f98, 8, 27 SEP Lt_GreekExtended },
   { 0x001fa0, 8, 122 SEP Ll_GreekExtended },
   { 0x001fa8, 8, 27 SEP Lt_GreekExtended },
   { 0x001fb0, 5, 122 SEP Ll_GreekExtended },
   { 0x001fb6, 2, 122 SEP Ll_GreekExtended },
   { 0x001fb8, 4, 69 SEP Lu_GreekExtended },
   { 0x001fbc, 1, 27 SEP Lt_GreekExtended },
   { 0x001fbd, 1, 15 SEP Sk_GreekExtended },
   { 0x001fbe, 1, 122 SEP Ll_GreekExtended },
   { 0x001fbf, 3, 15 SEP Sk_GreekExtended },
   { 0x001fc2, 3, 122 SEP Ll_GreekExtended },
   { 0x001fc6, 2, 122 SEP Ll_GreekExtended },
   { 0x001fc8, 4, 69 SEP Lu_GreekExtended },
   { 0x001fcc, 1, 27 SEP Lt_GreekExtended },
   { 0x001fcd, 3, 15 SEP Sk_GreekExtended },
   { 0x001fd0, 4, 122 SEP Ll_GreekExtended },
   { 0x001fd6, 2, 122 SEP Ll_GreekExtended },
   { 0x001fd8, 4, 69 SEP Lu_GreekExtended },
   { 0x001fdd, 3, 15 SEP Sk_GreekExtended },
   { 0x001fe0, 8, 122 SEP Ll_GreekExtended },
   { 0x001fe8, 5, 69 SEP Lu_GreekExtended },
   { 0x001fed, 3, 15 SEP Sk_GreekExtended },
   { 0x001ff2, 3, 122 SEP Ll_GreekExtended },
   { 0x001ff6, 2, 122 SEP Ll_GreekExtended },
   { 0x001ff8, 4, 69 SEP Lu_GreekExtended },
   { 0x001ffc, 1, 27 SEP Lt_GreekExtended },
   { 0x001ffd, 2, 15 SEP Sk_GreekExtended },
   { 0x002000, 12, 14 SEP Zs_GeneralPunctuation },
   { 0x00200c, 4, 19 SEP Cf_GeneralPunctuation },
   { 0x002010, 6, 6 SEP Pd_GeneralPunctuation },
   { 0x002016, 2, 38 SEP Po_GeneralPunctuation },
   { 0x002018, 1, 5 SEP Pi_GeneralPunctuation },
   { 0x002019, 1, 3 SEP Pf_GeneralPunctuation },
   { 0x00201a, 1, 3 SEP Ps_GeneralPunctuation },
   { 0x00201b, 2, 5 SEP Pi_GeneralPunctuation },
   { 0x00201d, 1, 3 SEP Pf_GeneralPunctuation },
   { 0x00201e, 1, 3 SEP Ps_GeneralPunctuation },
   { 0x00201f, 1, 5 SEP Pi_GeneralPunctuation },
   { 0x002020, 8, 38 SEP Po_GeneralPunctuation },
   { 0x002028, 1, 1 SEP Zl_GeneralPunctuation },
   { 0x002029, 1, 1 SEP Zp_GeneralPunctuation },
   { 0x00202a, 5, 19 SEP Cf_GeneralPunctuation },
   { 0x00202f, 1, 14 SEP Zs_GeneralPunctuation },
   { 0x002030, 9, 38 SEP Po_GeneralPunctuation },
   { 0x002039, 1, 5 SEP Pi_GeneralPunctuation },
   { 0x00203a, 1, 3 SEP Pf_GeneralPunctuation },
   { 0x00203b, 4, 38 SEP Po_GeneralPunctuation },
   { 0x00203f, 2, 2 SEP Pc_GeneralPunctuation },
   { 0x002041, 3, 38 SEP Po_GeneralPunctuation },
   { 0x002044, 1, 2 SEP Sm_GeneralPunctuation },
   { 0x002045, 1, 3 SEP Ps_GeneralPunctuation },
   { 0x002046, 1, 1 SEP Pe_GeneralPunctuation },
   { 0x002047, 11, 38 SEP Po_GeneralPunctuation },
   { 0x002052, 1, 2 SEP Sm_GeneralPunctuation },
   { 0x002057, 1, 38 SEP Po_GeneralPunctuation },
   { 0x00205f, 1, 14 SEP Zs_GeneralPunctuation },
   { 0x002060, 4, 19 SEP Cf_GeneralPunctuation },
   { 0x00206a, 6, 19 SEP Cf_GeneralPunctuation },
   { 0x002070, 1, 17 SEP No_SuperscriptsandSubscripts },
   { 0x002071, 1, 2 SEP Ll_SuperscriptsandSubscripts },
   { 0x002074, 6, 17 SEP No_SuperscriptsandSubscripts },
   { 0x00207a, 3, 6 SEP Sm_SuperscriptsandSubscripts },
   { 0x00207d, 1, 2 SEP Ps_SuperscriptsandSubscripts },
   { 0x00207e, 1, 2 SEP Pe_SuperscriptsandSubscripts },
   { 0x00207f, 1, 2 SEP Ll_SuperscriptsandSubscripts },
   { 0x002080, 10, 17 SEP No_SuperscriptsandSubscripts },
   { 0x00208a, 3, 6 SEP Sm_SuperscriptsandSubscripts },
   { 0x00208d, 1, 2 SEP Ps_SuperscriptsandSubscripts },
   { 0x00208e, 1, 2 SEP Pe_SuperscriptsandSubscripts },
   { 0x0020a0, 18, 18 SEP Sc_CurrencySymbols },
   { 0x0020d0, 13, 20 SEP Mn_CombiningDiacriticalMarksforSymbols },
   { 0x0020dd, 4, 7 SEP Me_CombiningDiacriticalMarksforSymbols },
   { 0x0020e1, 1, 20 SEP Mn_CombiningDiacriticalMarksforSymbols },
   { 0x0020e2, 3, 7 SEP Me_CombiningDiacriticalMarksforSymbols },
   { 0x0020e5, 6, 20 SEP Mn_CombiningDiacriticalMarksforSymbols },
   { 0x002100, 2, 25 SEP So_LetterlikeSymbols },
   { 0x002102, 1, 27 SEP Lu_LetterlikeSymbols },
   { 0x002103, 4, 25 SEP So_LetterlikeSymbols },
   { 0x002107, 1, 27 SEP Lu_LetterlikeSymbols },
   { 0x002108, 2, 25 SEP So_LetterlikeSymbols },
   { 0x00210a, 1, 12 SEP Ll_LetterlikeSymbols },
   { 0x00210b, 3, 27 SEP Lu_LetterlikeSymbols },
   { 0x00210e, 2, 12 SEP Ll_LetterlikeSymbols },
   { 0x002110, 3, 27 SEP Lu_LetterlikeSymbols },
   { 0x002113, 1, 12 SEP Ll_LetterlikeSymbols },
   { 0x002114, 1, 25 SEP So_LetterlikeSymbols },
   { 0x002115, 1, 27 SEP Lu_LetterlikeSymbols },
   { 0x002116, 3, 25 SEP So_LetterlikeSymbols },
   { 0x002119, 5, 27 SEP Lu_LetterlikeSymbols },
   { 0x00211e, 6, 25 SEP So_LetterlikeSymbols },
   { 0x002124, 1, 27 SEP Lu_LetterlikeSymbols },
   { 0x002125, 1, 25 SEP So_LetterlikeSymbols },
   { 0x002126, 1, 27 SEP Lu_LetterlikeSymbols },
   { 0x002127, 1, 25 SEP So_LetterlikeSymbols },
   { 0x002128, 1, 27 SEP Lu_LetterlikeSymbols },
   { 0x002129, 1, 25 SEP So_LetterlikeSymbols },
   { 0x00212a, 4, 27 SEP Lu_LetterlikeSymbols },
   { 0x00212e, 1, 25 SEP So_LetterlikeSymbols },
   { 0x00212f, 1, 12 SEP Ll_LetterlikeSymbols },
   { 0x002130, 2, 27 SEP Lu_LetterlikeSymbols },
   { 0x002132, 1, 25 SEP So_LetterlikeSymbols },
   { 0x002133, 1, 27 SEP Lu_LetterlikeSymbols },
   { 0x002134, 1, 12 SEP Ll_LetterlikeSymbols },
   { 0x002135, 4, 4 SEP Lo_LetterlikeSymbols },
   { 0x002139, 1, 12 SEP Ll_LetterlikeSymbols },
   { 0x00213a, 1, 25 SEP So_LetterlikeSymbols },
   { 0x00213d, 1, 12 SEP Ll_LetterlikeSymbols },
   { 0x00213e, 2, 27 SEP Lu_LetterlikeSymbols },
   { 0x002140, 5, 6 SEP Sm_LetterlikeSymbols },
   { 0x002145, 1, 27 SEP Lu_LetterlikeSymbols },
   { 0x002146, 4, 12 SEP Ll_LetterlikeSymbols },
   { 0x00214a, 1, 25 SEP So_LetterlikeSymbols },
   { 0x00214b, 1, 6 SEP Sm_LetterlikeSymbols },
   { 0x002153, 13, 13 SEP No_NumberForms },
   { 0x002160, 36, 36 SEP Nl_NumberForms },
   { 0x002190, 5, 15 SEP Sm_Arrows },
   { 0x002195, 5, 85 SEP So_Arrows },
   { 0x00219a, 2, 15 SEP Sm_Arrows },
   { 0x00219c, 4, 85 SEP So_Arrows },
   { 0x0021a0, 1, 15 SEP Sm_Arrows },
   { 0x0021a1, 2, 85 SEP So_Arrows },
   { 0x0021a3, 1, 15 SEP Sm_Arrows },
   { 0x0021a4, 2, 85 SEP So_Arrows },
   { 0x0021a6, 1, 15 SEP Sm_Arrows },
   { 0x0021a7, 7, 85 SEP So_Arrows },
   { 0x0021ae, 1, 15 SEP Sm_Arrows },
   { 0x0021af, 31, 85 SEP So_Arrows },
   { 0x0021ce, 2, 15 SEP Sm_Arrows },
   { 0x0021d0, 2, 85 SEP So_Arrows },
   { 0x0021d2, 1, 15 SEP Sm_Arrows },
   { 0x0021d3, 1, 85 SEP So_Arrows },
   { 0x0021d4, 1, 15 SEP Sm_Arrows },
   { 0x0021d5, 31, 85 SEP So_Arrows },
   { 0x0021f4, 268, 657 SEP Sm },
   { 0x002300, 8, 170 SEP So_MiscellaneousTechnical },
   { 0x002308, 4, 32 SEP Sm_MiscellaneousTechnical },
   { 0x00230c, 20, 170 SEP So_MiscellaneousTechnical },
   { 0x002320, 2, 32 SEP Sm_MiscellaneousTechnical },
   { 0x002322, 7, 170 SEP So_MiscellaneousTechnical },
   { 0x002329, 1, 2 SEP Ps_MiscellaneousTechnical },
   { 0x00232a, 1, 2 SEP Pe_MiscellaneousTechnical },
   { 0x00232b, 81, 170 SEP So_MiscellaneousTechnical },
   { 0x00237c, 1, 32 SEP Sm_MiscellaneousTechnical },
   { 0x00237d, 30, 170 SEP So_MiscellaneousTechnical },
   { 0x00239b, 25, 32 SEP Sm_MiscellaneousTechnical },
   { 0x0023b4, 1, 2 SEP Ps_MiscellaneousTechnical },
   { 0x0023b5, 1, 2 SEP Pe_MiscellaneousTechnical },
   { 0x0023b6, 1, 1 SEP Po_MiscellaneousTechnical },
   { 0x0023b7, 24, 170 SEP So_MiscellaneousTechnical },
   { 0x002400, 39, 39 SEP So_ControlPictures },
   { 0x002440, 11, 11 SEP So_OpticalCharacterRecognition },
   { 0x002460, 60, 81 SEP No_EnclosedAlphanumerics },
   { 0x00249c, 78, 78 SEP So_EnclosedAlphanumerics },
   { 0x0024ea, 21, 81 SEP No_EnclosedAlphanumerics },
   { 0x002500, 183, 183 SEP So },
   { 0x0025b7, 1, 10 SEP Sm_GeometricShapes },
   { 0x0025b8, 9, 63 SEP So_GeometricShapes },
   { 0x0025c1, 1, 10 SEP Sm_GeometricShapes },
   { 0x0025c2, 54, 63 SEP So_GeometricShapes },
   { 0x0025f8, 8, 10 SEP Sm_GeometricShapes },
   { 0x002600, 20, 132 SEP So_MiscellaneousSymbols },
   { 0x002616, 2, 132 SEP So_MiscellaneousSymbols },
   { 0x002619, 86, 132 SEP So_MiscellaneousSymbols },
   { 0x00266f, 1, 1 SEP Sm_MiscellaneousSymbols },
   { 0x002670, 14, 132 SEP So_MiscellaneousSymbols },
   { 0x002680, 10, 132 SEP So_MiscellaneousSymbols },
   { 0x002701, 4, 130 SEP So_Dingbats },
   { 0x002706, 4, 130 SEP So_Dingbats },
   { 0x00270c, 28, 130 SEP So_Dingbats },
   { 0x002729, 35, 130 SEP So_Dingbats },
   { 0x00274d, 1, 130 SEP So_Dingbats },
   { 0x00274f, 4, 130 SEP So_Dingbats },
   { 0x002756, 1, 130 SEP So_Dingbats },
   { 0x002758, 7, 130 SEP So_Dingbats },
   { 0x002761, 7, 130 SEP So_Dingbats },
   { 0x002768, 1, 7 SEP Ps_Dingbats },
   { 0x002769, 1, 7 SEP Pe_Dingbats },
   { 0x00276a, 1, 7 SEP Ps_Dingbats },
   { 0x00276b, 1, 7 SEP Pe_Dingbats },
   { 0x00276c, 1, 7 SEP Ps_Dingbats },
   { 0x00276d, 1, 7 SEP Pe_Dingbats },
   { 0x00276e, 1, 7 SEP Ps_Dingbats },
   { 0x00276f, 1, 7 SEP Pe_Dingbats },
   { 0x002770, 1, 7 SEP Ps_Dingbats },
   { 0x002771, 1, 7 SEP Pe_Dingbats },
   { 0x002772, 1, 7 SEP Ps_Dingbats },
   { 0x002773, 1, 7 SEP Pe_Dingbats },
   { 0x002774, 1, 7 SEP Ps_Dingbats },
   { 0x002775, 1, 7 SEP Pe_Dingbats },
   { 0x002776, 30, 30 SEP No_Dingbats },
   { 0x002794, 1, 130 SEP So_Dingbats },
   { 0x002798, 24, 130 SEP So_Dingbats },
   { 0x0027b1, 14, 130 SEP So_Dingbats },
   { 0x0027d0, 22, 22 SEP Sm_MiscellaneousMathematicalSymbols_A },
   { 0x0027e6, 1, 3 SEP Ps_MiscellaneousMathematicalSymbols_A },
   { 0x0027e7, 1, 3 SEP Pe_MiscellaneousMathematicalSymbols_A },
   { 0x0027e8, 1, 3 SEP Ps_MiscellaneousMathematicalSymbols_A },
   { 0x0027e9, 1, 3 SEP Pe_MiscellaneousMathematicalSymbols_A },
   { 0x0027ea, 1, 3 SEP Ps_MiscellaneousMathematicalSymbols_A },
   { 0x0027eb, 1, 3 SEP Pe_MiscellaneousMathematicalSymbols_A },
   { 0x0027f0, 16, 16 SEP Sm_SupplementalArrows_A },
   { 0x002800, 256, 256 SEP So_BraillePatterns },
   { 0x002900, 131, 657 SEP Sm },
   { 0x002983, 1, 14 SEP Ps_MiscellaneousMathematicalSymbols_B },
   { 0x002984, 1, 14 SEP Pe_MiscellaneousMathematicalSymbols_B },
   { 0x002985, 1, 14 SEP Ps_MiscellaneousMathematicalSymbols_B },
   { 0x002986, 1, 14 SEP Pe_MiscellaneousMathematicalSymbols_B },
   { 0x002987, 1, 14 SEP Ps_MiscellaneousMathematicalSymbols_B },
   { 0x002988, 1, 14 SEP Pe_MiscellaneousMathematicalSymbols_B },
   { 0x002989, 1, 14 SEP Ps_MiscellaneousMathematicalSymbols_B },
   { 0x00298a, 1, 14 SEP Pe_MiscellaneousMathematicalSymbols_B },
   { 0x00298b, 1, 14 SEP Ps_MiscellaneousMathematicalSymbols_B },
   { 0x00298c, 1, 14 SEP Pe_MiscellaneousMathematicalSymbols_B },
   { 0x00298d, 1, 14 SEP Ps_MiscellaneousMathematicalSymbols_B },
   { 0x00298e, 1, 14 SEP Pe_MiscellaneousMathematicalSymbols_B },
   { 0x00298f, 1, 14 SEP Ps_MiscellaneousMathematicalSymbols_B },
   { 0x002990, 1, 14 SEP Pe_MiscellaneousMathematicalSymbols_B },
   { 0x002991, 1, 14 SEP Ps_MiscellaneousMathematicalSymbols_B },
   { 0x002992, 1, 14 SEP Pe_MiscellaneousMathematicalSymbols_B },
   { 0x002993, 1, 14 SEP Ps_MiscellaneousMathematicalSymbols_B },
   { 0x002994, 1, 14 SEP Pe_MiscellaneousMathematicalSymbols_B },
   { 0x002995, 1, 14 SEP Ps_MiscellaneousMathematicalSymbols_B },
   { 0x002996, 1, 14 SEP Pe_MiscellaneousMathematicalSymbols_B },
   { 0x002997, 1, 14 SEP Ps_MiscellaneousMathematicalSymbols_B },
   { 0x002998, 1, 14 SEP Pe_MiscellaneousMathematicalSymbols_B },
   { 0x002999, 63, 95 SEP Sm_MiscellaneousMathematicalSymbols_B },
   { 0x0029d8, 1, 14 SEP Ps_MiscellaneousMathematicalSymbols_B },
   { 0x0029d9, 1, 14 SEP Pe_MiscellaneousMathematicalSymbols_B },
   { 0x0029da, 1, 14 SEP Ps_MiscellaneousMathematicalSymbols_B },
   { 0x0029db, 1, 14 SEP Pe_MiscellaneousMathematicalSymbols_B },
   { 0x0029dc, 32, 95 SEP Sm_MiscellaneousMathematicalSymbols_B },
   { 0x0029fc, 1, 14 SEP Ps_MiscellaneousMathematicalSymbols_B },
   { 0x0029fd, 1, 14 SEP Pe_MiscellaneousMathematicalSymbols_B },
   { 0x0029fe, 258, 657 SEP Sm },
   { 0x002e80, 26, 115 SEP So_CJKRadicalsSupplement },
   { 0x002e9b, 89, 115 SEP So_CJKRadicalsSupplement },
   { 0x002f00, 214, 214 SEP So_KangxiRadicals },
   { 0x002ff0, 12, 12 SEP So_IdeographicDescriptionCharacters },
   { 0x003000, 1, 1 SEP Zs_CJKSymbolsandPunctuation },
   { 0x003001, 3, 4 SEP Po_CJKSymbolsandPunctuation },
   { 0x003004, 1, 8 SEP So_CJKSymbolsandPunctuation },
   { 0x003005, 1, 7 SEP Lm_CJKSymbolsandPunctuation },
   { 0x003006, 1, 2 SEP Lo_CJKSymbolsandPunctuation },
   { 0x003007, 1, 13 SEP Nl_CJKSymbolsandPunctuation },
   { 0x003008, 1, 10 SEP Ps_CJKSymbolsandPunctuation },
   { 0x003009, 1, 11 SEP Pe_CJKSymbolsandPunctuation },
   { 0x00300a, 1, 10 SEP Ps_CJKSymbolsandPunctuation },
   { 0x00300b, 1, 11 SEP Pe_CJKSymbolsandPunctuation },
   { 0x00300c, 1, 10 SEP Ps_CJKSymbolsandPunctuation },
   { 0x00300d, 1, 11 SEP Pe_CJKSymbolsandPunctuation },
   { 0x00300e, 1, 10 SEP Ps_CJKSymbolsandPunctuation },
   { 0x00300f, 1, 11 SEP Pe_CJKSymbolsandPunctuation },
   { 0x003010, 1, 10 SEP Ps_CJKSymbolsandPunctuation },
   { 0x003011, 1, 11 SEP Pe_CJKSymbolsandPunctuation },
   { 0x003012, 2, 8 SEP So_CJKSymbolsandPunctuation },
   { 0x003014, 1, 10 SEP Ps_CJKSymbolsandPunctuation },
   { 0x003015, 1, 11 SEP Pe_CJKSymbolsandPunctuation },
   { 0x003016, 1, 10 SEP Ps_CJKSymbolsandPunctuation },
   { 0x003017, 1, 11 SEP Pe_CJKSymbolsandPunctuation },
   { 0x003018, 1, 10 SEP Ps_CJKSymbolsandPunctuation },
   { 0x003019, 1, 11 SEP Pe_CJKSymbolsandPunctuation },
   { 0x00301a, 1, 10 SEP Ps_CJKSymbolsandPunctuation },
   { 0x00301b, 1, 11 SEP Pe_CJKSymbolsandPunctuation },
   { 0x00301c, 1, 2 SEP Pd_CJKSymbolsandPunctuation },
   { 0x00301d, 1, 10 SEP Ps_CJKSymbolsandPunctuation },
   { 0x00301e, 2, 11 SEP Pe_CJKSymbolsandPunctuation },
   { 0x003020, 1, 8 SEP So_CJKSymbolsandPunctuation },
   { 0x003021, 9, 13 SEP Nl_CJKSymbolsandPunctuation },
   { 0x00302a, 6, 6 SEP Mn_CJKSymbolsandPunctuation },
   { 0x003030, 1, 2 SEP Pd_CJKSymbolsandPunctuation },
   { 0x003031, 5, 7 SEP Lm_CJKSymbolsandPunctuation },
   { 0x003036, 2, 8 SEP So_CJKSymbolsandPunctuation },
   { 0x003038, 3, 13 SEP Nl_CJKSymbolsandPunctuation },
   { 0x00303b, 1, 7 SEP Lm_CJKSymbolsandPunctuation },
   { 0x00303c, 1, 2 SEP Lo_CJKSymbolsandPunctuation },
   { 0x00303d, 1, 4 SEP Po_CJKSymbolsandPunctuation },
   { 0x00303e, 2, 8 SEP So_CJKSymbolsandPunctuation },
   { 0x003041, 86, 87 SEP Lo_Hiragana },
   { 0x003099, 2, 2 SEP Mn_Hiragana },
   { 0x00309b, 2, 2 SEP Sk_Hiragana },
   { 0x00309d, 2, 2 SEP Lm_Hiragana },
   { 0x00309f, 1, 87 SEP Lo_Hiragana },
   { 0x0030a0, 1, 1 SEP Pd_Katakana },
   { 0x0030a1, 90, 91 SEP Lo_Katakana },
   { 0x0030fb, 1, 1 SEP Pc_Katakana },
   { 0x0030fc, 3, 3 SEP Lm_Katakana },
   { 0x0030ff, 1, 91 SEP Lo_Katakana },
   { 0x003105, 40, 40 SEP Lo_Bopomofo },
   { 0x003131, 94, 94 SEP Lo_HangulCompatibilityJamo },
   { 0x003190, 2, 12 SEP So_Kanbun },
   { 0x003192, 4, 4 SEP No_Kanbun },
   { 0x003196, 10, 12 SEP So_Kanbun },
   { 0x0031a0, 24, 24 SEP Lo_BopomofoExtended },
   { 0x0031f0, 16, 16 SEP Lo_KatakanaPhoneticExtensions },
   { 0x003200, 29, 182 SEP So_EnclosedCJKLettersandMonths },
   { 0x003220, 10, 50 SEP No_EnclosedCJKLettersandMonths },
   { 0x00322a, 26, 182 SEP So_EnclosedCJKLettersandMonths },
   { 0x003251, 15, 50 SEP No_EnclosedCJKLettersandMonths },
   { 0x003260, 28, 182 SEP So_EnclosedCJKLettersandMonths },
   { 0x00327f, 1, 182 SEP So_EnclosedCJKLettersandMonths },
   { 0x003280, 10, 50 SEP No_EnclosedCJKLettersandMonths },
   { 0x00328a, 39, 182 SEP So_EnclosedCJKLettersandMonths },
   { 0x0032b1, 15, 50 SEP No_EnclosedCJKLettersandMonths },
   { 0x0032c0, 12, 182 SEP So_EnclosedCJKLettersandMonths },
   { 0x0032d0, 47, 182 SEP So_EnclosedCJKLettersandMonths },
   { 0x003300, 119, 249 SEP So_CJKCompatibility },
   { 0x00337b, 99, 249 SEP So_CJKCompatibility },
   { 0x0033e0, 31, 249 SEP So_CJKCompatibility },
   { 0x003400, 1, 2 SEP Lo_CJKUnifiedIdeographsExtensionA },
   { 0x004db5, 1, 2 SEP Lo_CJKUnifiedIdeographsExtensionA },
   { 0x004e00, 1, 2 SEP Lo_CJKUnifiedIdeographs },
   { 0x009fa5, 1, 2 SEP Lo_CJKUnifiedIdeographs },
   { 0x00a000, 1165, 1165 SEP Lo_YiSyllables },
   { 0x00a490, 55, 55 SEP So_YiRadicals },
   { 0x00ac00, 1, 2 SEP Lo_HangulSyllables },
   { 0x00d7a3, 1, 2 SEP Lo_HangulSyllables },
   { 0x00d800, 1, 1 SEP Cs_HighSurrogates },
   { 0x00db7f, 2, 4 SEP Cs },
   { 0x00dbff, 2, 4 SEP Cs },
   { 0x00dfff, 1, 1 SEP Cs_LowSurrogates },
   { 0x00e000, 1, 2 SEP Co_PrivateUseArea },
   { 0x00f8ff, 1, 2 SEP Co_PrivateUseArea },
   { 0x00f900, 302, 361 SEP Lo_CJKCompatibilityIdeographs },
   { 0x00fa30, 59, 361 SEP Lo_CJKCompatibilityIdeographs },
   { 0x00fb00, 7, 12 SEP Ll_AlphabeticPresentationForms },
   { 0x00fb13, 5, 12 SEP Ll_AlphabeticPresentationForms },
   { 0x00fb1d, 1, 34 SEP Lo_AlphabeticPresentationForms },
   { 0x00fb1e, 1, 1 SEP Mn_AlphabeticPresentationForms },
   { 0x00fb1f, 10, 34 SEP Lo_AlphabeticPresentationForms },
   { 0x00fb29, 1, 1 SEP Sm_AlphabeticPresentationForms },
   { 0x00fb2a, 13, 34 SEP Lo_AlphabeticPresentationForms },
   { 0x00fb38, 5, 34 SEP Lo_AlphabeticPresentationForms },
   { 0x00fb3e, 1, 34 SEP Lo_AlphabeticPresentationForms },
   { 0x00fb40, 2, 34 SEP Lo_AlphabeticPresentationForms },
   { 0x00fb43, 2, 34 SEP Lo_AlphabeticPresentationForms },
   { 0x00fb46, 108, 108 SEP Lo },
   { 0x00fbd3, 363, 493 SEP Lo_ArabicPresentationForms_A },
   { 0x00fd3e, 1, 1 SEP Ps_ArabicPresentationForms_A },
   { 0x00fd3f, 1, 1 SEP Pe_ArabicPresentationForms_A },
   { 0x00fd50, 64, 493 SEP Lo_ArabicPresentationForms_A },
   { 0x00fd92, 54, 493 SEP Lo_ArabicPresentationForms_A },
   { 0x00fdf0, 12, 493 SEP Lo_ArabicPresentationForms_A },
   { 0x00fdfc, 1, 1 SEP Sc_ArabicPresentationForms_A },
   { 0x00fe00, 16, 16 SEP Mn_VariationSelectors },
   { 0x00fe20, 4, 4 SEP Mn_CombiningHalfMarks },
   { 0x00fe30, 1, 7 SEP Po_CJKCompatibilityForms },
   { 0x00fe31, 2, 2 SEP Pd_CJKCompatibilityForms },
   { 0x00fe33, 2, 5 SEP Pc_CJKCompatibilityForms },
   { 0x00fe35, 1, 8 SEP Ps_CJKCompatibilityForms },
   { 0x00fe36, 1, 8 SEP Pe_CJKCompatibilityForms },
   { 0x00fe37, 1, 8 SEP Ps_CJKCompatibilityForms },
   { 0x00fe38, 1, 8 SEP Pe_CJKCompatibilityForms },
   { 0x00fe39, 1, 8 SEP Ps_CJKCompatibilityForms },
   { 0x00fe3a, 1, 8 SEP Pe_CJKCompatibilityForms },
   { 0x00fe3b, 1, 8 SEP Ps_CJKCompatibilityForms },
   { 0x00fe3c, 1, 8 SEP Pe_CJKCompatibilityForms },
   { 0x00fe3d, 1, 8 SEP Ps_CJKCompatibilityForms },
   { 0x00fe3e, 1, 8 SEP Pe_CJKCompatibilityForms },
   { 0x00fe3f, 1, 8 SEP Ps_CJKCompatibilityForms },
   { 0x00fe40, 1, 8 SEP Pe_CJKCompatibilityForms },
   { 0x00fe41, 1, 8 SEP Ps_CJKCompatibilityForms },
   { 0x00fe42, 1, 8 SEP Pe_CJKCompatibilityForms },
   { 0x00fe43, 1, 8 SEP Ps_CJKCompatibilityForms },
   { 0x00fe44, 1, 8 SEP Pe_CJKCompatibilityForms },
   { 0x00fe45, 2, 7 SEP Po_CJKCompatibilityForms },
   { 0x00fe49, 4, 7 SEP Po_CJKCompatibilityForms },
   { 0x00fe4d, 3, 5 SEP Pc_CJKCompatibilityForms },
   { 0x00fe50, 3, 13 SEP Po_SmallFormVariants },
   { 0x00fe54, 4, 13 SEP Po_SmallFormVariants },
   { 0x00fe58, 1, 2 SEP Pd_SmallFormVariants },
   { 0x00fe59, 1, 3 SEP Ps_SmallFormVariants },
   { 0x00fe5a, 1, 3 SEP Pe_SmallFormVariants },
   { 0x00fe5b, 1, 3 SEP Ps_SmallFormVariants },
   { 0x00fe5c, 1, 3 SEP Pe_SmallFormVariants },
   { 0x00fe5d, 1, 3 SEP Ps_SmallFormVariants },
   { 0x00fe5e, 1, 3 SEP Pe_SmallFormVariants },
   { 0x00fe5f, 3, 13 SEP Po_SmallFormVariants },
   { 0x00fe62, 1, 4 SEP Sm_SmallFormVariants },
   { 0x00fe63, 1, 2 SEP Pd_SmallFormVariants },
   { 0x00fe64, 3, 4 SEP Sm_SmallFormVariants },
   { 0x00fe68, 1, 13 SEP Po_SmallFormVariants },
   { 0x00fe69, 1, 1 SEP Sc_SmallFormVariants },
   { 0x00fe6a, 2, 13 SEP Po_SmallFormVariants },
   { 0x00fe70, 5, 140 SEP Lo_ArabicPresentationForms_B },
   { 0x00fe76, 135, 140 SEP Lo_ArabicPresentationForms_B },
   { 0x00feff, 1, 1 SEP Cf_ArabicPresentationForms_B },
   { 0x00ff01, 3, 17 SEP Po_HalfwidthandFullwidthForms },
   { 0x00ff04, 1, 5 SEP Sc_HalfwidthandFullwidthForms },
   { 0x00ff05, 3, 17 SEP Po_HalfwidthandFullwidthForms },
   { 0x00ff08, 1, 5 SEP Ps_HalfwidthandFullwidthForms },
   { 0x00ff09, 1, 5 SEP Pe_HalfwidthandFullwidthForms },
   { 0x00ff0a, 1, 17 SEP Po_HalfwidthandFullwidthForms },
   { 0x00ff0b, 1, 11 SEP Sm_HalfwidthandFullwidthForms },
   { 0x00ff0c, 1, 17 SEP Po_HalfwidthandFullwidthForms },
   { 0x00ff0d, 1, 1 SEP Pd_HalfwidthandFullwidthForms },
   { 0x00ff0e, 2, 17 SEP Po_HalfwidthandFullwidthForms },
   { 0x00ff10, 10, 10 SEP Nd_HalfwidthandFullwidthForms },
   { 0x00ff1a, 2, 17 SEP Po_HalfwidthandFullwidthForms },
   { 0x00ff1c, 3, 11 SEP Sm_HalfwidthandFullwidthForms },
   { 0x00ff1f, 2, 17 SEP Po_HalfwidthandFullwidthForms },
   { 0x00ff21, 26, 26 SEP Lu_HalfwidthandFullwidthForms },
   { 0x00ff3b, 1, 5 SEP Ps_HalfwidthandFullwidthForms },
   { 0x00ff3c, 1, 17 SEP Po_HalfwidthandFullwidthForms },
   { 0x00ff3d, 1, 5 SEP Pe_HalfwidthandFullwidthForms },
   { 0x00ff3e, 1, 3 SEP Sk_HalfwidthandFullwidthForms },
   { 0x00ff3f, 1, 2 SEP Pc_HalfwidthandFullwidthForms },
   { 0x00ff40, 1, 3 SEP Sk_HalfwidthandFullwidthForms },
   { 0x00ff41, 26, 26 SEP Ll_HalfwidthandFullwidthForms },
   { 0x00ff5b, 1, 5 SEP Ps_HalfwidthandFullwidthForms },
   { 0x00ff5c, 1, 11 SEP Sm_HalfwidthandFullwidthForms },
   { 0x00ff5d, 1, 5 SEP Pe_HalfwidthandFullwidthForms },
   { 0x00ff5e, 1, 11 SEP Sm_HalfwidthandFullwidthForms },
   { 0x00ff5f, 1, 5 SEP Ps_HalfwidthandFullwidthForms },
   { 0x00ff60, 1, 5 SEP Pe_HalfwidthandFullwidthForms },
   { 0x00ff61, 1, 17 SEP Po_HalfwidthandFullwidthForms },
   { 0x00ff62, 1, 5 SEP Ps_HalfwidthandFullwidthForms },
   { 0x00ff63, 1, 5 SEP Pe_HalfwidthandFullwidthForms },
   { 0x00ff64, 1, 17 SEP Po_HalfwidthandFullwidthForms },
   { 0x00ff65, 1, 2 SEP Pc_HalfwidthandFullwidthForms },
   { 0x00ff66, 10, 107 SEP Lo_HalfwidthandFullwidthForms },
   { 0x00ff70, 1, 3 SEP Lm_HalfwidthandFullwidthForms },
   { 0x00ff71, 45, 107 SEP Lo_HalfwidthandFullwidthForms },
   { 0x00ff9e, 2, 3 SEP Lm_HalfwidthandFullwidthForms },
   { 0x00ffa0, 31, 107 SEP Lo_HalfwidthandFullwidthForms },
   { 0x00ffc2, 6, 107 SEP Lo_HalfwidthandFullwidthForms },
   { 0x00ffca, 6, 107 SEP Lo_HalfwidthandFullwidthForms },
   { 0x00ffd2, 6, 107 SEP Lo_HalfwidthandFullwidthForms },
   { 0x00ffda, 3, 107 SEP Lo_HalfwidthandFullwidthForms },
   { 0x00ffe0, 2, 5 SEP Sc_HalfwidthandFullwidthForms },
   { 0x00ffe2, 1, 11 SEP Sm_HalfwidthandFullwidthForms },
   { 0x00ffe3, 1, 3 SEP Sk_HalfwidthandFullwidthForms },
   { 0x00ffe4, 1, 4 SEP So_HalfwidthandFullwidthForms },
   { 0x00ffe5, 2, 5 SEP Sc_HalfwidthandFullwidthForms },
   { 0x00ffe8, 1, 4 SEP So_HalfwidthandFullwidthForms },
   { 0x00ffe9, 4, 11 SEP Sm_HalfwidthandFullwidthForms },
   { 0x00ffed, 2, 4 SEP So_HalfwidthandFullwidthForms },
   { 0x00fff9, 3, 3 SEP Cf_Specials },
   { 0x00fffc, 2, 2 SEP So_Specials },
   { 0x010300, 31, 31 SEP Lo_OldItalic },
   { 0x010320, 4, 4 SEP No_OldItalic },
   { 0x010330, 26, 26 SEP Lo_Gothic },
   { 0x01034a, 1, 1 SEP Nl_Gothic },
   { 0x010400, 38, 38 SEP Lu_Deseret },
   { 0x010428, 38, 38 SEP Ll_Deseret },
   { 0x01d000, 246, 246 SEP So_ByzantineMusicalSymbols },
   { 0x01d100, 39, 181 SEP So_MusicalSymbols },
   { 0x01d12a, 59, 181 SEP So_MusicalSymbols },
   { 0x01d165, 2, 8 SEP Mc_MusicalSymbols },
   { 0x01d167, 3, 22 SEP Mn_MusicalSymbols },
   { 0x01d16a, 3, 181 SEP So_MusicalSymbols },
   { 0x01d16d, 6, 8 SEP Mc_MusicalSymbols },
   { 0x01d173, 8, 8 SEP Cf_MusicalSymbols },
   { 0x01d17b, 8, 22 SEP Mn_MusicalSymbols },
   { 0x01d183, 2, 181 SEP So_MusicalSymbols },
   { 0x01d185, 7, 22 SEP Mn_MusicalSymbols },
   { 0x01d18c, 30, 181 SEP So_MusicalSymbols },
   { 0x01d1aa, 4, 22 SEP Mn_MusicalSymbols },
   { 0x01d1ae, 48, 181 SEP So_MusicalSymbols },
   { 0x01d400, 26, 443 SEP Lu_MathematicalAlphanumericSymbols },
   { 0x01d41a, 26, 488 SEP Ll_MathematicalAlphanumericSymbols },
   { 0x01d434, 26, 443 SEP Lu_MathematicalAlphanumericSymbols },
   { 0x01d44e, 7, 488 SEP Ll_MathematicalAlphanumericSymbols },
   { 0x01d456, 18, 488 SEP Ll_MathematicalAlphanumericSymbols },
   { 0x01d468, 26, 443 SEP Lu_MathematicalAlphanumericSymbols },
   { 0x01d482, 26, 488 SEP Ll_MathematicalAlphanumericSymbols },
   { 0x01d49c, 1, 443 SEP Lu_MathematicalAlphanumericSymbols },
   { 0x01d49e, 2, 443 SEP Lu_MathematicalAlphanumericSymbols },
   { 0x01d4a2, 1, 443 SEP Lu_MathematicalAlphanumericSymbols },
   { 0x01d4a5, 2, 443 SEP Lu_MathematicalAlphanumericSymbols },
   { 0x01d4a9, 4, 443 SEP Lu_MathematicalAlphanumericSymbols },
   { 0x01d4ae, 8, 443 SEP Lu_MathematicalAlphanumericSymbols },
   { 0x01d4b6, 4, 488 SEP Ll_MathematicalAlphanumericSymbols },
   { 0x01d4bb, 1, 488 SEP Ll_MathematicalAlphanumericSymbols },
   { 0x01d4bd, 4, 488 SEP Ll_MathematicalAlphanumericSymbols },
   { 0x01d4c2, 2, 488 SEP Ll_MathematicalAlphanumericSymbols },
   { 0x01d4c5, 11, 488 SEP Ll_MathematicalAlphanumericSymbols },
   { 0x01d4d0, 26, 443 SEP Lu_MathematicalAlphanumericSymbols },
   { 0x01d4ea, 26, 488 SEP Ll_MathematicalAlphanumericSymbols },
   { 0x01d504, 2, 443 SEP Lu_MathematicalAlphanumericSymbols },
   { 0x01d507, 4, 443 SEP Lu_MathematicalAlphanumericSymbols },
   { 0x01d50d, 8, 443 SEP Lu_MathematicalAlphanumericSymbols },
   { 0x01d516, 7, 443 SEP Lu_MathematicalAlphanumericSymbols },
   { 0x01d51e, 26, 488 SEP Ll_MathematicalAlphanumericSymbols },
   { 0x01d538, 2, 443 SEP Lu_MathematicalAlphanumericSymbols },
   { 0x01d53b, 4, 443 SEP Lu_MathematicalAlphanumericSymbols },
   { 0x01d540, 5, 443 SEP Lu_MathematicalAlphanumericSymbols },
   { 0x01d546, 1, 443 SEP Lu_MathematicalAlphanumericSymbols },
   { 0x01d54a, 7, 443 SEP Lu_MathematicalAlphanumericSymbols },
   { 0x01d552, 26, 488 SEP Ll_MathematicalAlphanumericSymbols },
   { 0x01d56c, 26, 443 SEP Lu_MathematicalAlphanumericSymbols },
   { 0x01d586, 26, 488 SEP Ll_MathematicalAlphanumericSymbols },
   { 0x01d5a0, 26, 443 SEP Lu_MathematicalAlphanumericSymbols },
   { 0x01d5ba, 26, 488 SEP Ll_MathematicalAlphanumericSymbols },
   { 0x01d5d4, 26, 443 SEP Lu_MathematicalAlphanumericSymbols },
   { 0x01d5ee, 26, 488 SEP Ll_MathematicalAlphanumericSymbols },
   { 0x01d608, 26, 443 SEP Lu_MathematicalAlphanumericSymbols },
   { 0x01d622, 26, 488 SEP Ll_MathematicalAlphanumericSymbols },
   { 0x01d63c, 26, 443 SEP Lu_MathematicalAlphanumericSymbols },
   { 0x01d656, 26, 488 SEP Ll_MathematicalAlphanumericSymbols },
   { 0x01d670, 26, 443 SEP Lu_MathematicalAlphanumericSymbols },
   { 0x01d68a, 26, 488 SEP Ll_MathematicalAlphanumericSymbols },
   { 0x01d6a8, 25, 443 SEP Lu_MathematicalAlphanumericSymbols },
   { 0x01d6c1, 1, 10 SEP Sm_MathematicalAlphanumericSymbols },
   { 0x01d6c2, 25, 488 SEP Ll_MathematicalAlphanumericSymbols },
   { 0x01d6db, 1, 10 SEP Sm_MathematicalAlphanumericSymbols },
   { 0x01d6dc, 6, 488 SEP Ll_MathematicalAlphanumericSymbols },
   { 0x01d6e2, 25, 443 SEP Lu_MathematicalAlphanumericSymbols },
   { 0x01d6fb, 1, 10 SEP Sm_MathematicalAlphanumericSymbols },
   { 0x01d6fc, 25, 488 SEP Ll_MathematicalAlphanumericSymbols },
   { 0x01d715, 1, 10 SEP Sm_MathematicalAlphanumericSymbols },
   { 0x01d716, 6, 488 SEP Ll_MathematicalAlphanumericSymbols },
   { 0x01d71c, 25, 443 SEP Lu_MathematicalAlphanumericSymbols },
   { 0x01d735, 1, 10 SEP Sm_MathematicalAlphanumericSymbols },
   { 0x01d736, 25, 488 SEP Ll_MathematicalAlphanumericSymbols },
   { 0x01d74f, 1, 10 SEP Sm_MathematicalAlphanumericSymbols },
   { 0x01d750, 6, 488 SEP Ll_MathematicalAlphanumericSymbols },
   { 0x01d756, 25, 443 SEP Lu_MathematicalAlphanumericSymbols },
   { 0x01d76f, 1, 10 SEP Sm_MathematicalAlphanumericSymbols },
   { 0x01d770, 25, 488 SEP Ll_MathematicalAlphanumericSymbols },
   { 0x01d789, 1, 10 SEP Sm_MathematicalAlphanumericSymbols },
   { 0x01d78a, 6, 488 SEP Ll_MathematicalAlphanumericSymbols },
   { 0x01d790, 25, 443 SEP Lu_MathematicalAlphanumericSymbols },
   { 0x01d7a9, 1, 10 SEP Sm_MathematicalAlphanumericSymbols },
   { 0x01d7aa, 25, 488 SEP Ll_MathematicalAlphanumericSymbols },
   { 0x01d7c3, 1, 10 SEP Sm_MathematicalAlphanumericSymbols },
   { 0x01d7c4, 6, 488 SEP Ll_MathematicalAlphanumericSymbols },
   { 0x01d7ce, 50, 50 SEP Nd_MathematicalAlphanumericSymbols },
   { 0x020000, 1, 2 SEP Lo_CJKUnifiedIdeographsExtensionB },
   { 0x02a6d6, 1, 2 SEP Lo_CJKUnifiedIdeographsExtensionB },
   { 0x02f800, 542, 542 SEP Lo_CJKCompatibilityIdeographsSupplement },
   { 0x0e0001, 1, 97 SEP Cf_Tags },
   { 0x0e0020, 96, 97 SEP Cf_Tags },
   { 0x0f0000, 1, 2 SEP Co_SupplementaryPrivateUseArea_A },
   { 0x0ffffd, 1, 2 SEP Co_SupplementaryPrivateUseArea_A },
   { 0x100000, 1, 2 SEP Co_SupplementaryPrivateUseArea_B },
   { 0x10fffd, 1, 2 SEP Co_SupplementaryPrivateUseArea_B },
};

#define kNumRanges (sizeof(sUnicodeSpace)/sizeof(sUnicodeSpace[0]))


static PGPUInt32 sGetCategory( PGPUInt32 c )  {
	int i_l = 0;
	int i_r = sizeof(sUnicodeSpace)/sizeof(sUnicodeSpace[0]);
	int i=i = (i_r-i_l)/2;

	while( i_l < i_r-1 )  {
		if( c < sUnicodeSpace[i].base )  {
			i_r = i;
			i = (i_l+i)/2;
			continue;
		}
		if( c > sUnicodeSpace[i].base + sUnicodeSpace[i].length - 1 )  {
			i_l = i;
			i = (i+i_r)/2;
			continue;
		}
		
		/* c==sUnicodeSpace[i].base */
		return i;
	}

	/* here we have one of 3 possibilities: 'c' is in [i_l], in [i_r], or in between, which means 
	   that we don't recognize this character */

	if( c >= sUnicodeSpace[i_l].base && c < sUnicodeSpace[i_l].base+sUnicodeSpace[i_l].length )  {
		return i_l;
	}

	if( c >= sUnicodeSpace[i_r].base && c < sUnicodeSpace[i_r].base+sUnicodeSpace[i_r].length )  {
		return i_r;
	}
	
	return -1;	/* not found */
}

static PGPUInt32 sGetCategoryLength( PGPUInt32 c )  {
	PGPUInt32 index = sGetCategory( c );

	if( index != -1 )
		return sUnicodeSpace[index].total_length;
	else
		return OUT_OF_RANGE_LENGTH;
}

/* returns number of bits in v */
static PGPInt32 sLog2( PGPUInt32 v )  {
	PGPUInt32 mask = (PGPInt32)(-1);
	PGPUInt32 mask1 = 1;
	int log=0;

	if( !v )
		return 0;

	while( v & mask )  {
		mask <<= 1;
		mask1 <<= 1;
		log++;
	}

	/* round */
	mask1 >>= 1;	/* while executed at least once */

    if( v - mask1 < mask1/2 )
		log--;

	return log;
}

static PGPUInt32 pgpEstimateMinEntropyBits( const PGPUInt16 *passphrase, PGPUInt32 length )  {
	PGPUInt32 i;
	PGPUInt32 total_entropy=0;

	for( i=0; i<length; i++ )  
		total_entropy += sLog2( sGetCategoryLength( (PGPUInt32)passphrase[i] ) );

	return total_entropy;
}

/* or pre-8.0 way to estimate the entropy */
static PGPUInt32 pgpEstimateMaxEntropyBits( const PGPUInt16 *passphrase, PGPUInt32 length )  {
	PGPUInt32 i;
	PGPUInt32 scope=0;
	PGPUInt32 index;
	int num_unrecognized_ranges=0;
	PGPBoolean haveRanges[kNumRanges];
	PGPUInt32 max_index=0;

	pgpClearMemory( haveRanges, sizeof(haveRanges) );

	for( i=0; i<length; i++ )  {
		index = sGetCategory( (PGPUInt32)passphrase[i] );
		if( index != -1 )  {
			haveRanges[index] = TRUE;
			if( index > max_index )
				max_index = index;
		}
		else
			num_unrecognized_ranges++;
	}

	for( i=0; i<=max_index; i++ )  {
		if( haveRanges[i] )  {
			scope += sUnicodeSpace[i].total_length;
			haveRanges[i] = FALSE;
		}
	}
	scope += OUT_OF_RANGE_LENGTH*num_unrecognized_ranges;

	return length * sLog2( scope );
}

	PGPUInt32
pgpEstimatePassphraseQuality(
	const char *passphrase)
{
	const PGPUInt32 length = strlen( passphrase );
	PGPUInt16 wpassphrase[256];
	PGPUInt32 wlength=0;
	PGPUInt32 min_entropy;
	PGPUInt32 d;

#if defined(PGP_WIN32) || defined(PGP_OSX)
	pgpUTF8StringToUCS2 ( passphrase, length,
		wpassphrase, sizeof(wpassphrase)/sizeof(wpassphrase[0]), &wlength );
#else
	/* use equvalent of 
	mbstowcs( wpassphrase, passphrase, sizeof(wpassphrase)/sizeof(wpassphrase[0]) );
	wlength = wcslen( wpassphrase );
	*/
	(void)wpassphrase;
#endif

	if( wlength )  {
		min_entropy = pgpEstimateMinEntropyBits(wpassphrase, wlength);
		d = pgpEstimateMaxEntropyBits(wpassphrase, wlength);
		if( d < min_entropy )  {
			pgpAssert( 0 );	/* should not happen */
			d = 0;
		}
		else
			d -= min_entropy;
	}

	pgpClearMemory( wpassphrase, wlength*sizeof(wpassphrase[0]) );

	/* consider the real entropy the average of two; 
	   assume a maximum quality of 128 bits	

	   if we could not determine the length of the wide character string, 
	   divide the number of bytes by 2 as at least 2 bytes will be used for 
	   each unrecognized character
	*/

	d = wlength ? (min_entropy + d/2) : (OUT_OF_RANGE_LENGTH * length/2);

	return ( d > 128 ) ? 100 : (d * 100 / 128);
}
