/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.
	
	$Id: PGPframework.mm,v 1.17 2002/10/29 05:25:27 jeffc Exp $
____________________________________________________________________________*/

#import <mach/mach.h>

#import "PGP.h"
#import "pgpContext.h"
#import "pgpDebug.h"
#import "pgpKeyPriv.h"
#import "pgpPFLErrors.h"
#import "pgpRPCBackEndAPI.h"
#import "pgpRPCMsg.h"
#import "PGPserviceProtocol.h"
#import "PGPframework.h"


NSString * const	kPGPserviceID = @"com.pgp.PGPservice";
NSString * const	kPGPserviceName = @"PGPservice";
NSString * const	PGPsdkNotificationPassphraseCacheChanged = @"com.pgp.notification.PGPsdkNotificationPassphraseCacheChanged";
NSString * const	PGPsdkNotificationKeyDBChanged = @"com.pgp.notification.PGPsdkNotificationKeyDBChanged";
NSString * const	PGPsdkNotificationReasonKey = @"com.pgp.notification.PGPsdkNotificationReasonKey";
NSString * const	PGPsdkNotificationParam1Key = @"com.pgp.notification.PGPsdkNotificationValue1Key";
NSString * const	PGPsdkNotificationParam2Key = @"com.pgp.notification.PGPsdkNotificationValue2Key";
NSString * const	kDistributedNotificationQueueKey = @"com.pgp.framework.kDistributedNotificationQueueKey";

static id					sService = NULL;
static NSMachPort *			sTask = NULL;
static CPGPframework *		sFramework = NULL;



@implementation CPGPframework

	- (id)
init
{
	[super init];
	
	return self;
}



	- (void)
dealloc
{
	[[NSDistributedNotificationCenter defaultCenter] removeObserver:self];
	[super dealloc];
}


	- (void)
sdkNotificationInternal
	:(NSNotification *)	notification
{
	[self performSelectorOnMainThread:@selector(sdkNotification:) withObject:nil waitUntilDone:NO];
}


	- (void)
sdkNotification
	:(NSNotification *)	notification
{
//NSLog(@"Received sdk notification: %@", [[notification name] pathExtension]);
	pgpCallNotificationCallback(
		(PGPNotificationReason) [[[notification userInfo] objectForKey:PGPsdkNotificationReasonKey] intValue],
		[[[notification userInfo] objectForKey:PGPsdkNotificationParam1Key] unsignedIntValue],
		[[[notification userInfo] objectForKey:PGPsdkNotificationParam2Key] unsignedIntValue]);
}


	- (void)
run
	:(id)	inUnused
{
        [[NSDistributedNotificationCenter defaultCenter] addObserver:self selector:@selector(sdkNotificationInternal:)
                name:PGPsdkNotificationPassphraseCacheChanged object:NULL suspensionBehavior:NSNotificationSuspensionBehaviorCoalesce];
        [[NSDistributedNotificationCenter defaultCenter] addObserver:self selector:@selector(sdkNotificationInternal:)
                name:PGPsdkNotificationKeyDBChanged object:NULL suspensionBehavior:NSNotificationSuspensionBehaviorCoalesce];
		
		[[NSRunLoop currentRunLoop] run];
}

@end



	PGPBoolean
pgpRPCEnabled(void)
{
	return (sService != NULL);
}



	void
pgpNotifyClient(
	PGPUInt32				connectref,
	PGPNotificationReason	param)
{
	NSMutableDictionary *	threadDictionary = [[NSThread currentThread] threadDictionary];
	NSNotificationQueue *	queue;
	NSString *				name;
	
	queue = [threadDictionary objectForKey:kDistributedNotificationQueueKey];
//NSLog(@"found queue %p", queue);
	if (! queue) {
		queue = [[NSNotificationQueue alloc] initWithNotificationCenter:[NSDistributedNotificationCenter defaultCenter]];
		[threadDictionary setObject:queue forKey:kDistributedNotificationQueueKey];
//NSLog(@"created queue %p", queue);
	}
	switch (param) {
		case kPGPNotification_KeyDBChanged:
		{
			name = PGPsdkNotificationKeyDBChanged;
		}
		break;
		
		
		case kPGPNotification_PassphraseCacheChanged:
		{
			name = PGPsdkNotificationPassphraseCacheChanged;
		}
		break;
		
		
		default:
		{
			name = NULL;
			NSCAssert(TRUE, @"Unhandled case in pgpNotifiyClient");
		}
		break;
	}
	[queue enqueueNotification:[NSNotification notificationWithName:name object:NULL
		userInfo:[NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithInt:param], PGPsdkNotificationReasonKey,
		[NSNumber numberWithUnsignedInt:0], PGPsdkNotificationParam1Key, [NSNumber numberWithUnsignedInt:0],
		PGPsdkNotificationParam2Key, NULL]] postingStyle:NSPostNow];
//NSLog(@"Sent sdk %@ to %d", [name pathExtension], connectref);
}



	PGPError
pgpSDKInitBackEnd(
	PGPUInt32	backEndAPIVersion)
{
	PGPError			result = kPGPError_NoErr;
	NSAutoreleasePool *	pool = [[NSAutoreleasePool alloc] init];
	
	if (backEndAPIVersion != kPGPsdkBackEndAPIVersion) {
		result = kPGPError_ImproperInitialization;
	}
	if (IsntPGPError(result) && (! sTask)) {
		sTask = [[NSMachPort alloc] initWithMachPort:mach_task_self()];
		if (! sTask) {
			result = kPGPError_ImproperInitialization;
		}
	}
	if (IsntPGPError(result)) {
		NSTask *	task;
		
		if (sService) {
			[sService release];
			sService = NULL;
		}
		sService = [[NSConnection rootProxyForConnectionWithRegisteredName:kPGPserviceID host:nil] retain];
		if (! sService) {
			task = [NSTask launchedTaskWithLaunchPath:[[[NSBundle bundleForClass:[CPGPframework class]]
						pathForResource:kPGPserviceName ofType:@"app"]
						stringByAppendingString:@"/Contents/MacOS/PGPservice"] arguments:[NSArray array]];
			sService = [[NSConnection rootProxyForConnectionWithRegisteredName:kPGPserviceID host:nil] retain];
			for (int i = 0; (! sService) && (i < 10); i++) {
				[NSThread sleepUntilDate:[NSDate dateWithTimeIntervalSinceNow:1]];
				sService = [[NSConnection rootProxyForConnectionWithRegisteredName:kPGPserviceID host:nil] retain];
			}
		}
		if (! sService) {
			result = kPGPError_ImproperInitialization;
		}
	}
	if (IsntPGPError(result)) {
		if (sFramework) {
			[sFramework release];
			sFramework = NULL;
		}
		sFramework = [[CPGPframework alloc] init];
		if (! sFramework) {
			result = kPGPError_ImproperInitialization;
		}
	}
	if (IsntPGPError(result)) {
		[[sService connectionForProxy] enableMultipleThreads];
		[sService setProtocolForProxy:@protocol(PGPserviceProtocol)];
		[NSThread detachNewThreadSelector:@selector(run:) toTarget:sFramework withObject:nil];	//	Enables threading
		result = pgpConnect_backRPC();
	}
	if (IsPGPError(result)) {
		[sService release];
		sService = NULL;
		[sFramework release];
		sFramework = NULL;
	}
	[pool release];
	
	return result;
}



	void
pgpSDKCleanupBackEnd(void)
{
	NSAutoreleasePool *	pool = [[NSAutoreleasePool alloc] init];

	pgpDisconnect_backRPC();
	[sService release];
	sService = NULL;
	[sFramework release];
	sFramework = NULL;
	[pool release];
}



	PGPError
pgpConnect_backRPC()
{
	NSAutoreleasePool *	pool = [[NSAutoreleasePool alloc] init];
	PGPError			result = [sService pgpConnect_back:sTask];

	[pool release];
	
	return result;
}



	PGPError
pgpReconnect_backRPC()
{
	NSAutoreleasePool *	pool = [[NSAutoreleasePool alloc] init];
	PGPError			result = [sService pgpReconnect_back:sTask];

	[pool release];
	
	return result;
}



	PGPError
pgpDisconnect_backRPC()
{
	NSAutoreleasePool *	pool = [[NSAutoreleasePool alloc] init];
	PGPError			result = [sService pgpDisconnect_back:sTask];

	[pool release];
	
	return result;
}



	PGPError
pgpFetchObjectData_backRPC(
	PGPContextRef	context,
	PGPUInt32		objID,
	PGPByte **		bufPtr,
	PGPSize *		bufSize)
{
	NSAutoreleasePool *	pool = [[NSAutoreleasePool alloc] init];
	NSData *			data;
	PGPError			result = [sService pgpFetchObjectData_back:objID bufPtr:&data task:sTask];
	
	if (IsntPGPError(result)) {
		*bufSize = [data length];
		*bufPtr = (PGPByte *) PGPNewData(PGPPeekContextMemoryMgr(context), *bufSize, kPGPMemoryMgrFlags_None);
		if (*bufPtr) {
			[data getBytes:*bufPtr];
		} else {
			result = kPGPError_OutOfMemory;
		}
	}
	[pool release];
	
	return result;
}



	PGPError
pgpGetKeyByKeyID_backRPC(
	PGPContextRef		context,
	PGPUInt32			dbid,
	PGPKeyID const *	keyIDIn,
	PGPBoolean			dummyOK,
	PGPBoolean			deletedOK,
	PGPUInt32 *			outID)
{
	NSAutoreleasePool *	pool = [[NSAutoreleasePool alloc] init];
	PGPError			result = [sService pgpGetKeyByKeyID_back:dbid keyIDIn:[NSData dataWithBytes:keyIDIn
									length:sizeof(PGPKeyID)] dummyOK:dummyOK deletedOK:deletedOK outID:outID task:sTask];

	[pool release];
	
	return result;
}



	PGPError
pgpKeyEncrypt_backRPC(
	PGPContextRef				context,
	PGPUInt32					objID,
	PGPByte const *				inbuf,
	PGPSize						inbuflen,
	PGPPublicKeyMessageFormat	format,
	PGPByte **					outbuf,
	PGPSize *					outbuflen)
{
	NSAutoreleasePool *	pool = [[NSAutoreleasePool alloc] init];
	NSData *			data;
	PGPError			result = [sService pgpKeyEncrypt_back:objID inbuf:[NSData dataWithBytes:inbuf length:inbuflen]
									format:format outbuf:&data task:sTask];
	
	if (IsntPGPError(result)) {
		*outbuflen = [data length];
		*outbuf = (PGPByte *) PGPNewData(PGPPeekContextMemoryMgr(context), *outbuflen, kPGPMemoryMgrFlags_None);
		if (*outbuf) {
			[data getBytes:*outbuf];
		} else {
			result = kPGPError_OutOfMemory;
		}
	}
	[pool release];
	
	return result;
}



	PGPError
pgpKeyDecrypt_backRPC(
	PGPContextRef				context,
	PGPUInt32					objID,
	const PGPByte *				passphrase,
	PGPSize						pplen,
	PGPBoolean					hashedPhrase,
	PGPUInt32					cacheTimeOut,
	PGPBoolean					cacheGlobal,
	const PGPByte *				inbuf,
	PGPSize						inbuflen,
	PGPPublicKeyMessageFormat	format,
	PGPByte **					outbuf,
	PGPSize *					outbuflen)
{
	NSAutoreleasePool *	pool = [[NSAutoreleasePool alloc] init];
	NSData *			data;
	PGPError			result = [sService pgpKeyDecrypt_back:objID passphrase:[NSData dataWithBytes:passphrase length:pplen]
									hashedPhrase:hashedPhrase cacheTimeOut:cacheTimeOut cacheGlobal:cacheGlobal
									inbuf:[NSData dataWithBytes:inbuf length:inbuflen] format:format outbuf:&data
									task:sTask];
	
	if (IsntPGPError(result)) {
		*outbuflen = [data length];
		*outbuf = (PGPByte *) PGPNewData(PGPPeekContextMemoryMgr(context), *outbuflen, kPGPMemoryMgrFlags_None);
		if (*outbuf) {
			[data getBytes:*outbuf];
		} else {
			result = kPGPError_OutOfMemory;
		}
	}
	[pool release];
	
	return result;
}



	PGPInt32
pgpKeyVerify_backRPC(
	PGPContextRef				context,
	PGPUInt32					objID,
	PGPByte const *				inbuf,
	PGPSize						inbuflen,
	PGPHashAlgorithm			hashalg,
	PGPByte const *				hash,
	PGPSize						hashlen,
	PGPPublicKeyMessageFormat	format)
{
	NSAutoreleasePool *	pool = [[NSAutoreleasePool alloc] init];
	PGPInt32			result = [sService pgpKeyVerify_back:objID inbuf:[NSData dataWithBytes:inbuf length:inbuflen]
									hashalg:hashalg hash:[NSData dataWithBytes:hash length:hashlen] format:format
									task:sTask];

	[pool release];
	
	return result;
}



	PGPError
pgpKeySign_backRPC(
	PGPContextRef				context,
	PGPUInt32					objID,
	PGPByte const *				passphrase,
	PGPSize						pplen,
	PGPBoolean					hashedPhrase,
	PGPUInt32					cacheTimeOut,
	PGPBoolean					cacheGlobal,
	PGPHashAlgorithm			hashalg,
	PGPByte const *				hash,
	PGPSize						hashlen,
	PGPPublicKeyMessageFormat	format,
	PGPByte **					outbuf,
	PGPSize *					outbuflen)
{
	NSAutoreleasePool *	pool = [[NSAutoreleasePool alloc] init];
	NSData *			data;
	PGPError			result = [sService pgpKeySign_back:objID passphrase:[NSData dataWithBytes:passphrase length:pplen]
									hashedPhrase:hashedPhrase cacheTimeOut:cacheTimeOut cacheGlobal:cacheGlobal hashalg:hashalg
									hash:[NSData dataWithBytes:hash length:hashlen] format:format outbuf:&data task:sTask];
	
	if (IsntPGPError(result)) {
		*outbuflen = [data length];
		*outbuf = (PGPByte *) PGPNewData(PGPPeekContextMemoryMgr(context), *outbuflen, kPGPMemoryMgrFlags_None);
		if (*outbuf) {
			[data getBytes:*outbuf];
		} else {
			result = kPGPError_OutOfMemory;
		}
	}
	[pool release];
	
	return result;
}



	PGPBoolean
pgpSecPassphraseOK_backRPC(
	PGPContextRef	context,
	PGPUInt32		objID,
	PGPByte const *	passphrase,
	PGPSize			pplen,
	PGPBoolean		hashedPhrase,
	PGPUInt32		cacheTimeOut,
	PGPBoolean		cacheGlobal)
{
	NSAutoreleasePool *	pool = [[NSAutoreleasePool alloc] init];
	PGPBoolean			result = [sService pgpSecPassphraseOK_back:objID passphrase:[NSData dataWithBytes:passphrase
									length:pplen] hashedPhrase:hashedPhrase cacheTimeOut:cacheTimeOut cacheGlobal:cacheGlobal
									task:sTask];

	[pool release];
	
	return result;
}



	PGPError
pgpPurgePassphraseCache_backRPC(
	PGPContextRef	context)
{
	NSAutoreleasePool *	pool = [[NSAutoreleasePool alloc] init];
	PGPError			result = [sService pgpPurgePassphraseCache_back:sTask];

	[pool release];
	
	return result;
}



	PGPError
pgpCountCachedPassphrases_backRPC(
	PGPContextRef	context,
	PGPUInt32 *		nLocals,
	PGPUInt32 *		nGlobals,
	PGPUInt32 *		nOtherLocals)
{
	NSAutoreleasePool *	pool = [[NSAutoreleasePool alloc] init];
	PGPError			result = [sService pgpCountCachedPassphrases_back:nLocals nGlobals:nGlobals nOtherLocals:nOtherLocals
									task:sTask];

	[pool release];
	
	return result;
}



	PGPError
pgpKeyMaxSizes_backRPC(
	PGPContextRef				context,
	PGPUInt32					objID,
	PGPUInt32 *					maxEncryption,
	PGPUInt32 *					maxDecryption,
	PGPUInt32 *					maxSignature,
	PGPPublicKeyMessageFormat	format)
{
	NSAutoreleasePool *	pool = [[NSAutoreleasePool alloc] init];
	PGPUInt32			enc;
	PGPUInt32			dec;
	PGPUInt32			sig;
	PGPError			result = [sService pgpKeyMaxSizes_back:objID maxEncryption:&enc maxDecryption:&dec
									maxSignature:&sig format:format task:sTask];

	if (maxEncryption) {
		*maxEncryption = enc;
	}
	if (maxDecryption) {
		*maxDecryption = dec;
	}
	if (maxSignature) {
		*maxSignature = sig;
	}
	[pool release];
	
	return result;
}



	PGPError
pgpSecProperties_backRPC(
	PGPContextRef			context,
	PGPUInt32				objID,
	PGPBoolean *			needsPassphrase,
	PGPBoolean *			isSecretShared,
	PGPCipherAlgorithm *	lockAlg,
	PGPUInt32 *				lockBits)
{
	NSAutoreleasePool *	pool = [[NSAutoreleasePool alloc] init];
	PGPError			result = [sService pgpSecProperties_back:objID needsPassphrase:reinterpret_cast<BOOL *>(needsPassphrase)
									isSecretShared:reinterpret_cast<BOOL *>(isSecretShared) lockAlg:lockAlg lockBits:lockBits
									task:sTask];

	[pool release];
	
	return result;
}



	PGPError
pgpFetchKeyInfo_backRPC(
	PGPContextRef	context,
	PGPUInt32		objID,
	PGPByte **		bufPtr,
	PGPSize *		bufSize)
{
	NSAutoreleasePool *	pool = [[NSAutoreleasePool alloc] init];
	NSData *			data;
	PGPError			result = [sService pgpFetchKeyInfo_back:objID bufPtr:&data task:sTask];
	
	if (IsntPGPError(result)) {
		*bufSize = [data length];
		*bufPtr = (PGPByte *) PGPNewData(PGPPeekContextMemoryMgr(context), *bufSize, kPGPMemoryMgrFlags_None);
		if (*bufPtr) {
			[data getBytes:*bufPtr];
		} else {
			result = kPGPError_OutOfMemory;
		}
	}
	[pool release];
	
	return result;
}



	PGPError
pgpOpenKeyDBFile_backRPC(
	PGPContextRef			context,
	PGPOpenKeyDBFileOptions	openFlags,
	PFLFileSpecRef			pub,
	PFLFileSpecRef			priv,
	PGPUInt32 *				kdbid,
	PGPUInt32 *				numKeys,
	PGPUInt32 **			keyBuffer,
	PGPSize *				bufSize)
{
	NSAutoreleasePool *	pool = [[NSAutoreleasePool alloc] init];
	PGPByte *			pubdata = NULL;
	PGPSize				publen;
	PGPByte *			privdata = NULL;
	PGPSize				privlen;
	NSData *			data;
	PGPError			result;
	
	result = PFLExportFileSpec(pub, &pubdata, &publen);
	if (IsntPGPError(result)) {
		result = PFLExportFileSpec(priv, &privdata, &privlen);
	}
	if (IsntPGPError(result)) {
		result  = [sService pgpOpenKeyDBFile_back:openFlags pub:[NSData dataWithBytes:pubdata length:publen]
					priv:[NSData dataWithBytes:privdata length:privlen] kdbid:kdbid numKeys:numKeys
					keyBuffer:&data task:sTask];
	}
	if (IsntPGPError(result)) {
		*bufSize = [data length];
		*keyBuffer = (PGPUInt32 *) PGPNewData(PGPPeekContextMemoryMgr(context), *bufSize, kPGPMemoryMgrFlags_None);
		if (*keyBuffer) {
			[data getBytes:*keyBuffer];
		} else {
			result = kPGPError_OutOfMemory;
		}
	}
	if (pubdata) {
		PGPFreeData(pubdata);
	}
	if (privdata) {
		PGPFreeData(privdata);
	}
	[pool release];
	
	return result;
}



	PGPError
pgpImportKeyBinary_backRPC(
	PGPContextRef	context,
	const PGPByte *	buffer,
	PGPSize			length,
	PGPUInt32 *		kdbid,
	PGPUInt32 *		numKeys,
	PGPUInt32 **	keyArray,
	PGPSize *		keyArraySize)
{
	NSAutoreleasePool *	pool = [[NSAutoreleasePool alloc] init];
	NSData *			data;
	PGPError			result = [sService pgpImportKeyBinary_back:[NSData dataWithBytes:buffer length:length] kdbid:kdbid
									numKeys:numKeys keyArray:&data task:sTask];
	
	if (IsntPGPError(result)) {
		*keyArraySize = [data length];
		*keyArray = (PGPUInt32 *) PGPNewData(PGPPeekContextMemoryMgr(context), *keyArraySize, kPGPMemoryMgrFlags_None);
		if (*keyArray) {
			[data getBytes:*keyArray];
		} else {
			result = kPGPError_OutOfMemory;
		}
	}
	[pool release];
	
	return result;
}



	PGPError
pgpKeyDBFlush_backRPC(
	PGPContextRef	context,
	PGPUInt32		objID,
	PGPUInt32 **	changedkeylist,
	PGPSize *		changedkeylistsize)
{
	NSAutoreleasePool *	pool = [[NSAutoreleasePool alloc] init];
	NSData *			data;
	PGPError			result = [sService pgpKeyDBFlush_back:objID changedkeylist:&data task:sTask];
	
	if (IsntPGPError(result)) {
		*changedkeylistsize = [data length];
		*changedkeylist = (PGPUInt32 *) PGPNewData(PGPPeekContextMemoryMgr(context), *changedkeylistsize, kPGPMemoryMgrFlags_None);
		if (*changedkeylist) {
			[data getBytes:*changedkeylist];
		} else {
			result = kPGPError_OutOfMemory;
		}
	}
	[pool release];
	
	return result;
}



	void
pgpFreeKeyDB_backRPC(
	PGPContextRef	context,
	PGPUInt32		objID)
{
	NSAutoreleasePool *	pool = [[NSAutoreleasePool alloc] init];

	[sService pgpFreeKeyDB_back:objID task:sTask];
	[pool release];
}



	PGPError
pgpCacheKeyDB_backRPC(
	PGPContextRef	context,
	PGPUInt32		objID,
	PGPUInt32		timeoutSeconds)
{
	NSAutoreleasePool *	pool = [[NSAutoreleasePool alloc] init];
	PGPError			result = [sService pgpCacheKeyDB_back:objID timeoutSeconds:timeoutSeconds task:sTask];

	[pool release];
	
	return result;
}


	PGPError
pgpPurgeKeyDBCache_backRPC(
	PGPContextRef	context)
{
	NSAutoreleasePool *	pool = [[NSAutoreleasePool alloc] init];
	PGPError			result = [sService pgpPurgeKeyDBCache_back:sTask];

	[pool release];
	
	return result;
}



	PGPError
pgpSetKeyEnabled_backRPC(
	PGPContextRef	context,
	PGPUInt32		objID,
	PGPBoolean		enable)
{
	NSAutoreleasePool *	pool = [[NSAutoreleasePool alloc] init];
	PGPError			result = [sService pgpSetKeyEnabled_back:objID enable:enable task:sTask];

	[pool release];
	
	return result;
}



	PGPError
pgpSetKeyAxiomatic_backRPC(
	PGPContextRef	context,
	PGPUInt32		objID,
	PGPBoolean		setAxiomatic,
	PGPBoolean		checkPassphrase,
	char const *	passphrase,
	PGPSize			passphraseLength,
	PGPBoolean		hashedPhrase,
	PGPUInt32		cacheTimeOut,
	PGPBoolean		cacheGlobal)
{
	NSAutoreleasePool *	pool = [[NSAutoreleasePool alloc] init];
	PGPError			result = [sService pgpSetKeyAxiomatic_back:objID setAxiomatic:setAxiomatic
									checkPassphrase:checkPassphrase passphrase:[NSData dataWithBytes:passphrase
									length:passphraseLength] hashedPhrase:hashedPhrase cacheTimeOut:cacheTimeOut
									cacheGlobal:cacheGlobal task:sTask];

	[pool release];
	
	return result;
}



	PGPError
pgpPropagateTrust_backRPC(
	PGPContextRef	context,
	PGPUInt32		setid,
	PGPUInt32 *		keylist,
	PGPSize			keylistsize,
	PGPUInt32		altid,
	PGPUInt32 const	timenow,
	PGPUInt32 **	changedkeylist,
	PGPSize *		changedkeylistsize)
{
	NSAutoreleasePool *	pool = [[NSAutoreleasePool alloc] init];
	NSData *			data;
	PGPError			result = [sService pgpPropagateTrust_back:setid keylist:[NSData dataWithBytes:keylist
									length:keylistsize] altid:altid timenow:timenow changedkeylist:&data task:sTask];
	
	if (keylist) {
		PGPFreeData(keylist);
	}
	if (IsntPGPError(result)) {
		*changedkeylistsize = [data length];
		*changedkeylist = (PGPUInt32 *) PGPNewData(PGPPeekContextMemoryMgr(context), *changedkeylistsize, kPGPMemoryMgrFlags_None);
		if (*changedkeylist) {
			[data getBytes:*changedkeylist];
		} else {
			result = kPGPError_OutOfMemory;
		}
	}
	[pool release];
	
	return result;
}



	PGPError
pgpCheckKeyRingSigs_backRPC(
	PGPContextRef	context,
	PGPUInt32		setid,
	PGPUInt32 *		keylist,
	PGPSize			keylistsize,
	PGPUInt32		altid,
	PGPBoolean		checkAll,
	PGPUInt32 **	changedkeylist,
	PGPSize *		changedkeylistsize)
{
	NSAutoreleasePool *	pool = [[NSAutoreleasePool alloc] init];
	NSData *			data;
	PGPError			result = [sService pgpCheckKeyRingSigs_back:setid keylist:[NSData dataWithBytes:keylist
									length:keylistsize] altid:altid checkAll:checkAll changedkeylist:&data task:sTask];
	if (keylist) {
		PGPFreeData(keylist);
	}
	if (IsntPGPError(result)) {
		*changedkeylistsize = [data length];
		*changedkeylist = (PGPUInt32 *) PGPNewData(PGPPeekContextMemoryMgr(context), *changedkeylistsize, kPGPMemoryMgrFlags_None);
		if (*changedkeylist) {
			[data getBytes:*changedkeylist];
		} else {
			result = kPGPError_OutOfMemory;
		}
	}
	[pool release];
	
	return result;
}



	PGPError
pgpPrepareToCheckKeyRingSigs_backRPC(
	PGPContextRef	context,
	PGPUInt32		setid,
	PGPUInt32 *		keylist,
	PGPSize			keylistsize,
	PGPUInt32		altid,
	PGPBoolean		checkAll,
	PGPUInt32 *		nsigs,
	PGPUInt32 **	changedkeylist,
	PGPSize *		changedkeylistsize)
{
	NSAutoreleasePool *	pool = [[NSAutoreleasePool alloc] init];
	NSData *			data;
	PGPError			result = [sService pgpPrepareToCheckKeyRingSigs_back:setid keylist:[NSData dataWithBytes:keylist
									length:keylistsize] altid:altid checkAll:checkAll nsigs:nsigs changedkeylist:&data
									task:sTask];
	
	if (keylist) {
		PGPFreeData(keylist);
	}
	if (IsntPGPError(result)) {
		*changedkeylistsize = [data length];
		*changedkeylist = (PGPUInt32 *) PGPNewData(PGPPeekContextMemoryMgr(context), *changedkeylistsize, kPGPMemoryMgrFlags_None);
		if (*changedkeylist) {
			[data getBytes:*changedkeylist];
		} else {
			result = kPGPError_OutOfMemory;
		}
	}
	[pool release];
	
	return result;
}



	PGPError
pgpCheckSig_backRPC(
	PGPContextRef	context,
	PGPUInt32		setid,
	PGPUInt32		objid,
	PGPUInt32		altid,
	PGPBoolean		checkAll,
	PGPBoolean		revocationonly,
	PGPBoolean *	handled,
	PGPBoolean *	changed,
	PGPBoolean *	verified)
{
	NSAutoreleasePool *	pool = [[NSAutoreleasePool alloc] init];
	PGPError			result = [sService pgpCheckSig_back:setid objid:objid altid:altid checkAll:checkAll
									revocationonly:revocationonly handled:reinterpret_cast<BOOL *>(handled)
									changed:reinterpret_cast<BOOL *>(changed) verified:reinterpret_cast<BOOL *>(verified)
									task:sTask];

	[pool release];
	
	return result;
}



	PGPError
pgpDoGenerateKey_backRPC(
	PGPContextRef				context,
	PGPUInt32					keydbID,
	PGPUInt32					masterkeyID,
	PGPByte						pkalg,
	PGPUInt32					bits,
	PGPTime						creationDate,
	PGPUInt16					expirationDays,
	char const *				name,
	PGPSize						name_len, 
	char const *				passphrase,
	PGPSize						passphraseLength,
	PGPBoolean					passphraseIsKey,
	char const *				masterpass, 
	PGPSize						masterpassLength,
	PGPUInt32					cacheTimeOut,
	PGPBoolean					cacheGlobal,
	PGPEventHandlerProcPtr		progress,
	PGPUserValue				userValue,
	PGPBoolean					fastgen,
	PGPBoolean					checkentropy,
	PGPBoolean					usetoken,
	PGPUInt32					tokenid,
	PGPUInt32 *					adkkeylist,
	PGPSize						adkkeylistsize,
	PGPByte						adkclass,
	PGPUInt32 *					rakkeylist,
	PGPSize						rakkeylistsize,
	PGPByte						rakclass,
	PGPCipherAlgorithm const *	prefalg,
	PGPSize						prefalgLength,
	PGPByte const *				prefkeyserv,
	PGPSize						prefkeyservLength,
	PGPUInt32					keyflags,
	PGPBoolean					fkeyflags,
	PGPUInt32					keyservprefs,
	PGPBoolean					fkeyservprefs,
	PGPUInt32 **				newobjs,
	PGPSize *					newobjslength,
	PGPUInt32 *					newKey)
{
	NSAutoreleasePool *	pool = [[NSAutoreleasePool alloc] init];
	NSData *			data;
	PGPError			result = [sService pgpDoGenerateKey_back:keydbID masterkeyID:masterkeyID pkalg:pkalg bits:bits
									creationDate:creationDate expirationDays:expirationDays name:[NSData dataWithBytes:name
									length:name_len] passphrase:[NSData dataWithBytes:passphrase length:passphraseLength]
									passphraseIsKey:passphraseIsKey masterpass:[NSData dataWithBytes:masterpass
									length:masterpassLength] cacheTimeOut:cacheTimeOut cacheGlobal:cacheGlobal
									progress:progress userValue:userValue fastgen:fastgen checkentropy:checkentropy
									usetoken:usetoken tokenid:tokenid adkkeylist:[NSData dataWithBytes:adkkeylist
									length:adkkeylistsize] adkclass:adkclass rakkeylist:[NSData dataWithBytes:rakkeylist
									length:rakkeylistsize] rakclass:rakclass prefalg:[NSData dataWithBytes:prefalg
									length:prefalgLength] prefkeyserv:[NSData dataWithBytes:prefkeyserv
									length:prefkeyservLength] keyflags:keyflags fkeyflags:fkeyflags keyservprefs:keyservprefs
									fkeyservprefs:fkeyservprefs newobjs:&data newKey:newKey task:sTask];
	
	if (adkkeylist) {
		PGPFreeData(adkkeylist);
	}
	if (rakkeylist) {
		PGPFreeData(rakkeylist);
	}
	if (IsntPGPError(result)) {
		*newobjslength = [data length];
		*newobjs = (PGPUInt32 *) PGPNewData(PGPPeekContextMemoryMgr(context), *newobjslength, kPGPMemoryMgrFlags_None);
		if (*newobjs) {
			[data getBytes:*newobjs];
		} else {
			result = kPGPError_OutOfMemory;
		}
	}
	[pool release];
	
	return result;
}



	PGPError
pgpCreateKeypair_backRPC(
	PGPContextRef	context,
	PGPUInt32		keydbID,
	PGPByte *		seckeyBuf,
	PGPSize			seckeySize,
	PGPByte *		keyspecBuf,
	PGPSize			keyspecSize,
	char const *	name,
	PGPSize			namelen, 
	PGPUInt32 *		adklist,
	PGPSize			adklistsize,
	PGPByte			adkclass,
	PGPUInt32 *		raklist,
	PGPSize			raklistsize,
	PGPByte			rakclass,
	char const *	passphrase,
	PGPSize			passphraseLength,
	PGPBoolean		passphraseIsKey,
	PGPUInt32		cacheTimeOut,
	PGPBoolean		cacheGlobal,
	PGPUInt32 **	newobjs,
	PGPSize *		newobjslength,
	PGPUInt32 *		newkeyid)
{
	NSAutoreleasePool *	pool = [[NSAutoreleasePool alloc] init];
	NSData *			data;
	PGPError			result = [sService pgpCreateKeypair_back:keydbID seckeyBuf:[NSData dataWithBytes:seckeyBuf
									length:seckeySize] keyspecBuf:[NSData dataWithBytes:keyspecBuf length:keyspecSize]
									name:[NSData dataWithBytes:name length:namelen] adklist:[NSData dataWithBytes:adklist
									length:adklistsize] adkclass:adkclass raklist:[NSData dataWithBytes:raklist
									length:raklistsize] rakclass:rakclass passphrase:[NSData dataWithBytes:passphrase
									length:passphraseLength] passphraseIsKey:passphraseIsKey cacheTimeOut:cacheTimeOut
									cacheGlobal:cacheGlobal newobjs:&data newkeyid:newkeyid task:sTask];
	
	if (adklist) {
		PGPFreeData(adklist);
	}
	if (raklist) {
		PGPFreeData(raklist);
	}
	if (keyspecBuf) {
		PGPFreeData(keyspecBuf);
	}
	if (seckeyBuf) {
		PGPFreeData(seckeyBuf);
	}
	if (IsntPGPError(result)) {
		*newobjslength = [data length];
		*newobjs = (PGPUInt32 *) PGPNewData(PGPPeekContextMemoryMgr(context), *newobjslength, kPGPMemoryMgrFlags_None);
		if (*newobjs) {
			[data getBytes:*newobjs];
		} else {
			result = kPGPError_OutOfMemory;
		}
	}
	[pool release];
	
	return result;
}



	PGPError
pgpCreateSubkeypair_backRPC(
	PGPContextRef	context,
	PGPUInt32		masterkeyID,
	PGPByte *		seckeyBuf,
	PGPSize			seckeySize,
	PGPByte *		keyspecBuf,
	PGPSize			keyspecSize,
	char const *	passphrase,
	PGPSize			passphraseLength,
	PGPBoolean		passphraseIsKey,
	PGPUInt32		cacheTimeOut,
	PGPBoolean		cacheGlobal,
	PGPUInt32 **	newobjs,
	PGPSize *		newobjslength,
	PGPUInt32 *		newkeyid)
{
	NSAutoreleasePool *	pool = [[NSAutoreleasePool alloc] init];
	NSData *			data;
	PGPError			result = [sService pgpCreateSubkeypair_back:masterkeyID seckeyBuf:[NSData dataWithBytes:seckeyBuf
									length:seckeySize] keyspecBuf:[NSData dataWithBytes:keyspecBuf length:keyspecSize]
									passphrase:[NSData dataWithBytes:passphrase length:passphraseLength]
									passphraseIsKey:passphraseIsKey cacheTimeOut:cacheTimeOut cacheGlobal:cacheGlobal
									newobjs:&data newkeyid:newkeyid task:sTask];
	
	if (keyspecBuf) {
		PGPFreeData(keyspecBuf);
	}
	if (seckeyBuf) {
		PGPFreeData(seckeyBuf);
	}
	if (IsntPGPError(result)) {
		*newobjslength = [data length];
		*newobjs = (PGPUInt32 *) PGPNewData(PGPPeekContextMemoryMgr(context), *newobjslength, kPGPMemoryMgrFlags_None);
		if (*newobjs) {
			[data getBytes:*newobjs];
		} else {
			result = kPGPError_OutOfMemory;
		}
	}
	[pool release];
	
	return result;
}



	PGPError
pgpCountTokens_backRPC(
	PGPContextRef	context,
	PGPUInt32 *		numTokens)
{
	NSAutoreleasePool *	pool = [[NSAutoreleasePool alloc] init];
	PGPError			result = [sService pgpCountTokens_back:numTokens task:sTask];

	[pool release];
	
	return result;
}



   	PGPError
pgpGetTokenInfo_backRPC(
	PGPContextRef	context,
    PGPUInt32		tokNumber,
	PGPTokenInfo *	info)
{
	NSAutoreleasePool *	pool = [[NSAutoreleasePool alloc] init];
	PGPError			result = [sService pgpGetTokenInfo_back:tokNumber info:info task:sTask];

	[pool release];
	
	return result;
}



	PGPError
pgpDeleteKeyOnToken_backRPC(
	PGPContextRef		context, 
	PGPUInt32			keydbID, 
	const PGPKeyID *	keyID, 
	PGPUInt32			tokenNumber, 
	PGPByte const *		pin,
	PGPSize				pinLen)  
{
	NSAutoreleasePool *	pool = [[NSAutoreleasePool alloc] init];
	PGPError			result = [sService pgpDeleteKeyOnToken_back:keydbID keyID:[NSData dataWithBytes:keyID
									length:sizeof(PGPKeyID)] tokenNumber:tokenNumber pin:[NSData dataWithBytes:pin
									length:pinLen] task:sTask];

	[pool release];
	
	return result;
}



	PGPError
pgpWipeToken_backRPC(
	PGPContextRef	context,
	PGPUInt32 		tokNumber,
	PGPByte const *	passphrase,
	PGPSize			passphraseLength)
{
	NSAutoreleasePool *	pool = [[NSAutoreleasePool alloc] init];
	PGPError			result = [sService pgpWipeToken_back:tokNumber passphrase:[NSData dataWithBytes:passphrase
									length:passphraseLength] task:sTask];

	[pool release];
	
	return result;
}



	PGPError
pgpFormatToken_backRPC (
	PGPContextRef	context,
	PGPUInt32 		tokNumber,
	PGPByte const *	adminPin,
	PGPSize			adminPinLen, 
	PGPByte const *	newUserPin,
	PGPSize			newUserPinLen)
{
	NSAutoreleasePool *	pool = [[NSAutoreleasePool alloc] init];
	PGPError			result = [sService pgpFormatToken_back:tokNumber adminPin:[NSData dataWithBytes:adminPin
									length:adminPinLen] newUserPin:[NSData dataWithBytes:newUserPin
									length:newUserPinLen] task:sTask];

	[pool release];
	
	return result;
}



	PGPError
pgpTokenPassphraseIsValid_backRPC(
	PGPContextRef	context,
	PGPUInt32 		tokNumber,
	PGPByte const *	passphrase,
	PGPSize			passphraseLength)
{
	NSAutoreleasePool *	pool = [[NSAutoreleasePool alloc] init];
	PGPError			result = [sService pgpTokenPassphraseIsValid_back:tokNumber passphrase:[NSData dataWithBytes:passphrase
									length:passphraseLength] task:sTask];

	[pool release];
	
	return result;
}



	PGPError
pgpCopyKeyToToken_backRPC(
	PGPContextRef	context,
	PGPUInt32		objID,
	PGPUInt32		tokNumber,
	PGPBoolean		isMaster, 
	char const *	passphrase,
	PGPSize			passphraseLength,
	PGPBoolean		hashedPhrase,
	char const *	PIN,
	PGPSize			PINlength,
	PGPUInt32		cacheTimeOut,
	PGPBoolean		cacheGlobal)
{
	NSAutoreleasePool *	pool = [[NSAutoreleasePool alloc] init];
	PGPError			result = [sService pgpCopyKeyToToken_back:objID tokNumber:tokNumber isMaster:isMaster
									passphrase:[NSData dataWithBytes:passphrase length:passphraseLength]
									hashedPhrase:hashedPhrase PIN:[NSData dataWithBytes:PIN length:PINlength]
									cacheTimeOut:cacheTimeOut cacheGlobal:cacheGlobal task:sTask];

	[pool release];
	
	return result;
}



   	PGPError
pgpTokenImportX509_backRPC(
	PGPContextRef	context,
	const PGPByte *	keyID,
    const PGPByte *	userID,
	PGPSize			userID_len, 
    const PGPByte *	x509,
	PGPSize			x509_len,
    const PGPByte *	password,
	PGPSize			passwordLength)
{
	NSAutoreleasePool *	pool = [[NSAutoreleasePool alloc] init];
	PGPError			result = [sService pgpTokenImportX509_back:[NSData dataWithBytes:keyID length:sizeof(PGPKeyID)]
									userID:[NSData dataWithBytes:userID length:userID_len] x509:[NSData dataWithBytes:x509
									length:x509_len] password:[NSData dataWithBytes:password length:passwordLength] task:sTask];

	[pool release];
	
	return result;
}



	PGPError                
pgpTokenPutKeyContainer_backRPC( 
     PGPContextRef		context,
	 const PGPByte *	keyID, 
     const PGPByte *	password,
	 PGPSize			passwordLen, 
     const PGPByte *	cont,
	 PGPSize			contSize)
{
	NSAutoreleasePool *	pool = [[NSAutoreleasePool alloc] init];
	PGPError			result = [sService pgpTokenPutKeyContainer_back:[NSData dataWithBytes:keyID length:sizeof(PGPKeyID)]
									password:[NSData dataWithBytes:password length:passwordLen]
									cont:[NSData dataWithBytes:cont length:contSize] task:sTask];

	[pool release];
	
	return result;
}



	PGPError                
pgpTokenGetKeyContainer_backRPC( 
     PGPContextRef		context,
	 const PGPByte *	keyID, 
     const PGPByte *	password,
	 PGPSize			passwordSize,
     PGPByte **			contOut,
	 PGPSize *			contOutSize)
{
	NSAutoreleasePool *	pool = [[NSAutoreleasePool alloc] init];
	NSData *			data;
	PGPError			result = [sService pgpTokenGetKeyContainer_back:[NSData dataWithBytes:keyID length:sizeof(PGPKeyID)]
									password:[NSData dataWithBytes:password length:passwordSize] contOut:&data task:sTask];
	
	if (IsntPGPError(result) && contOut) {
		*contOutSize = [data length];
		*contOut = (PGPByte *) PGPNewData(PGPPeekContextMemoryMgr(context), *contOutSize, kPGPMemoryMgrFlags_None);
		if (*contOut) {
			[data getBytes:*contOut];
		} else {
			result = kPGPError_OutOfMemory;
		}
	}
	[pool release];
	
	return result;
}



    PGPError 
pgpSetPKCS11DrvFile_backRPC(
	const PGPByte *	module)  
{
	NSAutoreleasePool *	pool = [[NSAutoreleasePool alloc] init];
	PGPError			result = [sService pgpSetPKCS11DrvFile_back:[NSData dataWithBytes:module
									length:strlen(reinterpret_cast<const char *>(module)) + 1]
									task:sTask];

	[pool release];
	
	return result;
}



	PGPError
pgpAddUserID_backRPC(
	PGPContextRef		context,
	PGPUInt32			objid,
	PGPBoolean			isAttribute,
	PGPAttributeType	attributeType,
	char const *		userIDData,
	PGPSize		   		userIDLength,
	char const *		passphrase,
	PGPSize				passphraseLength,
	PGPBoolean			hashedPhrase,
	PGPUInt32			cacheTimeOut,
	PGPBoolean			cacheGlobal,
	PGPUInt32 **		newobjs,
	PGPSize *			newobjslength)

{
	NSAutoreleasePool *	pool = [[NSAutoreleasePool alloc] init];
	NSData *			data;
	PGPError			result = [sService pgpAddUserID_back:objid isAttribute:isAttribute attributeType:attributeType
									userIDData:[NSData dataWithBytes:userIDData length:userIDLength]
									passphrase:[NSData dataWithBytes:passphrase length:passphraseLength]
									hashedPhrase:hashedPhrase cacheTimeOut:cacheTimeOut cacheGlobal:cacheGlobal newobjs:&data
									task:sTask];
	
	if (IsntPGPError(result)) {
		*newobjslength = [data length];
		*newobjs = (PGPUInt32 *) PGPNewData(PGPPeekContextMemoryMgr(context), *newobjslength, kPGPMemoryMgrFlags_None);
		if (*newobjs) {
			[data getBytes:*newobjs];
		} else {
			result = kPGPError_OutOfMemory;
		}
	}
	[pool release];
	
	return result;
}



	PGPError
pgpCertifyUserID_backRPC(
	PGPContextRef	context,
	PGPUInt32		useridid,
	PGPUInt32		certifying_keyid,
	char const *	passphrase,
	PGPSize			passphraseLength,
	PGPBoolean		hashedPhrase,
	PGPUInt32		cacheTimeOut,
	PGPBoolean		cacheGlobal,
	PGPBoolean		exportable,
	PGPTime			creationDate,
	PGPUInt32		expiration,
	PGPByte			trustDepth,
	PGPByte			trustValue,
	char const *	sRegExp,
	PGPSize			sRegExpLength,
	PGPUInt32 **	newobjs,
	PGPSize *		newobjslength)
{
	NSAutoreleasePool *	pool = [[NSAutoreleasePool alloc] init];
	NSData *			data;
	PGPError			result = [sService pgpCertifyUserID_back:useridid certifying_keyid:certifying_keyid
									passphrase:[NSData dataWithBytes:passphrase length:passphraseLength]
									hashedPhrase:hashedPhrase cacheTimeOut:cacheTimeOut cacheGlobal:cacheGlobal
									exportable:exportable creationDate:creationDate expiration:expiration trustDepth:trustDepth
									trustValue:trustValue sRegExp:[NSData dataWithBytes:sRegExp length:sRegExpLength]
									newobjs:&data task:sTask];
	
	if (IsntPGPError(result)) {
		*newobjslength = [data length];
		*newobjs = (PGPUInt32 *) PGPNewData(PGPPeekContextMemoryMgr(context), *newobjslength, kPGPMemoryMgrFlags_None);
		if (*newobjs) {
			[data getBytes:*newobjs];
		} else {
			result = kPGPError_OutOfMemory;
		}
	}
	[pool release];
	
	return result;
}



	PGPError
pgpRevokeSig_backRPC(
	PGPContextRef	context,
	PGPUInt32		sigid,
	char const *	passphrase,
	PGPSize			passphraseLength,
	PGPBoolean		hashedPhrase,
	PGPUInt32		cacheTimeOut,
	PGPBoolean		cacheGlobal,
	PGPUInt32 **	newobjs,
	PGPSize *		newobjslength)
{
	NSAutoreleasePool *	pool = [[NSAutoreleasePool alloc] init];
	NSData *			data;
	PGPError			result = [sService pgpRevokeSig_back:sigid passphrase:[NSData dataWithBytes:passphrase
									length:passphraseLength] hashedPhrase:hashedPhrase cacheTimeOut:cacheTimeOut
									cacheGlobal:cacheGlobal newobjs:&data task:sTask];
	
	if (IsntPGPError(result)) {
		*newobjslength = [data length];
		*newobjs = (PGPUInt32 *) PGPNewData(PGPPeekContextMemoryMgr(context), *newobjslength, kPGPMemoryMgrFlags_None);
		if (*newobjs) {
			[data getBytes:*newobjs];
		} else {
			result = kPGPError_OutOfMemory;
		}
	}
	[pool release];
	
	return result;
}



	PGPError
pgpRevokeKey_backRPC(
	PGPContextRef		context,
	PGPUInt32			key_id,
	char const *		passphrase,
	PGPSize				passphraseLength,
	PGPBoolean			hashedPhrase,
	PGPUInt32			cacheTimeOut,
	PGPBoolean			cacheGlobal,
	PGPUInt32 **		newobjs,
	PGPSize *			newobjslength)
{
	NSAutoreleasePool *	pool = [[NSAutoreleasePool alloc] init];
	NSData *			data;
	PGPError			result = [sService pgpRevokeKey_back:key_id passphrase:[NSData dataWithBytes:passphrase
									length:passphraseLength] hashedPhrase:hashedPhrase cacheTimeOut:cacheTimeOut
									cacheGlobal:cacheGlobal newobjs:&data task:sTask];
	
	if (IsntPGPError(result)) {
		*newobjslength = [data length];
		*newobjs = (PGPUInt32 *) PGPNewData(PGPPeekContextMemoryMgr(context), *newobjslength, kPGPMemoryMgrFlags_None);
		if (*newobjs) {
			[data getBytes:*newobjs];
		} else {
			result = kPGPError_OutOfMemory;
		}
	}
	[pool release];
	
	return result;
}



	PGPError
pgpDoChangePassphrase_backRPC(
	PGPContextRef	context,
	PGPUInt32		keydb_id,
	PGPUInt32		key_id,
	PGPUInt32		masterkey_id, 
	const char *	oldphrase,
	PGPSize			oldphraseLength,
	const char *	newphrase,
	PGPSize			newphraseLength,
	PGPBoolean		newPassphraseIsKey,
	PGPUInt32		cacheTimeOut,
	PGPBoolean		cacheGlobal)
{
	NSAutoreleasePool *	pool = [[NSAutoreleasePool alloc] init];
	PGPError			result = [sService pgpDoChangePassphrase_back:keydb_id key_id:key_id masterkey_id:masterkey_id
									oldphrase:[NSData dataWithBytes:oldphrase length:oldphraseLength]
									newphrase:[NSData dataWithBytes:newphrase length:newphraseLength]
									newPassphraseIsKey:newPassphraseIsKey cacheTimeOut:cacheTimeOut cacheGlobal:cacheGlobal
									task:sTask];

	[pool release];
	
	return result;
}



	PGPError
pgpPassphraseIsValid_backRPC(
	PGPContextRef	context,
	PGPUInt32		key_id,
	const char *	passphrase,
	PGPSize			passphraseLength,
	PGPBoolean		hashedPhrase,
	PGPUInt32		cacheTimeOut,
	PGPBoolean		cacheGlobal,
	PGPBoolean *	isValid)
{
	NSAutoreleasePool *	pool = [[NSAutoreleasePool alloc] init];
	PGPError			result = [sService pgpPassphraseIsValid_back:key_id passphrase:[NSData dataWithBytes:passphrase
									length:passphraseLength] hashedPhrase:hashedPhrase cacheTimeOut:cacheTimeOut
									cacheGlobal:cacheGlobal isValid:reinterpret_cast<BOOL *>(isValid) task:sTask];

	[pool release];
	
	return result;
}



	PGPError
pgpSetKeyTrust_backRPC(
	PGPContextRef	context,
	PGPUInt32		key_id,
	PGPUInt32		trust)
{
	NSAutoreleasePool *	pool = [[NSAutoreleasePool alloc] init];
	PGPError			result = [sService pgpSetKeyTrust_back:key_id trust:trust task:sTask];

	[pool release];
	
	return result;
}



	PGPError
pgpGetPasskeyBuffer_backRPC(
	PGPContextRef		context,
	PGPUInt32			key_id,
	char const *		passphrase,
	PGPSize				passphraseLength,
	PGPByte **			passkeyBuffer,
	PGPSize *			passkeyBufferLength)
{
	NSAutoreleasePool *	pool = [[NSAutoreleasePool alloc] init];
	NSData *			data;
	PGPError			result = [sService pgpGetPasskeyBuffer_back:key_id passphrase:[NSData dataWithBytes:passphrase
									length:passphraseLength] passkeyBuffer:&data task:sTask];
	
	if (IsntPGPError(result)) {
		*passkeyBufferLength = [data length];
		*passkeyBuffer = (PGPByte *) PGPNewData(PGPPeekContextMemoryMgr(context), *passkeyBufferLength, kPGPMemoryMgrFlags_None);
		if (*passkeyBuffer) {
			[data getBytes:*passkeyBuffer];
		} else {
			result = kPGPError_OutOfMemory;
		}
	}
	[pool release];
	
	return result;
}



	PGPError
pgpAddKeyOptions_backRPC(
	PGPContextRef	context,
	PGPUInt32		key_id,
	const char *	passphrase,
	PGPSize			passphraseLength,
	PGPBoolean		hashedPhrase,
	PGPUInt32		cacheTimeOut,
	PGPBoolean		cacheGlobal,
	PGPUInt32 *		raklist,
	PGPSize			raklistsize,
	PGPUInt32		rakclass,
	PGPUInt32 **	newobjs,
	PGPSize *		newobjslength)
{
	NSAutoreleasePool *	pool = [[NSAutoreleasePool alloc] init];
	NSData *			data;
	PGPError			result = [sService pgpAddKeyOptions_back:key_id passphrase:[NSData dataWithBytes:passphrase
									length:passphraseLength] hashedPhrase:hashedPhrase cacheTimeOut:cacheTimeOut
									cacheGlobal:cacheGlobal raklist:[NSData dataWithBytes:raklist length:raklistsize]
									rakclass:rakclass newobjs:&data task:sTask];
	
	if (raklist) {
		PGPFreeData(raklist);
	}
	if (IsntPGPError(result)) {
		*newobjslength = [data length];
		*newobjs = (PGPUInt32 *) PGPNewData(PGPPeekContextMemoryMgr(context), *newobjslength, kPGPMemoryMgrFlags_None);
		if (*newobjs) {
			[data getBytes:*newobjs];
		} else {
			result = kPGPError_OutOfMemory;
		}
	}
	[pool release];
	
	return result;
}



	PGPError
pgpUpdateKeyOptions_backRPC(
	PGPContextRef				context,
	PGPUInt32					key_id,
	const char *				passphrase,
	PGPSize						passphraseLength,
	PGPBoolean					hashedPhrase,
	PGPUInt32					cacheTimeOut,
	PGPBoolean					cacheGlobal,
	const PGPCipherAlgorithm *	prefalg,
	PGPSize						prefalgLength,
	const PGPByte *				prefkeyserv,
	PGPSize						prefkeyservLength,
	PGPUInt32					keyflags,
	PGPBoolean					fkeyflags,
	PGPUInt32					keyservprefs,
	PGPBoolean					fkeyservprefs,
	PGPUInt32 **				newobjs,
	PGPSize *					newobjslength)
{
	NSAutoreleasePool *	pool = [[NSAutoreleasePool alloc] init];
	NSData *			data;
	PGPError			result = [sService pgpUpdateKeyOptions_back:key_id passphrase:[NSData dataWithBytes:passphrase
									length:passphraseLength] hashedPhrase:hashedPhrase cacheTimeOut:cacheTimeOut
									cacheGlobal:cacheGlobal prefalg:[NSData dataWithBytes:prefalg length:prefalgLength]
									prefkeyserv:[NSData dataWithBytes:prefkeyserv length:prefkeyservLength] keyflags:keyflags
									fkeyflags:fkeyflags keyservprefs:keyservprefs fkeyservprefs:fkeyservprefs newobjs:&data
									task:sTask];
	
	if (IsntPGPError(result)) {
		*newobjslength = [data length];
		*newobjs = (PGPUInt32 *) PGPNewData(PGPPeekContextMemoryMgr(context), *newobjslength, kPGPMemoryMgrFlags_None);
		if (*newobjs) {
			[data getBytes:*newobjs];
		} else {
			result = kPGPError_OutOfMemory;
		}
	}
	[pool release];
	
	return result;
}



	PGPError
pgpKeyDBAddObject_backRPC(
	PGPContextRef	context,
	PGPUInt32		kdb_id,
	PGPUInt32		obj_id,
	PGPUInt32 **	newobjs,
	PGPSize *		newobjslength,
	PGPUInt32 *		pnewobj)
{
	NSAutoreleasePool *	pool = [[NSAutoreleasePool alloc] init];
	NSData *			data;
	PGPError			result = [sService pgpKeyDBAddObject_back:kdb_id obj_id:obj_id newobjs:&data pnewobj:pnewobj
									task:sTask];
	
	if (IsntPGPError(result)) {
		*newobjslength = [data length];
		*newobjs = (PGPUInt32 *) PGPNewData(PGPPeekContextMemoryMgr(context), *newobjslength, kPGPMemoryMgrFlags_None);
		if (*newobjs) {
			[data getBytes:*newobjs];
		} else {
			result = kPGPError_OutOfMemory;
		}
	}
	[pool release];
	
	return result;
}



	PGPError
pgpKeyDBRemoveObject_backRPC(
	PGPContextRef	context,
	PGPUInt32		kdb_id,
	PGPUInt32		obj_id)
{
	NSAutoreleasePool *	pool = [[NSAutoreleasePool alloc] init];
	PGPError			result = [sService pgpKeyDBRemoveObject_back:kdb_id obj_id:obj_id task:sTask];

	[pool release];
	
	return result;
}



	PGPError
pgpCopyKeys_backRPC(
	PGPContextRef	context,
	PGPUInt32		srcid,
	PGPUInt32		dstid,
	PGPUInt32 *		keylist,
	PGPSize			keylistsize,
	PGPBoolean		neednewkeylist,
	PGPUInt32 **	newkeylist,
	PGPSize *		newkeylistsize)
{
	NSAutoreleasePool *	pool = [[NSAutoreleasePool alloc] init];
	NSData *			data;
	PGPError			result = [sService pgpCopyKeys_back:srcid dstid:dstid keylist:[NSData dataWithBytes:keylist
									length:keylistsize] neednewkeylist:neednewkeylist newkeylist:&data task:sTask];
	
	if (keylist) {
		PGPFreeData(keylist);
	}
	if (IsntPGPError(result) && neednewkeylist) {
		*newkeylistsize = [data length];
		*newkeylist = (PGPUInt32 *) PGPNewData(PGPPeekContextMemoryMgr(context), *newkeylistsize, kPGPMemoryMgrFlags_None);
		if (*newkeylist) {
			[data getBytes:*newkeylist];
		} else {
			result = kPGPError_OutOfMemory;
		}
	}
	[pool release];
	
	return result;
}



	PGPError
pgpKeyDBArray_backRPC(
	PGPContextRef	context,
	PGPUInt32		kdbid,
	PGPUInt32 *		numKeys,
	PGPUInt32 **	keyArray,
	PGPSize *		keyArraySize)
{
	NSAutoreleasePool *	pool = [[NSAutoreleasePool alloc] init];
	NSData *			data;
	PGPError			result = [sService pgpKeyDBArray_back:kdbid numKeys:numKeys keyArray:&data task:sTask];
	
	if (IsntPGPError(result)) {
		*keyArraySize = [data length];
		*keyArray = (PGPUInt32 *) PGPNewData(PGPPeekContextMemoryMgr(context), *keyArraySize, kPGPMemoryMgrFlags_None);
		if (*keyArray) {
			[data getBytes:*keyArray];
		} else {
			result = kPGPError_OutOfMemory;
		}
	}
	[pool release];
	
	return result;
}



	PGPError
pgpUpdateKeyDB_backRPC(
	PGPContextRef	context,
	PGPUInt32		kdbid,
	PGPUInt32 *		numNewKeys,
	PGPUInt32 **	newKeyArray,
	PGPSize *		newKeyArraySize,
	PGPUInt32 **	changedkeylist,
	PGPSize *		changedkeylistsize)
{
	NSAutoreleasePool *	pool = [[NSAutoreleasePool alloc] init];
	NSData *			data;
	NSData *			data2;
	PGPError			result = [sService pgpUpdateKeyDB_back:kdbid numNewKeys:numNewKeys newKeyArray:&data
									changedkeylist:&data2 task:sTask];
	
	if (IsntPGPError(result)) {
		*newKeyArraySize = [data length];
		*newKeyArray = (PGPUInt32 *) PGPNewData(PGPPeekContextMemoryMgr(context), *newKeyArraySize, kPGPMemoryMgrFlags_None);
		if (*newKeyArray) {
			[data getBytes:*newKeyArray];
			*changedkeylistsize = [data2 length];
			*changedkeylist = (PGPUInt32 *) PGPNewData(PGPPeekContextMemoryMgr(context), *changedkeylistsize, kPGPMemoryMgrFlags_None);
			if (*changedkeylist) {
				[data2 getBytes:*changedkeylist];
			} else {
				PGPFreeData(*newKeyArray);
				*newKeyArray = NULL;
				result = kPGPError_OutOfMemory;
			}
		} else {
			result = kPGPError_OutOfMemory;
		}
	}
	[pool release];
	
	return result;
}



	PGPError
pgpNewKeyDB_backRPC(
	PGPContextRef	context,
	PGPUInt32 *		keydbid)
{
	NSAutoreleasePool *	pool = [[NSAutoreleasePool alloc] init];
	PGPError			result = [sService pgpNewKeyDB_back:keydbid task:sTask];

	[pool release];
	
	return result;
}



	PGPError
pgpCertifyPrimaryUserID_backRPC(
	PGPContextRef	context,
	PGPUInt32		useridid,
	const char *	passphrase,
	PGPSize			passphraseLength,
	PGPBoolean		hashedPhrase,
	PGPUInt32		cacheTimeOut,
	PGPBoolean		cacheGlobal,
	PGPUInt32 **	newobjs,
	PGPSize *		newobjslength)
{
	NSAutoreleasePool *	pool = [[NSAutoreleasePool alloc] init];
	NSData *			data;
	PGPError			result = [sService pgpCertifyPrimaryUserID_back:useridid passphrase:[NSData dataWithBytes:passphrase
									length:passphraseLength] hashedPhrase:hashedPhrase cacheTimeOut:cacheTimeOut
									cacheGlobal:cacheGlobal newobjs:&data task:sTask];
	
	if (IsntPGPError(result)) {
		*newobjslength = [data length];
		*newobjs = (PGPUInt32 *) PGPNewData(PGPPeekContextMemoryMgr(context), *newobjslength, kPGPMemoryMgrFlags_None);
		if (*newobjs) {
			[data getBytes:*newobjs];
		} else {
			result = kPGPError_OutOfMemory;
		}
	}
	[pool release];
	
	return result;
}



	PGPError
pgpGlobalRandomPoolAddState_backRPC(
	PGPBoolean	addKeyState,
	PGPInt32	keyEvent,
	PGPBoolean	addMouseState,
	PGPBoolean	addSystemState)
{
	NSAutoreleasePool *	pool = [[NSAutoreleasePool alloc] init];
	PGPError			result = [sService pgpGlobalRandomPoolAddState_back:addKeyState keyEvent:keyEvent
									addMouseState:addMouseState addSystemState:addSystemState task:sTask];

	[pool release];
	
	return result;
}



	PGPError
pgpGlobalRandomPoolGetInfo_backRPC(
	PGPUInt32 *		entropy,
	PGPUInt32 *		size,
	PGPUInt32 *		minEntropy,
	PGPBoolean *	hasMinEntropy,
	PGPBoolean *	hasIntelRNG)
{
	NSAutoreleasePool *	pool = [[NSAutoreleasePool alloc] init];
	PGPUInt32			ent;
	PGPUInt32			siz;
	PGPUInt32			minEnt;
	BOOL				hasMinEnt;
	BOOL				hasInt;
	PGPError			result = [sService pgpGlobalRandomPoolGetInfo_back:&ent size:&siz minEntropy:&minEnt
									hasMinEntropy:&hasMinEnt hasIntelRNG:&hasInt task:sTask];

	if (entropy) {
		*entropy = ent;
	}
	if (size) {
		*size = siz;
	}
	if (minEntropy) {
		*minEntropy = minEnt;
	}
	if (hasMinEntropy) {
		*hasMinEntropy = hasMinEnt;
	}
	if (hasIntelRNG) {
		*hasIntelRNG = hasInt;
	}
	[pool release];
	
	return result;
}



	PGPError
pgpRandomGetBytesEntropy_backRPC(
	PGPContextRef	context,
	PGPUInt32		requestBytes,
	PGPUInt32		entropyBits,
	PGPByte **		randBuf,
	PGPSize *		randBufLen)
{
	NSAutoreleasePool *	pool = [[NSAutoreleasePool alloc] init];
	NSData *			data;
	PGPError			result = [sService pgpRandomGetBytesEntropy_back:requestBytes entropyBits:entropyBits randBuf:&data
									task:sTask];
	
	if (IsntPGPError(result)) {
		*randBufLen = [data length];
		*randBuf = (PGPByte *) PGPNewData(PGPPeekContextMemoryMgr(context), *randBufLen, kPGPMemoryMgrFlags_None);
		if (*randBuf) {
			[data getBytes:*randBuf];
		} else {
			result = kPGPError_OutOfMemory;
		}
	}
	[pool release];
	
	return result;
}



	PGPError
pgpRandomAddBytes_backRPC(
	PGPContextRef	context,
	PGPByte const *	buf,
	PGPSize			buflen)
{
	NSAutoreleasePool *	pool = [[NSAutoreleasePool alloc] init];
	PGPError			result = [sService pgpRandomAddBytes_back:[NSData dataWithBytes:buf length:buflen] task:sTask];

	[pool release];
	
	return result;
}



	PGPError
pgpRandomStir_backRPC(
	PGPContextRef	context)
{
	NSAutoreleasePool *	pool = [[NSAutoreleasePool alloc] init];
	PGPError			result = [sService pgpRandomStir_back:sTask];

	[pool release];
	
	return result;
}



	PGPError
pgpSetRandSeedFile_backRPC(
	PFLFileSpecRef	randFile)
{
	NSAutoreleasePool *	pool = [[NSAutoreleasePool alloc] init];
	PGPByte *			randData;
	PGPSize				randlen;
	PGPError			result = PFLExportFileSpec(randFile, &randData, &randlen);
	
	if (IsntPGPError(result)) {
		result = [sService pgpSetRandSeedFile_back:[NSData dataWithBytes:randData length:randlen] task:sTask];
		PGPFreeData(randData);
	}
	[pool release];
	
	return result;
}
