/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.
	
	$Id: PGPserviceProtocol.h,v 1.16 2002/08/06 20:10:55 dallen Exp $
____________________________________________________________________________*/

#import <Foundation/Foundation.h>

#import "pflTypes.h"
#import "pgpKeys.h"
#import "pgpPublicKey.h"

extern NSString * const	kPGPserviceID;
extern NSString * const	kPGPserviceName;


@protocol PGPserviceProtocol
	- (PGPError)	pgpConnect_back:(NSMachPort *)task;
	- (PGPError)	pgpReconnect_back:(NSMachPort *)task;
	- (PGPError)	pgpDisconnect_back:(NSMachPort *)task;
	- (PGPError)	pgpFetchObjectData_back:(PGPUInt32)objID bufPtr:(out NSData **)bufPtr task:(NSMachPort *)task;
	- (PGPError)	pgpGetKeyByKeyID_back:(PGPUInt32)dbid keyIDIn:(NSData *)keyIDIn dummyOK:(BOOL)dummyOK deletedOK:(BOOL)deletedOK
						outID:(out PGPUInt32 *)outID task:(NSMachPort *)task;
	- (PGPError)	pgpKeyEncrypt_back:(PGPUInt32)objID inbuf:(NSData *)inbuf format:(PGPPublicKeyMessageFormat)format
						outbuf:(out NSData **)outbuf task:(NSMachPort *)task;
	- (PGPError)	pgpKeyDecrypt_back:(PGPUInt32)objID passphrase:(NSData *)passphrase hashedPhrase:(BOOL)hashedPhrase
						cacheTimeOut:(PGPUInt32)cacheTimeOut cacheGlobal:(BOOL)cacheGlobal inbuf:(NSData *)inbuf
						format:(PGPPublicKeyMessageFormat)format outbuf:(out NSData **)outbuf task:(NSMachPort *)task;
	- (PGPInt32)	pgpKeyVerify_back:(PGPUInt32)objID inbuf:(NSData *)inbuf hashalg:(PGPHashAlgorithm)hashalg
						hash:(NSData *)hash format:(PGPPublicKeyMessageFormat)format task:(NSMachPort *)task;
	- (PGPError)	pgpKeySign_back:(PGPUInt32)objID passphrase:(NSData *)passphrase hashedPhrase:(BOOL)hashedPhrase
						cacheTimeOut:(PGPUInt32)cacheTimeOut cacheGlobal:(BOOL)cacheGlobal hashalg:(PGPHashAlgorithm)hashalg
						hash:(NSData *)hash format:(PGPPublicKeyMessageFormat)format outbuf:(out NSData **)outbuf
						task:(NSMachPort *)task;
	- (BOOL)		pgpSecPassphraseOK_back:(PGPUInt32)objID passphrase:(NSData *)passphrase hashedPhrase:(BOOL)hashedPhrase
						cacheTimeOut:(PGPUInt32)cacheTimeOut cacheGlobal:(BOOL)cacheGlobal task:(NSMachPort *)task;
	- (PGPError)	pgpPurgePassphraseCache_back:(NSMachPort *)task;
	- (PGPError)	pgpCountCachedPassphrases_back:(PGPUInt32 *)nLocals nGlobals:(PGPUInt32 *)nGlobals nOtherLocals:(PGPUInt32 *)nOtherLocals
						task:(NSMachPort *)task;
	- (PGPError)	pgpKeyMaxSizes_back:(PGPUInt32)objID maxEncryption:(out PGPUInt32 *)maxEncryption
						maxDecryption:(out PGPUInt32 *)maxDecryption maxSignature:(out PGPUInt32 *)maxSignature
						format:(PGPPublicKeyMessageFormat)format task:(NSMachPort *)task;
	- (PGPError)	pgpSecProperties_back:(PGPUInt32)objID needsPassphrase:(out BOOL *)needsPassphrase
						isSecretShared:(out BOOL *)isSecretShared lockAlg:(out PGPCipherAlgorithm *)lockAlg
						lockBits:(out PGPUInt32 *)lockBits task:(NSMachPort *)task;
	- (PGPError)	pgpFetchKeyInfo_back:(PGPUInt32)objID bufPtr:(out NSData **)bufPtr task:(NSMachPort *)task;
	- (PGPError)	pgpOpenKeyDBFile_back:(PGPOpenKeyDBFileOptions)openFlags pub:(NSData *)pub priv:(NSData *)priv
						kdbid:(out PGPUInt32 *)kdbid numKeys:(out PGPUInt32 *)numKeys keyBuffer:(out NSData **)keyBuffer
						task:(NSMachPort *)task;
	- (PGPError)	pgpImportKeyBinary_back:(NSData *)buffer kdbid:(out PGPUInt32 *)kdbid numKeys:(out PGPUInt32 *)numKeys
						keyArray:(out NSData **)keyArray task:(NSMachPort *)task;
	- (PGPError)	pgpKeyDBFlush_back:(PGPUInt32)objID changedkeylist:(out NSData **)changedkeylist
						task:(NSMachPort *)task;
	- (void)		pgpFreeKeyDB_back:(PGPUInt32)objID task:(NSMachPort *)task;
	- (PGPError)	pgpCacheKeyDB_back:(PGPUInt32)objID timeoutSeconds:(PGPUInt32)timeoutSeconds task:(NSMachPort *)task;
	- (PGPError)	pgpPurgeKeyDBCache_back:(NSMachPort *)task;
	- (PGPError)	pgpSetKeyEnabled_back:(PGPUInt32)objID enable:(BOOL)enable task:(NSMachPort *)task;
	- (PGPError)	pgpSetKeyAxiomatic_back:(PGPUInt32)objID setAxiomatic:(BOOL)setAxiomatic
						checkPassphrase:(BOOL)checkPassphrase passphrase:(NSData *)passphrase hashedPhrase:(BOOL)hashedPhrase
						cacheTimeOut:(PGPUInt32)cacheTimeOut cacheGlobal:(BOOL)cacheGlobal task:(NSMachPort *)task;
	- (PGPError)	pgpPropagateTrust_back:(PGPUInt32)setid keylist:(NSData *)keylist altid:(PGPUInt32)altid
						timenow:(PGPUInt32)timenow changedkeylist:(out NSData **)changedkeylist task:(NSMachPort *)task;
	- (PGPError)	pgpCheckKeyRingSigs_back:(PGPUInt32)setid keylist:(NSData *)keylist altid:(PGPUInt32)altid
						checkAll:(BOOL)checkAll changedkeylist:(out NSData **)changedkeylist task:(NSMachPort *)task;
	- (PGPError)	pgpPrepareToCheckKeyRingSigs_back:(PGPUInt32)setid keylist:(NSData *)keylist altid:(PGPUInt32)altid
						checkAll:(BOOL)checkAll nsigs:(out PGPUInt32 *)nsigs changedkeylist:(out NSData **)changedkeylist
						task:(NSMachPort *)task;
	- (PGPError)	pgpCheckSig_back:(PGPUInt32)setid objid:(PGPUInt32)objid altid:(PGPUInt32)altid checkAll:(BOOL)checkAll
						revocationonly:(BOOL)revocationonly handled:(out BOOL *)handled
						changed:(out BOOL *)changed verified:(out BOOL *)verified task:(NSMachPort *)task;
	- (PGPError)	pgpDoGenerateKey_back:(PGPUInt32)keydbID masterkeyID:(PGPUInt32)masterkeyID pkalg:(PGPByte)pkalg
						bits:(PGPUInt32)bits creationDate:(PGPTime)creationDate expirationDays:(PGPUInt16)expirationDays
						name:(NSData *)name passphrase:(NSData *)passphrase passphraseIsKey:(BOOL)passphraseIsKey
						masterpass:(NSData *)masterpass cacheTimeOut:(PGPUInt32)cacheTimeOut cacheGlobal:(BOOL)cacheGlobal
						progress:(PGPEventHandlerProcPtr)progress userValue:(PGPUserValue)userValue fastgen:(BOOL)fastgen
						checkentropy:(BOOL)checkentropy usetoken:(BOOL)usetoken tokenid:(PGPUInt32)tokenid
						adkkeylist:(NSData *)adkkeylist adkclass:(PGPByte)adkclass rakkeylist:(NSData *)rakkeylist
						rakclass:(PGPByte)rakclass prefalg:(NSData *)prefalg prefkeyserv:(NSData *)prefkeyserv
						keyflags:(PGPUInt32)keyflags fkeyflags:(BOOL)fkeyflags keyservprefs:(PGPUInt32)keyservprefs
						fkeyservprefs:(BOOL)fkeysesrvprefs newobjs:(out NSData **)newobjs newKey:(out PGPUInt32 *)newKey
						task:(NSMachPort *)task;
	- (PGPError)	pgpCreateKeypair_back:(PGPUInt32)keydbID seckeyBuf:(NSData *)seckeyBuf keyspecBuf:(NSData *)keyspecBuf
						name:(NSData *)name adklist:(NSData *)adklist adkclass:(PGPByte)adkclass raklist:(NSData *)raklist
						rakclass:(PGPByte)rakclass passphrase:(NSData *)passphrase passphraseIsKey:(BOOL)passphraseIsKey
						cacheTimeOut:(PGPUInt32)cacheTimeOut cacheGlobal:(BOOL)cacheGlobal
						newobjs:(out NSData **)newobjs newkeyid:(out PGPUInt32 *)newkeyid task:(NSMachPort *)task;
	- (PGPError)	pgpCreateSubkeypair_back:(PGPUInt32)masterkeyID seckeyBuf:(NSData *)seckeyBuf
						keyspecBuf:(NSData *)keyspecBuf passphrase:(NSData *)passphrase passphraseIsKey:(BOOL)passphraseIsKey
						cacheTimeOut:(PGPUInt32)cacheTimeOut cacheGlobal:(BOOL)cacheGlobal newobjs:(out NSData **)newobjs
						newkeyid:(out PGPUInt32 *)newkeyid task:(NSMachPort *)task;
	- (PGPError)	pgpCountTokens_back:(out PGPUInt32 *)numTokens task:(NSMachPort *)task;
	- (PGPError)	pgpGetTokenInfo_back:(PGPUInt32)tokNumber info:(out PGPTokenInfo *)info task:(NSMachPort *)task;
	- (PGPError)	pgpDeleteKeyOnToken_back:(PGPUInt32)keydbID keyID:(NSData *)keyID
						tokenNumber:(PGPUInt32)tokenNumber pin:(NSData *)pin task:(NSMachPort *)task;
	- (PGPError)	pgpWipeToken_back:(PGPUInt32)tokNumber passphrase:(NSData *)passphrase task:(NSMachPort *)task;
	- (PGPError)	pgpFormatToken_back:(PGPUInt32)tokNumber adminPin:(NSData *)adminPin newUserPin:(NSData *)newUserPin
						task:(NSMachPort *)task;
	- (PGPError)	pgpTokenPassphraseIsValid_back:(PGPUInt32)tokNumber passphrase:(NSData *)passphrase
						task:(NSMachPort *)task;
	- (PGPError)	pgpCopyKeyToToken_back:(PGPUInt32)objID tokNumber:(PGPUInt32)tokNumber isMaster:(BOOL)isMaster
						passphrase:(NSData *)passphrase hashedPhrase:(BOOL)hashedPhrase PIN:(NSData *)PIN
						cacheTimeOut:(PGPUInt32)cacheTimeOut cacheGlobal:(BOOL)cacheGlobal task:(NSMachPort *)task;
	- (PGPError)	pgpTokenImportX509_back:(NSData *)keyID userID:(NSData *)userID x509:(NSData *)x509
						password:(NSData *)password task:(NSMachPort *)task;
	- (PGPError)	pgpAddUserID_back:(PGPUInt32)objid isAttribute:(BOOL)isAttribute
						attributeType:(PGPAttributeType)attributeType userIDData:(NSData *)userIDData
						passphrase:(NSData *)passphrase hashedPhrase:(BOOL)hashedPhrase cacheTimeOut:(PGPUInt32)cacheTimeOut
						cacheGlobal:(BOOL)cacheGlobal newobjs:(out NSData **)newobjs task:(NSMachPort *)task;
	- (PGPError)	pgpCertifyUserID_back:(PGPUInt32)useridid certifying_keyid:(PGPUInt32)certifying_keyid
						passphrase:(NSData *)passphrase hashedPhrase:(BOOL)hashedPhrase cacheTimeOut:(PGPUInt32)cacheTimeOut
						cacheGlobal:(BOOL)cacheGlobal exportable:(BOOL)exportable creationDate:(PGPTime)creationDate
						expiration:(PGPUInt32)expiration trustDepth:(PGPByte)trustDepth trustValue:(PGPByte)trustValue
						sRegExp:(NSData *)sRegExp newobjs:(out NSData **)newobjs task:(NSMachPort *)task;
	- (PGPError)	pgpRevokeSig_back:(PGPUInt32)sigid passphrase:(NSData *)passphrase hashedPhrase:(BOOL)hashedPhrase
						cacheTimeOut:(PGPUInt32)cacheTimeOut cacheGlobal:(BOOL)cacheGlobal newobjs:(out NSData **)newobjs
						task:(NSMachPort *)task;
	- (PGPError)	pgpRevokeKey_back:(PGPUInt32)key_id passphrase:(NSData *)passphrase hashedPhrase:(BOOL)hashedPhrase
						cacheTimeOut:(PGPUInt32)cacheTimeOut cacheGlobal:(BOOL)cacheGlobal newobjs:(out NSData **)newobjs
						task:(NSMachPort *)task;
	- (PGPError)	pgpDoChangePassphrase_back:(PGPUInt32)keydb_id key_id:(PGPUInt32)key_id
						masterkey_id:(PGPUInt32)masterkey_id oldphrase:(NSData *)oldphrase newphrase:(NSData *)newphrase
						newPassphraseIsKey:(BOOL)newPassphraseIsKey cacheTimeOut:(PGPUInt32)cacheTimeOut
						cacheGlobal:(BOOL)cacheGlobal task:(NSMachPort *)task;
	- (PGPError)	pgpPassphraseIsValid_back:(PGPUInt32)key_id passphrase:(NSData *)passphrase hashedPhrase:(BOOL)hashedPhrase
						cacheTimeOut:(PGPUInt32)cacheTimeOut cacheGlobal:(BOOL)cacheGlobal isValid:(out BOOL *)isValid
						task:(NSMachPort *)task;
	- (PGPError)	pgpSetKeyTrust_back:(PGPUInt32)key_id trust:(PGPUInt32)trust task:(NSMachPort *)task;
	- (PGPError)	pgpGetPasskeyBuffer_back:(PGPUInt32)key_id passphrase:(NSData *)passphrase
						passkeyBuffer:(out NSData **)passkeyBuffer task:(NSMachPort *)task;
	- (PGPError)	pgpAddKeyOptions_back:(PGPUInt32)key_id passphrase:(NSData *)passphrase hashedPhrase:(BOOL)hashedPhrase
						cacheTimeOut:(PGPUInt32)cacheTimeOut cacheGlobal:(BOOL)cacheGlobal raklist:(NSData *)raklist
						rakclass:(PGPUInt32)rakclass newobjs:(out NSData **)newobjs task:(NSMachPort *)task;
	- (PGPError)	pgpUpdateKeyOptions_back:(PGPUInt32)key_id passphrase:(NSData *)passphrase hashedPhrase:(BOOL)hashedPhrase
						cacheTimeOut:(PGPUInt32)cacheTimeOut cacheGlobal:(BOOL)cacheGlobal prefalg:(NSData *)prefalg
						prefkeyserv:(NSData *)prefkeyserv keyflags:(PGPUInt32)keyflags fkeyflags:(BOOL)fkeyflags
						keyservprefs:(PGPUInt32)keyservprefs fkeyservprefs:(BOOL)fkeyservprefs newobjs:(out NSData **)newobjs
						task:(NSMachPort *)task;
	- (PGPError)	pgpKeyDBAddObject_back:(PGPUInt32)kdb_id obj_id:(PGPUInt32)obj_id newobjs:(out NSData **)newobjs
						pnewobj:(out PGPUInt32 *)pnewobj task:(NSMachPort *)task;
	- (PGPError)	pgpKeyDBRemoveObject_back:(PGPUInt32)kdb_id obj_id:(PGPUInt32)obj_id task:(NSMachPort *)task;
	- (PGPError)	pgpCopyKeys_back:(PGPUInt32)srcid dstid:(PGPUInt32)dstid keylist:(NSData *)keylist
						neednewkeylist:(BOOL)neednewkeylist newkeylist:(out NSData **)newkeylist task:(NSMachPort *)task;
	- (PGPError)	pgpKeyDBArray_back:(PGPUInt32)kdbid numKeys:(out PGPUInt32 *)numKeys keyArray:(out NSData **)keyArray
						task:(NSMachPort *)task;
	- (PGPError)	pgpUpdateKeyDB_back:(PGPUInt32)kdbid numNewKeys:(out PGPUInt32 *)numNewKeys
						newKeyArray:(out NSData **)newKeyArray changedkeylist:(out NSData **)changedkeylist
						task:(NSMachPort *)task;
	- (PGPError)	pgpNewKeyDB_back:(out PGPUInt32 *)keydbid task:(NSMachPort *)task;
	- (PGPError)	pgpCertifyPrimaryUserID_back:(PGPUInt32)useridid passphrase:(NSData *)passphrase
						hashedPhrase:(BOOL)hashedPhrase cacheTimeOut:(PGPUInt32)cacheTimeOut cacheGlobal:(BOOL)cacheGlobal
						newobjs:(out NSData **)newobjs task:(NSMachPort *)task;
	- (PGPError)	pgpGlobalRandomPoolAddState_back:(BOOL)addKeyState keyEvent:(PGPInt32)keyEvent
						addMouseState:(BOOL)addMouseState addSystemState:(BOOL)addSystemState task:(NSMachPort *)task;
	- (PGPError)	pgpGlobalRandomPoolGetInfo_back:(out PGPUInt32 *)entropy size:(out PGPUInt32 *)size
						minEntropy:(out PGPUInt32 *)minEntropy hasMinEntropy:(out BOOL *)hasMinEntropy
						hasIntelRNG:(out BOOL *)hasIntelRNG task:(NSMachPort *)task;
	- (PGPError)	pgpRandomGetBytesEntropy_back:(PGPUInt32)requestBytes entropyBits:(PGPUInt32)entropyBits
						randBuf:(out NSData **)randBuf task:(NSMachPort *)task;
	- (PGPError)	pgpRandomAddBytes_back:(NSData *)buf task:(NSMachPort *)task;
	- (PGPError)	pgpRandomStir_back:(NSMachPort *)task;
	- (PGPError)	pgpSetRandSeedFile_back:(NSData *)randFile task:(NSMachPort *)task;
	- (PGPError)	pgpTokenPutKeyContainer_back:(NSData *)keyID password:(NSData *)password cont:(NSData *)cont
						task:(NSMachPort *)task;
	- (PGPError)	pgpTokenGetKeyContainer_back:(NSData *)keyID password:(NSData *)password contOut:(out NSData **)contOut
						task:(NSMachPort *)task;
	- (PGPError)	pgpSetPKCS11DrvFile_back:(NSData *)module task:(NSMachPort *)task;
@end