/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.
	
	$Id: PGPCollectRandomDataDialog.m,v 1.3 2002/10/04 08:12:32 jason Exp $
____________________________________________________________________________*/

#import "PGPDialogClasses.h"
#include "pgpRandomPool.h"

#include <Carbon/Carbon.h>

#define kPGPCollectRandomDataDialogNib			@"PGPCollectRandomDataDialog"
#define kMinimumUserEntropy						300
#define kEntropyCollectionTimerInterval			0.05

@implementation CPGPCollectRandomDataDialog

- (id)init
{
	if (!mLoadedNib)
		[self loadNib:kPGPCollectRandomDataDialogNib];
	
	return self;
}

- (void)loadNib:(NSString*)nibFile
{
    if (nibFile && [nibFile length])
        [NSBundle loadNibNamed:nibFile owner:self];
	else
		PGPLog((@"%s:%d: Bad nibFile specified", __FILE__, __LINE__));
}

- (PGPUInt32)neededEntropyBits
{
	return mNeededEntropyBits;
}

- (void)setNeededEntropyBits:(PGPUInt32)neededEntropyBits
{
	mNeededEntropyBits = neededEntropyBits;
}

- (void)startTimer
{
	if (mEntropyTimer)
	{
		[mEntropyTimer invalidate];
		[mEntropyTimer release];
	}

	/*
	In order for our timer to work while running a modal dialog, we need to
	install it before the modal session has started, and we need to install
	it in NSModalPanelRunLoopMode.
	*/
	mEntropyTimer = [NSTimer timerWithTimeInterval:kEntropyCollectionTimerInterval
							 target:self
							 selector:@selector(collectEntropy:)
							 userInfo:nil
							 repeats:YES];
	
	[mEntropyTimer retain];
																
	[[NSRunLoop currentRunLoop] addTimer:mEntropyTimer forMode:NSModalPanelRunLoopMode];
}

- (void)stopTimer
{
    if (mEntropyTimer)
    {
        [mEntropyTimer invalidate];

        mEntropyTimer = NULL;
    }
}

- (void)collectEntropy:(NSTimer*)timer
{
	Point		mouseLoc;
	
	GetMouse(&mouseLoc);
	 
	if ((mouseLoc.v != mLastMouse.v) || (mouseLoc.h != mLastMouse.h))
	{
		PGPGlobalRandomPoolAddMouse(mouseLoc.h, mouseLoc.v);
		mLastMouse = mouseLoc;
	}
	
	[self updateProgressBar:timer];
}

- (void)textDidChange:(NSNotification*)notification
{
	NSString*	string = [mKeyThief stringValue];
	unichar		character;
	
	if ([string length])
	{
		character = [string characterAtIndex:0];
		
		PGPGlobalRandomPoolAddKeystroke(character);
		
		[self updateProgressBar:notification];
		
		[mKeyThief setStringValue:@""];
	}
}

- (void)updateProgressBar:(id)sender
{
	PGPUInt32		currentEntropy = PGPGlobalRandomPoolGetEntropy();
	PGPUInt32		complete;
	
    if (currentEntropy >= mNeededEntropy)
        complete = 100;
    else
        complete = ((currentEntropy - mStartEntropy) * 100) / (mNeededEntropy - mStartEntropy);
	
	if (complete > 100)
		complete = 100;
	
	[mRandomDataCollected setDoubleValue:complete];
	
	if (complete == 100)
    {
        [self stopTimer];
        
		[mDismissButton setTitle:kPGPUIString_OK];
        [mDismissButton setAction:@selector(ok:)];
        [mWindow setDefaultButtonCell:[mDismissButton cell]];
    }
}			

- (PGPError)runDialog
{
	NSRect		contentViewFrame = [[mWindow contentView] frame];
	NSRect		frame = [mKeyThief frame];
	
    [mDismissButton setTarget:self];
    [mDismissButton setAction:@selector(cancel:)];
    
	frame.origin.x = NSWidth(contentViewFrame) + 12;	/* Must conform to Aqua HI guidelines! :) */
	
	[mKeyThief setFrame:frame];
	
	[[NSNotificationCenter defaultCenter] addObserver:self
			selector:@selector(textDidChange:)
			name:NSControlTextDidChangeNotification
			object:nil];
	
	[mWindow makeFirstResponder:mKeyThief];
	
	mStartEntropy = PGPGlobalRandomPoolGetEntropy();
	mNeededEntropy = mNeededEntropyBits + mStartEntropy;

    if ((mNeededEntropy - mStartEntropy) < kMinimumUserEntropy)
        mNeededEntropy += kMinimumUserEntropy;
    
    if (mNeededEntropy > PGPGlobalRandomPoolGetSize())
        mNeededEntropy = PGPGlobalRandomPoolGetSize() - mStartEntropy;
    
	[self updateProgressBar:self];

	[self startTimer];

	return [super runDialog];
}

- (void)close
{
    [self stopTimer];
    
	[NSApp stopModal];
	[mWindow close];
}

- (IBAction)cancel:(id)sender
{
	mReturnCode = kPGPError_UserAbort;
	[self close];
}

- (IBAction)ok:(id)sender
{
	mReturnCode = kPGPError_NoErr;
	[self close];
}

- (void)dealloc
{
	[[NSNotificationCenter defaultCenter] removeObserver:self];
	
	[self stopTimer];
    					
	[super dealloc];
}

@end
