/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.
	
	$Id: PGPDialogProxies.cpp,v 1.2 2002/08/06 20:10:56 dallen Exp $
____________________________________________________________________________*/

#include "pgpDialogs.h"
#include "PGPDialogProxies.h"

PGPError configureDialog(PGPDialogProxyObjRef proxyObject)
{
	
	/* Handle generic dialog options */
	setContext(proxyObject, ((CPGPDialogOptions*) proxyObject->options)->mContext);
	setClientOptions(proxyObject, ((CPGPDialogOptions*) proxyObject->options)->mClientOptions);
	setWindowTitle(proxyObject, ((CPGPDialogOptions*) proxyObject->options)->mWindowTitle);
	setServerList(proxyObject, (PGPKeyServerSpec*) ((CPGPDialogOptions*) proxyObject->options)->mServerList);
	setServerCount(proxyObject, ((CPGPDialogOptions*) proxyObject->options)->mServerCount);
	setSearchBeforeDisplay(proxyObject, ((CPGPDialogOptions*) proxyObject->options)->mSearchBeforeDisplay);
	setNewKeys(proxyObject, ((CPGPDialogOptions*) proxyObject->options)->mNewKeys);
	setTLSContext(proxyObject, ((CPGPDialogOptions*) proxyObject->options)->mTLSContext);
	setPrompt(proxyObject, ((CPGPDialogOptions*) proxyObject->options)->mPrompt);
		
	/* Now handle dialog class specific options */
	switch (proxyObject->dlgClass)
	{
		case kPGPDialogClass_RandomDataDialog:
		
			setNeededEntropyBits(proxyObject, ((CPGPRandomDataDialogOptions*) proxyObject->options)->mNeededEntropyBits);
			
			break;
		
		case kPGPDialogClass_RecipientDialog:

			setRecipientDialogOptions(proxyObject, ((CPGPRecipientDialogOptions*) proxyObject->options)->mDialogOptions);
			setNumDefaultRecipients(proxyObject, ((CPGPRecipientDialogOptions*) proxyObject->options)->mNumDefaultRecipients);
			setDefaultRecipients(proxyObject, ((CPGPRecipientDialogOptions*) proxyObject->options)->mDefaultRecipients);
			setDisplayMarginalValidity(proxyObject, ((CPGPRecipientDialogOptions*) proxyObject->options)->mDisplayMarginalValidity);
			setIgnoreMarginalValidity(proxyObject, ((CPGPRecipientDialogOptions*) proxyObject->options)->mIgnoreMarginalValidity);
			setGroupSet(proxyObject, ((CPGPRecipientDialogOptions*) proxyObject->options)->mGroupSet);
			setClientKeySet(proxyObject, ((CPGPRecipientDialogOptions*) proxyObject->options)->mClientKeySet);
			setAlwaysDisplay(proxyObject, ((CPGPRecipientDialogOptions*) proxyObject->options)->mAlwaysDisplay);
			setAlwaysDisplayWithARRs(proxyObject, ((CPGPRecipientDialogOptions*) proxyObject->options)->mAlwaysDisplayWithARRs);
			setRecipientKeysPtr(proxyObject, ((CPGPRecipientDialogOptions*) proxyObject->options)->mRecipientKeysPtr);
			setRecipientCount(proxyObject, ((CPGPRecipientDialogOptions*) proxyObject->options)->mRecipientCount);
			setRecipientList(proxyObject, ((CPGPRecipientDialogOptions*) proxyObject->options)->mRecipientList);
			setEnforcement(proxyObject, ((CPGPRecipientDialogOptions*) proxyObject->options)->mEnforcement);
			
			break;

		case kPGPDialogClass_PassphraseDialog:
		
			setPassphrasePtr(proxyObject, ((CPGPPassphraseDialogOptions*) proxyObject->options)->mPassphrasePtr);
			setDialogOptions(proxyObject, ((CPGPPassphraseDialogOptions*) proxyObject->options)->mDialogOptions);
			setMinPassphraseLength(proxyObject, ((CPGPPassphraseDialogOptions*) proxyObject->options)->mMinPassphraseLength);
			setMinPassphraseQuality(proxyObject, ((CPGPPassphraseDialogOptions*) proxyObject->options)->mMinPassphraseQuality);
			setDialogTimeout(proxyObject, ((CPGPPassphraseDialogOptions*) proxyObject->options)->mDialogTimeout);
			
			switch (proxyObject->type)
			{
				case kPGPDialogProxyType_KeyPassphraseOptions:
			
                    setDefaultKey(proxyObject, ((CPGPKeyPassphraseDialogOptions*) proxyObject->options)->mDefaultKey);
					setVerifyPassphrase(proxyObject, ((CPGPKeyPassphraseDialogOptions*) proxyObject->options)->mVerifyPassphrase);
					setCache(proxyObject, ((CPGPKeyPassphraseDialogOptions*) proxyObject->options)->mCache);
					setCacheGlobal(proxyObject, ((CPGPKeyPassphraseDialogOptions*) proxyObject->options)->mCacheGlobal);
					setCacheTimeout(proxyObject, ((CPGPKeyPassphraseDialogOptions*) proxyObject->options)->mCacheTimeout);
					
					break;
				
				case kPGPDialogProxyType_ConfirmationPassphraseOptions:
				
					setShowsPassphraseQuality(proxyObject, ((CPGPConfirmationPassphraseDialogOptions*) proxyObject->options)->mShowPassphraseQuality);
							
					break;
					
				case kPGPDialogProxyType_SigningPassphraseOptions:

					setFindMatchingKey(proxyObject, ((CPGPSigningPassphraseDialogOptions*) proxyObject->options)->mFindMatchingKey);
					setDefaultKey(proxyObject, ((CPGPSigningPassphraseDialogOptions*) proxyObject->options)->mDefaultKey);
					setKeySet(proxyObject, ((CPGPSigningPassphraseDialogOptions*) proxyObject->options)->mKeySet);
					setPassphraseKeyPtr(proxyObject, ((CPGPSigningPassphraseDialogOptions*) proxyObject->options)->mPassphraseKeyPtr);
					setVerifyPassphrase(proxyObject, ((CPGPSigningPassphraseDialogOptions*) proxyObject->options)->mVerifyPassphrase);
					setCache(proxyObject, ((CPGPSigningPassphraseDialogOptions*) proxyObject->options)->mCache);
					setCacheGlobal(proxyObject, ((CPGPSigningPassphraseDialogOptions*) proxyObject->options)->mCacheGlobal);
					setCacheTimeout(proxyObject, ((CPGPSigningPassphraseDialogOptions*) proxyObject->options)->mCacheTimeout);
					
					break;
				
				case kPGPDialogProxyType_DecryptionPassphraseOptions:
				
					setFindMatchingKey(proxyObject, ((CPGPDecryptionPassphraseDialogOptions*) proxyObject->options)->mFindMatchingKey);
					setDefaultKey(proxyObject, ((CPGPDecryptionPassphraseDialogOptions*) proxyObject->options)->mDefaultKey);
					setKeySet(proxyObject, ((CPGPDecryptionPassphraseDialogOptions*) proxyObject->options)->mKeySet);
					setPassphraseKeyPtr(proxyObject, ((CPGPDecryptionPassphraseDialogOptions*) proxyObject->options)->mPassphraseKeyPtr);
					setVerifyPassphrase(proxyObject, ((CPGPDecryptionPassphraseDialogOptions*) proxyObject->options)->mVerifyPassphrase);
					setCache(proxyObject, ((CPGPDecryptionPassphraseDialogOptions*) proxyObject->options)->mCache);
					setCacheGlobal(proxyObject, ((CPGPDecryptionPassphraseDialogOptions*) proxyObject->options)->mCacheGlobal);
					setCacheTimeout(proxyObject, ((CPGPDecryptionPassphraseDialogOptions*) proxyObject->options)->mCacheTimeout);
					setKeyIDList(proxyObject, (PGPKeyID*) ((CPGPDecryptionPassphraseDialogOptions*) proxyObject->options)->mKeyIDList);
					setKeyIDCount(proxyObject, ((CPGPDecryptionPassphraseDialogOptions*) proxyObject->options)->mKeyIDCount);
					setMissingKeyIDList(proxyObject, ((CPGPDecryptionPassphraseDialogOptions*) proxyObject->options)->mMissingKeyIDList);
					setMissingKeyIDCount(proxyObject, ((CPGPDecryptionPassphraseDialogOptions*) proxyObject->options)->mMissingKeyIDCount);
					
				default:
					break;
			}
			
			break;
		
		case kPGPDialogClass_KeyServerDialog:
			
			switch (proxyObject->type)
			{
				case kPGPDialogProxyType_SearchKeyServerOptions:
					setSearchAllServers(proxyObject, ((CPGPSearchKeyServerDialogOptions*) proxyObject->options)->mSearchAllServers);
					setFilter(proxyObject, ((CPGPSearchKeyServerDialogOptions*) proxyObject->options)->mFilter);
					setKeyDescription(proxyObject, ((CPGPSearchKeyServerDialogOptions*) proxyObject->options)->mKeyDescription);
					
					break;
				
				case kPGPDialogProxyType_SendToKeyServerOptions:
					setKeysToSend(proxyObject, ((CPGPSendToKeyServerDialogOptions*) proxyObject->options)->mKeysToSend);
					setFailedKeys(proxyObject, ((CPGPSendToKeyServerDialogOptions*) proxyObject->options)->mFailedKeys);
					break;
				
				default:
					break;
			}
			
			break;
		
		default:
			break;
	}
	
	return kPGPError_NoErr;
}