/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.
	
	$Id: PGPOptionsDialog.m,v 1.3 2002/10/22 09:25:50 jeffc Exp $
____________________________________________________________________________*/

#import "PGPDialogClasses.h"

#define kPGPOptionsDialogNib					@"PGPOptionsDialog"
#define kPGPOptionsDialogHPadding				20
#define kPGPOptionsDialogVPadding				20
#define kPGPOptionsDialogYInsertionPoint		20
#define kPGPOptionsDialogCheckboxHeight			18
#define kPGPOptionsDialogPopUpTitleHeight		14
#define kPGPOptionsDialogPopUpButtonHeight		22
#define kPGPOptionsDialogPopUpButtonMinWidth	57

@interface CPGPOptionsView : NSView
{
}

/*
- (BOOL)isOpaque;
- (void)drawRect:(NSRect)rect;
*/

@end

@implementation CPGPOptionsView

/*
- (BOOL)isOpaque
{
	return NO;
}

- (void)drawRect:(NSRect)rect
{
	[[NSColor blackColor] set];
	[NSBezierPath strokeRect:[self bounds]];
}
*/

@end

@implementation CPGPOptionsDialog

- (id)initWithOptions:(PGPOptionListRef)options
{
	mOptions = options;
	
	if (!mLoadedNib)
		[self loadNib:kPGPOptionsDialogNib];
	
	if (!mOptionMapping)
		mOptionMapping = [[NSMutableDictionary alloc] init];
	
	return self;
}

- (void)loadNib:(NSString*)nibFile
{
	
    if (nibFile && [nibFile length])
        [NSBundle loadNibNamed:nibFile owner:self];
	else
		PGPLog((@"%s:%d: Bad nibFile specified", __FILE__, __LINE__));
	
	mLoadedNib = YES;
}

- (NSView*)optionsView
{
	PGPError			err;
	NSRect				frame = NSZeroRect;
	NSRect				srcFrame;
	NSRect				dstFrame;
	NSView*				dstView;
	CPGPOptionsView*	checkboxView = [[[CPGPOptionsView alloc] initWithFrame:NSZeroRect] autorelease];
	CPGPOptionsView*	popupbuttonView = [[[CPGPOptionsView alloc] initWithFrame:NSZeroRect] autorelease];
	CPGPOptionsView*	optionsView = [[[CPGPOptionsView alloc] initWithFrame:NSZeroRect] autorelease];
	
	if (!mOptions)
		return nil;
	
	err = [self buildCheckboxesInView:checkboxView withOptionList:mOptions horizontalPadding:12 verticalPadding:12 parentCheckbox:nil];
	if (IsPGPError(err))
		return nil;
	
	err = [self buildPopUpButtonsInView:popupbuttonView withOptionList:mOptions horizontalPadding: 12 verticalPadding:12];
	if (IsPGPError(err))
		return nil;
	
	frame.size.width = NSWidth([checkboxView frame]) + NSWidth([popupbuttonView frame]);
	frame.size.height = NSHeight([checkboxView frame]) > NSHeight([popupbuttonView frame]) ?
							NSHeight([checkboxView frame]) : NSHeight([popupbuttonView frame]);
							
	[optionsView setFrame:frame];
	[optionsView setAutoresizingMask:(NSViewWidthSizable | NSViewHeightSizable)];

	[popupbuttonView setAutoresizingMask:(NSViewMinXMargin | NSViewMaxYMargin)];
	
	if (NSHeight([checkboxView frame]) > NSHeight([popupbuttonView frame]))
	{
		srcFrame = [checkboxView frame];
		dstFrame = [popupbuttonView frame];
		dstView = popupbuttonView;
	}
	else
	{
		srcFrame = [popupbuttonView frame];
		dstFrame = [checkboxView frame];
		dstView = checkboxView;
	}
	
	dstFrame.origin.y += NSHeight(srcFrame) - NSHeight(dstFrame);
	[dstView setFrame:dstFrame];
			
	frame = [popupbuttonView frame];
	frame.origin.x = NSMaxX([checkboxView frame]);
	[popupbuttonView setFrame:frame];
	
	[optionsView addSubview:checkboxView];
	[optionsView addSubview:popupbuttonView];
	
	return optionsView;
}

- (PGPError)buildCheckboxesInView:(NSView*)checkboxContainer withOptionList:(PGPOptionListRef)optionList horizontalPadding:(PGPUInt32)hPadding verticalPadding:(PGPUInt32)vPadding parentCheckbox:(NSButton*)parent
{
	NSRect				containerFrame;
	BOOL				addSpacing = YES;
	PGPUInt32			optionIndex = 0;
	PGPOption       	optionData;
	
	if (!checkboxContainer || !optionList)
		return kPGPError_BadParams;

	containerFrame = [checkboxContainer frame];
	if (!NSHeight(containerFrame) || !NSWidth(containerFrame))
	{
		containerFrame.size.height = 2 * vPadding;
		containerFrame.size.width = 2 * hPadding;
		[checkboxContainer setFrame:containerFrame];
		addSpacing = NO;
	}
	
	while (IsntPGPError(pgpGetIndexedOption(optionList, optionIndex, FALSE, &optionData)))
	{
		if (optionData.type == kPGPOptionType_Checkbox)
		{
			PGPOUICheckboxDesc*			descriptor;
			NSButton*					checkbox;
			NSSize						checkboxSize;
			NSRect						checkboxFrame;
			NSRect						newFrame;
			NSView*						subview;
			float						newMinWidth;
			NSEnumerator*				viewEnumerator;

			descriptor = (PGPOUICheckboxDesc*) optionData.value.asPtr;

			if (checkbox = [self createCheckboxButton:descriptor parentCheckbox:parent])
			{
				containerFrame = [checkboxContainer frame];
				containerFrame.size.height += kPGPOptionsDialogCheckboxHeight;
				
				if (addSpacing)
					containerFrame.size.height += 4;
				else
					addSpacing = YES;
									
				checkboxSize = [[checkbox cell] cellSize];
				[checkbox setFrame:NSMakeRect(hPadding, vPadding, checkboxSize.width, kPGPOptionsDialogCheckboxHeight)];
				checkboxFrame = [checkbox frame];
				
				newMinWidth = NSMinX(checkboxFrame) + NSWidth(checkboxFrame) + hPadding;
				if (newMinWidth > NSWidth(containerFrame))
					containerFrame.size.width = newMinWidth;

				[checkboxContainer setFrame:containerFrame];
				
				viewEnumerator = [[checkboxContainer subviews] objectEnumerator];
				while (subview = [viewEnumerator nextObject])
				{
					newFrame = [subview frame];
					newFrame.origin.y += kPGPOptionsDialogCheckboxHeight + 4;
					[subview setFrame:newFrame];
				}
				
				[checkboxContainer addSubview:checkbox];
			}
			
			if (optionData.subOptions)
			{
				[self buildCheckboxesInView:checkboxContainer
							withOptionList:optionData.subOptions
							horizontalPadding:(hPadding + 17)
							verticalPadding:vPadding
							parentCheckbox:checkbox];
			}
		}
		
		++optionIndex;
	}
	
	return kPGPError_NoErr;
}

- (PGPError)buildPopUpButtonsInView:(NSView*)popupContainer withOptionList:(PGPOptionListRef)optionList horizontalPadding:(PGPUInt32)hPadding verticalPadding:(PGPUInt32)vPadding
{
	NSRect				containerFrame;
	BOOL				addSpacing = YES;
	PGPUInt32			optionIndex = 0;
	PGPOption       	optionData;
	
	if (!popupContainer || !optionList)
		return kPGPError_BadParams;

	containerFrame = [popupContainer frame];
	if (!NSHeight(containerFrame) || !NSWidth(containerFrame))
	{
		containerFrame.size.height = 2 * vPadding;
		containerFrame.size.width = 2 * hPadding;
		[popupContainer setFrame:containerFrame];
		addSpacing = NO;
	}
	
	while (IsntPGPError(pgpGetIndexedOption(optionList, optionIndex, FALSE, &optionData)))
	{
		if (optionData.type == kPGPOptionType_PopupList)
		{
			PGPOUIPopupListDesc*		descriptor;
			NSTextField*				popupTitle = nil;
			NSPopUpButton*				popupButton = nil;
			NSSize						popupButtonSize;
			NSSize						popupTitleSize;
			NSRect						popupTitleFrame;
			NSRect						popupButtonFrame;
			NSRect						newFrame;
			NSView*						subview;
			float						newMinWidth;
			NSEnumerator*				viewEnumerator;

			descriptor = (PGPOUIPopupListDesc*) optionData.value.asPtr;

			if ((popupTitle = [self createPopUpTitle:descriptor]) && (popupButton = [self createPopUpButton:descriptor]))
			{
				containerFrame = [popupContainer frame];
				containerFrame.size.height += kPGPOptionsDialogPopUpButtonHeight;
				
				if (addSpacing)
					containerFrame.size.height += 4;
				else
					addSpacing = YES;

				popupButtonSize = [popupButton frame].size;
				popupTitleSize = [[popupTitle cell] cellSize];
				
				[popupTitle setFrame:NSMakeRect(hPadding, vPadding + 6, popupTitleSize.width, kPGPOptionsDialogPopUpTitleHeight)];
				[popupButton setFrame:NSMakeRect(hPadding + NSWidth([popupTitle frame]) + 3, vPadding, popupButtonSize.width, popupButtonSize.height)];
				popupTitleFrame = [popupTitle frame];
				popupButtonFrame = [popupButton frame];

				newMinWidth = NSMinX(popupTitleFrame) + NSWidth(popupTitleFrame) + 8 + NSWidth(popupButtonFrame) + hPadding;
				if (newMinWidth > NSWidth(containerFrame))
					containerFrame.size.width = newMinWidth;
					
				[popupContainer setFrame:containerFrame];

				viewEnumerator = [[popupContainer subviews] objectEnumerator];
				while (subview = [viewEnumerator nextObject])
				{
					newFrame = [subview frame];
					newFrame.origin.y += kPGPOptionsDialogPopUpButtonHeight + 4;
					[subview setFrame:newFrame];
				}
				
				[popupContainer addSubview:popupTitle];
				[popupContainer addSubview:popupButton];
			}
			
			if (optionData.subOptions)
			{
				[self buildPopUpButtonsInView:popupContainer
							withOptionList:optionData.subOptions
							horizontalPadding:(hPadding + 17)
							verticalPadding:vPadding];
			}
		}
		
		++optionIndex;
	}
	
	return kPGPError_NoErr;
}

- (NSButton*)createCheckboxButton:(PGPOUICheckboxDesc*)description parentCheckbox:(NSButton*)parent
{
	NSButton*		checkbox;
	
	if (!description)
		return nil;
	
	checkbox = [[[NSButton alloc] init] autorelease];
	[checkbox setButtonType:NSSwitchButton];
	[[checkbox cell] setFont:[NSFont systemFontOfSize:[NSFont smallSystemFontSize]]];
	[[checkbox cell] setControlSize:NSSmallControlSize];
	[checkbox setTitle:[NSString stringWithUTF8String:description->title]];
	[checkbox setToolTip:[NSString stringWithUTF8String:description->description]];
	[checkbox setTag:description->itemID];
	[checkbox setState:(*description->valuePtr ? NSOnState : NSOffState)];
	[checkbox setTarget:self];
	[checkbox setAction:@selector(checkboxPressed:)];
	
	[mOptionMapping setObject:[NSData dataWithBytes:&(description->valuePtr) length:sizeof(PGPUInt32*)]
					forKey:[NSNumber numberWithInt:description->itemID]];
	
	if (parent)
		[[parent cell] setRepresentedObject:[NSData dataWithBytes:&checkbox length:sizeof(NSButton*)]];
	
	[self checkboxPressed:checkbox];
	
	return checkbox;
}

- (NSTextField*)createPopUpTitle:(PGPOUIPopupListDesc*)description
{
	NSTextField*			popupTitle;
	
	if (!description)
		return nil;
	
	popupTitle = [[[NSTextField alloc] init] autorelease];
	[popupTitle setFont:[NSFont systemFontOfSize:[NSFont smallSystemFontSize]]];
	[popupTitle setEditable:NO];
	[popupTitle setSelectable:NO];
	[popupTitle setBezeled:NO];
	[popupTitle setBordered:NO];
	[popupTitle setDrawsBackground:NO];
	[popupTitle setStringValue:[NSString stringWithUTF8String:description->title]];
	
	return popupTitle;
}

- (NSPopUpButton*)createPopUpButton:(PGPOUIPopupListDesc*)description
{
	NSPopUpButton*			popupButton;
	NSRect					frame = NSZeroRect;
	NSMutableArray*			itemStrings = [[[NSMutableArray alloc] init] autorelease];
	NSString*				string;
	NSAttributedString*		itemString;
	NSEnumerator*			itemEnumerator;
	PGPUInt32				i;
	
	if (!description)
		return nil;
	
	for (i = 0; i < description->numListItems; ++i)
	{
		NSDictionary*	attributes = [NSDictionary dictionaryWithObject:[NSFont systemFontOfSize:[NSFont smallSystemFontSize]]
												   forKey:NSFontAttributeName];
												   
		string = [NSString stringWithUTF8String:description->listItems[i]];
		itemString = [[[NSAttributedString alloc] initWithString:string attributes:attributes] autorelease];
		
		if ([itemString size].width > NSWidth(frame))
			frame.size.width = [itemString size].width;
			
		[itemStrings addObject:itemString];
	}
	
	frame.size.height = kPGPOptionsDialogPopUpButtonHeight;
	frame.size.width += kPGPOptionsDialogPopUpButtonMinWidth;
	
	popupButton = [[[NSPopUpButton alloc] initWithFrame:frame pullsDown:NO] autorelease];
	[[popupButton cell] setFont:[NSFont systemFontOfSize:[NSFont smallSystemFontSize]]];
	[[popupButton cell] setControlSize:NSSmallControlSize];
	[popupButton setToolTip:[NSString stringWithUTF8String:description->description]];
	[popupButton setTag:description->itemID];
	[popupButton setState:(*description->valuePtr ? NSOnState : NSOffState)];
	[popupButton setTarget:self];
	[popupButton setAction:@selector(popupButtonPressed:)];
	[popupButton setPullsDown:NO];
		
	[mOptionMapping setObject:[NSData dataWithBytes:&(description->valuePtr) length:sizeof(PGPUInt32*)]
					forKey:[NSNumber numberWithInt:description->itemID]];
	
	itemEnumerator = [itemStrings objectEnumerator];
	i = 0;
	
	while (itemString = [itemEnumerator nextObject])
		[popupButton insertItemWithTitle:[itemString string] atIndex:i++];

	[popupButton selectItemAtIndex:*(description->valuePtr)];
	[self popupButtonPressed:popupButton];
	
	return popupButton;
}

- (IBAction)checkboxPressed:(id)sender
{
	NSData*			data;
	NSButton*		child = nil;
	PGPUInt32*		valuePtr;
	
	if ([[sender cell] representedObject])
		child = *((NSButton**) [(NSData*) [[sender cell] representedObject] bytes]);
	
	if (child)
	{
		if ([sender state] == NSOnState)
			[child setEnabled:YES];
		else
			[child setEnabled:NO];
	}
	
	if (data = [mOptionMapping objectForKey:[NSNumber numberWithInt:[sender tag]]])
	{
		valuePtr = *((PGPUInt32**) [data bytes]);
		
		if (valuePtr)
			*valuePtr = ([sender state] == NSOnState ? 1 : 0);
	}
}

- (IBAction)popupButtonPressed:(id)sender
{
	NSData*			data;
	PGPUInt32*		valuePtr;

	if (data = [mOptionMapping objectForKey:[NSNumber numberWithInt:[sender tag]]])
	{
		valuePtr = *((PGPUInt32**) [data bytes]);
		
		if (valuePtr)
			*valuePtr = [sender indexOfSelectedItem];
	}
}

- (void)dealloc
{
	if (mOptionMapping)
		[mOptionMapping release];
	
	[super dealloc];
}

@end
