/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.
	
	$Id: PGPPassphraseDialogs.m,v 1.15 2002/11/08 23:05:17 jason Exp $
____________________________________________________________________________*/

#include "pgpMem.h"

#import "PGPDialogClasses.h"
#include "pgpErrors.h"
#import "CPGPPassphraseTextView.h"

#include <Carbon/Carbon.h>
#include <unistd.h>

#define CKERR		if (IsPGPError(err)) goto done

#define kPGPPassphraseDialogNib					@"PGPPassphraseDialog"
#define kPGPKeyPassphraseDialogNib				@"PGPKeyPassphraseDialog"
#define kPGPSignPassphraseDialogNib				@"PGPSignPassphraseDialog"
#define kPGPDecryptionPassphraseDialogNib		@"PGPDecrPassphraseDialog"
#define kPGPConfirmationPassphraseDialogNib		@"PGPConfPassphraseDialog"

#define kPGPNumberOfWiggles						5

#define kPGPUIString_PassphraseNotLongEnough	LSPU(@"Passphrase not long enough.", @"Passphrase not of sufficient length caption")
#define kPGPUIString_PassphraseNotLongEnoughMsg	LSPU(@"Passphrase is not of sufficient length. Please choose one with at least %d characters.", @"Passphrase not of sufficient length message")
#define kPGPUIString_PassphraseNotQuality		LSPU(@"Passphrase doesn't meet necessary level of quality.", @"Poor passphrase quality caption")
#define kPGPUIString_PassphraseNotQualityMsg	LSPU(@"Passphrase is not of sufficient quality. Please choose another.", @"Poor passphrase quality message")
#define kPGPUIString_FoundPassphraseMatch		LSPU(@"Found key matching passphrase", @"Found matching passphrase caption")
#define kPGPUIString_FoundPassphraseMatchMsg	LSPU(@"The key with user ID:\n\n%@\n\nmatches the passphrase.\n\nSign with this key instead?", @"Found matching passphrase message")
#define kPGPUIString_PassphraseMismatch			LSPU(@"Passphrases do not match up.", @"Passphrase mismatch caption")
#define kPGPUIString_PassphraseMismatchMsg		LSPU(@"The passphrase and its confirmation do not match. Please re-enter both the passphrase and the confirmation.", @"Passphrase mismatch message")
#define kPGPUIString_ReEnterPassphrase			LSPU(@"Re-enter the passphrase for the key shown above:", @"Re-Enter passphrase prompt")
#define kPGPUIString_ReEnterPassphraseForKey	LSPU(@"Re-enter the passphrase for your private key", @"Re-Enter key passphrase prompt")
#define kPGPUIString_PassphraseCached			LSPU(@"The passphrase of the currently selected key has been cached.", @"Passphrase cached")
#define kPGPUIString_SplitKey					LSPU(@"The currently selected key is a Blakely-Shamir split key. If you wish to use this key, press OK to proceed to the split key reconstitution dialog. Otherwise, select another key to use for signing.", @"Split key message")
#define kPGPUIString_NoPassphrase				LSPU(@"The currently selected key has no passphrase.", @"Key has no passphrase")
#define kPGPUIString_CannotDecryptText			LSPU(@"It is not possible to decrypt this message because your keyring does not contain any usable private key(s) corresponding to any of the above public key(s).", @"Cannot decrypt text")
#define kPGPUIString_RSA						LSPU(@"RSA", @"RSA")
#define kPGPUIString_DSA						LSPU(@"DH", @"DH")
#define kPGPUIString_DSS						LSPU(@"DSS", @"DSS")
#define kPGPUIString_EC							LSPU(@"EC", @"EC")
#define kPGPUIString_ElGamal					LSPU(@"ElGamal", @"ElGamal")
#define kPGPUIString_Unknown					LSPU(@"Unknown", @"Unknown")
#define kPGPUIString_NumUnknownKeys				LSPU(@"%d unknown key(s)", @"Number of unknown keys")

/* Can't include pgpDialogs.h until ObjC++... For the time being, we just re-declare this function. */
PGPKeyDBObjRef GetKeyForPassphrase(PGPKeySetRef keySet, const char *passphrase, PGPBoolean signing);

@implementation CPGPPassphraseDialog

- (id)init
{
	unsigned		i;
	
	if (!mLoadedNib)
		[self loadNib:kPGPPassphraseDialogNib];
	
	srandom(time(NULL));
	for (i = 0; i < kMaxPassphraseLength; ++i)
		mSpaceCountList[i] = ((unsigned long) random() % kMaxSpacesPerCharacter) + 1;
	
	return self;
}

- (void)loadNib:(NSString*)nibFile
{
    NSRect			frame;
    
	[NSBundle loadNibNamed:nibFile owner:self];
    
	[mWindow setDelegate:self];
	
	[mOptionsButton retain];
	[mCapsLockWarningField retain];
	
	[mHideTypingCheckbox setState:NSOnState];
	
	[mScrollView setBorderType:NSBezelBorder];
	[mScrollView setHasVerticalScroller:YES];
	
	frame = [mScrollView frame];
	
	frame.size = [NSScrollView contentSizeForFrameSize:frame.size
							   hasHorizontalScroller:NO
							   hasVerticalScroller:YES
							   borderType:NSBezelBorder];
	
	if (!mPassphraseView)
	{
		mPassphraseView = [[CPGPPassphraseTextView alloc] initWithFrame:NSMakeRect(0, 0, NSWidth(frame), NSHeight(frame))];
		[mPassphraseView setAutoresizingMask:(NSViewHeightSizable | NSViewWidthSizable)];
		[mPassphraseView setAutoresizesSubviews:NO];
		[mPassphraseView setDelegate:self];
		[[mPassphraseView layoutManager] setExtraLineFragmentRect:NSMakeRect(0, 0, NSWidth(frame), 14)
										 usedRect:NSMakeRect(0, 0, 10, 14)
										 textContainer:[mPassphraseView textContainer]];
	}
	
	if (!mSpaceView)
	{
		mSpaceView = [[CPGPPassphraseTextView alloc] initWithFrame:NSMakeRect(0, 0, NSWidth(frame), NSHeight(frame))];
		[mSpaceView setAutoresizingMask:(NSViewHeightSizable | NSViewWidthSizable)];
		[mSpaceView setAutoresizesSubviews:NO];
		[mSpaceView setDelegate:self];
		[[mSpaceView layoutManager] setExtraLineFragmentRect:NSMakeRect(0, 0, NSWidth(frame), 14)
									usedRect:NSMakeRect(0, 0, 10, 14)
									textContainer:[mSpaceView textContainer]];
	}
	
	[self hideText:nil];
	
	if ([mHideTypingCheckbox state] == NSOnState)
	{
		[mWindow setInitialFirstResponder:mSpaceView];
		[mWindow makeFirstResponder:mSpaceView];
	}
	else
	{
		[mWindow setInitialFirstResponder:mPassphraseView];
		[mWindow makeFirstResponder:mPassphraseView];
	}
}

- (void)setPrompt:(NSString*)prompt
{
	NSSize		cellSize;
	NSRect		windowRect = [[mWindow contentView] frame];
	NSRect		originalPromptRect = [mPrompt frame];
	NSRect		newPromptRect;
	
	if (prompt)
	{
		[mPrompt setStringValue:prompt];
		cellSize = [[mPrompt cell] cellSize];
		
		if (cellSize.width > NSWidth(originalPromptRect))
		{
			cellSize.height = NSHeight(originalPromptRect) * (int) (cellSize.width / NSWidth(originalPromptRect) + 1);
			
			newPromptRect.origin.x = originalPromptRect.origin.x;
			newPromptRect.origin.y = originalPromptRect.origin.y;
			newPromptRect.size.width = NSWidth(originalPromptRect);
			newPromptRect.size.height = cellSize.height;			
			
			windowRect.size.height += (cellSize.height - NSHeight(originalPromptRect));
			[mWindow setContentSize:windowRect.size];
			
			[mPrompt setFrame:newPromptRect];
		}
	}
	else
		[mPrompt setStringValue:@""];
}

- (char**)passphrasePtr
{
	return mPassphrasePtr;
}

- (void)setPassphrasePtr:(char**)passphrasePtr
{
	mPassphrasePtr = passphrasePtr;
}

- (PGPOptionListRef)dialogOptions
{
	return mDialogOptions;
}

- (void)setDialogOptions:(PGPOptionListRef)dialogOptions
{
	mDialogOptions = dialogOptions;
}

- (PGPUInt32)minPassphraseLength
{
	return mMinPassphraseLength;
}

- (void)setMinPassphraseLength:(PGPUInt32)minPassphraseLength
{
	mMinPassphraseLength = minPassphraseLength;
}

- (PGPUInt32)minPassphraseQuality
{
	return mMinPassphraseQuality;
}

- (void)setMinPassphraseQuality:(PGPUInt32)minPassphraseQuality
{
	mMinPassphraseQuality = minPassphraseQuality;
}

- (BOOL)passphraseLengthAndQualityOk:(const char*)passphrase;
{
	NSString*		string;
	
	if (!passphrase)
		return NO;
	
	string = [NSString stringWithUTF8String:passphrase];
	
	if (mMinPassphraseLength && ([string length] < mMinPassphraseLength))
	{
		NSBeginAlertSheet(kPGPUIString_PassphraseNotLongEnough,
						  kPGPUIString_OK,	/* defaultButton */
						  nil,				/* alternateButton */
						  nil,				/* otherButton */
						  mWindow,			/* docWindow */
						  nil,				/* modalDelegate */
						  nil,				/* willEndSelector */
						  nil,				/* didEndSelector */
						  nil,				/* contextInfo */
						  [NSString stringWithFormat:kPGPUIString_PassphraseNotLongEnoughMsg, mMinPassphraseLength]);
						  
		return NO;
	}
	
	if (mMinPassphraseQuality && (pgpEstimatePassphraseQuality(passphrase) < mMinPassphraseQuality))
	{
		NSBeginAlertSheet(kPGPUIString_PassphraseNotQuality,
						  kPGPUIString_OK,	/* defaultButton */
						  nil,				/* alternateButton */
						  nil,				/* otherButton */
						  mWindow,			/* docWindow */
						  nil,				/* modalDelegate */
						  nil,				/* willEndSelector */
						  nil,				/* didEndSelector */
						  nil,				/* contextInfo */
						  kPGPUIString_PassphraseNotQualityMsg);
						  
		return NO;
	}
	
	return YES;
}

- (void)updatePassphraseQuality:(PGPUInt32)quality sender:(id)sender
{
	mPassphraseQuality = quality;
}

- (PGPUInt32)dialogTimeout
{
	return mDialogTimeout;
}

- (void)setDialogTimeout:(PGPUInt32)timeout
{
	mDialogTimeout = timeout;
}

- (void)startTimer
{
	if (mDialogTimeout)
	{
		if (mDialogTimeoutTimer)
		{
			[mDialogTimeoutTimer invalidate];
			[mDialogTimeoutTimer release];
		}

		/*
		In order for our timer to work while running a modal dialog, we need to
		install it before the modal session has started, and we need to install
		it in NSModalPanelRunLoopMode.
		*/
		mDialogTimeoutTimer = [NSTimer timerWithTimeInterval:(mDialogTimeout * 60)
									   target:self
									   selector:@selector(dialogTimedOut:)
									   userInfo:nil
									   repeats:NO];
		
		[mDialogTimeoutTimer retain];
									   							   
		[[NSRunLoop currentRunLoop] addTimer:mDialogTimeoutTimer forMode:NSModalPanelRunLoopMode];
	}
}

- (void)resetTimer
{
	[self startTimer];
}

- (void)dialogTimedOut:(NSTimer*)timer
{
	if (mDialogTimeoutTimer)
	{
		[mDialogTimeoutTimer release];
		mDialogTimeoutTimer = nil;
	}
	
	[self cancel:timer];
}

- (PGPError)runDialog
{
	KeyMap		keyboardState;
	
	if (mDialogTimeout)
		[self startTimer];
	
	if ([self dialogOptions])
		[mOptionsButton setEnabled:YES];
	else
	{
		[mOptionsButton removeFromSuperview];
		[mOptionsButton setEnabled:NO];
	}
	
	[mCapsLockWarningField removeFromSuperview];	/* Be sure to retain this elsewhere if you plan on redisplaying it */
	
	GetKeys(keyboardState);
	
	if (keyboardState[1] & 0x2)
		[self capsLockStatusDidChange:YES];
	
	return [super runDialog];
}

- (void)close
{
	[mPassphraseView clearPassphrase];
	[mSpaceView clearPassphrase];
	
	[NSApp stopModal];
	[mWindow close];
}

- (IBAction)hideText:(id)sender
{
	if ([mHideTypingCheckbox state] == NSOnState)
	{
		[mScrollView setDocumentView:mSpaceView];
		[mWindow makeFirstResponder:mSpaceView];
	}
	else
	{
		[mScrollView setDocumentView:mPassphraseView];
		[mWindow makeFirstResponder:mPassphraseView];
	}
}

- (IBAction)cancel:(id)sender
{
	mReturnCode = kPGPError_UserAbort;
	
	[self close];
}

- (IBAction)ok:(id)sender
{
	const char*		utf8Passphrase = [[mPassphraseView string] UTF8String];
	unsigned		passphraseLength = strlen(utf8Passphrase);
	
	if (!mPassphrasePtr)
	{
		PGPLog((@"Missing passphrase ptr..."));
		mReturnCode = kPGPError_BadParams;
		[self close];
		
		return;
	}
	
	*mPassphrasePtr = PGPNewSecureData(PGPPeekContextMemoryMgr(mProxyObject->context), passphraseLength + 1, 0);
	
	if (!*mPassphrasePtr)
	{
		PGPLog((@"PGPNewSecureData failed to allocate the passphrase buffer"));
		mReturnCode = kPGPError_OutOfMemory;
		[self close];
		
		return;
	}
	
	pgpCopyMemory(utf8Passphrase, *mPassphrasePtr, passphraseLength);
	*((*mPassphrasePtr) + passphraseLength) = '\0';
			
	if ([self passphraseLengthAndQualityOk:*mPassphrasePtr])
	{
		mReturnCode = kPGPError_NoErr;
		[self close];
	}
	else
	{
		memset((char*) *mPassphrasePtr, '\0', passphraseLength);
		PGPFreeData(*mPassphrasePtr);
		*mPassphrasePtr = NULL;			
		[mPassphraseView clearPassphrase];
		[mSpaceView clearPassphrase];
		[self updatePassphraseQuality:0 sender:self];
		[self hideText:self];
	}
}

- (BOOL)passphraseView:(CPGPPassphraseTextView*)passphraseView shouldInsertText:(NSString*)string
{
	unsigned		numChars;
	unsigned		numSpaces;
	unsigned		spaceIndex;
	unsigned		i;
	char			spaceStr[kMaxSpacesPerCharacter + 1];	
	
	if (!string)
		return NO;
	
	numChars = [string length];
	
	/* If we've hit the max passphrase length, disable any further input. */
	if ([[mPassphraseView string] length] + numChars > kMaxPassphraseLength)
		return NO;
	
	spaceIndex = [[mPassphraseView string] length] ? [[mPassphraseView string] length] : 0;
	
	for (i = 0; i < numChars; ++i, ++spaceIndex)
	{
		numSpaces = mSpaceCountList[spaceIndex];
		memset(spaceStr, ' ', numSpaces);
		spaceStr[numSpaces] = '\0';
		
		if ((passphraseView == mSpaceView) || (passphraseView == mPassphraseView))
			[mSpaceView superInsertText:[NSString stringWithFormat:@"%s", spaceStr]];
	}
	
	if (passphraseView == mSpaceView)
	{
		[mPassphraseView superInsertText:string];
		
		return NO;
	}
	
	return YES;
}

- (BOOL)passphraseView:(CPGPPassphraseTextView*)passphraseView shouldDeleteBackward:(id)sender
{
	unsigned		numSpaces;
	unsigned		spaceIndex;
	unsigned		passLength;
	unsigned		spaceLength;
	unsigned		i;
	
	passLength = [[mPassphraseView string] length];
	spaceLength = [[mSpaceView string] length];
	
	if (((passphraseView == mSpaceView) || (passphraseView == mPassphraseView)) && passLength)
	{
		[mPassphraseView superDeleteBackward:self];
		
		spaceIndex = passLength - 1;
		numSpaces = mSpaceCountList[spaceIndex];
		
		for (i = 0; i < numSpaces; ++i)
			[mSpaceView superDeleteBackward:self];
		
		return NO;
	}
	
	return YES;
}

- (BOOL)shouldPassphraseViewDisplayMarkedText:(CPGPPassphraseTextView*)passphraseView
{
	if (passphraseView == mSpaceView)
		return NO;
	
	return YES;
}

- (void)capsLockStatusDidChange:(BOOL)enabled
{
	if (enabled)
		[[mWindow contentView] addSubview:mCapsLockWarningField];
	else
		[mCapsLockWarningField removeFromSuperview];
}

- (BOOL)windowShouldClose:(id)sender
{
	[self cancel:sender];

	return NO;
}

- (void)dealloc
{
	[mWindow setDelegate:nil];
	if (mDialogTimeoutTimer)
	{
		[mDialogTimeoutTimer invalidate];
		[mDialogTimeoutTimer release];
	}
	
	if (mSpaceView)
	{
		[mSpaceView removeFromSuperview];
		[mSpaceView release];
	}
	
	if (mPassphraseView)
	{
		[mPassphraseView removeFromSuperview];
		[mPassphraseView release];
	}
	
	if (mOptionsButton)
		[mOptionsButton release];
	
	if (mCapsLockWarningField)
		[mCapsLockWarningField release];
	
	[super dealloc];
}

@end

/*
---------------------------------------------------------------
*/

@implementation CPGPKeyPassphraseDialog

- (id)init
{
	BOOL			loadNib;
	
	loadNib = mLoadedNib ? NO : YES;
	
	if (loadNib)
		mLoadedNib = YES;
		
	[super init];	/* Our superclass will not load its nib if mLoadedNib is true. */
	
	if (loadNib)
		[self loadNib:kPGPKeyPassphraseDialogNib];
	
	return self;
}

- (void)loadNib:(NSString*)nibFile
{
    NSRect			frame;
    
	[NSBundle loadNibNamed:nibFile owner:self];
    
	[mWindow setDelegate:self];
			
	[mOptionsButton retain];
	[mCapsLockWarningField retain];
	
	[mHideTypingCheckbox setState:NSOnState];

	[mScrollView setBorderType:NSBezelBorder];
    [mScrollView setHasVerticalScroller:YES];
    
	frame = [mScrollView frame];
	
	frame.size = [NSScrollView contentSizeForFrameSize:frame.size
							   hasHorizontalScroller:NO
							   hasVerticalScroller:YES
							   borderType:NSBezelBorder];
	
	if (!mPassphraseView)
	{
		mPassphraseView = [[CPGPPassphraseTextView alloc] initWithFrame:NSMakeRect(0, 0, NSWidth(frame), NSHeight(frame))];
		[mPassphraseView setAutoresizingMask:(NSViewHeightSizable | NSViewWidthSizable)];
		[mPassphraseView setAutoresizesSubviews:NO];
		[mPassphraseView setDelegate:self];
		[[mPassphraseView layoutManager] setExtraLineFragmentRect:NSMakeRect(0, 0, NSWidth(frame), 14)
								   usedRect:NSMakeRect(0, 0, 10, 14)
								   textContainer:[mPassphraseView textContainer]];
	}
	
	if (!mSpaceView)
	{
		mSpaceView = [[CPGPPassphraseTextView alloc] initWithFrame:NSMakeRect(0, 0, NSWidth(frame), NSHeight(frame))];
		[mSpaceView setAutoresizingMask:(NSViewHeightSizable | NSViewWidthSizable)];
		[mSpaceView setAutoresizesSubviews:NO];
		[mSpaceView setDelegate:self];
		[[mSpaceView layoutManager] setExtraLineFragmentRect:NSMakeRect(0, 0, NSWidth(frame), 14)
								   usedRect:NSMakeRect(0, 0, 10, 14)
								   textContainer:[mSpaceView textContainer]];
	}
	
	[self hideText:nil];
	
	if ([mHideTypingCheckbox state] == NSOnState)
		[mWindow setInitialFirstResponder:mSpaceView];
	else
		[mWindow setInitialFirstResponder:mPassphraseView];
	
	mLoadedNib = YES;
}

- (PGPBoolean)verifyPassphrase:(const char*)passphrase
{
	PGPBoolean		valid;
	
	if (!passphrase)
		return FALSE;
		
	valid = PGPPassphraseIsValid(mDefaultKey,
								 PGPOPassphrase(mProxyObject->context, passphrase),
								 mCache ?
									PGPOCachePassphrase(mProxyObject->context, mCacheTimeout, mCacheGlobal) :
									PGPONullOption(mProxyObject->context),
								 PGPOLastOption(mProxyObject->context));
	
	return valid;
}

- (void)rejectPassphraseWithMessage:(NSString*)message
{
	unsigned	i;
	NSRect		originalRect = [mWindow frame];
	NSRect		wiggleRect = NSZeroRect;
	
	NSBeep();
	
	for (i = 0; i < kPGPNumberOfWiggles; ++i)
	{
		wiggleRect = originalRect;
		wiggleRect.origin.x -= 5;
		[mWindow setFrame:wiggleRect display:YES animate:NO];
		usleep(50000);
		wiggleRect = originalRect;
		wiggleRect.origin.x += 5;
		[mWindow setFrame:wiggleRect display:YES animate:NO];
	}
	
	[mWindow setFrame:originalRect display:YES animate:NO];
	
	[mPrompt setStringValue:message];
	[mPrompt displayIfNeeded];
	
	[mPassphraseView clearPassphrase];
	[mSpaceView clearPassphrase];
}

- (PGPBoolean)verifiesPassphrase
{
	return mVerifyPassphrase;
}

- (void)setVerifiesPassphrase:(PGPBoolean)flag
{
	mVerifyPassphrase = flag;
}

- (PGPBoolean)cache
{
	return mCache;
}

- (void)setCache:(PGPBoolean)flag
{
	mCache = flag;
}

- (PGPBoolean)cacheGlobal
{
	return mCacheGlobal;
}

- (void)setCacheGlobal:(PGPBoolean)flag
{
	mCacheGlobal = flag;
}

- (PGPUInt32)cacheTimeout
{
	return mCacheTimeout;
}

- (void)setCacheTimeout:(PGPUInt32)cacheTimeout
{
	mCacheTimeout = cacheTimeout;
}

- (PGPError)runDialog
{
	NSString*				userID = nil;
	
	if (mDefaultKey)
	{
		userID = PGPUIGetPrimaryUserIDName(mDefaultKey);
		
		if (userID)
			[mKeyField setStringValue:userID];
		else
			[mKeyField setStringValue:@""];
	}
	else
		[mKeyField setStringValue:@""];
	
	if ([mHideTypingCheckbox superview])
	{
		if ([mHideTypingCheckbox state] == NSOnState)
			[mWindow makeFirstResponder:mSpaceView];
		else
			[mWindow makeFirstResponder:mPassphraseView];
	}
	
	return [super runDialog];
}

- (IBAction)ok:(id)sender
{
	const char*		utf8Passphrase = [[mPassphraseView string] UTF8String];
	unsigned		passphraseLength = strlen(utf8Passphrase);
	
	if (!mPassphrasePtr)
	{
		PGPLog((@"Missing passphrase ptr..."));
		mReturnCode = kPGPError_BadParams;
		[self close];
		
		return;
	}
	
	*mPassphrasePtr = PGPNewSecureData(PGPPeekContextMemoryMgr(mProxyObject->context), passphraseLength + 1, 0);
	
	if (!*mPassphrasePtr)
	{
		PGPLog((@"PGPNewSecureData failed to allocate the passphrase buffer"));
		mReturnCode = kPGPError_OutOfMemory;
		[self close];
		
		return;
	}
	
	pgpCopyMemory(utf8Passphrase, *mPassphrasePtr, passphraseLength);
	*((*mPassphrasePtr) + passphraseLength) = '\0';
	
	if (![self passphraseLengthAndQualityOk:*mPassphrasePtr])
	{
		memset((char*) *mPassphrasePtr, '\0', passphraseLength);
		PGPFreeData(*mPassphrasePtr);
		*mPassphrasePtr = NULL;			
		[mPassphraseView clearPassphrase];
		[mSpaceView clearPassphrase];
		[self updatePassphraseQuality:0 sender:self];
		[self hideText:self];
		
		return;
	}
	
	if (mVerifyPassphrase)
	{
		if ([self verifyPassphrase:*mPassphrasePtr])
		{
			mReturnCode = kPGPError_NoErr;
			[self close];
		}
		else
		{
			memset((char*) *mPassphrasePtr, '\0', passphraseLength);
			PGPFreeData(*mPassphrasePtr);
			*mPassphrasePtr = NULL;	
			[self rejectPassphraseWithMessage:kPGPUIString_ReEnterPassphrase];
		}
	}
	else
	{
		mReturnCode = kPGPError_NoErr;
		[self close];
	}
}

- (void)dealloc
{
	[super dealloc];
}

@end

/*
---------------------------------------------------------------
*/

@implementation CPGPKeySetPassphraseDialog

- (PGPBoolean)findMatchingKey
{
	return mFindMatchingKey;
}

- (void)setFindMatchingKey:(PGPBoolean)flag
{
	mFindMatchingKey = flag;
}

- (PGPKeySetRef)keySet
{
	return mKeySet;
}

- (void)setKeySet:(PGPKeySetRef)keySet
{
	mKeySet = keySet;
}

- (PGPKeyDBObjRef*)passphraseKeyPtr
{
	return mPassphraseKeyPtr;
}

- (void)setPassphraseKeyPtr:(PGPKeyDBObjRef*)keyPtr
{
	mPassphraseKeyPtr = keyPtr;
	
	if (mPassphraseKeyPtr)
		*mPassphraseKeyPtr = kInvalidPGPKeyDBObjRef;
}

- (NSString*)generateSigningKeyString:(PGPKeyDBObjRef)key
{
	NSString*			userID = nil;
	PGPUInt32			algorithm = kPGPPublicKeyAlgorithm_Invalid;
	PGPUInt32			keyBits = 0;
	NSMutableString*	keyString = nil;
	
	if (!key)
		return nil;

	userID = PGPUIGetPrimaryUserIDName(key);
	PGPGetKeyDBObjNumericProperty(key, kPGPKeyProperty_AlgorithmID, &algorithm);
	PGPGetKeyDBObjNumericProperty(key, kPGPKeyProperty_Bits, &keyBits);
	
	if (userID)
	{
		keyString = [[NSMutableString alloc] init];
		[keyString appendFormat:@"%@   (", userID];
		
		switch (algorithm)
		{
			case kPGPPublicKeyAlgorithm_RSA:
			case kPGPPublicKeyAlgorithm_RSAEncryptOnly:
			case kPGPPublicKeyAlgorithm_RSASignOnly:
				[keyString appendFormat:@"%@", kPGPUIString_RSA];
				break;
			case kPGPPublicKeyAlgorithm_ElGamal:
				[keyString appendFormat:@"%@", kPGPUIString_ElGamal];
				break;
			case kPGPPublicKeyAlgorithm_DSA:
				[keyString appendFormat:@"%@", kPGPUIString_DSS];
				break;
			case kPGPPublicKeyAlgorithm_ECEncrypt:
			case kPGPPublicKeyAlgorithm_ECSign:
				[keyString appendFormat:@"%@", kPGPUIString_EC];
				break;
			default:
				[keyString appendFormat:@"%@", kPGPUIString_Unknown];
				break;
		}
		if (keyBits)
			[keyString appendFormat:@"/%d", keyBits];
		[keyString appendString:@")"];
	}
	
	return keyString;
}

- (NSString*)generateEncryptionKeyString:(PGPKeyDBObjRef)key
{
	NSString*			userID = nil;
	PGPUInt32			algorithm = kPGPPublicKeyAlgorithm_Invalid;
	PGPUInt32			keyBits = 0,
						subBits = 0;
	NSMutableString*	keyString = nil;
	PGPKeyIterRef		keyIter = kInvalidPGPKeyIterRef;
	PGPKeyDBObjRef		subKey = kInvalidPGPKeyDBObjRef;
	
	if (!key)
		return nil;

	userID = PGPUIGetPrimaryUserIDName(key);
	PGPGetKeyDBObjNumericProperty(key, kPGPKeyProperty_AlgorithmID, &algorithm);
	PGPGetKeyDBObjNumericProperty(key, kPGPKeyProperty_Bits, &keyBits);
	if (IsntPGPError(PGPNewKeyIterFromKeyDB(PGPPeekKeyDBObjKeyDB(key), &keyIter)))
	{
		PGPKeyIterSeek(keyIter, key);
		if (IsntPGPError(PGPKeyIterNextKeyDBObj(keyIter, kPGPKeyDBObjType_SubKey, &subKey)))
			PGPGetKeyDBObjNumericProperty( subKey, kPGPSubKeyProperty_Bits, &subBits );
		PGPFreeKeyIter( keyIter );
	}

	if (userID)
	{
		keyString = [[NSMutableString alloc] init];
		[keyString appendFormat:@"%@   (", userID];
		
		switch (algorithm)
		{
			case kPGPPublicKeyAlgorithm_RSA:
			case kPGPPublicKeyAlgorithm_RSAEncryptOnly:
			case kPGPPublicKeyAlgorithm_RSASignOnly:
				[keyString appendFormat:@"%@", kPGPUIString_RSA];
				break;
			case kPGPPublicKeyAlgorithm_ElGamal:
				[keyString appendFormat:@"%@", kPGPUIString_ElGamal];
				break;
			case kPGPPublicKeyAlgorithm_DSA:
				[keyString appendFormat:@"%@", kPGPUIString_DSA];
				break;
			case kPGPPublicKeyAlgorithm_ECEncrypt:
			case kPGPPublicKeyAlgorithm_ECSign:
				[keyString appendFormat:@"%@", kPGPUIString_EC];
				break;
			default:
				[keyString appendFormat:@"%@", kPGPUIString_Unknown];
				break;
		}
		if (subBits)
			[keyString appendFormat:@"/%d", subBits];
		else if (keyBits)
			[keyString appendFormat:@"/%d", keyBits];
		[keyString appendString:@")"];
	}
	
	return keyString;
}

@end

/*
---------------------------------------------------------------
*/

@implementation CPGPSignPassphraseDialog

- (id)init
{
	BOOL			loadNib;
	
	loadNib = mLoadedNib ? NO : YES;
	
	if (loadNib)
		mLoadedNib = YES;
		
	[super init];	/* Our superclass will not load its nib if mLoadedNib is true. */
		
	mDefaultKey = kInvalidPGPKeyDBObjRef;

	if (loadNib)
		[self loadNib:kPGPSignPassphraseDialogNib];
			
	return self;
}

- (void)loadNib:(NSString*)nibFile
{
    NSRect		frame;
	
	[NSBundle loadNibNamed:nibFile owner:self];
    
	[mPrompt retain];
	[mContainer retain];
	[mButtonContainer retain];
	[mOptionsButton retain];
	[mOptionsBox retain];
	[mCapsLockWarningField retain];
	[mHideTypingCheckbox retain];
	[mMessageText retain];
	[mScrollView retain];
	
	[mHideTypingCheckbox setState:NSOnState];
	
	[mScrollView setBorderType:NSBezelBorder];
	[mScrollView setHasVerticalScroller:YES];
	
	frame = [mScrollView frame];
	
	frame.size = [NSScrollView contentSizeForFrameSize:frame.size
							   hasHorizontalScroller:NO
							   hasVerticalScroller:YES
							   borderType:NSBezelBorder];
	
	if (!mPassphraseView)
	{
		mPassphraseView = [[CPGPPassphraseTextView alloc] initWithFrame:NSMakeRect(0, 0, NSWidth(frame), NSHeight(frame))];
		[mPassphraseView setAutoresizingMask:(NSViewHeightSizable | NSViewWidthSizable)];
		[mPassphraseView setAutoresizesSubviews:NO];
		[mPassphraseView setDelegate:self];
		[[mPassphraseView layoutManager] setExtraLineFragmentRect:NSMakeRect(0, 0, NSWidth(frame), 14)
								   usedRect:NSMakeRect(0, 0, 10, 14)
								   textContainer:[mPassphraseView textContainer]];
	}
	
	if (!mSpaceView)
	{
		mSpaceView = [[CPGPPassphraseTextView alloc] initWithFrame:NSMakeRect(0, 0, NSWidth(frame), NSHeight(frame))];
		[mSpaceView setAutoresizingMask:(NSViewHeightSizable | NSViewWidthSizable)];
		[mSpaceView setAutoresizesSubviews:NO];
		[mSpaceView setDelegate:self];
		[[mSpaceView layoutManager] setExtraLineFragmentRect:NSMakeRect(0, 0, NSWidth(frame), 14)
								   usedRect:NSMakeRect(0, 0, 10, 14)
								   textContainer:[mSpaceView textContainer]];
	}
	
	[self hideText:nil];
	
	mLoadedNib = YES;
}

- (void)setKeySet:(PGPKeySetRef)keySet
{
	[super setKeySet:keySet];
	
	[self buildKeyList];
	[self signingKeyChanged:self];
}

- (PGPError)runDialog
{
	if (mDialogOptions)
	{
		mOptionsDialog = [[CPGPOptionsDialog alloc] initWithOptions:mDialogOptions];
		
		if (mOptionsView = [mOptionsDialog optionsView])
		{
			[mOptionsBox setContentView:mOptionsView];
			[mOptionsBox sizeToFit];
			[mOptionsBox removeFromSuperview];
		}
	}
	else
	{
		[mOptionsButton removeFromSuperview];
		[mOptionsBox removeFromSuperview];
	}
    
	[self signingKeyChanged:nil];
	
	return [super runDialog];
}

- (PGPError)buildKeyList
{
	PGPError			err;
	PGPKeyListRef		keyList = NULL;
	PGPKeyIterRef		keyIter = NULL;
	PGPKeyDBObjRef		key = NULL;
	PGPBoolean			isSecret;
	PGPBoolean			isRevoked;
	PGPBoolean			hasExpired;
	PGPBoolean			canSign;
	NSString*			keyString;
	NSMenuItem*			currentItem;
	int					selectedKeyIndex = -1;

	err = PGPOrderKeySet(mKeySet, kPGPKeyOrdering_Validity, FALSE, &keyList); CKERR;
	err = PGPNewKeyIter(keyList, &keyIter); CKERR;
	
	while (IsntPGPError(PGPKeyIterNextKeyDBObj(keyIter, kPGPKeyDBObjType_Key, &key)))
	{
		PGPGetKeyDBObjBooleanProperty(key, kPGPKeyProperty_IsSecret, &isSecret);
		
		if (isSecret)
		{
			PGPGetKeyDBObjBooleanProperty(key, kPGPKeyProperty_IsRevoked, &isRevoked);
			PGPGetKeyDBObjBooleanProperty(key, kPGPKeyProperty_IsExpired, &hasExpired);
			PGPGetKeyDBObjBooleanProperty(key, kPGPKeyProperty_CanSign, &canSign);

			if (!isRevoked && !hasExpired && canSign)
			{
				keyString = [self generateSigningKeyString:(key)];
				[mKeySelectPopup addItemWithTitle:keyString];

				if (currentItem = [mKeySelectPopup itemWithTitle:keyString])
					[currentItem setRepresentedObject:[NSData dataWithBytes:&key length:sizeof(key)]];

				if (key == mDefaultKey)
					selectedKeyIndex = [mKeySelectPopup indexOfItemWithTitle:keyString];
				[keyString release];
			}
		}
	}
	
	if ([mKeySelectPopup numberOfItems] > 1)
		[mKeySelectPopup removeItemAtIndex:0];
	
	if (selectedKeyIndex != -1)
		[mKeySelectPopup selectItemAtIndex:--selectedKeyIndex];
	else
		[mKeySelectPopup selectItemAtIndex:0];
	
done:
	if (keyList)
		PGPFreeKeyList(keyList);
		
	if (keyIter)
		PGPFreeKeyIter(keyIter);

	return err;
}

- (PGPError)validatePassphrase:(const char*)passphrase forKey:(PGPKeyDBObjRef)key
{
	PGPBoolean		valid;
	NSString*		messageString;
	NSString*		userID = nil;
	
	if (!passphrase || !key)
		return kPGPError_BadParams;
	
	valid = PGPPassphraseIsValid(key,
								 PGPOPassphrase(mProxyObject->context, passphrase),
								 mCache ?
									PGPOCachePassphrase(mProxyObject->context, mCacheTimeout, mCacheGlobal) :
									PGPONullOption(mProxyObject->context),
								 PGPOLastOption(mProxyObject->context));
	
	if (valid)
	{
		*mPassphraseKeyPtr = key;
		return kPGPError_NoErr;
	}
	
	if (mFindMatchingKey)
	{
		if (key = GetKeyForPassphrase(mKeySet, passphrase, TRUE))
		{
			userID = PGPUIGetPrimaryUserIDName(key);
			
			messageString = [[[NSString alloc] initWithFormat:
								kPGPUIString_FoundPassphraseMatchMsg,
								userID] autorelease];

			NSBeginAlertSheet(kPGPUIString_FoundPassphraseMatch,
							  kPGPUIString_OK,		/* defaultButton */
							  kPGPUIString_Cancel,	/* alternateButton */
							  nil,					/* otherButton */
							  mWindow,				/* docWindow */
							  self,					/* modalDelegate */
							  nil,					/* willEndSelector */
							  @selector(sheetDidEnd:returnCode:contextInfo:),	/* didEndSelector */
							  key,					/* contextInfo */
							  messageString);
							  
			mFoundMatchingKey = YES;
			*mPassphraseKeyPtr = NULL;
			return kPGPError_NoErr;
		}
	}
	
	return kPGPError_BadPassphrase;
}

- (IBAction)signingKeyChanged:(id)sender
{
	NSMenuItem*			item = [mKeySelectPopup selectedItem];
	NSData*				representedObject;
	PGPKeyDBObjRef		key;
	PGPBoolean			needsPassphrase;
	PGPBoolean			splitKey;
	
	if (!item)
		return;
	
	if (!(representedObject = [item representedObject]))
		return;

	if (!(key = *((PGPKeyDBObjRef*) [representedObject bytes])))
		return;
			
	PGPGetKeyDBObjBooleanProperty(key, kPGPKeyProperty_NeedsPassphrase, &needsPassphrase);
	PGPGetKeyDBObjBooleanProperty(key, kPGPKeyProperty_IsSecretShared, &splitKey);
	
	if (needsPassphrase)
	{
		if (PGPPassphraseIsValid(key, PGPOLastOption(mProxyObject->context)))
		{
			[mScrollView removeFromSuperview];
			[mHideTypingCheckbox removeFromSuperview];
			[mPrompt removeFromSuperview];
			
			[mContainer addSubview:mMessageText];
			[mMessageText setStringValue:kPGPUIString_PassphraseCached];
			[mMessageText displayIfNeeded];
			
			[mWindow setInitialFirstResponder:mOkButton];
			[mWindow makeFirstResponder:mOkButton];
		}
		else if (splitKey)
		{
			[mScrollView removeFromSuperview];
			[mHideTypingCheckbox removeFromSuperview];
			[mPrompt removeFromSuperview];
			
			[mContainer addSubview:mMessageText];
			[mMessageText setStringValue:kPGPUIString_SplitKey];
			[mMessageText displayIfNeeded];
			
			[mWindow setInitialFirstResponder:mOkButton];
			[mWindow makeFirstResponder:mOkButton];
		}
		else
		{
			if (![mScrollView superview])
				[mContainer addSubview:mScrollView];
			
			if (![mHideTypingCheckbox superview])
				[mContainer addSubview:mHideTypingCheckbox];
			
			if (![mPrompt superview])
				[mContainer addSubview:mPrompt];
			
			[mMessageText removeFromSuperview];
			[mMessageText setStringValue:@""];
			
			[mPassphraseView clearPassphrase];
			[mSpaceView clearPassphrase];
			
			[mHideTypingCheckbox setState:NSOnState];
			[self hideText:self];
		}
	}
	else
	{
		[mScrollView removeFromSuperview];
		[mHideTypingCheckbox removeFromSuperview];
		[mPrompt removeFromSuperview];
		
		[mContainer addSubview:mMessageText];
		[mMessageText setStringValue:kPGPUIString_NoPassphrase];
		[mMessageText displayIfNeeded];
		
		[mWindow setInitialFirstResponder:mOkButton];
		[mWindow makeFirstResponder:mOkButton];
	}
	
	if ([mHideTypingCheckbox superview] && [mScrollView superview])
	{
		[mKeySelectPopup setNextKeyView:mHideTypingCheckbox];
		
		if ([mHideTypingCheckbox state] == NSOnState)
			[mHideTypingCheckbox setNextKeyView:mSpaceView];
		else
			[mHideTypingCheckbox setNextKeyView:mPassphraseView];
		
		[[mHideTypingCheckbox nextKeyView] setNextKeyView:mCancelButton];
	}
	else
		[mKeySelectPopup setNextKeyView:mCancelButton];
	
	[mCancelButton setNextKeyView:mOkButton];
	[mOkButton setNextKeyView:mKeySelectPopup];
}

- (IBAction)showOptions:(id)sender
{
	NSRect			frame;
	NSView*			contentView;
	
	if ([mOptionsBox superview])
	{
		frame = NSMakeRect(0, 0, NSWidth([[mWindow contentView] frame]), NSHeight([[mWindow contentView] frame]) - NSHeight([mOptionsBox frame]));
		contentView = [[[NSView alloc] initWithFrame:frame] autorelease];

		/* Place the control container to the top of the content view */
		[mContainer removeFromSuperview];
		frame = [mContainer frame];
		frame.origin.y = NSMaxY([contentView frame]) - NSHeight(frame);
		[mContainer setFrame:frame];
		[contentView addSubview:mContainer];
		
		[mOptionsBox removeFromSuperview];

		/* Place the button container at the bottom of the content view */
		[mButtonContainer removeFromSuperview];
		frame = [mButtonContainer frame];
		frame.origin.y = 0;
		[mButtonContainer setFrame:frame];
		[contentView addSubview:mButtonContainer];
		
		/* Resize the window. Set its new minSize and maxSize */
		frame = [mWindow frame];
		frame.origin.y += NSHeight([[mWindow contentView] frame]) - NSHeight([contentView frame]);
		frame.size.height -= NSHeight([[mWindow contentView] frame]);
		frame.size.height += NSHeight([contentView frame]);
		[mWindow setContentView:contentView];
		[mWindow setFrame:frame display:YES animate:NO];
		[mWindow setMinSize:NSMakeSize([mWindow minSize].width, [mWindow minSize].height - NSHeight([mOptionsBox frame]) - 20)];
		[mWindow setMaxSize:NSMakeSize([mWindow maxSize].width, [mWindow maxSize].height - NSHeight([mOptionsBox frame]) - 20)];
	}
	else
	{
		frame = NSMakeRect(0, 0, NSWidth([[mWindow contentView] frame]), NSHeight([[mWindow contentView] frame]) + NSHeight([mOptionsBox frame]));
		contentView = [[[NSView alloc] initWithFrame:frame] autorelease];
		
		/* Place the control container to the top of the content view */
		[mContainer removeFromSuperview];
		frame = [mContainer frame];
		frame.origin.y = NSMaxY([contentView frame]) - NSHeight(frame);
		[mContainer setFrame:frame];
		[contentView addSubview:mContainer];
		
		/* Place the options box underneath the control container */
		frame = [mOptionsBox frame];
		frame.origin.y = NSMinY([mContainer frame]) - NSHeight(frame) - 8;
		frame.size.width = NSWidth([[mWindow contentView] frame]) - 40;
		[mOptionsBox setFrame:frame];
		[contentView addSubview:mOptionsBox];
		
		/* Place the button container at the bottom of the content view */
		[mButtonContainer removeFromSuperview];
		frame = [mButtonContainer frame];
		frame.origin.y = 0;
		[mButtonContainer setFrame:frame];
		[contentView addSubview:mButtonContainer];
		
		/* Resize the window. Set its new minSize and maxSize */
		frame = [mWindow frame];
		frame.origin.y += NSHeight([[mWindow contentView] frame]) - NSHeight([contentView frame]);
		frame.size.height -= NSHeight([[mWindow contentView] frame]);
		frame.size.height += NSHeight([contentView frame]);
		[mWindow setContentView:contentView];
		[mWindow setFrame:frame display:YES animate:NO];
		[mWindow setMinSize:NSMakeSize([mWindow minSize].width, [mWindow minSize].height + NSHeight([mOptionsBox frame]) + 20)];
		[mWindow setMaxSize:NSMakeSize([mWindow maxSize].width, [mWindow maxSize].height + NSHeight([mOptionsBox frame]) + 20)];
	}
	
	[self hideText:nil];
}

- (IBAction)hideText:(id)sender
{
	if ([mHideTypingCheckbox state] == NSOnState)
	{
		[mScrollView setDocumentView:mSpaceView];
		[mWindow makeFirstResponder:mSpaceView];
		[mHideTypingCheckbox setNextKeyView:mSpaceView];
	}
	else
	{
		[mScrollView setDocumentView:mPassphraseView];
		[mWindow makeFirstResponder:mPassphraseView];
		[mHideTypingCheckbox setNextKeyView:mPassphraseView];
	}
}

- (IBAction)ok:(id)sender
{
	NSData*			representedObject;
	PGPKeyDBObjRef	key = NULL;
	PGPBoolean		splitKey;
	const char*		utf8Passphrase = [[mPassphraseView string] UTF8String];
	unsigned		passphraseLength = strlen(utf8Passphrase);
	PGPError		err;
	
	if (mFoundMatchingKey)
	{
		mFoundMatchingKey = NO;
		goto done;
	}
	
	if (!(representedObject = [[mKeySelectPopup selectedItem] representedObject]))
	{
		PGPLog((@"NSPopUpButton is missing a PGPKeyDBObjRef represented object"));
		mReturnCode = kPGPError_BadParams;
		goto done;
	}
	
	key = *((PGPKeyDBObjRef*) [representedObject bytes]);
	
	if (key)
	{
		PGPGetKeyDBObjBooleanProperty(key, kPGPKeyProperty_IsSecretShared, &splitKey);
		
		if (splitKey)
		{
			*mPassphraseKeyPtr = key;				
			mReturnCode = kPGPError_KeyUnusableForSignature;
			goto done;
		}
	}
	else
	{
		PGPLog((@"Represented PGPKeyDBObjRef is NULL"));
		mReturnCode = kPGPError_KeyInvalid;
		goto done;
	}
	
	if (mPassphrasePtr && !splitKey)
	{
		*mPassphrasePtr = PGPNewSecureData(PGPPeekContextMemoryMgr(mProxyObject->context), passphraseLength + 1, 0);
		
		if (!*mPassphrasePtr)
		{
			PGPLog((@"PGPNewSecureData failed to allocate the passphrase buffer"));
			mReturnCode = kPGPError_OutOfMemory;
			goto done;
		}
		
		pgpCopyMemory(utf8Passphrase, *mPassphrasePtr, passphraseLength);
		*((*mPassphrasePtr) + passphraseLength) = '\0';
		
		if (![self passphraseLengthAndQualityOk:*mPassphrasePtr])
		{
			memset((char*) *mPassphrasePtr, '\0', passphraseLength);
			PGPFreeData(*mPassphrasePtr);
			*mPassphrasePtr = NULL;			
			[mPassphraseView clearPassphrase];
			[mSpaceView clearPassphrase];
			[self updatePassphraseQuality:0 sender:self];
			[self hideText:self];
			
			return;
		}
		
		if (mVerifyPassphrase)
		{
			if (IsntPGPError(err = [self validatePassphrase:*mPassphrasePtr forKey:key]))
			{
				if (mFoundMatchingKey)
					return;
				
				if (*mPassphraseKeyPtr)
					mReturnCode = kPGPError_NoErr;
				
				goto done;
			}
			else
			{
				[self rejectPassphraseWithMessage:kPGPUIString_ReEnterPassphrase];
				memset((char*) *mPassphrasePtr, '\0', passphraseLength);
				PGPFreeData(*mPassphrasePtr);
				*mPassphrasePtr = NULL;
				
				[self hideText:self];
				
				return;
			}
		}
		else
		{
			mReturnCode = kPGPError_NoErr;
			goto done;
		}
	}
			
done:
	[self close];
	
	return;
}

- (void)capsLockStatusDidChange:(BOOL)enabled
{
	if (enabled)
		[mContainer addSubview:mCapsLockWarningField];
	else
		[mCapsLockWarningField removeFromSuperview];
}

- (void)sheetDidEnd:(NSWindow*)sheet returnCode:(int)returnCode contextInfo:(void*)contextInfo
{
	PGPKeyDBObjRef		key = contextInfo;
	
	if (returnCode == NSOKButton)
	{
		*mPassphraseKeyPtr = key;
		mReturnCode = kPGPError_NoErr;
		
		[self ok:self];
	}
	else
	{
		memset((char*) *mPassphrasePtr, '\0', strlen(*mPassphrasePtr));
		PGPFreeData(*mPassphrasePtr);
		*mPassphrasePtr = NULL;				

		mReturnCode = kPGPError_BadPassphrase;
		[self cancel:self];
	}
}

- (void)dealloc
{
	if (mPrompt)
		[mPrompt release];
	
	if (mOptionsBox)
		[mOptionsBox release];
	
	if (mOptionsDialog)
		[mOptionsDialog release];
	
	if (mContainer)
		[mContainer release];
	
	if (mButtonContainer)
		[mButtonContainer release];
	
	if (mHideTypingCheckbox)
		[mHideTypingCheckbox release];
	
	if (mMessageText)
		[mMessageText release];
	
	if (mScrollView)
		[mScrollView release];
	
	[super dealloc];
}

@end

/*
---------------------------------------------------------------
*/

@implementation CPGPDecryptionPassphraseDialog

- (id)init
{
	BOOL			loadNib;
	
	loadNib = mLoadedNib ? NO : YES;
	
	if (loadNib)
		mLoadedNib = YES;
		
	[super init];	/* Our superclass will not load its nib if mLoadedNib is true. */
		
	mDefaultKey = kInvalidPGPKeyDBObjRef;
	
	if (!mRecipientStrings)
		mRecipientStrings = [[NSMutableArray alloc] init];
		
	if (!mRecipientKeys)
		mRecipientKeys = [[NSMutableArray alloc] init];
	
	if (loadNib)
		[self loadNib:kPGPDecryptionPassphraseDialogNib];
	
	return self;
}

- (void)loadNib:(NSString*)nibFile
{
	NSTableColumn*		keyColumn;
	CBrowserCell*		dataCell;
	NSRect				frame;
	
	[NSBundle loadNibNamed:nibFile owner:self];
    
	[mPrompt2 retain];
	[mOptionsButton retain];
	[mCapsLockWarningField retain];
	[mHideTypingCheckbox retain];
	[mMessageText retain];
	[mScrollView retain];
	
	[mHideTypingCheckbox setState:NSOnState];
	
	[mScrollView setBorderType:NSBezelBorder];
	[mScrollView setHasVerticalScroller:YES];	
	
	frame = [mScrollView frame];
	
	frame.size = [NSScrollView contentSizeForFrameSize:frame.size
							   hasHorizontalScroller:NO
							   hasVerticalScroller:YES
							   borderType:NSBezelBorder];
	
	if (!mPassphraseView)
	{
		mPassphraseView = [[CPGPPassphraseTextView alloc] initWithFrame:NSMakeRect(0, 0, NSWidth(frame), NSHeight(frame))];
		[mPassphraseView setAutoresizingMask:(NSViewHeightSizable | NSViewWidthSizable)];
		[mPassphraseView setAutoresizesSubviews:NO];
		[mPassphraseView setDelegate:self];
		[[mPassphraseView layoutManager] setExtraLineFragmentRect:NSMakeRect(0, 0, NSWidth(frame), 14)
								   usedRect:NSMakeRect(0, 0, 10, 14)
								   textContainer:[mPassphraseView textContainer]];
	}
	
	if (!mSpaceView)
	{
		mSpaceView = [[CPGPPassphraseTextView alloc] initWithFrame:NSMakeRect(0, 0, NSWidth(frame), NSHeight(frame))];
		[mSpaceView setAutoresizingMask:(NSViewHeightSizable | NSViewWidthSizable)];
		[mSpaceView setAutoresizesSubviews:NO];
		[mSpaceView setDelegate:self];
		[[mSpaceView layoutManager] setExtraLineFragmentRect:NSMakeRect(0, 0, NSWidth(frame), 14)
								   usedRect:NSMakeRect(0, 0, 10, 14)
								   textContainer:[mSpaceView textContainer]];
	}
	
	[self hideText:nil];
	
	if ([mHideTypingCheckbox state] == NSOnState)
	{
		[mWindow setInitialFirstResponder:mSpaceView];
		[mWindow makeFirstResponder:mSpaceView];
		[mSpaceView setNextKeyView:mCancelButton];
		[mHideTypingCheckbox setNextKeyView:mSpaceView];
	}
	else
	{
		[mWindow setInitialFirstResponder:mPassphraseView];
		[mWindow makeFirstResponder:mPassphraseView];
		[mPassphraseView setNextKeyView:mCancelButton];
		[mHideTypingCheckbox setNextKeyView:mPassphraseView];
	}
	
	[mRecipientsTable setDelegate:self];
	[mRecipientsTable setDataSource:self];
	
	if (keyColumn = [mRecipientsTable tableColumnWithIdentifier:@"Recipient"])
	{
		dataCell = [[CBrowserCell alloc] init];
		[dataCell setLeaf:YES];
		[dataCell setWraps:NO];
		[dataCell setFont:[NSFont systemFontOfSize:10]];
		[keyColumn setDataCell:dataCell];
		[dataCell release];
	}
	
	mLoadedNib = YES;
}

- (PGPKeyID*)keyIDList
{
	return mKeyIDList;
}

- (void)setKeyIDList:(PGPKeyID*)keyIDList
{
	mKeyIDList = keyIDList;
}

- (PGPUInt32)keyIDCount
{
	return mKeyIDCount;
}

- (void)setKeyIDCount:(PGPUInt32)keyIDCount
{
	mKeyIDCount = keyIDCount;
}

- (PGPKeyID*)missingKeyIDList
{
	return mMissingKeyIDList;
}

- (void)setMissingKeyIDList:(PGPKeyID*)missingKeyIDList
{
	mMissingKeyIDList = missingKeyIDList;
}

- (PGPUInt32)missingKeyIDCount
{
	return mMissingKeyIDCount;
}

- (void)setMissingKeyIDCount:(PGPUInt32)missingKeyIDCount
{
	PGPKeyDBObjRef		nullKey = NULL;
	
	mMissingKeyIDCount = missingKeyIDCount;
	
	if (mMissingKeyIDCount)
	{
		[mRecipientStrings insertObject:[NSString stringWithFormat:
			kPGPUIString_NumUnknownKeys, mMissingKeyIDCount] atIndex:0];
		[mRecipientKeys insertObject:[NSData dataWithBytes:&nullKey length:sizeof(PGPKeyDBObjRef)] atIndex:0];
	}
}

- (void)setNewKeys:(PGPKeyDBRef*)newKeys
{
	PGPKeySetRef		keySet;

	[super setNewKeys:newKeys];
	
	if (mNewKeys && PGPKeyDBRefIsValid(*mNewKeys))
	{
		if (IsntPGPError(PGPNewKeySet(*mNewKeys, &keySet)))
		{
			[self addKeySet:keySet toRecipientsTable:mRecipientsTable];
			
			PGPFreeKeySet(keySet);
		}
	}
}

- (void)setKeySet:(PGPKeySetRef)keySet
{
	[super setKeySet:keySet];
	
	[self addKeySet:mKeySet toRecipientsTable:mRecipientsTable];
}

- (PGPError)addKeySet:(PGPKeySetRef)keySet toRecipientsTable:(NSTableView*)table
{
	PGPError			err;
	PGPKeyListRef		keyList = NULL;
	PGPKeyIterRef		keyIter = NULL;
	PGPKeyDBObjRef		key;
	PGPBoolean			isSecret;
	PGPBoolean			canDecrypt;
	PGPUInt32			algorithm;
	NSString*			keyString;
	
	if (!keySet || !table)
		return kPGPError_BadParams;

	err = PGPOrderKeySet(keySet, kPGPKeyOrdering_Validity, FALSE, &keyList); CKERR;
	err = PGPNewKeyIter(keyList, &keyIter); CKERR;

	while (IsntPGPError(PGPKeyIterNextKeyDBObj(keyIter, kPGPKeyDBObjType_Key, &key)))
	{
		PGPGetKeyDBObjBooleanProperty(key, kPGPKeyProperty_IsSecret, &isSecret);
		PGPGetKeyDBObjBooleanProperty(key, kPGPKeyProperty_CanDecrypt, &canDecrypt);
		PGPGetKeyDBObjNumericProperty(key, kPGPKeyProperty_AlgorithmID, &algorithm);

		if (isSecret && canDecrypt)
			mHaveSecretKey = TRUE;
		
		if (keyString = [self generateEncryptionKeyString:key])
		{
			[mRecipientStrings addObject:keyString];
			[mRecipientKeys addObject:[NSData dataWithBytes:&key length:sizeof(key)]];
			[keyString release];
		}
	}
	
	[table reloadData];
	[table display];
	
done:
	
	if (keyList)
		PGPFreeKeyList(keyList);
		
	if (keyIter)
		PGPFreeKeyIter(keyIter);
		
	return err;
}

- (PGPError)runDialog
{
	if (!mHaveSecretKey)
	{
		[mScrollView removeFromSuperview];
		[mHideTypingCheckbox removeFromSuperview];
		[mPrompt2 removeFromSuperview];
		[mOkButton setEnabled:NO];
						
		[mMessageText setStringValue:kPGPUIString_CannotDecryptText];
		[mMessageText displayIfNeeded];
		
		[mRecipientsTable setNextKeyView:nil];
		
		[mWindow makeFirstResponder:mRecipientsTable];
	}
	else
	{
		[mMessageText removeFromSuperview];
		
		[mWindow makeFirstResponder:mPassphraseView];
	}

	return [super runDialog];
}

- (IBAction)hideText:(id)sender
{
	if ([mHideTypingCheckbox state] == NSOnState)
	{
		[mScrollView setDocumentView:mSpaceView];
		[mWindow makeFirstResponder:mSpaceView];
		[mHideTypingCheckbox setNextKeyView:mSpaceView];
	}
	else
	{
		[mScrollView setDocumentView:mPassphraseView];
		[mWindow makeFirstResponder:mPassphraseView];
		[mHideTypingCheckbox setNextKeyView:mPassphraseView];
	}
}

- (IBAction)ok:(id)sender
{
	PGPKeyDBObjRef	key = NULL;
	const char*		utf8Passphrase = [[mPassphraseView string] UTF8String];
	unsigned		passphraseLength = strlen(utf8Passphrase);
	
	if (!mPassphrasePtr)
	{
		PGPLog((@"Missing passphrase ptr..."));
		mReturnCode = kPGPError_BadParams;
		[self close];
		
		return;
	}
	
	*mPassphrasePtr = PGPNewSecureData(PGPPeekContextMemoryMgr(mProxyObject->context), passphraseLength + 1, 0);
	
	if (!*mPassphrasePtr)
	{
		PGPLog((@"PGPNewSecureData failed to allocate the passphrase buffer"));
		mReturnCode = kPGPError_OutOfMemory;
		[self close];
		
		return;
	}
	
	pgpCopyMemory(utf8Passphrase, *mPassphrasePtr, passphraseLength);
	*((*mPassphrasePtr) + passphraseLength) = '\0';
	
	if (![self passphraseLengthAndQualityOk:*mPassphrasePtr])
	{
		memset((char*) *mPassphrasePtr, '\0', passphraseLength);
		PGPFreeData(*mPassphrasePtr);
		*mPassphrasePtr = NULL;			
		[mPassphraseView clearPassphrase];
		[mSpaceView clearPassphrase];
		[self updatePassphraseQuality:0 sender:self];
		[self hideText:self];
		
		return;
	}
	
	if (!mFindMatchingKey)
	{
		mReturnCode = kPGPError_NoErr;
		[self close];
	}
	else
	{
		key = GetKeyForPassphrase(mKeySet, *mPassphrasePtr, FALSE);
		
		if (PGPKeyDBObjRefIsValid(key))
		{
			*mPassphraseKeyPtr = key;
			
			mReturnCode = kPGPError_NoErr;
			[self close];
		}
		else
		{
			[self rejectPassphraseWithMessage:kPGPUIString_ReEnterPassphraseForKey];
			memset((char*) *mPassphrasePtr, '\0', passphraseLength);
			PGPFreeData(*mPassphrasePtr);
			*mPassphrasePtr = NULL;
		}
	}
}

- (int)numberOfRowsInTableView:(NSTableView*)tableView
{
	return [mRecipientStrings count];
}

- (id)tableView:(NSTableView*)tableView objectValueForTableColumn:(NSTableColumn*)tableColumn row:(int)row
{
	id				value = nil;
	
	NSParameterAssert((row >= 0) && (row < [mRecipientStrings count]));
	
	if ([[tableColumn identifier] isEqualToString:@"Recipient"])
		value = [mRecipientStrings objectAtIndex:row];
	
	return value;
}

- (void)tableView:(NSTableView*)table willDisplayCell:(id)cell forTableColumn:(NSTableColumn*)column row:(int)row
{
	PGPKeyDBObjRef		key = kInvalidPGPKeyDBObjRef;
	
	NSParameterAssert(table && cell && column);

	if (row != -1)
	{
		key = *((PGPKeyDBObjRef*) [[mRecipientKeys objectAtIndex:row] bytes]);
		
		if (key)
			[cell setImage:[[CPGPIcons shared] getKeyDBObjIcon:key]];
		else
			[cell setImage:nil];
	}
}

- (void)tableViewSelectionDidChange:(NSNotification*)notification
{
	int			row = [mRecipientsTable selectedRow];
	
	if (row != -1)
		[mRecipientsTable scrollRowToVisible:row];
}

- (void)dealloc
{
	[mWindow setDelegate:nil];
	if (mPrompt2)
		[mPrompt2 release];
	
	if (mHideTypingCheckbox)
		[mHideTypingCheckbox release];
	
	if (mScrollView)
		[mScrollView release];
	
	if (mRecipientStrings)
		[mRecipientStrings release];
	
	if (mRecipientKeys)
		[mRecipientKeys release];
	
	if (mMessageText)
		[mMessageText release];
	
	[super dealloc];
}

@end

/*
---------------------------------------------------------------
*/

@implementation CPGPConfirmationPassphraseDialog

- (id)init
{
	BOOL			loadNib;
	
	loadNib = mLoadedNib ? NO : YES;
	
	if (loadNib)
		mLoadedNib = YES;
		
	[super init];	/* Our superclass will not load its nib if mLoadedNib is true. */
	
	if (loadNib)
		[self loadNib:kPGPConfirmationPassphraseDialogNib];
	
	return self;
}

- (void)loadNib:(NSString*)nibFile
{
    NSRect			frame;
    
	[NSBundle loadNibNamed:nibFile owner:self];
    
	[mWindow setDelegate:self];
	
	[mOptionsButton retain];
	[mCapsLockWarningField retain];
	[mPassphraseQualityPrompt retain];
	[mPassphraseQualityIndicator retain];
	[mMinimumQualityPrompt retain];
	[mMinimumQualityIndicator retain];
	
	[mHideTypingCheckbox setState:NSOnState];
	
	[mScrollView setBorderType:NSBezelBorder];
	[mScrollView setHasVerticalScroller:YES];
	
	frame = [mScrollView frame];
	
	frame.size = [NSScrollView contentSizeForFrameSize:frame.size
							   hasHorizontalScroller:NO
							   hasVerticalScroller:YES
							   borderType:NSBezelBorder];
	
	if (!mPassphraseView)
	{
		mPassphraseView = [[CPGPPassphraseTextView alloc] initWithFrame:NSMakeRect(0, 0, NSWidth(frame), NSHeight(frame))];
		[mPassphraseView setAutoresizingMask:(NSViewHeightSizable | NSViewWidthSizable)];
		[mPassphraseView setAutoresizesSubviews:NO];
		[mPassphraseView setDelegate:self];
		[[mPassphraseView layoutManager] setExtraLineFragmentRect:NSMakeRect(0, 0, NSWidth(frame), 14)
								   usedRect:NSMakeRect(0, 0, 10, 14)
								   textContainer:[mPassphraseView textContainer]];
	}
	
	if (!mSpaceView)
	{
		mSpaceView = [[CPGPPassphraseTextView alloc] initWithFrame:NSMakeRect(0, 0, NSWidth(frame), NSHeight(frame))];
		[mSpaceView setAutoresizingMask:(NSViewHeightSizable | NSViewWidthSizable)];
		[mSpaceView setAutoresizesSubviews:NO];
		[mSpaceView setDelegate:self];
		[[mSpaceView layoutManager] setExtraLineFragmentRect:NSMakeRect(0, 0, NSWidth(frame), 14)
								   usedRect:NSMakeRect(0, 0, 10, 14)
								   textContainer:[mSpaceView textContainer]];
	}
	
    [mConfirmationScrollView setBorderType:NSBezelBorder];
	[mConfirmationScrollView setHasVerticalScroller:YES];
	
	frame = [mConfirmationScrollView frame];
	
	frame.size = [NSScrollView contentSizeForFrameSize:frame.size
							   hasHorizontalScroller:NO
							   hasVerticalScroller:YES
							   borderType:NSBezelBorder];
	
	if (!mConfirmationPassphraseView)
	{
		mConfirmationPassphraseView = [[CPGPPassphraseTextView alloc] initWithFrame:NSMakeRect(0, 0, NSWidth(frame), NSHeight(frame))];
		[mConfirmationPassphraseView setAutoresizingMask:(NSViewHeightSizable | NSViewWidthSizable)];
		[mConfirmationPassphraseView setAutoresizesSubviews:NO];
		[mConfirmationPassphraseView setDelegate:self];
		[[mConfirmationPassphraseView layoutManager] setExtraLineFragmentRect:NSMakeRect(0, 0, NSWidth(frame), 14)
								   usedRect:NSMakeRect(0, 0, 10, 14)
								   textContainer:[mConfirmationPassphraseView textContainer]];
	}
	
	if (!mConfirmationSpaceView)
	{
		mConfirmationSpaceView = [[CPGPPassphraseTextView alloc] initWithFrame:NSMakeRect(0, 0, NSWidth(frame), NSHeight(frame))];
		[mConfirmationSpaceView setAutoresizingMask:(NSViewHeightSizable | NSViewWidthSizable)];
		[mConfirmationSpaceView setAutoresizesSubviews:NO];
		[mConfirmationSpaceView setDelegate:self];
		[[mConfirmationSpaceView layoutManager] setExtraLineFragmentRect:NSMakeRect(0, 0, NSWidth(frame), 14)
								   usedRect:NSMakeRect(0, 0, 10, 14)
								   textContainer:[mConfirmationSpaceView textContainer]];
	}
	
	[self hideText:nil];
	
	if ([mHideTypingCheckbox state] == NSOnState)
		[mWindow setInitialFirstResponder:mSpaceView];
	else
		[mWindow setInitialFirstResponder:mPassphraseView];
	
	mLoadedNib = YES;
}

- (PGPBoolean)showsPassphraseQuality
{
	return mShowPassphraseQuality;
}

- (void)setShowsPassphraseQuality:(PGPBoolean)flag
{
	mShowPassphraseQuality = flag;
}

- (void)updatePassphraseQuality:(PGPUInt32)quality sender:(id)sender
{
	if (mShowPassphraseQuality)
	{
		if (sender == mPassphraseView)
		{
			[mPassphraseQualityIndicator setDoubleValue:quality];
	
			[super updatePassphraseQuality:quality sender:self];
		}
	}
}

- (PGPError)runDialog
{
	NSRect		windowFrame = [mWindow frame];
	
	if (!mShowPassphraseQuality)
	{
		[mPassphraseQualityPrompt removeFromSuperview];
		[mPassphraseQualityIndicator removeFromSuperview];
		[mMinimumQualityPrompt removeFromSuperview];
		[mMinimumQualityIndicator removeFromSuperview];
		
		windowFrame.size.height -= 30;
		[mWindow setFrame:windowFrame display:NO];
	}
	else
	{
		if (mMinPassphraseQuality)
			[mMinimumQualityIndicator setDoubleValue:mMinPassphraseQuality];
	}
    
	if ([mHideTypingCheckbox state] == NSOnState)
		[mWindow makeFirstResponder:mSpaceView];
	else
		[mWindow makeFirstResponder:mPassphraseView];
		
	return [super runDialog];
}

- (void)setupKeyViewLoop
{
	if ([mHideTypingCheckbox state] == NSOnState)
	{
		[mSpaceView setNextKeyView:mConfirmationSpaceView];
		[mConfirmationSpaceView setNextKeyView:mSpaceView];
	}
	else
	{
		[mPassphraseView setNextKeyView:mConfirmationPassphraseView];
		[mConfirmationPassphraseView setNextKeyView:mPassphraseView];
	}
}

- (void)close
{
	[mPassphraseView clearPassphrase];
	[mSpaceView clearPassphrase];
	[mConfirmationPassphraseView clearPassphrase];
	[mConfirmationSpaceView clearPassphrase];
	
	[NSApp stopModal];
	[mWindow close];
}

- (IBAction)hideText:(id)sender
{
	if ([mHideTypingCheckbox state] == NSOnState)
	{
		[mScrollView setDocumentView:mSpaceView];
		[mConfirmationScrollView setDocumentView:mConfirmationSpaceView];
	}
	else
	{
		[mScrollView setDocumentView:mPassphraseView];
		[mConfirmationScrollView setDocumentView:mConfirmationPassphraseView];
	}
	
	[self setupKeyViewLoop];
}

- (IBAction)ok:(id)sender
{
	const char*		utf8Passphrase = [[mPassphraseView string] UTF8String];
	const char*		utf8Confirmation = [[mConfirmationPassphraseView string] UTF8String];
	unsigned		passphraseLength = strlen(utf8Passphrase);
	unsigned		confirmationLength = strlen(utf8Confirmation);
	
	if (!mPassphrasePtr)
	{
		PGPLog((@"Missing passphrase ptr..."));
		mReturnCode = kPGPError_BadParams;
		[self close];
		
		return;
	}
	
	*mPassphrasePtr = PGPNewSecureData(PGPPeekContextMemoryMgr(mProxyObject->context), passphraseLength + 1, 0);
	
	if (!*mPassphrasePtr)
	{
		PGPLog((@"PGPNewSecureData failed to allocate the passphrase buffer"));
		mReturnCode = kPGPError_OutOfMemory;
		[self close];
		
		return;
	}
	
	pgpCopyMemory(utf8Passphrase, *mPassphrasePtr, passphraseLength);
	*((*mPassphrasePtr) + passphraseLength) = '\0';
	
	if (![self passphraseLengthAndQualityOk:*mPassphrasePtr])
	{
		memset((char*) *mPassphrasePtr, '\0', passphraseLength);
		PGPFreeData(*mPassphrasePtr);
		*mPassphrasePtr = NULL;			
		[mPassphraseView clearPassphrase];
		[mConfirmationPassphraseView clearPassphrase];
		[mSpaceView clearPassphrase];
		[mConfirmationSpaceView clearPassphrase];
		[self updatePassphraseQuality:0 sender:self];
		[self hideText:self];
		
		if ([mHideTypingCheckbox state] == NSOnState)
			[mWindow makeFirstResponder:mSpaceView];
		else
			[mWindow makeFirstResponder:mPassphraseView];
		
		return;
	}
	
	if (memcmp(utf8Passphrase, utf8Confirmation,
			passphraseLength > confirmationLength ? passphraseLength : confirmationLength) != 0)
	{
		NSBeginAlertSheet(kPGPUIString_PassphraseMismatch,
							kPGPUIString_OK,	/* defaultButton */
							nil,				/* alternateButton */
							nil,				/* otherButton */
							mWindow,			/* docWindow */
							nil,				/* modalDelegate */
							nil,				/* willEndSelector */
							nil,				/* didEndSelector */
							nil,				/* contextInfo */
							kPGPUIString_PassphraseMismatchMsg);
		
		memset((char*) *mPassphrasePtr, '\0', passphraseLength);
		PGPFreeData(*mPassphrasePtr);
		*mPassphrasePtr = NULL;									  
		[mPassphraseView clearPassphrase];
		[mConfirmationPassphraseView clearPassphrase];
		[mSpaceView clearPassphrase];
		[mConfirmationSpaceView clearPassphrase];
		[self updatePassphraseQuality:0 sender:self];
		[self hideText:self];
		
		if ([mHideTypingCheckbox state] == NSOnState)
			[mWindow makeFirstResponder:mSpaceView];
		else
			[mWindow makeFirstResponder:mPassphraseView];
		
		return;
	}
	else
	{
		mReturnCode = kPGPError_NoErr;
		[self close];
	}
}

- (BOOL)passphraseView:(CPGPPassphraseTextView*)passphraseView shouldInsertText:(NSString*)string
{
	unsigned				numChars;
	unsigned				numSpaces;
	unsigned				spaceIndex;
	unsigned				i;
	char					spaceStr[kMaxSpacesPerCharacter + 1];	
	CPGPPassphraseTextView*	selectedPassphraseView;
	CPGPPassphraseTextView*	selectedSpaceView;
	
	if (!string)
		return NO;
	
	numChars = [string length];
	
	if ((passphraseView == mPassphraseView) || (passphraseView == mSpaceView))
	{
		selectedPassphraseView = mPassphraseView;
		selectedSpaceView = mSpaceView;
	}
	else if ((passphraseView == mConfirmationPassphraseView) || (passphraseView == mConfirmationSpaceView))
	{
		selectedPassphraseView = mConfirmationPassphraseView;
		selectedSpaceView = mConfirmationSpaceView;
	}
	else
		return YES;
	
	/* If we've hit the max passphrase length, disable any further input. */
	if (([[selectedPassphraseView string] length] + numChars) > kMaxPassphraseLength)
		return NO;
	
	spaceIndex = [[selectedPassphraseView string] length] ? [[selectedPassphraseView string] length]: 0;
	
	for (i = 0; i < numChars; ++i, ++spaceIndex)
	{
		numSpaces = mSpaceCountList[spaceIndex];
		memset(spaceStr, ' ', numSpaces);
		spaceStr[numSpaces] = '\0';
		
		[selectedSpaceView superInsertText:[NSString stringWithFormat:@"%s", spaceStr]];
	}
	
	if ((passphraseView == mSpaceView) || (passphraseView == mConfirmationSpaceView))
	{
		[selectedPassphraseView superInsertText:string];
		
		return NO;
	}
	
	return YES;
}

- (BOOL)passphraseView:(CPGPPassphraseTextView*)passphraseView shouldDeleteBackward:(id)sender
{
	unsigned				numSpaces;
	unsigned				spaceIndex;
	unsigned				passLength;
	unsigned				spaceLength;
	unsigned				i;
	CPGPPassphraseTextView*	selectedPassphraseView;
	CPGPPassphraseTextView*	selectedSpaceView;
	
	if ((passphraseView == mPassphraseView) || (passphraseView == mSpaceView))
	{
		selectedPassphraseView = mPassphraseView;
		selectedSpaceView = mSpaceView;
	}
	else if ((passphraseView == mConfirmationPassphraseView) || (passphraseView == mConfirmationSpaceView))
	{
		selectedPassphraseView = mConfirmationPassphraseView;
		selectedSpaceView = mConfirmationSpaceView;
	}
	else
		return YES;
	
	passLength = [[selectedPassphraseView string] length];
	spaceLength = [[selectedSpaceView string] length];
	
	if (passLength)
	{
		[selectedPassphraseView superDeleteBackward:self];
		
		spaceIndex = passLength - 1;
		numSpaces = mSpaceCountList[spaceIndex];
		
		for (i = 0; i < numSpaces; ++i)
			[selectedSpaceView superDeleteBackward:self];
		
		return NO;
	}
	
	return YES;
}

- (BOOL)shouldPassphraseViewDisplayMarkedText:(CPGPPassphraseTextView*)passphraseView
{
	if ((passphraseView == mSpaceView) || (passphraseView == mConfirmationSpaceView))
		return NO;
	
	return YES;
}

- (void)dealloc
{
	[mWindow setDelegate:nil];
	if (mConfirmationPassphraseView)
	{
		[mConfirmationPassphraseView removeFromSuperview];
		[mConfirmationPassphraseView release];
	}
	
	if (mConfirmationSpaceView)
	{
		[mConfirmationSpaceView removeFromSuperview];
		[mConfirmationSpaceView release];
	}
	
	if (mPassphraseQualityPrompt)
		[mPassphraseQualityPrompt release];
	
	if (mPassphraseQualityIndicator)
		[mPassphraseQualityIndicator release];
	
	if (mMinimumQualityPrompt)
		[mMinimumQualityPrompt release];
	
	if (mMinimumQualityIndicator)
		[mMinimumQualityIndicator release];
	
	[super dealloc];
}

@end
